#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Create or Add DATA SEQUENCE Information."
    puts " -in  inName   \[None\]       Optional Input Table."
    puts " -seq seqName  \[csObs.tab\]  Table or PDB File Containing Sequence."
    puts " -out outName  \[stdout\]     Output Table."
    puts " -pdb                       Sequence Input is PDB File (Default for .pdb)."
    puts "Examples:"
    puts " 1. Print PDB Sequence Information to Screen:"
    puts "    addDataSeq.tcl -seq ref.pdb"
    puts " 2. Add DATA SEQUENCE Information to a Table:"
    puts "    addDataSeq.tcl -in dObsA.tab -seq csObs.tab -out new.tab"
    puts "    addDataSeq.tcl -in dObsA.tab -seq ref.pdb   -out new.tab"
    exit 0
   }

set inName  None
set seqName csObs.tab
set outName stdout
set outUnit stdout
set pdbFlag 0

getArgD $ARGV -in  inName
getArgD $ARGV -out outName
getArgD $ARGV -seq seqName

if {![strcasecmp [file extension $seqName] .pdb]} {set pdbFlag 1}

if {[flagLoc $ARGV -pdb]}   {set pdbFlag 1}
if {[flagLoc $ARGV -nopdb]} {set pdbFlag 0}

if {![file exists $seqName]} \
   {
    puts stderr "Error Finding Sequence Input $seqName"
    exit 1
   }

if {[strcasecmp $inName None]} \
   {
    if {![file exists $inName]} \
       {
        puts stderr "Error Finding Input Table $inName"
        exit 1
       }
   }

initResNames

set r1   0
set rN   0
set mode None

if {$pdbFlag} \
   {
    set seqTab [gdbRead table -in $seqName -pdb]
   } \
else \
   {
    set seqTab [gdbRead table -in $seqName]
   }

set seqList [getTabSeqList $seqTab r1 rN mode]
set dataStr [getSeqListSeqStr $seqList $r1 $rN]

if {![strcasecmp $inName None]} \
   {
    if {![strcasecmp $outName stdout]} \
       {
        set outUnit stdout
       } \
    elseif {![strcasecmp $outName stderr]} \
       {
        set outUnit stderr
       } \
    else \
       {
        set error [catch {set outUnit [open $outName w]}]
    
        if {$error} \
           {
            puts stderr "Error Opening Output File $outName"
            exit 1
           }
       }

    puts $outUnit $dataStr    
   } \
else \
   {
    set inTab [gdbRead table -in $inName]

    gdbAddNote $inTab $dataStr 
    gdbWrite   $inTab $outName
   }

if {[strcasecmp $outUnit stdout] && [strcasecmp $outUnit stderr]} \
   {
    catch "close $outUnit"
   }

exit 0
