#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Add Gauss-random noise to PDB Coords:"
    puts " -in    inName  \[ref.pdb\]    Name of Input PDB."
    puts " -out   outName \[noise.pdb\]  Name of Output PDB."
    puts " -iseed rand    \[54321\]      Random Number Seed (Zero for Auto)."
    puts " -sigma stdDev  \[1.0\]        Standard Deviation of Noise to Add."
#   puts " -tab   tabName \[None\]       Optional Table Name."
#   puts "Notes:"
#   puts " If a table name is given with the -tab option:"
#   puts " 1. Single atom:       Coord will be randomized."
#   puts " 2. Atoms I and J:     Angular randomization."
#   puts " 3. Atoms I J and K:   Angle IJK will be randomized."
#   puts " 4. Atoms I J K and L: Torsion IJKL will be randomized."
    exit 1
   }

set inName   ref.pdb
set outName  noise.pdb
set tabName  None
set sigma    1.0
set iseed    54321
set verbose  1

getArgD $ARGV -in    inName
getArgD $ARGV -out   outName
getArgD $ARGV -tab   tabName
getArgD $ARGV -sigma sigma
getArgD $ARGV -iseed iseed

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

set tabName  None

#
#----------------------------------------------------------------------

proc findAtom1 { entry pdbTab } \
{
   set atomName [gdbGet $entry ATOMNAME]
   set resID    [gdbGet $entry RESID]
   set cond     "RESID == $resID && atomMatch( ATOMNAME, \"$atomName\" )"

   set atomList [gdbSelect -from $pdbTab -cond $cond]

   return $atomList
}

#
#----------------------------------------------------------------------

srand $iseed

set noiseMode 0

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input PDB File $inName"
    exit 1
   }

if {[strcasecmp $tabName None]} \
   {
    if {![file exists $tabName]} \
       {
        puts stderr "Error Finding Input Table $tabName"
        exit 1
       } 

    set aTab     [gdbRead table -in $tabName]
    set varNames [gdbGetInfo $aTab -parms varNames]
    set eList    [gdbGetInfo $aTab -parms entries]

    foreach v {ATOMNAME ATOMNAME_I ATOMNAME_J ATOMNAME_K ATOMNAME_L} \
       {
        if {[lsearch -exact varNames $v] >= 0} {incr noiseMode}
       }
   }

set pdbTab [gdbRead table -in $inName -pdb]

switch $noiseMode \
   {
    1 {
       foreach e $eList \
          {
           set atomList [findAtom1 $e $pdbTab] 

           foreach atom $atomList \
              {
               set x [gdbGet $atom X]
               set y [gdbGet $atom Y]
               set z [gdbGet $atom Z]

               gdbSet $atom X [expr $x + $q*[rand -gauss]]
               gdbSet $atom Y [expr $y + $q*[rand -gauss]]
               gdbSet $atom Z [expr $z + $q*[rand -gauss]]
              }
          }
      }

    2 {
      }

    3 {
      }

    4 {
      }

    default {
             set atomList [gdbGetInfo $pdbTab -parms entries]
             set q        [expr $sigma*sqrt(1.0/3.0)]
 
             foreach atom $atomList \
                {
                 set x [gdbGet $atom X]
                 set y [gdbGet $atom Y]
                 set z [gdbGet $atom Z]

                 gdbSet $atom X [expr $x + $q*[rand -gauss]]
                 gdbSet $atom Y [expr $y + $q*[rand -gauss]]
                 gdbSet $atom Z [expr $z + $q*[rand -gauss]]
                }
            }
   }

gdbWrite $pdbTab $outName -pdb

exit
