#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Add Gauss-random noise to a table column:"
    puts " -in    inName  \[test.tab\]  Name of Input Table or PDB."
    puts " -out   outName \[noise.tab\] Name of Output Table,"
    puts " -var   tabVar  \[D\]         Name of Table Variable to Adjust."
    puts " -cond  cond    \[1\]         Selection Condition."
    puts " -iseed rand    \[54321\]     Random Number Seed (Zero for Auto)."
    puts "Noise Source (Use Only One):"
    puts " -sigma stdDev  \[1.0\]     Standard Deviation of Noise to Add."
    puts " -nvar  nVar    \[None\]    Optional Variable with Scale Factor"
    puts "                          for Noise to Add."
    puts "PDB Options:"
    puts " -pdb                     Input is PDB Data (Default for .pdb)."
    puts " -nopdb                   Input is NMRPipe-Format Table."
    puts " -hetatm                  Interpret HETATMs as ATOMs."
    puts "Notes:"
    puts " 1. Default for -out is \"noise.pdb\" for PDB Mode."
    puts " 2. Default for -var is \"X\" for PDB Mode."
    exit 1
   }

set inName   test.tab 
set outName  noise.tab
set tabVar   D
set noiseVar None
set sigma    1.0
set iseed    54321
set pdbMode  nopdb
set hetMode  nohetatm
set cond     1
set verbose  1

getArgD $ARGV -in    inName
getArgD $ARGV -nvar  noiseVar
getArgD $ARGV -sigma sigma
getArgD $ARGV -iseed iseed

getListArgD $ARGV -cond cond

if {![strcasecmp [file extension $inName] .pdb]} {set pdbMode pdb}
 
if {[flagLoc $ARGV -pdb]}       {set pdbMode pdb}
if {[flagLoc $ARGV -nopdb]}     {set pdbMode nopdb}

if {[flagLoc $ARGV -hetatm]}    {set hetMode hetatm}
if {[flagLoc $ARGV -nohetatm]}  {set hetMode nohetatm}

if {[flagLoc $ARGV -verb]}      {set verbose 1}
if {[flagLoc $ARGV -noverb]}    {set verbose 0}

if {![strcmp $pdbMode pdb]} \
   {
    set outName noise.pdb
    set tabVar  X
   }

getArgD $ARGV -out   outName
getArgD $ARGV -var   tabVar

set cond [string trim $cond "{}"]

#
#----------------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts stderr "Error finding input file $inName"
    exit 1
   }

set t      [gdbRead table -in $inName -$pdbMode -$hetMode]
set eList  [gdbSelect -from $t -cond $cond]
set eCount [gdbGetInfo $t -parms entryCount]
set sCount [llength $eList]

set vID [gdbFindByName $t $tabVar]
set nID [gdbFindByName $t $noiseVar]

if {![llength $vID]} \
   {
    puts stderr "Error finding table variable $tabVar"
    exit 1
   }

if {[strcasecmp $noiseVar None] && ![llength $nID]} \
   {
    puts stderr "Error finding noise variable $noiseVar"
    exit 1
   }

srand $iseed

set gList ""

if {![strcasecmp $noiseVar None]} \
   {
    foreach entry $eList \
       {
        set v [gdbGet $entry $tabVar]
        set g [expr $sigma*[rand -gauss]]

        lappend gList $g
        
        gdbSet $entry $tabVar [expr $v + $g]
       } 
   } \
else \
   {
    foreach entry $eList \
       {
        set v   [gdbGet $entry $tabVar]
        set dd  [gdbGet $entry $noiseVar]
        set g   [expr $sigma*$dd*[rand -gauss]]
        
        lappend gList $g

        gdbSet $entry $tabVar [expr $v + $g]
       }
   }

if {![strcmp $pdbMode pdb]} \
   {
    gdbWrite $t $outName -pdb -$hetMode
   } \
else \
   {
    gdbWrite $t $outName
   }

if {$verbose} \
   {
    set d [format %.3f [getStdDev $gList]]
    puts stderr "Entries: $eCount Selected: $sCount Std Dev of Added Noise: $d"
   }

exit
