#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set inName    None
set outName   "" 
set atomList  ""
set adjVal    0.0
set matchMode exact

#
#--------------------------------------------------------

if {$ARGC == 1 || [flagLoc $ARGV -help]} \
   {
    puts "$argv0: Adjust a Chemical Shift Table."
    puts " -in     inName   Chemical Shift Input."
    puts " -out    outName  Chemical Shift Output."
    puts " -atom   atomList List of Atoms to Adjust."
    puts " -adj    adjVal   PPM Adjustment to Add."
    puts "Atom Name Matching Modes:"
    puts " -exact           Match Atom Name Exactly (Default)."
    puts " -glob            Use glob-style Matching."
    puts " -regexp          Use Regular Expression Matching."
    exit 0
   }

getArgD $ARGV -in   inName
getArgD $ARGV -out  outName
getArgD $ARGV -adj  adjVal

getListArgD $ARGV -atom atomList

if {[flagLoc $ARGV -exact]}  {set matchMode exact}
if {[flagLoc $ARGV -glob]}   {set matchMode glob}
if {[flagLoc $ARGV -regexp]} {set matchMode regexp}

if {![file exists $inName]} \
   {
    puts stderr "Error: file $inName was not found."
    exit 0
   }

#
#-----------------------------------------------------------------------

set csTab [gdbRead table -in $inName -name "Adjusted Chemical Shifts"]
set eList [gdbGetInfo $csTab -parms entries]
set n     0

set str [format "% .2f" $adjVal]
set t   [sysTime %Y.%j.%H.%M]

gdbAddNote $csTab ""
gdbAddNote $csTab "REMARK Adjusted $atomList by adding $str $t"

foreach entry $eList \
   {
    set resID    [gdbGet $entry RESID]
    set resName  [gdbGet $entry RESNAME]
    set atomName [gdbGet $entry ATOMNAME]
    set shift    [gdbGet $entry SHIFT]

    if {$shift > 999} continue

    foreach atomExpr $atomList \
       {
        if {[lsearch -$matchMode $atomName $atomExpr] >= 0} \
           {
            set shift [expr $shift + $adjVal]

            gdbSet $entry SHIFT $shift
 
            incr n
            break
           }
       }
   }

puts stderr "Adjusted $inName $n of [llength $eList] entries."

gdbWrite $csTab $outName

exit
