#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Shift 1D Spectra to Align PPM coord with point coord."
    puts " -in     inList  \[None\] Input List or Template."
    puts " -verb                    Verbose Mode ON (Default)."
    puts " -noverb                  Verbose Mode OFF."
    puts "Notes:"
    puts " First spectrum's center position will be used as calibration point." 
    exit 1
   }

set inArg   None
set inList  ""
set verbose 1

getListArgD $ARGV -in inArg

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {![strcasecmp $inArg None]} {exit 0}

if {[string first "%" $inArg] >= 0} \
   {
    for {set i 1} {$i < 100000} {incr i} \
       {
        set inName [format $inArg $i]
        if {![file exists $inName]} break;

        lappend inList $inName
       }
   } \
else \
   {
    set inList $inArg 
   }

#
#--------------------------------------------------------------------------

set i 0

foreach inName $inList \
   {
    if {![file exists $inName]} \
       {
        if {$verbose} {puts "File Note Found: $inName"}
        continue 
       }

    set error [rdFDATA $inName fdata]

    if {$error} \
       {
        if {$verbose} {puts "Error Reading Header: $inName"}
        continue
       }

    incr i

    if {$i == 1} \
       {
        set xSize [getParm $fdata NDSIZE CUR_XDIM]
        set xMid  [expr int($xSize/2) + 1]
        set xPPM  [pnt2spec $fdata CUR_XDIM $xMid ppm]
       } \
    else \
       {
        set tStr [getParmStr $fdata FDTITLE NULL_DIM]
        set xLoc [spec2pnt $fdata CUR_XDIM ${xPPM}ppm]
        set dx   [expr $xLoc - $xMid]

        if {$dx > 0.0} \
           {
            set idx [expr int($dx + 0.5)]
           } \
        else \
           {
            set idx [expr int($dx - 0.5)]
           }
 
        if {$verbose} {puts [format "%-30s %-32s % 6.2f" $inName $tStr $dx]}

#       catch "exec nmrPipe -in $inName -out $inName -inPlace -fn FSH -ls $dx  -sw"

        if {$idx} \
           {
            catch "exec nmrPipe -in $inName -out $inName -inPlace -fn CS  -ls $idx -sw"
           }
       }
      
    sysFree $fdata
   }

exit
