#!/bin/csh

#
# apodAuto.com: automatic creation of apodize files for NLINLS fitting.

if (($#argv != 1) || (`flagLoc $argv -help`)) then
   echo Automatic creation of apodize files for NLINLS.
   echo Usage: $0 specName
   exit 1
endif

set DNAME = $argv[1] 

echo ""
echo "#"
echo "# Automatic creation of apodization files:"

showhdr -verb $DNAME > hdr.out

set DSIZE = (`fgrep "DATA SIZE:" hdr.out`)
set TSIZE = (`fgrep "TD SIZE:"   hdr.out`)
set FSIZE = (`fgrep "FT SIZE:"   hdr.out`)
set SW    = (`fgrep "SW Hz:"     hdr.out`)
set APOD  = (`fgrep "APOD NAME:" hdr.out`)
set Q1    = (`fgrep "APOD Q1:"   hdr.out`)
set Q2    = (`fgrep "APOD Q2:"   hdr.out`)
set Q3    = (`fgrep "APOD Q3:"   hdr.out`)
set ELB   = (APOD `fgrep "LB:"        hdr.out`)
set GLB   = (APOD `fgrep "GB:"        hdr.out`)
set GOFF  = (APOD `fgrep "GOFF:"      hdr.out`)

set dimCount = (`getParm -in $DNAME -parm FDDIMCOUNT -fmt %.0f`)

if ($dimCount >= 1) then
   if ($FSIZE[3] == None) then
      set f = (`MATH "2*$TSIZE[3]"`)
      set s = $SW[3]
   else
      set f = $FSIZE[3]
      set s = (`MATH "$f*$SW[3]/$DSIZE[3]"`)
   endif

   apodLB.com $TSIZE[3] $f apod.x $APOD[3] $Q1[3] $Q2[3] $Q3[3] $ELB[3] $GLB[3] $GOFF[3] 0.5 $s

   /bin/ls apod.x >  apod.list
endif

if ($dimCount >= 2) then
   if ($FSIZE[3] == None) then
      set f = (`MATH "2*$TSIZE[4]"`)
   else
      set f = $FSIZE[4]
      set s = (`MATH "$f*$SW[4]/$DSIZE[4]"`)
   endif

   apodLB.com $TSIZE[4] $f apod.y $APOD[4] $Q1[4] $Q2[4] $Q3[4] $ELB[4] $GLB[4] $GOFF[4] 0.5 $s

   /bin/ls apod.y >> apod.list
endif

if ($dimCount >= 3) then
   if ($FSIZE[5] == None) then
      set f = (`MATH "2*$TSIZE[5]"`)
      set s = $SW[5]
   else
      set f = $FSIZE[5]
      set s = (`MATH "$f*$SW[5]/$DSIZE[5]"`)
   endif

   apodLB.com $TSIZE[5] $f apod.z $APOD[5] $Q1[5] $Q2[5] $Q3[5] $ELB[5] $GLB[5] $GOFF[5] 0.5 $s

   /bin/ls apod.z >> apod.list
endif

if ($dimCount >= 4) then
   if ($FSIZE[6] == None) then
      set f = (`MATH "2*$TSIZE[6]"`)
      set s = $SW[6]
   else
      set f = $FSIZE[6]
      set s = (`MATH "$f*$SW[6]/$DSIZE[6]"`)
   endif

   apodLB.com $TSIZE[6] $f apod.a $APOD[6] $Q1[6] $Q2[6] $Q3[6] $ELB[6] $GLB[6] $GOFF[6]  0.5 $s

   /bin/ls apod.a >> apod.list
endif

echo ""
