#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

#
# assParm.tcl: extract parameters for strip assignment.


set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

#
# Initialize variables.

proc initParmVars {} \
{
   global objInfo ARGV stdfont

   setFonts

   set objInfo(prog,name)     "assParm"
   set objInfo(prog,inName)   "stripInfo.tcl"
   set objInfo(prog,outName)  "stripInfo.tcl"
   set objInfo(prog,maxSpec)  10 

   set objInfo(prog,vList) \
          {
           type name tabName ndim hi xName yName zName xVar yVar zVar
           xTol yTol zTol z1 zn space score nmatch assInit
          }

   set objInfo(font,menu) $stdfont

   for {set j 1} {$j <= $objInfo(prog,maxSpec)} {incr j} \
      {
       foreach var $objInfo(prog,vList) \
          {
           set objInfo(data$j,$var) ""
          }

       set objInfo(data$j,name)    None
       set objInfo(data$j,tabName) None
      }

   getArgD $ARGV -in objInfo(prog,inName)

   readStripInfo
}

#
# Read existing parameters.

proc readStripInfo {} \
{
   global objInfo

   if {"$objInfo(prog,inName)" != "None"} \
      {
       catch {source $objInfo(prog,inName)}
      }
}

#
# Save the current parameters.

proc saveStripInfo {} \
{
   global objInfo

   set types \
      {
       { {tcl}        {.tcl} }
       { {text}       {.txt} }
       { {tab}        {.tab} }
       { {seq}        {.seq} }
       { {dat}        {.dat .DAT} }
       { {fid}        {.fid} }
       { {ft}         {.ft*} }
       { {All Files}  *      }
      }

   set outName \
     [tk_getSaveFile -title "Prediction Summary Output" \
       -filetypes $types -initialfile $objInfo(prog,outName)]

   if {"$outName" != ""} {saveStripInfo2 $outName}
}

proc saveStripInfo2 { outName } \
{
   global objInfo

   set error [catch {set outUnit [open $outName w]}]

   if {$error} \
      {
       showError $objInfo(prog,name) "Error opening output file $outName"
       return 1
      }

   puts $outUnit ""
   puts $outUnit "set objInfo(data,expCount) 0"
   puts $outUnit ""

   for {set j 1} {$j <= $objInfo(prog,maxSpec)} {incr j} \
      {
       set obj data$j

       if {![info exists objInfo($obj,name)]} continue
       if {"$objInfo($obj,name)" == "None"}   continue
       if {"$objInfo($obj,name)" == ""}       continue

       puts $outUnit ""
       puts $outUnit "incr objInfo(data,expCount)"
       puts $outUnit "set  i \$objInfo(data,expCount)"
       puts $outUnit ""

       foreach var $objInfo(prog,vList) \
          {
           puts $outUnit \
            [format "%s %-25s %s" set objInfo(data\$i,$var) $objInfo($obj,$var)]
          }
      }
}

#
# Shift the current parameter page.

proc shiftStripInfo { jump } \
{
   global objInfo

   set currentPage [notebook_current $objInfo(winParm,notebook)]

   if {"$currentPage" == "2D Table"}    return
   if {"$currentPage" == "2D Spectrum"} return

   set maxSpec 0

   for {set j 1} {$j <= $objInfo(prog,maxSpec)} {incr i} \
      {
       if {"$objInfo(data$j,name)" != "" &&
             "$objInfo(data$j,name)" != "None"}
          {
           set maxSpec $j
          }
      }

   set i [lindex $currentPage 1]

   if {$i < 1 || $i > $maxSpec} return

   set j [expr $i + $jump]

   if {$j < 1}        {set j $maxSpec}
   if {$j > $maxSpec} {set j 1}

   foreach var $objInfo(prog,vList) \
      {
       set tmp $objInfo(data$i,$var)
       set objInfo(data$i,$var) $objInfo(data$j,$var)
       set objInfo(data$j,$var) $tmp
      }

   update
}

#
# Reset the current parameter page.

proc clearStripInfo {} \
{
}

#
# Save and quit.

proc closeStripInfo {} \
{
   global objInfo

   saveStripInfo
   exit
}

#
# Event handler to set an input name via file dialog.

proc setParmInName { e page obj var } \
{
   global objInfo

   set types \
      {
       { {All Files}  *      }
       { {ft}         {.ft*} }
       { {dat}        {.dat .DAT} }
       { {fid}        {.fid} }
       { {seq}        {.seq} }
       { {text}       {.txt} }
       { {tab}        {.tab} }
      }

   set inName [tk_getOpenFile -title "Spectral Data Input" -filetypes $types]

   if {"$inName" != ""} \
      {
       set inName [name2fmt $inName]
       set objInfo($obj,$var) $inName
      }

   if {"$var" == "name" && "$inName" != ""} \
      {
       setParmSpecName $page $obj $var
      }

   $e xview moveto 1.0

   update
}

#
# Set the spectrum type and related values.

proc setParmSpecType { page obj var type } \
{
    global objInfo

    set objInfo($obj,type) $type

    switch $type \
       {
        HSQC        {
                     set objInfo($obj,assInit) None
                     set objInfo($obj,nmatch)  2
                     set objInfo($obj,score)   1
                    }

        HNCA        {
                     set objInfo($obj,assInit) CA(i)|CA(i-1) 
                     set objInfo($obj,nmatch)  2
                     set objInfo($obj,score)   2
                    }

        HNCO        {
                     set objInfo($obj,assInit) CO(i-1) 
                     set objInfo($obj,nmatch)  0
                     set objInfo($obj,score)   1
                    }

        HNCACB      {
                     set objInfo($obj,assInit) CA(i)|CA(i-1)|CB(i)|CB(i-1) 
                     set objInfo($obj,nmatch)  4
                     set objInfo($obj,score)   2
                    }

        HN(CO)CACB  {
                     set objInfo($obj,assInit) CA(i-1)|CB(i-1) 
                     set objInfo($obj,nmatch)  2
                     set objInfo($obj,score)   2
                    }

        HNHA        {
                     set objInfo($obj,assInit) HN(i)|HA(i)
                     set objInfo($obj,nmatch)  2
                     set objInfo($obj,score)   1
                    }

        HBHANH      {
                     set objInfo($obj,assInit) HA(i)|HA(i-1)|HB(i)|HB(i-1)
                     set objInfo($obj,nmatch)  4
                     set objInfo($obj,score)   2
                    }

        HBHA(CO)NH  {
                     set objInfo($obj,assInit) HA(i-1)|HB(i-1)
                     set objInfo($obj,nmatch)  2
                     set objInfo($obj,score)   2
                    }

        CBCANH      {
                     set objInfo($obj,assInit) CA(i)|CA(i-1)|CB(i)|CB(i-1) 
                     set objInfo($obj,nmatch)  4
                     set objInfo($obj,score)   2
                    }

        CBCA(CO)NH  {
                     set objInfo($obj,assInit) CA(i-1)|CB(i-1) 
                     set objInfo($obj,nmatch)  2
                     set objInfo($obj,score)   2
                    }

        N15TOCSY    {
                     set objInfo($obj,assInit) H*(i)
                     set objInfo($obj,nmatch)  2
                     set objInfo($obj,score)   1
                    }

        N15NOE      {
                     set objInfo($obj,assInit) H*
                     set objInfo($obj,nmatch)  2
                     set objInfo($obj,score)   1
                    }

        default     {
                     set objInfo($obj,assInit) None 
                     set objInfo($obj,nmatch)  0
                     set objInfo($obj,score)   1
                    }
       }
}

#
# Set the spectrum name, read and interpret related values.

proc setParmSpecName { page obj var } \
{
   global objInfo

   set error [rdFDATA $objInfo($obj,$var) fdata]

   if {$error} \
      {
       showError $objInfo(prog,name) "Error reading file $objInfo($obj,$var)"
       exit
      }

   set objInfo($obj,ndim)   [getParmI $fdata FDDIMCOUNT   NULL_DIM]
   set objInfo($obj,tpFlag) [getParmI $fdata FDTRANSPOSED NULL_DIM]

   set dimList {X_PPM Y_PPM Z_PPM A_PPM}

   for {set dim 1} {$dim <= $objInfo($obj,ndim)} {incr dim} \
      {
       set code [getParmI   $fdata FDDIMORDER$dim NULL_DIM]
       set size [getParmI   $fdata NDSIZE         $dim]
       set obs  [getParmI   $fdata NDOBS          $dim]
       set lab  [getParmStr $fdata NDLABEL        $dim]
        
       set first [pnt2spec $fdata $dim 1     ppm]
       set last  [pnt2spec $fdata $dim $size ppm]
       set v     [lindex $dimList [expr $dim - 1]]

       if {$code == 2} \
          {
           set objInfo($obj,xName) $lab
           set objInfo($obj,xVar)  $v
           set objInfo($obj,xTol)  0.03 
          } \
       else \
          {
           if {$first > 120.0 && $last < 120.0} \
              {
               set objInfo($obj,yName) $lab
               set objInfo($obj,yVar)  $v
               set objInfo($obj,yTol)  0.15
              } \
           else \
              {
               set objInfo($obj,zName) $lab
               set objInfo($obj,zVar)  $v
               set objInfo($obj,z1)    [format %.3fppm $first]
               set objInfo($obj,zn)    [format %.3fppm $last]
               set objInfo($obj,space) 0

               if {$obs > 400.0} \
                  {
                   set objInfo($obj,zTol)  0.04
                  } \
               else \
                  {
                   set objInfo($obj,zTol)  0.25
                  }
              }
          }
      }

#
# Guess as spectrum type:

#
# Guess at peak table:

#
# Estimate contour level:

   readROI -roi 1 -ndim 2 -in $objInfo($obj,$var) \
      -x X_AXIS -y Y_AXIS -dz Z_AXIS 35% 0

   set roiInfo [getROIInfo -roi 1]

   set rdata   [lindex $roiInfo 0]
   set xSize   [lindex [lindex $roiInfo 2] 0]
   set ySize   [lindex [lindex $roiInfo 2] 1]
   set n       [expr $xSize*$ySize]

   set noiseRMS [vEstNoise $rdata $n]

   set objInfo($obj,hi) [format %.2e [expr 4.0*$noiseRMS]]

   sysFree $fdata
}

#
# Event handler for menus.

proc parmMenuProc { page obj var val } \
{
    global objInfo

    switch $var \
       {
        hi   {
              set hi ""

              if {"$val" == "Increase"} \
                 {
                  catch {set hi [expr $objInfo($obj,$var)*1.3]}
                 } \
              else \
                 {
                  catch {set hi [expr $objInfo($obj,$var)/1.3]}
                 }

              catch {set objInfo($obj,$var) [format %.2e $hi]}
             }

        type    {setParmSpecType $page $obj $var $val}

        default {set objInfo($obj,$var) [lindex $val 0]}
       }

    update
}

#
# Create a parameter menu.

proc initParmMenu { w type page obj var } \
{
    global objInfo

    menu $w

    switch $type \
       {
        type {
              set vList {
                         HSQC HNCA HNCO HNCACB HN(CO)CACB HNHA HBHANH \
                         HBHA(CO)NH CBCANH CBCA(CO)NH N15TOCSY N15NOE
                        }
             }

        axis {
              set vList {
                         1H 15N 13C H1 N15 C13 \
                         HN N CA CACB CO H H-acq H-ind HA HAHB
                        }
             }
 
        dim  {
              set vList {X_PPM Y_PPM Z_PPM A_PPM HN N}
             }
 
        tol  {
              set vList {
                         0.01 0.02 0.03 0.04 0.05 0.06 0.07 0.08 0.09 \
                         0.10 0.15 0.20 0.25 0.30 0.35 0.40 0.45 0.50
                        }
             }
 
        coord {
               set vList {
                          Auto 0% 100% \
                          -10.0ppm -5ppm -1ppm 0ppm 5ppm 10ppm 15ppm 20ppm \
                          25ppm 30ppm 35ppm 40ppm 45ppm 50ppm 55ppm 65ppm  \
                          70ppm 75ppm  100ppm 120ppm 130ppm 150ppm 160ppm  \
                          170ppm 180ppm 
                         }
              }
 
        count {
               set vList {0 1 2 3 4 5 6 7 8 9 10}
              }
 
        bool  {
               set vList {"1 (Yes)" "0 (No)"}
              }

        hi    {
               set vList {"Increase" "Decrease"}
              }
 
        ass   {
               set vList {
                          H* H*(i) N(i) HN(i) \
                          HN(i)|HA(i) HA(i)|HA(i-1) HA(i-i)|HB(i-1) \
                          HA(i)|HA(i-1)|HB(i)|HB(i-1)               \
                          C*(i) CA(i)|CA(i-1) CA(i-1)|CB(i-1)       \
                          CA(i)|CA(i-1)|CB(i)|CB(i-1) CO(i-1)
                         }
              }
 
        default {
                 set vList "None"
                }
       }
 
     foreach val $vList \
        {
         $w add command -label $val -font $objInfo(font,menu) \
            -command "parmMenuProc $page $obj $var \"$val\""
        }
}

#
# Create the main application window.

proc initParamWin {} \
{
   global objInfo env

   set obj  winParm

   set objInfo($obj,window)   .pwin
   set objInfo($obj,notebook) .pwin.n

   if {[winfo exists $objInfo($obj,window)]} return

   set objInfo($obj,winXSize) [expr 100 + 80*$objInfo(prog,maxSpec)] 
   set objInfo($obj,winYSize) 220 
   set objInfo($obj,pageList) ""

   toplevel $objInfo($obj,window) 

   wm title    $objInfo($obj,window) "Strip Parameters"
   wm geometry $objInfo($obj,window) +20+40

   frame  .pwin.cmnd

   button .pwin.left  -text "Shift Left"  -command "shiftStripInfo -1"
   button .pwin.right -text "Shift Right" -command "shiftStripInfo  1"
   button .pwin.clear -text "Clear Page"  -command clearStripInfo
   button .pwin.save  -text "Save"        -command saveStripInfo
   button .pwin.quit  -text "Done"        -command closeStripInfo

   lappend objInfo($obj,pageList) "2D Table"
   lappend objInfo($obj,pageList) "2D Spectrum"

   for {set j 1} {$j <= $objInfo(prog,maxSpec)} {incr j} \
      {
       lappend objInfo($obj,pageList) "Strip $j"
      }

   notebook_create $objInfo($obj,notebook) \
      -pages $objInfo($obj,pageList)       \
      -width  $objInfo($obj,winXSize)      \
      -height $objInfo($obj,winYSize)

   pack $objInfo($obj,notebook) -in $objInfo($obj,window) \
        -fill both -expand 1 -side top

   pack .pwin.cmnd -in $objInfo($obj,window) -side bottom -fill x -expand 1

   pack .pwin.left .pwin.right .pwin.clear .pwin.save .pwin.quit \
        -in .pwin.cmnd -side left -fill x -expand 1

#
# Parameter settings for the 3D spectra:

set pInfo \
   {
    {None    0  left   None                  None  None}
    {name    24 left   "Spectrum Name:"      file  green}
    {tabName 24 left   "Peak Table:"         file  green}
    {None    0  left   None                  None  None}
    {type    16 left   "Spectrum Type:"      type  yellow}
    {hi      16 left   "First Contour:"      hi    yellow}
    {None    0  left   None                  None  None}
    {xName   16 left   "HN-Axis Name:"       axis  gray}
    {yName   16 left   "N-Axis Name:"        axis  gray}
    {zName   16 left   "Z-Axis Name:"        axis  gray}
    {None    0  mid    None                  None  None}
    {xVar    12 mid    "HN-Axis Variable:"   dim   gray}
    {yVar    12 mid    "N-Axis Variable:"    dim   gray}
    {zVar    12 mid    "Z-Axis Variable:"    dim   gray}
    {None    0  mid    None                  None  None}
    {xTol    12 mid    "HN-Axis Tolerance:"  tol   yellow}
    {yTol    12 mid    "N-Axis Tolerance:"   tol   yellow}
    {zTol    12 mid    "Z-Axis Tolerance:"   tol   yellow}
    {None    0  right  None                  None  None}
    {z1      18 right  "First Coord:"        coord yellow}
    {zn      18 right  "Last Coord:"         coord yellow}
    {space   18 right  "Extra Space:"        count yellow}
    {None    0  right  None                  None  None}
    {score   18 right  "Score:"              count gray}
    {nmatch  18 right  "Expected Matches:"   count gray}
    {assInit 18 right  "Initial Assignment:" ass   gray}
   }

   set spaceCount 0

   for {set j 1} {$j <= $objInfo(prog,maxSpec)} {incr j} \
      {
       set pageName "Strip $j"
       set w        [notebook_frame $objInfo($obj,notebook) $pageName]
       set i        [expr $j + 1]
   
       frame  $w.left
       frame  $w.mid
       frame  $w.right

       pack $w.left $w.mid $w.right -side left -fill both -expand 1

       foreach p $pInfo \
          {
           set var  [lindex $p 0]
           set wide [lindex $p 1]
           set f    [lindex $p 2]
           set str  [lindex $p 3]
           set type [lindex $p 4]
           set bg   [lindex $p 5]

           if {"$var" == "None"} \
              {
               incr spaceCount

               frame $w.s$spaceCount -height 8
               pack  $w.s$spaceCount -in $w.$f -side top
               continue
              }

           frame $w.f$var
           label $w.l$var -text $str -width 16 -anchor w

           entry $w.e$var -textvariable objInfo(data$i,$var) \
             -width $wide -bg $bg

           $w.e$var xview moveto 1.0

           if {"$type" == "file"} \
              {
               button $w.m$var \
                 -command "setParmInName $w.e$var $j data$i $var" \
                 -relief raised -bitmap @$env(NMRTXT)/caret.map -fg green
              } \
           else \
              {
               menubutton $w.m$var -menu $w.m$var.menu \
                 -relief raised -bitmap @$env(NMRTXT)/caret.map

               initParmMenu $w.m$var.menu $type $j data$i $var
              }

           pack $w.l$var $w.e$var $w.m$var -in $w.f$var -side left
           pack $w.f$var -in $w.$f -side top
          }
      }
}

initParmVars
initParamWin
