#!/bin/csh

#
# Wrapper script for executing autoPipe.tcl 
#  Handles setting of working directory.
#  Handles execution of scripts created by autoPipe.tcl

if (`flagLoc $argv -help`) then
   autoPipe.tcl -wrapper $*
   exit 0
endif

if (`flagLoc $argv -info`) then
   autoPipe.tcl -wrapper $*
   exit 0
endif

set cdDir       = (`getArgD $argv -cd      .`) 
set argList     = (`getListArgDN $argv -args -- None`)

set origDir  = (`pwd`)
set fullDir  = $origDir
set vFlag    = 1

if (`flagLoc $argv -verb`) then
   set vFlag = 1
endif

if (`flagLoc $argv -noverb`) then
   set vFlag = 0
endif

if (($cdDir != ".") && ($cdDir != "./")) then
   if ($vFlag) then
      echo "autoPipe Changing Working Directory:"
      echo " $cdDir"
      echo ""
   endif

   cd $cdDir
   set fullDir = (`pwd`) 
endif

if ($vFlag) then
   echo "autoPipe Working Directory Status:"
   echo " `pwd`"
endif

if (`flagLoc $argv -cleanOnly`) then
   autoClean.com -com
   exit 0
endif

if (`flagLoc $argv -clean`) then
   autoClean.com -com
endif

set graphFlag = 1
set progFlag  = 1
set execFlag  = 1
set graphArg  = graph
set progArg   = progress

if (`flagLoc $argv -exec`) then
   set execFlag = 1
endif

if (`flagLoc $argv -noexec`) then
   set execFlag = 0
endif

if (`flagLoc $argv -graph`) then
   set graphFlag = 1
   set graphArg  = graph
endif

if (`flagLoc $argv -nograph`) then
   set graphFlag = 0
   set graphArg  = nograph
endif

if (`flagLoc $argv -progress`) then
   set progFlag = 1
   set progArg  = progress
endif

if (`flagLoc $argv -noprogress`) then
   set progFlag = 0
   set progArg  = noprogress
endif

date > timestamp.pipe
autoPipe.tcl -wrapper $* 

if ($status != 0) then
   echo "autoPipe Error while executing autoPipe.tcl ... Exiting."
   cd $origDir
   exit 1
endif

set cList = (auto_fid.com \
             auto_ft1.com \
             auto_ft2.com auto_ft2_ist.com \
             auto_ft2av.com auto_ft2av_ist.com \
             auto_ft2mc.com auto_ft2mc_ist.com \
             auto_xy.com auto_xz.com auto_xa.com \
             auto_xyz.com auto_xyza.com auto_scroll.com \
             auto_xyz_ist.com)

foreach c ($cList)
   if (-e $c) then
      if ($vFlag) then
         echo $c -$graphArg -$progArg $argList
      endif

      if ($execFlag != 0) then
         $c -$graphArg -$progArg $argList      

         if ($status != 0) then
            echo "autoPipe Error while executing $c ... Exiting."
            cd $origDir
            exit 1
         endif
      endif
   endif
end

cd $origDir
