#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Perform Centering and Scaling on a Data Series:"
   echo ' -in  inDir   [ft]           Input Directory.'
   echo ' -out outDir  [scaled]       Output Directory.' 
   echo ' -tmp tmpName [tmpavg$$.dat] Temporary Output.'
   echo "Notes:"
   echo "1. Centering is subtracting the average spectrum from"
   echo "   each input spectrum."
   echo "2. Scaling is dividing each spectrum by its variance."
   echo "3. Input and output directories must be different."
   exit 0
endif

set inDir   = (`getArgD $argv -in  ft`)
set outDir  = (`getArgD $argv -out scaled`)
set tmpName = (`getArgD $argv -tmp tmpavg$$.dat`)

set origDir = (`pwd`)
 
/bin/rm -rf $outDir 
mkdir $outDir 

cd $inDir
   set fList = (*)
cd $origDir

if (!($#fList)) then
   echo "autoScale Error: No Files Found in Directory $inDir"
   exit 1
endif

if (`string same $inDir $outDir`) then
   echo "autoScale Error: Input $inDir is the Same as Output $outDir"
   exit 1
endif

set c = (`MATH "1.0/$#fList"`)

nmrPipe -in $inDir/$fList[1] -out $tmpName -ov -fn SET -c 0.0

foreach f ($fList)
   addNMR -in1 $tmpName -in2 $inDir/$f -out $tmpName -add -c2 $c
end

foreach f ($fList)
   addNMR -in1 $inDir/$f -in2 $tmpName -out $outDir/$f -sub

   set v  = (`specStat.com -in $outDir/$f -stat vVariance -brief`)
   set vv = (`MATH "sqrt($v)"`)

   nmrPipe -in $outDir/$f -out $outDir/$f -inPlace -fn MULT -c $vv -inv
end

/bin/rm -f $tmpName

exit 0
