#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set mfrName    mfr.tab
set uEulerList ""

set r1         First 
set rN         Last 
set gc         10
set maxRank    3
set dcNMin     15
set alpha      3.0
set iseed      54321
set verbose    1

getArgD $ARGV -in      mfrName
getArgD $ARGV -r1      r1
getArgD $ARGV -rN      rN
getArgD $ARGV -retry   gc 
getArgD $ARGV -dcNMin  dcNMin
getArgD $ARGV -maxRank maxRank
getArgD $ARGV -alpha   alpha

getListArgD $ARGV -euler uEulerList

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

#
#---------------------------------------------------------------------------

proc getRotMat { rx ry rz } \
{
   set m [dynRotate -verb -rx $rx -ry $ry -rz $rz]

   set mList ""

   foreach i {0 1 2 4 5 6 8 9 10} \
      {
       lappend mList [lindex $m $i]
      }

   return $mList
}

proc rotDevProc {} \
{
   global t tName id pList qList yModel wList eulerList rotCount reportFlag
   global bestScore bestEuler verbose

   set yModel ""

   set rx [lindex $pList 0]
   set ry [lindex $pList 1]
   set rz [lindex $pList 2]

   set mAvg [getRotMat $rx $ry $rz]
   set k    0

   for {set i 0} {$i < $rotCount} {incr i} \
      {
       set w [lindex $wList $i]
       set devList ""

       for {set j 0} {$j < 9} {incr j} \
          {
           set vFrag [lindex $qList $k]
           set vAvg  [lindex $mAvg  $j]  
           set d     [expr $w*($vFrag - $vAvg)]

           lappend yModel  $d
           lappend devList $d

           incr k
          }

       puts [format " %2d. %.5f %s" $i [getRMS $devList] [lindex $eulerList $i]]
      }

   if {$reportFlag} \
      {
       set eulerInfo [dynEuler $mAvg]
       set nEuler    [lindex $eulerInfo 0]

       if {$nEuler > 0} \
          {
           set rx [adjustThisAngle [lindex $eulerInfo 1]]
           set ry [adjustThisAngle [lindex $eulerInfo 2]]
           set rz [adjustThisAngle [lindex $eulerInfo 3]]
          }

       set dev [getRMS $yModel]

       if {$dev < $bestScore} \
          {
           set bestEuler [format "% 7.2f % 7.2f % 7.2f" $rx $ry $rz]
           set bestScore $dev
          }

       set fmt \
        " Tensor Angle %s. Trial: %2d  Rot: % 7.2f % 7.2f % 7.2f Dev: %8.6f"

       if {$verbose} {puts [format $fmt $tName $id $rx $ry $rz $dev]}
      }
}

proc getEulerList { t } \
{
    global y wList eulerList rotCount varList entryList
    global uEulerList dcNMin alpha

    set eulerList ""  
    set wList ""
    set tName "" 
    set y     ""
 
    set rotCount  0

    set n [llength $uEulerList]
    set i 0

    if {![strcasecmp $t User]} \
       {
        set tName User

        if {$n % 3} \
           {
            puts stderr \
             "Error in User Euler List: Rx Ry Rz triplets are required."

            exit 1
           }

        while {$i < $n} \
           {
            set rx [lindex $uEulerList $i]
            incr i

            set ry [lindex $uEulerList $i]
            incr i

            set rz [lindex $uEulerList $i]
            incr i

            set m [getRotMat $rx $ry $rz]
            set w 1.0

            lappend eulerList "$rx $ry $rz"
            lappend wList $w

            foreach v $m {lappend y $v}

            incr rotCount
           }
       } \
    else \
       {
        set tName A$t

        if {[lsearch -exact $varList ${t}_RX] < 0} {return $tName}
 
        foreach e $entryList \
           {
            set avg   [gdbGet $e GROUP_AVG]
            set thisN [gdbGet $e DC_NMIN]

            if {$thisN < $dcNMin} continue

            set rx [adjustThisAngle [gdbGet $e ${t}_RX]]
            set ry [adjustThisAngle [gdbGet $e ${t}_RY]]
            set rz [adjustThisAngle [gdbGet $e ${t}_RZ]]

            set m  [getRotMat $rx $ry $rz]
            set w  [expr exp(-$alpha*$avg)]

            lappend eulerList "$rx $ry $rz"
            lappend wList $w

            foreach v $m {lappend y $v}

            incr rotCount
           }
       }

    return $tName
}

#
#------------------------------------------------------------------------

set entryList ""
set varList   ""
set tol       1.0e-8

if {[strcasecmp $mfrName None]} \
   {
    if {![file exists $mfrName]} \
       {
        puts stderr "Error Finding MFR Input File $mfrName"
        exit 1
       }

    set mfrTab [gdbRead table -in $mfrName]

    if {![strcasecmp $r1 First]} {set r1 [getTabMin $mfrTab D_RES1]}
    if {![strcasecmp $rN Last]}  {set rN [getTabMax $mfrTab D_RES1]}

    set cond       "RANK <= $maxRank && D_RES1 >= $r1 && D_RES1 <= $rN"
    set entryList  [gdbSelect -from $mfrTab -cond $cond] 
    set varList    [gdbGetInfo $mfrTab -parms varNames]
   }

foreach t "User B C D E F G H I J K L M N O P Q R S T U V W X Y Z" \
   {
    set tName [getEulerList $t]

    if {!$rotCount} continue
 
    set bestEuler "0.0 0.0 0.0"
    set bestScore 1.0e+16

    for {set i 1} {$i <= $gc} {incr i} \
       {
        set p ""

        srand [expr $i + $i*11 + $iseed]

        lappend p [expr 180.0 - 360.0*[rand]] 
        lappend p [expr 180.0*[rand]] 
        lappend p [expr 180.0 - 360.0*[rand]] 

        set ans \
         [modelXY -y $y -p $p -macro rotDevProc -res -gc 1 \
                  -tol $tol -id $i -iter 2000 -maxf 2000 -noverb]
       }

    if {$verbose} \
       {
        set fmt " Tensor Angle %s Best Value. Rot: %s Dev: %8.6f"
    
        puts ""
        puts [format $fmt $tName $bestEuler $bestScore]
        puts ""
       } \
    else \
       {
        puts [format "%s %.6f" $bestEuler $bestScore]
       }
   }

exit 0
