#!/bin/csh

set thisVersion = "2023.165.15.00"

set progName    = (`getArgD    $argv -progName basicFT4.com`)
set nusFlag     = (`getLogArgD $argv -us       -nus      0    1`)
set retainFlag  = (`getLogArgD $argv -noretain -retain   0    1`)
set appendFlag  = (`getLogArgD $argv -noappend -append   0    1`)

set nusZFFlag = 0

if (`flagLoc $argv -nusZF`) then
   set nusZFFlag = 1
endif

if ($nusZFFlag) then
   set reconMode = "ist"
else
   set reconMode = "ft"
endif

foreach i (ist rist smile nesta hmsIST maxent)
   if (`flagLoc $argv -$i`) then
      set reconMode = $i
      set nusZFFlag = 1
   endif
end

if (`string csame $reconMode maxent`) then
   set reconMode  = "ft"
   set nusZFFlag  = 0
endif

if (`string csame $reconMode rist`) then
   set reconMode  = "ist"
   set retainFlag = 1
endif

if (`flagLoc $argv -nonusZF`) then
   set nusZFFlag = 0
endif

if ($nusFlag == 0 && $nusZFFlag == 0) then
   set reconMode = "ft"
endif

if ($nusZFFlag) then
   set yNUSZFDef = "zf=1"
   set zNUSZFDef = "zf=1"
   set aNUSZFDef = "zf=1"
else
   set yNUSZFDef = "zf=0"
   set zNUSZFDef = "zf=0"
   set aNUSZFDef = "zf=0"
endif

set smileProg   = nusPipe
set nestaProg   = nesta_wrapper.com
set hmsISTProg  = hmsIST

@ skipCount = 0

if (`string csame $reconMode smile`) then
   if (`cmndExists.com $smileProg` == 0) then
      echo "$progName Error Finding SMILE Program $smileProg"
      exit 1
   endif
else if (`string csame $reconMode hmsIST`) then
   if (`cmndExists.com $hmsISTProg` == 0) then
      echo "$progName Error Finding hmsIST Program $hmsISTProg"
      exit 1
   endif
else if (`string csame $reconMode nesta`) then
   if (`cmndExists.com $nestaProg` == 0) then
      echo "$progName Error Finding NESTA Program $nestaProg"
      exit 1
   endif
endif

if (`flagLoc $argv -help`) then
   if (`string csame $reconMode smile`) then
      echo "$progName (Using basicFT4.com) $thisVersion"
      echo "Perform SMILE Processing for 4D NUS Data (Requires SMILE):"
      echo " https://spin.niddk.nih.gov/bax/software/smile"
      echo "Options:"
   else if (`string csame $reconMode hmsIST`) then
      echo "$progName (Using basicFT4.com) $thisVersion"
      echo "Perform hmsIST Processing for 4D NUS Data (Requires hmsIST):"
      echo " http://gwagner.med.harvard.edu/intranet/istHMS"
      echo "Options:"
   else if (`string csame $reconMode nesta`) then
      echo "$progName (Using basicFT4.com) $thisVersion"
      echo "Perform NESTA Processing for 4D NUS Data (Requires nestanmr):"
      echo " http://nestanmr.com"
      echo "Options:"
   else
      echo "$progName $thisVersion"
      echo "Perform Basic Processing for 4D Phase-Sensitive Data:"
   endif

   echo " -in     inName  [Auto]       4D Phase-Sensitive Time-Domain Input."
   echo " -out    outName [Auto]       4D Spectrum Output."
   echo " -ft1    ft1Name [None]       Optional Output for Interferogram Result."
   echo " -script sName   [nmr.com]    Processing Script Output."

   if (`string csame $reconMode smile`) then
      echo "SMILE Options (See Also: $smileProg -help -fn SMILE):"
      echo " -sample    sName   [Auto]    Sample Schedule Input Name."
      echo " -nusZF                       NUS Zero-Fill Extrapolation ON (Default)."
      echo " -nonusZF                     NUS Zero-Fill Extrapolation OFF."
      echo " -yNUSZF    zfArgs  [zf=1]    Amount of Y-Axis NUS Zero-Fill Extrapolation."
      echo " -zNUSZF    zfArgs  [zf=1]    Amount of Z-Axis NUS Zero-Fill Extrapolation."
      echo " -aNUSZF    zfArgs  [zf=1]    Amount of A-Axis NUS Zero-Fill Extrapolation."
      echo " -smileArgs sArgs   [None]    Additional SMILE Argument Info, or Keyword None."
      echo "                              https://spin.niddk.nih.gov/bax/software/smile"
      echo "                              See: $smileProg -fn SMILE -help for More."
   else if (`string csame $reconMode hmsIST`) then
      echo "hmsIST Options (See Also: $hmsISTProg -help):"
      echo " -nusZF                       NUS Zero-Fill Extrapolation ON (Default)."
      echo " -nonusZF                     NUS Zero-Fill Extrapolation OFF."
      echo " -yNUSZF    zfArgs  [zf=1]    Amount of Y-Axis NUS Zero-Fill Extrapolation."
      echo " -zNUSZF    zfArgs  [zf=1]    Amount of Z-Axis NUS Zero-Fill Extrapolation."
      echo " -aNUSZF    zfArgs  [zf=1]    Amount of A-Axis NUS Zero-Fill Extrapolation."
      echo " -hmsIter   iter    [-1]      Number of hmsIST Iterations, -1 for Auto."
      echo " -hmsArgs   hArgs   [...]     Additional hmsIST Argument Info, or Keyword None."
      echo "                              http://gwagner.med.harvard.edu/intranet/istHMS"
      echo "                              See: $hmsISTProg -help for More."
   else if (`string csame $reconMode NESTA`) then
      echo "NESTA Options (See Also: $nestaProg -help):"
      echo " -sample    sName   [Auto]    Sample Schedule Input Name."
      echo " -nusZF                       NUS Zero Fill Mode ON."
      echo " -yNUSZF    zfArgs  [zf=1]    Amount of Y-Axis NUS Zero-Fill Extrapolation."
      echo " -zNUSZF    zfArgs  [zf=1]    Amount of Z-Axis NUS Zero-Fill Extrapolation."
      echo " -aNUSZF    zfArgs  [zf=1]    Amount of A-Axis NUS Zero-Fill Extrapolation."
      echo " -nThread   n       [Auto]    Number of Parallel Threads (NESTA --threads)."
      echo " -nestaArgs sArgs   [...]     Additional NESTA Arguments, or Keyword None."
      echo " -IRL1                        Use IRL1 Regularization (Default)."
      echo " -L1                          Use L1 Regularization."
      echo " -GAUSS                       Use Gaussian-SL0 Regularization."
   endif

   echo "Other Options:"
   echo " -parent    pName   [None]    Aux Time-Domain Input for Parameter Extraction."
   echo " -noexec                      Create Processing Script, Do Not Process Data."
   echo " -list                        List Script Contents Without Processing."
   echo " -yz                          Exchange Y-Axis and Z-Axis of Result."
   echo " -ya                          Exchange Y-Axis and A-Axis of Result."
   echo " -ssNMR                       Set Defaults for Solid-State NMR Data."
   echo "Intermediate Results:"
   echo " -ft1Dir  ft1Dir  [ft1]       Directory for Intermediate FT1 Result."
   echo " -ft2Dir  ft2Dir  [ft2]       Directory for Intermediate FT2 Result."
   echo " -ft3Dir  ft3Dir  [ft3]       Directory for Intermediate FT3 Result."
   echo " -lp1Dir  lp1Dir  [lp1]       Directory for Intermediate LP Result."
   echo " -clean                       Delete Intermediate Results (Default)."
   echo " -noclean                     Retain Intermediate Results."
   echo "Processing Parameters (Use -x... -y... -z... -a...):"
   echo " -xSOL   sName   [NULL]       Solvent Function Name and Options (X-Axis Only)."
   echo " -xAPOD  aName   [SP]         Window Function Name."
   echo " -xQ1    q1      [0.50]       Window Parameter Q1."
   echo " -xQ2    q2      [0.95]       Window Parameter Q2."
   echo " -xQ3    q3      [2]          Window Parameter Q3 (Default for -yQ3/-zQ3 is 1)."
   echo " -xELB   elb     [0.0]        Exponential Line Broaden, Hz."
   echo " -xGLB   glb     [0.0]        Gaussian Line Broaden, Hz."
   echo " -xGOFF  goff    [0.0]        Gaussian Center, 0.0 to 1.0."
   echo " -xC1    c       [Auto]       First-Point Time-Domain Scale (0.5 1.0 Auto)."
   echo " -xP0    p0      [0.0]        Zero-Order Phase Correction (Or -xP0 Auto)."
   echo " -xP1    p1      [0.0]        First Order Phase Correction."
   echo " -xZFARG zfArgs  [zf=1,auto]  ZF (Zero Fill) Option."
   echo " -xFTARG ftArg   [None]       FT (Fourier Transform) Options (None neg alt)."
   echo " -xEXTX1 x1      [0%]         Extract Range Start (X-Axis Only) (% Hz ppm pts)."
   echo " -xEXTXN xn      [100%]       Extract Range End   (X-Axis Only) (% Hz ppm pts)."
   echo " -xPREFT  fnArgs [NULL]       Function to Execute Before Any Other Processing."
   echo " -xPOSTFT fnArgs [NULL]       Function to Execute After FT and Phase Correction."
   echo " -apodDF                      Adjust Window Function for Digital Oversampling."
   echo "Auto Phasing Options (Currently Only For X-Axis Zero Order Phase, -xP0 Auto):"
   echo " -apOrd   ord     [0]         Auto-Phase Baseline Order (-1 for No Correction)."
   echo " -apArgs  apArgs  [None]      Other Options for basicAutoPhase.com Utility."
   echo "Baseline Correction Function Name and Options:"
   echo " -xBASEARG       bArgs [NULL] X-Axis Baseline Correction After Y-Axis FT."
   echo " -yBASEARG       bArgs [NULL] Y-Axis Baseline Correction After Y-Axis FT."
   echo " -zBASEARG       bArgs [NULL] Z-Axis Baseline Correction After Z-Axis FT."
   echo " -aBASEARG       bArgs [NULL] A-Axis Baseline Correction After A-Axis FT."
   echo "Processing Parameters for X-Axis Only:"
   echo " -xBASEARG_FT1   bArgs [NULL] Baseline Correction After X-Axis FT and EXT."
   echo " -xBASEARG_INIT  bArgs [NULL] Baseline Correction After X-Axis FT, Before EXT."
   echo " -xBASEARG_FINAL bArgs [NULL] Baseline Correction After Z-Axis FT."
   echo " -xP0EXT         p0Ext [0.0]  Zero-Order Phase After Region Extraction."
   echo " -xP1EXT         p1Ext [0.0]  First Order Phase After Region Extraction."

   if ($nusFlag == 0) then
      echo "Linear Prediction Extrapolation (Indirect Dimensions Only, Not Used for -nusZF):"
      echo " -yLP    lpArgs   [NULL]      Y-Axis Linear Prediction Arguments."
      echo " -zLP    lpArgs   [NULL]      Z-Axis Linear Prediction Arguments."
      echo " -aLP    lpArgs   [NULL]      A-Axis Linear Prediction Arguments."
      echo " -inv                         Use LP Inverse Processing Scheme if Needed."
      echo "NUS Zero-Fill (NUS-style Extrapolation of Conventional Uniformly Sampled Data):"
      echo " -nusZF                       NUS Zero Fill Extrapolation ON."
      echo " -nonusZF                     NUS Zero Fill Extrapolation OFF (Default)."
      echo " -yNUSZF zfArgs   [zf=1]      Amount of Y-Axis NUS Zero-Fill Extrapolation."
      echo " -zNUSZF zfArgs   [zf=1]      Amount of Z-Axis NUS Zero-Fill Extrapolation."
      echo " -aNUSZF zfArgs   [zf=1]      Amount of A-Axis NUS Zero-Fill Extrapolation."
      echo "Extrapolation by NMRPipe IST (See Also: ist4D.com -help):"
      echo " -ist                         Use NMRPipe IST Reconstruction (Default)."
      echo " -retain                      Retain Original Time-Domain Data Points."
      echo " -rist                        Same as -ist -retain Option."
      echo "Extrapolation by SMILE (See Also: $smileProg -fn SMILE -help):"
      echo " -smile                       Use SMILE Reconstruction; Requires SMILE:"
      echo "                              https://spin.niddk.nih.gov/bax/software/smile"
      echo " -smileArgs sArgs [None]      SMILE Argument Info, or Keyword None;"
      echo "Extrapolation by hmsIST (See Also: $hmsISTProg -help):"
      echo " -hmsIST                      Use hmsIST Reconstruction; Requires hmsIST:"
      echo "                              http://gwagner.med.harvard.edu/intranet/istHMS"
      echo " -hmsIter   iter    [-1]      Number of hmsIST Iterations, -1 for Auto."
      echo " -hmsArgs   hArgs [None]      hmsIST Argument Info, or Keyword None;"
      echo "Extrapolation by NESTA (See Also: $nestaProg -help):"
      echo " -sample      sName   [Auto]  Sample Schedule Input Name."
      echo " -nThread     n       [Auto]  Number of Parallel Threads (NESTA --threads)."
      echo " -L1                          L1 Regularization (NESTA --method 1, Default)."
      echo " -IRL1                        IRL1 Regularization (NESTA --rwiter > 1)."
      echo " -GAUSS                       Gaussian-SL0 Regularization (NESTA --method 0)."
      echo " -nestaIter   iter    [Auto]  Number of Iterations (NESTA --iter)."
      echo " -nestaRWIter rwiter  [Auto]  Number of Reweighting Iterations (NESTA --rwiter)."
      echo " -nestaCutoff cuttoff [0.10]  Cutoff for Gaussian-SL0 (NESTA --cutoff)."
      echo " -nestaScale  scale   [0.98]  Scaling for Gaussian-SL0 (NESTA --scaling)."
      echo " -nestaArgs   sArgs   [...]   Additional NESTA Arguments, or Keyword None."
   endif

   echo ""
   echo "Notes:"
   echo " 1. Options -xSOL -xBASEARG_FT1 and -xBASEARG etc are expanded into NMRPipe"
   echo "    processing function names and command line arguments. Options should"
   echo "    contain no spaces. Flags are separated by commas, and options with"
   echo "    values are given using = signs. For example, solvent filter:"
   echo ""
   echo "      -xSOL     NULL            Becomes: nmrPipe -fn NULL (No Solvent Filter)"
   echo "      -xSOL     SOL             Becomes: nmrPipe -fn SOL"
   echo "      -xSOL     POLY,time       Becomes: nmrPipe -fn POLY -time"
   echo ""
   echo "    For baseline correction:"
   echo ""
   echo "      -xBASEARG NULL            Becomes: nmrPipe -fn NULL (No Correction)"
   echo "      -xBASEARG POLY,auto,ord=0 Becomes: nmrPipe -fn POLY -auto -ord 0"
   echo ""
   echo " 2. Options -xZFARG -xFTARG etc are expanded to command line arguments for ZF"
   echo "    and FT, for example:"
   echo ""
   echo "      -xZFARG zf=2,auto       Becomes: nmrPipe -fn ZF -zf 2 -auto"
   echo "      -xFTARG neg             Becomes: nmrPipe -fn FT -neg"
   echo "      -xFTARG neg,alt         Becomes: nmrPipe -fn FT -neg -alt"
   echo ""
   echo " 3. When first-point time-domain scaling is set to Auto (-xC1 Auto etc),"
   echo "    the scaling value will be set to 0.5 if the first-order phase correction"
   echo "    for that dimension is 0+/-10 degrees, or to 1.0 otherwise."
   echo ""
   echo " 4. Options -yLP etc are expanded to arguments for Linear Prediction:"
   echo ""
   echo "       -yLP NULL              No Linear Prediction is performed (Default)."
   echo "       -yLP fb,ord=12         Becomes: nmrPipe -fn LP -fb -ord 12"
   echo ""
   echo " 5. If the X-Axis label of the input spectrum starts with 'HN', default"
   echo "    parameters will be adjusted as follows:"
   echo ""
   echo "       -xSOL SOL -xBASEARG POLY,auto -xEXTX1 3% -xEXTXN 47%"
   echo ""
   echo " 6. If the -ssNMR flag is used, default parameters will be:"
   echo ""
   echo "       -xSOL NULL -xBASEARG NULL"
   echo ""
   echo " 7. If Y, Z, or A-Axis P1 is 360 +/- 10 degrees, default baseline will be:"
   echo ""
   echo "       -yBASEARG POLY,auto,ord=0  (For -yP1 set to 360 +/- 10)"
   echo "       -zBASEARG POLY,auto,ord=0  (For -zP1 set to 360 +/- 10)"
   echo "       -aBASEARG POLY,auto,ord=0  (For -aP1 set to 360 +/- 10)"
   echo ""
   echo " 8. Auto phasing is available only for zero-order phase correction in the"
   echo "    direct-detected dimension (-xP0 Auto). The auto-phasing, which is performed"
   echo "    via the utility basicAutoPhase.com, might not be ideal for all cases. Auto"
   echo "    phasing parameters for this utility can be adjusted via the -apArgs option."
   echo "    For example, to use the Y-Axis region 120.0 ppm to 114.0ppm for evaluating"
   echo "    phase correction:"
   echo ""
   echo "      $progName \"
   echo "       -xP0 Auto -xP1 0 -xEXTX1 10.6ppm -xEXTXN 5.5ppm \"
   echo "       -apArgs apy1=120.0ppm,apyn=114.0ppm"
   echo ""
   echo "    See 'basicAutoPhase.com -help' for more."
   echo ""

   if (`string csame $reconMode smile`) then
      echo "10. Commonly, only region of interest, FT, and phase correction options"
      echo "    need to be provided along with the sampling schedule name. Time-domain"
      echo "    extrapolation (-nusZF) is included by default. Typical example:"
      echo ""
      echo "      $progName -sample nuslist \"
      echo "       -xP0 -16 -xP1   0 -xEXTX1 2.6ppm -xEXTXN -0.8ppm \"
      echo "       -yP0 -90 -yP1 180 -yZFARG zf=2,auto -yFTARG alt"
      echo ""
      echo "11. Option -smileArgs is expanded to command line arguments for SMILE:"
      echo ""
      echo "      -smileArgs maxMem=2.0,thresh=0.9"
      echo ""
      echo "    Becomes: $smileProg -fn SMILE -maxMem 2.0 -thresh 0.9 ..."
      echo ""
      echo "    See '$smileProg -fn SMILE -help' for more."
      echo ""
   else if (`string csame $reconMode hmsIST`) then
      echo "10. Commonly, only region of interest, FT, and phase correction options"
      echo "    need to be provided; hmsIST deduces the NUS sampling schedule by"
      echo "    finding parts of the input data that are set to zero, so that no"
      echo "    sampling schedule input (-sample ...) or mask (-mask ...) is needed."
      echo "    Time-domain extrapolation (-nusZF) is included by default. Typical"
      echo "    example:"
      echo ""
      echo "      $progName \"
      echo "       -xP0 -16 -xP1   0 -xEXTX1 2.6ppm -xEXTXN -0.8ppm \"
      echo "       -yP0 -90 -yP1 180 -yZFARG zf=2,auto -yFTARG alt"
      echo ""
      echo "11. Option -hmsArgs is expanded to command line arguments for hmsIST:"
      echo ""
      echo "      -hmsArgs IST=D,thresh=0.95"
      echo ""
      echo "    Becomes: $hmsISTProg -IST D -thresh 0.95 ..."
      echo ""
      echo "    See '$hmsISTProg -help' for more."
      echo ""
   else if (`string csame $reconMode nesta`) then
      echo "10. Commonly, only region of interest, FT, and phase correction options"
      echo "    need to be provided along with the sampling schedule name. Time-domain"
      echo "    extrapolation (-nusZF) is included by default. Typical example:"
      echo ""
      echo "      $progName -sample nuslist \"
      echo "       -xP0 -16 -xP1   0 -xEXTX1 2.6ppm -xEXTXN -0.8ppm \"
      echo "       -yP0 -90 -yP1 180 -yZFARG zf=2,auto -yFTARG alt"
      echo ""
      echo "11. Option -nestaArgs is expanded to -- command line arguments for NESTA:"
      echo ""
      echo "      -nestaArgs --method=1,--rwiter=5"
      echo ""
      echo "    Becomes: nestanmr --method 1 --rwiter 5"
      echo ""
      echo "    See http://nestanmr.com for more."
      echo ""
      echo "    The defaults are:"
      echo ""
      echo "      --method 1 --iter 50 --rwiter 10 --cutoff 0.1 --scaling 0.98"
      echo ""
   else
      echo "10. Commonly, only region of interest, FT, and phase correction options"
      echo "    need to be provided. Typical example:"
      echo ""
      echo "      $progName \"
      echo "       -xP0 -16 -xP1   0 -xEXTX1 2.6ppm -xEXTXN -0.8ppm \"
      echo "       -yP0 -90 -yP1 180 -yZFARG zf=2,auto -yFTARG alt"
      echo ""
   endif

  if ($nusFlag == 0) then
      echo "11. Turn on NUS Zero Fill (NUS-style extrapolation of time-domain data)"
      echo "    with the -nusZF flag, and select a reconstruction method with one of:"
      echo "    -ist -rist -smile -hmsIST -nesta. The default is NMRPipe IST (-ist)."
      echo ""
      echo "12. Option -smileArgs is expanded to command line arguments for SMILE:"
      echo ""
      echo "      -smileArgs maxMem=2.0,thresh=0.9"
      echo ""
      echo "    Becomes: $smileProg -fn SMILE -maxMem 2.0 -thresh 0.9 ..."
      echo ""
      echo "    See '$smileProg -fn SMILE -help' for more."
      echo ""
      echo "13. Option -hmsArgs is expanded to command line arguments for hmsIST:"
      echo ""
      echo "      -hmsArgs IST=D,thresh=0.95"
      echo ""
      echo "    Becomes: $hmsISTProg -IST D -thresh 0.95 ..."
      echo ""
      echo "    See '$hmsISTProg -help' for more."
      echo ""
      echo "14. Option -nestaArgs is expanded to -- command line arguments for NESTA:"
      echo ""
      echo "      -nestaArgs --method=1,--rwiter=5"
      echo ""
      echo "    Becomes: nestanmr --method 1 --rwiter 5"
      echo ""
      echo "    See http://nestanmr.com for more."
      echo ""
      echo "    The defaults are:"
      echo ""
      echo "      --method 1 --iter 50 --rwiter 10 --cutoff 0.1 --scaling 0.98"
      echo ""
      echo "15. When using -nusZF -ist (or -rist), all options on the basicFT4.com command"
      echo "    line will be passed to the ist4D.com IST reconstruction script. For example,"
      echo "    to set the IST option for -istTMult:"
      echo ""
      echo "      basicFT4.com -nusZF -ist -istTMult 0.9 ..."
   endif

   exit 0
endif

set vFlag     = (`getLogArgD $argv -verb    -noverb   verb noverb`)
set listFlag  = (`getLogArgD $argv -nolist  -list     0    1`)
set ssNMRFlag = (`getLogArgD $argv -nossNMR -ssNMR    0    1`)
set cleanFlag = (`getLogArgD $argv -clean   -noclean  1    0`)
set execFlag  = (`getLogArgD $argv -exec    -noexec   1    0`)
set invFlag   = (`getLogArgD $argv -noinv   -inv      0    1`)

if (`string same $vFlag verb`) then
   set vVal = 1
else
   set vVal = 0
endif

set inName     = (`getArgD $argv -in        Auto`)
set parentName = (`getArgD $argv -parent    None`)
set outName    = (`getArgD $argv -out       Auto`)
set sampleName = (`getArgD $argv -sample    None`)
set scriptName = (`getArgD $argv -script    nmr.com`)
set zPlane     = (`getArgD $argv -zPlane    1`)
set aPlane     = (`getArgD $argv -aPlane    1`)

set nThreads   = (`getArgD $argv -nThread   Auto`)
set istMaxRes  = (`getArgD $argv -istMaxRes Auto`)
set hmsIter    = (`getArgD $argv -hmsIter   -1`)

set istName    = (`getArgDNF $argv -ist     None`)

set uSMILEARGS  = "None"
set uNESTAARGS  = "None"
set dNESTAARGS  = "None"
set uHMSARGS    = "None"

set nestaArgs   = "-nThread $nThreads"
set smileArgs   = ""
set smileFTArgs = ""
set hmsArgs     = ""
set apodDF      = ""

if (`string csame $sampleName None`) then
   set hmsSchedName = "/dev/null"
else
   set hmsSchedName = $sampleName
endif

if (`string csame $reconMode smile`) then
   set uSMILEARGS = (`getArgD $argv -smileArgs None`)

   if (`string csame $uSMILEARGS None`) then
      set smileArgs   = "-nDim 4 -maxIter 2048"
      set smileFTArgs = ""
   else
      set smileArgs   = "`expandArg $uSMILEARGS` -nocheckArgs -nDim 4 -maxIter 2048"
      set smileFTArgs = ""
   endif
endif

if (`string csame $reconMode hmsIST`) then
   set uHMSARGS   = (`getArgD $argv -hmsArgs   None`)

   if (`string notcsame $uHMSARGS None`) then
      set hmsArgs = "`expandArg $uHMSARGS`"
   endif
endif

if (`string csame $reconMode nesta`) then
   set uNESTAARGS = (`getArgD $argv -nestaArgs None`)

   if (`string notcsame $uNESTAARGS None`) then
      set nestaArgs = "$nestaArgs -nestaArgs `expandArg $uNESTAARGS`"
   endif
endif

if (`flagLoc $argv -apodDF`) then
   set apodDF = "-df"
endif

if (`flagLoc $argv -noapodDF`) then
   set apodDF = ""
endif

if ($listFlag) then
   if ($appendFlag == 0) then
      echo "#\!/bin/csh"
   endif

   echo ""
   echo "#"
   echo "# Generated by basicFT4.com Version $thisVersion"
   echo "#"
   prettyPrintCmnd.tcl $0 `stripArgs -list -noexec -append -noappend -- $argv`
   echo "#"
   echo ""
else
   if (`string same $vFlag verb`) then
      echo ""
      echo "basicFT4.com Version:            $thisVersion"
   endif
endif

if (`string csame $inName Auto`) then
   set inName = test.fid
   set fList  = (fid/test%03d.fid data/test%03d.fid fid/spec%03d.fid data/spec%03d.fid fid/test%04d.fid data/test%04d.fid test.fid spec.fid)

   foreach i ($fList)
      set thisName = (`nmrPrintf $i 1 1`)

      if (-e $thisName) then
         set inName = $i
         break
      endif
   end
endif

if (`string csame $outName Auto`) then
   if (`string csame $reconMode ist`) then
      if ($retainFlag) then
         set outName     = rist/test%04d.ft4
      else
         set outName     = ist/test%04.ft4
      endif
   else if (`string csame $reconMode smile`) then
      set outName     = smile/test%04d.ft4
   else if (`string csame $reconMode hmsIST`) then
      set outName     = hms_ist/test%04d.ft4
   else if (`string csame $reconMode nesta`) then
      set outName     = nesta/test%04d.ft4
   else
      set outName     = ft/test%04d.ft4
   endif
endif

if (`string csame $istName None`) then
   set istName   = $outName
endif

if (`string csame $reconMode smile` || `string csame $reconMode nesta`) then
   if ($nusFlag) then
      if (`string csame $sampleName None`) then
         nmrPrintf -stderr "%s\n" "$progName Error: No Sample Schedule Input Specified."
         exit 1
      endif

      if (!(-e $sampleName)) then
         nmrPrintf -stderr "%s\n" "$progName Error Finding Sample Schedule $sampleName"
         exit 1
      endif
   endif
endif

set origInName = $inName

set ft1Dir    = (`getArgD $argv -ft1Dir ft1`)
set ft2Dir    = (`getArgD $argv -ft2Dir ft2`)
set ft3Dir    = (`getArgD $argv -ft3Dir ft3`)
set lp1Dir    = (`getArgD $argv -lp1Dir lp1`)
set lp2Dir    = (`getArgD $argv -lp2Dir lp2`)
set tmpDir    = (`getArgD $argv -tmpDir tmp`)
set auxDir    = (`getArgD $argv -auxDir tmpax`)

set ft1Name   = (`getArgD $argv -ft1 None`)
set ft2Name   = (`getArgD $argv -ft2 $ft2Dir/test%05d.ft2`)
set ft3Name   = (`getArgD $argv -ft3 $ft3Dir/test%05d.ft3`)
set lp1Name   = (`getArgD $argv -lp1 $lp1Dir/test%05d.lp1`)
set lp2Name   = (`getArgD $argv -lp2 $lp2Dir/test%05d.lp2`)
set tmpName   = (`getArgD $argv -tmp $tmpDir/test%05d.ft1`)

@ yzaFlag = 0

if (`flagLoc $argv -yz`) then
   @ yzaFlag = 1
endif

if (`flagLoc $argv -ya`) then
   @ yzaFlag = 2
endif

if (`flagLoc $argv -noyz`) then
   @ yzaFlag = 0
endif

if (`flagLoc $argv -noya`) then
   @ yzaFlag = 0
endif

if (`string same $vFlag verb`) then
   if ($listFlag) then
   else
      echo "$progName Input:              $inName"

      if (`string csame $reconMode smile`) then
         if ($nusFlag) then
            echo "$progName Schedule:           $sampleName"
         endif

         echo "$progName SMILE Output:       $outName"
      else if (`string csame $reconMode nesta`) then
         if ($nusFlag) then
            echo "$progName Schedule:           $sampleName"
         endif

         echo "$progName NESTA Output:       $outName"
      else if (`string csame $reconMode ist`) then
         echo "$progName FT Output:          $ft3Name"
         echo "$progName IST Output:         $istName"

         if ($retainFlag) then
            echo "$progName Retain Time-Domain: Yes"
         else
            echo "$progName Retain Time-Domain: No"
         endif
     else if (`string csame $reconMode hmsIST`) then
         echo "$progName hmsIST Output:      $outName"
      else
         echo "$progName FT Output:          $outName"
      endif

      echo "$progName Script Output:      $scriptName"
      echo ""
   endif
endif

if (`string csame $parentName None`) then
   set thisName = (`nmrPrintf $origInName 1 1`)
else
   set thisName = (`nmrPrintf $parentName 1 1`)
endif

@ lpCount    = 0
@ yLPFlag    = 0
@ zLPFlag    = 0
@ aLPFlag    = 0

if (!(-e $thisName)) then
   echo "basicFT4.com Notice: Missing Input $inName (parent $parentName)."
   exit 1
endif

set xLAB = (`getParm -in $thisName -txt -fmt %s -parm NDLABEL -dim CUR_XDIM`)

if (`string match "HN*" $xLAB`) then
   set uxSOL     = "SOL"
   set uxBASEARG = "POLY,auto" 
   set uxEXTX1   = "3%"
   set uxEXTXN   = "47%"
else
   set uxSOL     = "NULL"
   set uxBASEARG = "NULL"
   set uxEXTX1   = "0%"
   set uxEXTXN   = "100%"
endif

if ($ssNMRFlag) then
   set uxSOL     = "NULL"
   set uxBASEARG = "NULL"
endif

set apArgs  = (`getArgD $argv -apArgs  None`)
set apOrd   = (`getArgD $argv -apOrd   0`)
set xSOL    = (`getArgD $argv -xSOL    $uxSOL`)
set xAPOD   = (`getArgD $argv -xAPOD   SP`)
set xQ1     = (`getArgD $argv -xQ1     0.5`)
set xQ2     = (`getArgD $argv -xQ2     0.95`)
set xQ3     = (`getArgD $argv -xQ3     2`)
set xELB    = (`getArgD $argv -xELB    0.0`)
set xGLB    = (`getArgD $argv -xGLB    0.0`)
set xGOFF   = (`getArgD $argv -xGOFF   0.0`)
set xC1     = (`getArgD $argv -xC1     Auto`)
set xZFARG  = (`getArgD $argv -xZFARG  zf=1,auto`)
set xFTARG  = (`getArgD $argv -xFTARG  None`)
set xP0     = (`getArgD $argv -xP0     0.0`)
set xP1     = (`getArgD $argv -xP1     0.0`)
set xP0EXT  = (`getArgD $argv -xP0EXT  0.0`)
set xP1EXT  = (`getArgD $argv -xP1EXT  0.0`)
set xEXTX1  = (`getArgD $argv -xEXTX1  $uxEXTX1`)
set xEXTXN  = (`getArgD $argv -xEXTXN  $uxEXTXN`)
set xPREFT  = (`getArgD $argv -xPREFT  NULL`)
set xPOSTFT = (`getArgD $argv -xPOSTFT NULL`)

if (`string csame $xZFARG None`) then
   set xZFARG = "zf=0"
endif

if (`string same $xP0EXT 0.0` && `string same $xP1EXT 0.0`) then
   @ usePSEXTFlag = 0
else
   @ usePSEXTFlag = 1
endif

set xBASEARG_INIT  = (`getArgD $argv -xBASEARG_INIT  NULL`)
set xBASEARG_FT1   = (`getArgD $argv -xBASEARG_FT1   NULL`)
set xBASEARG_FINAL = (`getArgD $argv -xBASEARG_FINAL NULL`)
set xBASEARG       = (`getArgD $argv -xBASEARG       $uxBASEARG`)

if (`string csame $xC1 Auto`) then
   set v = (`nmrPrintf "%f" $xP1`)
   @ i   = (`IMATH "abs($v) < 10.0"`)

   if (!($i)) then
      set xC1 = 1.0
   else
      set xC1 = 0.5
   endif
endif

set yAPOD   = (`getArgD $argv -yAPOD   SP`)
set yQ1     = (`getArgD $argv -yQ1     0.50`)
set yQ2     = (`getArgD $argv -yQ2     0.95`)
set yQ3     = (`getArgD $argv -yQ3     1`)
set yELB    = (`getArgD $argv -yELB    0.0`)
set yGLB    = (`getArgD $argv -yGLB    0.0`)
set yGOFF   = (`getArgD $argv -yGOFF   0.0`)
set yC1     = (`getArgD $argv -yC1     Auto`)
set yZFARG  = (`getArgD $argv -yZFARG  zf=1,auto`)
set yNUSZF  = (`getArgD $argv -yNUSZF  $yNUSZFDef`)
set yFTARG  = (`getArgD $argv -yFTARG  None`)
set yP0     = (`getArgD $argv -yP0     0.0`)
set yP1     = (`getArgD $argv -yP1     0.0`)
set yPREFT  = (`getArgD $argv -yPREFT  NULL`)
set yPOSTFT = (`getArgD $argv -yPOSTFT NULL`)

if (`string csame $yZFARG None`) then
   set yZFARG = "zf=0"
endif

if (`string csame $yNUSZF None`) then
   set yNUSZF = "zf=0"
endif

if (`IMATH "near( $yP1, 360, 10 )"`) then
   set uyBASEARG = "POLY,auto,ord=0"
else
   set uyBASEARG = "NULL"
endif

set yBASEARG     = (`getArgD $argv -yBASEARG $uyBASEARG`)
set yLP          = (`getArgD $argv -yLP      NULL`)

if (`string csame $yC1 Auto`) then
   set v = (`nmrPrintf "%f" $yP1`)
   @ i   = (`IMATH "abs($v) < 10.0"`)

   if (!($i)) then
      set yC1 = 1.0
   else
      set yC1 = 0.5
   endif
endif

if (!(`string csame $yLP NULL`)) then
   set yLP = "LP $yLP"
   @ yLPFlag = 1
   @ lpCount++
endif

set zAPOD   = (`getArgD $argv -zAPOD   SP`)
set zQ1     = (`getArgD $argv -zQ1     0.50`)
set zQ2     = (`getArgD $argv -zQ2     0.95`)
set zQ3     = (`getArgD $argv -zQ3     1`)
set zELB    = (`getArgD $argv -zELB    0.0`)
set zGLB    = (`getArgD $argv -zGLB    0.0`)
set zGOFF   = (`getArgD $argv -zGOFF   0.0`)
set zC1     = (`getArgD $argv -zC1     Auto`)
set zZFARG  = (`getArgD $argv -zZFARG  zf=1,auto`)
set zNUSZF  = (`getArgD $argv -zNUSZF  $zNUSZFDef`)
set zFTARG  = (`getArgD $argv -zFTARG  None`)
set zP0     = (`getArgD $argv -zP0     0.0`)
set zP1     = (`getArgD $argv -zP1     0.0`)
set zPREFT  = (`getArgD $argv -zPREFT  NULL`)
set zPOSTFT = (`getArgD $argv -zPOSTFT NULL`)

if (`string csame $zZFARG None`) then
   set zZFARG = "zf=0"
endif

if (`string csame $zNUSZF None`) then
   set zNUSZF = "zf=0"
endif

if (`IMATH "near( $zP1, 360, 10 )"`) then
   set uzBASEARG = "POLY,auto,ord=0"
else
   set uzBASEARG = "NULL"
endif

set zBASEARG = (`getArgD $argv -zBASEARG $uzBASEARG`)
set zLP      = (`getArgD $argv -zLP      NULL`)

if (`string csame $zC1 Auto`) then
   set v = (`nmrPrintf "%f" $zP1`)
   @ i   = (`IMATH "abs($v) < 10.0"`)

   if (!($i)) then
      set zC1 = 1.0
   else
      set zC1 = 0.5
   endif
endif

if (!(`string csame $zLP NULL`)) then
   set zLP = "LP $zLP"
   @ zLPFlag = 1
   @ lpCount++
endif

set aAPOD   = (`getArgD $argv -aAPOD   SP`)
set aQ1     = (`getArgD $argv -aQ1     0.50`)
set aQ2     = (`getArgD $argv -aQ2     0.95`)
set aQ3     = (`getArgD $argv -aQ3     1`)
set aELB    = (`getArgD $argv -aELB    0.0`)
set aGLB    = (`getArgD $argv -aGLB    0.0`)
set aGOFF   = (`getArgD $argv -aGOFF   0.0`)
set aC1     = (`getArgD $argv -aC1     Auto`)
set aZFARG  = (`getArgD $argv -aZFARG  zf=1,auto`)
set aNUSZF  = (`getArgD $argv -aNUSZF  $aNUSZFDef`)
set aFTARG  = (`getArgD $argv -aFTARG  None`)
set aP0     = (`getArgD $argv -aP0     0.0`)
set aP1     = (`getArgD $argv -aP1     0.0`)
set aPREFT  = (`getArgD $argv -aPREFT  NULL`)
set aPOSTFT = (`getArgD $argv -aPOSTFT NULL`)

if (`string csame $aZFARG None`) then
   set aZFARG = "zf=0"
endif

if (`string csame $aNUSZF None`) then
   set aNUSZF = "zf=0"
endif

if (`IMATH "near( $aP1, 360, 10 )"`) then
   set uaBASEARG = "POLY,auto,ord=0"
else
   set uaBASEARG = "NULL"
endif

set aBASEARG = (`getArgD $argv -aBASEARG $uaBASEARG`)
set aLP      = (`getArgD $argv -aLP      NULL`)

if (`string csame $aC1 Auto`) then
   set v = (`nmrPrintf "%f" $aP1`)
   @ i   = (`IMATH "abs($v) < 10.0"`)

   if (!($i)) then
      set aC1 = 1.0
   else
      set aC1 = 0.5
   endif
endif

if (!(`string csame $aLP NULL`)) then
   set aLP = "LP $aLP"
   @ aLPFlag = 1
   @ lpCount++
endif

if ($lpCount == 0) then
   @ invFlag = 0
else if ($lpCount == 1 && $aLPFlag == 1) then
   @ invFlag = 0
endif

if ($nusZFFlag) then
   set nusZFArgs = "-nusZF -yNUSZF $yNUSZF -zNUSZF $zNUSZF -aNUSZF $aNUSZF"
else
   set nusZFArgs = ""
endif

set yNUSZFArgs = "`expandArg $yNUSZF`"
set zNUSZFArgs = "`expandArg $zNUSZF`"
set aNUSZFArgs = "`expandArg $aNUSZF`"

if (`string csame $reconMode smile`) then
   if (`string same $vFlag verb`) then
      if (`flagLoc SMILE $smileArgs -report` == 0) then
         set smileArgs = "$smileArgs -report 1"
      endif
   endif

   if ($nusFlag) then
      set smileArgs = "$smileArgs -sample $sampleName"
   else
      set smileArgs = "$smileArgs -sample None"
   endif

   set thisARGV = "FT `expandArg $yFTARG`"

   if (`flagLoc $thisARGV -alt`) then
      set smileFTArgs = "$smileFTArgs -xAlt"
   endif

   if (`flagLoc $thisARGV -neg`) then
      set smileFTArgs = "$smileFTArgs -xNeg"
   endif

   set thisARGV = "FT `expandArg $zFTARG`"

   if (`flagLoc $thisARGV -alt`) then
      set smileFTArgs = "$smileFTArgs -yAlt"
   endif

   if (`flagLoc $thisARGV -neg`) then
      set smileFTArgs = "$smileFTArgs -yNeg"
   endif

   set thisARGV = "FT `expandArg $aFTARG`"

   if (`flagLoc $thisARGV -alt`) then
      set smileFTArgs = "$smileFTArgs -zAlt"
   endif

   if (`flagLoc $thisARGV -neg`) then
      set smileFTArgs = "$smileFTArgs -zNeg"
   endif
endif

if (`string csame $xAPOD SP`) then
   if (`string same $xGOFF 0.0`) then
      set xWinInfo = "SP -off $xQ1 -end $xQ2 -pow $xQ3 -elb $xELB -glb $xGLB -c $xC1 $apodDF"
   else
      set xWinInfo = "SP -off $xQ1 -end $xQ2 -pow $xQ3 -elb $xELB -glb $xGLB -goff $xGOFF -c $xC1 $apodDF"
   endif
else
   set xWinInfo = "APOD -qName $xAPOD -q1 $xQ1 -q2 $xQ2 -q3 $xQ3 -elb $xELB -glb $xGLB -goff $xGOFF -c $xC1 $apodDF"
endif

if (`string csame $yAPOD SP`) then
   if (`string same $yGOFF 0.0`) then
      set yWinInfo = "SP -off $yQ1 -end $yQ2 -pow $yQ3 -elb $yELB -glb $yGLB -c $yC1"
   else
      set yWinInfo = "SP -off $yQ1 -end $yQ2 -pow $yQ3 -elb $yELB -glb $yGLB -goff $yGOFF -c $yC1"
   endif
else
   set yWinInfo = "APOD -qName $yAPOD -q1 $yQ1 -q2 $yQ2 -q3 $yQ3 -elb $yELB -glb $yGLB -goff $yGOFF -c $yC1"
endif

if (`string csame $zAPOD SP`) then
   if (`string same $zGOFF 0.0`) then
      set zWinInfo = "SP -off $zQ1 -end $zQ2 -pow $zQ3 -elb $zELB -glb $zGLB -c $zC1"
   else
      set zWinInfo = "SP -off $zQ1 -end $zQ2 -pow $zQ3 -elb $zELB -glb $zGLB -goff $zGOFF -c $zC1"
   endif
else
   set zWinInfo = "APOD -qName $zAPOD -q1 $zQ1 -q2 $zQ2 -q3 $zQ3 -elb $zELB -glb $zGLB -goff $zGOFF -c $zC1"
endif

if (`string csame $aAPOD SP`) then
   if (`string same $aGOFF 0.0`) then
      set aWinInfo = "SP -off $aQ1 -end $aQ2 -pow $aQ3 -elb $aELB -glb $aGLB -c $aC1"
   else
      set aWinInfo = "SP -off $aQ1 -end $aQ2 -pow $aQ3 -elb $aELB -glb $aGLB -goff $aGOFF -c $aC1"
   endif
else
   set aWinInfo = "APOD -qName $aAPOD -q1 $aQ1 -q2 $aQ2 -q3 $aQ3 -elb $aELB -glb $aGLB -goff $aGOFF -c $aC1"
endif

@ fmtCount  = (`echo $inName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

set cleanList   = "$ft3Dir"
set thisOutName = $outName

if (`string csame $reconMode smile` || `string csame $reconMode hmsIST`) then
   if (`string csame $ft1Name None`) then
      set ft1Name   = $ft1Dir/test%05d.ft1
      set cleanList = "$cleanList $ft1Dir $ft2Dir $tmpDir"
   else
      set cleanList = "$cleanList $ft2Dir $tmpDir"
   endif
else if (`string csame $reconMode nesta`) then
   if (`string csame $ft1Name None`) then
      set ft1Name   = $ft1Dir/test%05d.ft1
      set cleanList = "$cleanList $ft1Dir $ft2Dir $tmpDir $auxDir"
   else
      set cleanList = "$cleanList $ft2Dir $tmpDir $auxDir"
   endif
else if (`string csame $reconMode ist`) then
   set cleanList   = "$cleanList $tmpDir"
   set thisOutName = $tmpDir/test%05d.ft4
endif

if ($listFlag) then
   if (`string csame $xP0 Auto`) then
      if (`string csame $apArgs None`) then
         set apArgInfo = ""
      else
         set apArgInfo = (`expandArg $apArgs`)
      endif

      if (`string notcsame $zPlane 1`) then
         set apArgInfo = "$apArgInfo -apzPlane $zPlane"
      endif

      if (`string notcsame $aPlane 1`) then
         set apArgInfo = "$apArgInfo -apaPlane $aPlane"
      endif

      echo "#"
      echo "# Auto-phase determination; arguments duplicated later on the"
      echo "# basicAutoPhase.com command line ignored:"

      if (`string same $vFlag verb`) then
         echo ""
         nmrPrintf 'nmrPrintf "Auto-Phase ...\\n\\n"'
         echo ""
      endif

      if (`string csame $apOrd None`) then
         set apOrd = -1
      endif

      echo ""
      echo "set xP0 = ("\`basicAutoPhase.com -in $origInName $apArgInfo -apxP1 $xP1 -apyP0 $yP0 -apyP1 $yP1 -apyFTARG $yFTARG -apOrd $apOrd\`")"
      echo ""

      if (`string same $vFlag verb`) then
         nmrPrintf 'nmrPrintf "Auto-Phase xP0: %%.1f\\n\\n" $xP0'
         echo ""
         echo ""
      endif

      set xP0 = '$xP0'
   endif

   if (`string csame $reconMode smile`) then
      echo ""
      echo "#"
      echo "# Arguments duplicated later on the SMILE command line are ignored:"
      echo ""
   endif

   if ($invFlag) then
      set cleanList = "$cleanList $lp1Dir"

      echo "xyz2pipe -in $inName -x -$vFlag \"

      if (`string notcsame $xPREFT NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $xPREFT` "\"
      endif

      if (`string notcsame $xSOL NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $xSOL` "\"
      endif

      echo "| nmrPipe -fn" $xWinInfo "\"
      echo "| nmrPipe -fn" ZF `expandArg $xZFARG` "\"
      echo "| nmrPipe -fn" FT `expandArg $xFTARG` "\"

      if ($usePSEXTFlag) then
         echo "| nmrPipe -fn" PS -p0 $xP0 -p1 $xP1 "\"
      else
         echo "| nmrPipe -fn" PS -p0 $xP0 -p1 $xP1 -di "\"
      endif

     if (`string notcsame $xBASEARG_INIT NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $xBASEARG_INIT` "\"
      endif

      echo "| nmrPipe -fn" EXT -x1 $xEXTX1 -xn $xEXTXN -sw "\"

      if ($usePSEXTFlag) then
         echo "| nmrPipe -fn" PS -p0 $xP0EXT -p1 $xP1EXT -di "\"
      endif

      if (`string notcsame $xBASEARG_FT1 NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $xBASEARG_FT1` "\"
      endif

      if (`string notcsame $xPOSTFT NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $xPOSTFT` "\"
      endif

      if (`string csame $reconMode smile`) then
         echo "| pipe2xyz -out $tmpName -a -ov"
         echo ""
         echo "xyz2pipe -in $tmpName -x \"
         echo "| $smileProg -fn SMILE" $smileArgs "\"

         if (`string same $yGOFF 0.0`) then
            echo "                    -xApod" $yAPOD -xQ1 $yQ1 -xQ2 $yQ2 -xQ3 $yQ3 -xELB $yELB -xGLB $yGLB "\"
         else
            echo "                    -xApod" $yAPOD -xQ1 $yQ1 -xQ2 $yQ2 -xQ3 $yQ3 -xELB $yELB -xGLB $yGLB -xGOFF $yGOFF "\"
         endif

         if (`string same $zGOFF 0.0`) then
            echo "                    -yApod" $zAPOD -yQ1 $zQ1 -yQ2 $zQ2 -yQ3 $zQ3 -yELB $zELB -yGLB $zGLB "\"
         else
            echo "                    -yApod" $zAPOD -yQ1 $zQ1 -yQ2 $zQ2 -yQ3 $zQ3 -yELB $zELB -yGLB $zGLB -yGOFF $zGOFF "\"
         endif

         if (`string same $aGOFF 0.0`) then
            echo "                    -zApod" $aAPOD -zQ1 $aQ1 -zQ2 $aQ2 -zQ3 $aQ3 -zELB $aELB -zGLB $aGLB "\"
         else
            echo "                    -zApod" $aAPOD -zQ1 $aQ1 -zQ2 $aQ2 -zQ3 $aQ3 -zELB $aELB -zGLB $aGLB -zGOFF $aGOFF "\"
         endif

         echo "                    -xT" `getZFSize.com $inName CUR_YDIM $yNUSZFArgs` -xP0 $yP0 -xP1 $yP1 "\"
         echo "                    -yT" `getZFSize.com $inName CUR_ZDIM $zNUSZFArgs` -yP0 $zP0 -yP1 $zP1 "\"
         echo "                    -zT" `getZFSize.com $inName CUR_ADIM $aNUSZFArgs` -zP0 $aP0 -zP1 $aP1 $smileFTArgs "\"
         echo "| pipe2xyz -out $ft1Name -x -ov"
         echo ""
         echo "xyz2pipe -in $ft1Name -x -$vFlag \"

         if (`string notcsame $yPREFT NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yPREFT` "\"
         endif

         echo "| nmrPipe -fn" ZF `expandArg $yZFARG` "\"
         echo "| nmrPipe -fn" FT `expandArg $yFTARG` "\"
         echo "| nmrPipe -fn" PS -p0 $yP0 -p1 $yP1 -di "\"

         if (`string notcsame $yBASEARG NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yBASEARG` "\"
         endif

         if (`string notcsame $yPOSTFT NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yPOSTFT` "\"
         endif

         echo "| nmrPipe -fn" TP "\"
         echo "| nmrPipe -fn" ZTP "\"
         echo "| pipe2xyz -out $ft2Name -a -ov"
         echo ""
         echo "xyz2pipe -in $ft2Name -z -$vFlag \"
      else if (`string csame $reconMode hmsIST`) then
         echo "| pipe2xyz -out $tmpName -a -ov"
         echo ""
         echo "xyz2pipe -in $tmpName -x \"
         echo "| $hmsISTProg" `expandArg $uHMSARGS` "-format padded_pipe -sched $hmsSchedName -dim 3 -itr $hmsIter -xN" `getZFSize.com $inName CUR_YDIM $yNUSZFArgs` "-yN" `getZFSize.com $inName CUR_ZDIM $zNUSZFArgs` "-zN" `getZFSize.com $inName CUR_ADIM $aNUSZFArgs` "-xc $yC1 -yc $zC1 -zc $aC1 -verb $vVal" "\"
         echo "| pipe2xyz -out $ft1Name -x -ov"
         echo ""
         echo "xyz2pipe -in $ft1Name -x -$vFlag \"

         if (`string notcsame $yPREFT NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yPREFT` "\"
         endif

         echo "| nmrPipe -fn" $yWinInfo "\"
         echo "| nmrPipe -fn" ZF `expandArg $yZFARG` "\"
         echo "| nmrPipe -fn" FT `expandArg $yFTARG` "\"
         echo "| nmrPipe -fn" PS -p0 $yP0 -p1 $yP1 -di "\"

         if (`string notcsame $yBASEARG NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yBASEARG` "\"
         endif

         if (`string notcsame $yPOSTFT NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yPOSTFT` "\"
         endif

         echo "| nmrPipe -fn" TP "\"
         echo "| nmrPipe -fn" ZTP "\"
         echo "| pipe2xyz -out $ft2Name -a -ov"
         echo ""
         echo "xyz2pipe -in $ft2Name -z -$vFlag \"
      else if (`string csame $reconMode nesta`) then
         echo "| pipe2xyz -out $tmpDir/test%03d.ft1 -x -ov"
         echo ""

         prettyPrintCmnd.tcl -nocomment \
            $nestaProg -$vFlag -ndim 3 -in $tmpDir/test%03d.ft1 -out $ft1Name -tmpDir $auxDir $nusZFArgs -sample $sampleName $nestaArgs

         echo ""
         echo "xyz2pipe -in $ft1Name -x -$vFlag \"
         echo "| nmrPipe -fn" TP "\"

         if (`string notcsame $yPREFT NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yPREFT` "\"
         endif

         if (`string notcsame $yLP NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yLP` "\"
         endif

         echo "| nmrPipe -fn" $yWinInfo "\"
         echo "| nmrPipe -fn" ZF `expandArg $yZFARG` "\"
         echo "| nmrPipe -fn" FT `expandArg $yFTARG` "\"
         echo "| nmrPipe -fn" PS -p0 $yP0 -p1 $yP1 -di "\"

         if (`string notcsame $yBASEARG NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yBASEARG` "\"
         endif

         if (`string notcsame $yPOSTFT NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yPOSTFT` "\"
         endif

         echo "| nmrPipe -fn" TP "\"
      else
         echo "| nmrPipe -fn" TP "\"

         if (`string notcsame $yPREFT NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yPREFT` "\"
         endif

         echo "| nmrPipe -fn" $yWinInfo "\"
         echo "| nmrPipe -fn" ZF `expandArg $yZFARG` "\"
         echo "| nmrPipe -fn" FT `expandArg $yFTARG` "\"
         echo "| nmrPipe -fn" PS -p0 $yP0 -p1 $yP1 -di "\"
   
         if (`string notcsame $yBASEARG NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yBASEARG` "\"
         endif

         if (`string notcsame $yPOSTFT NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yPOSTFT` "\"
         endif

         echo "| nmrPipe -fn" TP "\"
      endif

      if (`string notcsame $xBASEARG NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $xBASEARG` "\"
      endif

      echo "| nmrPipe -fn" ZTP "\"

      if (`string notcsame $zPREFT NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $zPREFT` "\"
      endif

      if (`string notcsame $reconMode smile`) then
         echo "| nmrPipe -fn" $zWinInfo "\"
      endif

      echo "| nmrPipe -fn" ZF `expandArg $zZFARG` "\"
      echo "| nmrPipe -fn" FT `expandArg $zFTARG` "\"
      echo "| nmrPipe -fn" PS -p0 $zP0 -p1 $zP1 -di "\"

      if (`string notcsame $zBASEARG NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $zBASEARG` "\"
      endif

      if (`string notcsame $zPOSTFT NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $zPOSTFT` "\"
      endif

      echo "| nmrPipe -fn" ZTP "\"
      echo "| pipe2xyz -out $ft3Name -x -ov"
      echo ""

      echo "xyz2pipe -in $ft3Name -a -$vFlag \"

      if (`string notcsame $aLP NULL`) then
        if (`string csame $reconMode smile`) then
            echo "| nmrPipe -fn" APOD -qName $aAPOD -inv -hdr "\"
            echo "| nmrPipe -fn" `expandArg0 $aLP` "\"
            echo "| nmrPipe -fn" APOD -qName $aAPOD -hdr "\"
         else
            echo "| nmrPipe -fn" `expandArg0 $aLP` "\"
         endif
      endif

      if (`string notcsame $reconMode smile`) then
         echo "| nmrPipe -fn" $aWinInfo "\"
      endif

      echo "| nmrPipe -fn" ZF `expandArg $aZFARG` "\"
      echo "| nmrPipe -fn" FT `expandArg $aFTARG` "\"
      echo "| nmrPipe -fn" PS -p0 $aP0 -p1 $aP1 -di "\"

      if (`string notcsame $aBASEARG NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $aBASEARG` "\"
      endif

      echo "| pipe2xyz -out $lp1Name -a -ov"
      echo ""

      echo "xyz2pipe -in $lp1Name -x -$vFlag \"

      if (`string csame $yLP NULL`) then
         if ($yzaFlag == 2) then
            echo "| nmrPipe -fn" TP "\"
         endif
      else
         echo "| nmrPipe -fn" TP "\"
         echo "| nmrPipe -fn" HT -auto "\"
         echo "| nmrPipe -fn" PS -inv -hdr "\"
         echo "| nmrPipe -fn" FT -inv "\"
         echo "| nmrPipe -fn" ZF -inv "\"
         echo "| nmrPipe -fn" APOD -qName $yAPOD -inv -hdr "\"
         echo "| nmrPipe -fn" `expandArg0 $yLP` "\"
         echo "| nmrPipe -fn" APOD -qName $yAPOD -hdr "\"
         echo "| nmrPipe -fn" ZF `expandArg $yZFARG` "\"
         echo "| nmrPipe -fn" FT "\"
         echo "| nmrPipe -fn" PS -hdr -di "\"

         if ($yzaFlag == 0) then
            echo "| nmrPipe -fn" TP "\"
         endif
      endif

      if ($yzaFlag == 2) then
         set cleanList = "$cleanList $lp2Dir"

         echo "| pipe2xyz -out $lp2Name -a -ov"
         echo ""
   
         echo "xyz2pipe -in $lp2Name -x -$vFlag \"
         echo "| nmrPipe -fn" TP "\"
      endif

      if (`string csame $zLP NULL`) then
         if ($yzaFlag == 2) then
            echo "| pipe2xyz -out $outName -z -ov"
         else if ($yzaFlag == 1) then
            echo "| pipe2xyz -out $outName -z -ov"
         else
            echo "| pipe2xyz -out $outName -x -ov"
         endif
      else
         if ($yzaFlag != 2) then
            echo "| nmrPipe -fn" ZTP "\"
         endif

         echo "| nmrPipe -fn" HT -auto "\"
         echo "| nmrPipe -fn" PS -inv -hdr "\"
         echo "| nmrPipe -fn" FT -inv "\"
         echo "| nmrPipe -fn" ZF -inv "\"
         echo "| nmrPipe -fn" APOD -qName $zAPOD -inv -hdr "\"
         echo "| nmrPipe -fn" `expandArg0 $zLP` "\"
         echo "| nmrPipe -fn" APOD -qName $zAPOD -hdr "\"
         echo "| nmrPipe -fn" ZF `expandArg $zZFARG` "\"
         echo "| nmrPipe -fn" FT "\"
         echo "| nmrPipe -fn" PS -hdr -di "\"

         if ($yzaFlag == 2) then
            echo "| nmrPipe -fn" ZTP "\"
            echo "| pipe2xyz -out $outName -y -ov"
         else if ($yzaFlag == 1) then
            if (`string csame $yLP NULL`) then
               echo "| nmrPipe -fn" TP "\"
               echo "| nmrPipe -fn" ZTP "\"
               echo "| pipe2xyz -out $outName -x -ov"
            else
               echo "| pipe2xyz -out $outName -y -ov"
            endif
         else
            echo "| nmrPipe -fn" ZTP "\"
            echo "| pipe2xyz -out $outName -x -ov"
         endif
      endif
   else
      echo "xyz2pipe -in $inName -x -$vFlag \"

      if (`string notcsame $xPREFT NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $xPREFT` "\"
      endif

      if (`string notcsame $xSOL NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $xSOL` "\"
      endif

      echo "| nmrPipe -fn" $xWinInfo "\"
      echo "| nmrPipe -fn" ZF `expandArg $xZFARG` "\"
      echo "| nmrPipe -fn" FT `expandArg $xFTARG` "\"

      if ($usePSEXTFlag) then
         echo "| nmrPipe -fn" PS -p0 $xP0 -p1 $xP1 "\"
      else
         echo "| nmrPipe -fn" PS -p0 $xP0 -p1 $xP1 -di "\"
      endif

      if (`string notcsame $xBASEARG_INIT NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $xBASEARG_INIT` "\"
      endif

      echo "| nmrPipe -fn" EXT -x1 $xEXTX1 -xn $xEXTXN -sw "\"

      if ($usePSEXTFlag) then
         echo "| nmrPipe -fn" PS -p0 $xP0EXT -p1 $xP1EXT -di "\"
      endif

      if (`string notcsame $xBASEARG_FT1 NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $xBASEARG_FT1` "\"
      endif

      if (`string notcsame $xPOSTFT NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $xPOSTFT` "\"
      endif

      if (`string csame $reconMode smile`) then
         echo "| pipe2xyz -out $tmpName -a -ov"
         echo ""
         echo "xyz2pipe -in $tmpName -x \"
         echo "| $smileProg -fn SMILE" $smileArgs "\"

         if (`string same $yGOFF 0.0`) then
            echo "                    -xApod" $yAPOD -xQ1 $yQ1 -xQ2 $yQ2 -xQ3 $yQ3 -xELB $yELB -xGLB $yGLB "\"
         else
            echo "                    -xApod" $yAPOD -xQ1 $yQ1 -xQ2 $yQ2 -xQ3 $yQ3 -xELB $yELB -xGLB $yGLB -xGOFF $yGOFF "\"
         endif

         if (`string same $zGOFF 0.0`) then
            echo "                    -yApod" $zAPOD -yQ1 $zQ1 -yQ2 $zQ2 -yQ3 $zQ3 -yELB $zELB -yGLB $zGLB "\"
         else
            echo "                    -yApod" $zAPOD -yQ1 $zQ1 -yQ2 $zQ2 -yQ3 $zQ3 -yELB $zELB -yGLB $zGLB -yGOFF $zGOFF "\"
         endif

         if (`string same $aGOFF 0.0`) then
            echo "                    -zApod" $aAPOD -zQ1 $aQ1 -zQ2 $aQ2 -zQ3 $aQ3 -zELB $aELB -zGLB $aGLB "\"
         else
            echo "                    -zApod" $aAPOD -zQ1 $aQ1 -zQ2 $aQ2 -zQ3 $aQ3 -zELB $aELB -zGLB $aGLB -zGOFF $aGOFF "\"
         endif

         echo "                    -xT" `getZFSize.com $inName CUR_YDIM $yNUSZFArgs` -xP0 $yP0 -xP1 $yP1 "\"
         echo "                    -yT" `getZFSize.com $inName CUR_ZDIM $zNUSZFArgs` -yP0 $zP0 -yP1 $zP1 "\"
         echo "                    -zT" `getZFSize.com $inName CUR_ADIM $aNUSZFArgs` -zP0 $aP0 -zP1 $aP1 $smileFTArgs "\"
         echo "| pipe2xyz -out $ft1Name -x -ov"
         echo ""
         echo "xyz2pipe -in $ft1Name -x -$vFlag \"

         if (`string notcsame $yPREFT NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yPREFT` "\"
         endif

         if (`string notcsame $yLP NULL`) then
            echo "| nmrPipe -fn" APOD -qName $yAPOD -inv -hdr "\"
            echo "| nmrPipe -fn" `expandArg0 $yLP` "\"
            echo "| nmrPipe -fn" APOD -qName $yAPOD -hdr "\"
         endif

         echo "| nmrPipe -fn" ZF `expandArg $yZFARG` "\"
         echo "| nmrPipe -fn" FT `expandArg $yFTARG` "\"
         echo "| nmrPipe -fn" PS -p0 $yP0 -p1 $yP1 -di "\"

         if (`string notcsame $yBASEARG NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yBASEARG` "\"
         endif

         if (`string notcsame $yPOSTFT NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yPOSTFT` "\"
         endif

         echo "| nmrPipe -fn" TP "\"
         echo "| nmrPipe -fn" ZTP "\"
         echo "| pipe2xyz -out $ft2Name -a -ov"
         echo ""
         echo "xyz2pipe -in $ft2Name -z -$vFlag \"
      else if (`string csame $reconMode hmsIST`) then
         echo "| pipe2xyz -out $tmpName -a -ov"
         echo ""
         echo "xyz2pipe -in $tmpName -x \"
         echo "| $hmsISTProg" `expandArg $uHMSARGS` "-format padded_pipe -sched $hmsSchedName -dim 3 -itr $hmsIter -xN" `getZFSize.com $inName CUR_YDIM $yNUSZFArgs` "-yN" `getZFSize.com $inName CUR_ZDIM $zNUSZFArgs` "-zN" `getZFSize.com $inName CUR_ADIM $aNUSZFArgs` "-xc $yC1 -yc $zC1 -zc $aC1 -verb $vVal" "\"
         echo "| pipe2xyz -out $ft1Name -x -ov"
         echo ""
         echo "xyz2pipe -in $ft1Name -x -$vFlag \"

         if (`string notcsame $yPREFT NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yPREFT` "\"
         endif

         if (`string notcsame $yLP NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yLP` "\"
         endif

         echo "| nmrPipe -fn" $yWinInfo "\"
         echo "| nmrPipe -fn" ZF `expandArg $yZFARG` "\"
         echo "| nmrPipe -fn" FT `expandArg $yFTARG` "\"
         echo "| nmrPipe -fn" PS -p0 $yP0 -p1 $yP1 -di "\"

         if (`string notcsame $yBASEARG NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yBASEARG` "\"
         endif

         if (`string notcsame $yPOSTFT NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yPOSTFT` "\"
         endif

         echo "| nmrPipe -fn" TP "\"
         echo "| nmrPipe -fn" ZTP "\"
         echo "| pipe2xyz -out $ft2Name -a -ov"
         echo ""
         echo "xyz2pipe -in $ft2Name -z -$vFlag \"
      else if (`string csame $reconMode nesta`) then
         echo "| pipe2xyz -out $tmpDir/test%03d.ft1 -x -ov"
         echo ""

         prettyPrintCmnd.tcl -nocomment \
            $nestaProg -$vFlag -ndim 3 -in $tmpDir/test%03d.ft1 -out $ft1Name -tmpDir $auxDir $nusZFArgs -sample $sampleName $nestaArgs 

         echo ""
         echo "xyz2pipe -in $ft1Name -x -$vFlag \"
         echo "| nmrPipe -fn" TP "\"

         if (`string notcsame $yPREFT NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yPREFT` "\"
         endif

         if (`string notcsame $yLP NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yLP` "\"
         endif

         echo "| nmrPipe -fn" $yWinInfo "\"
         echo "| nmrPipe -fn" ZF `expandArg $yZFARG` "\"
         echo "| nmrPipe -fn" FT `expandArg $yFTARG` "\"
         echo "| nmrPipe -fn" PS -p0 $yP0 -p1 $yP1 -di "\"

         if (`string notcsame $yBASEARG NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yBASEARG` "\"
         endif

         if (`string notcsame $yPOSTFT NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yPOSTFT` "\"
         endif

         echo "| nmrPipe -fn" TP "\"
      else
         echo "| nmrPipe -fn" TP "\"

         if (`string notcsame $yPREFT NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yPREFT` "\"
         endif

         if (`string notcsame $yLP NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yLP` "\"
         endif

         echo "| nmrPipe -fn" $yWinInfo "\"
         echo "| nmrPipe -fn" ZF `expandArg $yZFARG` "\"
         echo "| nmrPipe -fn" FT `expandArg $yFTARG` "\"
         echo "| nmrPipe -fn" PS -p0 $yP0 -p1 $yP1 -di "\"

         if (`string notcsame $yBASEARG NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yBASEARG` "\"
         endif

         if (`string notcsame $yPOSTFT NULL`) then
            echo "| nmrPipe -fn" `expandArg0 $yPOSTFT` "\"
         endif

         echo "| nmrPipe -fn" TP "\"
      endif

      if (`string notcsame $xBASEARG NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $xBASEARG` "\"
      endif

      echo "| nmrPipe -fn" ZTP "\"

      if (`string notcsame $zLP NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $zLP` "\"
      endif

      echo "| nmrPipe -fn" $zWinInfo "\"
      echo "| nmrPipe -fn" ZF `expandArg $zZFARG` "\"
      echo "| nmrPipe -fn" FT `expandArg $zFTARG` "\"
      echo "| nmrPipe -fn" PS -p0 $zP0 -p1 $zP1 -di "\"

      if (`string notcsame $zBASEARG NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $zBASEARG` "\"
      endif

      if (`string notcsame $zPOSTFT NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $zPOSTFT` "\"
      endif

      if ($yzaFlag == 0) then
         echo "| nmrPipe -fn" ZTP "\"
         echo "| pipe2xyz -out $ft3Name -x -ov"
         echo ""
         echo "xyz2pipe -in $ft3Name -a -$vFlag \"
      else if ($yzaFlag == 1) then
         echo "| nmrPipe -fn" TP "\"
         echo "| pipe2xyz -out $ft3Name -a -ov"
         echo ""
         echo "xyz2pipe -in $ft3Name -z -$vFlag \"
      else
         echo "| nmrPipe -fn" TP "\"
         echo "| nmrPipe -fn" ZTP "\"
         echo "| pipe2xyz -out $ft3Name -x -ov"
         echo ""
         echo "xyz2pipe -in $ft3Name -a -$vFlag \"
      endif

      if (`string notcsame $aLP NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $aLP` "\"
      endif

      echo "| nmrPipe -fn" $aWinInfo "\"
      echo "| nmrPipe -fn" ZF `expandArg $aZFARG` "\"
      echo "| nmrPipe -fn" FT `expandArg $aFTARG` "\"
      echo "| nmrPipe -fn" PS -p0 $aP0 -p1 $aP1 -di "\"

      if (`string notcsame $aBASEARG NULL`) then
         echo "| nmrPipe -fn" `expandArg0 $aBASEARG` "\"
      endif

      if ($yzaFlag == 0) then
         echo "| pipe2xyz -out $thisOutName -a -ov"
      else if ($yzaFlag == 1) then
         echo "| nmrPipe -fn TP \"
         echo "| nmrPipe -fn ZTP \"
         echo "| nmrPipe -fn TP \"
         echo "| pipe2xyz -out $thisOutName -a -ov"
      else
         echo "| pipe2xyz -out $thisOutName -y -ov"
      endif
   endif

   if (`string csame $reconMode ist`) then
      set skipArgs = "-in= -out= -istName= -noexec -list -nusZF -nonusZF -ist"
      set istArgs  = ""

      if (`string csame $istMaxRes Auto`) then
         echo ""
         echo "set istMaxRes = ("\`specStat.com -in $thisOutName -brief -stat istMaxRes\`")"
         set istArgs  = '-istMaxRes $istMaxRes'
         set skipArgs = "$skipArgs -istMaxRes="
      endif

      echo ""
      echo "#"
      echo "# Arguments duplicated later on the command line are ignored:"
      echo ""

      prettyPrintCmnd.tcl -nocomment \
        ist4D.com -us $istArgs -nusZF -in $inName -istName $istName -mask Auto `stripArgs $skipArgs -- $*`
   else
      if (`string notcsame $xBASEARG_FINAL NULL`) then
         echo ""
         echo "xyz2pipe -in $thisOutName -x \"
         echo "| nmrPipe -fn" `expandArg0 $xBASEARG_FINAL` "\"
         echo "| pipe2xyz -out $thisOutName -x -inPlace"
         echo ""
      endif
   endif

   if ($cleanFlag) then
      echo ""
      echo "/bin/rm -rf $cleanList"
   else
      echo ""
      echo "#"
      echo "# /bin/rm -rf $cleanList"
   endif
else
   basicFT4.com $* -list > $scriptName
   chmod a+rx $scriptName

   if ($execFlag) then
      $scriptName

      if (`string same $vFlag verb`) then
         echo ""
      endif
   endif
endif

exit 0
