#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

proc cacbInitVars {} \
{
   global inName outName cacbName rcName gw g1D g2D rcTab inTab cacbTab probTab
   global titleFont cacbDir env stdfont

   setOSType
   setFonts

   set cacbName cacb.tab
   set rcName   randcoil.tab
   set inName   shift.tab
   set outName  prob.tab

   if {[info exists env(NMRTXT)]} \
      {
       set cacbDir  $env(NMRTXT)
      } \
   else \
      {
       set cacbDir  .
      }

   set titleFont $stdfont
   set resList   ""

   set gw       1.0
   set g1D      1.0
   set g2D      1.0

   set rcTab    0
   set inTab    0
   set cacbTab  0
   set cacbTab  probTab
}

proc cacbInitArgs {} \
{
   global ARGC ARGV inName outName cacbDir cacbName rcName gw

   if {[flagLoc $ARGV -help]} \
      {
       puts " -in"
       puts " -out"
       puts " -dir"
       exit 1
      }

   getArgD $ARGV -in     inName
   getArgD $ARGV -out    outName 
   getArgD $ARGV -dir    cacbDir
   getArgD $ARGV -cacb   cacbName 
   getArgD $ARGV -rc     rcName
   getArgD $ARGV -gw     gw 
}

proc cacbInitTables {} \
{
   global rcName cacbDir cacbName inName inTab rcTab cacbTab
   global gw g1D g2D rcCA rcCB resList seqList

   gdbInit
   gdbCreate dBase

   set rcTab    [gdbRead table -in $cacbDir/$rcName]
   set cacbTab  [gdbRead table -in $cacbDir/$cacbName]
   set inTab    [gdbRead table -in $inName]

   set seqList  [getSeqList $inTab]

   set n        [gdbGetInfo $cacbTab -parms entryCount]

   set g1D      [expr $n*$gw*sqrt(3.14159265)]
   set g2D      [expr $n*$gw*$gw*3.14159265]

   set resList ""

   set entry [gdbFirst $rcTab]

   while {$entry} \
      {
       set resName [gdbGet $entry RESNAME]

       if {[strcmp $resName c] && [strcmp $resName #]} \
          {
           lappend resList $resName
          }

       set rcCA($resName) [gdbGet $entry CA]
       set rcCB($resName) [gdbGet $entry CB]

       set entry [gdbNext $entry]
      }
}

proc cacbInitWin {} \
{
   global axisfont titleFont idList

   frame .top

   label  .lab     -text "Index List:"
   entry  .idList  -textvariable idList -width 64

   bind .idList <Return> {+ showCACBProb}

   graph  .g -font $titleFont -title "Residue Type Probabilities" -halo 15

   bind   .g <ButtonPress-1> "showCACBPoint .g %x %y"

   .g legend configure -hide 1

   .g axis configure x -stepsize 10 \
        -tickfont $axisfont -ticklength 0.05i

   .g axis configure y -min 0.0 -max 1.0 -stepsize .1 \
        -tickfont $axisfont -ticklength 0.05i

   button .quit -text Quit -command exit

   pack .lab .idList -in .top -side left

   pack .top  -in . -side top
   pack .quit -in . -side bottom -expand 1 -fill x

   pack .g    -in . -side top -expand 1 -fill both 

   wm title . "CACB Residue Type Probabilities"
   wm geom  . =800x400+10+10

   update
}

proc gauss1D { cab } \
{
   global gw

   set p  0.0
   set x  [expr $cab/$gw]
   set q  [expr $x*$x]

   if {$q < 64.0} \
      {
       catch {set p [expr exp(-$q)]}
      }

   return $p
}

proc gauss2D { ca cb } \
{
   global gw

   set p 0.0
   set x [expr $ca/$gw]
   set y [expr $cb/$gw]
   set q [expr $x*$x + $y*$y]

   if {$q < 64.0} \
      {
       catch {set p [expr exp(-$q)]}
      }

   return $p
}

proc getProbCA { ssCA } \
{
   global gw g1D cacbTab


   set entry [gdbFirst $cacbTab]
   set p     0.0

   while {$entry} \
      {
       set thisCA [gdbGet $entry CA]
       set dCA    [expr $ssCA - $thisCA]
       set p      [expr $p + [gauss1D $dCA]]

       set entry [gdbNext $entry]
      }

   set $p [expr $p/$g1D]

   return $p
}

proc getProbCB { ssCB } \
{
   global gw g1D cacbTab


   set entry [gdbFirst $cacbTab]
   set p     0.0

   while {$entry} \
      {
       set thisCB [gdbGet $entry CB]
       set dCB    [expr $ssCB - $thisCB]
       set p      [expr $p + [gauss1D $dCB]]

       set entry [gdbNext $entry]
      }

   set $p [expr $p/$g1D]

   return $p
}

proc getProbCACB { ssCA ssCB } \
{
   global gw g2D cacbTab


   set entry [gdbFirst $cacbTab]
   set p     0.0

   while {$entry} \
      {
       set thisCA [gdbGet $entry CA]
       set thisCB [gdbGet $entry CB]
       set dCA    [expr $ssCB - $thisCA]
       set dCB    [expr $ssCB - $thisCB]
       set p      [expr $p + [gauss2D $dCA $dCB]]

       set entry [gdbNext $entry]
      }

   set $p [expr $p/$g2D]

   return $p
}

proc getThisResProb { ca cb resName } \
{
    global resList rcCA rcCB

    if {$ca < 999.0} \
       {
        set ssCA [expr $ca - $rcCA($resName)]

        if {$cb < 999.0} \
           {
            set ssCB [expr $cb - $rcCB($resName)]
            set p    [getProbCACB $ssCA $ssCB]
           } \
        else \
           {
            set p [getProbCA $ssCA]
           }
       } \
    else \
       {
        if {$cb < 999.0} \
           {
            set ssCB [expr $cb - $rcCB($resName)]
            set p    [getProbCB $ssCB]
           } \
        else \
           {
            set p [expr 1.0/([llength $resList] - 2)]
           }
       }
}

proc getNormProbs { probList } \
{
   set normList ""
   set sum      0.0

   foreach p $probList {set sum [expr $sum + $p]}

   if {$sum == 0.0} {return $probList}

   foreach p $probList {lappend normList [format %.3f [expr $p/$sum]]}

   return $normList
}

proc getAllResProb { ca cb } \
{
    global resList 

    set probList ""

    foreach resName $resList \
       {
        if {![strcmp $resName C]} \
           {
            set p1 [getThisResProb $ca $cb $resName]
            set p2 [getThisResProb $ca $cb c]
            set p  [expr $p1+$p2]
           } \
        elseif {![strcmp $resName H]} \
           {
            set p1 [getThisResProb $ca $cb $resName]
            set p2 [getThisResProb $ca $cb #]
            set p  [expr $p1+$p2]
           } \
        else \
           {
            set p [getThisResProb $ca $cb $resName]
           }

        lappend probList $p
       }

    return [getNormProbs $probList]
}

proc getAllProbs {} \
{
   global inTab resList outName probTab idList pArray
   global bigfont 

   set error [catch {set outUnit [open $outName w]}]
  
   if {$error} \
      {
       puts "Error opening output file $outName"
       exit
      }

   toplevel .msg
   wm title .msg "Probabilities ..."

   message .msg.rd -width 30c -font $bigfont -relief raised \
           -text "Please Wait ..." -bg red -fg white

   pack .msg.rd
   update

   set entry  [gdbFirst $inTab]
   set idList [gdbGet $entry INDEX]
   set n      [llength $resList]

   puts            $outUnit "VARS   INDEX $resList"
   puts -nonewline $outUnit "FORMAT %d "

   foreach resName $resList {puts -nonewline $outUnit " %.3f"}

   puts $outUnit ""

   while {$entry} \
      {
       set id [gdbGet $entry INDEX]
       set ca [gdbGet $entry CA]
       set cb [gdbGet $entry CB]

       .msg.rd configure -text "Finding probabilities for entry $id ..."
       update

       set probList [getAllResProb $ca $cb]

       puts $outUnit "$id $probList"

       for {set i 0} {$i < $n} {incr i} \
          {
           set resName [lindex $resList  $i]
           set p       [lindex $probList $i]

           set pArray($id,$resName) $p
          }
 
       set entry [gdbNext $entry]
      }

   close $outUnit

   set probTab [gdbRead table -in $outName -name "Residue Type Probabilities"]

   destroy .msg

   showTable -tab $probTab -win .probTable 
}

proc showCACBPoint { w xPix yPix } \
{

#
# x y dist index name

   set status [$w element closest $xPix $yPix cInfo]

   if {!$status} return

   set resID [expr int($cInfo(x))]
   set p     $cInfo(y)

   $w configure -title "Probability $p at RESID $resID"
}

proc showCACBProb {} \
{
   global idList resList pArray seqList

   set n     [llength $seqList]
   set nn    [llength $idList]
   set nnn   [expr $n - $nn]

   if {$nn == 0} \
      {
       set q 1.0
      } \
   else \
      {
       set q [expr 1.0/$nn]
      }

   set xData ""
   set yData ""

   for {set i 0} {$i < $nnn} {incr i} \
      {
       lappend xData [expr $i+1]

       set p 1.0
       set j $i

       foreach id $idList \
          {
           set resName [lindex $seqList $j]

           if {"$resName" != ""} \
              {
               set p [expr $p*$pArray($id,$resName)]
              }

           incr j
          }

       lappend yData $p
      }

   set pData [getNormProbs $yData]

   catch {.g element delete prob}

   .g element create prob -pixels 2 -linewidth 1 \
      -xdata $xData -ydata $pData -color darkOrchid -fill black
}

cacbInitVars
cacbInitArgs
cacbInitTables
cacbInitWin

getAllProbs
showCACBProb
