#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]
  
if {[flagLoc $ARGV -help]} \
   {
    puts "Analyze PDB H-Count vs Heavy Atom Count, Ignoring Water:"
    puts stderr " -in     pdbName \[ref.pdb\] Input PDB."
    puts stderr " -pctH   hPct    \[30.0\]    Target Minimum H Atoms, Percent." 
    puts stderr " -hetatm                   Interpret HETATMs as ATOMs."
    exit 1
   }

set inName  ref.pdb
set thresh  30.0
set hetArg  nohetatm

getArgD $ARGV -in   inName
getArgD $ARGV -pctH thresh

if {[flagLoc $ARGV -hetatm]}   {set hetArg hetatm}
if {[flagLoc $ARGV -nohetatm]} {set hetArg nohetatm}

set testVal 0
set nH      0
set nX      0
set nT      0
set pctH    0.0

if {[file exists $inName]} \
   {
    set cW "strcasecmp( RESNAME, 'HOH' ) && strcasecmp( RESNAME, 'H2O' ) && strcasecmp( RESNAME, 'WAT' )"
    set cH "strmatch( ATOMNAME, 'H*' ) || strmatch( ATOMNAME, '1H*' ) || strmatch( ATOMNAME, '2H*' ) || strmatch( ATOMNAME, '3H*' )"
    set cX "!($cH)"

    set tt [gdbRead table -in $inName -pdb -$hetArg]
    set t  [gdbSelect -new -from $tt -cond $cW]
    set nT [gdbGetInfo $t -parms entryCount]

    set hList [gdbSelect -from $t -cond $cH]
    set xList [gdbSelect -from $t -cond $cX]

    set nH    [llength $hList]
    set nX    [llength $xList]

    if {$nT != 0} {set pctH [expr 100.0*$nH/$nT]}
   }

if {$pctH >= $thresh} {set testVal 1}

puts [format "threshTest %d nH %d nX %d nT %d pctH %.2f ipctH %d" $testVal $nH $nX $nT $pctH [expr int($pctH)]]
 
exit 0
