#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
{
 puts stderr "Test an NMRPipe-related File for Proper Format:"
 puts stderr " -in      inName \[None\]   Input File to Test."
 puts stderr " -msgName mName  \[stdout\] Output File for Messages."
 puts stderr " -verb                    Verbose Mode ON." 
 puts stderr "Test Modes for File Type:"
 puts stderr " -text    File is Plain Text." 
 puts stderr " -tab     File is an NMRPipe Text Table (GDB Format)." 
 puts stderr " -star    File is an NMR-Star Format Text File."
 puts stderr " -bmrbseq File is a BMRB NMR-Star File with Sequence."
 puts stderr " -bmrb    File is a BMRB NMR-Star File with Sequence and Shifts."
 puts stderr " -gmc     File is a DYNAMO GMC Molecular Specification." 
 puts stderr " -pdb     File is a PDB Format Text File."
 puts stderr " -nmr     File is a Valid NMRPipe Spectral Data File." 
 puts stderr "Tests for File Size:"
 puts stderr " -minSize  minS  \[0\]       Minimum Allowable File Size."
 puts stderr " -maxSize  maxS  \[8388608\] Maximum Allowable File Size."
 puts stderr "Tests for Text Files:"
 puts stderr " -maxLen   maxL  \[5000\]    Maximum Allowable Line Length."
 puts stderr " -maxLines maxN  \[32000\]   Maximum Allowable Line Count."
 puts stderr " -key      kList           File has Lines with Given Keywords."
 puts stderr " -content  cList           File has Only the Given Content Words."
 puts stderr "Tests for NMRPipe-Format Text Table Files:"
 puts stderr " -var  vList  Table has Given Variables."
 puts stderr " -data vList  Table has Given DATA Statements."
 puts stderr " -atom aList  Table has Given Atom Specifications (_ I J ...)."
 puts stderr "Tests for Specific Text Table Formats:"
 puts stderr " -cs          File is an NMRPipe Chemical Shift Table." 
 puts stderr " -dc          File is an NMRPipe Dipolar Coupling Table." 
 puts stderr " -csCalc      File is an Obs vs Pred Chemical Shift Output." 
 puts stderr " -dcCalc      File is an Obs vs Pred Dipolar Coupling Output." 
 puts stderr " -dyn         File is a DYNAMO Restraint Table."
 puts stderr "Tests for DATA SEQUENCE Specification (cs and csCalc):"
 puts stderr " -seq                   Report SEQUENCE Mismatch Count."
 puts stderr " -maxBadSeq bMax \[None\] Maximum Allowable Mismatches."
 puts stderr "Tests for NMRPipe-Format Data Files:"
 puts stderr " -ndimMin nMin \[1\]    Minimum Allowable Dimension Count."
 puts stderr " -ndimMax nMax \[4\]    Maximum Allowable Dimension Count."
 puts stderr " -ndim    nDim        Sets Both -ndimMin and -ndimMax values."
 puts stderr " -time                All Dimensions Must be Time-Domain."
 puts stderr " -freq                All Dimensions Must be Frequency-Domain."
 exit 0
}

set inName      None
set varList     ""
set dataList    ""
set atomList    ""
set contentList ""
set keyList     ""

set verbose   0
set dbgFlag   0
set prFlag    0
set msgName   stderr
set msgUnit   stderr

set minDim   1
set maxDim   4
set ftMode   None

set maxLineLength    5000
set maxLineCount    32000 
set minFileSize         0
set maxFileSize   8388608
set maxBadSeq        None
 
foreach testType {text nmr tab pdb star bmrb bmrbseq cs dc csCalc dcCalc dyn seq gmc} \
   {
    set testFlag($testType) 0
   }

getArgD $ARGV -in      inName
getArgD $ARGV -msgName msgName

if {![strcasecmp $msgName stdout]} \
   {
    set msgUnit stdout
   } \
elseif {![strcasecmp $msgName stderr] || ![strcasecmp $msgName None]} \
   {
    set msgUnit stderr
   } \
else \
   {
    set error [catch {set msgUnit [open $msgName w]}]

    if {$error} \
       {
        puts stderr "Error Opening Message Output File $msgName"
        set msgUnit stderr
       }
   }

getListArgD $ARGV -var  varList
getListArgD $ARGV -atom atomList
getListArgD $ARGV -key  keyList
getListArgD $ARGV -data dataList

if {![strcasecmp $dataList None]} {set dataList ""}

if {[flagLoc $ARGV -nmr]}   {set testFlag(nmr) 1}
if {[flagLoc $ARGV -nonmr]} {set testFlag(nmr) 0}

foreach testType {tab pdb star bmrb bmrbseq cs dc csCalc dcCalc dyn seq gmc} \
   {
    if {[flagLoc $ARGV -$testType]}   {set testFlag($testType) 1}
    if {[flagLoc $ARGV -no$testType]} {set testFlag($testType) 0}

    if {$testFlag($testType)} {set testFlag(text) 1}
   }

if {$testFlag(nmr)} \
   {
    set minFileSize 2048

    set testFlag(text) 0
    set testFlag(tab)  0
   }

if {$testFlag(cs) || $testFlag(csCalc)} \
   {
    set testFlag(tab)  1

    lappend atomList   "_"
    lappend dataList   SEQUENCE
    lappend varList    SHIFT

    if {$testFlag(csCalc)} {lappend varList CS_OBS RC_SHIFT SIGMA}
   }

if {$testFlag(dc) || $testFlag(dcCalc)} \
   {
    set testFlag(tab) 1

    if {$testFlag(dcCalc)} {lappend dataList SAUPE EULER_ANGLES Da Dr}

    lappend atomList  I J 
    lappend varList   D

    if {$testFlag(dcCalc)} {lappend varList D_OBS DI}
   }

if {$testFlag(dyn)} \
   {
    set testFlag(tab) 1

    lappend atomList  "*"
    lappend varList   D
   }

if {$testFlag(pdb)} \
   {
    set maxLineLength 100
   }

if {$testFlag(star)} \
   {
    lappend keyList loop_ stop_
   }

if {$testFlag(bmrbseq)} \
   {
    lappend keyList loop_ _Residue_seq_code,_Entity_comp_index.ID _Residue_label,_Entity_comp_index.Comp_ID stop_
   }

if {$testFlag(bmrb)} \
   {
    lappend keyList loop_ _Residue_seq_code,_Atom_chem_shift.Seq_ID _Residue_label,_Atom_chem_shift.Comp_ID _Atom_name,_Atom_chem_shift.Atom_ID _Chem_shift_value,_Atom_chem_shift.Val stop_
   }

if {$testFlag(gmc)} \
   {
    lappend keyList version gmc_filename segment end
   }

if {[llength $keyList]}  {set testFlag(text) 1}
if {[llength $atomList]} {set testFlag(tab)  1}
if {[llength $varList]}  {set testFlag(tab)  1}

getArgD $ARGV -ndim     minDim
getArgD $ARGV -ndim     maxDim
getArgD $ARGV -ndimMin  minDim
getArgD $ARGV -ndimMax  maxDim

getArgD $ARGV -maxLen   maxLineLength
getArgD $ARGV -maxLines maxLineCount
getArgD $ARGV -minSize  minFileSize
getArgD $ARGV -maxSize  maxFileSize

getArgD $ARGV -maxBadSeq  maxBadSeq

if {[strcasecmp $maxBadSeq None]} \
   {
    set testFlag(seq) 1
   }

if {[flagLoc $ARGV -time]}   {set ftMode time}
if {[flagLoc $ARGV -freq]}   {set ftMode freq}

if {[flagLoc $ARGV -text]}   {set testFlag(text) 1}
if {[flagLoc $ARGV -notext]} {set testFlag(text) 0}

if {[flagLoc $ARGV -tab]}    {set testFlag(tab)  1}
if {[flagLoc $ARGV -notab]}  {set testFlag(tab)  0}

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -dbg]}    {set dbgFlag 1}
if {[flagLoc $ARGV -nodbg]}  {set dbgFlag 0}

#
#----------------------------------------------------------------------------

proc showMsg { msg } \
{
   global msgUnit msgName verbose

   if {$verbose} {puts $msgUnit $msg}
}

proc showMsgN { msg } \
{
   global msgUnit msgName verbose

   if {$verbose} {puts -nonewline $msgUnit $msg}
}

proc resetTextInfo { } \
{
   global textInfo keyList

   set textInfo(keyList) "# REMARK DATA VARS FORMAT ATOM HETATM MODEL ENDMDL END"

   foreach key $keyList {lappend textInfo(keyList) $key} 
   foreach key $textInfo(keyList) {set textInfo($key,count) 0}

   set textInfo(fileSize)      0
   set textInfo(keyLength)     0
   set textInfo(lineCount)     0
   set textInfo(maxLineLength) 0
   set textInfo(totalChars)    0
   set textInfo(nullChars)     0
   set textInfo(badChars)      0

   foreach key $textInfo(keyList) \
      {
       set textInfo($key,count) 0
       set nK [string length $key]

       if {$nK > $textInfo(keyLength)} {set textInfo(keyLength) $nK}
      }
}

proc testFileSize { thisName } \
{
   global textInfo minFileSize maxFileSize msgUnit verbose

   set error [catch {set n [file size $thisName]}]

   if {$error} \
      {
       puts "0"
       showMsg "Error Finding Input File Size."
       return 1
      }

   set textInfo(fileSize) $n

   if {[strcasecmp $minFileSize None]} \
      {
       if {$textInfo(fileSize) < $minFileSize} \
          {
           puts "0"
           showMsg "Format Error: File is Too Small, $n Bytes, $minFileSize Bytes Min."
           return 1
          }
      }

   if {[strcasecmp $maxFileSize None]} \
      {
       if {$textInfo(fileSize) > $maxFileSize} \
          {
           puts "0"
           showMsg "Format Error: File is Too Large, $n Bytes, $maxFileSize Bytes Max."
           return 1
          }
      }

   return 0
}

proc testText { thisName } \
{
   global textInfo maxLineLength maxLineCount dbgFlag prFlag msgUnit verbose

   set n  $textInfo(fileSize)
   set nn [expr $n + 64]

   set error [catch {set cBuff [sysMalloc $nn byte]}]

   if {$error} \
      {
       puts "0"
       showMsg "Error Allocating $nn Byte File Input Buffer."
       return 1
      }

   set inUnit [sysOpen $thisName r]

   if {$inUnit < 0} \
      {
       puts "0"
       showMsg "Error Opening Input File $thisName for Reading."
       return 1
      }

   set error [catch {sysRead $inUnit $cBuff $n}]

   if {$error} \
      {
       puts "0"
       showMsg "Error Reading $n Bytes from Input File $thisName"
       sysClose $inUnit
       return 1
      }

   sysClose $inUnit

   set textInfo(totalChars) $n
   set cList ""
   set nC    0

   for {set i 0} {$i < $n} {incr i} \
       {
        if {[strcasecmp $maxLineLength None]} \
           {
            if {$nC > $maxLineLength} \
               {
                puts "0"
                showMsg "Format Error: Input Lines Too Long, $maxLineLength Chars Max."
                checkExit 0
               }
           }

        if {[strcasecmp $maxLineCount None]} \
           {
            if {$textInfo(lineCount) > $maxLineCount} \
               {
                puts "0"
                showMsg "Format Error: Too Many Input Lines, $maxLineCount Lines Max."
                checkExit 0
               }
           }

        set c [getPtrVal $cBuff $i]

        if {$c == 0} \
           {
            incr textInfo(nullChars)
            lappend cList 32 
            incr nC
           } \
        elseif {$c == 9 || $c == 11 || $c == 12 || $c == 13} \
           {
            lappend cList 32 
            incr nC
           } \
        elseif {[string is print [format %c $c]] || $c >= 32 && $c <= 126} \
           {
            lappend cList $c 
            incr nC
           } \
        elseif {$c == 10} \
           {
            lappend cList 32 
            incr nC

            if {$nC > $textInfo(maxLineLength)} \
               {
                set textInfo(maxLineLength) $nC
               }

            if {$prFlag} {showByteChars $cList}

            updateKeyCount $cList

            incr textInfo(lineCount)
            set  cList ""
            set  nC    0

            continue
           } \
        else \
           {
            showMsg " Found Bad Character Code $c."
            incr textInfo(badChars)
            set c 32 
           }
       }

   if {$nC} \
      {
       updateKeyCount $cList
       incr textInfo(lineCount)
      }

   if {$dbgFlag} \
      {
       showMsg "File $thisName"
       showMsg " Line Count:      [format %6d $textInfo(lineCount)]"
       showMsg " Max Line Length: [format %6d $textInfo(maxLineLength)]"
       showMsg " Total Chars:     [format %6d $textInfo(totalChars)]"
       showMsg " Null Chars:      [format %6d $textInfo(nullChars)]" 
       showMsg " Bad Chars:       [format %6d $textInfo(badChars)]" 

       showMsg ""
       showMsg "Key Counts:"

       foreach key $textInfo(keyList) \
          {
           showMsg " Key [format %-18s $key:] [format %6d $textInfo($key,count)]"
          }
      }

   sysFree $cBuff

   if {$textInfo(badChars)} \
      {
       puts "0"
       showMsg "Format Error: Non-Text Characters in Input File."
       return $textInfo(badChars)
      }

   return 0
}

proc showByteChars { cList } \
{
   global msgUnit

   foreach c $cList {showMsgN [format %c $c]}
   showMsg ""
}

proc updateKeyCount { cList } \
{
   global textInfo

   set nn [expr $textInfo(keyLength) + 2]

   set foundKey 0

   set s ""
   set n 0

   foreach c $cList \
      {
       if {!$n && $c == 32} continue

# Allow space # - _ , = .

       if {$c == 32 || $c == 35 || $c == 44 || $c == 45 || $c == 46 || $c == 95 || $c == 61 \
           || ($c >= 48 && $c <= 57) || ($c >= 65 && $c <= 90) || ($c >= 97 && $c <= 122)} \
          {
           append s [format %c $c]
          } \
       else \
          {
           append s " "
          }

       incr n

       if {$n > $nn} break
      }

   foreach key $textInfo(keyList) \
      {
       foreach subkey [split $key ","] \
          {
           set loc [string first [string toupper $subkey] [string toupper $s]]

           if {$loc >= 0} \
              {
               incr textInfo($key,count)
               set foundKey 1
              }
          }
      }

   return $foundKey
}

proc testNMR { thisName } \
{
   global ftMode minDim maxDim dbgFlag msgUnit verbose FDATASIZE

   set fdata 0

   set error [catch {rdFDATA $thisName fdata}]

   if {$error} \
      {
       puts "0"
       showMsg "Format Error: Not Able to Read NMRPipe-Format Binary Header."
       return 1
      }

   set fdMagic [getParm $fdata FDMAGIC     NULL_DIM]
   set fdFVal  [getParm $fdata FDFLTORDER  NULL_DIM]

   if {$fdMagic != 0.0} \
      {
       puts "0"
       showMsg "Format Error: Unexpected Initial Value in NMRPipe Header."
       return 1
      }

   set errror [catch {set d [expr abs($fdFVal - 2.345)]}]
   if {$error} {set d 1.0}

   if {$d > 0.001} \
      {
       puts "0"
       showMsg "Format Error: Unexpected Byte-Order Specification in NMRPipe Header."
       return 1
      }

   set dimCount [getParmI $fdata FDDIMCOUNT NULL_DIM]

   if {$dimCount < $minDim || $dimCount > $maxDim} \
      {
       puts "0"

       showMsgN "Format Error: NMRPipe Dim Count $dimCount is out of expected bounds"

      if {$minDim == $maxDim} \
         {
          showMsg ", ${minDim}D data is required."
         } \
      else \
         {
          showMsg " $minDim to $maxDim."
         }

       return 1
      }

   if {$dbgFlag} {showMsg "NMRPipe-Format Data Sizes:"}

   for {set dim 1} {$dim <= $dimCount} {incr dim} \
      {
       set size    [getParmI $fdata NDSIZE        $dim]
       set dimCode [getParmI $fdata FDDIMORDER$dim NULL_DIM]

       set dimName [lindex "X Y Z A B C D" [expr $dim - 1]]

       if {$dbgFlag} {showMsg " ${dimName}-Axis $dimCode Size: [format %5d $size]"}

       if {$size < 1} \
          {
           puts "0"
           showMsg "Format Error: Unexpected ${dimName}-Axis Size $size in NMRPipe Header."
           return 1
          }

       if {$dimCode < 1 || $dimCode > 4} \
          {
           puts "0"
           showMsg "Format Error: Unexpected Dimension Code $dimCode in NMRPipe Header."
           return 1
          }

       set ftFlag [getParmI $fdata NDFTFLAG $dim]

       if {![strcasecmp $ftMode time] && $ftFlag} \
          {
           puts "0"
           showMsg "Format Error: NMRPipe Time-Domain Data is Expected."
           return 1
          } \
       elseif {![strcasecmp $ftMode freq] && !$ftFlag} \
          {
           puts "0"
           showMsg "Format Error: NMRPipe Freq-Domain Data is Expected."
           return 1
          }
      }

   if {[strcmp $fdata 0]} {catch "sysFree $fdata $FDATASIZE"}

   return 0
}

proc checkExit { status } \
{
   global msgName msgUnit

   if {[strcasecmp $msgUnit stdout] && [strcasecmp $msgUnit stderr]} \
      {
       catch "close $msgUnit"
      }

   exit $status
}

if {$dbgFlag} \
   {
    showMsg "Test Modes:"

    foreach testType {text tab pdb bmrb cs dc csCalc dcCalc dyn nmr} \
       {
        showMsg " Test [format %-7s $testType:] $testFlag($testType)"
       }
   }

initResNames
resetTextInfo

set inNameOrig $inName

set inTab 0
set vList ""
set dList ""

if {![strcasecmp $inName None]} \
   {
    puts "0"
    showMsg " Error: No Input File Specified."
    checkExit 0
   }

if {![file exists $inName]} {set inName [format $inNameOrig 1 1]} 

if {![file exists $inName]} \
   {
    puts "0"
    showMsg " Error Finding Input File $inNameOrig ($inName)."
    checkExit 0
   }

set error [testFileSize $inName]
if {$error} {checkExit 0}

if {$testFlag(nmr)} \
   {
    set error [testNMR $inName]
    checkExit 0
   }

if {$testFlag(text)} \
   {
    set error [testText $inName]
    if {$error} {checkExit 0} 
   }

if {$testFlag(tab) || [llength $dataList]} \
   {
    if {$verbose} \
       {
        set sMode nosilent
        set sMode silent
       } \
    else \
       {
        set sMode silent
       }

    set error [catch {set inTab [gdbRead table -in $inName -$sMode]}]

    if {$error} \
       {
        puts "0"
        showMsg "Format Error: Unexpected Content in NMRPipe-Format Table Input."
        checkExit 0
       }

    set vList [string toupper [gdbGetInfo $inTab -parms varNames]]

    getKeyData $inTab dInfo

    if {$testFlag(tab)} {lappend keyList VARS FORMAT}
   }

if {$testFlag(pdb)} \
   {
    if {!$textInfo(ATOM,count) && !$textInfo(HETATM,count)} \
       {
        puts "0"
        showMsg "Format Error: No ATOM/HETATM Entries in PDB Input File."
        checkExit 0
       }
   }

foreach key $dataList \
   {
    if {![info exists dInfo($key)]} \
       {
        puts "0"
        showMsg "Format Error: Table is missing expected DATA $key lines."
        checkExit 0
       } \
    else \
       {
        if {$dbgFlag} {showMsg "Found Required DATA $key Statement."}
       }
   }

foreach key $keyList \
   {
    if {!$textInfo($key,count)} \
       {
        puts "0"
        showMsg "Format Error: Missing Input Table Keyword $key."
        checkExit 0
       } \
    else \
       {
        if {$dbgFlag} {showMsg "Found Required Keyword $key."}
       }
   }

foreach aType $atomList \
   {
    if {![strcmp $aType "_"]} \
       {
        lappend varList RESID RESNAME ATOMNAME
       } \
    elseif {[strcmp $aType "*"]} \
       {
        lappend varList RESID_$aType RESNAME_$aType ATOMNAME_$aType
       }
   }

set vcList [string toupper $varList]

foreach var $vcList \
   {
    if {![strcasecmp $var None]} continue
    
    if {[lsearch -exact $vList $var] < 0} \
       {
        puts "0"
        showMsg "Format Error: Missing Input Table Variable $var."
        checkExit 0
       } \
    else \
       {
        if {$dbgFlag} {showMsg "Found Required Table Variable $var."}
       }
   } 

if {[lsearch -exact $atomList "*"] >= 0} \
   {
    set dynFlag 0

    if {[lsearch -exact $vcList RESID] >= 0 && \
          [lsearch -exact $vcList RESNAME] >= 0 && \
            [lsearch -exact $vcList ATOMNAME] >= 0} \
       {
        if {$dbgFlag} {showMsg "Found DYNAMO ATOMNAME Specification."}
        set dynFlag 1
       }
     
    if {[lsearch -exact $vcList RESID_I] >= 0 && \
          [lsearch -exact $vcList RESNAME_I] >= 0 && \
            [lsearch -exact $vcList ATOMNAME_I] >= 0} \
       {
        if {$dbgFlag} {showMsg "Found DYNAMO ATOMNAME_I Specification."}
        set dynFlag 1
       }
    
    if {!$dynFlag} \
       {
        puts "0" 
        showMsg "Format Error: Expected Input Table Atom Specifications:"
        showMsg " Expected RESID RESNAME ATOMNAME or RESID_I RESNAME_I etc."
        checkExit 0
       } 
   }

set nBadSeq 0

if {$testFlag(cs) || $testFlag(csCalc)} \
   {
    set seqList [getSeqList $inTab thisR1 thisRN]
    set eList   [gdbGetInfo $inTab -parms entries]
    set minRes  [getTabMin  $inTab RESID]

    if {![info exists dInfo(FIRST_RESID)] && $minRes != 1} \
       {
        showMsg "Format Notice: Table is missing DATA FIRST_RESID specification."
        showMsg "Format Notice: Assuming FIRST_RESID value of 1."
       }

    set badList ""

    foreach e $eList \
       {
        set resID    [gdbGet $e RESID]
        set resName  [gdbGet $e RESNAME]

        set resNameT [getResName1 $resName]
        set resNameS None

        if {$resID >= $thisR1 && $resID <= $thisRN} \
           {
            set resNameS [lindex $seqList [expr $resID - $thisR1]]

            if {[strcasecmp $resNameT $resNameS]} \
               {
                if {[lsearch -exact $badList $resID,$resNameT] >= 0} continue

                lappend badList $resID,$resNameT

                if {$nBadSeq == 21} \
                   {
                    showMsg "Format Error: SEQUENCE Mismatch. Further messages skipped."
                   } \
                elseif {$nBadSeq <= 20} \
                   {
                    showMsg "Format Error: SEQUENCE Mismatch. RESID [format %3d $resID] $resNameS $resNameT."
                   }

                incr nBadSeq
               }
           }
       }

    if {[strcasecmp $maxBadSeq None]} \
       {
        if {$nBadSeq > $maxBadSeq} \
           {
            showMsg "Format Error: Too Many Sequence Mismatches: $nBadSeq, $maxBadSeq Max."

            if {$testFlag(seq)} \
               {
                puts "0 $nBadSeq"
               } \
            else \
               {
                puts "0"
               }

            checkExit 0
           }
       }
   }

if {$testFlag(seq)} \
   {
    puts "1 $nBadSeq"
   } \
else \
   {
    puts "1"
   }

checkExit 0
