#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "In-Place Format Conversions of Spectral Series Listed in a Metadata Table."
   echo "Version 2025.071.20.32"
   echo " -meta    metaName [meta.tab]     Metadata Table (INDEX DIR_NAME TITLE)."
   echo " -list    listName [fid.list]     Output List of Converted Results In Their Original Directories."
   echo " -fidName fidName  [test.fid]     Output Name of Converted FID In Its Original Directory."
   echo " -fidDir  fidDir   [fid]          Optional Output Directory for Converted Results, or Keyword None."
   echo " -srcDir  srcDir   [.]            Source Directory."
   echo "Conversion Script Generation; Should be Last on Command-Line:"
   echo " -cmnd    convCmnd [bruker -AUTO] Script Generation Command."
   echo "Notes:"
   echo "  1. The command specified by the -cmnd option should create an fid.com script."
   echo "  2. The command specified by the -cmnd option should accept -in and -out options."
   echo "  3. If the -fidDir option is used, the output files will be named according to"
   echo "     the INDEX and TITLE from the metadata table."
   echo ""
   exit 0
endif

set convCmnd = "bruker -AUTO"
set loc      = (`flagLoc $argv -cmnd`)
set thisArgv = ($argv)

if ($loc >= 1) then
   @ locP = $loc - 1
   @ locN = $loc + 1

   set convCmnd = ($argv[$locN-])
   set thisArgv = ($argv[1-$locP])
endif

set srcDir   = (`getArgD $thisArgv -srcDir  .`)
set fidDir   = (`getArgD $thisArgv -fidDir  fid`)
set fidName  = (`getArgD $thisArgv -fidName test.fid`)
set tabName  = (`getArgD $thisArgv -meta    meta.tab`)
set listName = (`getArgD $thisArgv -list    fid.list`)

#
#----------------------------------------------------------------------------------

set origDir = (`pwd`)

cd $srcDir
   set srcDirFull = (`pwd`)
cd $origDir

/bin/rm -f $listName

if (`string csame $fidDir None`) then
   set fidDirFull = None
else
   /bin/rm -rf $fidDir
   mkdir $fidDir

   cd $fidDir
      set fidDirFull = (`pwd`)
   cd $origDir
endif

set idList  = (`getTabCol.tcl -in $tabName -var INDEX`)
set dirList = (`getTabCol.tcl -in $tabName -var DIR_NAME`)
set titList = (`getTabCol.tcl -in $tabName -var TITLE`)

echo ""

@ eCount = 0
@ i = 0

foreach d ($dirList)
   @ i++

   if (!(-d $srcDirFull/$d)) then
      echo "Conversion Error Finding Input Directory $srcDirFull/$d"
      @ eCount++
      continue
   endif

   cd $srcDirFull/$d
      set thisDir   = (`pwd`)
      set thisTitle = (`nmrPrintf "%03d-%s" $idList[$i] $titList[$i]`)

      /bin/rm -f $fidName mask.fid fid.com conv.out

      $convCmnd -user -title $thisTitle >& conv.out

      if (!(-e fid.com)) then
         echo "Conversion Error Creating Conversion Script $srcDirFull/$d/fid.com"
         @ eCount++
         continue
      endif

      fid.com >& conv.out

      if (!(-e $fidName)) then
         echo "Conversion Error Creating $srcDirFull/$d/$fidName"
         @ eCount++
         continue
      endif

      if (`string csame $fidDir None`) then
         set outName = $srcDirFull/$d/$fidName 
      else
         set outName = $fidDirFull/${thisTitle}.fid
         /bin/cp $fidName $outName
      endif

      echo $thisDir/$fidName >> $origDir/$listName

      echo "Convertinng FID $i of $#idList"':'
      echo " Input Directory:  $thisDir"
      echo " Output FID:       $outName"
      echo " Title:            $thisTitle"
      echo ""

      reportND.com $outName
      echo ""
   cd $origDir
end

if ($eCount != 0) then
   echo "Conversion Notice: Errors Occurred During Conversion."
   exit 1
endif

exit 0
