#!/bin/csh

if (`flagLoc $argv -help` || $#argv < 2) then
   echo "Copy Tau or other header values from one series to another. Examples:"
   echo "   copyTau.com sourceName destName [-parm FDTAU ...] [-all] [-verb|-noverb]"
   echo "Examples:"
   echo "   copyTau.com fid/test%03d.fid ft/test%03d.dat"
   echo "   copyTau.com fid/test%03d.fid ft/test%03d.dat -noverb"
   echo "   copyTau.com fid/test%03d.fid ist/test%03d.dat -parm FDTAU FDTEMPERATURE"
   echo "Notes:"
   echo " 1. Any options (-parm -all -verb -noverb) must be last on the command line."
   echo " 2. Option -all is equivalent to:"
   echo "    -parm FDTITLE FDCOMMENT FDTAU FDTEMPERATURE FDPRESSURE FDUSER1 ... FDUSER6"
   exit 0
endif 

set inName   = $argv[1]
set outName  = $argv[2]
set defParms = "FDTAU"
set vFlag    = 1

if (`flagLoc $argv -verb`) then
   set vFlag = 1
endif

if (`flagLoc $argv -noverb`) then
   set vFlag = 0
endif

if (`flagLoc $argv -all`) then
   set defParms = "FDTITLE FDCOMMENT FDTAU FDTEMPERATURE FDPRESSURE FDUSER1 FDUSER2 FDUSER3 FDUSER4 FDUSER5 FDUSER6"
endif

set parmList = (`getListArgD $argv -parm -- $defParms`)

if ($vFlag) then
   echo ""
endif

set i = 1

while ($i <= 99999999)
   set thisInName  = (`nmrPrintf $inName  $i`)
   set thisOutName = (`nmrPrintf $outName $i`)

   if (!(-e $thisInName) || !(-e $thisOutName)) then
      exit 0
   endif

   @ nU = 0

   foreach parm ($parmList)
      if (`string csame $parm FDTITLE`) then
         set oldVal = (`getParm -in $thisOutName -parm $parm -text -fmt %s`)
         set newVal = (`getParm -in $thisInName  -parm $parm -text -fmt %s`)

         if (`string same "$oldVal" "$newVal"` == 0) then
            if ($vFlag) then
               echo "Updating $thisOutName ${parm} Old: $oldVal New: $newVal"
            endif

            @ nU++
         endif

         sethdr $thisOutName -title "$newVal"
      else if (`string csame $parm  FDCOMMENT`) then
         set oldVal = (`getParm -in $thisOutName -parm $parm -text -fmt %s`)
         set newVal = (`getParm -in $thisInName  -parm $parm -text -fmt %s`)

         if (`string same "$oldVal" "$newVal"` == 0) then
            if ($vFlag) then
               echo "Updating $thisOutName ${parm} Old: $oldVal New: $newVal"
            endif

            @ nU++
         endif

         sethdr $thisOutName -comment "$newVal"
      else
         set oldVal = (`getParm -in $thisOutName -parm $parm`)
         set newVal = (`getParm -in $thisInName  -parm $parm`)

         if (`MATH "$oldVal != $newVal"`) then
            if ($vFlag) then
               echo "Updating $thisOutName ${parm} Old: $oldVal New: $newVal"
            endif

            @ nU++
         endif

         sethdr $thisOutName -fdata $parm $newVal
      endif
   end

   if ($nU == 0) then
      if ($vFlag) then
         echo "No Updates for $thisOutName"
      endif
   endif

   @ i++
end

if ($vFlag) then
   echo ""
endif

