#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"

set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
# INDEX 
# X_PPM Y_PPM Z_PPM X_AXIS Y_AXIS Z_AXIS DX DY DZ X1  X3  Y1  Y3  Z1  Z3  XW_HZ YW_HZ ZW_HZ 
# XTW   YTW   ZTW   X_P0  X_P1 Y_P0  Y_P1  Z_P0  Z_P1  X_COSJ1 X_COSJ2 Y_COSJ1 Y_COSJ2 Z_COSJ1 Z_COSJ2
# HEIGHT DHEIGHT VOL PCHI2 TYPE CLUSTID MEMCNT ASS

set sInfoList \
 {
 {h           h             h*}
 {c           c             c*}
 {hn          hn,n          hn/n}
 {hnco        hn,n,co       hn/n/co(i-1)}
 {hn(ca)co    hn,n,co       hn/n/co(i)     hn/n/co(i-1)}
 {cbcanh      hn,n,cacb    +hn/n/ca(i)    -hn/n/cb(i)   +hn/n/ca(i-1) -hn/n/cb(i-1)}
 {cbcaconh    hn,n,cacb     hn/n/ca(i-1)   hn/n/cb(i-1)}
 {cbca(co)nh  hn,n,cacb     hn/n/ca(i-1)   hn/n/cb(i-1)}
 {hnca        hn,n,ca       hn/n/ca(i)     hn/n/ca(i-1)}
 {hn(co)ca    hn,n,ca       hn/n/ca(i-1)}
 {hnha        hn,n,ha      +hn/n/ha(i)    -hn/n/hn(i)}
 {hn(co)ha    hn,n,ha       hn/n/ha(i-1)}
 {hbhanh      hn,n,hahb     hn/n/ha(i)     hn/n/hb(i)   hn/n/ha(i-1)  hn/n/hb(i-1)}
 {hbha(co)nh  hn,n,hahb     hn/n/ha(i-1)   hn/n/hb(i-1) hn/n/hahb}
 {hacaco      ha,ca,co      ha/ca/co(i)}
 {haca        ha,ca         ha/ca}
 {h(n)co      hn,co         hn/co(i-1)}
 {h(nca)co    hn,co         hn/co(i)}
 {h(n)ca      hn,ca         hn/ca(i)       hn/ca(i-1)}
 {h(nco)ca    hn,ca         hn/ca(i-1)}
 {cbca(n)h    hn,cacb      +hn/ca(i)      -hn/cb(i)    +hn/ca(i-1)   -hn/cb(i-1)}
 {cbca(con)h  hn,cacb       hn/ca(i-1)     hn/cb(i-1)}
 {h(n)ha      hn,ha        +hn/ha(i)      -hn/hn(i)}
 {h(nco)ha    hn,ha         hn/ha(i-1)}
 {hbha(n)h    hn,hahb       hn/ha(i)       hn/hb(i)     hn/ha(i-1)    hn/hb(i-1)}
 {hbha(con)h  hn,hahb       hn/ha(i-1)     hn/hb(i-1)   hn/hahb}
 {cbcanh4d    hn,n,cacb,co +hn/n/ca(i)/co(i-1)    -hn/n/cb(i)/co(i-1) +hn/n/ca(i-1)/co(i-1) -hn/n/cb(i-1)/co(i-1)}
 {cbcaconh4d  hn,n,cacb,co  hn/n/ca(i-1)/co(i-1)   hn/n/cb(i-1)/co(i-1)}
}

if {[flagLoc $ARGV -help]} \
{
 puts stderr "Create NMRPipe Peak Table from Chemical Shift table:"
 puts stderr " cs2pk.tcl \[options\] \[spectralParameterOptions\]"
 puts stderr " -in     inName  \[csObs.tab\]  TALOS Format CS Table Input."
 puts stderr " -ref    refName \[fid.com\]    Reference Conversion/Simulation."
 puts stderr " -ft     ftName  \[None\]       Optional Reference Spectrum."
 puts stderr " -out    outName \[simTD.tab\]  NMRPipe Format Output Peak Table."
 puts stderr " -com    comName \[simTD.com\]  Output Simulation Script."
 puts stderr " -sim    fidName \[Auto\]       NMRPipe Format Output FID, or Keyword stdout."
 puts stderr " -type   sType   \[hn\]         Spectrum Type."
 puts stderr " -var    sVar    \[SHIFT\]      CS Table Shift Input Variable." 
 puts stderr " -hi     defHi   \[100.0\]      Default Signal Amplitude."
 puts stderr " -noise  sigma   \[1.0\]        Std Dev of Gaussian Noise."
 puts stderr " -full                          Include Full Set of Output Peak Table Variables."
 puts stderr "Axis Order; use keywords X Y Z or A, such as: X Z Y:" 
 puts stderr " -tAxis  tList   \[Auto\]       Time Domain (FID) Axis Order."
 puts stderr " -fAxis  fList   \[Auto\]       Freq Domain (Peak Table) Order."
 puts stderr "Time-Domain Exponential Decay, Values in Hz:"
 puts stderr " -xlw    xw      \[10.0\]       Decay for Signal X-Axis."
 puts stderr " -ylw    yw      \[0.0\]        Decay for Signal Y-Axis."
 puts stderr " -zlw    zw      \[0.0\]        Decay for Signal Z-Axis."
 puts stderr " -apod                        Adjust Decay to Account for Apodization (Default)."
 puts stderr " -noapod                      Do Not Adjust Decay According to Apodization."
 puts stderr "Phase Correction; Also -y... -z..."
 puts stderr " -xP0    xP0     \[0.0\]        Zero Order Phase for X-Axis."
 puts stderr " -xP1    xP1     \[0.0\]        First Order Phase for X-Axis."
 puts stderr "Other Simulation Arguments; Also -y... -z..."
 puts stderr " -xT -xFD -xN -xMODE -xSW -xOBS -xCAR -xFT -xLAB -xAPOD -xQ1 -xQ2 -xQ3 -xC1"
 puts stderr "Spectrum Types and Signal Axis:"

 foreach sItem $sInfoList \
    {
     puts -nonewline stderr " [string toupper [lindex $sItem 0]]"

     set aList [split [lindex $sItem 1] ","]
     set n     [llength $aList]
     set i     0

     puts -nonewline stderr " ${n}D"

     foreach aItem $aList \
        {
         set dim [lindex "x y z a" $i]
         puts -nonewline stderr " [string toupper $dim=$aItem]"
         incr i
        }

     puts -nonewline stderr ":\n  "

     foreach s [lrange $sItem 2 end] \
        {
         puts -nonewline stderr [format " %s" $s]
        }

     puts stderr  ""
    }

 exit 1
}

set inName     csObs.tab 
set refName    fid.com
set ftName     None
set outName    simTD.tab 
set simName    Auto
set comName    simTD.com
set shiftVar   SHIFT
set devVar     NONE
set sType      hn
set hiDef      100.0
set noise      1.0

set fullFlag   0
set apodFlag   1

set tmpHdrName tmp.hdr
set tmpTabName tmp.tab 

set tAxisInfo  Auto
set fAxisInfo  Auto

set aInfo(xlw) 10.0
set aInfo(ylw)  0.0
set aInfo(zlw)  0.0
set aInfo(alw)  0.0

getArgD $ARGV -in    inName
getArgD $ARGV -ref   refName
getArgD $ARGV -ft    ftName
getArgD $ARGV -out   outName
getArgD $ARGV -com   comName
getArgD $ARGV -sim   simName
getArgD $ARGV -type  sType
getArgD $ARGV -var   shiftVar
getArgD $ARGV -hi    hiDef 
getArgD $ARGV -noise noise

getListArgD $ARGV -tAxis tAxisInfo
getListArgD $ARGV -fAxis fAxisInfo

if {[flagLoc $ARGV -apod]}   {set apodFlag 1}
if {[flagLoc $ARGV -noapod]} {set apodFlag 0}

if {[flagLoc $ARGV -full]}   {set fullFlag 1}
if {[flagLoc $ARGV -nofull]} {set fullFlag 0}

set tAxisInfo [string tolower [join $tAxisInfo ""]]
set fAxisInfo [string tolower [join $fAxisInfo ""]]

if {![strcasecmp $tAxisInfo Auto]} {set tAxisInfo xyza}
if {![strcasecmp $fAxisInfo Auto]} {set fAxisInfo $tAxisInfo}

foreach i {x y z a} \
   {
    if {[string first $i $tAxisInfo] < 0} {append tAxisInfo $i}
    if {[string first $i $fAxisInfo] < 0} {append fAxisInfo $i}
   }

#
#-----------------------------------------------------------------------------

proc isNear { eI eJ } \
{
   global vList dList dimCount

   for {set i 0} {$i < $dimCount} {incr i} \
      {
       set v     [lindex $vList $i]
       set d     [lindex $dList $i]

       set xI  [gdbGet $eI $v]
       set xJ  [gdbGet $eJ $v]
       set dIJ [expr $xI - $xJ]

       if {abs($dIJ) > $d} {return 0}
      }

   return 1
}

proc getShifts { resID pk } \
{
   global csInfo

   set atomList  ""
   set shiftList ""

   if {[string first "(I-1)" $pk] >= 0} \
      {
       set thisResID [expr $resID - 1]
       set p "(i-1)"
      } \
   elseif {[string first "(I+1)" $pk] >= 0} \
      {
       set thisResID [expr $resID + 1]
       set p "(i+1)"
      } \
   else \
      {
       set thisResID $resID
       set p ""
      }

   set atomName [lindex [split $pk "()"] 0]

   if {[string first "*" $atomName] >= 0} \
      {
       if {![info exists csInfo($thisResID,atomList)]} {return $shiftList}

       foreach thisAtom $csInfo($thisResID,atomList) \
          {
           if {![string match $atomName $thisAtom]} continue

           if {![info exists csInfo($thisResID,$thisAtom,shift)]} continue
           if {![llength $csInfo($thisResID,$thisAtom,shift)]}    continue

           foreach shift $csInfo($thisResID,$thisAtom,shift) \
              {
               lappend shiftList $shift,$thisAtom$p
              }
          }
      } \
   else \
      {
       if {![strcasecmp $atomName HA]} \
          {
           set atomList "HA HA1 HA2 HA3"
          } \
       elseif {![strcasecmp $atomName HB]} \
          {
           set atomList "HB HB1 HB2 HB3"
          } \
       else \
          {
           set atomList $atomName
          }

       foreach atomName $atomList \
          {
           if {![info exists csInfo($thisResID,$atomName,shift)]} continue 
           if {![llength $csInfo($thisResID,$atomName,shift)]}    continue
    
           set shift [getAvg $csInfo($thisResID,$atomName,shift)]
    
           lappend shiftList $shift,$atomName$p
          }
      }

   return $shiftList
}

proc getFIDInfo {} \
{
   global refName aInfo dimCount tAxisInfo

   if {![strcasecmp $refName None]} return
 
   if {![file exists $refName]} \
      {
       puts stderr "Notice: input FID script $refName was not found."
       puts stderr "Using default spectral parameters."
       set refName None
       return
      }

   set comList "bin2pipe bruk2pipe var2pipe jeol2pipe simSpecND simTimeND"

   set inUnit [open $refName r]
   set txt    [read $inUnit]

   close $inUnit

   set tList [join $txt " "]
   set fList [split $tList |]
   set cList ""

   foreach cmnd $fList \
      {
       set loc -1

       foreach k $comList \
          {
           set loc [lsearch -exact $cmnd $k]

           if {$loc >= 0} \
              {
               set cList [lrange $cmnd $loc end]
               break
              }
          }

       if {$loc >= 0} break
      }

   if {![llength $cList]} \
      {
       puts stderr "Error finding conversion info in script $refName"
       exit 1
      }

   arg2array $cmnd aInfo

   if {![strcasecmp $aInfo(argv0) simTimeND]} \
      {
       for {set i 0} {$i < $dimCount} {incr i} \
          {
           set dim [lindex "x y z a" $i]

           if {![info exists aInfo(${dim}FD)]} \
              {
               if {[info exists aInfo(${dim}N)]} \
                  {
                   set aInfo(${dim}FD) $aInfo(${dim}N)
                  }
              }
          }
      } \
   else \
      {
       for {set i 0} {$i < $dimCount} {incr i} \
          {
           set dim [lindex "x y z a" $i]

           if {![info exists aInfo(${dim}FD)]} \
              {   
               if {[info exists aInfo(${dim}T)]} \
                  {   
                   set aInfo(${dim}FD) [nextPower2 [expr 2*$aInfo(${dim}T)]]
                  }
              }
          }
      }
}

proc getFIDArgs {} \
{
   global ARGV aInfo

   foreach dim {x y z a} \
      {
       foreach p {lw T FD N MODE SW OBS CAR FT LAB P0 P1 APOD Q1 Q2 Q3 C1} \
          {
           getArgD $ARGV -$dim$p aInfo($dim$p)
          }
      }
}

proc getSpecInfo {} \
{
   global aInfo dimInfo dimCount noise fdata tAxisInfo fAxisInfo
   global tmpHdrName tmpTabName ftName FDATASIZE

   if {$noise != 0.0} \
      {
       set thisNoise $noise
      } \
   else \
      {
       set thisNoise 1.0
      }

   set fdata [sysMalloc $FDATASIZE float]
   set cmnd  "buildHdr $fdata -ndim $dimCount"  

   if {$dimCount > 1} {append cmnd " -aq2D States"}

#
# Some parameters depend on previous ones, so order of this list matters:

   foreach p {T FD N MODE SW OBS CAR FT LAB P0 P1 APOD Q1 Q2 Q3 C1} \
      {
       for {set i 0} {$i < $dimCount} {incr i} \
          {
           set prefix [lindex "x y z a" $i]
           set dim    [lindex "x y z a" $i]
           set type   [lindex $dimInfo  $i]
      
           append cmnd " -$prefix$p [getSpecParm $dim $type $p $i Freq]"
          }
      }

   set error [catch {eval $cmnd}]

#
# If a reference spectrum exists, use it to extract header and apodization details,
# otherwise, use the header created above.

   if {[strcasecmp $ftName None]} \
      {
       set thisName [format $ftName 1 1]

       if {![file exists $thisName]} \
          {
           puts stderr "Error Finding Reference Spectrum $ftName ($thisName)."
           exit 1
          }

       set error [rdFDATA $ftName fdata]

       catch "file delete $tmpTabName"

       set cmnd "exec showApod -in $ftName -sigma $thisNoise -lb"

       for {set i 0} {$i < $dimCount} {incr i} \
          {
           set dim [lindex "x y z a" $i]
           set lb  [expr $aInfo(${dim}lw)*$aInfo(${dim}T)/$aInfo(${dim}SW)]

           append cmnd " $lb"
          }

       append cmnd " > $tmpTabName"

       set error [catch {eval $cmnd}]

       if {![file exists $tmpTabName]} \
          {
           puts stderr "Error Creating showApod Output $tmpTabName"
           exit 1
          }

       set aTab  [gdbRead table -in $tmpTabName]
       set eList [gdbGetInfo $aTab -parms entries]

       catch "file delete $tmpTabName"
      } \
   else \
      {
       catch {file delete $tmpHdrName}
       catch {file delete $tmpTabName}

       set error [catch {set outUnit [sysOpen $tmpHdrName w]}]

       sysWrite $outUnit $fdata $FDATASIZE
       sysClose $outUnit

       if {![file exists $tmpHdrName]} \
          {
           puts stderr "Error Creating Temporary Header $tmpHdrName"
           exit 1
          }

       set cmnd "exec showApod -in $tmpHdrName -sigma $thisNoise -lb"

       for {set i 0} {$i < $dimCount} {incr i} \
          {
           set dim [lindex "x y z a" $i]
           set lb  [expr $aInfo(${dim}lw)*$aInfo(${dim}T)/$aInfo(${dim}SW)]
    
           append cmnd " $lb"
          }

       append cmnd " > $tmpTabName"

       set error [catch {eval $cmnd}]

       if {![file exists $tmpTabName]} \
          {
           puts stderr "Error Creating showApod Output $tmpTabName"
           exit 1
          }

       set aTab  [gdbRead table -in $tmpTabName]
       set eList [gdbGetInfo $aTab -parms entries]

       catch "file delete $tmpHdrName"
       catch "file delete $tmpTabName"
      }

   foreach e $eList \
      {
       set dim [string tolower [string index [gdbGet $e AXIS] 0]]

       set aInfo(apod,${dim}LWFinal) [gdbGet $e LW_FINAL] 
       set aInfo(apod,${dim}LWAdj)   [gdbGet $e LW_ADJ] 
       set aInfo(apod,${dim}LAB)     [gdbGet $e LABEL] 
      }

   set str [string toupper [string range $tAxisInfo 0 [expr $dimCount - 1]]]

   puts stderr ""
   puts -nonewline stderr "Time-Domain Axis: $str"

   for {set i 0} {$i < $dimCount} {incr i} \
      {
       set dim [string index $tAxisInfo $i]
       puts -nonewline stderr " $aInfo(${dim}LAB)"
      }

   set str [string toupper [string range $fAxisInfo 0 [expr $dimCount - 1]]]

   puts stderr ""
   puts -nonewline stderr "Freq-Domain Axis: $str"

   for {set i 0} {$i < $dimCount} {incr i} \
      {
       set dim [string index $fAxisInfo $i]
       puts -nonewline stderr " $aInfo(${dim}LAB)"
      }

   puts stderr ""
}

proc initSimScript {} \
{
   global ARGV aInfo dimInfo dimCount comName outName simName tAxisInfo noise

   set error [catch {set outUnit [open $comName w 0744]}]

   if {$error} \
      {
       puts stderr "Error Creating Simulation Script $comName"
       exit 1 
      }

   puts $outUnit "#!/bin/csh"
   puts $outUnit ""
   puts $outUnit "#"
   puts $outUnit "# cs2pk.tcl [lrange $ARGV 3 end]"
   puts $outUnit ""

   puts $outUnit "simTimeND -ndim $dimCount -in $outName -rms $noise -scale 1.0 -nots \\"

   if {$dimCount > 1} {append cmnd " -aq2D States"}

   foreach p {T N MODE SW OBS CAR LAB P0 P1} \
      {
       for {set i 0} {$i < $dimCount} {incr i} \
          {
           set prefix [lindex "x y z a" $i]
           set dim    [string tolower [string index $tAxisInfo $i]]
           set type   [lindex $dimInfo  $i]
           set fmt    " %-7s %9s"

           puts -nonewline $outUnit [format $fmt "-$prefix$p" [getSpecParm $dim $type $p $i Time]]
          }

       puts $outUnit " \\"
      }

    switch $dimCount \
       {
        1 {
           if {![strcasecmp $simName Auto]}  {set simName sim.fid}
           puts $outUnit " -out $simName -verb -ov"
          } 

        2 {
           if {![strcasecmp $simName Auto]}  {set simName sim.fid}
           puts $outUnit " -aq2D States -out $simName -verb -ov"
          }

        3 {
           if {![strcasecmp $simName Auto]}  {set simName sim/test%03d.fid}
           puts $outUnit " -aq2D States -out $simName -verb -ov"
          }

        4 {
           if {![strcasecmp $simName Auto]} {set simName sim/test%03d.fid}
           puts $outUnit " -aq2D States -out $simName -verb -ov"
          }
       }

    close $outUnit
}

proc getSpecParm { dim type p i mode } \
{
   global aInfo

   if {![strcasecmp $mode Freq]} \
      {
       if {![strcasecmp $p MODE]} {set aInfo($dim$p) Real}
       if {![strcasecmp $p FT]}   {set aInfo($dim$p) Freq}
      } \
   else \
      {
       if {![strcasecmp $p MODE]} {set aInfo($dim$p) Complex}
       if {![strcasecmp $p FT]}   {set aInfo($dim$p) Time}
      }

   if {![strcasecmp $p N]}    {set aInfo($dim$p) $aInfo(${dim}FD)}

   if {[info exists aInfo($dim$p)]} {return $aInfo($dim$p)}

   switch -exact [string toupper $p] \
      {
       T       {set aInfo($dim$p) [getDefTD  $dim]}
       FD      {set aInfo($dim$p) [getDefFD  $dim]}

       SW      {set aInfo($dim$p) [getDefSW  $type]}
       OBS     {set aInfo($dim$p) [getDefObs $type]}
       CAR     {set aInfo($dim$p) [getDefCar $type]}

       LAB     {set aInfo($dim$p) [getDefLab $type $dim]}

       P0      {set aInfo($dim$p) 0.0}
       P1      {set aInfo($dim$p) 0.0}

       APOD    {set aInfo($dim$p) SP}
       Q1      {set aInfo($dim$p) 0.50}
       Q2      {set aInfo($dim$p) 0.95}

       Q3      {
                if {!$i} \
                   {
                    set aInfo($dim$p) 2
                   } \
                else \
                   {
                    set aInfo($dim$p) 1
                   }
               }

       C1      {
                if {abs($aInfo(${dim}P1)) < 5.0} \
                   {
                    set aInfo($dim$p) 0.5
                   } \
                else \
                   {
                    set aInfo($dim$p) 1.0
                   }
               }

       default {set aInfo($dim$p) 0}
      }

    return $aInfo($dim$p)
}

proc getDefLab { type dim } \
{
   global dimInfo dimCount

   set mList [lsearch -exact -all $dimInfo [string toupper $type]]
   if {[llength $mList] < 2} {return $type}

   return $type$dim
}

proc getDefTD { dim } \
{
   switch -exact [string toupper $dim] \
      {
       X {return 512}
       Y {return 128}
       Z {return 64}
       A {return 32}
      }
}

proc getDefFD { dim } \
{
   global aInfo

   return [nextPower2 [expr 2*$aInfo(${dim}T)]]
}

proc getDefSW { type } \
{
   switch -exact [string toupper $type] \
      {
       HN      {return 10000.0}
       H       {return 10000.0}
       N       {return 2000.0}
       CO      {return 1400.0}
       CA      {return 4500.0}
       CACB    {return 8000.0}
       C       {return 8000.0}
       HA      {return 4500.0}
       HAHB    {return 4500.0}
       default {return 10000.0}
      }
}

proc getDefCar { type } \
{
   switch -exact [string toupper $type] \
      {
       HN      {return 4.745}
       H       {return 4.745}
       N       {return 118.0}
       CO      {return 177.0}
       CA      {return 56.0}
       CACB    {return 46.0}
       C       {return 50.0}
       HA      {return 4.745}
       HAHB    {return 4.745}
       default {return 0.0}
      }
}

proc getDefObs { type } \
{
   switch -exact [string toupper $type] \
      {
       HN      {return 600.0}
       H       {return 600.0}
       N       {return 61.0}
       C       {return 151.0}
       CO      {return 151.0}
       CA      {return 151.0}
       CACB    {return 151.0}
       HA      {return 600.0}
       HAHB    {return 600.0}
       default {return 600.0}
      }
}

proc showPkHdr { outUnit sInfo } \
{
   global dimCount fullFlag

   set str [join [split $sInfo " "] " "]

   puts $outUnit "#"
   puts $outUnit "# Synthetic Peak Table:"

   puts -nonewline $outUnit "#"

   for {set i 0} {$i < [llength $sInfo]} {incr i} \
      {
       if {$i == 1} continue
       puts -nonewline $outUnit " [lindex $sInfo $i]"
      }

   puts $outUnit ""
   puts $outUnit ""

   puts -nonewline $outUnit "VARS   INDEX"

   set pAxisInfo XYZA

   for {set i 0} {$i < $dimCount} {incr i} \
      {
       set d [string toupper [string index $pAxisInfo $i]]
       puts -nonewline $outUnit [format " %s_PPM" $d] 
      }

   for {set i 0} {$i < $dimCount} {incr i} \
      {
       set d [string toupper [string index $pAxisInfo $i]]
       puts -nonewline $outUnit [format " %s_HZ" $d]
      }

   for {set i 0} {$i < $dimCount} {incr i} \
      {
       set d [string toupper [string index $pAxisInfo $i]]
       puts -nonewline $outUnit [format " %s_AXIS" $d]
      }

   for {set i 0} {$i < $dimCount} {incr i} \
      {
       set d [string toupper [string index $pAxisInfo $i]]
       puts -nonewline $outUnit [format " D%s" $d]
      }

   for {set i 0} {$i < $dimCount} {incr i} \
      {
       set d [string toupper [string index $pAxisInfo $i]]
       puts -nonewline $outUnit [format " %s1 %s3" $d $d]
      }

   for {set i 0} {$i < $dimCount} {incr i} \
      {
       set d [string toupper [string index $pAxisInfo $i]]
       puts -nonewline $outUnit [format " %sTW" $d]
      }

   for {set i 0} {$i < $dimCount} {incr i} \
      {
       set d [string toupper [string index $pAxisInfo $i]]
       puts -nonewline $outUnit [format " %sW_HZ" $d] 
      }

   for {set i 0} {$i < $dimCount} {incr i} \
      {
       set d [string toupper [string index $pAxisInfo $i]]
       puts -nonewline $outUnit [format " %sW" $d]
      }

   if {$fullFlag} \
      {
       for {set i 0} {$i < $dimCount} {incr i} \
          {
           set d [string toupper [string index $pAxisInfo $i]]
           puts -nonewline $outUnit [format " %s_P0 %s_P1" $d $d]
          }

       for {set i 0} {$i < $dimCount} {incr i} \
          {
           set d [string toupper [string index $pAxisInfo $i]]
           puts -nonewline $outUnit [format " %s_COSJ1 %s_COSJ2 %s_COSJ3" $d $d $d]
          }
      }

   puts $outUnit " HEIGHT DHEIGHT VOL PCHI2 TYPE CLUSTID MEMCNT ASS"
  
   puts -nonewline $outUnit "FORMAT %4d"

   for {set i 0} {$i < $dimCount} {incr i} {puts -nonewline $outUnit " %9.4f"}
   for {set i 0} {$i < $dimCount} {incr i} {puts -nonewline $outUnit " %10.4f"}
   for {set i 0} {$i < $dimCount} {incr i} {puts -nonewline $outUnit " %9.4f"}
   for {set i 0} {$i < $dimCount} {incr i} {puts -nonewline $outUnit " %7.4f"}
   for {set i 0} {$i < $dimCount} {incr i} {puts -nonewline $outUnit " %4d %4d"}

   for {set i 0} {$i < $dimCount} {incr i} {puts -nonewline $outUnit " %7.4f"}
   for {set i 0} {$i < $dimCount} {incr i} {puts -nonewline $outUnit " %7.4f"}
   for {set i 0} {$i < $dimCount} {incr i} {puts -nonewline $outUnit " %7.4f"}

   if {$fullFlag} \
      {
       for {set i 0} {$i < $dimCount} {incr i} \
          {
           set d [string toupper [string index $pAxisInfo $i]]
           puts -nonewline $outUnit " %6.1f %6.1f"
          }

       for {set i 0} {$i < $dimCount} {incr i} \
          {
           set d [string toupper [string index $pAxisInfo $i]]
           puts -nonewline $outUnit " %6.3f %6.3f %6.3f"
          }
      }

   puts $outUnit " %e %e %e %.5f %d %4d %2d %s"
   puts $outUnit ""
}
 
proc showShifts {outUnit dimCount ass shiftList hi id} \
{
   global aInfo tAxisInfo fAxisInfo fdata noise globAtomFlag apodFlag fullFlag

   set xxx [lindex "x y z a" [string first x $tAxisInfo]]
   set yyy [lindex "x y z a" [string first y $tAxisInfo]]
   set zzz [lindex "x y z a" [string first z $tAxisInfo]]
   set aaa [lindex "x y z a" [string first a $tAxisInfo]]

   set xx  [string index $fAxisInfo 0]
   set yy  [string index $fAxisInfo 1]
   set zz  [string index $fAxisInfo 2]
   set aa  [string index $fAxisInfo 3]

   for {set i 0} {$i < $dimCount} {incr i} \
      {
       set dim  [lindex "x y z a" $i]
       set dest [lindex "$xx $yy $zz $aa" $i]

       if {$apodFlag} \
          {
           set w($dim)    [expr $aInfo(${dim}lw)*$aInfo(${dim}T)/$aInfo(${dim}SW)]
           set wPts($dim) $aInfo(apod,${dim}LWFinal)
           set wHz($dim)  [expr $wPts($dim)*$aInfo(${dim}SW)/$aInfo(${dim}FD)]
          } \
       else \
          {
           set w($dim)    [expr $aInfo(${dim}lw)*$aInfo(${dim}T)/$aInfo(${dim}SW)]
           set wPts($dim) $w($dim)
           set wHz($dim)  [expr $wPts($dim)*$aInfo(${dim}SW)/$aInfo(${dim}FD)]
          }
      }

# INDEX
# [%s_PPM ...] [%s_HZ ...]  [%s_AXIS ...] [D%s ...] [%s1 %s3 ...]
# [%sTW ...]   [%sW_HZ ...] [%sW ...] 
# [%s_P0  %s_P1 ...] [%s_COSJ1 %s_COSJ2 %s_COSJ3 ...]
# HEIGHT DHEIGHT VOL PCHI2 TYPE CLUSTID MEMCNT ASS

   switch $dimCount \
      {
       1 {
          foreach xItem [lindex $shiftList 0] \
           {
            incr id

            set s(x) [lindex [split $xItem ","] 0]
            set a(x) [lindex [split $xItem ","] 1]

            if {$globAtomFlag} \
               {
                set thisAss $ass:$a(x)
               } \
            else \
               {
                set thisAss $ass
               }

            set sPts(x) [spec2pnt $fdata x $s(x)ppm]
            set sHz(x)  [pnt2spec $fdata x $sPts(x) hz]

            set s1(x)   [expr int($sPts(x) - 2.5*$wPts(x))]
            set s3(x)   [expr int($sPts(x) + 2.5*$wPts(x))]

            set vol     [expr $hi*$wPts(x)]
            set ds      [expr $noise/$hi]

            set fmt " %4d"
            puts -nonewline $outUnit [format $fmt $id]

            set fmt " % 9.4f"
            puts -nonewline $outUnit [format $fmt $s($xx)]

            set fmt " % 9.4f"
            puts -nonewline $outUnit [format $fmt $sHz($xx)]

            set fmt " % 9.4f"
            puts -nonewline $outUnit [format $fmt $sPts($xx)]

            set fmt " % 7.4f"
            puts -nonewline $outUnit [format $fmt $ds]

            set fmt " %4d %4d"
            puts -nonewline $outUnit [format $fmt $s1($xx) $s3($xx)]

            set fmt " %7.4f"
            puts -nonewline $outUnit [format $fmt $w($xx)]
            puts -nonewline $outUnit [format $fmt $wHz($xx)]
            puts -nonewline $outUnit [format $fmt $wPts($xx)]

            if {$fullFlag} \
               {
                puts -nonewline $outUnit [format " %6.1f %6.1f" 0.0 0.0]
                puts -nonewline $outUnit [format " %6.3f %6.3f %6.3f" 0.0 0.0 0.0]
               }

            set fmt " % .4e %.4e % .4e %.5f %d %4d %2d %s"
            puts $outUnit [format $fmt $hi $noise $vol 0.0 1 $id 1 $thisAss]
           }
         }

       2 {
          foreach xItem [lindex $shiftList 0] \
           {
            set s(x) [lindex [split $xItem ","] 0]
            set a(x) [lindex [split $xItem ","] 1]

            set sPts(x) [spec2pnt $fdata x $s(x)ppm]
            set sHz(x)  [pnt2spec $fdata x $sPts(x) hz]

            set s1(x)   [expr int($sPts(x) - 2.5*$wPts(x))]
            set s3(x)   [expr int($sPts(x) + 2.5*$wPts(x))]

            set vol     [expr $hi*$wPts(x)*$wPts(y)]
            set ds      [expr $noise/$hi]

            foreach yItem [lindex $shiftList 1] \
             {
              incr id

              set s(y) [lindex [split $yItem ","] 0]
              set a(y) [lindex [split $yItem ","] 1]

              set sPts(y) [spec2pnt $fdata y $s(y)ppm]
              set sHz(y)  [pnt2spec $fdata y $sPts(y) hz]

              set s1(y)   [expr int($sPts(y) - 2.5*$wPts(y))]
              set s3(y)   [expr int($sPts(y) + 2.5*$wPts(y))]

              set fmt " %4d"
              puts -nonewline $outUnit [format $fmt $id]

              set fmt " % 9.4f % 9.4f"
              puts -nonewline $outUnit [format $fmt $s($xx) $s($yy)]

              set fmt " % 9.4f % 9.4f"
              puts -nonewline $outUnit [format $fmt $sHz($xx) $sHz($yy)]

              set fmt " % 9.4f % 9.4f"
              puts -nonewline $outUnit [format $fmt $sPts($xx) $sPts($yy)]

              set fmt " % 7.4f % 7.4f"
              puts -nonewline $outUnit [format $fmt $ds $ds]

              set fmt " %4d %4d %4d %4d"
              puts -nonewline $outUnit [format $fmt $s1($xx) $s3($xx) $s1($yy) $s3($yy)]

              set fmt " %7.4f %7.4f"
              puts -nonewline $outUnit [format $fmt $w($xx)    $w($yy)]
              puts -nonewline $outUnit [format $fmt $wHz($xx)  $wHz($yy)]
              puts -nonewline $outUnit [format $fmt $wPts($xx) $wPts($yy)]

              if {$fullFlag} \
                 {
                  puts -nonewline $outUnit [format " %6.1f %6.1f" 0.0 0.0]
                  puts -nonewline $outUnit [format " %6.1f %6.1f" 0.0 0.0]
                  puts -nonewline $outUnit [format " %6.3f %6.3f %6.3f" 0.0 0.0 0.0]
                  puts -nonewline $outUnit [format " %6.3f %6.3f %6.3f" 0.0 0.0 0.0]
                 }

              set fmt " % .4e %.4e % .4e %.5f %d %4d %2d %s"
              puts $outUnit [format $fmt $hi $noise $vol 0.0 1 $id 1 $ass]
             }
           }
         }

       3 {
          foreach xItem [lindex $shiftList 0] \
           {
            set s(x) [lindex [split $xItem ","] 0]
            set a(x) [lindex [split $xItem ","] 1]

            set sPts(x) [spec2pnt $fdata x $s(x)ppm]
            set sHz(x)  [pnt2spec $fdata x $sPts(x) hz]

            set s1(x)   [expr int($sPts(x) - 2.5*$wPts(x))]
            set s3(x)   [expr int($sPts(x) + 2.5*$wPts(x))]

            set vol     [expr $hi*$wPts(x)*$wPts(y)*$wPts(z)]
            set ds      [expr $noise/$hi]

            foreach yItem [lindex $shiftList 1] \
             {
              set s(y) [lindex [split $yItem ","] 0]
              set a(y) [lindex [split $yItem ","] 1]

              set sPts(y) [spec2pnt $fdata y $s(y)ppm]
              set sHz(y)  [pnt2spec $fdata y $sPts(y) hz]

              set s1(y)   [expr int($sPts(y) - 2.5*$wPts(y))]
              set s3(y)   [expr int($sPts(y) + 2.5*$wPts(y))]

              foreach zItem [lindex $shiftList 2] \
               {
                set s(z) [lindex [split $zItem ","] 0]
                set a(z) [lindex [split $zItem ","] 1]

                set sPts(z) [spec2pnt $fdata z $s(z)ppm]
                set sHz(z)  [pnt2spec $fdata z $sPts(z) hz]

                set s1(z)   [expr int($sPts(z) - 2.5*$wPts(z))]
                set s3(z)   [expr int($sPts(z) + 2.5*$wPts(z))]

                incr id

                set fmt " %4d"
                puts -nonewline $outUnit [format $fmt $id]

                set fmt " % 9.4f % 9.4f % 9.4f"
                puts -nonewline $outUnit [format $fmt $s($xx) $s($yy) $s($zz)]

                set fmt " % 9.4f % 9.4f %9.4f"
                puts -nonewline $outUnit [format $fmt $sHz($xx) $sHz($yy) $sHz($zz)]

                set fmt " % 9.4f % 9.4f % 9.4f"
                puts -nonewline $outUnit [format $fmt $sPts($xx) $sPts($yy) $sPts($zz)]

                set fmt " % 7.4f % 7.4f % 7.4f"
                puts -nonewline $outUnit [format $fmt $ds $ds $ds]

                set fmt " %4d %4d %4d %4d %4d %4d"
                puts -nonewline $outUnit [format $fmt $s1($xx) $s3($xx) $s1($yy) $s3($yy) $s1($zz) $s3($zz)]

                set fmt " %7.4f %7.4f %7.4f"
                puts -nonewline $outUnit [format $fmt $w($xx)    $w($yy)    $w($zz)]
                puts -nonewline $outUnit [format $fmt $wHz($xx)  $wHz($yy)  $wHz($zz)]
                puts -nonewline $outUnit [format $fmt $wPts($xx) $wPts($yy) $wPts($zz)]

                if {$fullFlag} \
                   {
                    puts -nonewline $outUnit [format " %6.1f %6.1f" 0.0 0.0]
                    puts -nonewline $outUnit [format " %6.1f %6.1f" 0.0 0.0]
                    puts -nonewline $outUnit [format " %6.1f %6.1f" 0.0 0.0]
                    puts -nonewline $outUnit [format " %6.3f %6.3f %6.3f" 0.0 0.0 0.0]
                    puts -nonewline $outUnit [format " %6.3f %6.3f %6.3f" 0.0 0.0 0.0]
                    puts -nonewline $outUnit [format " %6.3f %6.3f %6.3f" 0.0 0.0 0.0]
                   }

                set fmt " % .4e %.4e % .4e %.5f %d %4d %2d %s"

                puts $outUnit [format $fmt $hi $noise $vol 0.0 1 $id 1 $ass:$a($zz)]
               }
             }
           }
         }

       4 {
          foreach xItem [lindex $shiftList 0] \
           {
            set s(x) [lindex [split $xItem ","] 0]
            set a(x) [lindex [split $xItem ","] 1]

            set sPts(x) [spec2pnt $fdata x $s(x)ppm]
            set sHz(x)  [pnt2spec $fdata x $sPts(x) hz]

            set s1(x)   [expr int($sPts(x) - 2.5*$wPts(x))]
            set s3(x)   [expr int($sPts(x) + 2.5*$wPts(x))]

            set vol [expr $hi*$wPts(x)*$wPts(y)*$wPts(z)*$wPts(a)]
            set ds  [expr $noise/$hi]

            foreach yItem [lindex $shiftList 1] \
             {
              set s(y) [lindex [split $yItem ","] 0]
              set a(y) [lindex [split $yItem ","] 1]

              set sPts(y) [spec2pnt $fdata y $s(y)ppm]
              set sHz(y)  [pnt2spec $fdata y $sPts(y) hz]

              set s1(y)   [expr int($sPts(y) - 2.5*$wPts(y))]
              set s3(y)   [expr int($sPts(y) + 2.5*$wPts(y))]

              foreach zItem [lindex $shiftList 2] \
               {
                set s(z) [lindex [split $zItem ","] 0]
                set a(z) [lindex [split $zItem ","] 1]

                set sPts(z) [spec2pnt $fdata z $s(z)ppm]
                set sHz(z)  [pnt2spec $fdata z $sPts(z) hz]

                set s1(z)   [expr int($sPts(z) - 2.5*$wPts(z))]
                set s3(z)   [expr int($sPts(z) + 2.5*$wPts(z))]

                foreach aItem [lindex $shiftList 3] \
                 {
                  set s(a) [lindex [split $aItem ","] 0]
                  set a(a) [lindex [split $aItem ","] 1]

                  set sPts(a) [spec2pnt $fdata a $s(a)ppm]
                  set sHz(a)  [pnt2spec $fdata a $sPts(a) hz]

                  set s1(a)   [expr int($sPts(a) - 2.5*$wPts(a))]
                  set s3(a)   [expr int($sPts(a) + 2.5*$wPts(a))]

                  incr id

                  set fmt " %4d"
                  puts -nonewline $outUnit [format $fmt $id]

                  set fmt " % 9.4f % 9.4f % 9.4f % 9.4f"
                  puts -nonewline $outUnit [format $fmt $s($xx) $s($yy) $s($zz) $s($aa)]

                  set fmt " % 9.4f % 9.4f %9.4f %9.4f"
                  puts -nonewline $outUnit [format $fmt $sHz($xx) $sHz($yy) $sHz($zz) $sHz($aa)]

                  set fmt " % 9.4f % 9.4f % 9.4f % 9.4f"
                  puts -nonewline $outUnit [format $fmt $sPts($xx) $sPts($yy) $sPts($zz) $sPts($aa)]

                  set fmt " % 7.4f % 7.4f % 7.4f % 7.4f"
                  puts -nonewline $outUnit [format $fmt $ds $ds $ds $ds]

                  set fmt " %4d %4d %4d %4d %4d %4d %4d %4d"
                  puts -nonewline $outUnit [format $fmt $s1($xx) $s3($xx) $s1($yy) $s3($yy) $s1($zz) $s3($zz) $s1($aa) $s3($aa)]

                  set fmt " %7.4f %7.4f %7.4f %7.4f"
                  puts -nonewline $outUnit [format $fmt $w($xx) $w($yy) $w($zz) $w($zz)]
                  puts -nonewline $outUnit [format $fmt $wHz($xx) $wHz($yy) $wHz($zz) $wHz($aa)]
                  puts -nonewline $outUnit [format $fmt $wPts($xx) $wPts($yy) $wPts($zz) $wPts($aa)]

                  if {$fullFlag} \
                     {
                      puts -nonewline $outUnit [format " %6.1f %6.1f" 0.0 0.0]
                      puts -nonewline $outUnit [format " %6.1f %6.1f" 0.0 0.0]
                      puts -nonewline $outUnit [format " %6.1f %6.1f" 0.0 0.0]
                      puts -nonewline $outUnit [format " %6.1f %6.1f" 0.0 0.0]
                      puts -nonewline $outUnit [format " %6.3f %6.3f %6.3f" 0.0 0.0 0.0]
                      puts -nonewline $outUnit [format " %6.3f %6.3f %6.3f" 0.0 0.0 0.0]
                      puts -nonewline $outUnit [format " %6.3f %6.3f %6.3f" 0.0 0.0 0.0]
                      puts -nonewline $outUnit [format " %6.3f %6.3f %6.3f" 0.0 0.0 0.0]
                     }

                  set fmt " % .4e %.4e % .4e %.5f %d %4d %2d %s"

                  puts $outUnit [format $fmt $hi $noise $vol 0.0 1 $id 1 $ass:$a($zz),$a($aa)]
                 }
               }
             }
           } 
         }
      }

   return $id
}

proc getShiftDev { entry atomName } \
{
   global varList devVar

   if {[lsearch -exact $varList $devVar] >= 0} \
      {
       set d [gdbGet $entry $devVar]
       if {$d < 999.0} {return $d}
      }

   if {[string match -nocase HA* $atomName]} {return 0.37} 
   if {[string match -nocase HB* $atomName]} {return 0.37} 

   if {![strcasecmp CA $atomName]} {return 1.19} 
   if {![strcasecmp CB $atomName]} {return 1.31} 
   if {![strcasecmp C  $atomName]} {return 1.40} 
   if {![strcasecmp N  $atomName]} {return 3.74} 

   return 9999.0
}

#
#-----------------------------------------------------------------------------

initResNames

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input CS Table $inName"
    exit 1
   }

if {![strcasecmp $outName stdout]} \
   {
    set outUnit stdout
   } \
else \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "Error Opening Peak Table Output File $outName"
        exit 1
       }
   }

set sInfo        ""
set globAtomFlag 0

foreach sItem $sInfoList \
   {
    set thisType [lindex $sItem 0]

    if {![strcasecmp $thisType $sType]} \
       {
        set sInfo [string toupper $sItem]
        if {[string first "*" $sInfo] >= 0} {set globAtomFlag 1}
        break
       }
   }

if {![llength $sInfo]} \
   {
    puts stderr "Error: unknown table type $sType."
    exit 1
   }

set dimInfo  [split [string toupper [lindex $sInfo 1]] ","]
set dimCount [llength $dimInfo]

set csTab   [gdbRead table -in $inName]
set eList   [gdbGetInfo $csTab -parms entries]
set seqList [getSeqList $csTab r1 rN]
set varList [gdbGetInfo $csTab -parms varNames]

foreach e $eList \
   {
    set resID    [gdbGet $e RESID]
    set shift    [gdbGet $e $shiftVar]
    set resName  [string toupper [gdbGet $e RESNAME]]
    set atomName [string toupper [gdbGet $e ATOMNAME]]
    set dev      [getShiftDev $e $atomName]

    if {$shift < 999.0} {lappend csInfo($resID,$atomName,shift) $shift}
    if {$dev   < 999.0} {lappend csInfo($resID,$atomName,dev)   $dev}

    lappend csInfo($resID,atomList) $atomName

    if {![strcasecmp $atomName C] && !$globAtomFlag} \
       {
        set atomName CO

        if {$shift < 999.0} {lappend csInfo($resID,$atomName,shift) $shift}
        if {$dev   < 999.0} {lappend csInfo($resID,$atomName,dev)   $dev}

        lappend csInfo($resID,atomList) $atomName
       }
   }

getFIDInfo
getFIDArgs
getSpecInfo
initSimScript

showPkHdr $outUnit $sInfo

set nn [llength $sInfo]
set n  [expr $nn - 2]

set nGood 0
set nMiss 0
set id    0

for {set resID $r1} {$resID <= $rN} {incr resID} \
   {
    set resName [lindex $seqList [expr $resID - $r1]]
    set ass $resID$resName

    for {set i 2} {$i < $nn} {incr i} \
       {
        set pkInfo   [split [lindex $sInfo $i] "/"]
        set spkInfo  [string trim $pkInfo "+-"]

        if {$resID == $r1 && [string first "(I-1)" $pkInfo] >= 0} continue
        if {$resID == $rN && [string first "(I+1)" $pkInfo] >= 0} continue

        if {![strcasecmp $resName P]} \
           {
            if {[lsearch -exact $spkInfo HN] >= 0} continue 
           }

        if {![strcasecmp $resName G]} \
           {
            if {[lsearch -exact $spkInfo HB] >= 0} continue 
            if {[lsearch -exact $spkInfo CB] >= 0} continue 

            if {[lsearch -exact $spkInfo HB(I)] >= 0} continue 
            if {[lsearch -exact $spkInfo CB(I)] >= 0} continue 
           }

        set c [string index $pkInfo 0]

        if {![strcmp $c "-"]} \
           {
            set hi -$hiDef
           } \
        else \
           {
            set hi $hiDef 
           }

        if {[string first "(I-1)" $pkInfo] >= 0} {set hi [expr 0.5*$hi]}

        set shiftList ""
 
        foreach pk $spkInfo \
           {
            set thisList [getShifts $resID $pk]

            if {![llength $thisList]} \
               {
                set shiftList ""

                set fmt "  Missing Shifts for %-5s %s"
                puts stderr [format $fmt ${ass}: [lindex $sInfo $i]]

                incr nMiss 
                break
               }

            lappend shiftList $thisList
            incr nGood [llength $thisList]
           }

        if {![llength $shiftList]} continue

        set id [showShifts $outUnit $dimCount $ass $shiftList $hi $id]
       }
   }

close $outUnit

puts stderr "Shifts Used: $nGood  Shifts Missing: $nMiss"
if {!$nGood} {exit 1}

puts stderr ""
puts stderr "Cluster Analysis:"

set pkTab [gdbRead table -in $outName]
set eList [gdbGetInfo $pkTab -parms entries]
set entry [gdbFirst $pkTab]

set vList ""
set dList ""

for {set i 0} {$i < $dimCount} {incr i} \
   {
    set dim [lindex "X Y Z A" $i] 

    lappend vList ${dim}_AXIS
    lappend dList [expr 2.5*[gdbGet $entry ${dim}W]]
   }

set clustIDList [findClusters clustInfo $eList isNear 1]
set i 0

foreach entry $eList \
   {
    set clustID [lindex $clustIDList $i]
    set memCnt  $clustInfo(memCount,$clustID)

    gdbSet $entry CLUSTID $clustID
    gdbSet $entry MEMCNT  $memCnt
    incr i
   }

gdbWrite $pkTab $outName

exit 0
