#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Prepares DC Tables Used by dcEnsemble.tcl and dcSum.tcl:"
    puts " -dc     dcName   \[dObsA.tab\]    Input DC Table." 
    puts " -pdb    pdbList  \[s\[0-9\]*.pdb\]  Input PDB List." 
    puts " -ignore                         Ignore Chain and Seg Info."
    puts " -verb                           Verbose Mode ON (Default)."
    puts " -noverb                         Verbose Mode OFF."
    exit
   }

set outDir     .
set dcName     dObsA.tab
set pdbList    [lsort -dictionary [glob -nocomplain s\[0-9\]*.pdb]]
set tensor     A
set verbose    1
set ignoreFlag 0

getArgD     $ARGV -in     dcName
getArgD     $ARGV -dc     dcName
getArgD     $ARGV -outDir outDir
getListArgD $ARGV -pdb    pdbList

if {[flagLoc $ARGV -ignore]}   {set ignoreFlag 1}
if {[flagLoc $ARGV -noignore]} {set ignoreFlag 0}

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {![file isdirectory $outDir]} {catch "file mkdir $outDir"}

#
#--------------------------------------------------------------------------

if {![file exists $dcName]} \
   {
    puts stderr "Error Finding DC Input $dcName"
    exit 1
   }

foreach pdbName $pdbList \
   {
    if {![file exists $pdbName]} \
       {
        puts stderr "Error Finding PDB Input $pdbName"
        exit 1
       }
   }

dynInit -fast 

if {$ignoreFlag} \
   {
    dynSetChainMode -ignore
    dynSetSegMode   -ignore
   }

set dcRoot [file root [file tail $dcName]]

if {[string length $dcRoot] == 5 && [string first dObs $dcName] == 0} \
   {
    set tensor [string index $dcRoot 4]
   }

set dSrc [dynRead -dc -in $dcName]

foreach pdbName $pdbList \
   {
    set pdbSrc [dynRead -pdb -in $pdbName]
    set s      [file tail [file root $pdbName]]

    set outName $outDir/dCalc$tensor.$s.tab
 
    set pInfo  [dynGetInfo -src $pdbSrc]
    set minRes [lindex $pInfo 0]
    set maxRes [lindex $pInfo 1]

    set dInfo [dynSimulate -dc $dSrc -src $pdbSrc -s1 $minRes -sN $maxRes -a1 $minRes -verb -updateS]

    dynWrite -dc $dSrc -out $outName -verb 2

    if {$verbose} \
       {
        set dRMS [lindex $dInfo 0]
        set nMax [lindex $dInfo end]
        set n    [lindex $dInfo 2]

        set fmt "%s %s RMS: %6.3fHz N: %3d %3d"

        puts [format $fmt $pdbName $outName $dRMS $nMax $n]
       } \
    else \
       {
        puts $outName
       }
   }

exit
