#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts stderr "Delete Samples from a Non-Uniform Sampling (NUS) Schedule:"
    puts stderr " -in     schedName \[nuslist\] Input NUS Schedule."
    puts stderr " -n      delCount  \[1\]       Number of Samples to Delete."
    puts stderr " -iseed  iseed     \[539731\]  Random Number Seed."
    puts stderr " -verb                       Verbose Mode ON (Default)."
    puts stderr " -noverb                     Verbose Mode OFF."
    puts stderr "Notes:"
    puts stderr " 1. The new schedule will be written to standard output."
    puts stderr " 2. The first and last samples will always be retained."
    exit 0
   }

set inName   nuslist
set iseed    539731
set delCount 1
set verbose  1

getArgD $ARGV -in    schedName
getArgD $ARGV -n     delCount
getArgD $ARGV -iseed iseed

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

#
#------------------------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts stderr "deleteNUS Error Finding NUS Schedule Input $inName"
    exit 1
   }

set sInfo(lineCount) 0

set lineCount [getTextLines $inName sInfo]

if {$lineCount < 1} \
   {
    puts stderr "deleteNUS Error Reading Data in NUS Schedule Input $inName"
    exit 1
   }

set n $sInfo(lineCount)

set nFinal [expr $n - $delCount]
if {$nFinal < 2} {set nFinal 2}

set delCountActual [expr $n - $nFinal]
if {$delCountActual < 0} {set delCountActual 0}

if {$verbose} \
   {
    puts stderr "deleteNUS. NUS Schedule Input:     $inName"
    puts stderr "deleteNUS. Initial Sample Count:   $n"
    puts stderr "deleteNUS. Final Sample Count:     $nFinal"
    puts stderr "deleteNUS. Samples Deleted:        $delCountActual"

    puts -nonewline stderr "deleteNUS. Sample List:           "
   }

if {$n == 2} \
   {
    puts $sInfo(1)
    puts $sInfo(2)
    return
   } \
elseif {$n == 1} \
   {
    puts $sInfo(1)
    return
   }

srand $iseed

set rList ""

for {set i 0} {$i < $n - 2} {incr i} {lappend rList [rand]}

set delList [lrange [getVecSortLoc $rList] 0 [expr $delCount - 1]]

for {set i 0} {$i < $n} {incr i} \
   {
    if {[lsearch -integer $delList $i] >= 0} \
       {
        if {$verbose && ($n <= 32)} {puts -nonewline stderr " \[$sInfo($i)\]"}
       } \
    else \
       {
        if {$verbose && ($n <= 32)} {puts -nonewline stderr " $sInfo($i)"}
        puts " $sInfo($i)"
       }
   }

if {$verbose && ($n <= 32)} {puts stderr ""}


exit 0
