#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Batch Conversion of JEOL JDF Format Data (2022.130.10.50):"
   echo " -in       inList   [*.jdf *.JDF]  Input JDF Format Data."
   echo " -outDir   outDir   [.]            Base Directory for Output."
   echo " -postfix  postfix  [_data]        Postfix Text for Output Directories."
   echo " -cp                               Copy Input Data to Output Directory (Default)."
   echo " -nocp                             Do Not Copy Input Data to Output Directory."
   echo "Options for delta Conversion Command (Should Be Last on Command-Line):"
   echo " -convArgs convArgs [None]         Options for delta Command (See delta -help)."
   echo "Notes:"
   echo " 1. A directory will be created for each JDF input."
   echo " 2. The directories will be created under the base directory specified"
   echo "    via the -outDir argument."
   echo " 3. The output directories will be created with the name of the JDF"
   echo "    input file and the -post postfix text. For example,"
   echo "    input ubiqhnca.jdf would have output directory ubiqhnca_data."
   exit 0
endif

set loc      = (`flagLoc $argv -convArgs`)
set thisArgv = ($argv)
set convArgs = ()

if ($loc >= 1) then
   @ locP = $loc - 1
   @ locN = $loc + 1

   set convArgs = ($argv[$locN-])
   set thisArgv = ($argv[1-$locP])
endif

set thisList  = (`find ./ -maxdepth 1 -name \*.jdf -print` `find ./ -maxdepth 1 -name \*.JDF -print`)
set inList    = (`getListArgD $thisArgv -in -- $thisList`)

set outDir    = (`getArgD $thisArgv -outDir  .`)
set postfix   = (`getArgD $thisArgv -postfix _data`)

set execFlag  = 1
set cpFlag    = 1

if (`flagLoc $thisArgv -cp`) then
   set cpFlag = 1
endif

if (`flagLoc $thisArgv -nocp`) then
   set cpFlag = 0
endif

if (`flagLoc $thisArgv -exec`) then
   set execFlag = 1
endif

if (`flagLoc $thisArgv -noexec`) then
   set execFlag = 0
endif

if (`string csame $convArgs None`) then
   set convArgs = ""
endif

#
#-------------------------------------------------------------------------------

set origDir = (`pwd`)

if (!(-d $outDir)) then
   mkdir $outDir
endif

foreach inName ($inList)
   set tailName   = (`string tail $inName`)
   set rootName   = (`string root $tailName`)

   set thisInDir  = (`string path $inName`)
   set thisOutDir = $rootName$postfix
   
   echo "JDF Input:        $inName"
   echo "Output Directory: $thisOutDir"

   if (!(-d $thisOutDir)) then
      mkdir $thisOutDir
   endif

   if ($cpFlag) then
      /bin/cp $inName $thisOutDir

      cd $thisOutDir
         /bin/rm -f fid.com

         delta -AUTO -in $tailName $convArgs

         if ($execFlag) then
            ./fid.com
         endif
      cd $origDir
   else
      /bin/rm -f $thisOutDir/fid.com

      cd $thisInDir
         set fullInDir  = (`pwd`)
      cd $origDir

      cd $thisOutDir
         set fullOutDir = (`pwd`)
      cd $origDir

      delta -AUTO -in $fullInDir/$tailName -outDir $fullOutDir -script $thisOutDir/fid.com $convArgs

      if ($execFlag) then
         cd $thisOutDir
            ./fid.com
         cd $origDir
      endif
   endif

   echo ""
end
