#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

# setenv NMRPIPE_BIGFONT   "-adobe-helvetica-bold-r-*-*-*-180-*-*-*-*-*-*"
# setenv NMRPIPE_SMALLFONT "-adobe-helvetica-bold-r-*-*-*-140-*-*-*-*-*-*"
# setenv NMRPIPE_AXISFONT  "-adobe-helvetica-bold-r-*-*-*-140-*-*-*-*-*-*"

if {[flagLoc $ARGV -help]} \
   {
    puts "Utility to Capture Cluster Analysis Scatter Plot Graphs."
    exit 0
   }

set colorList  {pink darkOrchid red cyan green blue orange magenta forestGreen violet yellow firebrick brown gold}

set xSize      1800 
set ySize       980 

set leftMargin   20 
set rightMargin 100
set topMargin    20 
set bottomMargin 80

set charX 12
set charY 18

set inName pcaCoords.tab
set xVar   PCA2
set yVar   PCA3

set snapDir snap
set imageID 0

getArgD $ARGV -in    inName

getArgD $ARGV -xSize xSize
getArgD $ARGV -ySize xSize

getArgD $ARGV -topMargin    topMargin
getArgD $ARGV -bottomMargin bottomMargin
getArgD $ARGV -leftMargin   leftMargin
getArgD $ARGV -rightMargin  rightMargin

getArgD $ARGV -x xVar
getArgD $ARGV -y yVar

getArgD $ARGV -charX charX
getArgD $ARGV -charY charY

getListArgD $ARGV -colors colorList

set cx1 $leftMargin 
set cxn [expr $xSize - $rightMargin]
set cy1 $topMargin 
set cyn [expr $ySize - $bottomMargin]

set colorCount [llength $colorList]
#
#--------------------------------------------------------------------------------------------------------------------

proc captureScreen { } \
{
   global snapDir imageID

   update

   if {![file exists $snapDir]} {catch "file mkdir $snapDir"}

   set im [image create photo]
   winop snap .c $im

   set n [llength [glob -nocomplain $snapDir/*.gif]]

   for {set i 1} {$i <= $n + 1} {incr i} \
      {
       set outName [format $snapDir/im%05d.gif $i]
       if {[file exists $outName]} continue

       set imageID $i
       break
      }

   set outName [format $snapDir/im%05d.gif $imageID]

   $im write -format gif $outName

   image delete $im
}

canvas .c -width $xSize -height $ySize -border 0
pack   .c -side top -fill both

.c create pixmap 0 0 -width $xSize -height $ySize -win 1

defineCell -x $cx1 $cxn -y $cy1 $cyn -cell 1 -verb -win 1

set tInit  [gdbRead table -in $inName]
set eList  [gdbGetInfo $tInit -parms entries]
set eCount [llength $eList]

set cWide [expr 1 + $cxn - $cx1]
set cHigh [expr 1 + $cyn - $cy1]

set xMin [getTabStat getMin $tInit $xVar]
set xMax [getTabStat getMax $tInit $xVar]

set yMin [getTabStat getMin $tInit $yVar]
set yMax [getTabStat getMax $tInit $yVar]

set xRange [expr $xMax - $xMin]
set yRange [expr $yMax - $yMin]

set xMin [expr $xMin - 0.10*$xRange]
set xMax [expr $xMax + 0.10*$xRange]

set yMin [expr $yMin - 0.10*$yRange]
set yMax [expr $yMax + 0.10*$yRange]

set xRange [expr $xMax - $xMin]
set yRange [expr $yMax - $yMin]

if {$xRange == 0.0} {set xRange 1.0}
if {$yRange == 0.0} {set yRange 1.0}

drawRec -win 1 -x 0 $xSize -y 0 $ySize -fg gray30

drawRec -win 1 -x $cx1 $cxn -y $cy1 $cyn -fg black 
drawBox -win 1 -x $cx1 $cxn -y $cy1 $cyn -fg gray10

readROI -in load.dat -roi 1

set step -1

for {set nc $eCount} {$nc > 2} {incr nc $step} \
   {
    puts $nc

    exec clustTab.tcl -in $inName -out clust.tab -nc $nc -x $xVar $yVar

    set t     [gdbRead table -in clust.tab]
    set eList [gdbGetInfo $t -parms entries]

    drawRec -win 1 -cell 1 -x $cx1 $cxn -y $cy1 $cyn -fg black 

    set clustIDList [listUniq [getTabVals $t CLUSTID]]
    set clustCount  [llength $clustIDList]

    set twoCount     0
    set singCount    0
    set multiCount   0
    set maxClustSize 0

    foreach clustID $clustIDList \
       {
        set thisList  [gdbSelect -from $t -cond "CLUSTID == $clustID"]
        set firstID   [lindex [lsort -integer [getEListVals $thisList INDEX]] 0]
        set thisCount [llength $thisList]

        if {$thisCount > $maxClustSize} {set maxClustSize $thisCount}

        if {$thisCount == 1} \
           {
            set colorInfo($clustID) gray30
           } \
        else \
           {
            set colorInfo($clustID) [lindex $colorList [expr $firstID % $colorCount]]
            incr twoCount
           }
       }

    foreach e $eList \
       {
        set tInfo [split [gdbGet $e TITLE] "-"]
        set type  [lindex $tInfo 1]-[lindex $tInfo 3]-[lindex $tInfo 4]
        set type  [lindex $tInfo 4]

        set clustID  [gdbGet $e CLUSTID]
        set memCount [gdbGet $e MEMCNT]

        set x0    [gdbGet $e $xVar]
        set y0    [gdbGet $e $yVar]

        set xPix [expr $cx1 + int(($cWide - 1)*($x0 - $xMin)/$xRange) - $charX/2]
        set yPix [expr $cyn - int(($cHigh - 1)*($y0 - $yMin)/$yRange) + $charY/2]

        if {$memCount == 1} \
           {
            incr singCount
           } \
        else \
           {
            incr multiCount
           }

        drawText -x $xPix -y $yPix -fg $colorInfo($clustID) -text $type
       }

    drawROI -win 1 -cell 1 -axisOnly -roi 1 -fg gray -colorA gray \
            -axis Bottom,orig=$xMin,end=$xMax,lab=$xVar,units="None" Right,orig=$yMin,end=$yMax,lab=$yVar,units=None

    drawBox -win 1 -cell 1 -x $cx1 $cxn -y $cy1 $cyn -fg gray

    set fmt "Spectra Clustered: %3d   Single Spectra: %3d   Clusters of Two or More: %3d   Max Cluster Size: %3d   Total Clusters: %3d"

    drawText -win1 -cell 1 -x [expr $cx1 + $charX] -y [expr $cy1 + 3*$charY/2] -fg white -big -text \
             [format $fmt $multiCount $singCount $twoCount $maxClustSize $clustCount]

    captureScreen

    gdbFree $t
   }


