#!/bin/csh

if ($#argv < 2) then
   echo Extract XA plane from 4D hypercomplex FID:
   echo " Usage: ext_xa.com inTemplate outName [yLoc] [zLoc]"
   echo ""
   echo " Examples:"
   echo "    ext_xa.com fid/test%02d%03d.fid xa.fid     (Plane Series Input)"
   echo "    ext_xa.com fid/test%03d.fid     xa.fid 1 2 (Cube Series Input)"
   echo ""
   echo "Notes:"
   echo " 1. Values for yLoc or aLoc are usually 1 or 2, for the first real"
   echo "    or first imaginary point in the indirect dimension."
   echo " 2. 3D Input can be single-file or a template with a one-integer"
   echo "    format (Plane series such as test%03d.fid)."
   echo " 4. 4D Input can be single-file, a template with a one-integer"
   echo "    format (Cube series such as test%03d.fid), or a template with"
   echo "    a two-integer format (Plane series such as test%03d%03d.fid)."
   exit 1
endif

set yLoc    = 1
set zLoc    = 1
 
set inName  = $1
set outName = $2

if ($#argv > 2) then
   set yLoc = $3
endif

if ($#argv > 3) then
   set zLoc = $4
endif

set thisName = (`nmrPrintf $inName 1 1`)

if (!(-e $thisName)) then
   echo $0 Error Finding Input File $thisName
   exit 1
endif

set dimCount = (`getParm -in $thisName -parm FDDIMCOUNT -fmt %.0f`)
set y1Loc    = (`IMATH "2*($yLoc - 1) + 1"`)
set ynLoc    = (`IMATH "$y1Loc + 1"`)

readROI -in $inName -ndim 3 -complex -silent \
  -x X_AXIS 0% 100% -y A_AXIS 0% 100% -z Y_AXIS $y1Loc $ynLoc -da Z_AXIS $zLoc 0 -out stdout \
| nmrPipe -fn MAC -macro $NMRTXT/xa_ri.M -noRd -noWr -all -out $outName -ov 

