#!/bin/csh

if ($#argv < 2) then
   echo Extract XY plane from 3D or 4D hypercomplex FID:
   echo " 3D Usage: ext_xy.com inTemplate outName [zLoc]"
   echo " 4D Usage: ext_xy.com inTemplate outName [zLoc] [aLoc]"
   echo ""
   echo " 3D Examples:"
   echo "    ext_xy.com fid/test%03d.fid xy.fid"
   echo "    ext_xy.com fid/test%03d.fid xy.fid 2"
   echo " 4D Examples:"
   echo "    ext_xy.com fid/test%02d%03d.fid xy.fid     (Plane Series Input)"
   echo "    ext_xy.com fid/test%03d.fid     xy.fid 1 2 (Cube Series Input)"
   echo ""
   echo "Notes:"
   echo " 1. Values for yLoc or aLoc are usually 1 or 2, for the first real"
   echo "    or first imaginary point in the indirect dimension."
   echo " 2. 3D Input can be single-file or a template with a one-integer"
   echo "    format (Plane series such as test%03d.fid)."
   echo " 4. 4D Input can be single-file, a template with a one-integer"
   echo "    format (Cube series such as test%03d.fid), or a template with"
   echo "    a two-integer format (Plane series such as test%03d%03d.fid)."
   exit 1
endif

set zLoc    = 1
set aLoc    = 1
 
set inName  = $1
set outName = $2

if ($#argv > 2) then
   set zLoc = $3
endif

if ($#argv > 3) then
   set aLoc = $4
endif

set thisName = (`nmrPrintf $inName 1 1`)

if (!(-e $thisName)) then
   echo $0 Error Finding Input File $thisName
   exit 1
endif

@ fmtCount   = (`echo $inName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)
set dimCount = (`getParm -in $thisName -parm FDDIMCOUNT -fmt %.0f`)

if (`string same $dimCount 4`) then
   if (`string same $fmtCount 2`) then
      /bin/cp `nmrPrintf $inName $aLoc $zLoc` $outName
   else
      readROI -in $inName -ndim 3 -complex -silent \
        -x X_AXIS 0% 100% -y Y_AXIS 0% 100% -dz Z_AXIS $zLoc 0 -da A_AXIS $aLoc 0 -out stdout \
      | nmrPipe -fn MAC -macro $NMRTXT/xy_ri.M -noRd -noWr -all -out $outName -ov 
   endif
else if (`string same $dimCount 3`) then
   if (`string same $fmtCount 1`) then
      /bin/cp `nmrPrintf $inName $zLoc` $outName
   else
      readROI -in $inName -ndim 3 -complex -silent \
        -x X_AXIS 0% 100% -y Y_AXIS 0% 100% -dz Z_AXIS $zLoc 0 -out stdout \
      | nmrPipe -fn MAC -macro $NMRTXT/xy_ri.M -noRd -noWr -all -out $outName -ov 
   endif
else
   /bin/cp $inName $outName 
endif

