#!/bin/csh

if ($#argv < 2) then
   echo Extract XZ plane from 3D or 4D hypercomplex FID:
   echo " 3D Usage: ext_xz.com inTemplate outName [yLoc]"
   echo " 4D Usage: ext_xz.com inTemplate outName [yLoc] [aLoc]"
   echo ""
   echo " 3D Examples:"
   echo "    ext_xz.com fid/test%03d.fid xz.fid"
   echo "    ext_xz.com fid/test%03d.fid xz.fid 2"
   echo " 4D Examples:"
   echo "    ext_xz.com fid/test%02d%03d.fid xz.fid     (Plane Series Input)"
   echo "    ext_xz.com fid/test%03d.fid     xz.fid 1 2 (Cube Series Input)"
   echo ""
   echo "Notes:"
   echo " 1. Values for yLoc or aLoc are usually 1 or 2, for the first real"
   echo "    or first imaginary point in the indirect dimension."
   echo " 2. 3D Input can be single-file or a template with a one-integer"
   echo "    format (Plane series such as test%03d.fid)."
   echo " 4. 4D Input can be single-file, a template with a one-integer"
   echo "    format (Cube series such as test%03d.fid), or a template with"
   echo "    a two-integer format (Plane series such as test%03d%03d.fid)."
   exit 1
endif

set yLoc    = 1
set aLoc    = 1
 
set inName  = $1
set outName = $2

if ($#argv > 2) then
   set yLoc = $3
endif

if ($#argv > 3) then
   set aLoc = $4
endif

set thisName = (`nmrPrintf $inName 1 1`)

if (!(-e $thisName)) then
   echo $0 Error Finding Input File $thisName
   exit 1
endif

set dimCount = (`getParm -in $thisName -parm FDDIMCOUNT -fmt %.0f`)
set y1Loc    = (`IMATH "2*($yLoc - 1) + 1"`)
set ynLoc    = (`IMATH "$y1Loc + 1"`)

if (`string same $dimCount 4`) then
   readROI -in $inName -ndim 3 -complex -silent \
     -x X_AXIS 0% 100% -y Z_AXIS 0% 100% -z Y_AXIS $y1Loc $ynLoc -da A_AXIS $aLoc 0 -out stdout \
   | nmrPipe -fn MAC -macro $NMRTXT/xz_ri.M -noRd -noWr -all -out $outName -ov 
else
   readROI -in $inName -ndim 3 -complex -silent \
     -x X_AXIS 0% 100% -y Z_AXIS 0% 100% -z Y_AXIS $y1Loc $ynLoc -out stdout \
   | nmrPipe -fn MAC -macro $NMRTXT/xz_ri.M -noRd -noWr -all -out $outName -ov 
endif

