#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Truncate or Extend a List of Values So That it Has N Members:"
   echo " -n    n       Number of Output Values Desired."
   echo " -def  defVal  Default Value for Extending List."
   echo " -list valList Input List of Values."
   echo "Notes:"
   echo " 1. Option -list should be last on the command line."
   echo " 2. If no option -n is given, the original list is returned."
   echo " 3. If no option -def is given, the last item in the list"
   echo "    is used as the default for extending the list."
   exit 0
endif

set thisArgv = ($argv)
set valList  = ""

set loc   = (`flagLoc $argv -list`)
set nArgs = $#argv

if ($loc) then
   @ locPrev = $loc - 1
   @ locNext = $loc + 1

   set thisArgv = ($argv[1-$locPrev])
   @ loc++
   set valList  = ($argv[$locNext-$nArgs])
else
   exit 0
endif

@ nOrig = $#valList

set n   = (`getArgD $thisArgv -n $nOrig`)
set loc = (`flagLoc $thisArgv -def`)

if ($n < 1) then
   exit 0
endif

if ($loc) then
   @ loc++
   set defVal = $thisArgv[$loc]
else
   set defVal = $valList[$nOrig]
endif

@ i = 1

while ($i <= $n)
   if ($i > $nOrig) then
      set val = $defVal
   else
      set val = $valList[$i]
   endif

   if ($i == 1) then
      nmrPrintf "%s"  $val
   else
      nmrPrintf " %s" $val
   endif

   @ i++
end

exit 0
