#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"
 
set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]
 
if {[flagLoc $ARGV -help]} \
   {
    puts "Simple Comparison of Related 1D Spectra:"
    puts " -in inList  \[None\]     List of Related Spectra to Compare."
    puts " -x1  x1Loc  \[0%\]       Start Coord of Analysis Region."
    puts " -xn  xnLoc  \[100%\]     End Coord of Analysis Region."
    exit 1
   }

set inList None
set x1     0%
set xn     100%

getListArgD $ARGV -in inList

getArgD $ARGV -x1 x1
getArgD $ARGV -xn xn

if {![strcasecmp $inList None]} {exit 0}

#
#----------------------------------------------------------------------------

set inCount  [llength $inList]
set sizeList ""
set maxList  ""

for {set i 0} {$i < $inCount} {incr i} \
   {
    set inName [lindex $inList $i]
    set id     [expr $i + 1]

    if {![file exists $inName]} \
       {
        puts stderr "Error Finding Input File: $inName"
        exit 1
       }

    readROI -in $inName -ndim 1 -roi $id -x X_AXIS $x1 $xn

    set rInfo [getROIInfo -roi $id]

    set rdata($i) [lindex $rInfo 0]
    set fdata($i) [lindex $rInfo 1]
    set size($i)  [lindex [lindex $rInfo 2] 0]

    set noise($i) [getParm $fdata($i) FDNOISE NULL_DIM]

    lappend maxList [vMax $rdata($i) $size($i)]
    lappend sizeList $size($i)
   }

set n     [getMin $sizeList] 
set workI [sysMalloc $n float]
set workJ [sysMalloc $n float]

for {set i 0} {$i < $inCount - 1} {incr i} \
   {
    for {set j [expr $i + 1]} {$j < $inCount} {incr j} \
       {
        set sigma  [getMax "$noise($i) $noise($j)"]
        set maxVal [getMax $maxList]

        vvMove $workI $rdata($i) $n
        vvMove $workJ $rdata($j) $n

        vvSub $workI $workJ  $n
        vsDiv $workI $maxVal $n

        set rms [vRMS $workI $n]

        puts "[lindex $inList $i] [lindex $inList $j] $rms"
       }
   }

exit
