#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"
 
set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

#
# Do to:
#  adjust pk tol for data size.
 
if {[flagLoc $ARGV -help]} \
{
puts "Simple Auto 1D Process, Peak Detect, and Plot;"
puts "to be Executed in the Bruker or Varian Data Directory:"
puts " -lb      lbHz   \[0.2\]          EM Line Broaden, Hz."
puts " -zf      nZF    \[1\]            Zero Fill Count, or SIZE=N."
puts " -noise   sigma  \[Auto\]         Noise Estimate."
puts " -thresh  thresh \[Auto\]         Peak Detection Threshold."
puts " -mult    nMult  \[7.0\]          Noise Factor for Threshold."
puts " -bcDX    bcDX   \[32\]           Baseline Correct Window, Pts."
puts " -pkDX    pkDX   \[10\]           Peak Analysis Window, Pts."
puts " -p0      p0     \[0.0\]          P0 Phasing to Apply, or Keyword MC."
puts " -p1      p1     \[0.0\]          P1 Phasing to Apply."
puts " -ord     bcOrd  \[4\]            Baseline Correction Order."
puts "Optional NMRPipe Functions (End Argument List in --):"
puts " -fnT     tName  \[NULL\]         Optional NMRPipe Time-Domain Step."
puts " -fnF     fName  \[NULL\]         Optional NMRPipe Freq-Domain Step."
puts " -fnB     bName  \[NULL\]         Optional NMRPipe Post-Baseline Step."
puts "Plot Labels and Scaling; Use % Label to Scale as Percent of Range:"
puts " -plotMin pMin   \[-5%\]          Data Height for Plot Min Value."
puts " -plotMax pMax   \[Auto\]         Data Height for Plot Max Value." 
puts " -pkScale nPk    \[0\]            Number of Largest Peaks to Ignore."
puts " -pkLab                         Draw Peak Labels on Plot."
puts "Spectral Regions and Locations:"
puts " -p0X1    p0X1   \[3.0ppm\]       P0-AutoPhase Range Start (Also: -psX1)."
puts " -p0XN    p0XN   \[-0.3ppm\]      P0-AutoPhase Range End   (Also: -psXN)."
puts " -p1X1    p1X1   \[None\]         P1-AutoPhase Range Start."
puts " -p1XN    p1XN   \[None\]         P1-AutoPhase Range End."
puts " -extX1   ex1Loc \[0%\]           Extract Region Start."
puts " -extXN   exNLoc \[100%\]         Extract Region End."
puts " -noiseX1 nx1Loc \[5%\]           Noise Region Start."
puts " -noiseXN nxNLoc \[10%\]          Noise Region End."
puts " -solX0   sX0    \[50%\]          Solvent Location, or Keyword None."
puts " -extFirst                       Extract Performed Prior to Phasing."
puts " -noextFirst                     Extract Performed as Last Step (Default)."
puts "Normalization:"
puts " -norm                          Normalization ON."
puts " -nonorm                        Normalization OFF (Default)."
puts " -normX1   normX1 \[0%\]          Normalization Region Start."
puts " -normXN   normXN \[100%\]        Normalization Region End."
puts " -normMode nMode  \[Sum\]         Normalization Mode: Sum Max RMS"
puts " -normVal  nVal   \[1.0\]         Desired Value After Normalization."
puts "Calibration:"
puts " -ref                          PPM Reference Calibration ON."
puts " -noref                        PPM Reference Calibration OFF (Default)."
puts " -refX1    refX1  \[0.4ppm\]     PPM Reference Region Start."
puts " -refXN    refXN  \[97%\]        PPM Reference Region End."
puts " -refMax                       Reference by Maximum Peak in Region."
puts " -refLeft                      Reference by Leftmost Peak in Region."
puts " -refRight                     Reference by Rightmost Peak in Region (Default)."
puts " -refSet  rPPM      \[0.0\]    PPM Value for Reference Signal."
puts "File Name Specification:"
puts " -dir     dirName   \[.\]          Default Directory."
puts " -in      rawFID    \[Auto\]       Spectrometer Binary Input Data."
puts " -fid     pipeFID   \[test.fid\]   NMRPipe FID Output."
puts " -ft      ftName    \[test.ft1\]   NMRPipe Spectrum Output."
puts " -aux     auxName   \[axt.ft1\]    Auxiliary Spectrum Output."
puts " -parm    parmName  \[parm1D.txt\] Parameter Settings File." 
puts "Output Scripts Created:"
puts " -conv  convScript \[fid.com\]     NMRPipe Conversion Script Output."
puts " -proc  procScript \[ft.com\]      NMRPipe Processing Script Output."
puts " -log   logName    \[ezProc.log\]  Log File."
puts "Generation of Title from Directory Name:"
puts " -tChar sChar      \[_\]           Title Seperator Character." 
puts " -tHead hN         \[0\]           Head Elements to Skip." 
puts " -tTail tN         \[1\]           Tail Elements to Skip." 
puts "Optional Output (Use 'None' for No Output):"
puts " -tab  tabName    \[test.tab\]      NMRPipe Peak Table Output."
puts " -ps   psName     \[test.ps\]       NMRPipe PostScript Output."
puts "PostScript Plot Parameters (Use Units 'in' or 'cm'):"
puts " -plotX1   x1     \[0.5in\]         Plot Region X1 Limit."
puts " -plotXN   xn     \[10.0in\]        Plot Region XN Limit."
puts " -plotY1   y1     \[2.0in\]         Plot Region Y1 Limit."
puts " -plotYN   yn     \[7.0in\]         Plot Region YN Limit."
puts "Copy of Result Files:"
puts " -saveDir  sDir   \[save\]          Directory to Copy Result Files."
puts " -saveName sName  \[%index_%srcDir] Template Name for Result Files."
puts "Special Arguments:"
puts " -clean                   Delete Existing Output Files and Exit."
puts " -auto                    Automatically Process Input."
puts " -notk                    No Graphical Interface."
exit 1
}

proc ezProc1DCmnd { msg } \
{
   global inList wishFlag

   set str  [string trim $msg]
   set cmnd [lindex $str 0]
   set w    .ezProc1D

   if {![strcasecmp $cmnd append]} \
      {
       set i [string first " " $str]
       set thisName [string trim [string range $str [expr $i + 1] end]]

       lappend inList $thisName

       puts stderr "ezProc1D Added Input $thisName"

       if {$wishFlag} \
          {
           wm title $w "Added $thisName" 
           update
          }
      } \
   elseif {![strcasecmp $cmnd reset]} \
      {
       set inList ""
      } \
   elseif {![strcasecmp $cmnd process]} \
      {
       execProc1D
      } \
   elseif {![strcasecmp $cmnd exit]} \
      {
       proc1DExit
      }

   return 0
}

proc initProc1DVars {} \
{
   global fidName ftName auxName convName procName saveDir saveName
   global zoomFlag cleanFlag autoFlag pkLabFlag extFirstFlag p0X1 p0XN p1X1 p1XN xP0 xP1
   global fnT fnF fnB refX1 refXN extX1 extXN noiseX1 noiseXN specID solX0
   global normX1 normXN normMode normVal normFlag autoPhaseMode
   global titSplitChar titHeadSkip titTailSkip 

   global tabName psName parmName logName lb zfInfo mult bcDX bcOrder pkDX 
   global cScale cx1 cxn cy1 cyn plotMin plotMax noise thresh pkScale noiseAdj
   global initDir inName inType dirName inList minVal maxVal
   global refVal refTabName refFlag refLeftFlag procFlag
   global cancelFlag buttonBG buttonFG roiValid 

   global eList pkTab pkCount 

   set initDir   [file normalize .]
   set dirName   .

   set saveDir   save
   set saveName  %index_%srcDir 

   set inName    Auto
   set inType    Auto 
   set inList    ""

   set fidName    test.fid
   set ftName     test.ft1
   set auxName    axt.ft1
   set convName   fid.com
   set procName   ft.com
   set tabName    test.tab
   set psName     test.ps
   set parmName   parm1D.txt
   set logName    ezProc.log
   set refTabName refPPM.tab

   set titSplitChar _
   set titHeadSkip  0
   set titTailSkip  1

   set lb        1.0
   set zfInfo    "zf=1"
   set mult      7 
   set bcDX      32
   set pkDX      10
   set noiseAdj  1.0
   set specID    Auto
   set cScale    1.0
   set fnT       NULL
   set fnF       NULL
   set fnB       NULL

   set zoomFlag     0
   set cleanFlag    0
   set autoFlag     0
   set procFlag     1
   set extFirstFlag 0

   set refVal      0.0
   set refFlag     0
   set refLeftFlag 0
   set pkLabFlag   0

   set xP0       0.0
   set xP1       0.0

   set p0X1      3.0ppm
   set p0XN     -0.3ppm
   set p1X1      None
   set p1XN      None
   set extX1     0% 
   set extXN     100%
   set refX1     0.4ppm
   set refXN     97%
   set normX1    0%
   set normXN    100%
   set normMode  Sum
   set normVal   1.0
   set normFlag  0
   set noiseX1   5%
   set noiseXN   10%
   set solX0     50%
   set bcOrder   4

   set autoPhaseMode Min

   set cx1       0.5in
   set cxn      10.0in
   set cy1       2.0in
   set cyn       7.0in

   set noise     Auto 
   set thresh    Auto 
   set plotMin   -5%
   set plotMax   Auto
   set pkScale   0

   set eList     ""
   set pkTab     0
   set pkCount   0

   set minVal    -1.0e+16
   set maxVal     1.0e+16

   set roiValid(0) 0
   set roiValid(1) 0
   set roiValid(2) 0
   set roiValid(3) 0

   set buttonBG    gray30
   set buttonFG    black

   set cancelFlag  0

   return 0
}

proc initProc1DArgs {} \
{
   global initDir dirName inList cleanFlag autoFlag refVal refFlag refLeftFlag solX0 bcOrder
   global p0X1 p0XN p1X1 p1XN xP0 xP1 extX1 extXN noiseX1 noiseXN refX1 refXN
   global inName inType specID fidName ftName auxName convName procName 
   global tabName psName parmName logName lb zfInfo mult bcDX pkDX noiseAdj
   global fnT fnF fnB cx1 cxn cy1 cyn plotMin plotMax noise thresh pkScale procFlag
   global titSplitChar titHeadSkip titTailSkip saveDir saveName
   global normX1 normXN normMode normVal normFlag autoPhaseMode
   global pkLabFlag zoomFlag extFirstFlag 

   global eList pkTab pkCount
   global ARGC ARGV

   set ARGV [getListArgDT $ARGV -fnT fnT]
   set ARGV [getListArgDT $ARGV -fnF fnF]
   set ARGV [getListArgDT $ARGV -fnB fnB]
   set ARGC [llength $ARGV]

   set fnT [noSingleQuote $fnT]
   set fnF [noSingleQuote $fnF]
   set fnB [noSingleQuote $fnB]

   getListArgD $ARGV -in inList

   getArgD $ARGV -dir      dirName
   getArgD $ARGV -saveDir  saveDir
   getArgD $ARGV -saveName saveName

   if {[strcasecmp $dirName None] && [strcmp $dirName "."]} \
      {
       set error [proc1DCD $dirName]
      }

   if {[strcasecmp $dirName None]} {set dirName [file normalize $dirName]}
   if {[strcasecmp $saveDir None]} {set saveDir [file normalize $saveDir]}

   getArgD $ARGV -type     inType
   getArgD $ARGV -fid      fidName
   getArgD $ARGV -ft       ftName
   getArgD $ARGV -conv     convName
   getArgD $ARGV -aux      auxName
   getArgD $ARGV -parm     parmName
   getArgD $ARGV -proc     procName
   getArgD $ARGV -tab      tabName
   getArgD $ARGV -ps       psName
   getArgD $ARGV -log      logName

   getArgD $ARGV -tChar    titSplitChar
   getArgD $ARGV -tHead    titHeadSkip
   getArgD $ARGV -tTail    titTailSkip

   getArgD $ARGV -lb       lb
   getArgD $ARGV -zf       zfInfo
   getArgD $ARGV -noise    noise 
   getArgD $ARGV -thresh   thresh
   getArgD $ARGV -bcDX     bcDX
   getArgD $ARGV -pkDX     pkDX
   getArgD $ARGV -mult     mult
   getArgD $ARGV -nadj     noiseAdj
   getArgD $ARGV -id       specID

   getArgD $ARGV -plotMin  plotMin 
   getArgD $ARGV -plotMax  plotMax 
   getArgD $ARGV -pkScale  pkScale 

   getArgD $ARGV -plotX1   cx1
   getArgD $ARGV -plotXN   cxn
   getArgD $ARGV -plotY1   cy1
   getArgD $ARGV -plotYN   cyn

   getArgD $ARGV -p0       xP0
   getArgD $ARGV -p1       xP1

   if {![strcasecmp $xP0 MC]} \
      {
       set p0X1 MC
       set p0XN MC
       set p1X1 None
       set p1XN None
      }

   getArgD $ARGV -autoPhase autoPhaseMode 

   getArgD $ARGV -psX1     p0X1
   getArgD $ARGV -p0X1     p0X1

   if {![strcasecmp $p0X1 MC]} {set p0XN MC}

   getArgD $ARGV -psXN     p0XN
   getArgD $ARGV -p0XN     p0XN
 
   getArgD $ARGV -p1X1     p1X1
   getArgD $ARGV -p1XN     p1XN

   getArgD $ARGV -extX1    extX1
   getArgD $ARGV -extXN    extXN
   getArgD $ARGV -refX1    refX1
   getArgD $ARGV -refXN    refXN
   getArgD $ARGV -noiseX1  noiseX1
   getArgD $ARGV -noiseXN  noiseXN
   getArgD $ARGV -solX0    solX0
   getArgD $ARGV -bcOrd    bcOrder

   getArgD $ARGV -normX1   normX1
   getArgD $ARGV -normXN   normXN
   getArgD $ARGV -normMode normMode
   getArgD $ARGV -normVal  normVal

   if {[flagLoc $ARGV -proc]}     {set procFlag 1}
   if {[flagLoc $ARGV -noproc]}   {set procFlag 0}

   if {[flagLoc $ARGV -zoom]}     {set zoomFlag 1}
   if {[flagLoc $ARGV -nozoom]}   {set zoomFlag 0}

   if {[flagLoc $ARGV -auto]}     {set autoFlag 1}
   if {[flagLoc $ARGV -noauto]}   {set autoFlag 0}

   if {[flagLoc $ARGV -ref]}      {set refFlag 1}
   if {[flagLoc $ARGV -noref]}    {set refFlag 0}

   if {[flagLoc $ARGV -norm]}     {set normFlag 1}
   if {[flagLoc $ARGV -nonorm]}   {set normFlag 0}

   if {[flagLoc $ARGV -refLeft]}  {set refLeftFlag  1}
   if {[flagLoc $ARGV -refRight]} {set refLeftFlag  0}
   if {[flagLoc $ARGV -refMax]}   {set refLeftFlag -1}

   if {[flagLoc $ARGV -pkLab]}    {set pkLabFlag 1}
   if {[flagLoc $ARGV -nopklab]}  {set pkLabFlag 0}

   if {[flagLoc $ARGV -clean]}    {set cleanFlag 1}
   if {[flagLoc $ARGV -noclean]}  {set cleanFlag 0}

   if {[flagLoc $ARGV -extFirst]}   {set extFirstFlag 1}
   if {[flagLoc $ARGV -noextFirst]} {set extFirstFlag 0}

   if {$cleanFlag} \
      {
       proc1DClean
       exit 0
      }

   if {[strcasecmp $fnT NULL]} {puts "Extra Time-Domain Step:   $fnT"}
   if {[strcasecmp $fnF NULL]} {puts "Extra Freq-Domain Step:   $fnF"}
   if {[strcasecmp $fnB NULL]} {puts "Extra Post-Baseline Step: $fnB"}

   return 0
}

#
#------------------------------------------------------------------------------

proc proc1DCD { d } \
{
   if {![file isdirectory $d]} \
      {
       showThisError ezProc1D "Error: Requested Directory $d is not a directory."
       return 1
      }

   cd $d  
}

proc proc1DClean {} \
{
   global fidName ftName auxName convName procName tabName psName logName
   global inList cleanFlag

   if {!$cleanFlag} {return 0}

   set fList \
    "$fidName $ftName $auxName $convName $procName $tabName $psName $logName"

   lappend fList pk.tcl
   lappend fList pkROI_1d.dat
 
   if {[llength $inList]} \
      {
       set d [pwd]

       foreach iName $inList \
          {
           set dName [getDirName $iName]
           cd  $dName

           puts "Directory $dName:"

           foreach fName $fList \
              {
               if {![strcasecmp $fName None]} continue
               if {![file exists $fName]} continue
               puts "Deleting $fName"
               catch "file delete $fName"
              }

           cd $d
          }
      } \
   else \
      {
       foreach fName $fList \
          {
           if {![strcasecmp $fName None]} continue
           if {![file exists $fName]} continue

           puts "Deleting $fName"
           catch "file delete $fName"
          }
      }

    return 0
}

proc initProc1DWin {} \
{
   global inName inType specID saveName saveDir fidName ftName auxName convName 
   global lb noise thresh bcDX bcOrder pkDX mult noiseAdj specID procFlag 
   global procName tabName psName logName parmName
   global wishFlag autoFlag fixedfont env

   global xP0 xP1 p0X1 p0XN p1X1 p1XN extX1 extXN noiseX1 noiseXN refX1 refXN 
   global zoomFlag refFlag refLeftFlag pkLabFlag pkScale buttonBG buttonFG
   global normX1 normXN normMode normVal normFlag autoPhaseMode
   global plotMin plotMax

   if {!$wishFlag} {return 0}

   setFonts

   set w .ezProc1D

   toplevel $w 

   wm geom $w -20-70 

   frame $w.left   -relief groove -border 3
   frame $w.right  -relief groove -border 3
   frame $w.bottom 
   frame $w.top    -relief groove -border 3
   frame $w.in
   frame $w.cmnd

   set menuInfo(ppmMCMenu) \
    {
     None MC
      12.0ppm 11.5ppm 11.0ppm 10.5ppm 10.0ppm 9.0ppm 8.0ppm 7.0ppm 6.0ppm
      5.0ppm  4.5ppm 4.0ppm   3.5ppm  3.0ppm 2.0ppm 1.0ppm 0.5ppm 0.2ppm
      0.1ppm  0.0ppm -0.1ppm -0.2ppm -0.5ppm -1.0ppm
      0% 3% 5% 10% 40% 50% 60% 90% 95% 97% 100%
    }

   set menuInfo(ppmMenu) \
    {
     None
      12.0ppm 11.5ppm 11.0ppm 10.5ppm 10.0ppm 9.0ppm 8.0ppm 7.0ppm 6.0ppm 
      5.0ppm  4.5ppm 4.0ppm   3.5ppm  3.0ppm 2.0ppm 1.0ppm 0.5ppm 0.2ppm
      0.1ppm  0.0ppm -0.1ppm -0.2ppm -0.5ppm -1.0ppm 
      0% 3% 5% 10% 40% 50% 60% 90% 95% 97% 100%
    }

   set menuInfo(hiMenu) \
    {Auto -10% -5% -1% 0% 1% 5% 10% 90% 95% 100% 105% 115%}

   set menuInfo(refMenu)   {None "Leftmost Peak" "Rightmost Peak"}

   set menuInfo(phaseMenu) {0.0 45.0 90.0 180.0 -180.0 -90.0 -45.0}
 
   set eInfo \
      {
       {p0X1    p0XN    "Auto-P0 Region Limits"    ppmMCMenu}
       {p1X1    p1XN    "Auto-P1 Region Limits"    ppmMenu}
       {xP0     xP1     "Additional P0 and P1"     phaseMenu}
       {noiseX1 noiseXN "Noise Region Limits"      ppmMenu}
       {extX1   extXN   "Extract Region Limits"    ppmMenu}
       {plotMin plotMax "Plot Height Limits"       hiMenu}
       {normX1  normXN  "Normalization Limits"     ppmMenu}
       {refX1   refXN   "Reference Peak Limits"    ppmMenu}
      }

   foreach e $eInfo \
      {
       set v1 [lindex $e 0]
       set vN [lindex $e 1]
       set s  [lindex $e 2]
       set m  [lindex $e 3]

       frame $w.lim$v1
       label $w.limLab$v1 -text "$s:" -width 22 -anchor e

       entry $w.limVal$v1 -textvariable $v1 -width 10 \
             -validate key -validatecommand "rejectEntryWhitespace %S"

       entry $w.limVal$vN -textvariable $vN -width 10 \
             -validate key -validatecommand "rejectEntryWhitespace %S"

       menubutton $w.limMenu$v1 -menu $w.limMenu$v1.menu \
          -relief raised -bitmap @$env(NMRTXT)/caret.map

       label $w.limSpace$v1 -width 1 -text " "

       menubutton $w.limMenu$vN -menu $w.limMenu$vN.menu \
          -relief raised -bitmap @$env(NMRTXT)/caret.map

       menu $w.limMenu$v1.menu -tearoff false
       menu $w.limMenu$vN.menu -tearoff false

       foreach i $menuInfo($m) \
          {
           $w.limMenu$v1.menu add command -label $i -font $fixedfont \
             -command "setProc1DVal $v1 $i"

           $w.limMenu$vN.menu add command -label $i -font $fixedfont \
             -command "setProc1DVal $vN $i"
          }

       pack $w.limLab$v1 $w.limVal$v1 $w.limMenu$v1 $w.limSpace$v1 \
            $w.limVal$vN $w.limMenu$vN -in $w.lim$v1 -side left

       pack $w.lim$v1 -in $w.left -side top

       bind  $w.limLab$v1 <Any-Enter> "showTitle \"$s\" $v1 $vN"
       bind  $w.limLab$v1 <Any-Leave> "showTitle ezProc1D"
      }

   frame $w.cb
   frame $w.ref 
   frame $w.norm
   frame $w.cbTop
   frame $w.cbMid
   frame $w.cbBot

   checkbutton $w.extFirst -selectcolor blue -anchor e -text \
               "Extract Prior to Phase" -variable extFirstFlag 

   checkbutton $w.limZoom -selectcolor blue -anchor e -text \
               "Auto Zoom" -variable zoomFlag

   checkbutton $w.pkLabFlag -selectcolor blue -anchor e -text \
               "Label Peaks" -variable pkLabFlag

   entry       $w.refVal -width 5 -textvariable refVal

   checkbutton $w.refFlag -selectcolor blue -anchor e -text \
               "Reference PPM" -variable refFlag

   radiobutton $w.refMax -selectcolor pink -anchor e -text \
               "Max" -variable refLeftFlag -value -1

   radiobutton $w.refLeft -selectcolor pink -anchor e -text \
               "Left" -variable refLeftFlag -value 1
              
   radiobutton $w.refRight -selectcolor pink -anchor e -text \
               "Right" -variable refLeftFlag -value 0

   entry       $w.normVal -width 5 -textvariable normVal

   checkbutton $w.normFlag -selectcolor blue -anchor e -text \
               "Normalize" -variable normFlag

   radiobutton $w.normSum -selectcolor pink -anchor e -text \
               "Sum" -variable normMode -value Sum 

   radiobutton $w.normMax -selectcolor pink -anchor e -text \
               "Max" -variable normMode -value Max

   radiobutton $w.normVar -selectcolor pink -anchor e -text \
               "RMS" -variable normMode -value RMS 

   pack $w.refFlag  $w.refVal  $w.refMax  $w.refLeft $w.refRight -in $w.ref -side left 
   pack $w.normFlag $w.normVal $w.normSum $w.normMax $w.normVar  -in $w.norm -side left 
   pack $w.norm -in $w.cbMid -side left -padx 2 -pady 2
   pack $w.ref  -in $w.cbTop -side left -padx 2 -pady 2

   pack $w.extFirst $w.limZoom $w.pkLabFlag -in $w.cbBot -side left -expand 1 -fill x
   pack $w.cbTop $w.cbMid $w.cbBot -in $w.cb -side top 
   pack $w.cb -in $w.left -side top -expand 1 -fill x

   set menuInfo(zfInfo) \
    {
     zf=1 zf=2 zf=3 \
     size=1024 size=2048 size=4096 size=8092 \
     size=16384 size=32769 size=65536 size=131072
    }

   set menuInfo(lb) {0.0 0.1 0.2 0.5 1.0 2.0 3.0 4.0 5.0 10.0 15.0 20.0 25.0}

   set menuInfo(noise)   {Auto 0.001% 0.1% 1% 10% 25%}
   set menuInfo(thresh)  {Auto 0.001% 0.1% 1% 10% 25%}
   set menuInfo(bcDX)    {4 8 10 12 16 20 32 64 128}
   set menuInfo(bcOrder) {None 0 1 2 3 4 5 6 7 8}
   set menuInfo(pkDX)    {1 2 3 4 5 10 15 20 25}
   set menuInfo(solX0)   {None 50% 4.5ppm 4.6ppm 4.7ppm 4.8ppm 4.9ppm 5.0ppm}

   set menuInfo(autoPhaseMode) {None Min RMS}

   set menuInfo(mult) \
    {1.0 2.0 3.0 4.0 5.0 6.0 7.0 8.0 9.0 10.0 20.0 25.0 50.0 75.0 100.0 1000.0}

   set eInfo \
    {
     {zfInfo        "Zero Fill"}
     {lb            "Exponential Line Broaden, Hz"}
     {autoPhaseMode "Auto-Phase Metric"}
     {noise         "Noise Level"}
     {thresh        "Peak Detect Threshold"}
     {mult          "Factor for Auto Threshold"}
     {pkDX          "Peak Analysis Window, Pts"}
     {bcDX          "Baseline Correct Window, Pts"}
     {bcOrder       "Baseline Correct Order"}
     {solX0         "Solvent Position"}
    }

   foreach e $eInfo \
      {
       set v [lindex $e 0]
       set s [lindex $e 1]

       frame $w.proc$v
       label $w.procLab$v -text "$s:" -width 27 -anchor e

       entry $w.procVal$v -textvariable $v -width 10 \
             -validate key -validatecommand "rejectEntryWhitespace %S"

       menubutton $w.procMenu$v -menu $w.procMenu$v.menu \
          -relief raised -bitmap @$env(NMRTXT)/caret.map

       menu $w.procMenu$v.menu -tearoff false

       foreach i $menuInfo($v) \
          {
           $w.procMenu$v.menu add command -label $i -font $fixedfont \
             -command "setProc1DVal $v $i"
          }

       pack $w.procLab$v $w.procVal$v $w.procMenu$v -in $w.proc$v -side left
       pack $w.proc$v -in $w.right -side top

       bind  $w.procLab$v <Any-Enter> "showTitle \"$s\" $v"
       bind  $w.procLab$v <Any-Leave> "showTitle ezProc1D"
      }

   set eInfo \
    {
     {convName Left  18 "Conversion Script"}
     {procName Left  18 "Processing Script"}
     {parmName Left  18 "EZProc Parameters"}
     {fidName  Mid   12 "FID Name"}
     {ftName   Mid   12 "FT Name"}
     {auxName  Mid   12 "Aux Name"}
     {tabName  Right 17 "Peak Table Name"}
     {psName   Right 17 "PostScript Name"}
     {logName  Right 17 "Log File Name"}
    }

   frame $w.fileFrameLeft
   frame $w.fileFrameMid
   frame $w.fileFrameRight

   foreach e $eInfo \
      {
       set v [lindex $e 0]
       set f [lindex $e 1]
       set n [lindex $e 2]
       set s [lindex $e 3]

       frame $w.file$v
       label $w.fileLab$v -text "$s:" -width $n -anchor e
       entry $w.fileVal$v -textvariable $v -width 14
       
       pack $w.fileLab$v $w.fileVal$v -in $w.file$v -side left
       pack $w.file$v -in $w.fileFrame$f -side top

       bind  $w.fileLab$v <Any-Enter> "showTitle \"$s\" $v"
       bind  $w.fileLab$v <Any-Leave> "showTitle ezProc1D"
      }

   pack $w.fileFrameLeft  $w.fileFrameMid $w.fileFrameRight \
        -in $w.bottom -side left -expand 1 -fill x

   pack $w.left $w.right -in $w.top -side left -expand 1 -fill both

   frame $w.file1
   frame $w.file2

   frame $w.inDirFrame
   frame $w.inNameFrame
   frame $w.inTypeFrame

   frame $w.saveDirFrame
   frame $w.saveNameFrame
   frame $w.specIDFrame

#
   label $w.inDirLab -text "Input Directory:" -width 14 -anchor e
   entry $w.inDirVal -textvariable dirName -width 36 

   menubutton $w.inDirButton -relief raised -fg green \
    -bitmap @$env(NMRTXT)/caret.map

#
   set menuInfo(saveDir) {"Select Directory ..." None}

   label $w.saveDirLab -text "Copy Result to:" -width 14 -anchor e
   entry $w.saveDirVal -textvariable saveDir -width 36 

   menubutton $w.saveDirMenu -relief raised -fg green \
    -menu $w.saveDirMenu.menu -bitmap @$env(NMRTXT)/caret.map

   menu $w.saveDirMenu.menu -tearoff false

   foreach i $menuInfo(saveDir) \
      {
       $w.saveDirMenu.menu add command -label $i -font $fixedfont \
         -command "setProc1DVal saveDir \"$i\""
      }

#
   set menuInfo(inName) {Auto fid ser}

   label $w.inNameLab -text "Input Name:" -width 10 -anchor e
   entry $w.inNameVal -textvariable inName -width 14 

   menubutton $w.inNameMenu -relief raised -menu $w.inNameMenu.menu \
    -bitmap @$env(NMRTXT)/caret.map

   menu $w.inNameMenu.menu -tearoff false

   foreach i $menuInfo(inName) \
      {
       $w.inNameMenu.menu add command -label $i -font $fixedfont \
         -command "setProc1DVal inName $i"
      }

#
   set menuInfo(inType) {Auto bruker varian delta}

   label $w.inTypeLab -text "Input Type:" -width 10 -anchor e
   entry $w.inTypeVal -textvariable inType -width 5 

   menubutton $w.inTypeMenu -relief raised -menu $w.inTypeMenu.menu \
    -bitmap @$env(NMRTXT)/caret.map 

   menu $w.inTypeMenu.menu -tearoff false

   foreach i $menuInfo(inType) \
      {
       $w.inTypeMenu.menu add command -label $i -font $fixedfont \
         -command "setProc1DVal inType $i"
      }

#
   set menuInfo(saveName) {%index %srcDir}

   label $w.saveNameLab -text "Save Name:" -width 10 -anchor e
   entry $w.saveNameVal -textvariable saveName -width 14 

   menubutton $w.saveNameMenu -relief raised -menu $w.saveNameMenu.menu \
    -bitmap @$env(NMRTXT)/caret.map

   menu $w.saveNameMenu.menu -tearoff false

   foreach i $menuInfo(saveName) \
      {
       $w.saveNameMenu.menu add command -label $i -font $fixedfont \
         -command "setProc1DVal saveName $i"
      }

#
   set menuInfo(specID) {1 Auto}

   label $w.specIDLab -text "Index:" -width 10 -anchor e
   entry $w.specIDVal -textvariable specID -width 5

   menubutton $w.specIDMenu -relief raised -menu $w.specIDMenu.menu \
    -bitmap @$env(NMRTXT)/caret.map

   menu $w.specIDMenu.menu -tearoff false

   foreach i $menuInfo(specID) \
      {
       $w.specIDMenu.menu add command -label $i -font $fixedfont \
         -command "setProc1DVal specID $i"
      }

   pack $w.inDirLab  $w.inDirVal  $w.inDirButton -in $w.inDirFrame  -side left
   pack $w.inNameLab $w.inNameVal $w.inNameMenu  -in $w.inNameFrame -side left
   pack $w.inTypeLab $w.inTypeVal $w.inTypeMenu  -in $w.inTypeFrame -side left

   pack $w.saveDirLab  $w.saveDirVal  $w.saveDirMenu   -in $w.saveDirFrame \
        -side left

   pack $w.saveNameLab $w.saveNameVal $w.saveNameMenu  -in $w.saveNameFrame \
        -side left

   pack $w.specIDLab   $w.specIDVal   $w.specIDMenu    -in $w.specIDFrame \
        -side left

   pack $w.inDirFrame   $w.inNameFrame   $w.inTypeFrame -in $w.file1 \
        -side left -padx 6 
 
   pack $w.saveDirFrame $w.saveNameFrame $w.specIDFrame -in $w.file2 \
        -side left -padx 6

   pack $w.file1 $w.file2 -in $w.in -side top    

   bind  $w.inDirLab <Any-Enter> "showTitle \"Input Dir\" dirName"
   bind  $w.inDirLab <Any-Leave> "showTitle ezProc1D"

   bind  $w.inNameLab <Any-Enter> "showTitle \"Input Name\" inName"
   bind  $w.inNameLab <Any-Leave> "showTitle ezProc1D"

   bind  $w.inTypeLab <Any-Enter> "showTitle \"Input Type\" inType"
   bind  $w.inTypeLab <Any-Leave> "showTitle ezProc1D"

   bind  $w.saveDirLab <Any-Enter> "showTitle \"Save Dir\" saveDir"
   bind  $w.saveDirLab <Any-Leave> "showTitle ezProc1D"

   bind  $w.saveNameLab <Any-Enter> "showTitle \"Save Name Template\" saveName"
   bind  $w.saveNameLab <Any-Leave> "showTitle ezProc1D"

   bind  $w.specIDLab <Any-Enter> "showTitle \"Spectrum Index\" specID"
   bind  $w.specIDLab <Any-Leave> "showTitle ezProc1D"

   bind  $w.inDirButton <ButtonRelease-1> "+ setProc1DInput"

   button $w.load -text "Load EZProc Parameters" -command loadProc1DParms
   button $w.save -text "Save EZProc Parameters" -command saveProc1DParms
   button $w.quit -text "Quit"                   -command proc1DExit 

   if {$procFlag} \
      {
       button $w.proc -text "Process" -command execProc1D
      } 

   if {$procFlag} \
      {
       pack $w.proc $w.load $w.save $w.quit -in $w.cmnd \
            -side left -expand 1 -fill x
      } \
   else \
      {
       pack $w.load $w.save $w.quit -in $w.cmnd -side left -expand 1 -fill x
      }

   pack $w.top    -in $w -side top -expand 1 -fill both
   pack $w.bottom -in $w -side top -expand 1 -fill both -pady 8
   pack $w.in     -in $w -side top -pady 6
   pack $w.cmnd   -in $w -side top -expand 1 -fill x

   set buttonFG [$w.quit cget -fg]
   set buttonBG [$w.save cget -bg]
}

proc proc1DCancelON {} \
{
   global wishFlag cancelFlag

   set cancelFlag 0

   if {!$wishFlag} return

   set w .ezProc1D

   $w.quit configure -text Cancel -fg white -bg red -command proc1DCancelSet
   update
}

proc proc1DCancelOFF {} \
{
   global wishFlag cancelFlag buttonBG buttonFG

   set cancelFlag 0

   if {!$wishFlag} return

   set w .ezProc1D

   $w.quit configure -text Quit -fg $buttonFG -bg $buttonBG -command proc1DExit
   update
}

proc proc1DCancelSet {} \
{
   global cancelFlag
   set cancelFlag 1
}

proc initProc1DViewer {} \
{
   global wishFlag dirName

   if {!$wishFlag} {return 0}

   wm geom  . +20-50
   wm title . "ezProc1D 2015.004.15.38"

   set xSize1D 600
   set ySize1D 240

   set qx1   10
   set qxn   [expr $xSize1D - 10]
   set qy1   10
   set qyn   [expr $ySize1D - 10]

   set sx1   20
   set sxn   [expr $sx1 + 100]
   set sy1   20 
   set syn   [expr $sy1 + 40]

   canvas .c -width $xSize1D -height $ySize1D -border 0
   pack   .c -in . -side top -expand 1 -fill both

   .c create pixmap 0 0 -width $xSize1D -height $ySize1D -win 1 

   defineCell -x $qx1 $qxn -y $qy1 $qyn -cell 2 -win 1
   defineCell -x $sx1 $sxn -y $sy1 $syn -cell 3 -win 1
 
   drawRec -x 0% 100% -y 0% 100% -fg gray20 
}

proc rejectEntryWhitespace s \
{
  if {![strcmp $s " "]} {return 0}
  return 1
}

proc showTitle args \
{
   set vList [lrange $args 1 end]
   set n     [llength $vList]
   set s     [lindex $args 0]

   if {$n} {append s ":"}

   foreach v [lrange $args 1 end] \
      {
       global $v
       append s " [subst \$$v]"
      }

   wm title .ezProc1D $s

   update
}

proc setProc1DVal { varPtr val } \
{
   global $varPtr p0X1 p0XN

   if {![strcmp $varPtr saveDir]} \
      {
       if {[strcasecmp $val None]} \
          {
           setProc1DSaveDir
           return
          }
      }

   eval "set $varPtr $val"

   if {![strcmp $varPtr p0X1] || ![strcasecmp $varPtr p0XN]} \
      {
       if {![strcasecmp $val MC]} \
          {
           set p0XN MC
           set p1XN MC
          }
      }
 
   update
}

proc setProc1DInput {} \
{
   global dirName inName 

   set types \
      {
       { {All Files}  *      }
       { {ser}        {*ser* *SER*} }
       { {fid}        {*fid* *FID*} }
       { {jdf}        {.jdf .JDF} }
       { {dat}        {.dat .DAT} }
       { {bin}        {.bin .BIN} }
       { {nv}         {.nv .NV} }
      }

   set thisName \
    [tk_getOpenFile -title "Spectrometer Input" -filetypes $types \
                    -initialdir $dirName]

   if {![string length $thisName]} {return 1}
   if {![file exists $thisName]}   {return 1}

   set fullName [file normalize $thisName]
   set dirName  [getDirName $fullName]
   set inName   [file tail  $fullName]

   return 0 
}

proc setProc1DSaveDir {} \
{
   global saveDir 

   set newDir \
    [tk_chooseDirectory -mustexist false -title "Save Directory" \
                        -initialdir $saveDir]

   if {![string length $newDir]} return

   if {[strcasecmp $newDir None]} {set newDir [file normalize $newDir]}

   set saveDir $newDir
}

proc saveProc1DParms {} \
{
   global procName parmName saveDir

# parmName

   set vList \
    {
     inName inType fidName ftName auxName convName procName \
     tabName psName logName zfInfo lb mult bcDX pkDX noiseAdj \
     cx1 cxn cy1 cyn plotMin plotMax noise thresh pkScale \
     saveName zoomFlag cleanFlag refFlag refLeftFlag refVal pkLabFlag \
     autoPhaseMode xP0 xP1 p0X1 p0XN p1X1 p1XN extX1 extXN refX1 refXN 
     normX1 normXN normMode normVal normFlag extFirstFlag
     noiseX1 noiseXN bcOrder solX0
    }

   if {![strcasecmp $saveDir None]} {lappend vList saveDir}
 
   set types \
      {  
       { {Text}       {.txt}  }
       { {TCL}        {.tcl}  }
       { {tab}        {.tab}  }
       { {List}       {.list} }
       { {All Files}  *       }
      }

   set outName \
    [tk_getSaveFile -title "Parameter Output" -filetypes $types \
                    -initialfile $parmName]

   if {![string length $outName]} return

   set parmName $outName

   set error [catch {set outUnit [open $parmName w]}]

   if {$error} \
      {
       showThisError ezProc1D "Error Opening Output File $parmName"
       return 1
      }

   foreach v $vList \
      {
       global $v
       catch {eval "puts $outUnit \"set $v \$$v\""}
      }

   close $outUnit

   return 0
}

proc loadProc1DParms {} \
{
   global parmName

   set types \
      {
       { {Text}       {.txt}  }
       { {TCL}        {.tcl}  }
       { {tab}        {.tab}  }
       { {List}       {.list} }
       { {All Files}  *       }
      }

   set thisName \
    [tk_getOpenFile -title "Parameter Input" -filetypes $types \
                    -initialfile $parmName]

   if {![string length $thisName]} {return 0}

   if {![file exists $thisName]} \
      {
       showThisError ezProc1D "Error Opening Input File $thisName"
       return 1
      }

   set parmName $thisName

   set error [readProc1DParams]

   return $error
}

proc readProc1DParms {} \
{
   global parmName psX1 psXN p0X1 p0XN

   if {![file exists $parmName]} {return 0} 

   puts stderr "Reading Parameters from $parmName"

   set error [catch {set inUnit [open $parmName r]}]

   set psX1 Null
   set psXN Null

   while {[gets $inUnit line] >= 0} \
      {
       set key [lindex $line 0]
       if {[strcmp $key set]} continue       

       set var [lindex $line 1]
       set val [lrange $line 2 end]

       global $var

       set $var $val
      }

   if {[info exists psX1]} \
      {
       if {[strcasecmp $psX1 Null]} {set p0X1 $psX1}
      }

   if {[info exists psXN]} \
      {
       if {[strcasecmp $psXN Null]} {set p0XN $psXN}
      }

   close $inUnit

   return 0 
}

proc showThisError { id msg } \
{
   global wishFlag

   showMsg $msg

   if {!$wishFlag} return

   set w .errLogWin

   if {![winfo exists $w]} \
      {
       toplevel $w
       wm title $w "ezProc1D Error Log"

       text $w.errText \
            -height 12 -width 80 -relief groove -borderwidth 3 -background white -foreground firebrick \
            -yscrollcommand "$w.scrollY set" 

       scrollbar $w.scrollY -command "$w.errText yview" -relief sunken

       pack $w.scrollY -side right -fill y
       pack $w.errText -side left  -padx 1m -pady 1m

       update
      }

   $w.errText insert end "$msg\n" 
}

proc showMsg { msg } \
{
   global logName

   puts stderr $msg
   exec echo $msg >> $logName
}

proc logTCLCmnd { cmnd } \
{
   global logName

   puts stderr $cmnd
   catch {eval "exec echo $cmnd >> $logName"}
   catch {eval $cmnd}
}

proc logSysCmnd { cmnd } \
{
   global logName

   puts stderr $cmnd
   catch {eval "exec echo $cmnd >> $logName"}
   catch {eval "exec $cmnd >> $logName"}
}        


proc absSortDesc { v1 v2 } \
{
   if {abs($v1) > abs($v2)} {return -1}
   if {abs($v1) < abs($v2)} {return  1}

   return 0
}

proc findInType { defType thisDir } \
{
   if {[strcasecmp $defType Auto]}  {return $defType} 

   if {[file exists $thisDir/procpar]} {return varian} 

   set aqList [glob -nocomplain $thisDir/acq*]
   if {[llength $aqList]} {return bruker}

   set jList [glob -nocomplain $thisDir/*.jdf* $thisDir/*.JDF*] 
   if {[llength $aqList]} {return delta}

   return None
}

proc findInName { thisType defName thisDir } \
{
   if {[strcasecmp $defName Auto]} {return $defName}

   switch -exact [string tolower $thisType] \
      {
       bruker  {set fList "ser ser.Z ser.z ser.gz fid fid.Z fid.z fid.gz"}
       varian  {set fList "fid.Z fid.z fid.gz fid"}
       delta   {set fList [glob -nocomplain $thisDir/*.jdf* $thisDir/*.JDF*]}
       default {set fList ""}
      }

   foreach thisName $fList \
      {
       if {![file exists $thisDir/$thisName]} continue
       if {[file isdirectory $thisDir/$thisName]}  continue

       return $thisDir/$thisName
      }

   return None
}

proc proc1DLogStart {} \
{
   global dirName

   showMsg "NMRPipe Conversion and Processing at [sysTime %Y.%j.%H.%M]"
   logSysCmnd "ls -l $dirName"

   return 0
}

proc proc1DLogConclude {} \
{
   global dirName 

   logSysCmnd "ls -l $dirName"
   return 0
}

proc getProc1DTitle { d } \
{
   global titSplitChar titHeadSkip titTailSkip 

   if {![file exists $d/title] && [file exists $d/text]} \
      {
       set tStr [join [string trim [getNthLine $d/text 1]] -]

       if {[string length $tStr]} \
          {
           catch {exec echo $tStr > $d/title}
           return $tStr
          }
      }

   if {[file exists $d/title]} \
      {
       set tStr [join [string trim [exec cat $d/title]] -]
       if {[string length $tStr]} {return $tStr}
      }

   set tStr  [file root [file tail [file normalize $d]]]
   set tList [split $tStr $titSplitChar]
   set n     [llength $tList]

   if {$n > $titTailSkip} \
      {
       set tList [lrange  $tList 0 [expr $n - ($titTailSkip + 1)]]
       set n     [llength $tList]
      }
   
   if {$n > $titHeadSkip} \
      {
       set tList [lrange $tList $titHeadSkip end] 
      } 

   set tStr [join $tList $titSplitChar]

   catch {exec echo $tStr > $d/title}

   return $tStr
}

proc getProc1DScale { d } \
{
   global cScale

   set cScale 1.0

   if {![file exists $d/scale]} return

   set cScale [exec cat $d/scale]

   return 
}

proc getPctVal { p thisMin thisMax } \
{
   if {[string first "%" $p] < 0} {return $p}

   set pct [string trim $p "%"]
   set r   [getMaxAbs "$thisMin $thisMax"]
   set v   [format %.3e [expr $pct*$r/100.0]]

   return $v
}

proc proc1DConv {} \
{
   global inName inType convName dirName fidName specID

   set inType [findInType $inType $dirName]

   if {[strcasecmp $inName None] && [strcasecmp $convName None]} \
      {
       if {![strcasecmp $inType None]} \
          {
           showMsg "Error Determining Input Data Type. Dir: $dirName"
           return 1
          }

       set inName [findInName $inType $inName $dirName]

       if {![file exists $inName]} \
          {
           showMsg "Error Finding Input for $inType: $inName Dir: $dirName"
           return 1
          }

       showMsg \
        "Creating Conversion Script $convName Command: $inType Dir: $dirName"

       set tStr [getProc1DTitle $dirName]

       set    cmnd "$inType -auto -notk -nosleep -script $convName -cd $dirName"
       append cmnd " -amx -extra -title $tStr -u1 $specID -end "

       logTCLCmnd "file delete $convName"
       logSysCmnd $cmnd

       if {![file exists $convName]} \
          {
           showMsg "Error Creating Conversion Script $convName Dir: $dirName"
           return 1 
          }

       logTCLCmnd "file delete $fidName"
       logSysCmnd "$convName"

       if {![file exists $fidName]} \
          {
           showMsg "Error Creating NMRPipe FID $fidName Dir: $dirName"
           return 1 
          }

       showMsg "Created NMRPipe FID $fidName"
      }

   return 0
}

proc proc1DProc {} \
{ 
   global fidName ftName auxName procName dirName refVal refFlag refTabName refLeftFlag extFirstFlag
   global fnT fnF fnB cScale lb zfInfo refX1 refXN xP0 xP1 p0X1 p0XN p1X1 p1XN autoPhaseMode
   global normFlag normMode normX1 normXN normVal extX1 extXN bcDX pkDX 

   if {[strcasecmp $ftName None] && \
        [strcasecmp $fidName None] && \
          [strcasecmp $procName None]} \
      {
       if {![file exists $fidName]} \
          {
           showMsg "Error Finding NMRPipe FID $fidName"
           return 1
          }

       showMsg "Creating Processing Script $procName"
 
       logTCLCmnd "file delete $procName"

       getProc1DScale $dirName

       catch "set fp [open $procName w 0744]"
  
       set fScale 0.5 

       if {$xP1 != 0.0} {set fScale 1.0}
       if {[strcasecmp $p1X1 None] && [strcasecmp $p1XN None]} {set fScale 1.0}

       set zfList [split $zfInfo "=,"]
       set n      [llength $zfList]
       set zfArgs ""

       if {$n == 0} \
          {
           set zfArgs "-zf 1 -auto"
          } \
       elseif {$n == 1} \
          {
           set zfKey [lindex $zfList 0]
           set zfArgs "-zf $zfKey -auto"
          } \
       else \
          {
           for {set i 0} {$i < $n} {incr i} \
              {
               set zfKey [lindex $zfList $i]

               if {![strcasecmp $zfKey ZF] && $i < $n - 1} \
                  {
                   incr i
                   set zfArgs "-zf   [lindex $zfList $i]"
                   if {[lsearch -exact $zfList noauto] < 0} {append zfArgs " -auto"}
                  } \
               elseif {![strcasecmp $zfKey SIZE] && $i < $n - 1} \
                  {
                   incr i
                   set zfArgs "-size [lindex $zfList $i] && $i < $n - 1"
                   if {[lsearch -exact $zfList auto] >= 0} {append zfArgs " -auto"}
                  } \
               elseif {![strcasecmp $zfKey PAD] && $i < $n - 1} \
                  {
                   incr i
                   set zfArgs "-pad  [lindex $zfList $i]"
                   if {[lsearch -exact $zfList auto] >= 0} {append zfArgs " -auto"}
                  }
              }
          }

       if {![string length $zfArgs]} {set zfArgs "-zf 1 -auto"}
 
       puts $fp "#!/bin/csh -f" 
       puts $fp ""
       puts $fp "nmrPipe -in $fidName \\"
       puts $fp "| nmrPipe -fn $fnT \\"
       puts $fp "| nmrPipe -fn MULT -c $cScale \\"
       puts $fp "| nmrPipe -fn EM -lb $lb -c $fScale \\"
       puts $fp "| nmrPipe -fn ZF $zfArgs \\"
       puts $fp "| nmrPipe -fn FT \\"
       puts $fp "| nmrPipe -fn $fnF \\"

       if {$extFirstFlag} \
          {
           puts $fp "| nmrPipe -fn EXT -x1 $extX1 -xn $extXN -sw \\"
          }

       if {![strcasecmp $autoPhaseMode RMS]} \
          {
           set p0Name aph1DRMS.M
           set p1Name aphP1_1DRMS.M 
          } \
       else \
          {
           set p0Name aph1D.M
           set p1Name aphP1_1D.M 
          }

       if {[strcasecmp $autoPhaseMode None]} \
          {
           if {$extFirstFlag} {insertBaseCorrect $fp}

           if {[strcasecmp $p1X1 None] && [strcasecmp $p1XN None]} \
              {
               if {[strcasecmp $p0X1 None] && [strcasecmp $p0XN None]} \
                  {
                   puts $fp "| nmrPipe  -fn MAC -macro \$NMRTXT/$p0Name -all \\"
                   puts $fp "           -reg -str psX1 $p0X1 psX2 $p0XN -var xP1 0 \\"
                  }

               insertBaseCorrect $fp

               puts $fp "| nmrPipe  -fn MAC -macro \$NMRTXT/$p0Name -all \\"
               puts $fp "           -reg -str psX1 $p0X1 psX2 $p0XN -var xP1 0 \\"
               puts $fp "| nmrPipe  -fn MAC -macro \$NMRTXT/$p1Name -all -str \\"
               puts $fp "           p0X1 $p0X1 p0X2 $p0XN  p1X1 $p1X1 p1X2 $p1XN \\"

               insertBaseCorrect $fp

               if {[strcasecmp $p0X1 None] && [strcasecmp $p0XN None]} \
                  {
                   puts $fp "| nmrPipe  -fn MAC -macro \$NMRTXT/$p0Name -all \\"
                   puts $fp "           -reg -str psX1 $p0X1 psX2 $p0XN -var xP1 0 \\"
                  }

               insertBaseCorrect $fp

               puts $fp "| nmrPipe  -fn MAC -macro \$NMRTXT/$p1Name -all -str \\"
               puts $fp "           p0X1 $p0X1 p0X2 $p0XN  p1X1 $p1X1 p1X2 $p1XN \\"
              } \
           elseif {[strcasecmp $p0X1 None] && [strcasecmp $p0XN None]} \
              {
               puts $fp "| nmrPipe  -fn MAC -macro \$NMRTXT/$p0Name -all \\"
               puts $fp "           -reg -str psX1 $p0X1 psX2 $p0XN -var xP1 0 \\"

               insertBaseCorrect $fp

               puts $fp "| nmrPipe  -fn MAC -macro \$NMRTXT/$p0Name -all \\"
               puts $fp "           -reg -str psX1 $p0X1 psX2 $p0XN -var xP1 0 \\"
              }
          }

       if {![strcasecmp $xP0 MC] || ![strcasecmp $p0X1 MC]} \
          {
           puts $fp "| nmrPipe -fn MC \\"
          } \
       else \
          {
           puts $fp "| nmrPipe -fn PS -p0 $xP0 -p1 $xP1 -di \\"
          }

       if {!$extFirstFlag} {insertBaseCorrect $fp}

       puts $fp "| nmrPipe -fn $fnB \\"
       puts $fp "  -out $auxName -ov"
       puts $fp ""

       if {$refFlag} \
          {
           set thisTabName $refTabName

           if {$refLeftFlag == 1} \
              {
               set rfArg left
              } \
           elseif {$refLeftFlag == 0} \
              {
               set rfArg right
              } \
           else \
              {
               set rfArg max
#              set thisTabName None
              }

           puts $fp "ref1D.tcl -noskip -in $auxName -tab $thisTabName -ref $refVal -x1 $refX1 -xn $refXN -$rfArg"
           puts $fp ""
          }

       puts $fp "nmrPipe -in $auxName \\"

       if {!$extFirstFlag} \
          {
           puts $fp "| nmrPipe -fn EXT -x1 $extX1 -xn $extXN -sw \\"
           insertBaseCorrect $fp
          }

       puts $fp "  -out $ftName -ov"
       puts $fp ""

       if {$normFlag && [strcasecmp $normMode None] && [strcasecmp $normX1 None] && [strcasecmp $normXN None]} \
          {
           set vName vSum

           if {![strcasecmp $normMode RMS]} {set vName vRMS}
           if {![strcasecmp $normMode Max]} {set vName vMaxAbs}

           puts $fp "set sInfo = (`readROI -ndim 1 -in $ftName -out None -silent -x X_AXIS $normX1 $normXN -vec $vName` 1.0)"

           puts $fp ""
           puts $fp "nmrPipe -in $auxName \\"
           puts $fp "| nmrPipe -fn MULT -c $normVal \\"
           puts $fp "| nmrPipe -fn MULT -c \$sInfo\[1\] -inv \\"
           puts $fp "  -out $auxName -ov -inPlace"
           puts $fp ""

           puts $fp ""
           puts $fp "nmrPipe -in $ftName \\"
           puts $fp "| nmrPipe -fn MULT -c $normVal \\"
           puts $fp "| nmrPipe -fn MULT -c \$sInfo\[1\] -inv \\"
           puts $fp "  -out $ftName -ov -inPlace"
           puts $fp ""
          }
 
       close $fp

       logTCLCmnd "file delete $ftName"
       logTCLCmnd "file delete $auxName"
       logSysCmnd "$procName"

       if {![file exists $ftName]} \
          {
           showMsg "Error Creating NMRPipe Spectrum $ftName"
           return 1
          }

       showMsg "Created NMRPipe Spectrum $ftName"
      }

   return 0
}

proc insertBaseCorrect { fp } \
{
   global solX0 bcDX bcOrder

   if {[strcasecmp $bcOrder None]} \
      {
       if {![strcasecmp $solX0 None]} \
          {
           puts $fp "| nmrPipe -fn POLY -auto -window $bcDX -ord $bcOrder \\"
          } \
       else \
          {
           puts $fp "| nmrPipe -fn POLY -auto -window $bcDX -xn $solX0 -ord $bcOrder \\"
           puts $fp "| nmrPipe -fn POLY -auto -window $bcDX -x1 $solX0 -ord $bcOrder \\"
          }
      }
}

#
# Perform noise analysis:

proc proc1DAnal {} \
{
   global ftName auxName noiseX1 noiseXN noise thresh mult minVal maxVal
   global roiValid

   if {[strcasecmp $ftName None]} \
      {
       if {![file exists $ftName]} \
          {
           showMsg "Error Finding NMRPipe Spectrum $ftName"
           return 1
          }

       showMsg "Reading Spectral Data: $ftName"
   
       readROI -ndim 1 -in $ftName  -x X_AXIS 0% 100% -roi 1

       set minVal  [getROIStat 1 vMin]
       set maxVal  [getROIStat 1 vMax]

       set roiValid(1) 1
      }

   if {[strcasecmp $ftName None]} \
      {
       if {![file exists $ftName]} \
          {
           showMsg "Error Finding NMRPipe Spectrum $ftName"
           return 1
          }

       if {![strcasecmp $auxName None]} {set auxName $ftName}

       if {[file exists $auxName]} \
          {
           readROI -ndim 1 -in $auxName -x X_AXIS $noiseX1 $noiseXN -roi 2

           set dev [getROIStat 2 vStdDev]

           if {![strcasecmp $noise  Auto]} {set noise [format %.3e $dev]}

           set noise [getPctVal $noise $minVal $maxVal]

           if {![strcasecmp $thresh Auto]} \
              {
               set thresh [format %.3e [expr $noise*$mult]]
              }

           set thresh [getPctVal $thresh $minVal $maxVal]

           showMsg \
            "Noise Region From $auxName: Noise: $noise Threshold: $thresh"

           set roiValid(2) 1
          } \
       else \
          {
           set range [expr $maxVal - $minVal]
   
           if {![strcasecmp $noise  Auto]} {set noise  [expr $range/100]}
           set noise [getPctVal $noise $minVal $maxVal]

           if {![strcasecmp $thresh Auto]} {set thresh [expr $noise*$mult]}
           set thresh [getPctVal $thresh $minVal $maxVal]
          }

       set error [rdFDATA $ftName fdata]

       if {!$error} \
          {
           setParm $fdata FDNOISE   $noise   NULL_DIM
           setParm $fdata FDPTHRESH $thresh  NULL_DIM
           setParm $fdata FDNTHRESH [expr -1*$thresh] NULL_DIM

           wrFDATA $ftName $fdata

           sysFree $fdata
          }
      }

   return 0
}

#
# Perform peak analysis:

proc proc1DPk {} \
{
   global tabName ftName eList pkTab pkCount zoomFlag 
   global thresh noise pkDX roiValid

   if {[strcasecmp $tabName None] && [strcasecmp $ftName None]} \
      {
       logTCLCmnd "file delete $tabName"
 
       pkFindROI -roi 1 \
         -sigma $noise -pChi 1e-04 -plus $thresh -minus -$thresh \
         -dx       $pkDX \
         -idx      $pkDX \
         -tol      10.00 \
         -hiAdj     1.20 \
         -lw       15.00 \
         -reject -mask -out $tabName -verb

       if {![file exists $tabName]} \
          {
           showMsg "Error Creating NMRPipe Peak Table $tabName"
           return 1
          }

       set pkTab   [gdbRead table -in $tabName]
       set eList   [gdbGetInfo $pkTab -parms entries]
       set pkCount [llength $eList]

       if {$zoomFlag && $pkCount > 0} \
          {
           set ppmList [getTabVals $pkTab X_PPM]
           set dx1     [getMax $ppmList] 
           set dxn     [getMin $ppmList] 

           set dx1 [expr  0.2 + int(10.0*$dx1 + 0.5)/10.0]ppm
           set dxn [expr -0.2 + int(10.0*$dxn - 0.5)/10.0]ppm
          } \
       else \
          {
           set dx1 0%
           set dxn 100%
          }
  
       showMsg "Plot Spectral Window Limits: $dx1 to $dxn"

       readROI -ndim 1 -in $ftName -x X_AXIS $dx1 $dxn -roi 1

       set roiValid(1) 1
      }

   return 0
}

proc proc1DPlot {} \
{
   global pkTab pkScale pkCount eList psName ftName minVal maxVal noise thresh
   global plotMin plotMax cx1 cxn cy1 cyn pkLabFlag
   
#
# Interpret plotMin and plotMax:

   if {$pkScale > 0 && $pkCount > $pkScale + 2} \
      {
       set hiList  [getTabVals $pkTab HEIGHT]
       set sList   [lsort -command absSortDesc $hiList]
       set hiList  [lrange $sList $pkScale end]

       set thisMin [getMin $hiList]
       set thisMax [getMax $hiList]

       if {$thisMax > 0.0 && $thisMin > 0.0} {set thisMin 0.0}
       if {$thisMax < 0.0 && $thisMin < 0.0} {set thisMax 0.0}
      } \
   else \
      {
       set thisMin $minVal
       set thisMax $maxVal
      }

   set range [expr abs($thisMax - $thisMin)]

   if {![strcasecmp $plotMin Auto]} \
      {
       set plotMin [expr $thisMin - 0.05*$range]
      } \
   else \
      {
       if {[string first % $plotMin] >= 0} \
          {
           set plotMin [string trim $plotMin %]
           set plotMin [expr $plotMin*$range/100.0]
           set range   [expr abs($thisMax - $plotMin)]
          } 
      }

   if {![strcasecmp $plotMax Auto]} \
      {
       set plotMax [expr $thisMax + 0.30*$range]
      } \
   else \
      {
       if {[string first % $plotMax] >= 0} \
          {
           set plotMax [string trim $plotMax %]
           set plotMax [expr $plotMax*$range/100.0]
          } 
      }

    set fmt "Plot Height Limits: %.3e to %.3e"

    showMsg [format $fmt $plotMin $plotMax]

#
# Open the plot file:

   if {[strcasecmp $psName None]} \
      {
       if {![file exists $ftName]} \
          {
           showMsg "Error Finding NMRPipe Spectrum $ftName"
           return 1
          }

       logTCLCmnd "file delete $psName"
       showMsg "Creating PostScript Output: $psName"

       plotOpen -plotName $psName -landscape -color

       defineCell -cell 1 -roiRef 1 -x $cx1 $cxn -y $cy1 $cyn
  
       if {$pkLabFlag} \
          { 
           drawRec -cell 1 -x 0% 100% -y 0% 100% -fg lightGray
          }
      }

#
# Plot the peak labels:

   if {$pkLabFlag && ($pkCount > 0) && [strcasecmp $psName None]} \
      {
       set range [expr $plotMax - $plotMin]
       set y0    [expr 100.0*($plotMax - 0.0)/$range]
       set y1    [expr 100.0*($plotMax - $thresh)/$range]
       set yn    [expr 100.0*($plotMax + $thresh)/$range]

       drawRec  -cell 1 -x 0% 100% -y ${y1}% ${yn}% -fg pink
       drawLine -cell 1 -x 0% 100% -y ${y0}% ${y0}% -fg white

       set i 0

       foreach entry $eList \
          {
           incr i

           set txA    [expr 100.0*($i - 0.6)/$pkCount]
           set txB    [expr 100.0*($i - 0.4)/$pkCount]

           set xPPM   [gdbGet $entry X_PPM]ppm
           set hi     [gdbGet $entry HEIGHT]
           set id     [gdbGet $entry INDEX]

           if {$hi > $maxVal} {set hi $maxVal}
           if {$hi < $minVal} {set hi $minVal}

           if {$hi > 0.0} \
              {
               set y1 [expr -1.5 + 100.0*($plotMax - $hi)/$range]
              } \
           else \
              {
               set y1 [expr  1.5 + 100.0*($plotMax - $hi)/$range]
              }

           set tyA   0
           set tyB  -5

           drawLine -cell 1 -roiRef 1 -x $xPPM $xPPM -y 98% 100% -fg blue
   
           if {$y1 > 15} \
              {
               drawLine -cell 1 -roiRef 1 \
                        -x $xPPM $xPPM -y ${y1}%  15% -fg white
              }

           drawLine -cell 1 -roiRef 1 -x $xPPM ${txA}% -y 15% ${tyA}% -fg white

           drawText -cell 1 -roiRef 1 -center -rotate 90 \
                    -x ${txB}% -y ${tyB}% -bg None \
                    -fg black -nobox -text $xPPM
          }
      }

#
# Plot the spectrum:


   if {[strcasecmp $psName None] && [strcasecmp $ftName None]} \
      {
       if {![file exists $ftName]} \
          {
           showMsg "Error Finding NMRPipe Spectrum $ftName"
           return 1
          }

       if {$pkCount < 1} \
          {
           set y0 [expr 100.0*($plotMax - 0.0)/$range]

           drawLine -cell 1 -x 0% 100% -y ${y0}% ${y0}% -fg white    
          }

       drawROI -cell 1 -roi 1 -fg black -axis Bottom -top \
               -fhi $plotMin -lhi $plotMax -nobox

       drawLine -cell 1 -x 0% 100% -y 100% 100% -fg black
      }

#
# Close the plot file:

   if {[strcasecmp $psName None]} \
      {
       drawBox -cell 1 -x 0% 100% -y 0% 100% -fg black

       set    msg "NMRPipe [sysTime %Y.%j.%H.%M]"
       append msg " Dir: [pwd] User: [exec whoami]" 

       set    fmt  " Min: %.3e Max: %.3e Noise: %.3e Thresh: %.3e" 
       append msg [format $fmt $plotMin $plotMax $noise $thresh]

       if {$pkCount > 0} {append msg " Peaks Listed: $pkCount"}

       if {$pkLabFlag} {drawText -x $cx1 -y 98% -fg black -text $msg}

       plotClose
      }

   return 0
}

proc proc1DView {} \
{
   global plotMin plotMax roiValid wishFlag

   if {!$wishFlag} {return 0}

   wm title . [pwd] 

   drawRec -x 0% 100% -y 0% 100% -fg gray20 

   if {$roiValid(1)} \
      {
       drawRec -cell 2 -fg black -x 0% 100% -y 0% 100%

       drawLine -cell 2 -roiRef 1 -x 0.0ppm 0.0ppm -y 0% 100% -fg red

       drawROI -cell 2 -roi 1 -bg None -top -fhi $plotMin -lhi $plotMax \
               -fg white -color1 yellow

       set rInfo [getROIInfo -roi 1]
       set fdata [lindex $rInfo 1]

       set tStr [getParmStr $fdata FDTITLE NULL_DIM]

       drawText -cell 3 -x 110% -y 30% -fg green -text $tStr
      }

   if {$roiValid(2)} \
      {
       drawROI -cell 3 -roi 2 -bg black -top -fg white -color1 yellow 
      }

   update

   return 0
}

proc execProc1D {} \
{
   global saveDir dirName inList inName cancelFlag wishFlag specID

   set cancelFlag 0
   set error      0

   set w        .ezProc1D
   set origName $inName
   set origID   $specID

   if {![strcasecmp $specID Auto]} \
      {
       set nList ""

       foreach i {ft tab ps} \
          {
           lappend nList [llength [glob -nocomplain $saveDir/$i/*]]

           set specID [getMax $nList]
           set specID [expr 1 + int($specID)]

           if {$wishFlag} update 
          }
      }

   set n [llength $inList]

   if {$n} \
      {
       proc1DCancelON

       for {set i 1} {$i <= $n} {incr i} \
          {
           set thisName [lindex $inList [expr $i - 1]]
 
           if {![file exists $thisName]} continue
           if {![strcasecmp $origName Auto]} {set inName Auto}

           set thisName [file normalize $thisName]

           if {[file isdirectory $thisName]} \
              {
               set dirName $thisName
              } \
           else \
              {
               set dirName [getDirName $thisName]
               set inName  [file tail  $thisName]
              } 

           if {$wishFlag} \
              {
               wm title $w "Processing Spectrum $i of $n."
               update
              }

           if {[proc1DCurrent]} {incr error}

           if {$cancelFlag} \
              {
               set ans \
                [tk_dialog .canDialog Cancel "Cancel Processing?" \
                           question 1 Cancel Continue]

               if {!$ans} break               
              }

           set cancelFlag 0
           incr specID
          }

       proc1DCancelOFF
      } \
   else \
      {
       if {[proc1DCurrent]} {incr error}
       incr specID
      }

   if {![strcasecmp $origID Auto]} {set specID $origID}

   set inName $origName

   if {$wishFlag} \
      {
       wm title $w "ezProc1D Done."
       update
      }
 
   return $error
}

proc saveProc1DVars {} \
{
   global proc1DSInfo

   set proc1DSInfo(vList) {inName inType plotMin plotMax noise thresh} 

   foreach v $proc1DSInfo(vList) \
      {
       global $v
       set proc1DSInfo($v) [subst \$$v]
      }
}

proc restoreProc1DVars {} \
{
   global proc1DSInfo

   foreach v $proc1DSInfo(vList) \
      {
       global $v
       set $v $proc1DSInfo($v)
      }

}

proc proc1DSave {} \
{
   global ftName psName tabName logName
   global saveDir saveName

   if {![strcasecmp $saveDir  None]} {return 0}
   if {![strcasecmp $saveName None]} {return 0}

   set thisDir  [file normalize $saveDir]
   set thisName [getProc1DSaveName]

   showMsg "Using Save Directory: $thisDir"
   showMsg "Using Save Template:  $thisName"

   if {![file exists $thisDir]} {catch "mkdir $thisDir"}

   if {[strcasecmp $ftName None]} \
      {
       if {![file exists $thisDir/ft]} {logTCLCmnd "file mkdir $thisDir/ft"} 
       logTCLCmnd "exec /bin/cp $ftName $thisDir/ft/$thisName.ft1"
      } 
  
   if {[strcasecmp $tabName None]} \
      {
       if {![file exists $thisDir/tab]} {logTCLCmnd "file mkdir $thisDir/tab"}
       logTCLCmnd "exec /bin/cp $tabName $thisDir/tab/$thisName.tab"
      }

   if {[strcasecmp $psName None]} \
      {
       if {![file exists $thisDir/ps]} {logTCLCmnd "file mkdir $thisDir/ps"}
       logTCLCmnd "exec /bin/cp $psName $thisDir/ps/$thisName.ps"
      } 

   return 0
}

proc getProc1DSaveName {} \
{
   global saveName specID dirName

   set thisName $saveName

   set key "%index"
   set loc [string first $key $thisName]
   set n   [string length $key]
   set h   ""
 
   if {$loc >= 0} \
      {
       if {$loc > 0} {set h [string range $thisName 0 [expr $loc - 1]]}

       set s [format %04d $specID]
       set t [string range $thisName [expr $loc + $n] end]

       set thisName $h$s$t
      }

   set key "%srcDir"
   set loc [string first $key $thisName]
   set n   [string length $key]
   set h   ""

   if {$loc >= 0} \
      {
       if {$loc > 0} {set h [string range $thisName 0 [expr $loc - 1]]}

       set dName [removePct [file normalize $dirName]]

       set dList [removeNullElements [split $dName "/"]]
       set nn    [llength $dList]

       set str [lindex $dList end]
       set s $str

       if {[isInt $s]} \
          {
           if {$s >= 0 && $nn > 1} \
              {
               set i [format %03d $s]
               set s [lindex $dList [expr $n - 2]]_$i
              }
          } \
       else \
          {
           set s [file root $str]
          }

       set t [string range $thisName [expr $loc + $n] end]
       set thisName $h$s$t
      }

   return $thisName
}

proc proc1DCurrent {} \
{
   global dirName wishFlag

   set pList { 
              proc1DLogStart \
              proc1DConv proc1DProc proc1DAnal proc1DPk 
              proc1DPlot proc1DView proc1DSave \
              proc1DLogConclude
             }

   saveProc1DVars

   set origDir [pwd]
   set error   0     

   proc1DCD $dirName

   foreach p $pList \
      {
       set error [$p]

       if {$error} \
          {
           showThisError ezProc1D "Error Processing $dirName ($p)"
           proc1DCD $origDir
           restoreProc1DVars
           return $error
          }
      }

   proc1DCD $origDir
   restoreProc1DVars

   if {$wishFlag} update

   return $error
}

proc noSingleQuote { line } \
{
   set n   [string length $line]
   set str ""

   for {set i 0} {$i < $n} {incr i} \
      {
       set c [string index $line $i]

       if {[strcmp $c ']} {append str $c} 
      }

   return $str
}

proc proc1DExit {} \
{
   exit 0
}

initProc1DVars
initProc1DArgs
readProc1DParms
initProc1DWin
initProc1DViewer

if {$autoFlag} \
   {
    execProc1D
    proc1DExit
   }

