#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

proc fileTreeInit args  \
{
   global fileTreeInfo env bigfont

   set ARGV [concat fileTreeInfo $args]

   if {![info exists fileTreeInfo(count)]} \
      {
       set fileTreeInfo(count) 0
       image create bitmap fileTreeDocImage -file $env(NMRTXT)/shapeDoc.xbm
       if {![info exists bigfont]} {setFonts}
      }

   incr fileTreeInfo(count) 
   set  id $fileTreeInfo(count) 

   set w     .fileTreeView$id
   set wName $w

   toplevel $w

   set fileTreeInfo($id,window)       $w
   set fileTreeInfo($id,title)        $w
   set fileTreeInfo($id,treeWin)      $w.tree
   set fileTreeInfo($id,origName)     ""
   set fileTreeInfo($id,fileName)     ""
   set fileTreeInfo($id,selectProc)   "" 
   set fileTreeInfo($id,textvariable) ""
   set fileTreeInfo($id,mode)         open
   set fileTreeInfo($id,maxLevels)    3
   set fileTreeInfo($id,baseDir)      [pwd] 
   set fileTreeInfo($id,tree)         [tree create]

   getArgD $ARGV -dir          fileTreeInfo($id,baseDir)
   getArgD $ARGV -file         fileTreeInfo($id,fileName)
   getArgD $ARGV -textvariable fileTreeInfo($id,textvariable)
   getArgD $ARGV -command      fileTreeInfo($id,selectProc)
   getArgD $ARGV -mode         fileTreeInfo($id,mode)
   getArgD $ARGV -title        fileTreeInfo($id,title)
   getArgD $ARGV -maxLevels    fileTreeInfo($id,maxLevels)

   set fileTreeInfo($id,origName)    $fileTreeInfo($id,fileName)
   set fileTreeInfo($id,origDir)     $fileTreeInfo($id,baseDir)
   set fileTreeInfo($id,baseDirUser) $fileTreeInfo($id,baseDir)

   frame $w.path
   frame $w.top 
   frame $w.bot
   frame $w.hdr
   frame $w.base
   frame $w.maxd
   frame $w.file
   frame $w.cmnd

   treeview $w.tree \
       -width 0 -selectmode single    \
       -yscrollcommand "$w.vs set"    \
       -xscrollcommand "$w.hs set"    \
       -selectcommand "fileTreeItemSelect $id %p" \
       -opencommand   "fileTreeItemOpen   $id %p" \
       -closecommand  "fileTreeItemClose  $id %p" \
       -tree $fileTreeInfo($id,tree) -width 500 -height 360

   scrollbar $w.vs -orient vertical   -command "$w.tree yview"
   scrollbar $w.hs -orient horizontal -command "$w.tree xview"

   entry $w.baseDir -textvariable fileTreeInfo($id,baseDirUser) -font $bigfont

   label $w.filelab  -text "File:" -font $bigfont
   entry $w.fileName -textvariable fileTreeInfo($id,fileName) -font $bigfont

   label   $w.maxdlab -text "Depth:" -font $bigfont
   label   $w.maxdspa -text " "      -font $bigfont -width 1

   spinbox $w.maxdent -textvariable fileTreeInfo($id,maxLevels) -width 2 \
                      -font $bigfont -increment 1 -from 1 -to 100        \
                      -format %.0f -command "fileTreeUpdateDepth $id" 
                      
   button $w.select -text "Select File" -command "fileTreeSelectFile $id"
   button $w.quit   -text "Cancel"      -command "fileTreeCancel     $id"

   pack $w.tree -in $w.top -side left -expand 1 -fill both
   pack $w.vs   -in $w.top -side left -expand 1 -fill y
   pack $w.hs   -in $w.bot -side top  -expand 1 -fill x

   pack $w.select $w.quit -in $w.cmnd -side left -expand 1 -fill x

   pack $w.maxdlab $w.maxdent $w.maxdspa -in $w.maxd -side left
   pack $w.base -in $w.hdr  -side left
   pack $w.maxd -in $w.hdr  -side right

   pack $w.hdr     -in $wName -side top -expand 1 -fill x
   pack $w.baseDir -in $wName -side top -expand 1 -fill x
   pack $w.top     -in $wName -side top -expand 1 -fill both 
   pack $w.bot     -in $wName -side top -expand 1 -fill x

   pack $w.filelab  -in $w.file -side left
   pack $w.fileName -in $w.file -side left -expand 1 -fill x

   pack $w.file -in $wName -side top -expand 1 -fill x
   pack $w.cmnd -in $wName -side top -expand 1 -fill x

   update

   fileTreeSetBaseDir $id $fileTreeInfo($id,baseDir) 

   bind $w.baseDir <Return> "+ fileTreeUpdateBaseDirUser $id"
}

proc fileTreeTitle { id } \
{
   global fileTreeInfo

   wm title $fileTreeInfo($id,window) $fileTreeInfo($id,title)
   update
}

proc fileTreeUpdateBaseDirUser { id } \
{
   global fileTreeInfo
   if {![info exists fileTreeInfo($id,tree)]} return

   if {![file isdirectory $fileTreeInfo($id,baseDirUser)]} return

   catch "$fileTreeInfo($id,tree) delete root"

   set fileTreeInfo($id,baseDir) $fileTreeInfo($id,baseDirUser)
   set thisDir $fileTreeInfo($id,baseDir)

   fileTreeNextNodes $id root $thisDir 0
   fileTreeTitle $id
}

proc fileTreeUpdateDepth { id } \
{
   global fileTreeInfo

   if {![info exists fileTreeInfo($id,tree)]} return

   catch "$fileTreeInfo($id,tree) delete root"

   set thisDir $fileTreeInfo($id,baseDir)

   fileTreeNextNodes $id root $thisDir 0
   fileTreeTitle $id
}

proc fileTreeItemOpen { id node } \
{
   global fileTreeInfo

    set thisDir $fileTreeInfo($id,baseDir)/[join $node "/"]
    if {![file exists $thisDir]} return
}

proc fileTreeItemClose args \
{
   global fileTreeInfo
}

proc fileTreeItemSelect args \
{
   global fileTreeInfo

   set id [lindex $args 0]

   if {![info  exists fileTreeInfo($id,treeWin)]}  return
   if {![winfo exists $fileTreeInfo($id,treeWin)]} return

   set s [$fileTreeInfo($id,treeWin) curselection]
   if {![llength $s]} return

   set pathList [$fileTreeInfo($id,treeWin) get -full $s]

   if {![strcasecmp $fileTreeInfo($id,mode) open]} \
      {
       if {![strcmp [lindex $pathList end] "."]} return

       set newName $fileTreeInfo($id,baseDir)/[join $pathList "/"]

       if {![file isdirectory $newName]} \
          {
           set fileTreeInfo($id,fileName) $newName
          }
      } \
   elseif {![strcasecmp $fileTreeInfo($id,mode) opendir]} \
      {
       if {![strcmp [lindex $pathList end] "."]} \
          {
           set pathList [lrange $pathList 0 end-1]
           set newName $fileTreeInfo($id,baseDir)/[join $pathList "/"]
          } 

       if {[file isdirectory $newName]} \
          {
           set fileTreeInfo($id,fileName) $newName
          } 
      }
}

proc fileTreeSelectFile { id } \
{
   global fileTreeInfo

   global fileTreeInfo

   catch "destroy $fileTreeInfo($id,window)"
   catch "tree destroy $fileTreeInfo($id,tree)"
   update

   set var $fileTreeInfo($id,textvariable)
   if {![llength $var] || ![strcasecmp $var None]} return

   global $var

   set $var $treeViewInfo($id,origName) $treeViewInfo($id,fileName)
}

proc fileTreeCancel { id } \
{
   global fileTreeInfo

   catch "destroy $fileTreeInfo($id,window)"
   catch "tree destroy $fileTreeInfo($id,tree)"
   update

   set var $fileTreeInfo($id,textvariable)
   if {![llength $var] || ![strcasecmp $var None]} return

   global $var

   set $var $treeViewInfo($id,origName)
}

proc fileTreeSetBaseDir { id dir } \
{
   global fileTreeInfo bigfont

   set w $fileTreeInfo($id,window)

   set fileTreeInfo($id,baseDir) $dir

   set baseList [split [file normalize $dir] "/"]
   set i 0

   for {set i 0} {$i < 1024} {incr i} \
      {
       if {![winfo exists $w.path$i]} break
       catch "destroy $w.path$i"
      }

   set d ""

   foreach f "$baseList" \
      {
       if {![llength $f]} \
          {
           append d /
           button $w.path$i -text "$f /" -font $bigfont -command "updateBaseDir $id $d"
          } \
       else \
          {
           append d $f/
           button $w.path$i -text "$f/" -font $bigfont -command "updateBaseDir $id $d"
          }

       pack   $w.path$i -side left -in $w.base
       incr i
      }

    update

    fileTreeNextNodes $id root $dir 0
    fileTreeTitle $id

    update
}

proc updateBaseDir { id dir } \
{
   global fileTreeInfo

   if {![info exists fileTreeInfo($id,tree)]} return

   catch "$fileTreeInfo($id,tree) delete root"

   set fileTreeInfo($id,baseDir) $dir

   fileTreeNextNodes $id root $dir 0
   fileTreeTitle $id
}

proc fileTreeNextNodes { id parent dir level } \
{
   global fileTreeInfo

   if {![info  exists fileTreeInfo($id,treeWin)]}  return
   if {![winfo exists $fileTreeInfo($id,treeWin)]} return

   if {![file isdirectory $dir]} \
      {
       return
      }

   set type [file type $dir]

   if {![strcasecmp $type link]} \
      {
       return
      }

   wm title $fileTreeInfo($id,window) $dir
   update

   incr level

   foreach f [lsort -dictionary [glob -nocomplain $dir/*]] \
      {
       set type [file type $f]

       if {![strcasecmp $type link]} \
          {
           continue 
          }

       if {[file isdir $f]} \
          {
           wm title $fileTreeInfo($id,window) $f
           update

           set node [$fileTreeInfo($id,tree) insert $parent -label [file tail $f]]
           if {$level >= $fileTreeInfo($id,maxLevels)} \
              {
               set node [$fileTreeInfo($id,tree) insert $node -label "."]
              } \
           else \
              {
               fileTreeNextNodes $id $node $f $level
              }

          } \
       else \
          {
           set node [$fileTreeInfo($id,tree) insert $parent -label [file tail $f]]

           $fileTreeInfo($id,treeWin) entry configure $node \
              -icons      "fileTreeDocImage fileTreeDocImage"   \
              -activeicon "fileTreeDocImage fileTreeDocImage"
          }
      }
}

proc doFileTreeNextNodes { id node dir } \
{
   puts "next node do $id, $node, $dir"
}

fileTreeInit
