#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Find Scale Factor f to Minimize Spectrum1 - f*Spectrum2:"
    puts " -in1     inName1  \[A.dat\]  Input Spectrum 1."
    puts " -in2     inName2  \[B.dat\]  Input Spectrum 2." 
    puts " -out     outName  \[None\]   Optional Scaled Output, or Keyword None."
    puts " -rms                       Report RMS with Scale Factor."
    puts "Region of Interest (readROI Options) Should be Last on Command Line:"
    puts " -regInfo roiInfo  \[-ndim 2 -x X_AXIS 0% 100% ...]"
    exit 0
   }

set inName1 A.dat 
set inName2 B.dat 
set outName None
set rmsFlag 0

getArgD $ARGV -in1 inName1
getArgD $ARGV -in2 inName2
getArgD $ARGV -out outName

if {![file exists $inName1]} \
   {
    puts stderr "FindScale Error Finding Input 1 $inName1"
    exit 1
   }

if {![file exists $inName2]} \
   {
    puts stderr "FindScale Error Finding Input 2 $inName2"
    exit 1
   }

set sInfo    [getSpecSizeInfo $inName1]
set dimCount [lindex $sInfo 1]

if {$dimCount == 1} \
   {
    set regInfo "-ndim 1 -x X_AXIS 0% 100%"
   } \
elseif {$dimCount == 2} \
   {
    set regInfo "-ndim 2 -x X_AXIS 0% 100% -y Y_AXIS 0% 100%"
   } \
elseif {$dimCount == 3} \
   {
    set regInfo "-ndim 3 -x X_AXIS 0% 100% -y Y_AXIS 0% 100% -z Z_AXIS 0% 100%"
   } \
else \
   {
    puts stderr "FindScale Error Finding Dimension Count."
    exit 1
   }

set loc [flagLoc $ARGV -regInfo]

if {$loc} {set regInfo [lrange $ARGV [expr $loc + 1] end]}

if {[flagLoc $ARGV -rms]}   {set rmsFlag 1}
if {[flagLoc $ARGV -norms]} {set rmsFlag 0}

set error [catch {eval "readROI -roi 1 -in $inName1 $regInfo"}]

if {$error} \
   {
    puts stderr "FindScale Error Executing: readROI -roi 1 -in $inName1 $regInfo"
    exit 1
   }

set error [catch {eval "readROI -roi 2 -in $inName2 $regInfo"}]

if {$error} \
   {
    puts stderr "FindScale Error Executing: readROI -roi 2 -in $inName2 $regInfo"
    exit 1
   }

set rdata1 [lindex [getROIInfo -roi 1] 0]
set size1  [getROISize 1]

set rdata2 [lindex [getROIInfo -roi 2] 0]
set size2  [getROISize 2]

if {$size1 != $size2} \
   {
    puts stderr "FindScale Error: Region Size Mismatch ($size1 $size2)."
    puts stderr " readROI -roi 1 -in $inName1 $regInfo"
    puts stderr " readROI -roi 2 -in $inName2 $regInfo"
   }

set s [vvGetScale $rdata1 $rdata2 $size1]

if {$rmsFlag} \
   {
    vsMult $rdata2 $s $size1
    vvSub  $rdata1 $rdata2 $size1

    set r [vRMS $rdata1 $size1]

    puts -nonewline "scale $s rms $r"
   } \
else \
   {
    puts -nonewline $s
   }

if {[strcasecmp $outName None]} \
   {
    if {$dimCount == 3} \
       {
        catch "exec xyz2pipe -in $inName2 | nmrPipe -fn MULT -c $s | pipe2xyz -out $outName -ov"
       } \
    else \
       {
        catch "exec nmrPipe -in $inName2 -fn MULT -c $s -out $outName -ov"
       }
   }

exit 0
