#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Fix Header of RNMRTK MaxEnt Output:"
   echo " -in     inName [ft/test%03d.ft3]   Spectrum to Adjust."
   echo " -parent pName  [fid/test%03d.fid]  Parent Data of Spectrum."
   echo " -nocopy                            Update Key Values in Header (Default)."
   echo " -copy                              Copy Header from Parent, and Update."
   exit 0
endif

set parentName  = (`getArgD $argv -parent fid/test%03d.fid`)
set inName      = (`getArgD $argv -in     ft/test%03d.ft3`)
set copyFlag    = 0


if (`flagLoc $argv -copy`) then
   set copyFlag = 1
endif

if (`flagLoc $argv -nocopy`) then
   set copyFlag = 0
endif

set thisParent  = (`nmrPrintf $parentName 1 1`)
set thisName    = (`nmrPrintf $inName     1 1`)

if (!(-e $thisParent)) then
   echo "fixRNMRTK Error Finding Parent Data $parentName"
   exit 1
endif

if (!(-e $thisName)) then
   echo "fixRNMRTK Error Finding Input $inName"
   exit 1
endif

@ fmtCount   = (`echo $inName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

set xTSize = (`getParm -in $parentName -parm NDAPOD   -dim CUR_XDIM -fmt %.0f`)
set xName  = (`getParm -in $parentName -parm NDLABEL -text -fmt %s -dim CUR_XDIM`)
set yName  = (`getParm -in $parentName -parm NDLABEL -text -fmt %s -dim CUR_YDIM`)
set zName  = (`getParm -in $parentName -parm NDLABEL -text -fmt %s -dim CUR_ZDIM`)
set aName  = (`getParm -in $parentName -parm NDLABEL -text -fmt %s -dim CUR_ADIM`)

set dimCount = (`getParm -in $thisName -parm FDDIMCOUNT -dim NULL_DIM -fmt %.0f`)
set xFSize   = (`getParm -in $thisName -parm NDSIZE     -dim CUR_XDIM -fmt %.0f`)
set yFSize   = (`getParm -in $thisName -parm NDSIZE     -dim CUR_YDIM -fmt %.0f`)
set zFSize   = (`getParm -in $thisName -parm NDSIZE     -dim CUR_ZDIM -fmt %.0f`)
set aFSize   = (`getParm -in $thisName -parm NDSIZE     -dim CUR_ADIM -fmt %.0f`)

set yTSize = (`IMATH "$yFSize"`)
set zTSize = (`IMATH "$zFSize"`)
set aTSize = (`IMATH "$aFSize"`)

set inList   = ""
set pipeFlag = 0
set cubeFlag = 0 

if ($fmtCount == 0) then
   set inList = $inName

   if ($dimCount == 3 || $dimCount == 4) then
      set pipeFlag = 1
   endif
else if ($fmtCount == 1) then
   @ i = 0

   while ($i <= 100000)
      @ i++

      set testName = (`nmrPrintf $inName $i`)

      if (!(-e $testName)) then
         break
      endif

      set inList = ($inList $testName)
   end

   if ($dimCount == 4) then
      set cubeFlag = 1
   endif
else if ($fmtCount == 2) then
  @ i = 0

   while ($i <= 100000)
      @ i++
      @ j = 0
      @ n = 0

      while ($j <= 100000)
         @ j++

         set testName = (`nmrPrintf $inName $i $j`)

         if (!(-e $testName)) then
            break
         endif

         set inList = ($inList $testName)
         @ n++
      end

      if ($n == 0) then
         break
      endif
   end
endif

@ fileCount = $#inList

foreach thisName ($inList)
   if ($copyFlag) then
      copyHdr.tcl -in $thisParent -out $thisName
   endif

   if ($dimCount == 1) then
     sethdr $thisName -fileCount $fileCount \
            -xLAB $xName    \
            -xT   $xTSize   \
            -xTD  $xTSize   \
            -xFD  $xFSize
   else if ($dimCount == 2) then
      sethdr $thisName -fileCount $fileCount \
            -xLAB $xName  -yLAB $yName   \
            -xT   $xTSize -yT   $yTSize  \
            -xTD  $xTSize -yTD  $yTSize  \
            -xFD  $xFSize -yFD  $yFSize 
   else if ($dimCount == 3) then
      sethdr $thisName -fileCount $fileCount -pipeFlag $pipeFlag \
            -xLAB $xName  -yLAB $yName   -zLAB $zName  \
            -xT   $xTSize -yT   $yTSize  -zT   $zTSize \
            -xTD  $xTSize -yTD  $yTSize  -zTD  $zTSize \
            -xFD  $xFSize -yFD  $yFSize  -zFD  $zFSize
   else if ($dimCount == 4) then
      sethdr $thisName -fileCount $fileCount -pipeFlag $pipeFlag -cubeFlag $cubeFlag \
            -xLAB $xName  -yLAB $yName   -zLAB $zName  -aLAB $aName  \
            -xT   $xTSize -yT   $yTSize  -zT   $zTSize -aT   $aTSize \
            -xTD  $xTSize -yTD  $yTSize  -zTD  $zTSize -aTD  $aTSize \
            -xFD  $xFSize -yFD  $yFSize  -zFD  $zFSize -aFD  $aFSize
   endif
end

