#
# Procedure to build a font choice window:

proc getFontChoice args \
{
   global glbFontChoiceInfo

   set thisArgv [concat getFontChoice $args]

   if {![info exists glbFontChoiceInfo(id)]} \
      {
       set glbFontChoiceInfo(id) 0
      }

   incr glbFontChoiceInfo(id)

   set id  $glbFontChoiceInfo(id)
   set obj fontChoice$id

   set w  .glbFontChoice$id

   toplevel $w

   wm title $w Fonts

   glbInitFontChoiceInfo

   set glbFontChoiceInfo($obj,window)     $w
   set glbFontChoiceInfo($obj,fgColor)    navyblue
   set glbFontChoiceInfo($obj,bgColor)    None
   set glbFontChoiceInfo($obj,origFont)   $glbFontChoiceInfo(stdfont)
   set glbFontChoiceInfo($obj,font)       $glbFontChoiceInfo(stdfont)
   set glbFontChoiceInfo($obj,id)         $id
   set glbFontChoiceInfo($obj,cmnd)       glbFontChoiceProc
   set glbFontChoiceInfo($obj,title)      "Font Selection"
   set glbFontChoiceInfo($obj,xlsFlag)    0
   set glbFontChoiceInfo($obj,applyCount) 0
   
   getArgD $thisArgv -font    glbFontChoiceInfo($obj,font)
   getArgD $thisArgv -bg      glbFontChoiceInfo($obj,bgColor)
   getArgD $thisArgv -fg      glbFontChoiceInfo($obj,fgColor)
   getArgD $thisArgv -title   glbFontChoiceInfo($obj,title)
   getArgD $thisArgv -command glbFontChoiceInfo($obj,cmnd)

   if {[flagLoc $thisArgv -xls]}   {set glbFontChoiceInfo($obj,xlsFlag) 1}
   if {[flagLoc $thisArgv -noxls]} {set glbFontChoiceInfo($obj,xlsFlag) 0}

   glbInitFontChoiceXLS $obj

   set glbFontChoiceInfo($obj,origFont) $glbFontChoiceInfo($obj,font)

   set f $glbFontChoiceInfo($obj,font)

   foreach style $glbFontChoiceInfo(styleList) \
      {
       set glbFontChoiceInfo($obj,$style,flag) \
        [glbGetFontChoiceStyleVal $f $style] 
      }

   set glbFontChoiceInfo($obj,size)   [glbGetFontChoiceStyleVal $f size]
   set glbFontChoiceInfo($obj,family) [glbGetFontChoiceStyleVal $f family]

#
# Build interface:

   set nN $glbFontChoiceInfo(stdListN)
   set nS $glbFontChoiceInfo(maxNameLength)

   label  $w.l -text $glbFontChoiceInfo($obj,title)
   pack   $w.l  -in $w -side top

   frame $w.left    
   frame $w.right   
   frame $w.sliders -relief groove -border 2

   frame     $w.ls
   label     $w.lstd -text "Standard Fonts"
   scrollbar $w.sls -orient vert -command "$w.lbs yview"

   listbox   $w.lbs -height 4 -width $nS -selectmode single \
                    -yscrollcommand "$w.sls set" -exportselection 0

   pack $w.lbs $w.sls -in $w.ls -side left -expand 1 -fill y

   frame     $w.lx
   frame     $w.space1 -height 4
   label     $w.lfam -text "Font Family"
   scrollbar $w.slx -orient vert -command "$w.lbx yview"

   listbox   $w.lbx -height 10 -width $nS -selectmode single \
                    -yscrollcommand "$w.slx set" -exportselection 0

   pack $w.lbx $w.slx -in $w.lx -side left -expand 1 -fill y

   frame     $w.lp
   label     $w.lsize -text "Font Size"
   scrollbar $w.slp -orient vert -command "$w.lbp yview"

   listbox   $w.lbp -height 14 -width 6 -selectmode single \
                    -yscrollcommand "$w.slp set" -exportselection 0

   pack $w.lbp $w.slp -in $w.lp -side left -expand 1 -fill y
   
   pack $w.lstd $w.ls $w.space1 $w.lfam $w.lx \
        -in $w.left -side top -expand 1 -fill y

   pack $w.lsize $w.lp -in $w.right -side top -expand 1 -fill y

   pack $w.left $w.right -in $w.sliders -side left -padx 3 -pady 3
   pack $w.sliders -in $w -side top -padx 3 -pady 3

   set glbFontChoiceInfo($obj,stdWin)  $w.lbs
   set glbFontChoiceInfo($obj,famWin)  $w.lbx
   set glbFontChoiceInfo($obj,sizeWin) $w.lbp
   set glbFontChoiceInfo($obj,textWin) $w.txt

   foreach f $glbFontChoiceInfo(stdNames)  {$w.lbs insert end "$f Font"}
   foreach f $glbFontChoiceInfo(famList)   {$w.lbx insert end $f}
   foreach s $glbFontChoiceInfo(sizeList)  {$w.lbp insert end $s}

   frame $w.style

   foreach style $glbFontChoiceInfo(styleList) \
      {
       checkbutton $w.style$style -text $style -selectcolor blue \
                   -variable glbFontChoiceInfo($obj,$style,flag) \
                   -command "glbSetFontChoiceStyle $obj $style"

       pack $w.style$style -in $w.style -side left
      }

   pack $w.style -in $w -side top -pady 2 -anchor center

   text $w.txt -height 3 -width 12 -font $glbFontChoiceInfo($obj,font) \
        -foreground $glbFontChoiceInfo($obj,fgColor) -relief groove -border 2 

   if {[strcasecmp $glbFontChoiceInfo($obj,bgColor) None]} \
      {
       $w.txt configure -background $glbFontChoiceInfo($obj,bgColor)
      }

   pack $w.txt -in $w -side top -expand 1 -fill both -padx 2 -pady 2 -anchor center

   $w.txt insert end "ABCDEFGHIJK\n" line1
   $w.txt insert end "abcdefhijk\n"  line2
   $w.txt insert end "1234567890"    line3

   $w.txt tag configure line1 -justify center
   $w.txt tag configure line2 -justify center
   $w.txt tag configure line3 -justify center

   entry   $w.e -textvariable glbFontChoiceInfo($obj,font) -width 44
   pack    $w.e  -in $w -side top -expand 1 -fill x -pady 2

   frame   $w.cmnd
   button  $w.apl  -text Apply  -command "glbFontApplyChoice  $obj"
   button  $w.sel  -text Select -command "glbFontAcceptChoice $obj"
   button  $w.quit -text Cancel -command "glbFontCancelChoice $obj"

   pack $w.apl $w.sel $w.quit -in $w.cmnd -side left -expand 1 -fill x
   pack $w.cmnd -in $w -side top -expand 1 -fill x

   bind $w.lbs <ButtonRelease-1> "+ glbSetStdFontChoice    $obj $w.lbs"
   bind $w.lbx <ButtonRelease-1> "+ glbSetFontChoiceFamily $obj $w.lbx"
   bind $w.lbp <ButtonRelease-1> "+ glbSetFontChoiceSize   $obj $w.lbp"

   update

   glbUpdateFontSelect $obj 

   return $w
}

proc glbUpdateStdFontSelect { obj } \
{
   global glbFontChoiceInfo  

   set f   $glbFontChoiceInfo($obj,font)
   set loc 0

   set iList [$glbFontChoiceInfo($obj,stdWin) curselection]
   
   if {[llength $iList]} \
      {
       set loc [lindex $iList 0]
       set thisF [lindex $glbFontChoiceInfo(stdFonts) $loc]

       if {![strcasecmp $f $thisF]} \
          {
           $glbFontChoiceInfo($obj,stdWin) selection clear 0 end
           $glbFontChoiceInfo($obj,stdWin) selection set $loc 
           $glbFontChoiceInfo($obj,stdWin) see $loc 
           return
          }
      }

   $glbFontChoiceInfo($obj,stdWin) selection clear 0 end
   $glbFontChoiceInfo($obj,stdWin) see 0

   foreach thisF $glbFontChoiceInfo(stdFonts) \
      {
       if {![strcasecmp $thisF $f]} \
          {
           $glbFontChoiceInfo($obj,stdWin) selection set $loc
           $glbFontChoiceInfo($obj,stdWin) see $loc
           break
          }

       incr loc
      }
}

proc glbUpdateFontSelect { obj } \
{
   global glbFontChoiceInfo 

   set f   $glbFontChoiceInfo($obj,font)
   set loc 0

   glbUpdateStdFontSelect $obj

   set thisF [font actual $f -family]
   set fList [string tolower $glbFontChoiceInfo(famList)]
   set loc   [lsearch -exact $fList $thisF]

   if {$loc >= 0} \
      {
       set glbFontChoiceInfo($obj,family) [lindex $glbFontChoiceInfo(famList) $loc]

       $glbFontChoiceInfo($obj,famWin) selection clear 0 end
       $glbFontChoiceInfo($obj,famWin) selection set $loc 
       $glbFontChoiceInfo($obj,famWin) see $loc 
      }

   set glbFontChoiceInfo($obj,size) [glbGetFontChoiceStyleVal $f size] 

   set size $glbFontChoiceInfo($obj,size)

   if {$size > 0 && $size <= $glbFontChoiceInfo(sizeListN)} \
      {
       $glbFontChoiceInfo($obj,sizeWin) selection clear 0 end
       $glbFontChoiceInfo($obj,sizeWin) selection set [expr $size - 1] 
       $glbFontChoiceInfo($obj,sizeWin) see [expr $size - 1]
      }

   foreach style $glbFontChoiceInfo(styleList) \
      {
       set glbFontChoiceInfo($obj,$style,flag) [glbGetFontChoiceStyleVal $f $style] 
      }
 
   update
}

proc glbShowFontChoice { obj } \
{
   global glbFontChoiceInfo 

   $glbFontChoiceInfo($obj,textWin) configure -font $glbFontChoiceInfo($obj,font)
   glbCopyFontChoice $obj

   update
}

proc glbCopyFontChoice { obj } \
{
   global glbFontChoiceInfo 

   clipboard clear
   clipboard append -- $glbFontChoiceInfo($obj,font)
}

proc glbUpdateFontChoice { obj } \
{
   global glbFontChoiceInfo

   set f ""

   lappend f $glbFontChoiceInfo($obj,family)
   lappend f $glbFontChoiceInfo($obj,size)

   foreach style $glbFontChoiceInfo(styleList) \
      {
       if {!$glbFontChoiceInfo($obj,$style,flag)} continue
       lappend f $style
      }

   set glbFontChoiceInfo($obj,font) $f

   glbUpdateStdFontSelect $obj
   glbShowFontChoice $obj
}

proc glbGetX11Fonts { obj } \
{
   global glbFontChoiceInfo

   set fmt    "-*-%s-%s-%s-*-*-*-%s-*-*-*-*-*-*"
   set famStr [string tolower $glbFontChoiceInfo($obj,family)]
   set wStr   *
   set sStr   *
   set size   *

   if {$glbFontChoiceInfo($obj,bold,flag)}   {set wStr bold}
   if {$glbFontChoiceInfo($obj,italic,flag)} {set sStr r}

   set size [format %d [expr 10*$glbFontChoiceInfo($obj,size)]]

   set f [format $fmt $famStr $wStr $sStr $size]
   set fList ""

   set error [catch {set fList [eval exec xlsfonts -fn $f]}]
}

proc glbSetFontChoiceStyle { obj style } \
{
   global glbFontChoiceInfo

   set val None 

   if {[info exists glbFontChoiceInfo($obj,$style,flag)]} \
      {
       set val $glbFontChoiceInfo($obj,$style,flag)
       set glbFontChoiceInfo($obj,$style,flag) $val
      }

   glbUpdateFontChoice $obj
}

proc glbSetStdFontChoice { obj w } \
{
   global glbFontChoiceInfo

   set iList [$w curselection]
   if {![llength $iList]} return

   set loc [lindex $iList 0]
   if {$loc < 0 || $loc > $glbFontChoiceInfo(stdListN)} return

   set glbFontChoiceInfo($obj,font) [lindex $glbFontChoiceInfo(stdFonts) $loc] 

   glbUpdateFontSelect $obj 
   glbShowFontChoice   $obj
}

proc glbSetFontChoiceFamily { obj w } \
{
   global glbFontChoiceInfo

   set iList [$w curselection]
   if {![llength $iList]} return

   set loc [lindex $iList 0]
   if {$loc < 0 || $loc > $glbFontChoiceInfo(famListN)} return

   set glbFontChoiceInfo($obj,family) [lindex $glbFontChoiceInfo(famList) $loc] 

   glbUpdateFontChoice $obj
}

proc glbSetFontChoiceSize { obj w } \
{
   global glbFontChoiceInfo

   set iList [$w curselection]
   if {![llength $iList]} return

   set loc [lindex $iList 0]
   if {$loc < 0 || $loc > $glbFontChoiceInfo(sizeListN)} return

   set glbFontChoiceInfo($obj,size) [lindex $glbFontChoiceInfo(sizeList) $loc] 

   glbUpdateFontChoice $obj
}

proc glbGetFontChoiceStyleVal { f style } \
{
   global glbFontChoiceInfo

   switch $style \
      {
       bold       {if {![strcmp [font actual $f -weight] bold]}   {return 1}}
       italic     {if {![strcmp [font actual $f -slant]  italic]} {return 1}}
       underline  {if {![strcmp [font actual $f -underline]  1]}  {return 1}}
       overstrike {if {![strcmp [font actual $f -overstrike] 1]}  {return 1}}
       family     {return [font actual $f -family]}
       size       {return [font actual $f -size]}
      }

   return 0
}

proc glbFindFontChoice { obj f } \
{
   global glbFontChoiceInfo 

   set id [glbGetFontChoiceID $f]
   set n  0

   if {[info exists glbFontChoiceInfo(font,$id)]} \
      {
       set n [llength $glbFontChoiceInfo(font,$id)]
      }

   if {$n} \
      {
       set glbFontChoiceInfo($obj,font) [lindex $glbFontChoiceInfo(font,$id) 0]
      } \
   else \
      {
       set glbFontChoiceInfo($obj,font) [glbGetFontChoiceTK $f]
      }
}

proc glbInitFontChoiceXLS { obj } \
{
   global glbFontChoiceInfo 

   if {!$glbFontChoiceInfo($obj,xlsFlag)} return
   if {$glbFontChoiceInfo(xlsListN) > 0}  return 

   set glbFontChoiceInfo(xlsList) $glbFontChoiceInfo(stdFonts)
   set xlsList ""

   set error [catch {set xlsList [exec xlsfonts]}]
   
   if {$error || ![info exists xlsList]} \
      {
       set glbFontChoiceInfo(xlsList)  ""
       set glbFontChoiceInfo(xlsListN) 0
       return
      }

   foreach f $xlsList {lappend glbFontChoiceInfo(xlsList) $f}

   set glbFontChoiceInfo(xlsListN) [llength $glbFontChoiceInfo(xlsList)]
   if {!$glbFontChoiceInfo(xlsListN)} return

   foreach f $glbFontChoiceInfo(xlsList) \
      {
       set id [glbGetFontChoiceID $f]
       lappend glbFontChoice(font,$id) $f
      }
}

proc glbGetFontChoiceTK { obj } \
{
   set fam  [glbGetFontChoiceStyleVal $f family]
   set size [glbGetFontChoiceStyleVal $f size]

   set fStr "$fam $size 

   foreach style {$glbFontChoiceInfo(styleList)} \
      {
       set val [glbGetFontChoiceStyleVal $f $style]
       if {[strcmp $val 0]} {append fStr " $style"}
      }

   return $fStr
}

proc glbGetFontChoiceID { f } \
{
   global glbFontChoiceInfo env

   set fam  [glbGetFontChoiceStyleVal $f family]
   set size [glbGetFontChoiceStyleVal $f size]

   set vStr ""

   foreach style $glbFontChoiceInfo(styleList) \
      {
       append vStr [glbGetFontChoiceStyleVal $f $style]
      }

   return $fam,$size,$vStr
}
 
proc glbInitFontChoiceInfo {} \
{
   global glbFontChoiceInfo env

   if {[info exists glbFontChoiceInfo(famList)]} \
      {
       if {[llength $glbFontChoiceInfo(famList)]} return
      }

   set glbFontChoiceInfo(xlsList)  ""
   set glbFontChoiceInfo(xlsListN) 0

   set glbFontChoiceInfo(stdList)  \
    {stdfont bigfont smallfont boldfont fixedfont axisfont typingfont}

   set glbFontChoiceInfo(stdFonts) ""

   set glbFontChoiceInfo(stdNames) {Standard Big Small Bold Fixed Axis Fixed}

   set glbFontChoiceInfo(bigfont)   \
     "-adobe-helvetica-medium-r-*-*-*-160-*-*-*-*-*-*"

   set glbFontChoiceInfo(smallfont) \
     "-adobe-helvetica-medium-r-*-*-*-100-*-*-*-*-*-*"

   set glbFontChoiceInfo(stdfont)   \
     "-adobe-helvetica-medium-r-*-*-*-100-*-*-*-*-*-*"

   set glbFontChoiceInfo(boldfont)  \
     "-adobe-helvetica-bold-r-*-*-*-120-*-*-*-*-*-*"

   set glbFontChoiceInfo(fixedfont)  \
     "-*-courier-medium-r-*-*-*-120-*-*-*-*-*-*"

   set glbFontChoiceInfo(typingfont) \
     "-*-courier-medium-r-*-*-*-100-*-*-*-*-*-*"

   set glbFontChoiceInfo(axisfont)   \
     "-*-lucidatypewriter-bold-r-*-*-*-100-*-*-*-*-*-*"

   if {[info exists smallfont]}  {set glbFontChoiceInfo(smallfont)  $smallfont}
   if {[info exists bigfont]}    {set glbFontChoiceInfo(bigfont)    $bigfont}
   if {[info exists stdfont]}    {set glbFontChoiceInfo(stdfont)    $stdfont}
   if {[info exists boldfont]}   {set glbFontChoiceInfo(boldfont)   $boldfont}
   if {[info exists fixedfont]}  {set glbFontChoiceInfo(fixedfont)  $fixedfont}
   if {[info exists typingfont]} {set glbFontChoiceInfo(typingfont) $typingfont}
   if {[info exists axisfont]}   {set glbFontChoiceInfo(axisfont)   $axisfont}

   foreach s $glbFontChoiceInfo(stdList) \
      {
       lappend glbFontChoiceInfo(stdFonts) $glbFontChoiceInfo($s)
      }

   set glbFontChoiceInfo(famList) [lsort [font families]]

   set n 12

   foreach f $glbFontChoiceInfo(famList) \
      {
       set nF [string length $f]
       if {$nF > $n} {set n $nF}
      }

   set glbFontChoiceInfo(maxNameLength) $n
   set glbFontChoiceInfo(sizeList)      [getIota 1 1 72]
   set glbFontChoiceInfo(styleList)     {bold italic underline overstrike} 

   set glbFontChoiceInfo(stdListN)   [llength $glbFontChoiceInfo(stdList)]
   set glbFontChoiceInfo(sizeListN)  [llength $glbFontChoiceInfo(sizeList)]
   set glbFontChoiceInfo(styleListN) [llength $glbFontChoiceInfo(styleList)]
   set glbFontChoiceInfo(famListN)   [llength $glbFontChoiceInfo(famList)]
}

proc glbFontApplyChoice { obj } \
{
   global glbFontChoiceInfo

   if {[strcasecmp $glbFontChoiceInfo($obj,cmnd) None]} \
      {
       catch "eval $glbFontChoiceInfo($obj,cmnd) $glbFontChoiceInfo($obj,font)"
      }  

   incr glbFontChoiceInfo($obj,applyCount)
}

proc glbFontAcceptChoice { obj } \
{
   global glbFontChoiceInfo

   glbFontApplyChoice $obj
   catch "destroy $glbFontChoiceInfo($obj,window)"
}

proc glbFontCancelChoice { obj } \
{
   global glbFontChoiceInfo

   if {$glbFontChoiceInfo($obj,applyCount)} \
      {
       set glbFontChoiceInfo($obj,font) $glbFontChoiceInfo($obj,origFont)
       glbFontApplyChoice $obj
      }

   catch "destroy $glbFontChoiceInfo($obj,window)"
}

proc glbFontChoiceProc args \
{
   puts [string trim $args "{}"]
}
