#
# Statistics utilities for table columns.

proc getTabVals { tab var } \
{
   set vList ""

   if {![strcmp $var "*"]} {set var [gdbGetInfo $tab -parms varNames]}

   set entry [gdbFirst $tab]

   while {$entry} \
      {
       lappend vList  [gdbGet $entry $var]
       set entry [gdbNext $entry]
      }

   return $vList
}

proc getTabUniqVals args \
{
   set nullFlag 0
   set nullStr  "(Null)"

   set tab [lindex $args 0]
   set var [lindex $args 1]
   set n   [llength $args]

   if {$n > 2} \
      {
       set nullStr [lindex $args 2]
       set nullFlag 1
      }
 
   set vList ""

   set entry [gdbFirst $tab]

   while {$entry} \
      {
       set v [gdbGet $entry $var]
       if {$nullFlag && ![strcasecmp $v "(Null)"]} {set v $nullStr}
       if {[lsearch -exact $vList $v] < 0} {lappend vList $v}
       set entry [gdbNext $entry]
      }

   return $vList
}

proc getTabStat { procName tab var } \
{
   set vList [getTabVals $tab $var]
   set s     [$procName $vList]

   return $s
}

proc getTabStatU { procName tab var uVal } \
{
   set vList ""

   set entry [gdbFirst $tab]

   while {$entry} \
      {
       set v [gdbGet $entry $var]
       if {$v < $uVal} {lappend vList $v}
       set entry [gdbNext $entry]
      }

   set s [$procName $vList]

   return $s
}

proc getEListStat { procName eList var } \
{
   set vList [getEListVals $eList $var] 
   set s     [$procName $vList]

   return $s
}

proc getEntryVals { e varList } \
{
   set valList ""

   foreach v $varList {lappend valList [gdbGet $e $v]}

   return $valList 
}

proc getEntryValsFmt { e var } \
{
   set t [gdbGetInfo $e -parms parent]

   if {![strcmp $var "*"]} \
      {
       set var [gdbGetInfo $t -parms varNames]
      }

   set fmt ""

   foreach v $var \
      {
       set vPtr [gdbFindByName $t $v]

       if {[llength $vPtr]} \
          {
           lappend fmt [gdbGetInfo $vPtr -parms varFmt]
          } \
       else \
          {
           lappend fmt "%s"
          }
      }

   set n [llength $var]
   set s ""

   for {set i 0} {$i < $n} {incr i} \
      {
       append s [format [lindex $fmt $i] [gdbGet $e [lindex $var $i]]] 
       if {$i < $n - 1} {append s " "}
      } 

   return $s
}

proc getEListVals { eList var } \
{
   if {![strcmp $var "*"] && [llength $eList]} \
      {
       set e   [lindex $eList 0]
       set t   [gdbGetInfo $e -parms parent]
       set var [gdbGetInfo $t -parms varNames]
      }

   set vList ""

   foreach e $eList \
      {
       lappend vList [gdbGet $e $var]
      }

   return $vList 
}

proc getTabMax { tab var } \
{
   set entry [gdbFirst $tab]
   set val   ""

   if {$entry}       {set val [gdbGet $entry $var]}
   if {"$val" == ""} {return 0}

   while {$entry} \
      {
       set x [gdbGet $entry $var]
       if {$x > $val} {set val $x}
       set entry [gdbNext $entry]
      }

   return $val
}

proc getTabMin { tab var } \
{
   set entry [gdbFirst $tab]
   set val   ""

   if {$entry}       {set val [gdbGet $entry $var]}
   if {"$val" == ""} {return 0}

   while {$entry} \
      {
       set x [gdbGet $entry $var]
       if {$x < $val} {set val $x}
       set entry [gdbNext $entry]
      }

   return $val
}

proc getTabSum { tab var } \
{
   set entry [gdbFirst $tab]
   set val   ""

   if {$entry}       {set val [gdbGet $entry $var]}
   if {"$val" == ""} {return 0}

   set val 0.0

   while {$entry} \
      {
       set val   [expr $val + [gdbGet $entry $var]]
       set entry [gdbNext $entry]
      }

   return $val
}

proc getTabAvg { tab var } \
{
   set entry [gdbFirst $tab]
   set val   ""

   if {$entry}       {set val [gdbGet $entry $var]}
   if {"$val" == ""} {return 0.0}

   set val 0.0
   set n   0

   while {$entry} \
      {
       set  val   [expr $val + [gdbGet $entry $var]]
       set  entry [gdbNext $entry]
       incr n
      }

   if {$n > 1} {set val [expr $val/double($n)]}

   return $val
}

proc getTabStdDev { tab var } \
{
   set entry [gdbFirst $tab]
   set val   ""

   if {$entry}       {set val [gdbGet $entry $var]}
   if {"$val" == ""} {return 0.0}

   set val 0.0
   set sum 0.0
   set n   0

   while {$entry} \
      {
       set  sum [expr $sum + [gdbGet $entry $var]]
       set  entry [gdbNext $entry]
       incr n
      }

   if {$n > 1} {set sum [expr $sum/double($n)]}

   set entry [gdbFirst $tab]

   while {$entry} \
      {
       set d     [expr [gdbGet $entry $var] - $sum]
       set val   [expr $val + $d*$d]
       set entry [gdbNext $entry]
      }

   if {$n > 1} \
      {
       set val [expr sqrt($val/double($n-1))]
      } \
   else \
      {
       set val [expr sqrt($val)]
      }

   return $val
}

proc getEListMax { eList var } \
{
   set val 0

   set entry [lindex $eList 0]

   if {[llength $entry]} {set val [gdbGet $entry $var]}

   foreach entry $eList \
      {
       set x [gdbGet $entry $var]
       if {$x > $val} {set val $x}
      }

   return $val
}

proc getEListMin { eList var } \
{
   set val 0

   set entry [lindex $eList 0]

   if {[llength $entry]} {set val [gdbGet $entry $var]}

   foreach entry $eList \
      {
       set x [gdbGet $entry $var]
       if {$x < $val} {set val $x}
      }

   return $val
}

proc getEListSum { eList var } \
{
   set val 0

   foreach entry $eList \
      {
       set val [expr $val + [gdbGet $entry $var]]
      }

   return $val
}

proc getEListAvg { eList var } \
{
   set val 0.0
   set n   [llength $eList]

   foreach entry $eList \
      {
       set val [expr $val + [gdbGet $entry $var]]
      }

   if {$n > 1} {set val [expr $val/double($n)]}

   return $val
}

proc getEListStdDev { eList var } \
{
   set val 0.0
   set sum 0.0
   set n   [llength $eList]

   foreach entry $eList \
      {
       set sum [expr $sum + [gdbGet $entry $var]]
      }

   if {$n > 1} {set sum [expr $sum/double($n)]}

   foreach entry $eList \
      {
       set d   [expr [gdbGet $entry $var] - $sum]
       set val [expr $val + $d*$d]
      }

   if {$n > 1} \
      {
       set val [expr sqrt($val/double($n-1))]
      } \
   else \
      {
       set val [expr sqrt($val)]
      }

   return $val
}

#
# Bottom.
