#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set progTitle    "Generate a Table of Synthetic Peaks."
set progVersion  "2021.083.06.34"

#
# frank.delaglio@nist.gov

if {[flagLoc $ARGV -help]} \
   {
    puts "Generate a Table of Synthetic Peaks to Insert into Existing Time-Domain Data:"
    puts "frank.delaglio@nist.gov  Version $progVersion"
    puts "Input Data:"
    puts " -fid      fidName   \[fid/test%03d.fid\]   Reference FID Name."
    puts " -ft       ftName    \[ft/test%03d.fid\]    Reference Spectrum Name."
    puts " -empty    eName     \[empty/test%03d.ft3\] Mask of Empty Spectral Regions."
    puts " -avoid    tabName   \[None\]               Table of Additional Peak Positions to Avoid."
    puts " -tab      tabName   \[largest.tab\]        Table of Largest Peaks in Spectrum."
    puts " -xyTab    xyTabName \[None\]               Table of Positions for xy=tab Program."
    puts " -zTab     zTabName  \[None\]               Table of Positions for z=tab Program."
    puts " -reg      regList   \[5% 95% 10% 90% ...\] Allowed 3D Region for Peaks, Pts, ppm % (x1 xn y1 yn z1 zn)."
    puts " -trials   maxT      \[1024\]               Maximum Allowed Trials for Creating Peaks."
    puts " -xzy                                    Same As: -yName Z_AXIS -zName Y_AXIS -yVar Z -zVar Y"
    puts "Other Output:"
    puts " -hn       hnName   \[None\]               Output Table of 2D XY Coords for Strips (Also: -hc)."
    puts " -empty2D  eName    \[None\]               2D Projection of Empty Region Mask."
    puts "Simulation Amplitudes:"
    puts " -hiMin    hiMin    \[25.0\]               Minimum Time-Domain Amplitude."
    puts " -hiMax    hiMax    \[100.0\]              Maximum Time-Domain Amplitude."
    puts " -hiMinAlt hiMin    \[250.0\]              Alternative Minimum Time-Domain Amplitude (For NOE Diagonal)."
    puts " -hiMaxAlt hiMax    \[1000.0\]             Alternative Maximum Time-Domain Amplitude (For NOE Diagonal)."
    puts " -hiScale  hiScale  \[1.0\]                Scale Factor for Amplitudes."
    puts "Simulation Type:"
    puts " -specType type     \[hnca\]               Simulation Type: hnco hnca cbcaconh cbcanh n15noe c13noe nmr"
    puts " -minPk    nMin     \[Auto\]               Minimum Peaks per Strip."
    puts " -maxPk    nMax     \[Auto\]               Maximum Peaks per Strip."
    puts " -jCount   jCount   \[2]\                  Max Coupling Values per Dimension (0 to 4)."
    puts " -iseed    iseed    \[7625297\]            Random Number Seed, 0 for Auto."
    puts " -id       id       \[1\]                  Starting Peak Index Number."
    puts " -note     note     \[None\]               Comment About the Simulation."
    puts " -hdr                                    Print Peak Table Header (Default)."
    puts " -nohdr                                  Do Not Print Peak Table Header."
    puts " -nocomment                              Do Not Print Strip Program Comments."
    puts " -hdrOnly                                Print Header Only, then Exit."
    puts "Input Spectrum Axis Definitions:"
    puts " -xName     xName    \[X_AXIS\]   X-Axis (Usually Axis of the HN Dimension)." 
    puts " -yName     yName    \[Y_AXIS\]   Y-Axis (Usually Axis of the 15N Dimension)."
    puts " -zName     zName    \[Z_AXIS\]   Z-Axis (Such CA Dimension Axis of an HNCA)."
    puts "Peak Table Definitions:"
    puts " -xVar      xVar     \[X\]        Prefix for X-Axis Variables (Such as X_PPM)."
    puts " -yVar      yVar     \[Y\]        Prefix for Y-Axis Variables (Such as Y_PPM)."
    puts " -zVar      zVar     \[Z\]        Prefix for Z-Axis Variables (Such as Z_PPM)."
    puts "Line Widths (Time-Domain Decay, Use -x -y -z):"
    puts " -xwMin     xwMin    \[0.0\]      Minimum Decay, Hz."
    puts " -xwMax     xwMax    \[0.0\]      Maximum Decay, Hz."
    puts "Phase Correction Perturbation (Use -x -y -z):"
    puts " -xP0Min    p0Min    \[0.0\]      Minimum Zero-Order Phase Perturbation."
    puts " -xP0Max    p0Max    \[0.0\]      Maximum Zero-Order Phase Perturbation."
    puts " -xP1Min    p1Min    \[0.0\]      Minimum First-Order Phase Perturbation."
    puts " -xP1Max    p1Max    \[0.0\]      Maximum First-Order Phase Perturbation."
    puts " -xP0MinAlt p0MinAlt \[0.0\]      Alternate Minimum Zero-Order Perturbation (For CBCANH)."
    puts " -xP0MaxAlt p0MaxAlt \[0.0\]      Alternate Maximum Zero-Order Perturbation (For CBCANH)."
    puts "Coupling Values (Max Number of Values Set by -jCount. Use -x -y -z):"
    puts " -xJMin     jMinList \[0.0 ...\]  List of Minimum J Values, Hz."       
    puts " -xJMax     jMaxList \[0.0 ...\]  List of Maximum J Values, Hz."       
    puts "Values for Overlap Testing:"
    puts " -dx        dx       \[Auto\]     X-Axis Overlap Distance, ppm."
    puts " -dy        dy       \[Auto\]     Y-Axis Overlap Distance, ppm."
    puts " -dz        dz       \[Auto\]     Z-Axis Overlap Distance, ppm."
    puts " -thresh    thresh   \[Auto\]     Mask Threshold for Identifying Empty XY Coords." 
    puts "Values for Overlap Testing (-dx Auto etc):"
    puts " -dHN       dHN      \[0.06\]     HN  Direct Dimension Distance, ppm."
    puts " -d1H       d1H      \[0.06\]     1H  Indirect Dimension Distance, ppm."
    puts " -d15N      d15N     \[0.60\]     15N Dimension Distance, ppm."
    puts " -d13C      d13C     \[1.60\]     13C Dimension Distance, ppm."
    puts " -dNMR      dNMR     \[1.00\]     Generic Dimension Distance, ppm (-specType nmr)."
    puts "Strip Program Defaults:"
    puts " -prog      prog     \[count=10,xy=novel,z=existing,noNewOverlap,noSpecOverlap\]"
    puts "Strip Program Values:"
    puts " count      \[10\]   Number of Strips to Simulate, or Keyword All for xy=tab z=tab"
    puts " xy                X,Y Position Keyword: Existing Novel Any (Not for Use with x or y)."
    puts " x                 X Position Keyword:   Existing Novel Any (Not for Use with xy)."
    puts " y                 Y Position Keyword:   Existing Novel Any (Not for Use with xy)."
    puts " z                 Z Position Keyword:   Existing Any."
    puts "Strip Program Notes, Other Overlap Tests Still Apply (Input Table Specified by -tab):"
    puts " xy=existing       XY Position is Taken from a Randomly Selected Peak in Input Table."
    puts " xy=novel          XY Position is Random, With No Substantial Signals Anywhere in Z-Axis."
    puts " xy=any            XY Position is Random, Other Overlap Tests Still Apply."
    puts " xy=tab            XY Position is Taken from Table Specified by the -xyTab Option."
    puts " z=existing        Z Position is Taken from a Randomly Selected Peak in Input Table."
    puts " z=any             Z Position is Random, Other Overlap Tests Still Apply."
    puts " z=tab             Z Position is Taken from Table Specified by the -zTab Option."
    puts "Other Values That Can Be Changed by Strip Program:"
    puts " iseed             See -iseed Above; Resets Random Number Generator."
    puts " hiMin             See -hiMin Above."
    puts " hiMax             See -hiMax Above."
    puts " hiMinAlt          See -hiMinAlt Above."
    puts " hiMaxAlt          See -hiMaxAlt Above."
    puts " hiScale           See -hiScale Above."
    puts " xwMin             See -xwMin Above."
    puts " xwMax             See -xwMax Above."
    puts " dxOv              X Distance Between Peaks Forced to Overlap, ppm (Default is -dx Above)."
    puts " dyOv              Y Distance Between Peaks Forced to Overlap, ppm (Default is -dy Above)."
    puts " dzOv              Z Distance Between Peaks Forced to Overlap, ppm (Default is -dz Above)."
    puts "Strip Program Overlap Tests (Not used for xy=tab or z=tab):"
    puts " noNewOverlap      No Overlap is Allowed with Inserted Peaks (Default)."
    puts " newOverlapOK      Overlap with Inserted Peaks is Allowed."
    puts " noSpecOverlap     No Overlap is Allowed with Existing Peaks in Spectrum (Default)."
    puts " specOverlapOK     Overlap with Existing Peaks is Allowed."
    puts " forceXOverlap     Force X Overlap with Existing Peak by dxOv (For xy=Existing)."
    puts " forceYOverlap     Force Y Overlap with Existing Peak by dyOv (For xy=Existing)."
    puts " forceZOverlap     Force Z Overlap with New Peak by dzOv."
    exit
   }

set fidName     fid/test%03d.fid
set ftName      ft/test%03d.ft3
set emptyName   empty/test%03d.ft3 
set tabName     largest.tab
set xyTabName   None
set zTabName    None
set emptyName2D None
set avoidName   None
set hnName      None
set noteText    None
set dimCount    3
set xzyFlag     0

set progList    "count=10,xy=novel,z=existing,noNewOverlap,noSpecOverlap"

set xSpecDim    X_AXIS
set ySpecDim    Y_AXIS
set zSpecDim    Z_AXIS

set xVarDim     X
set yVarDim     Y
set zVarDim     Z

set hiMin       25.0
set hiMax       100.0
set hiMinAlt    250.0
set hiMaxAlt    1000.0
set hiScale     1.0

set regList     "5% 95% 10% 90% 10% 90%"
set iseed       7625297
set specType    hnca
set jCount      2

set minPkPerStrip Auto
set maxPkPerStrip Auto

set firstID     1
set hdrFlag     1
set commentFlag 1
set verbose     1

set dx          Auto
set dy          Auto
set dz          Auto

set dHN         0.06
set d15N        0.60
set d13C        1.60 
set d1H         0.06
set dNMR        1.00
set thresh      Auto

set maxTrials   1024

set xyMode            Novel
set xMode             None
set yMode             None
set zMode             Existing

set stripCount        10
set noNewOverlapFlag  1
set noSpecOverlapFlag 1
set forceXOverlapFlag 0
set forceYOverlapFlag 0
set forceZOverlapFlag 0

if {[flagLoc $ARGV -xzy]} \
   {
    set xzyFlag     1

    set ySpecDim    Z_AXIS
    set zSpecDim    Y_AXIS
    
    set yVarDim     Z
    set zVarDim     Y
   }

getArgD $ARGV -fid      fidName
getArgD $ARGV -ft       ftName
getArgD $ARGV -hn       hnName
getArgD $ARGV -hc       hnName
getArgD $ARGV -tab      tabName
getArgD $ARGV -xyTab    xyTabName
getArgD $ARGV -zTab     zTabName
getArgD $ARGV -id       firstID
getArgD $ARGV -jCount   jCount

getArgD $ARGV -minPk    minPkPerStrip
getArgD $ARGV -maxPk    maxPkPerStrip 

getArgD $ARGV -empty    emptyName
getArgD $ARGV -empty2D  emptyName2D
getArgD $ARGV -avoid    avoidName

getArgD $ARGV -specType specType
getArgD $ARGV -iseed    iseed
getArgD $ARGV -trials   maxTrial

getArgD $ARGV -dHN      dHN
getArgD $ARGV -d15N     d15N
getArgD $ARGV -d13C     d13C
getArgD $ARGV -d1H      d1H
getArgD $ARGV -dNMR     dNMR

getArgD $ARGV -hiMin    hiMin
getArgD $ARGV -hiMin    hiMax
getArgD $ARGV -hiMax    hiMax

getArgD $ARGV -hiMinAlt hiMinAlt
getArgD $ARGV -hiMinAlt hiMaxAlt
getArgD $ARGV -hiMaxAlt hiMaxAlt

getArgD $ARGV -hiScale  hiScale

getArgD $ARGV -xName    xSpecDim
getArgD $ARGV -yName    ySpecDim 
getArgD $ARGV -zName    zSpecDim 

getArgD $ARGV -xVar     xVarDim
getArgD $ARGV -yVar     yVarDim
getArgD $ARGV -zVar     zVarDim

getArgD $ARGV -thresh   thresh
getArgD $ARGV -dx       dx
getArgD $ARGV -dy       dy
getArgD $ARGV -dz       dz
 
getListArgD $ARGV -reg  regList
getListArgD $ARGV -note noteText
getListArgD $ARGV -prog progList

if {![strcasecmp $yVarDim Z] && ![strcasecmp $zVarDim Y]} {set xzyFlag 1}

if {[flagLoc $ARGV -verb]}    {set verbose 1}
if {[flagLoc $ARGV -noverb]}  {set verbose 0}

if {[flagLoc $ARGV -hdr]}     {set hdrFlag 1}
if {[flagLoc $ARGV -nohdr]}   {set hdrFlag 0}
if {[flagLoc $ARGV -hdrOnly]} {set hdrFlag 2}

if {[flagLoc $ARGV -nocomment]} {set commentFlag 0}

if {$jCount < 0} {set jCount 0}
if {$jCount > 4} {set jCount 4}

foreach dim {x y z} \
   {
    set lwMin($dim) 0.0
    set lwMax($dim) 0.0

    set p0Min($dim) 0.0
    set p0Max($dim) 0.0

    set p1Min($dim) 0.0
    set p1Max($dim) 0.0

    set p0MinAlt($dim) 0.0
    set p0MaxAlt($dim) 0.0

    getArgD $ARGV -${dim}wMin lwMin($dim)
    getArgD $ARGV -${dim}wMin lwMax($dim)
    getArgD $ARGV -${dim}wMax lwMax($dim)

    getArgD $ARGV -${dim}P0Min p0Min($dim)
    getArgD $ARGV -${dim}P0Min p0Max($dim)
    getArgD $ARGV -${dim}P0Max p0Max($dim)

    getArgD $ARGV -${dim}P1Min p1Min($dim)
    getArgD $ARGV -${dim}P1Min p1Max($dim)
    getArgD $ARGV -${dim}P1Max p1Max($dim)

    getArgD $ARGV -${dim}P0MinAlt p0MinAlt($dim)
    getArgD $ARGV -${dim}P0MinAlt p0MaxAlt($dim)
    getArgD $ARGV -${dim}P0MaxAlt p0MaxAlt($dim)

    set jMinList [lrepeat 0.0 $jCount]
    set jMaxList [lrepeat 0.0 $jCount]

    getListArgD $ARGV -${dim}JMin jMinList
    getListArgD $ARGV -${dim}JMin jMaxList
    getListArgD $ARGV -${dim}JMax jMaxList

    for {set j 1} {$j <= $jCount} {incr j} {lappend jMinList 0.0}
    for {set j 1} {$j <= $jCount} {incr j} {lappend jMaxList 0.0}
 
    for {set j 1} {$j <= $jCount} {incr j} \
       {
        set jValMin($dim,$j) [lindex $jMinList [expr $j - 1]] 
        set jValMax($dim,$j) [lindex $jMaxList [expr $j - 1]] 
       }
   }

if {![strcasecmp $specType HNCO]} \
   {
    if {![strcasecmp $dx Auto]} {set dx $dHN}
    if {![strcasecmp $dy Auto]} {set dy $d15N}
    if {![strcasecmp $dz Auto]} {set dz $d13C}

    if {![strcasecmp $minPkPerStrip Auto]} {set minPkPerStrip 1}
    if {![strcasecmp $maxPkPerStrip Auto]} {set maxPkPerStrip 1}
   } \
elseif {![strcasecmp $specType HNCA] || ![strcasecmp $specType CBCACONH]} \
   {
    if {![strcasecmp $dx Auto]} {set dx $dHN}
    if {![strcasecmp $dy Auto]} {set dy $d15N}
    if {![strcasecmp $dz Auto]} {set dz $d13C}

    if {![strcasecmp $minPkPerStrip Auto]} {set minPkPerStrip 2}
    if {![strcasecmp $maxPkPerStrip Auto]} {set maxPkPerStrip 2}
   } \
elseif {![strcasecmp $specType CBCANH]} \
   {
    if {![strcasecmp $dx Auto]} {set dx $dHN}
    if {![strcasecmp $dy Auto]} {set dy $d15N}
    if {![strcasecmp $dz Auto]} {set dz $d13C}

    if {![strcasecmp $minPkPerStrip Auto]} {set minPkPerStrip 4}
    if {![strcasecmp $maxPkPerStrip Auto]} {set maxPkPerStrip 4}
   } \
elseif {![strcasecmp $specType N15NOE]} \
   {
    if {![strcasecmp $dx Auto]} {set dx $dHN}
    if {![strcasecmp $dy Auto]} {set dy $d15N}
    if {![strcasecmp $dz Auto]} {set dz $d1H}

    if {![strcasecmp $minPkPerStrip Auto]} {set minPkPerStrip 2}
    if {![strcasecmp $maxPkPerStrip Auto]} {set maxPkPerStrip 5}
   } \
elseif {![strcasecmp $specType C13NOE]} \
   {
    if {![strcasecmp $dx Auto]} {set dx $dHN}
    if {![strcasecmp $dy Auto]} {set dy $d13C}
    if {![strcasecmp $dz Auto]} {set dz $d1H}

    if {![strcasecmp $minPkPerStrip Auto]} {set minPkPerStrip 2}
    if {![strcasecmp $maxPkPerStrip Auto]} {set maxPkPerStrip 5}
   } \
elseif {![strcasecmp $specType nmr]} \
   {
    if {![strcasecmp $dx Auto]} {set dx $dHN}
    if {![strcasecmp $dy Auto]} {set dy $d15N}
    if {![strcasecmp $dz Auto]} {set dz $dNMR}

    if {![strcasecmp $minPkPerStrip Auto]} {set minPkPerStrip 2}
    if {![strcasecmp $maxPkPerStrip Auto]} {set maxPkPerStrip 5}
   } \
else \
   {
    showErrorMessage "GenSimTab Error: Unknown Spectrum type $specType"
    exit 1
   }

set dxOv $dx
set dyOv $dy
set dzOv $dz

if {$minPkPerStrip < 1} {set minPkPerStrip 1}
if {$maxPkPerStrip < $minPkPerStrip} {set maxPkPerStrip $minPkPerStrip}

srand $iseed

#
#-----------------------------------------------------------------------------------------------------

proc showMessage args \
{
   global verbose

   if {!$verbose} return

   foreach i $args {puts -nonewline stderr [string trim $i "{}"]}
   puts stderr ""
}

proc showErrorMessage args \
{
   foreach i $args {puts -nonewline stderr [string trim $i "{}"]}
   puts stderr ""
}

proc showProg { progID } \
{
   global xMode yMode zMode xyMode noNewOverlapFlag noSpecOverlapFlag stripCount hiMin hiMax hiScale
   global forceXOverlapFlag forceYOverlapFlag forceZOverlapFlag dxOv dyOv dzOv

   puts ""
   puts "#"
   puts "# Program: $progID"
   puts "#  Strip Count:  $stripCount"

   if {![strcasecmp $xyMode None]} \
      {
       puts "#  X Position:   [string totitle $xMode]"
       puts "#  Y Position:   [string totitle $yMode]"
      } \
   else \
      {
       puts "#  XY Position:  [string totitle $xyMode]"
      }

   puts "#  Z Position:   [string totitle $zMode]"
   puts "#"
   puts "#  Height Min:   $hiMin"
   puts "#  Height Max:   $hiMax"
   puts "#  Height Scale: $hiScale"
   puts "#"

   if {$noNewOverlapFlag} \
      {
       puts "#  Overlap with New Peaks Allowed:      No"
      } \
   else \
      {
       puts "#  Overlap with New Peaks Allows:      Yes"
      }

   if {$noSpecOverlapFlag} \
      {
       puts "#  Overlap with Existing Peaks Allowed: No"
      } \
   else \
      {
       puts "#  Overlap with Existing Peaks Allowed: Yes"
      }

   if {$forceXOverlapFlag || $forceYOverlapFlag || $forceZOverlapFlag} \
      {
       puts "#"
       if {$forceXOverlapFlag} {puts "#  Forced X Overlap: $dxOv ppm"}
       if {$forceYOverlapFlag} {puts "#  Forced Y Overlap: $dyOv ppm"}
       if {$forceZOverlapFlag} {puts "#  Forced Z Overlap: $dzOv ppm"}
      }

   puts ""
}

proc showHdr {} \
{
   global ARGV progVersion jCount varList varListXZY fmtList noteText fdataFT xSpecDim ySpecDim zSpecDim xzyFlag
   
   puts ""
   puts "#"
   puts "# Generated by genSimTab.tcl Version $progVersion frank.delaglio@nist.gov"
   puts "#"

   if {[strcasecmp $noteText None]} \
      {
#      puts "# $noteText"
#      puts "#"
      }

   prettyPrintCmnd stdout "# " "[file tail [lindex $ARGV 0]] [lrange $ARGV 3 end]"

   set varList    "INDEX X_PPM Y_PPM Z_PPM X_AXIS Y_AXIS Z_AXIS X1 X3 Y1 Y3 Z1 Z3 XW_HZ YW_HZ ZW_HZ XTW YTW ZTW X_P0 X_P1 Y_P0 Y_P1 Z_P0 Z_P1"
   set varListXZY "INDEX X_PPM Z_PPM Y_PPM X_AXIS Z_AXIS Y_AXIS X1 X3 Z1 Z3 Y1 Y3 XW_HZ ZW_HZ YW_HZ XTW ZTW YTW X_P0 X_P1 Z_P0 Z_P1 Y_P0 Y_P1"

   foreach dim {X Y Z} \
      {
       for {set j 1} {$j <= $jCount} {incr j} {lappend varList ${dim}_COSJ$j}
      }

  foreach dim {X Z Y} \
      {
       for {set j 1} {$j <= $jCount} {incr j} {lappend varListXZY ${dim}_COSJ$j}
      }
 
   lappend varList    "HEIGHT" "ASS"
   lappend varListXZY "HEIGHT" "ASS"

   set fmtList "%4d %8.4f %8.4f %8.4f %8.4f %8.4f %8.4f %4d %4d %4d %4d %4d %4d %7.4f %7.4f %7.4f %7.4f %7.4f %7.4f %6.1f %6.1f %6.1f %6.1f %6.1f %6.1f"

   foreach dim {X Y Z} \
      {
       for {set j 1} {$j <= $jCount} {incr j} {lappend fmtList "%6.3f"}
      }

   lappend fmtList "%+.5e" "%s"

   set varCount [llength $varList]
   set varStr   ""
   set fmtStr   ""

   for {set i 0} {$i < $varCount} {incr i} \
      {
       set v [lindex $varList $i]
       set f [lindex $fmtList $i]

       set nMax [string length $v]
       set nF   [string length $f]

       if {$nF > $nMax} {set nMax $nF}

       set varStr "$varStr [format %-${nMax}s $v]"
       set fmtStr "$fmtStr [format %-${nMax}s $f]"
      }

   set thisXName   [getParmStr $fdataFT NDLABEL $xSpecDim]
   set thisYName   [getParmStr $fdataFT NDLABEL $ySpecDim]
   set thisZName   [getParmStr $fdataFT NDLABEL $zSpecDim]

   set thisXSize   [getParmI   $fdataFT NDSIZE  $xSpecDim]
   set thisYSize   [getParmI   $fdataFT NDSIZE  $ySpecDim]
   set thisZSize   [getParmI   $fdataFT NDSIZE  $zSpecDim]

   set thisX1      [pnt2spec $fdataFT $xSpecDim 1 ppm]
   set thisY1      [pnt2spec $fdataFT $ySpecDim 1 ppm]
   set thisZ1      [pnt2spec $fdataFT $zSpecDim 1 ppm]

   set thisXN      [pnt2spec $fdataFT $xSpecDim $thisXSize ppm]
   set thisYN      [pnt2spec $fdataFT $ySpecDim $thisYSize ppm]
   set thisZN      [pnt2spec $fdataFT $zSpecDim $thisZSize ppm]

   puts ""

   if {$xzyFlag} \
      {
       puts [format "DATA X_AXIS %-4s  1 %4d %8.3fppm %8.3fppm" $thisXName $thisXSize $thisX1 $thisXN] 
       puts [format "DATA Y_AXIS %-4s  1 %4d %8.3fppm %8.3fppm" $thisZName $thisZSize $thisZ1 $thisZN] 
       puts [format "DATA Z_AXIS %-4s  1 %4d %8.3fppm %8.3fppm" $thisYName $thisYSize $thisY1 $thisYN] 
      } \
   else \
      {
       puts [format "DATA X_AXIS %-4s  1 %4d %8.3fppm %8.3fppm" $thisXName $thisXSize $thisX1 $thisXN] 
       puts [format "DATA Y_AXIS %-4s  1 %4d %8.3fppm %8.3fppm" $thisYName $thisYSize $thisY1 $thisYN] 
       puts [format "DATA Z_AXIS %-4s  1 %4d %8.3fppm %8.3fppm" $thisZName $thisZSize $thisZ1 $thisZN] 
      }

   puts ""
   puts "VARS   $varStr"
   puts "FORMAT $fmtStr"
   puts ""
}

proc showPkTab { pkList } \
{
    global jCount varList varListXZY fmtList currentID xzyFlag

    if {$xzyFlag} \
       {
        set thisVarList $varListXZY
       } \
    else \
       {
        set thisVarList $varList
       }

    set n [llength $thisVarList]

    foreach pk $pkList \
       {
        set stripID [getNamedItemD strip $pk 0]
        set peakID  [getNamedItemD peak  $pk 0]
        set progID  [getNamedItemD prog  $pk 0]

        set pk "index $currentID $pk ass strip_${stripID},peak_${peakID},prog_${progID}"

        for {set i 0} {$i < $n} {incr i} \
           {
            set v [lindex $thisVarList $i]
            set f [lindex $fmtList     $i]

            set val [getNamedItemD [string tolower $v] $pk 0.0]

            puts -nonewline [format " $f" $val]
           }

        puts ""

        incr currentID 
       }
}

proc findOverlap2D { x0 y0 thisList } \
{
   global dx dy

   set loc 0

   foreach pk $thisList \
      {
       if {abs($x0 - [lindex $pk 1]) <= $dx && abs($y0 - [lindex $pk 3]) <= $dy} {return $loc}
       incr loc
      }

   return -1
}

proc findOverlap3D { x0 y0 z0 thisList } \
{
   global dx dy dz

   set loc 0

   foreach pk $thisList \
      {
       if {abs($x0 - [lindex $pk 1]) <= $dx && abs($y0 - [lindex $pk 3]) <= $dy && abs($z0 - [lindex $pk 5]) < $dz} {return $loc}
       incr loc
      }

   return -1
}

proc findXYPos { i thisThresh } \
{
   global maxTrials xyMode xMode yMode xPosList yPosList pkList avoidList pkCout x1Reg xnReg y1Reg ynReg xRange yRange xName yName zName pkCount
   global xListAux yListAux forceXOverlapFlag forceYOverlapFlag dxOv dyOv

   set xPos None
   set yPos None

   if {![strcasecmp $xyMode tab]} \
      {
       set nX   [llength $xListAux]
       set nY   [llength $yListAux]

       if {!$nX || !$nY} \
          {
           showErrorMessage "GenSimTab Error: No XY Coords Found for xy=tab"
           exit 1
          }

       set loc $i

       while {$loc < 1}   {set loc [expr $loc + $nX]}
       while {$loc > $nX} {set loc [expr $loc - $nX]}

       set xPos [lindex $xListAux [expr $loc - 1]]

       set loc $i

       while {$loc < 1}   {set loc [expr $loc + $nY]}
       while {$loc > $nY} {set loc [expr $loc - $nY]}

       set yPos [lindex $yListAux [expr $loc - 1]]

       return "$xPos $yPos"
      } \
   elseif {![strcasecmp $xyMode Existing]} \
      {
       for {set xyTrial 1} {$xyTrial <= $maxTrials} {incr xyTrial} \
          {
           set loc    [expr int([rand]*$pkCount)]
           set xPos   [lindex $xPosList $loc]
           set yPos   [lindex $yPosList $loc]

           if {$forceXOverlapFlag} \
              {
                if {[rand] > 0.5} \
                   {
                    set xPos [expr $xPos + $dxOv]
                   } \
                else \
                   {
                    set xPos [expr $xPos - $dxOv]
                   }
              }

           if {$forceYOverlapFlag} \
              {
                if {[rand] > 0.5} \
                   {
                    set yPos [expr $yPos + $dyOv]
                   } \
                else \
                   {
                    set yPos [expr $yPos - $dyOv]
                   }
              }

           if {$xPos > $x1Reg || $xPos < $xnReg} continue
           if {$yPos > $y1Reg || $yPos < $ynReg} continue

           if {$forceXOverlapFlag || $forceYOverlapFlag} \
              {
               set loc -1
              } \
           else \
              {
               set loc [findOverlap2D $xPos $yPos $pkList]
              }

           if {$loc < 0} {return "$xPos $yPos"}
 
           set thisPk   [lindex $pkList $loc]
           set thisXPos [lindex $thisPk 1]
           set thisYPos [lindex $thisPk 3]
           set thisZPos [lindex $thisPk 5]
     
           set xyzFmt "(XY Overlap with New Peak %s: % 8.4fppm %s: % 8.4fppm %s: % 8.4fppm)"
           set fmt    " Strip: %2d XY Trial: %4d  Rejecting Peak Position.    %s: % 8.4fppm %s: % 8.4fppm $xyzFmt"

           showMessage [format $fmt $i $xyTrial $xName $xPos $yName $yPos $xName $thisXPos $yName $thisYPos $zName $thisZPos]
          }

       return ""
      } \
   elseif {![strcasecmp $xyMode Any]} \
      {
       for {set xyTrial 1} {$xyTrial <= $maxTrials} {incr xyTrial} \
          {
           set xPos [format "%.4f" [expr $x1Reg - [rand]*$xRange]]
           set yPos [format "%.4f" [expr $y1Reg - [rand]*$yRange]]

           set loc  [findOverlap2D $xPos $yPos $pkList]

           if {$loc < 0} {return "$xPos $yPos"}

           set thisPk   [lindex $pkList $loc]
           set thisXPos [lindex $thisPk 1]
           set thisYPos [lindex $thisPk 3]
           set thisZPos [lindex $thisPk 5]

           set xyzFmt "(XY Overlap with New Peak %s: % 8.4fppm %s: % 8.4fppm %s: % 8.4fppm)"
           set fmt    " Strip: %2d XY Trial: %4d  Rejecting Peak Position.    %s: % 8.4fppm %s: % 8.4fppm $xyzFmt"

           showMessage [format $fmt $i $xyTrial $xName $xPos $yName $yPos $xName $thisXPos $yName $thisYPos $zName $thisZPos]
          }

       return ""
      } 

   if {![strcasecmp $xMode Existing]} \
      {
       set loc    [expr int([rand]*$pkCount)]
       set xPos   [lindex $xPosList $loc]
      } \
   elseif {![strcasecmp $xMode Any]} \
      {
       set xPos [format "%.4f" [expr $x1Reg - [rand]*$xRange]]
      }

   if {![strcasecmp $yMode Existing]} \
      {
       set loc    [expr int([rand]*$pkCount)]
       set yPos   [lindex $yPosList $loc]
      } \
   elseif {![strcasecmp $yMode Any]} \
      {
       set yPos [format "%.4f" [expr $y1Reg - [rand]*$yRange]]
      } 

   for {set xyTrial 1} {$xyTrial <= $maxTrials} {incr xyTrial} \
      { 
       if {![strcasecmp $xyMode Novel]} \
          {
           set xPos [format "%.4f" [expr $x1Reg - [rand]*$xRange]]
           set yPos [format "%.4f" [expr $y1Reg - [rand]*$yRange]]
          }

       if {![strcasecmp $xMode Novel]} \
          {         
           set xPos [format "%.4f" [expr $x1Reg - [rand]*$xRange]]
          }

       if {![strcasecmp $yMode Novel]} \
          {
           set yPos [format "%.4f" [expr $y1Reg - [rand]*$yRange]]
          }

       if {![strcasecmp $xPos None]} {set xPos [format "%.4f" [expr $x1Reg - [rand]*$xRange]]}
       if {![strcasecmp $yPos None]} {set yPos [format "%.4f" [expr $y1Reg - [rand]*$yRange]]}

       if {$xPos > $x1Reg || $xPos < $xnReg} continue
       if {$yPos > $y1Reg || $yPos < $ynReg} continue

       readROI -ndim 1 -roi 4 -inROI 2 -dx X_AXIS ${xPos}ppm 0 -dy Y_AXIS ${yPos}ppm 0

       set val [lindex [getROIVals 4] 0]

       if {$val <= $thisThresh} \
          {
           set fmt " Strip: %2d XY Trial: %4d  Rejecting Mask Position.    %s: % 8.4fppm %s: % 8.4fppm Mask Value: %.3f Threshold: %.3f"
           showMessage [format $fmt $i $xyTrial $xName $xPos $yName $yPos $val $thisThresh]
           continue
          }

       set fmt " Strip: %2d XY Trial: %4d  Accepting Mask Position.    %s: % 8.4fppm %s: % 8.4fppm"
       showMessage [format $fmt $i $xyTrial $xName $xPos $yName $yPos]

       set loc  [findOverlap2D $xPos $yPos $pkList]

       if {$loc >= 0} \
          {
           set thisPk   [lindex $pkList $loc]
           set thisXPos [lindex $thisPk 1]
           set thisYPos [lindex $thisPk 3]
           set thisZPos [lindex $thisPk 5]

           set xyzFmt "(XY Overlap with New Peak %s: % 8.4fppm %s: % 8.4fppm %s: % 8.4fppm)"
           set fmt    " Strip: %2d XY Trial: %4d  Rejecting Peak Position.    %s: % 8.4fppm %s: % 8.4fppm $xyzFmt"

           showMessage [format $fmt $i $xyTrial $xName $xPos $yName $yPos $xName $thisXPos $yName $thisYPos $zName $thisZPos]
           continue
          }

        set loc  [findOverlap2D $xPos $yPos $avoidList]

        if {$loc >= 0} \
           {
            set thisPk   [lindex $avoidList $loc]
            set thisXPos [lindex $thisPk 1]
            set thisYPos [lindex $thisPk 3]
            set thisZPos [lindex $thisPk 5]

            set xyzFmt "(XY Overlap with Peak to Avoid %s: % 8.4fppm %s: % 8.4fppm %s: % 8.4fppm)"
            set fmt    " Strip: %2d XY Trial: %4d  Rejecting Peak Position.    %s: % 8.4fppm %s: % 8.4fppm $xyzFmt"

            showMessage [format $fmt $i $xyTrial $xName $xPos $yName $yPos $xName $thisXPos $yName $thisYPos $zName $thisZPos]
            continue
           }

       return "$xPos $yPos"
      }

   return ""
}

proc findStripPeaks { xPos yPos stripPkCount stripID progID } \
{
    global pkList avoidList pkCount maxTrials specType xName yName zName zPosList zMode z1Reg znReg zRange
    global zListAux noSpecOverlapFlag noNewOverlapFlag forceZOverlapFlag dzOv

    set stripList ""

    if {![strcasecmp $zMode tab]} \
       {
        set nZ [llength $zListAux]

        if {!$nZ} \
           {
            showErrorMessage "GenSimTab Error: No Z Coords Found for z=tab"
            exit 1
           }

        for {set loc 1} {$loc <= $stripPkCount} {incr loc} \
           {
            while {$loc < 1}   {set loc [expr $loc + $nX]}
            while {$loc > $nZ} {set loc [expr $loc - $nZ]}

            if {$loc == 1 && (![strcasecmp $specType N15NOE] || ![strcasecmp $specType C13NOE])} \
               {
                set zPos $xPos
               } \
            else \
               {
                set zPos [lindex $zListAux [expr $loc - 1]]
               }

            lappend stripList "x_ppm $xPos y_ppm $yPos z_ppm $zPos prog $progID strip $stripID peak $loc"
           }

        return $stripList
       }
 
    for {set j 1} {$j <= $stripPkCount} {incr j} \
       {
        set foundFlag 0

        if {$j == 1 && (![strcasecmp $specType N15NOE] || ![strcasecmp $specType C13NOE])} \
           {
            set zTrialCount 1
           } \
        else \
           {
            set zTrialCount $maxTrials
           }

        for {set zTrial 1} {$zTrial <= $maxTrials} {incr zTrial} \
           {
            if {$j == 1 && (![strcasecmp $specType N15NOE] || ![strcasecmp $specType C13NOE])} \
               {
                set zPos $xPos
               } \
            else \
               {
                if {![strcasecmp $zMode tab]} \
                   {
                    set nZ [llength $zListAux]

                    if {!$nZ} \
                       {
                        showErrorMessage "GenSimTab Error: No Z Coords Found for z=tab"
                        exit 1
                       }

                    set loc $i

                    while {$loc < 1}   {set loc [expr $loc + $nX]}
                    while {$loc > $nZ} {set loc [expr $loc - $nZ]}

                    set zPos [lindex $zListAux [expr $loc - 1]]
                   } \
                elseif {![strcasecmp $zMode Existing]} \
                   {
                    set loc  [expr int([rand]*$pkCount)]
                    set zPos [lindex $zPosList $loc]
                   } \
                else \
                   {
                    set zPos [format "%.4f" [expr $z1Reg - [rand]*$zRange]]
                   }
               }

            set n [llength $stripList]

            if {$forceZOverlapFlag && $n >= 1 && ($n % 2)} \
               {
                set zPos [lindex [lindex $stripList end] 5]

                if {[rand] > 0.5} \
                   {
                    set zPos [expr $zPos + $dzOv]
                   } \
                else \
                   {
                    set zPos [expr $zPos - $dzOv]
                   }
               }

            if {$zPos > $z1Reg || $zPos < $znReg} continue

            if {$noNewOverlapFlag} \
               {
                set loc [findOverlap3D $xPos $yPos $zPos $pkList]

                if {$loc >= 0} \
                   {
                    set xyzFmt "(Overlaps New Peak %s: % 8.4fppm %s: % 8.4fppm %s: % 8.4fppm)"
                    set fmt    "  Z Trial: %4d Peak: %d Rejecting Position:  %s % 8.4fppm  $xyzFmt"

                    set thisPk   [lindex $pkList $loc]
                    set thisXPos [lindex $thisPk 1]
                    set thisYPos [lindex $thisPk 3]
                    set thisZPos [lindex $thisPk 5]

                    showMessage [format $fmt $zTrial $j $zName $zPos $xName $thisXPos $yName $thisYPos $zName $thisZPos]
                    continue
                   }

                if {$forceZOverlapFlag} \
                   {
                    set loc -1
                   } \
                else \
                   {
                    set loc [findOverlap3D $xPos $yPos $zPos $stripList]
                   }

                if {$loc >= 0} \
                   {
                    set xyzFmt "(Overlaps Proposed Peak %s: % 8.4fppm %s: % 8.4fppm %s: % 8.4fppm)"
                    set fmt    "  Z Trial: %4d Peak: %d Rejecting Position: %s % 8.4fppm   $xyzFmt"

                    set thisPk   [lindex $stripList $loc]
                    set thisXPos [lindex $thisPk 1]
                    set thisYPos [lindex $thisPk 3]
                    set thisZPos [lindex $thisPk 5]

                    showMessage [format $fmt $zTrial $j $zName $zPos $xName $thisXPos $yName $thisYPos $zName $thisZPos]
                    continue
                   }
               }

            if {$noSpecOverlapFlag} \
               {
                readROI -ndim 1 -roi 5 -inROI 1 -dx X_AXIS ${xPos}ppm 0 -dy Y_AXIS ${yPos}ppm 0 -dz Z_AXIS ${zPos}ppm

                set val [lindex [getROIVals 5] 0]

                if {$val < 1.0} \
                   {
                    set fmt " Strip: %2d Z Trial: %4d  Rejecting Mask Position.    %s: % 8.4fppm %s: % 8.4fppm %s: % 8.4fppm"
                    showMessage [format $fmt $stripID $zTrial $xName $xPos $yName $yPos $zName $zPos]
                    continue
                   }
               }

            set loc [findOverlap3D $xPos $yPos $zPos $avoidList]

            if {$loc >= 0} \
               {
                set xyzFmt "(Overlaps Peak to Avoid %s: % 8.4fppm %s: % 8.4fppm %s: % 8.4fppm)"
                set fmt    "  Z Trial: %4d Peak: %d Rejecting Position:  %s % 8.4fppm  $xyzFmt"

                set thisPk   [lindex $avoidList $loc]
                set thisXPos [lindex $thisPk 1]
                set thisYPos [lindex $thisPk 3]
                set thisZPos [lindex $thisPk 5]

                showMessage [format $fmt $zTrial $j $zName $zPos $xName $thisXPos $yName $thisYPos $zName $thisZPos]
                continue
               }

            showMessage [format "  Z Trial: %4d Peak: %d Accepting Position: %s % 8.4fppm" $zTrial $j $zName $zPos]

            lappend stripList "x_ppm $xPos y_ppm $yPos z_ppm $zPos prog $progID strip $stripID peak $j"
            incr foundFlag 1
            break
           }

        if {!$foundFlag} {return $stripList}
       }

    return $stripList
}

proc getValInRange { minVal maxVal } \
{
   if {$minVal == $maxVal} {return $minVal}

   set mid [expr 0.5*($maxVal + $minVal)]
   set val [expr $mid + 0.35*($maxVal - $minVal)*[rand -gauss]]

   if {$val < $minVal} {set val $minVal}
   if {$val > $maxVal} {set val $maxVal}

   return $val
}

proc addStripParams { stripList } \
{
   global p0Min p0Max p1Min p1Max p0MinAlt p0MaxAlt lwMin lwMax jValMin jValMax hiMin hiMax hiMinAlt hiMaxAlt hiScale jCount specType xzyFlag
   global fdataFID fdataFT

   set n [llength $stripList]

   for {set i 1} {$i <= $n} {incr i} {set pkInfo($i) [lindex $stripList [expr $i - 1]]}
         
   foreach dim {x y z} \
      {
       set outDim $dim

       if {$xzyFlag} \
          {
           if {![strcasecmp $dim y]} \
              {
               set outDim z
              } \
           elseif {![strcasecmp $dim z]} \
              {
               set outDim y
              }
          }

       set thisP0    [format "%.1f" [getValInRange $p0Min($dim)    $p0Max($dim)]]
       set thisP0Alt [format "%.1f" [getValInRange $p0MinAlt($dim) $p0MaxAlt($dim)]]
       set thisP1    [format "%.1f" [getValInRange $p1Min($dim)    $p1Max($dim)]]

       set thisLWHz  [format "%.4f" [getValInRange $lwMin($dim)    $lwMax($dim)]]
       set thisLWPts [format "%.4f" [spec2pntW $fdataFID $outDim ${thisLWHz}Hz]]

       set thisOff   [getParmI $fdataFT NDX1 $outDim]

       if {$thisOff > 0} {set thisOff [expr $thisOff - 1]}

       for {set j 1} {$j <= $jCount} {incr j} \
          {
           set thisJVal($j) [format "%.3f" [getValInRange $jValMin($dim,$j) $jValMax($dim,$j)]]
          }

       for {set i 1} {$i <= $n} {incr i} \
          {
           set thisPPM [getNamedItemD ${dim}_ppm $pkInfo($i) 0.0]
           set thisPts [format "%.4f" [expr $thisOff + [spec2pnt $fdataFT $outDim ${thisPPM}ppm]]]

           set thisX1  [expr int($thisPts + 0.5) - int(2.0*$thisLWPts + 1)]
           set thisX3  [expr int($thisPts + 0.5) + int(2.0*$thisLWPts + 1)]

           set pkInfo($i) [concat $pkInfo($i) ${dim}_axis $thisPts ${dim}1 $thisX1 ${dim}3 $thisX3 ${dim}tw_hz $thisLWHz ${dim}tw $thisLWPts]

           if {![strcasecmp $specType CBCANH] && $i > 2} \
              {
               set pkInfo($i) [concat $pkInfo($i) ${dim}_p0 $thisP0Alt]
              } \
           else \
              {
               set pkInfo($i) [concat $pkInfo($i) ${dim}_p0 $thisP0]
              }

           if {$thisP1 != 0.0} {concat $pkInfo($i) ${dim}_p1 $thisP1}

           for {set j 1} {$j <= $jCount} {incr j} \
              {
               if {$thisJVal($j) != 0.0} \
                  {
                   set jValPts    [format %.3f [spec2pntW $fdataFID $outDim $thisJVal($j)Hz]]
                   set pkInfo($i) [concat $pkInfo($i) ${dim}_cosj${j}_hz $thisJVal($j) ${dim}_cosj$j $jValPts]
                  }
              }

           if {![strcasecmp $dim z]} \
              {
               if {$i == 1 && (![strcasecmp $specType N15NOE] || ![strcasecmp $specType C13NOE])} \
                  {
                   set hi [expr $hiScale*[getValInRange $hiMinAlt $hiMaxAlt]]
                  } \
               else \
                  {
                   set hi [expr $hiScale*[getValInRange $hiMin $hiMax]]
                  }

               if {![strcasecmp $specType CBCANH] && $i > 2} {set hi [expr -1.0*$hi]}

               set pkInfo($i) [concat $pkInfo($i) height $hi]
              }
          }
      }

   set newStripList ""

   for {set i 1} {$i <= $n} {incr i} {lappend newStripList $pkInfo($i)}

   return $newStripList
}

proc showHNTab { xyList } \
{
   global hnName

   if {![strcasecmp $hnName None]} return

   set error [catch {set outUnit [open $hnName w]}]

   puts $outUnit "VARS   INDEX X_PPM Y_PPM ASS"
   puts $outUnit "FORMAT %4d  %8.4f %8.4f %s"
   puts $outUnit ""

   set id 1

   set fmt " %4d  %8.4f %8.4f strip_%s,prog_%s"

   foreach pk $xyList \
      {
       puts $outUnit [format $fmt $id [getNamedItem x_ppm $pk] [getNamedItem y_ppm $pk] [getNamedItem strip $pk] [getNamedItem prog $pk]]
       incr id
      }

   catch "close $outUnit"
}

set currentID $firstID
set avoidList ""
set pkList    ""
set xyList    ""
set xListAux  ""
set yListAux  ""
set zListAux  ""

foreach f "$fidName $ftName $emptyName" \
   {
    if {![file exists [format $f 1 1]]} \
       {
        showErrorMessage "GenSimTab Error Finding Input Data $f"
        exit 1
       }
   }

if {[strcasecmp $xyTabName None]} \
   {
    if {![file exists $xyTabName]} \
       {
        showErrorMessage "GenSimTab Error Finding XY Table $xyTabName"
        exit 1
       }

    set xyTab    [gdbRead table -i $xyTabName]

    set xListAux [getTabVals $xyTab ${xVarDim}_PPM]
    set yListAux [getTabVals $xyTab ${yVarDim}_PPM]
   }

if {[strcasecmp $zTabName None]} \
   {
    if {![file exists $zTabName]} \
       {
        showErrorMessage "GenSimTab Error Finding Z Table $zTabName"
        exit 1
       }

    set zTab     [gdbRead table -i $xyTabName]
    set zListAux [getTabVals $zTab ${zVarDim}_PPM]
   }

set error [rdFDATA $ftName  fdataFT]
set error [rdFDATA $fidName fdataFID]

readROI -roi 1 -ndim 3 -in $emptyName -X $xSpecDim 0% 100% -y $ySpecDim 0% 100% -z $zSpecDim 0% 100%
readROI -roi 2 -ndim 2 -in $emptyName -X $xSpecDim 0% 100% -y $ySpecDim 0% 100% -z $zSpecDim 0% 100%

readROI -roi 3 -ndim 3 -in $ftName \
        -x $xSpecDim  [lindex $regList 0] [lindex $regList 1] \
        -y $ySpecDim  [lindex $regList 2] [lindex $regList 3] \
        -z $zSpecDim  [lindex $regList 4] [lindex $regList 5] \

if {[strcasecmp $emptyName2D None]} {writeROI -roi 2 -out $emptyName2D -ov}

set roiInfo   [getROIInfo -roi 1]

set rdataFull [lindex $roiInfo 0]
set fdataFull [lindex $roiInfo 1]

set xSize     [lindex [lindex $roiInfo 2] 0]
set ySize     [lindex [lindex $roiInfo 2] 1]
set zSize     [lindex [lindex $roiInfo 2] 2]

set roiInfo   [getROIInfo -roi 3]

set rdataReg  [lindex $roiInfo 0]
set fdataReg  [lindex $roiInfo 1]

set xSizeReg  [lindex [lindex $roiInfo 2] 0]
set ySizeReg  [lindex [lindex $roiInfo 2] 1]
set zSizeReg  [lindex [lindex $roiInfo 2] 2]

set xName   [getParmStr $fdataFull NDLABEL X_AXIS]
set yName   [getParmStr $fdataFull NDLABEL Y_AXIS]
set zName   [getParmStr $fdataFull NDLABEL Z_AXIS]

set x1      [pnt2spec $fdataFull X_AXIS 1 ppm]
set y1      [pnt2spec $fdataFull Y_AXIS 1 ppm]
set z1      [pnt2spec $fdataFull Z_AXIS 1 ppm]

set xn      [pnt2spec $fdataFull X_AXIS $xSize ppm]
set yn      [pnt2spec $fdataFull Y_AXIS $ySize ppm]
set zn      [pnt2spec $fdataFull Z_AXIS $zSize ppm]

set x1Reg   [pnt2spec $fdataReg X_AXIS 1 ppm]
set y1Reg   [pnt2spec $fdataReg Y_AXIS 1 ppm]
set z1Reg   [pnt2spec $fdataReg Z_AXIS 1 ppm]

set xnReg   [pnt2spec $fdataReg X_AXIS $xSizeReg ppm]
set ynReg   [pnt2spec $fdataReg Y_AXIS $ySizeReg ppm]
set znReg   [pnt2spec $fdataReg Z_AXIS $zSizeReg ppm]

set pkTab   [gdbRead table -in $tabName]
set eList   [gdbGetInfo $pkTab -parms entries]
set pkCount [llength $eList]

set eListAvoid ""

if {[strcasecmp $avoidName None]} \
   {
    if {![file exists $avoidName]} \
       {
        showErrorMessage "GenSimTab Error Finding Peak Table $avoidName"
        exit 1
       }

    set avoidTab   [gdbRead table -in $avoidName]
    set eListAvoid [gdbGetInfo $avoidTab -parms entries]
  
    foreach e $eListAvoid \
       {
        lappend avoidList "x_ppm [gdbGet $e X_PPM] y_ppm [gdbGet $e Y_PPM] z_ppm [gdbGet $e Z_PPM]"
       } 
   }

set idList   [getTabVals $pkTab INDEX]
set xPosList [getTabVals $pkTab ${xVarDim}_PPM]
set yPosList [getTabVals $pkTab ${yVarDim}_PPM]
set zPosList [getTabVals $pkTab ${zVarDim}_PPM]

showMessage "Full Spectrum $ftName:"
showMessage [format " X-Axis: %-4s Size: %4d pts % 8.3fppm % 8.3fppm" $xName $xSize $x1 $xn]
showMessage [format " Y-Axis: %-4s Size: %4d pts % 8.3fppm % 8.3fppm" $yName $ySize $y1 $yn]
showMessage [format " Z-Axis: %-4s Size: %4d pts % 8.3fppm % 8.3fppm" $zName $zSize $z1 $zn]

showMessage "Allowed Simulation Region ($regList):"
showMessage [format " X-Axis: %-4s Size: %4d pts % 8.3fppm % 8.3fppm" $xName $xSizeReg $x1Reg $xnReg]
showMessage [format " Y-Axis: %-4s Size: %4d pts % 8.3fppm % 8.3fppm" $yName $ySizeReg $y1Reg $ynReg]
showMessage [format " Z-Axis: %-4s Size: %4d pts % 8.3fppm % 8.3fppm" $zName $zSizeReg $z1Reg $znReg]

set xRange [expr $x1Reg - $xnReg]
set yRange [expr $y1Reg - $ynReg]
set zRange [expr $z1Reg - $znReg]

if {![strcasecmp $thresh Auto]} {set thresh [expr 1.0 - 1.0/($zSize + 1)]}

showMessage ""
showMessage [format "Mask Threshold: %.4f" $thresh]

if {$hdrFlag} showHdr
if {$hdrFlag == 2} {exit 0}

set totalStrips 0
set stripID     0
set progID      0

set hiMinOrig   $hiMin
set hiMaxOrig   $hiMax
set hiScaleOrig $hiScale
set iseedOrig   $iseed
set dzOrig      $dz

foreach dim {x y z} \
   {
    set lwMinOrig($dim) $lwMin($dim)
    set lwMaxOrig($dim) $lwMax($dim)
   }

foreach pInfo $progList \
   {
    incr progID

    set stripCount 10

    set xyMode     Novel
    set xMode      None
    set yMode      None
    set zMode      Existing

    set hiMin      $hiMinOrig
    set hiMax      $hiMaxOrig
    set hiScale    $hiScaleOrig

    set noNewOverlapFlag  1
    set noSpecOverlapFlag 1

    set forceXOverlapFlag 0
    set forceYOverlapFlag 0
    set forceZOverlapFlag 0

    set PARGV "genSimTab [expandArg $pInfo]"

    if {[flagLoc $PARGV -iseed]} \
       {
        getArgD $PARGV -iseed iseed
        srand $iseed
       }

    foreach dim {x y z} \
       {
        set lwMin($dim) $lwMinOrig($dim) 
        set lwMax($dim) $lwMaxOrig($dim)

        getArgD $PARGV -${dim}wMin lwMin($dim)
        getArgD $PARGV -${dim}wMin lwMax($dim)
        getArgD $PARGV -${dim}wMax lwMax($dim)
       }

    getArgD $PARGV -count stripCount

    getArgD $PARGV -xy    xyMode
    getArgD $PARGV -x     xMode
    getArgD $PARGV -y     yMode
    getArgD $PARGV -z     zMode

    getArgD $PARGV -dxOv  dxOv
    getArgD $PARGV -dyOv  dyOv
    getArgD $PARGV -dzOv  dzOv

    getArgD $PARGV -hiMin   hiMin
    getArgD $PARGV -hiMin   hiMax
    getArgD $PARGV -hiMax   hiMax

    getArgD $PARGV -hiMinAlt hiMinAlt
    getArgD $PARGV -hiMinAlt hiMaxAlt
    getArgD $PARGV -hiMaxAlt hiMaxAlt

    getArgD $PARGV -hiScale hiScale

    if {[flagLoc $PARGV -noNewOverlap]} {set noNewOverlapFlag 1}
    if {[flagLoc $PARGV -newOverlapOK]} {set noNewOverlapFlag 0}

    if {[flagLoc $PARGV -noSpecOverlap]} {set noSpecOverlapFlag 1}
    if {[flagLoc $PARGV -specOverlapOK]} {set noSpecOverlapFlag 0}

    if {[flagLoc $PARGV -forceXOverlap]} {set forceXOverlapFlag 1}
    if {[flagLoc $PARGV -forceYOverlap]} {set forceYOverlapFlag 1}
    if {[flagLoc $PARGV -forceZOverlap]} {set forceZOverlapFlag 1}

    if {![strcasecmp $stripCount All]}   {set stripCount [getMax "[llength $xListAux] [llength $yListAux] [llength $zListAux]"]}

    if {[strcasecmp $xMode None] || [strcasecmp $yMode None]} \
       {
        if {[flagLoc $PARGV -xy]} \
           {
            showErrorMessage "GenSimTab Error:    Strip Program Contains xy mixed with x and y specification."
            showErrorMessage "Strip Program Info: $pInfo ($PARGV)"
            exit 1
           }

        set xyMode None
       }

    if {![strcasecmp $xyMode Any]} \
       {
        set xyMode None
        set xMode  Any
        set yMode  Any
       }

    if {$commentFlag} {showProg $progID}

    for {set i 1} {$i <= $stripCount} {incr i} \
       {
        incr stripID

        showMessage "Program: $progID Strip $i Strip ID: $stripID"

        set stripPkCount [expr $minPkPerStrip + int([rand]*(1 + $maxPkPerStrip - $minPkPerStrip))]
        set stripList    ""

        for {set xyzTrial 1} {$xyzTrial <= $maxTrials} {incr xyzTrial} \
           {
            set xyInfo [findXYPos $i $thresh]

            if {![llength $xyInfo]} \
               {
                showErrorMessage [format "GenSimTab Error. Program: %d Strip: %2d Failed to Generate XY Position." $progID $stripID]
                exit 1
               }

            set xPos [lindex $xyInfo 0]
            set yPos [lindex $xyInfo 1]

            set stripList [findStripPeaks $xPos $yPos $stripPkCount $stripID $progID]

            if {[llength $stripList] == $stripPkCount} \
               {
                lappend xyList "x_ppm $xPos y_ppm $yPos strip $stripID prog $progID"

                set stripList [addStripParams $stripList]

                showPkTab $stripList

                foreach pk $stripList \
                   {
                    showMessage "Final Peak for Strip $stripID: $pk"
                    lappend pkList $pk
                   }

                break
               }
           }

        if {[llength $stripList] != $stripPkCount} \
           {
            showErrorMessage [format "GenSimTab Error. Program: %d Strip: %2d Failed to Generate Peaks." $progID $stripID]
            exit 1
           }
       }
   }

showHNTab $xyList

exit 0 
