#!/bin/csh

#
# Directories
# Specific files.
# Extensions

set nonomatch=1

set dList = (base dat data dif empty fid qfid ft qft ft1 ft2 ft3 ftproj hms_ist hms_ist_nuszf hms_ist_nonuszf hmsist hmsist hmsist_nuszf hmsist_nonuszf pcalp inspect interp ist ist_lp istmult \
             ist_nuszf ist_nonuszf istres istspec isttmp lp mask maxent maxent_nuszf maxent_nonuszf mem ml nesta nesta_nuszf nesta_nonuszf nus nuszf nonuszf nusproj \
             rist rist_nuszf rist_nonuszf sim simFID smile tmp tmpax tmpaux snap snapPCA xyz xzy yzx)

set fList = (apod.x apod.y apod.z apod.a apod.list axt.tab fake.nuslist fid_compress fid_final fid_full     \
             hdr.out hn.proj.tab mask_full nlin.spec.list nuslist_zf pk.tcl ser_compress ser_final ser_full \
             sim.spec.list test.tab test.tab.old smile.log resample.log conv.out proc.out conv.log proc.log fid.info)

set eList = (.dat .fid .ft1 .ft2 .ft3 .ft4 .ft .tmp .old %)

set suffixList = None

@ nD = 0

if (`flagLoc $argv -help`) then
   echo "Delete Existing NMRPipe Results:"
   echo " -dList   dList [None] Additional Directories to Delete."
   echo " -fList   fList [None] Additional Files to Delete."
   echo " -eList   eList [None] Additional Extenstions of Files to Delete."
   echo " -dSuffix sList [None] Additional Suffixes of Directories to Delete."
   echo " -fSuffix sList [None] Additional Suffixes of Files to Delete."
   echo "Delete PCA Output Directories (pca001 ...):"
   echo " -pca                Delete PCA Directories (Default)."
   echo " -nopca              Do Not Delete PCA Directories."
   echo "Suppressing Deletion:"
   echo " -dSkip dSkip [None] Directories Not to Delete."
   echo " -fSkip fSkip [None] Directories Not to Delete."
   echo " -eSkip eSkip [None] Extensions Not to Delete."
   echo ""
   echo "Default Directories Deleted:"
  
   @ n = $#dList
   @ i = 0
   @ j = 0
 
   foreach d ($dList)
      nmrPrintf " %-16s" $d
      @ i++
      @ j++

      if ($i == 4 || $j == $n) then
         echo ""
         @ i = 0
      endif
   end

   echo ""
   echo "Default Files Deleted:"

   @ n = $#fList
   @ i = 0
   @ j = 0

   foreach f ($fList)
      nmrPrintf " %-16s" $f
      @ i++
      @ j++

      if ($i == 4 || $j == $n) then
         echo ""
         @ i = 0
      endif
   end

   echo ""
   echo "Default Extensions of Files Deleted:"

   @ n = $#eList
   @ i = 0
   @ j = 0

   foreach e ($eList)
      nmrPrintf " %-16s" $e
      @ i++
      @ j++

      if ($i == 4 || $j == $n) then
         echo ""
         @ i = 0
      endif
   end

   exit 0
endif

set dListExtra  = (`getListArgD $argv -dList   -- None`)
set fListExtra  = (`getListArgD $argv -fList   -- None`)
set eListExtra  = (`getListArgD $argv -eList   -- None`)
set dSuffixList = (`getListArgD $argv -dSuffix -- None`)
set fSuffixList = (`getListArgD $argv -fSuffix -- None`)

set dListSkip  = (`getListArgD $argv -dSkip -- None`)
set fListSkip  = (`getListArgD $argv -fSkip -- None`)
set eListSkip  = (`getListArgD $argv -eSkip -- None`)

set pcaFlag = 1

if (`flagLoc $argv -pca`) then
   set pcaFlag = 1
endif

if (`flagLoc $argv -nopca`) then
   set pcaFlag = 0
endif

if (`string csame $dListExtra None`) then
   set dListExtra = ""
endif

if (`string csame $fListExtra None`) then
   set fListExtra = ""
endif

if (`string csame $eListExtra None`) then
   set eListExtra = ""
endif

if (`string csame $dSuffixList None`) then
   set dSuffixList = ""
endif

if (`string csame $fSuffixList None`) then
   set fSuffixList = ""
endif

set dListFull = ($dList $dListExtra)
set fListFull = ($fList $fListExtra)
set eListFull = ($eList $eListExtra)
set origDir   = (`pwd`)

foreach d (*)
   foreach s ($dSuffixList)
      if (-d "$d" && `string match \*$s $d`) then
         echo "Adding Directory $d matching suffix $s ..."
         set dListFull = ($dListFull $d)
         break
      endif    
   end
end

foreach f (*)
   foreach s ($fSuffixList)
      if (-e "$f" && !(-d "$f") && `string match \*$s $f`) then
         echo "Adding File $f matching suffix $s ..."
         set fListFull = ($fListFull "$f")
         break
      endif
   end
end

if (`string csame $dListSkip None`) then
   set dList = ($dListFull)
else
   set dList = ""

   foreach d ($dListFull)
      if (`string lsearch $dListSkip $d`) then
         echo "Skipping Directory $d"
         continue
      endif

      set dList = ($dList $d) 
   end
endif

if (`string csame $fListSkip None`) then
   set fList = ($fListFull)
else
   set fList = ""

   foreach f ($fListFull)
      if (`string lsearch $fListSkip $f`) then
         echo "Skipping File $f"
         continue
      endif
   
      set fList = ($fList $f) 
   end
endif

if (`string csame $eListSkip None`) then
   set eList = ($eListFull)
else
   set eList = ""
      
   foreach e ($eListFull)
      if (`string lsearch $eListSkip $e`) then
         echo "Skipping Extension $e"
         continue
      endif

      set eList = ($eList $e)
   end
endif

foreach d ($dList)
   if (-d "$d") then
      if (`string same "$d" data`) then
         echo "Special Case: Testing Directory $d ..."

         set thisList = ($d/*)

         @ nn = 0

         foreach thisName ($thisList)
            if (-d "$d"/$thisName) then
               @ nn++
            else
               set thisExt = (`string ext $thisName`)

               if (`string clsearch fid ft1 ft2 ft3 ft4 dat $thisExt` == 0) then
                  @ nn++
               endif
            endif
         end

         echo " Total Files in Directory:       $#thisList"
         echo " Files Without Known Extensions: $nn"

         if ($nn != 0) then
            echo " Skipping Directory $d ..."
            continue
         endif
      endif

      echo "Deleting Directory $origDir/$d"
      /bin/rm -rf "$d"
      @ nD++
   endif
end

foreach f ($fList)
   if ((-e "$f") && !(-d "$f")) then
      echo "Deleting File $origDir/$f"
      /bin/rm -f "$f"
      @ nD++
   endif
end

foreach f (*)
   if (-d "$f") then
      continue
   endif

   foreach e ($eList)
      set ee = (`string ext "$f"`)

      if (`string same "$e" .$ee`) then
         echo "Deleting File $origDir/$f with Extension $e"
         /bin/rm -f "$f"
         @ nD++
      endif
   end
end

if ($pcaFlag) then
   foreach d (`find ./ -maxdepth 1 -type d -name pca\[0-9\]\* -print`)
      if (-d "$d"/pca) then
         echo $d
         /bin/rm -rf "$d"
         @ nD++
      endif
   end

   foreach d (`find ./ -maxdepth 1 -type d -name dist\[0-9\]\* -print`)
      if (-d "$d"/pca) then
         echo $d
         /bin/rm -rf "$d"
         @ nD++
      endif
   end
endif

if ($nD) then
   echo ""
endif
