#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Report the Spectrum With the Largest Statistic:"
   echo " -in   inList [ft/*]     List of Input Spectra."
   echo " -stat stat   [vMaxAbs]  Statistic Function."
   echo " -max                    Select by Maximum Statistic Value (Default)."
   echo " -min                    Select by Minimum Statistic Value."
   echo " -verb                   Report Statistic Value."
   echo "Statistics Vector Function Names (see specStat.com -help):"
   echo " vSum vSumSq vSumPos   vSumNeg vAvg    vStdDev vVariance"
   echo " vMin vMax   vMaxAbs   vMinAbs vMedian vRMS    vEstNoise"
   echo " N    ndim   istMaxRes"
   exit 0
endif

set inList = (`getListArgD $argv -in -- Auto`)

if (`string csame $inList Auto`) then
   if (-d ft) then
      set inList = (ft/*)
   else
      set inList = ""
   endif
endif

if ($#inList == 0) then
   exit 0
endif

set statName = (`getArgD $argv -stat vMaxAbs`)
set cmpStr   = ">"

if (`flagLoc $argv -min`) then
   set cmpStr = "<"
endif

set bestSpec = $inList[1]
set bestVal  = 0.0

foreach inName ($inList)
   set rms = (`specStat.com -in $inName -brief -stat $statName`)

   if (`IMATH "$rms $cmpStr $bestVal"`) then
      set bestSpec = $inName
      set bestVal  = $rms
   endif
end

if (`flagLoc $argv -verb`) then
   nmrPrintf "%s %s" $bestSpec $bestVal
else
   nmrPrintf %s $bestSpec
endif

exit 0
