#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "List Files, Directories, and NMRPipe Templates:"
    puts " -cd   cdDir \[.\]                  Directory to List."
    puts " -glob pList \[*.fid *.ft* *.dat\]  Patterns to Include."
    puts "Notes:"
    puts " 1. Output will contain a header line, followed by files,"
    puts "    directories, and templates, one per line."
    puts " 2. The header line contains exactly six items, giving the"
    puts "    number of files, directories, and templates listed, for"
    puts "    example:"
    puts ""
    puts "    fileCount 128 dirCount 2 templateCount 2"
    exit 0
   }

set globList "*.fid *.ft* *.dat"
set cdDir    .

getListArgD $ARGV -glob globList
getArgD     $ARGV -cd   cdDir

set globList [listFlatten $globList]

#
#----------------------------------------------------------------------------------

proc splitName { thisName } \
{
    set n [string length $thisName]

    set sInfo ""
    set s     ""

    set wasDigit 0
    
    for {set i 0} {$i < $n} {incr i} \
       {
        set c [string index $thisName $i]

        if {[string is digit $c]} \
           {
            if {!$wasDigit} {append s " "}
            set wasDigit 1
           } \
        else \
           {
            if {$wasDigit} {append s " "}
            set wasDigit 0
           }

        append s $c
       }

    foreach i $s {lappend sInfo $i}

    return $sInfo
}

proc getFmtTemplate { nInfo fmtLoc fmt } \
{
   set t ""
   set i 0

   foreach item $nInfo \
      {
       if {$i == $fmtLoc} \
          {
           append t $fmt
          } \
       else \
          {
           append t $item
          }

       incr i
      }
    
  return $t 
}

proc getFmt1Loc { nInfo fmtPtr } \
{
   upvar $fmtPtr fmt

   set fmt ""

   foreach fmt {%06d %05d %04d %03d %02d} \
      {
       set i   [format $fmt 1]
       set loc [lsearch -exact $nInfo $i]
       if {$loc >= 0} {return $loc}
      }

   set fmt ""

   return -1
}

proc getFmt2Loc { nInfo fmtPtr } \
{
   upvar $fmtPtr fmt

   set fmt ""

   foreach fmt {%05d%05d %04d%04d %03d%04d %03d%03d %02d%03d} \
      {
       set i   [format $fmt 1 1]
       set loc [lsearch -exact $nInfo $i]
       if {$loc >= 0} {return $loc}
      }

   set fmt ""

   return -1
}

proc getFmt1Matches { t fList } \
{
   global zMax

   set n [llength $fList]
   set mList ""

   for {set i 1} {$i <= $n} {incr i} \
      {
       set f [format $t $i]
       if {[lsearch -exact $fList $f] < 0} break

       lappend mList $f
       set zMax($t) $i
      }

   return $mList 
}

proc getFmt2Matches { t fList } \
{
   global zMax aMax

   set n [llength $fList]
   set mList ""

   for {set j 1} {$j <= $n} {incr j} \
      {
       set mCount 0

       for {set i 1} {$i <= $n} {incr i} \
          {
           set f [format $t $j $i]
           if {[lsearch -exact $fList $f] < 0} break
    
           lappend mList $f
           incr mCount

           set zMax($t) $i
          }

       if {!$mCount} break

       set aMax($t) $j
      }

   return $mList
}
 
set origDir [pwd]

if {[strcmp $cdDir .]} {cd $cdDir}

set dirList      ""
set fileList     ""
set templateList ""
set t1List       ""
set t2List       ""

set allList    [lsort -ascii [glob -nocomplain *]]
set unusedList ""

foreach i $allList \
   {
    if {[file isdirectory $i]} \
       {
        lappend dirList $i
        continue
       } 

    lappend unusedList $i
   }

set allList  $unusedList

foreach i $allList \
   {
    foreach p $globList \
       {
        if {[string match $p $i]} \
           {
            lappend fileList $i
           }
       }
   }

foreach thisName $fileList \
   {
    set nameInfo [splitName  $thisName]
    set fmtLoc   [getFmt1Loc $nameInfo thisFmt]

    if {$fmtLoc >= 0} \
       {
        lappend t1List [getFmtTemplate $nameInfo $fmtLoc $thisFmt]
        continue
       }

   set fmtLoc   [getFmt2Loc $nameInfo thisFmt]

    if {$fmtLoc >= 0} \
       {
        lappend t2List [getFmtTemplate $nameInfo $fmtLoc $thisFmt]
        continue
       }
   }

set unusedList $fileList

foreach t $t1List \
   {
    set zMax($t) 0
    set aMax($t) 0

    set usedList [getFmt1Matches $t $unusedList]
    if {[llength $usedList] < 4} continue

    set unusedList [listDisjoint $unusedList $usedList]
    lappend templateList $t
   }

foreach t $t2List \
   {
    set zMax($t) 0
    set aMax($t) 0

    set usedList [getFmt2Matches $t $unusedList]
    if {[llength $usedList] < 4} continue

    set unusedList [listDisjoint $unusedList $usedList]
    lappend templateList $t
   }

puts "fileCount [llength $fileList] dirCount [llength $dirList] templateCount [llength $templateList]"

foreach thisName $fileList     {puts $thisName}
foreach thisName $dirList      {puts $thisName}

foreach thisName $templateList \
   {
    if {$zMax($t) > 0 && $aMax($t) > 0} \
       {
        puts "$thisName 1 $aMax($t) 1 $zMax($t)"
       } \
    elseif {$zMax($t) > 0} \
       {
        puts "$thisName 1 $zMax($t)"
       } \
    else \
       {
        puts $thisName
       }
   }

exit 0
