#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]
  
if {[flagLoc $ARGV -help]} \
   {
    puts "Get Named Items from Lines in a Text File or Stream:"
    puts " -in  inName  \[stdin\]   Input File."
    puts " -var varList \[INDEX\]   List of Named Items to Extract."
    puts " -def defList \[None\]    List of Corresponding Default Values."
    puts " -fmt fmtList \[%s\]      List of Corresponding Formats for Output."
    puts " -verb                  Verbose Mode ON (Output Includes Item Names)."
    exit 0
   }

set inName  stdin
set varList "INDEX"
set defList "None"
set fmtList "%s"

set hdrFlag 0
set verbose 0

getArgD $ARGV -in inName

getListArgD $ARGV -var varList
getListArgD $ARGV -def defList
getListArgD $ARGV -fmt fmtList

if {[flagLoc $ARGV -hdr]}   {set hdrFlag 1}
if {[flagLoc $ARGV -nohdr]} {set hdrFlag 0}

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {![strcasecmp $inName stdin] || ![strcmp $inName "-"]} \
   {
    set inUnit stdin
   } \
else \
   {
    if {![file exists $inName]} \
       {
        puts stderr "getNamedItems Error Finding Input File $inName"
        exit 1
       }

    set error [catch {set inUnit [open $inName r]}]

    if {$error} \
       {
        puts stderr "getNamedItems Error Opening Input File $inName"
        exit 1
       }
   }

set defFmt [lindex $fmtList end]
set defVal [lindex $defList end]

foreach v $varList \
   {
    lappend fmtList $defFmt
    lappend defList $defVal
   }

if {$hdrFlag} \
   {
    puts -nonewline "VARS  "

    set i 1

    foreach var $varList \
       {
        if {$verbose} \
           {
            puts -nonewline " VARNAME$i [string toupper $var]"
           } \
        else \
           {
            puts -nonewline " [string toupper $var]"
           }

        incr i
       }

    puts ""
    puts -nonewline "FORMAT"

    set i 0

    foreach var $varList \
       {
        set fmt [lindex $fmtList $i]

        if {$verbose} \
           {
            puts -nonewline " $s $fmt"
           } \
        else \
           {
            puts -nonewline " $fmt" 
           }

        incr i
       }

    puts ""
    puts ""
   }

while {[gets $inUnit line] >= 0} \
   {
    if {[llength $line] < 2} continue

    set key [lindex $line 0]
    set c   [string index $line 0]

    if {![strcmp $c "#"]} continue

    set valList ""
    set i 0

    foreach var $varList \
       {
        set val [getNamedItemD $var $line $defVal]
        set fmt [lindex $fmtList $i]
        incr i

        if {$i} {puts -nonewline " "}

        if {$verbose} \
           {
            puts -nonewline "$var [safeFormat $fmt $val]"
           } \
        else \
           {
            puts -nonewline [safeFormat $fmt $val]
           }
       }

    puts ""
   }

exit 0
