#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Extract Simulation (simSpecND) Arguments from Existing Data:"
   echo " -fid fidName [test.fid]   Input Time-Domain Data."
   echo " -ft  ftName  [test.ft2]   Corresponding Input Spectrum."
   exit 0
endif

set fidName = (`getArgD $argv -fid test.fid`)
set ftName  = (`getArgD $argv -ft  test.ft2`)

set thisFIDName = (`nmrPrintf $fidName 1 1`)
set thisFTName  = (`nmrPrintf $ftName  1 1`)

set uDimInfo = (X Y Z A)
set lDimInfo = (x y z a)

if (!(-e $thisFIDName)) then
   nmrPrintf -stderr "getSimParms Error Finding Input Time-Domain Data $fidName" 
   exit 1
endif

if (!(-e $thisFTName)) then
   nmrPrintf -stderr "getSimParms Error Finding Input Spectrum $ftName" 
   exit 1
endif

set dimCount = (`getParm -in $thisFIDName -parm FDDIMCOUNT -dim NULL_DIM -fmt %.0f`)
set dmxVal   = (`getParm -in $thisFIDName -parm FDDMXVAL   -dim NULL_DIM`)

nmrPrintf "-ndim %d" $dimCount

if ($dimCount != 1) then
   nmrPrintf " -aq2D Complex"
endif

if (`IMATH "$dmxVal == 0.0"`) then
   set xMODE = Complex
else
   set xMODE = DQD
endif

set i = 1

while ($i <= $dimCount)
   set lDim = $lDimInfo[$i]
   set uDim = $uDimInfo[$i]

   set thisN    = `getParm -in $thisFTName  -parm NDFTSIZE -dim CUR_${uDim}DIM -fmt %.0f`
   set thisMode = "Complex"

   if (`string csame $uDim X`) then
      set thisMode = $xMODE
   endif

   nmrPrintf " -%sN %d"     $lDim $thisN
   nmrPrintf " -%sT %d"     $lDim `getParm -in $thisFIDName  -parm NDAPOD  -dim CUR_${uDim}DIM -fmt %.0f`
   nmrPrintf " -%sMODE %s"  $lDim $thisMode
   nmrPrintf " -%sP0 %.3f"  $lDim `getParm -in $thisFTName   -parm NDP0    -dim CUR_${uDim}DIM`
   nmrPrintf " -%sP1 %.3f"  $lDim `getParm -in $thisFTName   -parm NDP1    -dim CUR_${uDim}DIM`
   nmrPrintf " -%sSW %.3f"  $lDim `getParm -in $thisFIDName  -parm NDSW    -dim CUR_${uDim}DIM`
   nmrPrintf " -%sOBS %.3f" $lDim `getParm -in $thisFIDName  -parm NDOBS   -dim CUR_${uDim}DIM`
   nmrPrintf " -%sCAR %.3f" $lDim `getParm -in $thisFIDName  -parm NDCAR   -dim CUR_${uDim}DIM`
   nmrPrintf " -%sLAB %s"   $lDim `getParm -in $thisFIDName  -parm NDLABEL -dim CUR_${uDim}DIM -text -fmt %s`
   @ i++
end

echo ""

exit 0
