#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "$argv0: Get Contour Height for Spectral Drawing."
    puts " -in   inList \[test.ft2\] NMRPipe-format Input or List."
    puts " -arg  aName  \[hi\]       Argument Name to Use."
    puts " -hi   hiVal  \[None\]     Contour Height, Value, % or Keywords Auto Full."
    puts " -plev nP     \[10\]       Number of Positive Levels."
    puts " -nlev nN     \[10\]       Number of Negative Levels."
    puts " -mult m      \[7.0\]      Factor for Auto Mode."
    puts " -info                   Report Only Max, Min, and Noise Estimate."
    puts " -verb                   Verbose Mode ON."
    puts " -brief                  Brief Mode ON."
    exit 1
   }

set inList     test.ft2
set argName    hi 
set hi         None 
set plev       10
set nlev       10
set mult       7.0
set briefFlag  0
set infoFlag   0
set useMaxFlag 1
set verbose    0

set maxVal     0.0
set minVal     0.0
set noise      0.0
set noiseND    0.0

getListArgD $ARGV -in inList

getArgD $ARGV -arg  argName

getArgD $ARGV -plev plev
getArgD $ARGV -nlev nlev
getArgD $ARGV -mult mult

getArgD $ARGV -$argName hi

if {[flagLoc $ARGV -brief]}   {set briefFlag 1}
if {[flagLoc $ARGV -nobrief]} {set briefFlag 0}

if {[flagLoc $ARGV -info]}   {set infoFlag 1}
if {[flagLoc $ARGV -noinfo]} {set infoFlag 0}

if {$briefFlag} {set verbose 0}

if {[flagLoc $ARGV -useMax]}  {set useMaxFlag 1}
if {[flagLoc $ARGV -useMin]}  {set useMaxFlag 0}

if {[flagLoc $ARGV -verb]}    {set verbose 1}
if {[flagLoc $ARGV -noverb]}  {set verbose 0}

if {!$infoFlag && ![strcasecmp $hi None]} {exit 0}

set inName   [lindex $inList 0]
set thisName [format $inName 1 1]
set inCount  [lindex $inList]

if {![file exists $thisName]} {exit 0}

if {$infoFlag} \
   {
    set rangeList   ""
    set minList     ""
    set maxList     ""
    set noiseList   ""
    set noiseNDList ""

    set minVal 0.0
    set maxVal 0.0

    foreach inName $inList \
       {
        lappend rangeList   [getSpecRange   $inName minVal maxVal]
        lappend noiseList   [estSpecNoise   $inName]
        lappend noiseNDList [estSpecNoiseND $inName]

        lappend minList $minVal
        lappend maxList $maxVal
       }

    set minVal     [getMin $minList]
    set maxVal     [getMax $maxList]
    set noiseMin   [getMin $noiseList]
    set noiseNDMin [getMin $noiseNDList]
    set noise      [getMax $noiseList]
    set noiseND    [getMax $noiseNDList]

    if {$inCount > 1} \
       {
        puts "min $minVal max $maxVal noiseMin $noiseMin noiseNDMin $noiseNDMin noise $noise noiseND $noiseND"
       } \
    else \
       {
        puts "min $minVal max $maxVal noise $noise noiseND $noiseND"
       }
   } \
else \
   {
    set hiList ""

    foreach inName $inList \
       {
        set hiInfo     [getSpecContHi $inName $hi $plev $nlev $mult]
        lappend hiList [lindex $hiInfo 0]
       }

    if {$useMaxFlag} \
       {
        set hi [getMax $hiList]
       } \
    else \
       {
        set hi [getMin $hiList]
       }
 
    if {$verbose} \
       {
        puts $hiInfo
       } \
    else \
       {
        if {$briefFlag} \
           {
            puts "$hi"
           } \
        else \
           {
            puts "-$argName $hi"
           }
       }
   }

exit 0
