#!/bin/csh -f

if (`flagLoc $argv -help`) then
   echo "Extract Sizes of Spectral Data:"
   echo " -in   inName   [test.fid] Input Name or Template."
   echo " -parm parmList [All]      Parameters to List."
   exit 1
endif

@ briefFlag = 0

set inName   = (`getArgD     $argv -in      test.fid`)
set parmList = (`getListArgD $argv -parm -- All`)

if (`flagLoc $argv -brief`) then
   @ briefFlag = 1
endif

#
#------------------------------------------------------------------------------

set thisName = (`nmrPrintf $inName 1 1`)

set dimCount     = (`getParm -in $thisName -parm FDDIMCOUNT -fmt %.0f`)
set trueDimCount = $dimCount

set pipeFlag     = (`getParm -in $thisName -parm FDPIPEFLAG -fmt %.0f`)
set cubeFlag     = (`getParm -in $thisName -parm FDCUBEFLAG -fmt %.0f`)
set fmtCount     = (`echo $inName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

set thisVal      = (`getParm -in $thisName -parm FDQUADFLAG -dim NULL_DIM -fmt %.0f`)

if ($thisVal == 0) then
   set quadSize = 2
else
   set quadSize = 1
endif

set xSize     = 0
set ySize     = 0
set zSize     = 0
set aSize     = 0

set xQuadSize = 0
set yQuadSize = 0
set zQuadSize = 0
set aQuadSize = 0

if ($dimCount >= 1) then
   set xSize   = (`getParm -in $thisName -parm NDSIZE     -dim CUR_XDIM -fmt %.0f`)
   set thisVal = (`getParm -in $thisName -parm NDQUADFLAG -dim CUR_XDIM -fmt %.0f`)

   if ($thisVal == 0) then
      set xQuadSize = 2
   else
      set xQuadSize = 1
   endif
else
   set xSize     = 1
   set xQuadSize = 1
endif

if ($dimCount >= 2) then
   set ySize   = (`getParm -in $thisName -parm NDSIZE     -dim CUR_YDIM -fmt %.0f`)
   set thisVal = (`getParm -in $thisName -parm NDQUADFLAG -dim CUR_YDIM -fmt %.0f`)

   if ($thisVal == 0) then
      set yQuadSize = 2
   else
      set yQuadSize = 1
   endif
else
   set ySize     = 1
   set yQuadSize = 1
endif

if ($dimCount >= 3) then
   set zSize   = (`getParm -in $thisName -parm NDSIZE     -dim CUR_ZDIM -fmt %.0f`)
   set thisVal = (`getParm -in $thisName -parm NDQUADFLAG -dim CUR_ZDIM -fmt %.0f`)

   if ($thisVal == 0) then
      set zQuadSize = 2
   else
      set zQuadSize = 1
   endif
else
   set zSize     = 1
   set zQuadSize = 1
endif

if ($dimCount >= 4) then
   set aSize   = (`getParm -in $thisName -parm NDSIZE     -dim CUR_ADIM -fmt %.0f`)
   set thisVal = (`getParm -in $thisName -parm NDQUADFLAG -dim CUR_ADIM -fmt %.0f`)

   if ($thisVal == 0) then
      set aQuadSize = 2
   else
      set aQuadSize = 1
   endif
else
   set aSize     = 1
   set aQuadSize = 1
endif

if ($fmtCount == 0) then
   if ($dimCount == 3) then
      if ($pipeFlag == 0) then
         set trueDimCount = 2
         set zSize        = 1
         set zQuadSize    = 1 
      endif
   else if ($dimCount == 4) then
      if ($pipeFlag == 0 && $cubeFlag == 1) then
         set trueDimCount = 3
         set aSize        = 1
         set aQuadSize    = 1
      endif
   endif
endif

echo inName $inName testName $thisName \
            dimCount $dimCount trueDimCount $trueDimCount quadSize $quadSize \
            xSize $xSize xQuadSize $xQuadSize \
            ySize $ySize yQuadSize $yQuadSize \
            zSize $zSize zQuadSize $zQuadSize \
            aSize $aSize aQuadSize $aQuadSize

