#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Compute Statistics on Lists of Values, Spectral Data, or Tables:"
    puts " -stat  statList \[None\] Names of Statistics Functions to Compute."
    puts "X Input (Required, Use Only One):"
    puts " -x     xList    \[None\] List of X Values."
    puts " -xSpec xName    \[None\] Spectrum of X Values."
    puts " -xVar  xVarName \[None\] Table X Variable Name (Requires -tab)."
    puts "Y Input (Optional, Use Only One):"
    puts " -y     yList    \[None\] List of Y Values."
    puts " -ySpec yName    \[None\] Spectrum of Y Values."
    puts " -yVar  yVarName \[None\] Table Y Variable Name (Requires -tab)."
    puts "S (Scaling Value) Input (Optional, Use Only One):"
    puts " -s     sList    \[None\] List of S Values."
    puts " -sSpec sName    \[None\] Spectrum of S Values."
    puts " -sVar  sVarName \[None\] Table S Variable Name (Requires -tab)."
    puts "Table or Text Input (Required for -xVar -yVar -sVar):"
    puts " -tab   tabName  \[None\] Input File Name."
    puts " -txt                   Input is Text File of Name/Value Pairs."
    puts " -pdb                   Input is PDB Data (Default for .pdb)."
    puts " -nopdb                 Input is NMRPipe-Format Table."
    puts " -hetatm                Interpret HETATMs as ATOMs."
    puts "Values for X' = cx*X + offx and Y' = cy*Y + offy:" 
    puts " -cx    cx       \[1.0\]  Scale Factor for X Values."
    puts " -cy    cy       \[1.0\]  Scale Factor for Y Values."
    puts " -offx  offX     \[0.0\]  Offset for X Values."
    puts " -offy  offY     \[0.0\]  Offset for Y Values."
    puts "Special Parameters:"
    puts " -df    fd       \[1\]    Degrees of Freedom (Required for PChi Statistic)."
    puts " -undef uVal     \[None\] Threshold for Undefined Values."
    puts "Statistics Modes:"
    puts " -vs             Compute Simple Statistics using Q = X' - Y'."
    puts " -novs           Compute Simple Statistics using Q = X' (Default)."
    puts "Scaling Modes:"
    puts " -noscale        Compute Simple Statistics for Q     (Default)."
    puts " -norm           Compute Simple Statistics for Q/S."
    puts " -scale          Compute Simple Statistics for Q*S."
    puts "Other Options:"
    puts " -int            Truncate Results to Integer."
    puts " -angle          Treat Values as Angles in Degrees."
    puts " -verb           Verbose Mode ON (Default)."
    puts " -noverb         Verbose Mode OFF (Also: -brief)."
    puts "Simple Statistics on Q:"
    puts " Avg Sum SumSq Variance StdDev RMS" 
    puts " Max Min MaxAbs MinAbs SMaxAbs SMinAbs Range Count"
    puts " Median SimpleMedian"
    puts "Operations that Return a Vector:"
    puts " VecSort VecSortLoc VecAbs VecInt VecVal Reverse"
    puts "Statistics on X' vs Y':"
    puts " PearsonR Scale"
    puts "Statistics Using X' Y' and S:"
    puts " Chi2 PChi (Requires -df value)."
    exit 1
   }

set xListOrig ""
set yListOrig ""
set sListOrig ""

set xListSpec None
set yListSpec None
set sListSpec None

set xVar      None
set yVar      None
set sVar      None

set tabName   None
set pdbMode   nopdb
set hetMode   nohetatm
set txtFlag   0
set statList  ""

set cx        1.0
set cy        1.0
set offx      0.0
set offy      0.0
set df        1

set verbose   1
set vsFlag    0
set intFlag   0
set angleFlag 0
set scaleMode None
set uVal      None

getArgD $ARGV -cx    cx
getArgD $ARGV -cy    cy
getArgD $ARGV -offx  offx
getArgD $ARGV -offy  offy
getArgD $ARGV -df    df
getArgD $ARGV -undef uVal

getArgD $ARGV -tab   tabName
getArgD $ARGV -xVar  xVar
getArgD $ARGV -yVar  yVar
getArgD $ARGV -sVar  sVar

if {![strcasecmp [file extension $tabName] .pdb]} {set pdbMode pdb}

getListArgD $ARGV -stat statList

if {[flagLoc $ARGV -verb]}    {set verbose 1}
if {[flagLoc $ARGV -noverb]}  {set verbose 0}
if {[flagLoc $ARGV -brief]}   {set verbose 0}

if {[flagLoc $ARGV -txt]}     {set txtFlag 1}
if {[flagLoc $ARGV -notxt]}   {set txtFlag 0}

if {[flagLoc $ARGV -int]}     {set intFlag 1}
if {[flagLoc $ARGV -noint]}   {set intFlag 0}

if {[flagLoc $ARGV -angle]}   {set angleFlag 1}
if {[flagLoc $ARGV -noangle]} {set angleFlag 0}

if {[flagLoc $ARGV -vs]}      {set vsFlag 1}
if {[flagLoc $ARGV -novs]}    {set vsFlag 0}

if {[flagLoc $ARGV -norm]}    {set scaleMode norm}
if {[flagLoc $ARGV -nonorm]}  {set scaleMode none}

if {[flagLoc $ARGV -scale]}   {set scaleMode scale}
if {[flagLoc $ARGV -noscale]} {set scaleMode none}

if {[flagLoc $ARGV -pdb]}       {set pdbMode pdb}
if {[flagLoc $ARGV -nopdb]}     {set pdbMode nopdb}

if {[flagLoc $ARGV -hetatm]}    {set hetMode hetatm}
if {[flagLoc $ARGV -nohetatm]}  {set hetMode nohetatm}
  
getArgD $ARGV -xSpec xListSpec
getArgD $ARGV -ySpec yListSpec
getArgD $ARGV -sSpec sListSpec

if {[strcasecmp $xListSpec None]} \
   {
    if {![file exists $xListSpec]} \
       {
        puts stderr "GetStat Error Finding X Spectrum $xListSpec"
        exit 1
       }

    set sizeList [getSpecSizeList $xListSpec]
    set dimCount [llength $sizeList]

    if {$dimCount} \
       {
        readROI -roi 1 -ndim $dimCount -in $xListSpec

        set xListOrig [getROIVals 1] 
        set n         [llength $xListOrig]

        if {$verbose} \
           {
            puts "[format %-10s $xListSpec] Dimensions: $dimCount Sizes: $sizeList N: $n"
           }
       }
   }

if {[strcasecmp $yListSpec None]} \
   {
    if {![file exists $yListSpec]} \
       {
        puts stderr "GetStat Error Finding Y Spectrum $yListSpec"
        exit 1
       }

    set sizeList [getSpecSizeList $yListSpec]
    set dimCount [llength $sizeList]

    if {$dimCount} \
       {
        readROI -roi $dimCount -ndim 1 -in $yListSpec 
        set yListOrig [getROIVals 1]
        set n         [llength $yListOrig]

        if {$verbose} \
           {
            puts "[format %-10s $yListSpec] Dimensions: $dimCount Sizes: $sizeList N: $n"
           }
       }
   }

if {[strcasecmp $sListSpec None]} \
   {
    if {![file exists $sListSpec]} \
       {
        puts stderr "GetStat Error Finding S Spectrum $sListSpec"
        exit 1
       }

    set sizeList [getSpecSizeList $sListSpec]
    set dimCount [llength $sizeList]

    if {$dimCount} \
       {
        readROI -roi $dimCount -ndim 1 -in $sListSpec
        set sListOrig [getROIVals 1]
        set n         [llength $sListOrig]

        if {$verbose} \
           {
            puts "[format %-10s $sListSpec] Dimensions: $dimCount Sizes: $sizeList N: $n"
           }
       }
   }

set txtInfo(lineCount) 0

if {[strcasecmp $tabName None]} \
   {
    if {![file exists $tabName]} \
       {
        puts stderr "GetStat Error Finding Input Table $tabName"
        exit 1
       }

    if {$txtFlag} \
       {
        set lineCount [getTextLines $tabName txtInfo]

        if {$lineCount < 1} \
           {
            puts stderr "GetStat Error Reading Input Text File $tabName"
            exit 1
           }

        if {[strcasecmp $xVar None]} \
           {
            for {set i 0} {$i < $lineCount} {incr i} {lappend xListOrig [getNamedItemD $xVar $txtInfo($i) 0]}
           }

        if {[strcasecmp $xVar None]} \
           {
            for {set i 0} {$i < $lineCount} {incr i} {lappend yListOrig [getNamedItemD $yVar $txtInfo($i) 0]}
           }

        if {[strcasecmp $sVar None]} \
           {
            for {set i 0} {$i < $lineCount} {incr i} {lappend sListOrig [getNamedItemD $sVar $txtInfo($i) 0]}
           }
       } \
    else \
       {
        set inTab [gdbRead table -in $tabName -$pdbMode -$hetMode]
        set vList [gdbGetInfo $inTab -parms varNames]

        if {[strcasecmp $xVar None]} \
           {
            if {[lsearch -exact $vList $xVar] < 0} \
               {
                puts stderr "GetStat Error Finding Input Table X Variable $xVar"
                exit 1
               }

            set xListOrig [getTabVals $inTab $xVar]
           }

        if {[strcasecmp $yVar None]} \
           {
            if {[lsearch -exact $vList $yVar] < 0} \
               {
                puts stderr "GetStat Error Finding Input Table Y Variable $yVar"
                exit 1
               }

            set yListOrig [getTabVals $inTab $yVar]
           }

        if {[strcasecmp $sVar None]} \
           {
            if {[lsearch -exact $vList $sVar] < 0} \
               {
                puts stderr "GetStat Error Finding Input Table S Variable $sVar"
                exit 1
               }

            set sListOrig [getTabVals $inTab $sVar]
           }
       }
   }

getListArgD $ARGV -x xListOrig
getListArgD $ARGV -y yListOrig
getListArgD $ARGV -s sListOrig

if {![strcasecmp $xListOrig None]} {set xListOrig ""}
if {![strcasecmp $yListOrig None]} {set yListOrig ""}
if {![strcasecmp $sListOrig None]} {set sListOrig ""}

#
#---------------------------------------------------------------------------

set nX [llength $xListOrig]
set nY [llength $yListOrig]
set nS [llength $sListOrig]

set xList ""
set yList ""
set sList ""

set xpList ""
set ypList ""

set dList "" 
set rList ""
set qList "" 

if {!$nX} {exit 0}
 
while {$nY < $nX} \
   {
    lappend yListOrig 0.0
    incr nY
   }

if {!$nS} \
   {
    set sListOrig 1.0
    incr nS
   }

set sVal [lindex $sListOrig 0]

while {$nS < $nX} \
   {
    lappend sListOrig $sVal
    incr nS
   }

for {set i 0} {$i < $nX} {incr i} \
   {
    set xVal [lindex $xListOrig $i]
    set yVal [lindex $yListOrig $i]
    set sVal [lindex $sListOrig $i]

    if {[strcasecmp $uVal None]} \
       {
        if {$xVal > $uVal} continue
        if {$yVal > $uVal} continue
        if {$sVal > $uVal} continue
       }

    if {$cx == 1.0 && $offx == 0.0} \
       {
        lappend xList $xVal
       } \
    else \
       {
        lappend xList [expr $cx*$xVal + $offx]
       }

    if {$cy == 1.0 && $offy == 0.0} \
       {
        lappend yList $yVal
       } \
    else \
       {
        lappend yList [expr $cy*$yVal + $offy]
       }

    lappend sList $sVal
   }

set nX [llength $xList]

for {set i 0} {$i < $nX} {incr i} \
    {
     set xVal [lindex $xList $i]
     set yVal [lindex $yList $i]

     if {$angleFlag} \
        {
         set dVal [angleDiff $xVal $yVal]
        } \
     else \
        {
         set dVal [expr $xVal - $yVal]
        }

     lappend dList $dVal
    }

if {$vsFlag} \
   {
    set rList $dList
   } \
else \
   {
    set rList $xList
   }


if {![strcasecmp $scaleMode norm]} \
   {
    for {set i 0} {$i < $nX} {incr i} \
       {
        set xVal [lindex $xList $i]
        set yVal [lindex $yList $i]
        set rVal [lindex $rList $i]
        set sVal [lindex $sList $i]

        if {$sVal == 0.0} {set sVal 1.0}

        lappend qList  [expr $rVal/$sVal]
        lappend xpList [expr $xVal/$sVal]
        lappend ypList [expr $yVal/$sVal]
       }
   } \
elseif {![strcasecmp $scaleMode scale]} \
   {
    for {set i 0} {$i < $nX} {incr i} \
       {
        set xVal [lindex $xList $i]
        set yVal [lindex $yList $i]
        set rVal [lindex $rList $i]
        set sVal [lindex $sList $i]

        lappend qList  [expr $rVal*$sVal]
        lappend xpList [expr $xVal*$sVal]
        lappend ypList [expr $yVal*$sVal]
       }
   } \
else \
   {
    set xpList $xList
    set ypList $yList
    set qList  $rList
   }

set procListQ \
  {
   Sum SumSq Avg Min Max MaxAbs MinAbs SMaxAbs SMinAbs \
   Median SimpleMedian Variance StdDev RMS Range Count
   VecSort VecSortLoc VecAbs VecInt VecVal Reverse
  }

foreach sName $statList \
   {
    set pName None
    set val   None 

    if {$angleFlag} \
       {
        if {![strcasecmp $sName Avg]}    {set pName AngleAvg} 
        if {![strcasecmp $sName StdDev]} {set pName AngleStdDev} 
       }

    if {![strcasecmp $pName None]} \
       {
        foreach p $procListQ \
           {
            if {[strcasecmp $sName $p]} continue 
            set pName $p
            break
           }
       }

    if {[strcasecmp $pName None]} \
       {
        set val [get$pName $qList]
       } \
    else \
       {
        if {![strcasecmp $sName Scale]} \
           {
            set pName Scale
            set val [get$pName $xpList $ypList]
           }

        if {![strcasecmp $sName PearsonR]} \
           {
            set pName PearsonR
            set val [get$pName $xpList $ypList]
           }

        if {![strcasecmp $sName Chi2]} \
           {
            set pName Chi2
            set val [get$pName $xList $yList $sList]
           }

        if {![strcasecmp $sName PChi]} \
           {
            set pName PChi
            set val [get$pName $dList $sList $df]
           }
       }
    
    if {[strcasecmp $val None]} \
       {
        if {$intFlag} {set val [expr int($val)]}
       }

    if {$verbose} \
       {
        puts "$sName $val"
       } \
    else \
       {
        puts $val
       }
   }

exit 0
