#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]
  
if {[flagLoc $ARGV -help]} \
{
puts stderr "Extract Row Data from an NMRPipe format table:"
puts stderr " -in  tabName \[test.tab\] Input Table."
puts stderr " -n   rowList \[0\]        Row Numbers, * for All, First Row is 0."
puts stderr " -var varList \[*\]        Variables to Extract, Use * in Patterns."
puts stderr " -fmt fmtList \[%s\]       Formats for Output, * for FORMAT Value."
puts stderr " -hdr                    Output Variable List as Header."
puts stderr " -csv                    Output as Comma-Separated Values (CSV)."
puts stderr "PDB Options:"
puts stderr " -pdb                    Input is PDB Data (Default for .pdb)."
puts stderr " -nopdb                  Input is NMRPipe-Format Table."
puts stderr " -hetatm                 Interpret HETATMs as ATOMs."
exit 1
}

set inName  test.tab
set pdbMode nopdb
set hetMode nohetatm
set hdrFlag 0
set csvFlag 0

set rowList "0"
set varList "*"
set fmtList "%s"

getArgD     $ARGV -in  inName
getListArgD $ARGV -n   rowList
getListArgD $ARGV -var varList
getListArgD $ARGV -fmt fmtList

if {![strcasecmp [file extension $inName] .pdb]} {set pdbMode pdb}

if {[flagLoc $ARGV -hdr]}       {set hdrFlag 1}
if {[flagLoc $ARGV -nohdr]}     {set hdrFlag 0}

if {[flagLoc $ARGV -csv]}       {set csvFlag 1}
if {[flagLoc $ARGV -nocsv]}     {set csvFlag 0}

if {[flagLoc $ARGV -pdb]}       {set pdbMode pdb}
if {[flagLoc $ARGV -nopdb]}     {set pdbMode nopdb}

if {[flagLoc $ARGV -hetatm]}    {set hetMode hetatm}
if {[flagLoc $ARGV -nohetatm]}  {set hetMode nohetatm}

#
#------------------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input File $inName"
    exit 1
   }

if {$csvFlag} \
   {
    set sepChar ","
   } \
else \
   {
    set sepChar ""
   }

set inTab   [gdbRead table -in $inName -$pdbMode -$hetMode]
set vList   [gdbGetInfo $inTab -parms varNames]
set eList   [gdbGetInfo $inTab -parms entries]
set eCount  [llength $eList]
set varList [listMatch $vList $varList]

if {![strcmp $rowList "*"]} \
   {
    set rowList ""
    for {set i 0} {$i < $eCount} {incr i} {lappend rowList $i}
   }

if {![strcmp $fmtList "*"]} \
   {
    set fmtList ""
    foreach varName $varList {lappend fmtList "*"}
   }

foreach varName $varList {lappend fmtList "%s"}

set fList ""
set i 0

foreach v $varList \
   {
    set f [lindex $fmtList $i]

    if {[lsearch -exact $vList $v] < 0} \
       {
        lappend fList " (None) "
       } \
    else \
       {
        if {![strcmp $f "*"]} \
           {
            set vPtr [gdbFindByName $inTab $v]
            set f    [gdbGetInfo $vPtr -parms varFmt]
           }

        lappend fList $f
       }

    incr i
   }

set fmtList $fList

if {$hdrFlag} \
   {
    set i 0

    foreach varName $varList \
      {
       if {$i} {puts -nonewline "$sepChar "}
       puts -nonewline $varName
       incr i
      }

    puts ""
   }

foreach loc $rowList \
   {
    set i 0

    foreach varName $varList \
      { 
       if {$i} {puts -nonewline "$sepChar "}
    
       if {[lsearch -exact $vList $varName] < 0 || $loc < 0 || $loc > $eCount - 1} \
          {
           set str " (None) "
          } \
       else \
          {
           set str [format [lindex $fmtList $i] [gdbGet [lindex $eList $loc] $varName]]
          }

       puts -nonewline $str
       incr i
      }

    puts ""
   }

exit 0
