#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Interpolate Spectra According to First Spectrum in a Series:"
   echo "Result is stored as Pseudo-3D Data:"
   echo " -inDir  inDir   [ft]     Input Directory of *.ft2 Files."
   echo " -outDir outDir  [interp] Output Directory."
   echo " -ref    refName [None]   Reference Spectrum or Keyword None."
   echo "Notes:"
   echo " If -ref None is used, the first spectrum in the series will"
   echo " be used as the reference."
   exit 0
endif

set inDir   = (`getArgD $argv -inDir  ft`)
set outDir  = (`getArgD $argv -outDir interp`)
set refName = (`getArgD $argv -ref    None`)

if (!(-d $inDir)) then
   echo "Error Finding Interpolation Input Directory $inDir"
   exit 1
endif

/bin/rm -rf $outDir 
mkdir $outDir

set inList = ($inDir/*)

if ($#inList == 0) then
   echo "Error Finding Interpolation Input Files in Directory $inDir"
   exit 1
endif

if (`string csame $refName None`) then
   set refName = $inList[1]
endif

if (!(-e $refName)) then
      echo "Error Finding Interpolation Reference Data $refName"
   exit 1
endif

@ i = 0

foreach f ($inList)
   @ i++
   set thisName = (`nmrPrintf %s/%03d.ft2 $outDir $i`)
   echo Interpolating $f Reference: $refName Output: $thisName

   interpNMR -in $f -out $thisName -ref $refName
end

seriesT.com $outDir/*.ft2

exit 0
