#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts stderr "Extract values from JCAMP file:"
    puts stderr " -in   inName \[acqus\]    Name of JCAMP-format Input File."
    puts stderr " -parm pList  \[All\]      List of Parameters to Extract, or Keyword All."
    puts stderr " -verb                   Verbose Mode ON (Default)."
    puts stderr " -noverb                 Verbose Mode OFF." 
    puts stderr "Value Types to Extract:"
    puts stderr " -private                Extract ##$ Private Parameters (Default)."
    puts stderr " -noprivate              Do Not Extract ##$ Parameters."
    puts stderr " -specific               Extract ##. Datatype-Specific Parameters."
    puts stderr " -nospecific             Do Not Extract ##. Parameters (Default)."
    puts stderr " -generic                Extract ##  Generic Parameters."
    puts stderr " -nogeneric              Do Not Extract ## Parameters  (Default)."
    puts stderr " -all                    Extract All of the Above (##$ ##. ##)."
    puts stderr "Other Output:"
    puts stderr " -comment                List Comments."
    puts stderr " -hdr                    List Header Parameters."
    exit 0
   }

set inName   acqus 
set pList    "*"

set privateFlag  1
set genericFlag  0
set specificFlag 0
set colFlag      0

set verbose      1

getArgD     $ARGV -in   inName
getListArgD $ARGV -parm pList

if {[flagLoc $ARGV -all]} \
   {
    set privateFlag  1
    set genericFlag  1
    set specificFlag 1
   }

if {[flagLoc $ARGV -private]}     {set privateFlag 1}
if {[flagLoc $ARGV -noprivate]}   {set privateFlag 0}

if {[flagLoc $ARGV -generic]}     {set genericFlag 1}
if {[flagLoc $ARGV -nogenericic]} {set genericFlag 0}

if {[flagLoc $ARGV -specific]}    {set specificFlag 1}
if {[flagLoc $ARGV -nospecific]}  {set specificFlag 0}

if {[flagLoc $ARGV -col]}   {set colFlag 1}
if {[flagLoc $ARGV -nocol]} {set colFlag 0}

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {![file exists $inName]} \
   {
    puts stderr "Error Finding parameter input file $inName"
    exit 1
   }

if {!$privateFlag || $genericFlag || $specificFlag} \
   {
    set n [rdJCAMPAll $inName acqus $privateFlag $genericFlag $specificFlag]
   } \
else \
   {
    set n [rdJCAMP    $inName acqus]
   }

if {![info exists acqus(.parmList)]} \
   {
    if {$verbose} {puts stderr "Error Finding parameter contents."}
    exit 0
   }

if {![strcasecmp $pList All] || ![strcmp $pList "*"]} \
   {
    set pList $acqus(.parmList)
   }

set n 0

foreach p $pList \
   {
    set nP [string length $p]
    if {$nP > $n} {set n $nP}
   }

incr n

set fmt "%-${n}s %-10s %s"

if {[llength $pList] && [strcasecmp $pList None]} \
   {
    foreach p $pList \
       {

        if {[info exists acqus($p.type)]} \
           {
            set type $acqus($p.type)
           } \
        else \
           {
            set type NONE
           }

        if {[info exists acqus($p)]} \
           {
            set val $acqus($p)
           } \
        else \
           {
            set val  MISSING
            set type MISSING
           }

        if {$verbose} \
           {
            puts [format $fmt $p $type $val]
           } \
        else \
           {
            if {$colFlag} \
               {
                foreach v $val {puts $v}
               } \
            else \
               {
                puts $val
               }
           }
       }
   }

if {[flagLoc $ARGV -hdr]} \
   {
    if {[info exists acqus(.hdr)]} \
       {
        puts "Header Parameters: [llength $acqus(.hdr)]" 

        foreach v $acqus(.hdr) \
           {
            puts $v
           }
       } \
    else \
       {
        puts "Header Parameters: 0" 
       }
   }

if {[flagLoc $ARGV -comment]} \
   {
    if {[info exists acqus(.remark)]} \
       {
        puts "Comment Lines: [llength $acqus(.remark)]"

        foreach v $acqus(.remark) \
           {
            puts $v
           }
       } \
    else \
       {
        puts "Comment Lines: 0"
       }
   }

exit 0
