#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

proc getVal { valLabel valDefault line } \
{
   set loc [lsearch -exact $line $valLabel]

   if {$loc == -1} \
      {
       return $valDefault
      }

   return [lindex $line [expr $loc + 1]]
}

#
# Initialize:

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set error      0
set badCount   0
set memCount   0
set usedVal    0

set inName     ""
set fnList     ""

#
# Print help text if needed:

   if {$ARGC == 1 || [flagLoc $ARGV "-HELP"]} \
      {
       puts stderr "Leak: analyze memory.c debug output for leaks."
       puts stderr "Required Arguments:"
       puts stderr " -in   inName    File Name of Debug Output."
       return 1
      }

#
# Test for required arguments:

   if {![flagLoc $ARGV -in]}  {set error 1}

   if {$error} \
      {
       puts stderr "MatchTab Error: one or more missing arguments."
       return 1
      }

getArgD $ARGV -in inName

#
# Read the text.
# For each _MEM debug line:
#   Create a list and array entries for the function listed.
#   Increment the array entries.

    set error [catch {set inUnit [open $inName r]}]

    if {$error} \
       {
        puts stderr "Leak Error opening input file $inName"
        return -1
       }

    while {[gets $inUnit line] >= 0} \
       {
        if {![string compare _MEM [lindex "$line NULL" 0]]}\
           {
            incr memCount

            set usedVal  [getVal Now:    $usedVal $line]
            set fnName   [getVal Name:   None     $line]
            set addVal   [getVal Add:    0        $line]
            set delVal   [getVal Delete: 0        $line]
            set allocVal [getVal Alloc:  0        $line]
            set freeVal  [getVal Free:   0        $line]

            if {-1 == [lsearch -exact $fnList $fnName]} \
               {
                lappend fnList $fnName

                set addList($fnName) 0
                set delList($fnName) 0
               }

            if {[string compare $freeVal "0"]} \
               {
                if {[info exists ptrList($freeVal)]} \
                   {
                    unset ptrList($freeVal)
                   } \
                else \
                   {
                    if {[strcasecmp $freeVal "nil"] && [strcasecmp $freeVal "(nil)"] && [strcasecmp $freeVal "null"] && [strcasecmp $freeVal "(null)"]} \
                       {
                        puts "Ptr $freeVal in $fnName was freed but not allocated."
                        incr badCount
                       }
                   }
               }

            if {[string compare $allocVal "0"]} \
               {
                if {![info exists ptrList($allocVal)]} \
                   {
                    set ptrList($allocVal) $fnName
                   } \
                else \
                   {
                    puts "Ptr $allocVal in $fnName was overwritten."
                    incr badCount
                   }
               }

            incr addList($fnName) $addVal
            incr delList($fnName) $delVal
           }
       }

    close $inUnit

#
# Scan the tallies for inconsistancies:

    set ptrList(0) None

    foreach i $fnList \
       {
        if {$addList($i) != $delList($i)} \
           {
            puts \
              "Possible leak in $i: Allocated: $addList($i) Freed: $delList($i)"

            incr badCount
           }
       }

    set i [array startsearch ptrList]

    while {[array anymore ptrList $i]} \
       {
        set  allocVal [array nextelement ptrList $i]

        if {$allocVal != 0} \
           {
            puts "Ptr $allocVal in $ptrList($allocVal) is not freed."
            incr badCount
           }
       }

    if {$usedVal != 0} \
       {
        puts "$usedVal bytes still allocated."
        incr badCount
       }

    puts "Leak check of $memCount entries completed with $badCount warnings."

exit
