#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

if {[flagLoc $ARGV -help]} \
{
 puts stderr "Convert NMRPipe PCA Loading File to Text Data."
 puts stderr " -in   loadName \[load.dat\] NMRPipe-Format PCA Loading File."
 puts stderr " -id   idList   \[1 2 ...\]  ID Values, or Keyword None."
 puts stderr " -hdr                        Print Header (Default)."
 puts stderr " -nohdr                      Suppress Header."
 puts stderr " -csv                        Print as Comma Separated, No Header."
 exit 0
}

set inName  load.dat
set hdrFlag 1
set csvFlag 0
set idList  ""

getArgD $ARGV -in inName
getListArgD $ARGV -id idList

if {[flagLoc $ARGV -hdr]}   {set hdrFlag 1}
if {[flagLoc $ARGV -nohdr]} {set hdrFlag 0}

if {[flagLoc $ARGV -csv]}   {set csvFlag 1}
if {[flagLoc $ARGV -nocsv]} {set csvFlag 0}

if {$csvFlag} {set hdrFlag 0}

#
#--------------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Loading Input $inName"
    exit 1
   }

readROI -ndim 2 -roi 1 -in $inName -x Y_AXIS 0% 100% -y X_AXIS 0% 100%

set rInfo [getROIInfo -roi 1]
set rdata [lindex $rInfo 0]
set sList [lindex $rInfo 2]
set xSize [lindex $sList 0]
set ySize [lindex $sList 1]

if {![strcasecmp $idList None]} \
   {
    set nID 0
   } \
else \
   {
    set nID [llength $idList]

    if {!$nID} \
       {
        for {set id 1} {$id <= $ySize} {incr id} {lappend idList $id}  
        set nID $ySize
       } \
    elseif {$nID != $ySize} \
       {
        puts stderr "Error: mismatch in data size and ID list."
        exit 1
       }
   }

set nMax 1

foreach id $idList \
   {
    set n [string length $id]
    if {$n > $nMax} {set nMax $n}
   }

incr nMax

set  idFmt " %-${nMax}s "
set  cFmt  " % .5e "
set  cFmtN " % .5e "

if {$csvFlag} \
   {
    set idFmt "%s,"
    set cFmt  "%s,"
    set cFmtN "%s"
   }

if {$hdrFlag} \
   {
    puts -nonewline "VARS   "
    if {$nID} {puts -nonewline " ID   "}
    for {set ix 1} {$ix <= $xSize} {incr ix} {puts -nonewline " C$ix  "}
    puts ""

    puts -nonewline "FORMAT "
    if {$nID} {puts -nonewline "$idFmt "}
    for {set ix 1} {$ix <= $xSize} {incr ix} {puts -nonewline "%.5e "}
    puts ""
    puts ""
   }

set loc 0

for {set iy 0} {$iy < $ySize} {incr iy} \
   {
    if {$nID} {puts -nonewline [format $idFmt [lindex $idList $iy]]}

    set fmt $cFmt
 
    for {set ix 1} {$ix <= $xSize} {incr ix} \
       {
        if {$ix == $xSize} {set fmt $cFmtN}
        puts -nonewline [format $fmt [getPtrVal $rdata $loc]]
        incr loc
       }

    puts ""
   }

exit 0
