#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "mask.tcl: Apply Rectangular Region Masks to a 2D Series."
    puts " -in    inDir     \[ft\]         Directory of 2D Data to Process."
    puts " -tab   tabName   \[mask.tab\]   Table of 2D Regions."
    puts " -mask  maskName  \[mask.dat\]   2D Spectral Input for Mask Creation"
    puts " -def   defVal    \[0.0\]        Default Value of Mask."
    puts " -reg   regVal    \[1.0\]        Value for Regions."
    puts " -apply"
    exit 1
   }

set inDir     ft
set maskName  mask.dat
set tabName   mask.tab
set applyFlag 0

set defVal   0.0
set regVal   1.0

getArgD $ARGV -in   inDir
getArgD $ARGV -tab  tabName
getArgD $ARGV -mask maskName
getArgD $ARGV -def  defVal
getArgD $ARGV -reg  regVal

if {[flagLoc $ARGV -apply]}   {set applyFlag 1}
if {[flagLoc $ARGV -noapply]} {set applyFlag 0}

#
#---------------------------------------------------------

set inList [lsort -dictionary [glob -nocomplain $inDir/*]]
set inName [lindex $inList 0]

set maskTab [gdbRead table -in $tabName]
set entry   [gdbFirst $maskTab]

catch {exec nmrPipe -in $inName -out $maskName -ov -fn SET -c $defVal}

puts "Creating mask from $tabName ..."

while {$entry} \
   {
    set x1 [gdbGet $entry X1]ppm
    set xn [gdbGet $entry XN]ppm
    set y1 [gdbGet $entry Y1]ppm
    set yn [gdbGet $entry YN]ppm

    set s  [gdbGet $entry SCALE] 
   
    if {"$s" == ""} {set s $regVal}
 
    puts "$x1->$xn $y1->$yn Scale: $s"

    catch {exec nmrPipe -fn SET -c $s -x1 $x1 -xn $xn \
                -select $y1 $yn -in $maskName -out $maskName -inPlace}

    set entry [gdbNext $entry]
   }

if {$applyFlag} \
   {
    foreach i $inList \
       {
        puts  "Applying mask to $i"
        catch {exec addNMR -in1 $i -in2 $maskName -out $i -mul}
       }
   }

exit 0
