#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
# Print help text if needed:

if {$ARGC == 1 || [flagLoc $ARGV "-help"]} \
   {
    puts stderr "MatchTab: Match and Append Variables from Table 2 to Table 1."
    puts stderr " -in1  inName1 \[None\]    File Name of Input Table 1."
    puts stderr " -in2  inName2 \[None\]    File Name of Input Table 2."
    puts stderr " -out  outName \[stdout\]  File Name of Output Table."
    puts stderr "Input Variables:"
    puts stderr " -var1 varList1  List of Variables Names to Match in Table 1."
    puts stderr " -var2 varList2  Corresponding Variable Names in Table 2."
    puts stderr " -var  varList   Sets Both varList1 and varList2."
    puts stderr " -tol  tolList   Comparison Tolerances, if Any;"
    puts stderr "                 Use Keywords case and nocase for Strings." 
    puts stderr "Output Variables:"
    puts stderr " -add  addList   Variables from Table 2 to Add."
    puts stderr " -new  newList   New Variable Names for addList."
    puts stderr " -pre  prefix    Prefix for New Variable Names."
    puts stderr " -post postfix   Postfix for New Variable Names."
    puts stderr "Match Count Arguments:"
    puts stderr " -append                 Append Multiple Matches."
    puts stderr " -skip                   Skip Entries with No Matches."
    puts stderr "Text Header Arguments:"
    puts stderr " -text1                  Include Text from Table 1 (Default)."
    puts stderr " -notext1                No Text from Table 1."
    puts stderr " -text2                  Include Text from Table 2 (Default)."
    puts stderr " -notext2                No Text from Table 2."
    puts stderr "PDB Arguments:"
    puts stderr " -pdb1                   Input 1 is PDB (Default for .pdb)."
    puts stderr " -nopdb1                 Input 1 is NMRPipe-Format Table."
    puts stderr " -hetatm1                Input 1, Interpret HETATMs as ATOMs."
    puts stderr " -pdb2                   Input 2 is PDB (Default for .pdb)."
    puts stderr " -nopdb1                 Input 2 is NMRPipe-Format Table."
    puts stderr " -hetatm2                Input 2, Interpret HETATMs as ATOMs."
    return 1
   }

#
#-------------------------------------------------------------------------------

set inName1  None
set inName2  None
set outName  stdout

set mList1   ""
set mList2   ""
set tolList  ""

set addList  "" 
set newList  ""

set prefix   ""
set postfix  ""

set appendFlag 0
set skipFlag   0
set textFlag1  1
set textFlag2  1

set pdbMode1   nopdb
set hetMode1   nohetatm

set pdbMode2   nopdb
set hetMode2   nohetatm

getArgD $ARGV -in1 inName1
getArgD $ARGV -in2 inName2
getArgD $ARGV -out outName

getArgD $ARGV -pre  prefix
getArgD $ARGV -post postfix

getListArgD $ARGV -var  mList1
getListArgD $ARGV -var  mList2

getListArgD $ARGV -var1 mList1
getListArgD $ARGV -var2 mList2
getListArgD $ARGV -tol  tolList

getListArgD $ARGV -add  addList
getListArgD $ARGV -new  newList

if {[flagLoc $ARGV -append]}   {set appendFlag 1}
if {[flagLoc $ARGV -noappend]} {set appendFlag 0}

if {[flagLoc $ARGV -skip]}     {set skipFlag 1}
if {[flagLoc $ARGV -noskip]}   {set skipFlag 0}

if {[flagLoc $ARGV -text1]}   {set textFlag1 1}
if {[flagLoc $ARGV -notext1]} {set textFlag1 0}

if {[flagLoc $ARGV -text2]}   {set textFlag2 1}
if {[flagLoc $ARGV -notext2]} {set textFlag2 0}

if {![strcasecmp [file extension $inName1] .pdb]} {set pdbMode1 pdb}
if {![strcasecmp [file extension $inName2] .pdb]} {set pdbMode2 pdb}

if {[flagLoc $ARGV -pdb1]}    {set pdbMode1 pdb}
if {[flagLoc $ARGV -nopdb1]}  {set pdbMode1 nopdb}

if {[flagLoc $ARGV -hetatm1]}    {set hetMode1 hetatm}
if {[flagLoc $ARGV -nohetatm1]}  {set hetMode1 nohetatm}

if {[flagLoc $ARGV -pdb2]}    {set pdbMode2 pdb}
if {[flagLoc $ARGV -nopdb2]}  {set pdbMode2 nopdb}

if {[flagLoc $ARGV -hetatm2]}    {set hetMode2 hetatm}
if {[flagLoc $ARGV -nohetatm2]}  {set hetMode2 nohetatm}

#
#-------------------------------------------------------------------------------

if {![strcasecmp $inName1 None]} \
   {
    puts stderr "Error: No Input File 1 specified."
    exit 1
   } \
else \
   {
    if {![file exists $inName1]} \
       {
        puts stderr "Error Finding Input File 1: $inName1"
        exit 1
       }    

    set tab1 [gdbRead table -in $inName1 -$pdbMode1 -$hetMode1]
   }

if {![strcasecmp $inName2 None]} \
   {
    puts stderr "Error: No Input File 2 specified."
    exit 1
   } \
else \
   {
    if {![file exists $inName2]} \
       {
        puts stderr "Error Finding Input File 2: $inName2"
        exit 1
       }

    set tab2 [gdbRead table -in $inName2 -$pdbMode2 -$hetMode2]
   }

if {![strcasecmp $outName stdout] || ![strcasecmp $outName "-"]} \
   {
    set outUnit stdout
   } \
else \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "Error Opening Output File $outName"
        exit 1
       }
   }

set eList1 [gdbGetInfo $tab1 -parms entries]
set vList1 [gdbGetInfo $tab1 -parms varNames]
set fList1 [gdbGetInfo $tab1 -parms varFmts]
set tList1 [gdbGetInfo $tab1 -parms varTypes]

set eList2 [gdbGetInfo $tab2 -parms entries]
set vList2 [gdbGetInfo $tab2 -parms varNames]
set fList2 [gdbGetInfo $tab2 -parms varFmts]
set tList2 [gdbGetInfo $tab2 -parms varTypes]

if {[llength $mList1] != [llength $mList2]} \
   {
    puts stderr "Error: mismatched lengths for -var1 and -var2 lists."
    exit 1
   }

foreach v $mList1 \
   {
    if {[lsearch -exact $vList1 $v] < 0} \
       {
        puts stderr "Error Finding Variable $v in Table 1: $inName1"
        exit 1
       }
   }

foreach v $mList2 \
   {
    if {[lsearch -exact $vList2 $v] < 0} \
       {
        puts stderr "Error Finding Variable $v in Table 2: $inName2"
        exit 1
       }
   }

if {![llength $addList]} {set addList $vList2}
if {![llength $newList]} {set newList $addList}

set qList ""

if {![llength $tolList]} \
   {
    foreach v $mList1 \
       {
        set loc [lsearch -exact $vList1 $v]
        set t   [lindex $tList1 $loc]

        if {![strcasecmp $t float]} \
           {
            lappend tolList 0.0 
           } \
        elseif {![strcasecmp $t int]} \
           {
            lappend tolList 0 
           } \
        else \
           {
            lappend tolList case
           }

        lappend qList $t 
       }
   }

set rList ""

foreach v $addList \
   {
    set loc [lsearch -exact $vList2 $v]
    lappend rList [lindex $fList2 $loc]
   }

if {[llength $tolList] != [llength $mList1]} \
   {
    puts stderr "Error: mismatched lengths for -var and -tol lists."
    exit 1
   }

if {$textFlag1} {showTabText $tab1 $outUnit}
if {$textFlag2} {showTabText $tab2 $outUnit}

puts -nonewline $outUnit "VARS  "
foreach v $vList1  {puts -nonewline $outUnit " $v"} 
foreach v $newList {puts -nonewline $outUnit " $prefix$v$postfix"} 
puts $outUnit ""

puts -nonewline $outUnit "FORMAT"
foreach f $fList1 {puts -nonewline $outUnit " $f"} 
foreach f $rList  {puts -nonewline $outUnit " $f"} 
puts $outUnit ""

set nM  [llength $mList1]
set nV1 [llength $vList1]
set nV2 [llength $newList]

puts $outUnit ""

foreach e1 $eList1 \
   {
    set cond ""

    for {set i 0} {$i < $nM} {incr i} \
        {
         set type [lindex $qList   $i]
         set tol  [lindex $tolList $i] 
         set var1 [lindex $mList1  $i]
         set var2 [lindex $mList2  $i]

         set val1 [gdbGet $e1  $var1]

         if {$i} {append cond " && "}

         if {![strcasecmp $type text]} \
            {
             if {![strcasecmp $tol case]} \
                {
                 append cond "!strcmp( $var2, '$val1' )" 
                } \
             else \
                {
                 append cond "!strcasecmp( $var2, '$val1' )" 
                }
            } \
         elseif {![strcasecmp $type int]} \
            {
             append cond "(abs($var2 - $val1) <= $tol)" 
            } \
         else \
            {
             append cond "(abs($var2 - $val1) <= $tol)" 
            } 
        }

    set selList [gdbSelect -from $tab2 -cond $cond]
    set nS [llength $selList]

    if {!$nS} \
       {
        if {$skipFlag} continue
       } \
    elseif {$nS > 1} \
       {
        set selList [lindex $selList 0]
       }

    for {set i 0} {$i < $nV1} {incr i} \
       {
        set f1 [lindex $fList1 $i]
        set v1 [lindex $vList1 $i]

        puts -nonewline $outUnit [format " $f1" [gdbGet $e1 $v1]]
       }

    for {set i 0} {$i < $nV2} {incr i} \
       {
        set f2 [lindex $rList   $i]
        set v2 [lindex $addList $i]
        set j  0

        foreach e2 $selList \
           {
            if {!$j} \
               {
                puts -nonewline $outUnit " "
               } \
            else \
               {
                puts -nonewline $outUnit ","
               }

            puts -nonewline $outUnit [format $f2 [gdbGet $e2 $v2]]
            incr j
           }
       }

    puts $outUnit ""
   }

if {[strcasecmp $outUnit stdout]} {close $outUnit}

exit 0
