#!/bin/csh

set thisVersion = "2020.034.12.16"

set progName = (`getArgD    $argv -progName maxent3D.com`)

@ skipCount = 0

if (`flagLoc $argv -help`) then
   echo "$progName $thisVersion"
   echo "RNMRTK MaxEnt for NUS: Alpha Version (Under Development);"
   echo "Requires Rowland NMR Toolkit:"
   echo " -in          inName  [Auto]    3D Phase-Sensitive Time-Domain Input."
   echo " -sample      sName   [Auto]    NUS Sampling Schedule Input."
   echo " -sampleCount sName   [Auto]    Number of Valid Samples in Schedule."
   echo " -out         outName [Auto]    3D Spectrum Output."
   echo " -ft1         ft1Name [None]    Optional Output for Interferogram Result."
   echo " -script      sName   [nmr.com] Processing Script Output."
   echo " -nus                           Input is NUS (Default)."
   echo " -us                            Input is Uniformly Sampled (US)."
   echo "MaxEnt Parameters:"
   echo " -nThread   nT      [Auto]    Number of Parallel Threads to Use."
   echo " -aim       aVal    [Auto]    MaxEnt AIM Value."
   echo " -def       dVal    [Auto]    MaxEnt DEF Value."
   echo " -lambda    lVal    [Auto]    MaxEnt LAMBDA Value."
   echo " -noiseFT1  nVal    [Auto]    Noise Estimate for Interferogram."
   echo " -aimAdj    aaVal   [1.0]     AIM Parameter Adjustment Factor."
   echo " -defAdj    daVal   [0.2]     DEF Parameter Adjustment Factor."
   echo "Other Options:" 
   echo " -scaleTo   maxVal  [None]    Scale to Given Max Value, or None for No Scaling."
   echo " -noexec                      Create Processing Script, Do Not Process Data."
   echo " -list                        List Script Contents Without Processing."
   echo " -yz                          Exchange Y-Axis and Z-Axis of Result."
   echo " -noyz                        Retain Original Axis Order (Default)."
   echo " -ssNMR                       Set Defaults for Solid-State NMR Data."
   echo "Intermediate Results:"
   echo " -ft1Dir  ft1Dir  [ft1]       Directory for Intermediate FT1 Result."
   echo " -ft2Dir  ft2Dir  [ft2]       Directory for Intermediate FT2 Result."
   echo " -ft3Dir  ft3Dir  [ft3]       Directory for Intermediate FT3 Result."
   echo " -clean                       Delete Intermediate Directories (Default)."
   echo " -noclean                     Retain Intermediate Directories."
   echo "Processing Parameters (Use -x... -y... -z...):"
   echo " -xSOL   sName   [NULL]       Solvent Function Name and Options (X-Axis Only)."
   echo " -xAPOD  aName   [SP]         Window Function Name."
   echo " -xQ1    q1      [0.50]       Window Parameter Q1."
   echo " -xQ2    q2      [0.95]       Window Parameter Q2."
   echo " -xQ3    q3      [2]          Window Parameter Q3 (Default for -yQ3/-zQ3 is 1)."
   echo " -xELB   elb     [0.0]        Exponential Line Broaden, Hz."
   echo " -xGLB   glb     [0.0]        Gaussian Line Broaden, Hz."
   echo " -xGOFF  goff    [0.0]        Gaussian Center, 0.0 to 1.0."
   echo " -xC1    c       [Auto]       First-Point Time-Domain Scale (0.5 1.0 Auto)."
   echo " -xP0    p0      [0.0]        Zero-Order Phase Correction."
   echo " -xP1    p1      [0.0]        First Order Phase Correction."
   echo " -xZFARG zfArgs  [zf=1,auto]  ZF (Zero Fill) Option."
   echo " -xFTARG ftArg   [None]       FT (Fourier Transform) Options (None neg alt)."
   echo " -xEXTX1 x1      [0%]         Extract Range Start (X-Axis Only) (% Hz ppm pts)."
   echo " -xEXTXN xn      [100%]       Extract Range End   (X-Axis Only) (% Hz ppm pts)."
   echo " -xPREFT  fnArgs  [NULL]      Function to Execute Before Any Other Processing."
   echo " -xPOSTFT fnArgs  [NULL]      Function to Execute After FT and Phase Correction."
   echo " -apodDF                      Adjust Window Function for Digital Oversampling."
   echo "NUS Zero-Fill Mode (Time-Domain Extrapolation by MaxEnt):"
   echo " -yNUSZF  zfArgs  [zf=1]      Amount of Y-Axis NUS Zero-Fill Extrapolation."
   echo " -zNUSZF  zfArgs  [zf=1]      Amount of Z-Axis NUS Zero-Fill Extrapolation."
   echo "Baseline Correction Function Name and Options (Use -x... -y... -z...):"
   echo " -xBASEARG_FT1 bArgs  [NULL]  Interferogram Baseline Correction (X-Axis Only)."
   echo " -xBASEARG     bArgs  [NULL]  Spectrum Baseline Correction."
   echo "Processing Parameters for X-Axis Only:"
   echo " -xBASEARG_INIT bArgs [NULL]  Baseline Correction Before Region Extraction."
   echo " -xP0EXT        p0Ext [0.0]   Zero-Order Phase After Region Extraction."
   echo " -xP1EXT        p1Ext [0.0]   First Order Phase After Region Extraction."
   echo "Notes:"
   echo " 1. Options -xSOL -xBASEARG_FT1 and -xBASEARG etc are expanded into NMRPipe"
   echo "    processing function names and command line arguments. Options"
   echo "    should contain no spaces. Flags are separated by commas, and options"
   echo "    with values are given using = signs. For example, solvent filter:"
   echo ""
   echo "      -xSOL     NULL            Becomes: nmrPipe -fn NULL (No Solvent Filter)"
   echo "      -xSOL     SOL             Becomes: nmrPipe -fn SOL"
   echo "      -xSOL     POLY,time       Becomes: nmrPipe -fn POLY -time"
   echo ""
   echo "    For baseline correction:"
   echo ""
   echo "      -xBASEARG NULL            Becomes: nmrPipe -fn NULL (No Correction)"
   echo "      -xBASEARG POLY,auto,ord=0 Becomes: nmrPipe -fn POLY -auto -ord 0"
   echo ""
   echo " 2. Options -xZFARG -xFTARG etc are expanded to command line arguments for ZF"
   echo "    and FT, for example:"
   echo ""
   echo "      -xZFARG zf=2,auto         Becomes: nmrPipe -fn ZF -zf 2 -auto"
   echo "      -xFTARG neg               Becomes: nmrPipe -fn FT -neg"
   echo "      -xFTARG neg,alt           Becomes: nmrPipe -fn FT -neg -alt"
   echo ""
   echo " 3. When first-point time-domain scaling is set to Auto (-xC1 Auto etc),"
   echo "    the scaling value will be set to 0.5 if the first-order phase correction"
   echo "    for that dimension is 0+/-10 degrees, or to 1.0 otherwise."
   echo ""
   echo " 4. If the X-Axis label of the input spectrum starts with 'HN', default"
   echo "    parameters will be adjusted as follows:"
   echo ""
   echo "       -xSOL SOL -xBASEARG POLY,auto -xEXTX1 3% -xEXTXN 47%"
   echo ""
   echo " 5. If the -ssNMR flag is used, default parameters will be:"
   echo ""
   echo "       -xSOL NULL -xBASEARG NULL"
   echo ""
   echo " 6. If Y-Axis or Z-Axis P1 is 360 +/- 10 degrees, default baseline will be:"
   echo ""
   echo "       -yBASEARG POLY,auto,ord=0  (For -yP1 set to 360 +/- 10)"
   echo "       -zBASEARG POLY,auto,ord=0  (For -zP1 set to 360 +/- 10)"
   echo ""
   echo " 7. Commonly, only region of interest, FT, and phase correction options" 
   echo "    need to be provided. Typical example:"
   echo ""
   echo "    $progName -yz \"
   echo "     -xP0 8 -xP1 0 -xEXTX1 9.9ppm -xEXTXN 5.4ppm \"
   echo "     -zFTARG alt"
   echo ""
   exit 0
endif

set vFlag     = (`getLogArgD $argv -verb    -noverb verb noverb`)
set listFlag  = (`getLogArgD $argv -nolist  -list    0   1`)
set ssNMRFlag = (`getLogArgD $argv -nossNMR -ssNMR   0   1`)
set cleanFlag = (`getLogArgD $argv -clean   -noclean 1   0`)
set execFlag  = (`getLogArgD $argv -exec    -noexec  1   0`)
set yzFlag    = (`getLogArgD $argv -noyz    -yz      0   1`)

set aimVal     = (`getArgD $argv -aim       Auto`)
set defVal     = (`getArgD $argv -def       Auto`)
set lambdaVal  = (`getArgD $argv -lambda    Auto`)
set noiseVal   = (`getArgD $argv -noiseFT1  Auto`)
set aimAdj     = (`getArgD $argv -aimAdj    1.0`)
set defAdj     = (`getArgD $argv -defAdj    0.2`)
set nThreads   = (`getArgD $argv -nThread   Auto`)

set inName      = (`getArgD $argv -in          Auto`)
set parentName  = (`getArgD $argv -parent      None`)
set outName     = (`getArgD $argv -out         Auto`)
set sampleName  = (`getArgD $argv -sample      Auto`)
set sampleCount = (`getArgD $argv -sampleCount Auto`)
set scriptName  = (`getArgD $argv -script      nmr.com`)
set prefix      = (`getArgD $argv -prefix      maxent_rnmrtk$$_`)

set ft1Dir     = (`getArgD $argv -ft1Dir    ft1`)
set ft2Dir     = (`getArgD $argv -ft2Dir    ft2`)
set ft3Dir     = (`getArgD $argv -ft3Dir    ft3`)
set quadDir    = (`getArgD $argv -quadDir   quad`)
set tmpDir     = (`getArgD $argv -tmpDir    tmp`)

set ft1Name    = (`getArgD $argv -ft1       None`)
set ft2Name    = (`getArgD $argv -ft2       $ft2Dir/test%05d.ft2`)
set ft3Name    = (`getArgD $argv -ft3       $ft3Dir/test%05d.ft3`)
set tmpName    = (`getArgD $argv -tmp       $tmpDir/test%05d.ft3`)
set quadName   = (`getArgD $argv -tmpquad   $quadDir/test%05d.ft3`)

set scaleToVal   = (`getArgD $argv -scaleTo   None`)
set apodDF       = ""
set constAIMFlag = 0
set nusFlag      = 1

if (`string csame $sampleName None`) then
   @ nusFlag = 0
endif

if (`flagLoc $argv -nus`) then
   @ nusFlag = 1
endif

if (`flagLoc $argv -us`) then
   @ nusFlag = 0
endif

if ($nusFlag == 0) then
   set sampleName = None
endif

if (`flagLoc $argv -apodDF`) then
   set apodDF = "-df"
endif

if (`flagLoc $argv -noapodDF`) then
   set apodDF = ""
endif

if (`flagLoc $argv -constAIM`) then
   set constAIMFlag = 1 
endif

if (`flagLoc $argv -noconstAIM` || `flagLoc $argv -constLAMBDA`) then
   set constAIMFlag = 0
endif

if ($listFlag) then
   if ($nusFlag) then
      echo "#\!/bin/csh"
      echo ""
   endif

   echo "#"
   echo "# Generated by $progName Version $thisVersion"
   echo "#"
   prettyPrintCmnd.tcl $0 `stripArgs -list -noexec -constAIM -parent= -- $argv`
   echo "#"
   echo ""
else
   if (`string same $vFlag verb`) then
      echo ""
      echo "$progName Version:           $thisVersion"
   endif
endif

if (`string csame $inName Auto`) then
   set inName = test.fid
   set fList  = (fid/test%03d.fid data/test%03d.fid fid/spec%03d.fid data/spec%03d.fid fid/test%04d.fid data/test%04d.fid test.fid spec.fid)

   foreach i ($fList)
      set thisName = (`nmrPrintf $i 1`)

      if (-e $thisName) then
         set inName = $i
         break
      endif
   end
endif

if (`string csame $sampleName Auto`) then
   set fList = (nuslist sampling.sch `find ./ -maxdepth 1 -name schedule\[0-9\]\* -print` `find ./ -maxdepth 1 -name \*.hdr_3 -print` vclist fake.nuslist)

   foreach f ($fList)
      if (-e $f) then
         set sampleName = $f
         break
      endif
   end
endif

if (`string csame $sampleName Auto`) then
   nmrPrintf -stderr "%s\n" "$progName Error Finding Auto Mode Sample Schedule."
   exit 1
endif

if (`string notcsame $sampleName None`) then
   if (!(-e $sampleName)) then
      nmrPrintf -stderr "%s\n" "$progName Error Finding Sample Schedule $sampleName"
      exit 1
   endif
endif

set origInName = $inName

if (`string csame $outName Auto`) then
   set outName = maxent/test%03d.ft3
endif

if (`string same $vFlag verb`) then
   if ($listFlag) then
   else
      echo "$progName Input:             $inName"
      echo "$progName Sampling Schedule: $sampleName"
      echo "$progName FT Output:         $ft3Name"
      echo "$progName MaxEnt Output:     $outName"
      echo "$progName Script Output:     $scriptName"
      echo ""
   endif
endif

if (`string csame $parentName None`) then
   set thisName = (`nmrPrintf $origInName 1`)
else
   set thisName = (`nmrPrintf $parentName 1`)
endif

if (!(-e $thisName)) then
   nmrPrintf -stderr "%s\n" "$progName Error: Missing Input $inName (parent $parentName)."
   exit 1
endif

set xLAB = (`getParm -in $thisName -txt -fmt %s -parm NDLABEL -dim CUR_XDIM`)

if (`string match "HN*" $xLAB`) then
   set uxSOL     = "SOL"
   set uxBASEARG = "POLY,auto" 
   set uxEXTX1   = "3%"
   set uxEXTXN   = "47%"
else
   set uxSOL     = "NULL"
   set uxBASEARG = "NULL"
   set uxEXTX1   = "0%"
   set uxEXTXN   = "100%"
endif

if ($ssNMRFlag) then
   set uxSOL     = "NULL"
   set uxBASEARG = "NULL"
endif

set xSOL   = (`getArgD $argv -xSOL   $uxSOL`)
set xAPOD  = (`getArgD $argv -xAPOD  SP`)
set xQ1    = (`getArgD $argv -xQ1    0.5`)
set xQ2    = (`getArgD $argv -xQ2    0.95`)
set xQ3    = (`getArgD $argv -xQ3    2`)
set xELB   = (`getArgD $argv -xELB   0.0`)
set xGLB   = (`getArgD $argv -xGLB   0.0`)
set xGOFF  = (`getArgD $argv -xGOFF  0.0`)
set xC1    = (`getArgD $argv -xC1    Auto`)
set xZFARG = (`getArgD $argv -xZFARG zf=1,auto`)
set xFTARG = (`getArgD $argv -xFTARG None`)
set xP0    = (`getArgD $argv -xP0    0.0`)
set xP1    = (`getArgD $argv -xP1    0.0`)
set xP0EXT = (`getArgD $argv -xP0EXT 0.0`)
set xP1EXT = (`getArgD $argv -xP1EXT 0.0`)
set xEXTX1 = (`getArgD $argv -xEXTX1 $uxEXTX1`)
set xEXTXN = (`getArgD $argv -xEXTXN $uxEXTXN`)

if (`string same $xP0EXT 0.0` && `string same $xP1EXT 0.0`) then
   @ usePSEXTFlag = 0
else
   @ usePSEXTFlag = 1
endif

set xBASEARG_INIT = (`getArgD $argv -xBASEARG_INIT  NULL`)
set xBASEARG_FT1  = (`getArgD $argv -xBASEARG_FT1   NULL`)
set xBASEARG      = (`getArgD $argv -xBASEARG       $uxBASEARG`)

if (`string csame $xC1 Auto`) then
   set v = (`nmrPrintf "%f" $xP1`)
   @ i   = (`IMATH "abs($v) < 10.0"`)

   if (!($i)) then
      set xC1 = 1.0
   else
      set xC1 = 0.5
   endif
endif

set yAPOD  = (`getArgD $argv -yAPOD  SP`)
set yQ1    = (`getArgD $argv -yQ1    0.50`)
set yQ2    = (`getArgD $argv -yQ2    0.95`)
set yQ3    = (`getArgD $argv -yQ3    1`)
set yELB   = (`getArgD $argv -yELB   0.0`)
set yGLB   = (`getArgD $argv -yGLB   0.0`)
set yGOFF  = (`getArgD $argv -yGOFF  0.0`)
set yC1    = (`getArgD $argv -yC1    Auto`)
set yZFARG = (`getArgD $argv -yZFARG zf=1,auto`)
set yNUSZF = (`getArgD $argv -yNUSZF zf=1`)
set yFTARG = (`getArgD $argv -yFTARG None`)
set yP0    = (`getArgD $argv -yP0    0.0`)
set yP1    = (`getArgD $argv -yP1    0.0`)
set yNUSZF = (`getArgD $argv -yNUSZF zf=1`)

if (`IMATH "near( $yP1, 360, 10 )"`) then
   set uyBASEARG = "POLY,auto,ord=0"
else
   set uyBASEARG = "NULL"
endif

set yBASEARG = (`getArgD $argv -yBASEARG $uyBASEARG`)

if (`string csame $yC1 Auto`) then
   set v = (`nmrPrintf "%f" $yP1`)
   @ i   = (`IMATH "abs($v) < 10.0"`)

   if (!($i)) then
      set yC1 = 1.0
   else
      set yC1 = 0.5
   endif
endif

set zAPOD  = (`getArgD $argv -zAPOD  SP`)
set zQ1    = (`getArgD $argv -zQ1    0.50`)
set zQ2    = (`getArgD $argv -zQ2    0.95`)
set zQ3    = (`getArgD $argv -zQ3    1`)
set zELB   = (`getArgD $argv -zELB   0.0`)
set zGLB   = (`getArgD $argv -zGLB   0.0`)
set zGOFF  = (`getArgD $argv -zGOFF  0.0`)
set zC1    = (`getArgD $argv -zC1    Auto`)
set zZFARG = (`getArgD $argv -zZFARG zf=1,auto`)
set zNUSZF = (`getArgD $argv -zNUSZF zf=1`)
set zFTARG = (`getArgD $argv -zFTARG None`)
set zP0    = (`getArgD $argv -zP0    0.0`)
set zP1    = (`getArgD $argv -zP1    0.0`)
set zNUSZF = (`getArgD $argv -zNUSZF zf=1`)

if (`IMATH "near( $zP1, 360, 10 )"`) then
   set uzBASEARG = "POLY,auto,ord=0"
else
   set uzBASEARG = "NULL"
endif

set zBASEARG = (`getArgD $argv -zBASEARG $uzBASEARG`)

if (`string csame $zC1 Auto`) then
   set v = (`nmrPrintf "%f" $zP1`)
   @ i   = (`IMATH "abs($v) < 10.0"`)

   if (!($i)) then
      set zC1 = 1.0
   else
      set zC1 = 0.5
   endif
endif

if (`string csame $xAPOD SP`) then
   if (`string same $xGOFF 0.0`) then
      set xWinInfo = "SP -off $xQ1 -end $xQ2 -pow $xQ3 -elb $xELB -glb $xGLB -c $xC1 $apodDF"
   else
      set xWinInfo = "SP -off $xQ1 -end $xQ2 -pow $xQ3 -elb $xELB -glb $xGLB -goff $xGOFF -c $xC1 $apodDF"
   endif
else
   set xWinInfo = "APOD -qName $xAPOD -q1 $xQ1 -q2 $xQ2 -q3 $xQ3 -elb $xELB -glb $xGLB -goff $xGOFF -c $xC1 $apodDF"
endif

if (`string csame $yAPOD SP`) then
   if (`string same $yGOFF 0.0`) then
      set yWinInfo = "SP -off $yQ1 -end $yQ2 -pow $yQ3 -elb $yELB -glb $yGLB -c $yC1"
   else
      set yWinInfo = "SP -off $yQ1 -end $yQ2 -pow $yQ3 -elb $yELB -glb $yGLB -goff $yGOFF -c $yC1"
   endif
else
   set yWinInfo = "APOD -qName $yAPOD -q1 $yQ1 -q2 $yQ2 -q3 $yQ3 -elb $yELB -glb $yGLB -goff $yGOFF -c $yC1"
endif

if (`string csame $zAPOD SP`) then
   if (`string same $zGOFF 0.0`) then
      set zWinInfo = "SP -off $zQ1 -end $zQ2 -pow $zQ3 -elb $zELB -glb $zGLB -c $zC1"
   else
      set zWinInfo = "SP -off $zQ1 -end $zQ2 -pow $zQ3 -elb $zELB -glb $zGLB -goff $zGOFF -c $zC1"
   endif
else
   set zWinInfo = "APOD -qName $zAPOD -q1 $zQ1 -q2 $zQ2 -q3 $zQ3 -elb $zELB -glb $zGLB -goff $zGOFF -c $zC1"
endif

set cleanList = "$ft2Dir $ft3Dir $quadDir $tmpDir"

if (`string csame $ft1Name None`) then
   set ft1Name   = $ft1Dir/test%05d.ft1
   set cleanList = "$cleanList $ft1Dir"
endif

set thisOutName = $outName

if ($yzFlag) then
   set outDim = y
else
   set outDim = z
endif

set tkFT1Name      = ${prefix}ft1.sec
set tkFT1ParmName  = ${prefix}ft1.par
set tkFT3Name      = ${prefix}ft3.sec
set tkFT3ParmName  = ${prefix}ft3.par
set tkParmName     = ${prefix}parm.txt
set tkFTScriptName = ${prefix}tmpft.com
set tkSchedName    = ${prefix}.sched
set tkLogNameA     = ${prefix}A.log
set tkLogNameL     = ${prefix}L.log

set zfSchedName    = ${prefix}_nusZF.nuslist
set zfOutName      = ${prefix}_nusZF.fid

set cleanList = "$cleanList $tkFT1Name $tkFT3Name $tkFT1ParmName $tkFT3ParmName $tkSchedName $tkFTScriptName $zfSchedName $zfOutName"

set ySIGNARGV = "FT `expandArg $yFTARG`"
set zSIGNARGV = "FT `expandArg $zFTARG`"

if ($listFlag) then
   if (`string csame $sampleName None`) then
      if (`string same $vFlag verb`) then
         echo 'echo "NUS Zero-Fill Extrapolation: Creating Zero Fill and Mask Data ..."'
         echo ''
      endif

      prettyPrintCmnd.tcl -nocomment nusZF.com -in $inName -out $zfOutName -mask None -sample $zfSchedName \
                          -xNUSZF None -yNUSZF $yNUSZF -zNUSZF $zNUSZF -$vFlag

      echo ''

      set sampleName = $zfSchedName
   endif

   if (`string same $vFlag verb`) then
      echo 'echo "Performing Initial Fourier Processing ..."'
      echo ''
   endif

   echo "#"
   echo "# Arguments duplicated later on the command line are ignored:"
   echo ""

   prettyPrintCmnd.tcl -nocomment basicFT3.com -noyz -ft1 $ft1Name -out $ft3Name -script $tkFTScriptName \
                       -yZFARG $yZFARG -zZFARG $zZFARG `stripArgs -list -noexec -ft1= -out= -yz -maxent -nusZF -- $argv`

   if (`string same $vFlag verb`) then
      echo ''
      echo 'echo "Expanding for Imaginary Data ..."'
      echo 'echo ""'
   endif

   echo ""
   echo "xyz2pipe -in $ft3Name -z -$vFlag \"
   echo "| nmrPipe -fn NULL -ai \"
   echo "| pipe2xyz -out $quadName -z"
   echo ""
   echo "xyz2pipe -in $quadName -y -$vFlag \"
   echo "| nmrPipe -fn NULL -ai \"
   echo "| pipe2xyz -out $quadName -y -inPlace"
   echo ""

   if (`string same $vFlag verb`) then
      echo 'echo ""'
      echo 'echo "Performing First-Point Scaling and Sign Adjustment ..."'
      echo 'echo ""'
      echo ''
   endif

   echo "xyz2pipe -in $ft1Name -$vFlag \"
   echo "| nmrPipe -fn TP \"
   echo "| nmrPipe -fn SP -off 0.5 -end 0.5 -pow 1 -c $yC1 \"

   if (`flagLoc $ySIGNARGV -alt`) then
      echo "| nmrPipe -fn SIGN -alt \"
   endif

   if (`flagLoc $ySIGNARGV -neg` == 0) then
      echo "| nmrPipe -fn SIGN -i \"
   endif

   echo "| nmrPipe -fn TP \"
   echo "| nmrPipe -fn ZTP \"
   echo "| nmrPipe -fn SP -off 0.5 -end 0.5 -pow 1 -c $zC1 \"

   if (`flagLoc $zSIGNARGV -alt`) then
      echo "| nmrPipe -fn SIGN -alt \"
   endif

   if (`flagLoc $zSIGNARGV -neg` == 0) then
      echo "| nmrPipe -fn SIGN -i \"
   endif

   echo "| nmrPipe -fn ZTP \"
   echo "| pipe2xyz -out $ft1Name -inPlace"

   echo ""
   echo "set xTSize  = ("\`getParm -in $ft3Name -parm NDAPOD -dim CUR_XDIM -fmt %.0f\`")"
   echo "set yTSize  = ("\`getParm -in $ft3Name -parm NDAPOD -dim CUR_YDIM -fmt %.0f\`")"
   echo "set zTSize  = ("\`getParm -in $ft3Name -parm NDAPOD -dim CUR_ZDIM -fmt %.0f\`")"
   echo ""
   echo "set xFSize  = ("\`getParm -in $ft3Name -parm NDSIZE -dim CUR_XDIM -fmt %.0f\`")"
   echo "set yFSize  = ("\`getParm -in $ft3Name -parm NDSIZE -dim CUR_YDIM -fmt %.0f\`")"
   echo "set zFSize  = ("\`getParm -in $ft3Name -parm NDSIZE -dim CUR_ZDIM -fmt %.0f\`")"
   echo ""

   if (`string csame $nThreads Auto`) then
      echo 'set nThreads    = (`getThreadCount.tcl -f 0.5`)'
   else
      echo "set nThreads    = $nThreads"
   endif

   echo "set nusInfo     = ("\`cat $sampleName\`")"

   if (`string csame $sampleCount Auto`) then
      echo 'set n           = $#nusInfo'
      echo 'set sampleCount = `IMATH "$n/2"`'
   else
      echo 'set n           =' `IMATH "2*$sampleCount"`
   endif

  if (`string same $vFlag verb`) then
      echo ''
      echo 'echo ""'
      echo 'echo "Converting Sampling Schedule to RNMRTK Format ..."'
   endif

   echo ""
   echo "/bin/rm -f $tkSchedName"
   echo ""
   echo "sample_schedule_changer.prl $sampleName 2 1 > $tkSchedName"
   echo ""
   echo "#"
   echo "# @ i = 0"
   echo "#"
   echo '# while ($i < $n)'
   echo '#   @ i++'
   echo '#   @ iy = `IMATH "$nusInfo[$i] + 1"`'
   echo '#   @ i++'
   echo '#   @ iz = `IMATH "$nusInfo[$i] + 1"`'
   echo '#'
   echo '#   echo `nmrPrintf " %3d %3d" $iy $iz` >>' $tkSchedName
   echo "# end"
   echo ""

   if (`string same $vFlag verb`) then
      echo 'echo "Converting NMRPipe-Format Interferogram to RNMRTK Format ..."'
      echo ''
   endif

   echo 'setenv MP_SET_NUMTHREADS $nThreads'
   echo ""
   echo "spectrum-translator -sp $ft1Name -st nmrpipe -df $tkFT1Name -dt rowland --force"
   echo ""

   if (`string csame $noiseVal Auto`) then
      echo "set noiseFT1 = ("\`specStat.com -in $ft1Name -brief -stat vEstNoise\`")"
   else
      echo "set noiseFT1 = $noiseVal"
   endif

   echo ""
   echo 'set noiseFT1 = (`nmrPrintf %f $noiseFT1`)'
   echo "set aimAdj   = (`nmrPrintf %f $aimAdj`)"
   echo "set defAdj   = (`nmrPrintf %f $defAdj`)"
   echo ""

   if (`string csame $aimVal Auto`) then
      echo 'set aimVal   = (`MATH "$aimAdj*$noiseFT1"`)'
   else
      echo "set aimVal   = $aimVal"
   endif

   if (`string csame $defVal Auto`) then
      echo 'set defVal   = (`MATH "$defAdj*$noiseFT1*sqrt($sampleCount)/sqrt($yFSize*$zFSize)"`)'
   else
      echo "set defVal   = $defVal"
   endif

   echo ""
   echo 'set aimVal   = (`nmrPrintf %f $aimVal`)'
   echo 'set defVal   = (`nmrPrintf %f $defVal`)'
   echo ""

   echo ""
   echo "set yP0      = $yP0"
   echo "set yP1      = $yP1"
   echo "set zP0      = $zP0"
   echo "set zP1      = $zP1"
   echo ""

   if (`string csame $lambdaVal Auto`) then
      if (`string csame $vFlag verb`) then  
         echo 'echo ""'
         echo 'echo "Estimating LAMBDA Value By Constant AIM ..."'
      endif

      echo ""
      echo 'echo "DEBUG       2"                     > ' $tkParmName
      echo 'echo "NLOOPS      1000"                  >>' $tkParmName
      echo 'echo "AIM         $aimVal"               >>' $tkParmName
      echo 'echo "DEF         $defVal"               >>' $tkParmName
      echo 'echo "SCALEFIRST  1.0"                   >>' $tkParmName
      echo 'echo "NOUT        $yFSize $zFSize"       >>' $tkParmName
      echo 'echo "PHASE       0.0 0.0 0.0 0.0"       >>' $tkParmName
      echo 'echo "LW          0.0 0.0"               >>' $tkParmName
      echo 'echo "JVALUE      0.0 0.0"               >>' $tkParmName
      echo 'echo "SCHED       './$tkSchedName'" >>' $tkParmName
      echo ""
      echo "/bin/rm -f $tkLogNameA $tkLogNameL"
      echo ""

      if (`string csame $vFlag verb`) then
         echo 'section -c $xFSize 2 $yFSize 2 $zFSize 2' 
         echo "rnmrtk LOAD  $tkFT1Name"
         echo ""
         echo 'echo ""'
         echo 'echo "MaxEnt Processing, $xFSize Planes:"'
         echo ""
         echo "msa2d  t1 t2 ./$tkParmName | tee -a $tkLogNameA | showMaxentProgress.tcl"
      else
         echo 'section -c $xFSize 2 $yFSize 2 $zFSize 2  >>' $tkLogNameA
         echo "rnmrtk LOAD  $tkFT1Name        >>" $tkLogNameA
         echo "msa2d  t1 t2 ./$tkParmName     >>" $tkLogNameA
      endif

      echo ""
      echo 'set lambdaList = (`fgrep "Lambda ="' $tkLogNameA '| sed "s/=/= /g" | awk' \' '{print $6}' \' '| sort -n | tail -n 10`)'
      echo 'set lambdaVal  = (`getStat.tcl -x $lambdaList -stat Avg -noverb`)'
      echo ""
 
      echo 'if (`string isfloat $lambdaVal` == 0) then'
      echo '   echo "MaxEnt3D Error Finding Lambda Value. Aborting."'
      echo '   exit 1'
      echo 'endif'
   else
      echo "set lambdaVal   = $lambdaVal"
   endif

   echo 'set lambdaVal   = (`nmrPrintf %f $lambdaVal`)'

   if (`string csame $vFlag verb`) then  
      echo ''
      echo 'echo ""'

      if ($constAIMFlag) then
         echo 'echo "'$progName 'Mode:          Constant AIM"'
      else
         echo 'echo "'$progName 'Mode:          Constant LAMBDA"'
      endif

      echo 'echo "'$progName 'FT1 Noise:     $noiseFT1"'

      if ($constAIMFlag) then
         echo 'echo "'$progName 'DEF:           $defVal"'
         echo 'echo "'$progName 'AIM:           $aimVal"'
      else
         echo 'echo "'$progName 'DEF:           $defVal"'
         echo 'echo "'$progName 'LAMBDA         $lambdaVal"'
      endif

      echo 'echo ""'
   endif
 
   echo ""
   echo 'echo "DEBUG       2"                     > ' $tkParmName
   echo 'echo "NLOOPS      1000"                  >>' $tkParmName

   if ($constAIMFlag) then
      echo 'echo "AIM         $aimVal"               >>' $tkParmName
      echo 'echo "DEF         $defVal"               >>' $tkParmName
   else
      echo 'echo "DEF         $defVal"               >>' $tkParmName
      echo 'echo "LAMBDA      $lambdaVal"            >>' $tkParmName
   endif

   echo 'echo "SCALEFIRST  1.0"                   >>' $tkParmName
   echo 'echo "NOUT        $yFSize $zFSize"       >>' $tkParmName
#  echo 'echo "PHASE       $yP0 $yP1 $zP0 $zP1"   >>' $tkParmName
   echo 'echo "PHASE       0.0 0.0 0.0 0.0"       >>' $tkParmName
   echo 'echo "LW          0.0 0.0"               >>' $tkParmName
   echo 'echo "JVALUE      0.0 0.0"               >>' $tkParmName
   echo 'echo "SCHED       './$tkSchedName'" >>' $tkParmName
   echo ""

   if (`string csame $vFlag verb`) then
      echo 'section -c $xFSize 2 $yFSize 2 $zFSize 2 >>' $tkLogNameL
      echo "rnmrtk LOAD  $tkFT1Name"
      echo ""
      echo 'echo ""'
      echo 'echo "MaxEnt Processing, $xFSize Planes:"'
      echo ""
      echo "msa2d  t1 t2 ./$tkParmName | tee -a $tkLogNameL | showMaxentProgress.tcl"
      echo "rnmrtk SAVE  $tkFT3Name" 
   else
      echo 'section -c $xFSize 2 $yFSize 2 $zFSize 2 >>' $tkLogNameL
      echo "rnmrtk LOAD  $tkFT1Name       >>" $tkLogNameL
      echo "msa2d  t1 t2 ./$tkParmName    >>" $tkLogNameL
      echo "rnmrtk SAVE  $tkFT3Name       >>" $tkLogNameL
   endif

   echo ""

   if (`string same $vFlag verb`) then
      echo 'echo ""'
      echo 'echo "Converting MaxEnt Spectrum to NMRPipe Format ..."'
      echo ''
   endif

   echo "spectrum-translator -sf $tkFT3Name -st rowland -dp $tmpName -dt nmrpipe --force"
   echo ""

   if (`string same $vFlag verb`) then
      echo 'echo "Updating Header ..."'
      echo 'echo ""'
      echo ''
   endif

   echo "fixRNMRTK.com -parent $quadName -in $tmpName -copy"
   echo ""

   if (`string same $vFlag verb`) then
      echo 'echo "Performing Final Phase and Baseline Correction ..."'
      echo 'echo ""'
      echo ''
   endif

   if ($yzFlag) then
      echo "xyz2pipe -in $tmpName -$vFlag \"
      echo "| nmrPipe -fn TP \"
      echo "| nmrPipe -fn" PS -p0 $yP0 -p1 $yP1 -di "\"
      echo "| nmrPipe -fn TP \"
      echo "| nmrPipe -fn ZTP \"
      echo "| nmrPipe -fn" PS -p0 $zP0 -p1 $zP1 -di "\"
      echo "| nmrPipe -fn ZTP \"

      if (`string csame $xBASEARG NULL` == 0) then
         echo "| nmrPipe -fn `expandArg0 $xBASEARG` \"
      endif

      echo "| nmrPipe -fn TP \"

      if (`string csame $yBASEARG NULL` == 0) then
         echo "| nmrPipe -fn `expandArg0 $yBASEARG` \"
      endif

      echo "| nmrPipe -fn ZTP \"

      if (`string csame $zBASEARG NULL` == 0) then
         echo "| nmrPipe -fn `expandArg0 $zBASEARG` \"
      endif

      echo "| nmrPipe -fn TP \"
      echo "| pipe2xyz -out $outName -ov"
   else
      echo "xyz2pipe -in $tmpName \"
      echo "| nmrPipe -fn TP \"
      echo "| nmrPipe -fn" PS -p0 $yP0 -p1 $yP1 -di "\"
      echo "| nmrPipe -fn TP \"
      echo "| nmrPipe -fn ZTP \"
      echo "| nmrPipe -fn" PS -p0 $zP0 -p1 $zP1 -di "\"
      echo "| nmrPipe -fn ZTP \"

      if (`string csame $xBASEARG NULL` == 0) then
         echo "| nmrPipe -fn `expandArg0 $xBASEARG` \"
      endif  

      if (`string csame $yBASEARG NULL` == 0) then
         echo "| nmrPipe -fn TP \"
         echo "| nmrPipe -fn `expandArg0 $yBASEARG` \"
         echo "| nmrPipe -fn TP \"
      endif

      if (`string csame $zBASEARG NULL` == 0) then
         echo "| nmrPipe -fn ZTP \"
         echo "| nmrPipe -fn `expandArg0 $zBASEARG` \"
         echo "| nmrPipe -fn ZTP \"
      endif

      echo "| pipe2xyz -out $outName -ov"
   endif
endif

if ($listFlag) then
   if (`string csame $scaleToVal None`) then
      @ skipCount++
   else
      echo ""
      echo rescale.com -in $outName -out $outName -max $scaleToVal
      echo ""
   endif
endif

if ($listFlag) then
   if ($cleanFlag) then
      echo ""
      echo "/bin/rm -rf $cleanList"
   else
      echo ""
      echo "#"
      echo "# /bin/rm -rf $cleanList"
   endif
else
   $progName $* -list > $scriptName
   chmod a+rx $scriptName

   if ($execFlag) then
      $scriptName

      if (`string same $vFlag verb`) then
         echo ""
      endif
   endif
endif

exit 0
