#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
{
 puts "MFR: Molecular Fragment Replacement Search."
 puts " -segLength  fragN   \[7\]          Fragment Size, Residues."
 puts " -scoreCount scoreN  \[10\]         Number of Fragments to Save."
 puts " -r1         r1      \[First\]      First Residue in Search Range."
 puts " -rN         rN      \[Last\]       Last  Residue in Search Range."
 puts " -ref        refName \[ext.pdb\]    Reference PDB Structure;"
 puts "                                  Used for Sequence."
 puts " -excl       eList   \[None\]       List of Proteins Exclude."
 puts " -pHomo      pHomo   \[100.0\]      Seq Homology Limit (%) to Exclude."
 puts "Input Tables:"
 puts " -dc       dcList  \[dObs*.tab\]    Input Dipolar Coupling (DC) Tables."
 puts " -cs       csName  \[csObs.tab\]    Input Chemical Shift Table."
 puts " -j        jName   \[jcoup.tab\]    Input J-Coupling Table."
 puts " -torsion  tName   \[torsions.tab\] Input Torsion Table."
 puts " -out      outName \[mfr.tab\]      Output Fragment Table."
 puts "Weighting Factors:"
 puts " -ramaThresh ramaT    \[0.00\]      Ramachandran Map Threshold."
 puts " -ramaW      ramaW    \[0.02\]      Ramachandran Map Weight."
 puts " -dcW        dcW      \[1.00\]      Dipolar Coupling (DC) Weight."
 puts " -ddcW       ddcW     \[0.00\]      DC Derivative Weight."
 puts " -jW         jW       \[0.01\]      J Weight."
 puts " -torW       torW     \[0.05\]      Torsion Weight."
 puts " -homoW      homoW    \[0.01\]      Residue Type Homology Weight."
 puts " -rmsW       rmsW     \[0.00\]      Backbone Coordinate RMS Weight."
 puts " -phiPsiW    phiPsiW  \[0.00\]      Backbone Angle RMS Weight."
 puts " -alpha      expAlpha \[3.00\]      Weight for MFR Da/Dr Estimate."
 puts " -dcN        dcNMin   \[15\]        Minimum DC Count For Tensor."
 puts " -retry      dcRetry  \[100\]       Number of Retries, Tensor Fit."
 puts "Chemical Shift (CS) Options:"
 puts " -csThresh   csThresh \[3.00\]      Max Allowable CS Score."
 puts " -csW        csW      \[0.50\]      Overall CS Score Weight."
 puts " -csWList    awList   \[None\]      Atom Name/CS Weight Pair List."
 puts " -csClip     csClipF  \[3.0\]       CS Clip Factor, in Std Deviations;"
 puts "                                  Use 0.0 for No Clipping."
 puts "Dipolar Tensor Calculation Modes:"
 puts " -dcSVD                          Use SVD (Default)."
 puts " -dcDa                           Use Fixed Da."
 puts " -dcDaDr                         Use Fixed Da and Dr." 
 puts " -dcRh                           Use Fixed Rh (Rh = Dr/Da)."
 puts " -dcDaDrRot                      Use Fixed Da, Dr, Rotation."
 puts "Optional Dipolar Coupling Tensor Reference Values:"
 puts " -daRef     daList               Da Values for Each Tensor."
 puts " -drRef     drList               Dr Values for Each Tensor."
 puts " -rhRef     rhList               Rh Values for Each Tensor."
 puts "Optional Tensor Orientations, Relative to First Tensor:"
 puts " -rxRef     rxList               X Rotation Offsets."
 puts " -ryRef     ryList               Y Rotation Offsets."
 puts " -rzRef     rzList               Z Rotation Offsets."
 puts "Use of Backbone Coordinate or Angle RMS for Report or Search:"
 puts " -rms                            Report Backbone Coord RMS."
 puts " -phiPsi                         Report Backbone Angle RMS."
 puts " -rmsFast                        Fast, Less Accurate Coord RMS."
 puts " -rmsSearch                      Search by Coord RMS Only."
 puts " -phiPsiSearch                   Search by Angle RMS Only."
 puts "Analysis of Structural Variation Within Group:"
 puts " -dist     fDist    \[0.5\]        Max RMSD Between Similar Fragments."
 puts "Database Location:"
 puts " -pdbList  listName \[PDBH_TAB\]       List Table of PDB Files."
 puts " -pdbDir   pdbDir   \[PDBH_DIR\]       Location of PDB Files."
 puts " -csDir    csDir    \[NMRBASE/pdb/cs\] Shift Tables for PDBs."
 puts "Special Modes for Rescoring MFR Table or Refined Fragments:"
 puts "Resets Default Output Name to mfrScore.tab:"
 puts " -rescore  mfrInName \[None\]      MFR Table for Rescore."
 puts " -fragDir  fragDir   \[None\]      Directory of Refined Fragments."
 puts " -initDir  initDir   \[init\]      Directory of Initial Fragments."
 puts " -useFragName                    Use FRAG_NAME as PDB Input Name."
 exit 0 
}

#
#-----------------------------------------------------------------------------

proc isSimilarFrag { i j } \
{
   global dMatrix fragDist 

   if {$dMatrix($i,$j) <= $fragDist} {return 1}

   return 0
}

proc updateMFREntries { mfrName pdbDir fragDir fragNameFlag } \
{
   if {![file exists $mfrName]} \
      {
       puts stderr "Error Finding MFR Input $inName."
       return 1
      }

   set mfrTab [gdbRead table -in $mfrName]
   set eList  [gdbGetInfo $mfrTab -parms entries]
   set dList  ""

   foreach entry $eList \
      {
       set r1 [gdbGet $entry D_RES1]
       set rN [gdbGet $entry D_RESN]
       set id [gdbGet $entry RANK]

       if {![strcasecmp $fragDir None]} \
          {
           set thisName [addExt $pdbDir/[gdbGet $entry PDB_NAME] pdb]
          } \
       else \
          {
           if {$fragNameFlag} \
              {
               set thisName [addExt $fragDir/[gdbGet $entry FRAG_NAME] pdb]
              } \
           else \
              {
               set fmt "%s/frag_%03d_%03d_%02d.pdb"
               set thisName [format $fmt $fragDir $r1 $rN $id]
              }
          }

       if {![file exists $thisName]} {lappend dList $entry} 
      }

   if {![llength $dList]} return

   gdbDestroy $dList

   gdbWrite $mfrTab $mfrName
}

proc getMFRGroupInfo { mfrName pdbDir fragDir alpha dcNMin fragNameFlag } \
{
   if {![file exists $mfrName]} \
      {
       puts stderr "Error Finding MFR Input $inName for Group RMS."
       return 1
      }

   puts ""
   puts "Finding Coord RMS Within Each Fragment Group:"
   puts ""

   set mfrTab [gdbRead table -in $mfrName]

   getMFRGroupVars $mfrTab
   getMFRGroupRMS  $mfrTab $pdbDir $fragDir $fragNameFlag

   set dadrInfo [mfr2DaDrXYZ $mfrTab $alpha $dcNMin]
   set dadrInfo [mfr2DaDr    $mfrTab $alpha $dcNMin]
   set dotInfo  [mfr2Dot     $mfrTab $alpha $dcNMin]
   set hReport  [mfrPDBHist  $mfrTab 5]

   gdbWrite $mfrTab $mfrName
}

proc getMFRGroupRMS { mfrTab pdbDir fragDir fragNameFlag } \
{
   global dMatrix bbAtoms

   set eList    [gdbGetInfo $mfrTab -parms entries]
   set pairList ""

   foreach entry $eList \
      {
       set r1 [gdbGet $entry D_RES1]
       set rN [gdbGet $entry D_RESN]
       set id [gdbGet $entry RANK]

       gdbSet $entry GROUP_PHIPSI 0.0
       gdbSet $entry GROUP_RMS    0.0
       gdbSet $entry GROUP_AVG    0.0

       if {![strcasecmp $fragDir None]} \
          {
           set thisName [addExt $pdbDir/[gdbGet $entry PDB_NAME] pdb]
          } \
       else \
          {
           if {$fragNameFlag} \
              {
               set thisName [addExt $fragDir/[gdbGet $entry FRAG_NAME] pdb]
              } \
           else \
              {
               set fmt "%s/frag_%03d_%03d_%02d.pdb"
               set thisName [format $fmt $fragDir $r1 $rN $id]
              }
          }

       set pair $r1,$rN

       if {[lsearch -exact $pairList $pair] < 0} \
          {
           lappend pairList $pair

           set pairInfo($pair) ""
           set nameInfo($pair) ""
          }

       lappend pairInfo($pair) $entry
       lappend nameInfo($pair) $thisName
      }

   foreach pair $pairList \
      {
       set eList $pairInfo($pair)
       set pList [split $pair ,]

       set r1 [lindex $pList 0]
       set rN [lindex $pList 1]

       if {![llength $pairInfo($pair)]} \
          {
           puts [format "Residues: %3d %3d  No Fragments Selected." $r1 $rN]
           continue
          }

       set fragLength [expr 1 + $rN - $r1]

       set pdbList  ""
       set srcList  ""  
       set locList  ""
       set a1List   ""
       set aNList   ""
       set i 0

       set entry [lindex $eList 0]

       if {![strcasecmp $fragDir None]} \
          {
           foreach entry $eList \
              {
               set thisName [lindex $nameInfo($pair) $i]

               set a1 [gdbGet $entry RES1]
               set aN [gdbGet $entry RESN]

               lappend srcList \
                [dynRead -pdb -in $thisName -atomName $bbAtoms -r1 $a1 -rN $aN]

               lappend a1List  $a1
               lappend aNList  $aN
               lappend pdbList $thisName
               lappend locList $i

               incr i
              }
          } \
       else \
          {
           set a1 $r1
           set aN $rN

           foreach entry $eList \
              {
               set thisName [lindex $nameInfo($pair) $i]

               lappend srcList [dynRead -pdb -in $thisName -atomName $bbAtoms]
               lappend a1List  $a1 
               lappend aNList  $aN
               lappend pdbList $thisName
               lappend locList $i

               incr i
              }
          }

       set n [llength $srcList]

       if {!$n} continue

       for {set j 0} {$j < $fragLength} {incr j} \
          {
           set phiInfo($j) ""
           set psiInfo($j) ""
          }

       for {set i 0} {$i < $n} {incr i} \
          {
           set src [lindex $srcList $i]
           set a1  [lindex $a1List  $i]
           set aN  [lindex $aNList  $i]
           set j   0

           for {set res $a1} {$res <= $aN} {incr res} \
              {
               set ppInfo [dynGetPhiPsi -src $src -res $res]
               set phi    [lindex $ppInfo 0]
               set psi    [lindex $ppInfo 1]

               if {$phi < 999} {lappend phiInfo($j) $phi} 
               if {$psi < 999} {lappend psiInfo($j) $psi} 

               incr j
              }
          }

       set ppDevList ""

       for {set j 0} {$j < $fragLength} {incr j} \
          {
           if {[llength $phiInfo($j)]} \
              {
               lappend ppDevList [getStdDevAngle $phiInfo($j)]
              }

           if {[llength $psiInfo($j)]} \
              {
               lappend ppDevList [getStdDevAngle $psiInfo($j)]
              }
          }

       set phiPsiAvg [getAvg $ppDevList]

       set aName [lindex $nameInfo($pair) 0]
       set a1    [lindex $a1List 0]
       set aN    [lindex $aNList 0]

       set    cmnd "" 
       append cmnd "dynAvg.tcl -in $pdbList -r1 $a1 -rN $aN"
       append cmnd " -a1 $a1List -out None -align None -nosort -adj -skip"
       append cmnd " -noverb"

       set rmsInfo ""
       set error   [catch {set rmsInfo [eval "exec $cmnd"]}]

       if {$error || [llength $rmsInfo] != $n} \
          {
           puts stderr "Error Finding Group Coord RMS ($r1 $rN): $error"
           puts stderr "Command: $cmnd"
           continue
          }

       catch "unset dMatrix"
       catch "unset clustInfo"

       for {set i 0} {$i < $n} {incr i} \
          {
           set srcI [lindex $srcList $i]
           set rr1  [lindex $a1List  $i]
           set rrN  [lindex $aNList  $i]

           for {set j $i} {$j < $n} {incr j} \
              {
               set dMatrix($i,$j) 0.0

               if {$i == $j} continue

               set srcJ [lindex $srcList $j]
               set aa1  [lindex $a1List  $j]
               set aaN  [lindex $aNList  $j]

               set aInfo \
                [dynAlign -refSrc $srcI -aSrc $srcJ -r1 $rr1 -rN $rrN -a1 $aa1] 

               set rms [lindex $aInfo end]

               set dMatrix($i,$j) $rms
               set dMatrix($j,$i) $rms
              }
          }

       set clustIDList [findClusters clustInfo $locList isSimilarFrag 0]
       set i 0

       foreach entry $eList \
          {
           set clustID [lindex $clustIDList $i]
           set memCnt  $clustInfo(memCount,$clustID)
 
           gdbSet $entry CLUSTID $clustID
           gdbSet $entry MEMCNT  $memCnt
           incr i
          }

       set avg [getAvg $rmsInfo]
       set i 0

       set fmt \
        "Residues: %3d %3d Angle RMS: %4.1f Coord RMS: %5.3f Clusters: %2d"

       puts [format $fmt $r1 $rN $phiPsiAvg $avg $clustInfo(clusters)] 

       foreach entry $eList \
          {
           set pdbName [file tail [lindex $pdbList $i]]
           set rms     [lindex $rmsInfo $i]
           set a1      [lindex $a1List  $i]
           set aN      [lindex $aNList  $i]
           set clustID [lindex $clustIDList $i]

           gdbSet $entry GROUP_PHIPSI $phiPsiAvg

           gdbSet $entry GROUP_RMS $rms
           gdbSet $entry GROUP_AVG $avg

           set fmt "  %3d %3d %28s %6.3f %2d"

           puts [format $fmt $a1 $aN $pdbName $rms $clustID]

           incr i
          }
       
       puts ""

       foreach src $srcList {dynFree $src}
      }

   puts ""
}

proc getMFRGroupVars { mfrTab } \
{
   foreach varName {CLUSTID MEMCNT} \
      {
       set varID [gdbFindByName $mfrTab $varName]

       if {![llength $varID]} \
          {
           gdbCreate variable \
            -name $varName -parent $mfrTab -varType int -varFmt %2d
          }
      }

   foreach varName {GROUP_PHIPSI} \
      {
       set varID [gdbFindByName $mfrTab $varName]

       if {![llength $varID]} \
          {
           gdbCreate variable \
            -name $varName -parent $mfrTab -varType float -varFmt %6.2f
          }
      }

   foreach varName {GROUP_RMS GROUP_AVG} \
      {
       set varID [gdbFindByName $mfrTab $varName]

       if {![llength $varID]} \
          {
           gdbCreate variable \
            -name $varName -parent $mfrTab -varType float -varFmt %6.3f
          }
      }
}

#
#-----------------------------------------------------------------------------

set dNameList   [glob -nocomplain dObs*.tab]
set csName      csObs.tab
set jName       jcoup.tab
set tName       torsions.tab
set refName     ext.pdb
set outName     mfr.tab
set scoreMode   search
set rescoreName None 
set fragDir     None
set initDir     init

set rmsFlag         0
set phiPsiFlag      0
set rmsFastFlag     0
set rmsOnlyFlag     0
set phiPsiOnlyFlag  0

set firstRes    First
set lastRes      Last
set scoreCount     10
set segLength       7
set headLength      1
set tailLength      1
set alpha         3.0

set maxPDBCount 500000 

set dcMode      "-dcSVD"
 
set retryCount    100
set dcNMin         15 
set dcW          1.00
set ddcW         0.00
set csWList      None
set csW          0.50
set csClip       3.00
set jW           0.01
set torW         0.05
set ramaT        0.00
set ramaW        0.02
set homoW        0.01
set phiPsiW      0.00
set rmsW         0.00
set csT          3.00
set pHomo        100.0
set fragDist     0.50
set uFrac        0.90

set glyOnlyFlag  0
set proOnlyFlag  0
set fragNameFlag 0

set daRefList    ""
set drRefList    ""
set rhRefList    ""
set rxRefList    ""
set ryRefList    ""
set rzRefList    ""

set pdbDir       $env(PDBH_DIR)
set pdbListName  $env(PDBH_TAB)
set csDir        $env(NMRBASE)/pdb/cs

set exclList     ""

#
#-----------------------------------------------------------------------------

if {[flagLoc $ARGV -talos]} \
   {
    set pdbDir       $env(TALOS_DIR)/pdb
    set pdbListName  $env(TALOS_DIR)/tab/resolution.tab
    set csDir        $env(TALOS_DIR)/pred
   }

getArgD $ARGV -rescore rescoreName
getArgD $ARGV -initDir initDir
getArgD $ARGV -fragDir fragDir

if {[strcasecmp $rescoreName None]} \
   {
    if {![file exists $rescoreName]} \
       {
        puts stderr "Error finding Rescore Fragment Table $rescoreName"
        exit 1 
       }

    if {[strcasecmp $fragDir None]} \
       {
        if {![file isdirectory $fragDir]} \
           {
            puts stderr "Error finding Rescore Fragment Directory $fragDir"
            exit 1 
           }

        set pdbListName $rescoreName
        set pdbDir $fragDir
        set scoreMode frag
       } \
    else \
       {
        set pdbListName $rescoreName
        set scoreMode table 
       }

    set outName mfrScore.tab
   }

if {[flagLoc $ARGV -rms]}            {set rmsFlag 1}
if {[flagLoc $ARGV -norms]}          {set rmsFlag 0}

if {[flagLoc $ARGV -rmsSearch]}      {set rmsOnlyFlag 1}
if {[flagLoc $ARGV -normsSearch]}    {set rmsOnlyFlag 0}

if {[flagLoc $ARGV -rmsFast]}        {set rmsFastFlag 1}
if {[flagLoc $ARGV -normsFast]}      {set rmsFastFlag 0}

if {$rmsFlag || $rmsOnlyFlag} {set phiPsiFlag 1}

if {[flagLoc $ARGV -glyOnly]}        {set glyOnlyFlag 1}
if {[flagLoc $ARGV -noglyOnly]}      {set glyOnlyFlag 0}

if {[flagLoc $ARGV -proOnly]}        {set proOnlyFlag 1}
if {[flagLoc $ARGV -noproOnly]}      {set proOnlyFlag 0}

if {[flagLoc $ARGV -phiPsi]}         {set phiPsiFlag 1}
if {[flagLoc $ARGV -nophiPsi]}       {set phiPsiFlag 0}

if {[flagLoc $ARGV -phiPsiSearch]}   {set phiPsiOnlyFlag 1}
if {[flagLoc $ARGV -nophiPsiSearch]} {set phiPsiOnlyFlag 0}

if {[flagLoc $ARGV -useFragName]}    {set fragNameFlag 1}

if {[flagLoc $ARGV -dcSVD]}     {set dcMode "-dcSVD"}
if {[flagLoc $ARGV -dcDa]}      {set dcMode "-dcDa"}
if {[flagLoc $ARGV -dcDaDr]}    {set dcMode "-dcDaDr"}
if {[flagLoc $ARGV -dcRh]}      {set dcMode "-dcRh"}
if {[flagLoc $ARGV -dcDaDrRot]} {set dcMode "-dcDaDrRot"}

if {$phiPsiOnlyFlag} {set phiPsiW 1.00}
if {$rmsOnlyFlag}    {set rmsW    1.00}

if {$rmsOnlyFlag || $phiPsiOnlyFlag} \
   {
    set dcW          0.00
    set ddcW         0.00
    set csW          0.00
    set jW           0.00
    set torW         0.00
    set ramaW        0.00
    set homoW        0.00
    set ramaT        0.00
   }

getListArgD $ARGV -dc   dNameList
if {![strcasecmp $dNameList None]} {set dNameList ""}

getArgD $ARGV -csDir    csDir
getArgD $ARGV -cs       csName
getArgD $ARGV -j        jName
getArgD $ARGV -torsion  tName
getArgD $ARGV -ref      refName
getArgD $ARGV -out      outName

getArgD $ARGV -rmsW     rmsW
getArgD $ARGV -phiPsiW  phiPsiW

getArgD $ARGV -retry    retryCount
getArgD $ARGV -dcN      dcNMin
getArgD $ARGV -dcW      dcW
getArgD $ARGV -ddcW     ddcW
getArgD $ARGV -jW       jW
getArgD $ARGV -torW     torW
getArgD $ARGV -homoW    homoW
getArgD $ARGV -rmsW     rmsW
getArgD $ARGV -ramaW    ramaW

getListArgD $ARGV -csWList csWList

getArgD $ARGV -csW        csW
getArgD $ARGV -csT        csT
getArgD $ARGV -csThresh   csT
getArgD $ARGV -csClip     csClip

getArgD $ARGV -ramaT      ramaT
getArgD $ARGV -ramaThresh ramaT

getArgD $ARGV -pHomo      pHomo
getArgD $ARGV -alpha      alpha
getArgD $ARGV -dist       fragDist
getArgD $ARGV -uFrac      uFrac

getArgD $ARGV -pdbDir     pdbDir
getArgD $ARGV -pdbList    pdbListName

getArgD $ARGV -segLength  segLength
getArgD $ARGV -scoreCount scoreCount
getArgD $ARGV -r1         firstRes
getArgD $ARGV -rN         lastRes
getArgD $ARGV -n          maxPDBCount
getArgD $ARGV -head       headLength
getArgD $ARGV -tail       tailLength

getListArgD $ARGV -excl exclList

set newList ""

foreach eName $exclList \
   {
    lappend newList [file root [file tail $eName]]
   }

set exclList $newList

foreach dName $dNameList \
   {
    lappend daRefList 0.0
    lappend drRefList 0.0
    lappend rhRefList 0.0

    lappend rxRefList 0.0
    lappend ryRefList 0.0
    lappend rzRefList 0.0
   }

getListArgD $ARGV -daRef daRefList
getListArgD $ARGV -drRef drRefList
getListArgD $ARGV -rhRef rhRefList

getListArgD $ARGV -rxRef rxRefList
getListArgD $ARGV -ryRef ryRefList
getListArgD $ARGV -rzRef rzRefList

#
#-----------------------------------------------------------------------------

proc getPDBCS { thisPDBName } \
{
   global csDir

   if {![strcasecmp $csDir None]} {return 0}

   set rootName   [file root [file tail $thisPDBName]]
   set thisCSName $csDir/$rootName.tab

   if {![file exists $thisCSName]} {return 0}

   set csSrc [dynRead -cs -in $thisCSName -verb]

   return $csSrc
}

#
#-----------------------------------------------------------------------------

set atomNames "C CA CB N HN HA HA1 HA2"
set bbAtoms   "C CA N"
set dSrcList  "" 
set dSrcAll   "" 
set dSrcN     0 

set csSrc 0
set jSrc  0
set tSrc  0

dynInit -warn

dynSetChainMode -ignore
dynSetSegMode   -ignore

if {![file exists $refName]} \
   {
    puts stderr "Error finding PDB Reference File $refName"
    exit 1 
   }

if {![file exists $pdbListName]} \
   {
    puts stderr "Error finding PDB List File $pdbListName"
    exit 1 
   }

if {[llength $daRefList] != [llength $dNameList] ||
    [llength $drRefList] != [llength $dNameList]} \
   {
    puts stderr "Error: mismatched Da/Dr and DC Name List."
    exit 1 
   }

if {[llength $rxRefList] != [llength $dNameList] ||
    [llength $ryRefList] != [llength $dNameList] ||
    [llength $rzRefList] != [llength $dNameList]} \
   {
    puts stderr "Error: mismatched X/Y/Z Rotation Offset and DC Name List."
    exit 1
   }

set rotCount 0
set i 0

foreach dName $dNameList \
   {
    if {![file exists $dName]} \
       {
        puts stderr "Error Finding DC Input $dName"
        exit 1
       }

    if {[lindex $rxRefList $i] != 0.0} {incr rotCount}
    if {[lindex $ryRefList $i] != 0.0} {incr rotCount}
    if {[lindex $rzRefList $i] != 0.0} {incr rotCount}

    incr i
   }

set refSrc [dynRead -pdb -in $refName -verb]
set rInfo  [dynGetInfo -src $refSrc]
set r1     [lindex $rInfo 0]
set rN     [lindex $rInfo 1]

if {![strcasecmp $firstRes First]} {set firstRes $r1}
if {![strcasecmp $lastRes  Last]}  {set lastRes  $rN}

if {$rmsOnlyFlag} \
   {
    set dSrcList  0 
    set dSrcAll   0 
    set dSrcN     0
   } \
else \
   {
    if {[file exists $csName]} \
       {
        set csSrc [dynRead -cs -in $csName -csWList $csWList -verb]
       }

    if {[file exists $tName]}  {set tSrc  [dynRead -torsion -in $tName  -verb]}
    if {[file exists $jName]}  {set jSrc  [dynRead -j       -in $jName  -verb]}

    set dSrcAll [dynRead -dc -in $dNameList -list -verb]
    set dSrcN   [lindex $dSrcAll end]
    set i 0

    foreach dName $dNameList \
       {
        set dSrc [lindex $dSrcAll $i]

        set da [lindex $daRefList $i]
        set dr [lindex $drRefList $i]
        set rh [lindex $rhRefList $i]

        set rx [lindex $rxRefList $i]
        set ry [lindex $ryRefList $i]
        set rz [lindex $rzRefList $i]

        switch -exact -- $dcMode \
           {
            "-dcSVD"  {dynSetTensor -svd  -dc $dSrc} 

            "-dcDaDr" {dynSetTensor -dadr -dc $dSrc -daRef $da -drRef $dr}

            "-dcDa"   {dynSetTensor -fit  -dc $dSrc -daRef $da}

            "-dcRh"   {dynSetTensor -dadr -dc $dSrc -rhRef $rh}

            "-dcDaDrRot" {
                          if {!$i} \
                             {
                              dynSetTensor -dadr -dc $dSrc -rotref \
                               -rxRef $rx -ryRef $ry -rzRef $rz \
                               -daRef $da -drRef $dr
                             } \
                          else \
                             {
                              dynSetTensor -fixed -dc $dSrc -rotref \
                               -rxRef $rx -ryRef $ry -rzRef $rz \
                               -daRef $da -drRef $dr
                             }
                         }

            default      {dynSetTensor -dc $dSrc -svd}
           }

        lappend dSrcList $dSrc

        incr i
       }
   }

#
#-----------------------------------------------------------------------------

if {![strcasecmp $scoreMode search]} \
   {
    set rescoreArg "-norescore"
   } \
else \
   {
    set rescoreArg "-rescore"
   }

if {$glyOnlyFlag} \
   {
    set glyArg "-glyOnly"
   } \
else \
   {
    set glyArg "-noglyOnly"
   }

if {$proOnlyFlag} \
   {
    set proArg "-proOnly"
   } \
else \
   {
    set proArg "-noproOnly"
   }

if {$phiPsiFlag || $phiPsiW != 0.0} \
   {
    set phiPsiArg "-phiPsi"
   } \
else \
   {
    set phiPsiArg "-nophiPsi"
   }

if {$rmsFlag || $rmsW != 0.0} \
   {
    set rmsArg "-rms"
   } \
else \
   {
    set rmsArg "-norms"
   }

if {$rmsFastFlag} \
   {
    set rmsFastArg "-rmsFast"
   } \
else \
   {
    set rmsFastArg "-normsFast"
   }

if {![strcasecmp $csWList None]} \
   {
    set wchiArg "-nowchi"
   } \
else \
   {
    set wchiArg "-wchi"
   }

set searchSrc \
 [dynSearch -init \
            -src $refSrc -undefFrac $uFrac $rescoreArg $dcMode \
            -segLength $segLength -scoreCount $scoreCount \
            -rmsW $rmsW $rmsArg $rmsFastArg $phiPsiArg $glyArg $proArg \
            -phiPsiW $phiPsiW -homoW $homoW -ramaW $ramaW -ramaThresh $ramaT \
            -cs $csSrc -dc $dSrcList -j $jSrc -torsion $tSrc \
            -csThresh $csT -csW $csW -csClip $csClip -dcW $dcW -ddcW $ddcW -dcNMin $dcNMin \
            -retry $retryCount -jW $jW -torsionW $torW $wchiArg]

set listTab    [gdbRead table -in $pdbListName]
set eList      [gdbGetInfo $listTab -parms entries]
 
set tInit      [lindex [sysTime] 0]
set totalCount 0
set i          0

foreach entry $eList \
   {
    set pdbName  [addExt [gdbGet $entry PDB_NAME] pdb]
    set rootName [file root [file tail $pdbName]]
    set p        [gdbGet $entry HOMO_PCT]
    set aPDBName $pdbDir/$pdbName

    if {[lsearch -exact $exclList $rootName] >= 0} \
       {
        puts "Excluding $pdbName"
        continue
       }

    if {$p > $pHomo} \
       {
        puts "Excluding $pdbName (Sequence Identity $p%)"
        continue
       }

    set resCount 0

    switch $scoreMode \
       {
        search {
                if {![file exists $aPDBName]} \
                   {
                    puts "Skipping $pdbName"
                    continue
                   }

                set aSrc  [dynRead -pdb -in $aPDBName -atomName $atomNames]
                set pInfo [dynGetInfo -src $aSrc]
                set a1    [lindex $pInfo 0]
                set aN    [lindex $pInfo 1]

                set resCount [expr 1 + $aN - $a1]
                set csSimSrc [getPDBCS $aPDBName]

                puts [format "%4d. %2d %4d %s" $i $a1 $aN $pdbName]
 
                set count \
                 [dynSearch -search $searchSrc -r1 $firstRes -rN $lastRes \
                            -csSrc $csSimSrc -src $aSrc -pdbName $pdbName]

                puts "       New Fragments: $count"

                dynFree -cs $csSimSrc
               }

        frag {
              set r1  [gdbGet $entry D_RES1]
              set rN  [gdbGet $entry D_RESN]
              set a1  [gdbGet $entry RES1]
              set aN  [gdbGet $entry RESN]
              set id  [gdbGet $entry RANK]

              if {$r1 < $firstRes || $r1 > $lastRes} break

              set resCount [expr 1 + $aN - $a1]

              set rr1    [expr $r1 - $headLength]
              set rrN    [expr $rN + $tailLength]
              set aa1    [expr $a1 - $headLength]
              set aaN    [expr $aN + $tailLength]
              set seqOff [expr $a1 - $r1]

              if {$fragNameFlag} \
                 {
                  set fragName [addExt $fragDir/[gdbGet $entry FRAG_NAME] pdb]
                  set seqName  [addExt $initDir/[gdbGet $entry FRAG_NAME] pdb]
                  set thisName [file root [file tail $fragName]]
                 } \
              else \
                 {
                  set fmt "%s/frag_%03d_%03d_%02d.pdb"

                  set fragName [format $fmt $fragDir $r1 $rN $id]
                  set seqName  [format $fmt $initDir $r1 $rN $id]
                  set thisName [file root [file tail $fragName]]
                 }

              if {![file exists $fragName] || ![file exists $seqName]} \
                 {
                  puts "Skipping $fragName"
                  continue
                 }

              set aSrc \
               [dynRead -pdb -in $fragName \
                        -atomName $atomNames -r1 $rr1 -rN $rrN]

              set seqSrc \
               [dynRead -pdb -in $seqName \
                        -atomName $atomNames -r1 $aa1 -rN $aaN]

              set fmt \
               "%4d. Frag: %2d Residues: %3d %3d PDB: %3d %3d %12s %s"

              puts [format $fmt $i $id $a1 $aN $r1 $rN $pdbName $thisName]

              dynSearch -search $searchSrc \
                        -src $aSrc -pdbName $pdbName \
                        -seqSrc $seqSrc -seqName $seqName -fragName $thisName \
                        -r1 $r1 -rN $rN -a1 $r1 -aN $rN -seqOff $seqOff 

              dynFree $seqSrc
             }

        table {
               if {![file exists $aPDBName]} \
                  {
                   puts "Skipping $pdbName"
                   continue
                  }

               set r1  [gdbGet $entry D_RES1]
               set rN  [gdbGet $entry D_RESN]
               set a1  [gdbGet $entry RES1]
               set aN  [gdbGet $entry RESN]
               set id  [gdbGet $entry RANK]

               if {$r1 < $firstRes || $r1 > $lastRes} break

              set resCount [expr 1 + $aN - $a1]

               set aa1 [expr $a1 - $headLength]
               set aaN [expr $aN + $tailLength]

               set aSrc [dynRead -pdb -in $aPDBName \
                                 -atomName $atomNames -r1 $aa1 -rN $aaN]

               set fmt "%4d. Frag: %2d Residues: %3d %3d PDB: %3d %3d %s"

               puts [format $fmt $i $id $r1 $rN $a1 $aN $pdbName]

               dynSearch -search $searchSrc \
                         -src $aSrc -pdbName $pdbName \
                         -r1 $r1 -rN $rN -a1 $a1 -aN $aN
              }
       }

    set totalCount [expr $totalCount + $resCount]

    dynFree $aSrc

    incr i

    if {$i >= $maxPDBCount} break
   }

set tFinal [lindex [sysTime] 0]

puts "[expr $tFinal - $tInit] sec, $totalCount residues."

dynSearch -save -search $searchSrc -out $outName
dynSearch -free -search $searchSrc

updateMFREntries $outName $pdbDir $fragDir $fragNameFlag

if {![flagLoc $ARGV -quick]} \
   {
    getMFRGroupInfo  $outName $pdbDir $fragDir $alpha $dcNMin $fragNameFlag
   }

exit
