#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- -notk "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set mfrName     mfr.tab
set maxRank     10 
set verbose     0

set pdbName     ext.pdb
set tName       torsions.tab 
set acName      ac.tab
set segName     SEG1 
set atomNames   "C CA N"
set pdbDir      $env(PDBH_DIR)

set dPhi        15.0
set dPsi        15.0

set tFC          1.0
set acFC        10.0
set dLo          0.0
set dHi          0.0

set firstRes     First
set lastRes      Last
set fixedFlag    0
set centerFlag   1
set hdrFlag      1
set id           1

if {[flagLoc $ARGV -help]} \
   {
    puts "DYNAMO Torsion or AC Restraints from MFR Search."
    puts " -mfr     mfrName  \[mfr.tab\]       Input Fragment Table."
    puts " -pdb     pdbName  \[ext.pdb\]       PDB File Input."
    puts " -torsion tName    \[torsions.tab]   Torsion Table Output (Or None)."
    puts " -ac      acName   \[ac.tab]         AC Table Output (Or None)."
    puts " -seg     segName  \[SEG1\]          Segment Name for Output."
    puts " -atom    aNames   \[C CA N\]        Atom Names for AC Restraints."
    puts " -r1      r1       \[First\]         First Residue in Output Range."
    puts " -rN      rN       \[Last\]          Last Residue in Output Range."
    puts " -pdbDir  pdbDir   \[PDBH_DIR\]      Location of MFR PDB Files."
    puts " -maxRank maxRank  \[10\]            Max Rank of Fragments."
    puts " -acFC    acFC     \[10.0\]          AC Force Constant Scale." 
    puts " -tFC     tFC      \[1.0\]           Torsion Force Constant Scale." 
    puts " -dPhi    dPhi     \[15.0\]          Torsion Phi +/- Range, Degrees." 
    puts " -dPsi    dPsi     \[15.0\]          Torsion Psi +/- Range, Degrees."
    puts " -id      firstID  \[1\]             First Restraint ID Number."
    exit 1
   }

getListArgD $ARGV -mfr  mfrName
getListArgD $ARGV -atom atomNames

getArgD $ARGV -torsion tName
getArgD $ARGV -seg     segName
getArgD $ARGV -ac      acName
getArgD $ARGV -maxRank maxRank
getArgD $ARGV -pdb     pdbName
getArgD $ARGV -pdbDir  pdbDir

getArgD $ARGV -r1      firstRes
getArgD $ARGV -rN      lastRes
getArgD $ARGV -dPhi    dPhi
getArgD $ARGV -dPsi    dPsi
getArgD $ARGV -acFC    acFC
getArgD $ARGV -tFC     tFC
getArgD $ARGV -id      id

if {[flagLoc $ARGV -center]}   {set centerFlag 1}
if {[flagLoc $ARGV -nocenter]} {set centerFlag 0}

if {[flagLoc $ARGV -hdr]}      {set hdrFlag 1}
if {[flagLoc $ARGV -nohdr]}    {set hdrFlag 0}

#
#------------------------------------------------------------------------------

set acUnit stdout
set tUnit  stdout

dynInit -fast

if {![file exists $pdbName]} \
   {
    puts stderr "Error Finding PDB Reference Input $pdbName"
    exit 1
   }

set refSrc   [dynRead -pdb -in $pdbName]
set pInfo    [dynGetInfo -src $refSrc]

if {![strcasecmp $firstRes First]} {set firstRes [lindex $pInfo 0]}
if {![strcasecmp $lastRes  Last]}  {set lastRes  [lindex $pInfo 1]}

if {![file exists $mfrName]} \
   {
    puts stderr "Error Finding MFR Input $mfrName"
    exit 1
   }

if {[strcasecmp $acName None]} \
   {
    set error [catch {set acUnit [open $acName w]}]

    if {$error} \
       {
        puts stderr "Error Opening Output AC Table $acName"
        exit 1
       }
   }
  
if {[strcasecmp $tName None]} \
   {
    set error [catch {set tUnit [open $tName w]}]

    if {$error} \
       {
        puts stderr "Error Opening Output Torsion Table $tName"
        exit 1
       }
   }
 
if {$hdrFlag && [strcasecmp $acName None]} \
   {
    puts $acUnit \
    "VARS   INDEX GROUP FIXED RESID RESNAME ATOMNAME SEGNAME X Y Z D_LO D_HI FC"

    puts $acUnit \
    "FORMAT %4d %2d %d %4d %-4s %-4s %-4s %8.3f %8.3f %8.3f %7.2f %7.2f %.3f"

    puts $acUnit ""
   }

if {$hdrFlag && [strcasecmp $tName None]} \
   {
    puts -nonewline $tUnit "VARS INDEX GROUP"
    puts -nonewline $tUnit " RESID_I RESNAME_I ATOMNAME_I SEGNAME_I"
    puts -nonewline $tUnit " RESID_J RESNAME_J ATOMNAME_J SEGNAME_J"
    puts -nonewline $tUnit " RESID_K RESNAME_K ATOMNAME_K SEGNAME_K"
    puts -nonewline $tUnit " RESID_L RESNAME_L ATOMNAME_L SEGNAME_L"
    puts -nonewline $tUnit " ANGLE ANGLE_LO ANGLE_HI FC"
    puts $tUnit ""

    puts -nonewline $tUnit "FORMAT %4d %2d"
    puts -nonewline $tUnit " %4d %-4s %-4s %-4s"
    puts -nonewline $tUnit " %4d %-4s %-4s %-4s"
    puts -nonewline $tUnit " %4d %-4s %-4s %-4s"
    puts -nonewline $tUnit " %4d %-4s %-4s %-4s"
    puts -nonewline $tUnit " %9.3f %9.3f %9.3f %.3f"
    puts $tUnit ""
    puts $tUnit ""
   }

set inAtoms [listUnion $atomNames "C CA N"]
set mfrTab  [gdbRead table -in $mfrName]
set eList   [gdbGetInfo $mfrTab -parms entries]

set lastID -666666

foreach entry $eList \
   {
    set pdbName  [gdbGet $entry PDB_NAME]
    set rootName [file root [file tail $pdbName]]

    set rank [gdbGet $entry RANK]

    if {$rank > $maxRank} continue

    set r1   [gdbGet $entry D_RES1]
    set rN   [gdbGet $entry D_RESN]
    set a1   [gdbGet $entry RES1]
    set aN   [gdbGet $entry RESN]
    set aRMS [gdbGet $entry PHIPSI_RMS]
    set aa1  [expr $a1 - 1]
    set aaN  [expr $aN + 1]

    puts stderr \
     [format " %4d %2d %4d %4d %s" $r1 $rank $a1 $aN $pdbName]

    set pdbSrc [dynRead -pdb -in $pdbDir/$pdbName \
                        -r1 $aa1 -rN $aaN -atomName $inAtoms]

    if {$centerFlag} {dynTranslate -src $pdbSrc -center -all}

    set r0 $r1

    for {set a0 $a1} {$a0 <= $aN} {incr a0} \
       {
        set rPrev [expr $r0 - 1]
        set rNext [expr $r0 + 1]

        if {$rPrev < $firstRes || $rNext > $lastRes} continue
        if {$r0    < $firstRes || $r0    > $lastRes} continue
 
        set resName [dynGetInfo -src $refSrc -res $r0    -resName]
        set resPrev [dynGetInfo -src $refSrc -res $rPrev -resName]
        set resNext [dynGetInfo -src $refSrc -res $rNext -resName]

        if {[strcasecmp $acName None]} \
           {
            foreach atom $atomNames \
               {
                set aList [dynSelect -atom $atom -res $a0 -src $pdbSrc -list]

                if {[llength $aList] != 1} continue

                set aInfo [dynGetCoord -src $pdbSrc -atomEntry $aList]

                set x [lindex $aInfo 0]
                set y [lindex $aInfo 1]
                set z [lindex $aInfo 2]

                set fmt    " %4d %2d %d %4d %-4s %-4s %-4s"
                append fmt " % 7.3f % 7.3f % 7.3f  %7.2f %7.2f %.3f"

                puts $acUnit \
                 [format $fmt $id $rank $fixedFlag \
                              $r0 $resName $atom $segName $x $y $z \
                              $dLo $dHi $acFC]
               }
           }

        if {[strcasecmp $tName None]} \
           {
            set refInfo [dynGetPhiPsi -src $pdbSrc -res $a0]

            set phi [lindex $refInfo 0]
            set psi [lindex $refInfo 1]

            if {$phi < 999.0 && $psi < 999.0} \
               {
                set f1 " %4d %-4s %-4s %-4s"
                set f2 " % 8.2f % 8.2f % 8.2f %.3f"

                if {![strcasecmp $dPhi Auto]} \
                   {
                    set phiLo [expr $phi - $aRMS]
                    set phiHi [expr $phi + $aRMS]
                   } \
                else \
                   {
                    set phiLo [expr $phi - $dPhi]
                    set phiHi [expr $phi + $dPhi]
                   }

                puts -nonewline $tUnit [format " %4d %2d" $id $rank]
                puts -nonewline $tUnit [format $f1 $rPrev $resPrev C  $segName]
                puts -nonewline $tUnit [format $f1 $r0    $resName N  $segName]
                puts -nonewline $tUnit [format $f1 $r0    $resName CA $segName]
                puts -nonewline $tUnit [format $f1 $r0    $resName C  $segName]
                puts -nonewline $tUnit [format $f2 $phi $phiLo $phiHi $tFC]
                puts $tUnit ""

                if {![strcasecmp $dPsi Auto]} \
                   {
                    set psiLo [expr $psi - $aRMS]
                    set psiHi [expr $psi + $aRMS]
                   } \
                else \
                   {
                    set psiLo [expr $psi - $dPhi]
                    set psiHi [expr $psi + $dPhi]
                   }

                puts -nonewline $tUnit [format " %4d %2d" $id $rank]
                puts -nonewline $tUnit [format $f1 $r0    $resName N  $segName]
                puts -nonewline $tUnit [format $f1 $r0    $resName CA $segName]
                puts -nonewline $tUnit [format $f1 $r0    $resName C  $segName]
                puts -nonewline $tUnit [format $f1 $rNext $resNext N  $segName] 
                puts -nonewline $tUnit [format $f2 $psi $psiLo $psiHi $tFC]
                puts $tUnit ""
               }
          }

        incr r0
       }

    if {$lastID != $r1} \
       {
        set lastID $r1
        incr id
       }

    dynFree $pdbSrc
   }

exit
