#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

wm iconify .

if {[flagLoc $ARGV -help]} \
   {
    puts "Select Fragments from Molecular Fragment Replacement Search."
    puts " -mfr    mfrList   \[mfr.tab\]       Input Fragment Tables."
    puts " -maxRank maxRank  \[10\]            Max Rank of Fragments to Use."
    puts " -inPDB            \[init.pdb\]      PDB File Input."
    puts " -outPDB           \[mfrSurf.pdb\]   PDB File Output (Or None)."
    puts " -r1      r1       \[First\]         First Residue to Use."
    puts " -rN      rN       \[Last\]          Last Residue to Use."
    puts " -range   rRange   \[All\]           +/- Residue Range Sweep Score."
    puts " -pdbDir           \[PDBH_DIR\]      PDB Database Directory."
    puts "NMR Parameters:"
    puts " -dc      dcList   \[dObs*.tab\]     Input Dipolar Coupling Tables."
    puts " -cs      csName   \[csObs.tab\]     Input Chemical Shift Table."
    puts " -j       jName    \[jcoup.tab\]     Input J-Coupling Table."
    puts " -torsion tName    \[torsions.tab\]  Input Torsion Table."
    puts "Weighting Factors:"
    puts " -dcW     dcW     \[2.00\]             DC Weight."
    puts " -csW     csW     \[0.50\]             CS Weight."
    puts " -jW      jW      \[0.01\]             J Weight."
    puts " -torW    torW    \[0.05\]             Torsion Weight."
    puts " -ramaW   ramaW   \[0.02\]             Ramachandran Map Weight."
    puts "Other Parameters:"
    puts " -trial   tCount  \[40000\]          Number of Adjustment Trials."
    puts " -sweep   sCount  \[5000\]           Trials Between Phi/Psi Sweep."
    puts " -iter    iCount  \[10\]             Max Iterations per Residue."
    puts " -frac    sFrac   \[0.10\]           Fraction of Residues to Sweep."
    exit 1
   }

set mfrNameList   mfr.tab
set inPDBName     init.pdb
set outPDBName    sel.pdb
set rasmolPDBName None
set pdbDir        $env(PDBH_DIR)

set maxRank       10
set rasmolFlag    0

set dNameList   "dObsA.tab dObsB.tab"
set csName      csObs.tab
set jName       jcoup.tab
set tName       torsions.tab

set r1          First
set rN          Last
set range       All

set trialCount  30000
set sweepInc    5000
set sweepFrac   0.20
set sweepIter   10 

set dcW         2.00
set csW         0.50
set jW          0.01
set torW        0.05
set ramaW       0.02

set atomNames   "C CA CB N HN HA HA1 HA2"
set iseed       54321
set verbose     0

set dNameList [glob -nocomplain dObs*.tab]

getListArgD $ARGV -atom atomNames
getListArgD $ARGV -mfr  mfrNameList
getListArgD $ARGV -dc   dNameList

getArgD $ARGV -maxRank maxRank
getArgD $ARGV -inPDB   inPDBName
getArgD $ARGV -outPDB  outPDBName
getArgD $ARGV -pdbDir  pdbDir

getArgD $ARGV -cs      csName
getArgD $ARGV -j       jName
getArgD $ARGV -torsion tName

getArgD $ARGV -dcW     dcW
getArgD $ARGV -csW     csW
getArgD $ARGV -jW      jW
getArgD $ARGV -torW    torW
getArgD $ARGV -ramaW   ramaW

getArgD $ARGV -r1      r1
getArgD $ARGV -rN      rN
getArgD $ARGV -range   range
getArgD $ARGV -trial   trialCount
getArgD $ARGV -sweep   sweepInc
getArgD $ARGV -frac    sweepFrac
getArgD $ARGV -iter    sweepIter

getArgD $ARGV -iseed   iseed

if {[flagLoc $ARGV -rasmol]} {set rasmolFlag 1}

#
#------------------------------------------------------------------------------

proc initMFRSelect {} \
{
   global fragCount adjCount restoreLim trial trialCount 
   global sweepFrac sweepInc sweepFrac

   global dSrcList csSrc jSrc tSrc origSrc refSrc fullSrc iseed rasmolFlag
   global currentScore bestScore env rasmolPDBName outPDBName adjR1 adjRN

   dynInit

   srand $iseed

   dynSetSegMode   -ignore
   dynSetChainMode -ignore

   set currentScore 99999.0
   set bestScore    99999.0

   set sweepCount 0
   set fragCount  0
   set adjCount   0
   set trial      0
   set restoreLim 1000

   set adjR1      0
   set adjRN      0

   set fullSrc    0
   set origSrc    0
   set refSrc     0
   set csSrc      0
   set jSrc       0
   set tSrc       0

   set dSrcList   "" 

   if {!$rasmolFlag} return

   if {[lsearch -exact [winfo interps] rasmol] < 0} \
      {
       catch "exec xterm -iconic -e $env(NMRBIN)/rasmol &"
       catch "exec sleep 3"
      }

   if {[lsearch -exact [winfo interps] rasmol] < 0} \
      {
       set $rasmolFlag 0
      }

   if {[string first / $outPDBName] == 0} \
      {
       set rasmolPDBName $outPDBName 
      } \
   else \
      {
       set rasmolPDBName [pwd]/$outPDBName
      }

   send rasmol zoom 200
   send rasmol set fontsize 16
}

proc readMFR { } \
{
   global mfrNameList maxRank mfrInfo fragCount pdbDir r1 rN 

   set bbNames   "C CA N"
   set fragCount 0
 
   foreach mfrName $mfrNameList \
      {
       if {![file exists $mfrName]} \
          {
           puts stderr "Error finding MFR input $mfrName"
           exit 1
          }

       set mfrTab [gdbRead table -in $mfrName]
       set eList  [gdbGetInfo $mfrTab -parms entries]

       foreach entry $eList \
          {
           set pdbName  [gdbGet $entry PDB_NAME]
           set rootName [file root [file tail $pdbName]]

           set rank [gdbGet $entry RANK]

           if {$rank > $maxRank} continue

           set s1   [gdbGet $entry D_RES1]
           set sN   [gdbGet $entry D_RESN]
           set a1   [gdbGet $entry RES1]
           set aN   [gdbGet $entry RESN]
           set id   [gdbGet $entry INDEX]
           set aa1  [expr $a1 - 1]
           set aaN  [expr $aN + 1]

           if {$s1 < $r1 || $s1 > $rN} continue
           if {$sN < $r1 || $sN > $rN} continue

           set pdbSrc [dynRead -pdb -in $pdbDir/$pdbName \
                               -r1 $aa1 -rN $aaN -atomName $bbNames]

           set phiList ""
           set psiList ""

           for {set resID $a1} {$resID <= $aN} {incr resID} \
              {
               set refInfo [dynGetPhiPsi -src $pdbSrc -res $resID]
               set phi     [lindex $refInfo 0]
               set psi     [lindex $refInfo 1]
             
               if {$phi < 999.0 && $psi < 999.0} \
                  {
                   lappend phiList $phi
                   lappend psiList $psi
                  } \
               else \
                  {
                   set phiList ""
                   break
                  }
              }

           if {![llength $phiList]} continue
 
           set fmt "%5d %4d %4d %2d   %-8s %4d %4d"
 
           set mfrInfo($fragCount,phi) $phiList
           set mfrInfo($fragCount,psi) $psiList
           set mfrInfo($fragCount,r1)  $s1
           set mfrInfo($fragCount,rN)  $sN

           incr fragCount

           puts [format $fmt $fragCount $s1 $sN $rank $rootName $a1 $aN]

           dynFree $pdbSrc
          }

        gdbFree mfrTab
       }
}

proc readSelectInfo {} \
{
   global inPDBName csName dNameList jName tName atomNames r1 rN pdbInfo 
   global dSrcList csSrc jSrc tSrc refSrc origSrc fullSrc
   global dcW csW jW tW ramaW sweepCount sweepFrac

   if {![file exists $inPDBName]} \
      {
       puts stderr "Error finding PDB Input $inPDBName"
       exit 1
      }

   set refSrc  [dynRead -pdb -in $inPDBName -atomName $atomNames -verb]
   set origSrc [dynRead -pdb -in $inPDBName -atomName $atomNames]
   set fullSrc [dynRead -pdb -in $inPDBName]

   set pInfo  [dynGetInfo -src $refSrc]
   set minRes [lindex $pInfo 0]
   set maxRes [lindex $pInfo 1]

   if {![strcasecmp $r1 First]} {set r1 $minRes}
   if {![strcasecmp $rN Last]}  {set rN $maxRes}

   set sweepCount [expr 1 + int($sweepFrac*($rN - $r1))]

   for {set res $minRes} {$res <= $maxRes} {incr res} \
      {
       set aInfo [dynGetPhiPsi -src $refSrc -res $res]

       set pdbInfo($res,phi) [lindex $aInfo 0]
       set pdbInfo($res,psi) [lindex $aInfo 1]
      }

   if {[file exists $csName]} {set csSrc [dynRead -cs      -in $csName -verb]}
   if {[file exists $tName]}  {set tSrc  [dynRead -torsion -in $tName  -verb]}
   if {[file exists $jName]}  {set jSrc  [dynRead -j       -in $jName  -verb]}

   set dNames ""

   foreach dName $dNameList \
      {
       if {[file exists $dName]} {lappend dNames $dName} 
      }

   if {[llength $dNames]} \
      {
       set dSrcList [dynRead -dc -in $dNames -list -verb]
      }

   if {![llength $dSrcList]} {set dcW 0.0}

   if {!$csSrc} {set csW 0.0}
   if {!$jSrc}  {set jW  0.0}
   if {!$tSrc}  {set tW  0.0}
}

proc scoreMFR {} \
{
   global dcW csW jW tW ramaW r1 rN range currentRes currentRange
   global dSrcList csSrc jSrc tSrc refSrc

   global dcScore csScore jScore tScore ramaScore currentScore
   global dcInfo  csInfo  jInfo  tInfo  ramaInfo

   if {![strcasecmp $currentRange All]} \
      {
       set qr1 $r1
       set qrN $rN
      } \
   else \
      {
       set qr1 [expr $currentRes - $range]
       set qrN [expr $currentRes + $range]

       if {$qr1 < $r1} {set qr1 $r1} 
       if {$qrN < $rN} {set qrN $rN} 
      }

   set dcScore    0.0
   set csScore    0.0
   set jScore     0.0
   set tScore     0.0
   set ramaScore  0.0

   set currentScore 9999999.0
   set scoreCount   0
 
   if {$dcW} \
      {
       set dSrc [lindex $dSrcList end]

       set dInfo \
        [dynSimulate -dcList $dSrc -src $refSrc -s1 $qr1 -sN $qrN -a1 $qr1]

       set n [llength $dInfo]

       set dcScore [lindex $dInfo [expr $n - 4]]
       set dcN     [lindex $dInfo end]

       if {$dcN} {incr scoreCount}
      }

   if {$csW} \
      {
       set csInfo \
        [dynSimulate -cs $csSrc -wchi -src $refSrc -s1 $qr1 -sN $qrN -a1 $qr1]

       if {[llength $csInfo]} \
          {
           set csScore [lindex $csInfo 1]
           set csN     [lindex $csInfo end]

           if {$csN} \
              {
               set csScore [expr $csScore/double($csN)]
               incr scoreCount
              }
         }
      }

   if {$jW} \
      {
       set jInfo \
        [dynSimulate -j $jSrc -src $refSrc -s1 $qr1 -sN $qrN -a1 $qr1]

       if {[llength $jInfo]} \
          {
           set jScore [lindex $jInfo 0]
           set jN     [lindex $jInfo end]
    
           if {$jN}   {incr scoreCount}
          }
      }

   if {$tW} \
      {
       set tInfo \
        [dynSimulate -torsion $tSrc -src $refSrc -s1 $qr1 -sN $qrN -a1 $qr1]

       if {[llength $tInfo]} \
          {
           set tScore [lindex $tInfo 0]
           set tN     [lindex $tInfo end]

           if {$tN}   {incr scoreCount}
          }
      }

   if {$ramaW} \
      {
       set ramaInfo \
        [dynRama -refSrc $refSrc -aSrc $refSrc -s1 $qr1 -sN $qrN -a1 $qr1]

       if {[llength $ramaInfo]} {set ramaScore [lindex $ramaInfo 0]}
      }

   if {!$scoreCount} return

   set currentScore 0.0

   if {$dcW}   {set currentScore [expr $currentScore + $dcW*$dcScore]}
   if {$csW}   {set currentScore [expr $currentScore + $csW*$csScore]}
   if {$jW}    {set currentScore [expr $currentScore + $jW*$jScore]}
   if {$tW}    {set currentScore [expr $currentScore + $tW*$tScore]}
   if {$ramaW} {set currentScore [expr $currentScore + $ramaW*$ramaScore]}
}

proc saveMFRScore {} \
{
   global bestScore dcScore csScore jScore tScore ramaScore currentScore
   global outPDBName fullSrc fullSrc rasmolFlag rasmolPDBName trial adjR1 adjRN

   updateMFR

   dynWrite -pdb -src $fullSrc -out $outPDBName

   if {$rasmolFlag} \
      {
       send rasmol zap
#      send rasmol ribbons off
       send rasmol load $rasmolPDBName
#      send rasmol colour cpk
#      send rasmol ribbon off
#      send rasmol wireframe
       send rasmol colour white
       send rasmol select $adjR1-$adjRN
       send rasmol colour red 
       send rasmol select *
      }

   puts [format "  %9s: %8.6f" Score   $bestScore]
   puts [format "  %9s: %8.6f" DC      $dcScore]
   puts [format "  %9s: %8.6f" CS      $csScore]
   puts [format "  %9s: %8.6f" J       $jScore]
   puts [format "  %9s: %8.6f" Torsion $tScore]
   puts [format "  %9s: %8.6f" Rama    $ramaScore]
   puts ""
}

proc insertFrag { fragLoc } \
{
   global mfrInfo pdbInfo fragInfo refSrc adjCount adjR1 adjRN

   set r1 $mfrInfo($fragLoc,r1)
   set rN $mfrInfo($fragLoc,rN)

   set phiList $mfrInfo($fragLoc,phi)
   set psiList $mfrInfo($fragLoc,psi)

   set fragInfo(r1) $r1
   set fragInfo(rN) $rN

   set fragInfo(phi) ""
   set fragInfo(psi) ""

   set adjR1 $r1
   set adjRN $rN

   set i 0

   for {set res $r1} {$res <= $rN} {incr res} \
      {
       lappend fragInfo(phi) $pdbInfo($res,phi)
       lappend fragInfo(psi) $pdbInfo($res,psi)

       set phi [lindex $phiList $i]
       set psi [lindex $psiList $i]

       dynSetPhiPsi -res $res -phi $phi -psi $psi -src $refSrc

       set pdbInfo($res,phi) $phi
       set pdbInfo($res,psi) $psi

       incr i
      }

   incr adjCount
}

proc adjMFR3 {} \
{
   global currentRes bestScore currentScore sweepIter adjR1 adjRN
   global pdbInfo r1 rN refSrc

   set res $currentRes

   if {$res < $r1 + 1} return 
   if {$res > $rN - 1} return 

   scoreMFR

   set bestScore $currentScore

   set phiPrevSum 0
   set psiPrevSum 0
   set phiSum     0
   set psiSum     0
   set phiNextSum 0
   set psiNextSum 0

   set dList {-3 0 3}
   set qList {-3 0 3}

   set rPrev [expr $res - 1]
   set rNext [expr $res + 1]

   set adjR1 $rPrev
   set adjRN $rNext

   set phiList "$pdbInfo($rPrev,phi) $pdbInfo($res,phi) $pdbInfo($rNext,phi)"
   set psiList "$pdbInfo($rPrev,psi) $pdbInfo($res,psi) $pdbInfo($rNext,psi)"

   set nn 0

   for {set i 1} {$i <= $sweepIter} {incr i} \
   {
    set dPhiList "0 0 0"
    set dPsiList "0 0 0"

    set n 0

    foreach dPhiPrev $dList \
    {
     set phiPrev [expr $pdbInfo($rPrev,phi) + $dPhiPrev]

     foreach dPsiPrev $dList \
     {
      set psiPrev [expr $pdbInfo($rPrev,psi) + $dPsiPrev]

      foreach dPhi $qList \
      {
       set phi [expr $pdbInfo($res,phi) + $dPhi]

       foreach dPsi $qList \
       {
        set psi [expr $pdbInfo($res,psi) + $dPsi]

        foreach dPhiNext $dList \
        {
         set phiNext [expr $pdbInfo($rNext,phi) + $dPhiNext]

         foreach dPsiNext $dList \
         {
          set psiNext [expr $pdbInfo($rNext,psi) + $dPsiNext]

          if {!$dPhiPrev && !$dPsiPrev && \
              !$dPhi     && !$dPsi     && \
              !$dPhiNext && !$dPsiNext} continue

          dynSetPhiPsi -res $rPrev -src $refSrc -phi $phiPrev -psi $psiPrev
          dynSetPhiPsi -res $res   -src $refSrc -phi $phi     -psi $psi 
          dynSetPhiPsi -res $rNext -src $refSrc -phi $phiNext -psi $psiNext

          scoreMFR

          set fmt1 "Res: %d Iter: %2d New: %8.6f Prev: %8.6f"
          set fmt2 " Adj: %+d,%+d %+d,%+d %+d,%+d"

          if {$currentScore < $bestScore} \
             {
              puts -nonewline [format $fmt1 $res $i $currentScore $bestScore]

              puts [format $fmt2 \
               [expr $phiPrevSum + $dPhiPrev] [expr $psiPrevSum + $dPsiPrev] \
               [expr $phiSum     + $dPhi]     [expr $psiSum     + $dPsi] \
               [expr $phiNextSum + $dPhiNext] [expr $psiNextSum + $dPsiNext]]

              set  bestScore $currentScore

              incr nn
              incr n

              set dPhiList "$dPhiPrev $dPhi $dPhiNext"
              set dPhiList "$dPsiPrev $dPsi $dPsiNext"

              set phiList  "$phiPrev $phi $phiNext"
              set psiList  "$psiPrev $psi $psiNext"
             }
         }
        }
       }
      }
     }
    }

    set phiPrev [lindex $phiList 0]
    set phi     [lindex $phiList 1]
    set phiNext [lindex $phiList 2]

    set psiPrev [lindex $psiList 0]
    set psi     [lindex $psiList 1]
    set psiNext [lindex $psiList 2]

    set pdbInfo($rPrev,phi) $phiPrev
    set pdbInfo($rPrev,psi) $psiPrev
    set pdbInfo($res,phi)   $phi
    set pdbInfo($res,psi)   $psi
    set pdbInfo($rNext,phi) $phiNext
    set pdbInfo($rNext,psi) $psiNext

    set dPhiPrev [lindex $dPhiList 0]
    set dPhi     [lindex $dPhiList 1]
    set dPhiNext [lindex $dPhiList 2]

    set dPsiPrev [lindex $dPsiList 0]
    set dPsi     [lindex $dPsiList 1]
    set dPsiNext [lindex $dPsiList 2]

    incr phiPrevSum $dPhiPrev
    incr psiPrevSum $dPsiPrev
    incr phiSum     $dPhi
    incr psiSum     $dPsi
    incr phiNextSum $dPhiNext
    incr psiNextSum $dPsiNext

    restoreMFR
    scoreMFR

    if {!$n} break
   }

   if {$nn} \
      {
       puts ""
       saveMFRScore
      }
}

proc removeFrag {} \
{
   global fragInfo pdbInfo refSrc adjCount

   set r1 $fragInfo(r1)
   set rN $fragInfo(rN)

   set phiList $fragInfo(phi)
   set psiList $fragInfo(psi)

   set i 0

   for {set res $r1} {$res <= $rN} {incr res} \
      {
       set phi [lindex $phiList $i]
       set psi [lindex $psiList $i]

       dynSetPhiPsi -res $res -phi $phi -psi $psi -src $refSrc

       set pdbInfo($res,phi) $phi
       set pdbInfo($res,psi) $psi

       incr i
      }

   incr adjCount
}

proc updateMFR {} \
{
   global fullSrc pdbInfo r1 rN

   for {set res $r1} {$res <= $rN} {incr res} \
      {
       dynSetPhiPsi -res $res -src $fullSrc \
                    -phi $pdbInfo($res,phi) -psi $pdbInfo($res,psi)
      }
}

proc restoreMFR {} \
{
   global refSrc origSrc pdbInfo r1 rN adjCount

   dynCopy -dest $refSrc -src $origSrc

   for {set res $r1} {$res <= $rN} {incr res} \
      {
       dynSetPhiPsi -res $res -src $refSrc \
                    -phi $pdbInfo($res,phi) -psi $pdbInfo($res,psi)
      }

   set adjCount 0
}

proc sweepMFR {} \
{
   global r1 rN sweepCount currentRange range currentRes

   puts "Sweeping:"

   set currentRange $range

   for {set i 0} {$i < $sweepCount} {incr i} \
      {
       set currentRes [expr $r1 + int([rand]*($rN - $r1))]

       adjMFR3 
       update
      }

   set currentRange All 
}

proc selectMFR {} \
{
   global mfrInfo trialCount outPDBName refSrc bestScore currentScore
   global fragCount adjCount restoreLim trial sweepInc adjR1 adjRN
   global range currentRange currentRes

   set currentRes   $adjR1
   set currentRange All

   scoreMFR
   saveMFRScore

   set fragMax   [expr $fragCount - 1]
   set bestScore $currentScore

   for {set trial 1} {$trial <= $trialCount} {incr trial} \
      {
       if {!($trial % 100)} update

       set fragLoc [expr int([rand]*$fragCount)]

       if {$fragLoc < 0} {set fragLoc 0}
       if {$fragLoc > $fragMax} {set fragLoc $fragMax}

       insertFrag $fragLoc

       scoreMFR

       if {$currentScore < $bestScore} \
          {
           puts "Fragment at $adjR1-$adjRN.  Trial $trial:"

           set bestScore $currentScore

           saveMFRScore
          } \
       else \
          {
           removeFrag
          }

       if {$adjCount > $restoreLim} restoreMFR 
       if {!($trial % $sweepInc)}   sweepMFR
      }

   saveMFRScore
}

initMFRSelect
readSelectInfo
readMFR
selectMFR

exit
