#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "mstat.tcl: Display Statistics of an NMRPipe Format Table Column."
    puts " -in     inName   \[None\]  Input Table."
    puts " -var    varName  \[None\]  Input Variable Name."
    puts " -stat   statList \[None\]  TCL Statistical Functions to Use."
    puts " -fmt    fmtList  \[None\]  Output Format."
    puts " -select cond     \[1\]     Input Selection Condition."
    puts "PDB Options:"
    puts " -pdb                     Input is PDB Data (Default for .pdb)."
    puts " -nopdb                   Input is NMRPipe-Format Table."
    puts " -hetatm                  Interpret HETATMs as ATOMs."
    exit 1
   }

set inName   None
set varName  None
set statList None
set fmtList  None
set pdbMode  nopdb
set hetMode  nohetatm
set cond     1

getArgD $ARGV -in  inName
getArgD $ARGV -var varName

getListArgD $ARGV -select cond
getListArgD $ARGV -stat   statList 
getListArgD $ARGV -fmt    fmtList

if {![strcasecmp [file extension $inName] .pdb]} {set pdbMode pdb}

if {[flagLoc $ARGV -pdb]}       {set pdbMode pdb}
if {[flagLoc $ARGV -nopdb]}     {set pdbMode nopdb}

if {[flagLoc $ARGV -hetatm]}    {set hetMode hetatm}
if {[flagLoc $ARGV -nohetatm]}  {set hetMode nohetatm}

#
#-------------------------------------------------------------

if {![file exists $inName]} {exit 1}

set cond    [string trim $cond "{}"]
set t       [gdbRead table -in $inName -$pdbMode -$hetMode]

set varList [gdbGetInfo $t -parms varNames]
set valList ""
set s ""

set loc [lsearch -exact $varList $varName]

if {$loc < 0} {exit 1}

if {[strcmp $cond 1] || [strcasecmp $statList None]} \
   {
    set eList [gdbSelect -from $t -cond $cond]
    foreach e $eList {lappend valList [gdbGet $e $varName]}
   }

if {![strcasecmp $statList None]} \
   {
    if {[strcmp $cond 1]} \
       {
        catch {set s [exec echo [join $valList \n] | mstat - 0]}
       } \
    else \
       {
        catch {set s [exec mstat $inName $loc]}
       }

    puts $s

    exit 0
   }

set fmt [lindex $fmtList 0]

for {set i 0} {$i < [llength $statList] - [llength $fmtList]} {incr i} \
   {
    lappend fmtList $fmt
   }

set i 0

foreach stat $statList \
   {
    set fmt [lindex $fmtList $i]

    if {![strcasecmp $fmt None]} \
       {
        puts [$stat $valList]
       } \
    else \
       {
        puts [format $fmt [$stat $valList]]
       }

    incr i
   }
