#!/bin/sh
# The next line restarts using TCL \
exec nmrWish "$0" -notk -- "$@"

proc showHelp {} \
{
   global argv0

   puts "$argv0: Find protein molecular weigth from Nucleic Acid sequence:"   
   puts " $argv0 < seqText"
   puts "  -c13          Find MW Using 13C." 
   puts "  -noc13        Find MW Using 13C (Default)." 
   puts "  -n15          Find MW Using 15C." 
   puts "  -non15        Find MW Using 15C (Default)." 
   puts "  -antisense    Interpret Nucleic Acid Sequence as Anti-sense."
   puts "  -sense        Interpret Nucleic Acid as Sense (Default)." 
   puts "  -codonID id   Starting Codon ID   (Default = 1)."
   puts "  -resID   id   Starting Residue ID (Default = 1)."
}

proc na2mwInit {} \
{ 
   global use13C use15N antiSense 
   global dna2aa aaMW atomMW
   global resID codonID

   set use13C      0
   set use15N      0
   set antiSense   0
   set resID       1
   set codonID     1

   set atomMW(O) 15.9994 
   set atomMW(H) 1.00794 
   set atomMW(N) 14.00674
   set atomMW(C) 12.0107 

   set dna2aa(TTT) F
   set dna2aa(TTC) F
   set dna2aa(TTA) L
   set dna2aa(TTG) L

   set dna2aa(CTT) L
   set dna2aa(CTC) L
   set dna2aa(CTA) L
   set dna2aa(CTG) L

   set dna2aa(ATT) I
   set dna2aa(ATC) I
   set dna2aa(ATA) I
   set dna2aa(ATG) M

   set dna2aa(GTT) V
   set dna2aa(GTC) V
   set dna2aa(GTA) V
   set dna2aa(GTG) V

   set dna2aa(TCT) S
   set dna2aa(TCC) S
   set dna2aa(TCG) S
   set dna2aa(TCA) S

   set dna2aa(CCT) P
   set dna2aa(CCC) P
   set dna2aa(CCA) P
   set dna2aa(CCG) P

   set dna2aa(ACT) T
   set dna2aa(ACC) T
   set dna2aa(ACA) T
   set dna2aa(ACG) T

   set dna2aa(GCT) A
   set dna2aa(GCC) A
   set dna2aa(GCA) A
   set dna2aa(GCG) A

   set dna2aa(TAT) Y
   set dna2aa(TAC) Y
   set dna2aa(TAA) STOP
   set dna2aa(TAG) STOP

   set dna2aa(CAT) H
   set dna2aa(CAC) H
   set dna2aa(CAA) Q
   set dna2aa(CAG) Q

   set dna2aa(AAT) N
   set dna2aa(AAC) N
   set dna2aa(AAA) K
   set dna2aa(AAG) K
   
   set dna2aa(GAT) D
   set dna2aa(GAC) D
   set dna2aa(GAA) E
   set dna2aa(GAG) E
   
   set dna2aa(TGT) C
   set dna2aa(TGC) C
   set dna2aa(TGA) STOP
   set dna2aa(TGG) W
   
   set dna2aa(CGT) R
   set dna2aa(CGC) R
   set dna2aa(CGA) R
   set dna2aa(CGG) R

   set dna2aa(AGT) S
   set dna2aa(AGC) S
   set dna2aa(AGA) R
   set dna2aa(AGG) R

   set dna2aa(GGT) G
   set dna2aa(GGC) G
   set dna2aa(GGA) G
   set dna2aa(GGG) G

   set aaMW(A)  71.09  
   set aaMW(R) 156.19 
   set aaMW(N) 114.11 
   set aaMW(D) 115.09 
   set aaMW(C) 103.15 
   set aaMW(Q) 128.14 
   set aaMW(E) 129.12 
   set aaMW(G)  57.05 
   set aaMW(H) 137.14 
   set aaMW(I) 113.16 
   set aaMW(L) 113.16 
   set aaMW(K) 128.17 
   set aaMW(M) 131.19 
   set aaMW(F) 147.18  
   set aaMW(P)  97.12 
   set aaMW(S)  87.08 
   set aaMW(T) 101.11 
   set aaMW(W) 186.21 
   set aaMW(Y) 163.18 
   set aaMW(V)  99.14
   set aaMW(*)    0.00
   set aaMW(STOP) 0.00

   set aaC13MW(A) 3.0
   set aaC13MW(R) 0.0
   set aaC13MW(N) 0.0
   set aaC13MW(D) 0.0
   set aaC13MW(C) 0.0
   set aaC13MW(Q) 0.0
   set aaC13MW(E) 0.0
   set aaC13MW(G) 2.0
   set aaC13MW(H) 0.0
   set aaC13MW(I) 0.0
   set aaC13MW(L) 0.0
   set aaC13MW(K) 0.0
   set aaC13MW(M) 0.0
   set aaC13MW(F) 0.0
   set aaC13MW(P) 0.0
   set aaC13MW(S) 0.0
   set aaC13MW(T) 0.0
   set aaC13MW(W) 0.0
   set aaC13MW(Y) 0.0
   set aaC13MW(V) 0.0
   set aaC13MW(*)    0.00
   set aaC13MW(STOP) 0.00

   set aaN15MW(A) 0.0
   set aaN15MW(R) 1.0
   set aaN15MW(N) 0.0
   set aaN15MW(D) 0.0
   set aaN15MW(C) 0.0
   set aaN15MW(Q) 0.0
   set aaN15MW(E) 0.0
   set aaN15MW(G) 0.0
   set aaN15MW(H) 0.0
   set aaN15MW(I) 0.0
   set aaN15MW(L) 0.0
   set aaN15MW(K) 0.0
   set aaN15MW(M) 0.0
   set aaN15MW(F) 0.0
   set aaN15MW(P) 0.0
   set aaN15MW(S) 0.0
   set aaN15MW(T) 0.0
   set aaN15MW(W) 0.0
   set aaN15MW(Y) 0.0
   set aaN15MW(V) 0.0
   set aaN15MW(*)    0.00
   set aaN15MW(STOP) 0.00
}

proc na2mwArgs {} \
{
   global use13C use15N antiSense resID codonID argv
   
   set i 0

   foreach item $argv \
      {
       switch -- $item \
          {
           "-help"       {showHelp}
           "-c13"        {set use13C 1}
           "-noc13"      {set use13C 0}
           "-n15"        {set use15N 1}
           "-non15"      {set use15N 0}
           "-sense"      {set antiSense 0}
           "-antisense"  {set antiSense 1}

           "-codonID"    {set codonID [lindex $argv [expr $i+1]]}
           "-resID"      {set resID   [lindex $argv [expr $i+1]]}

           default       {}
          }

       incr i
      }
}

proc getNAList args \
{
   global use13C use15N antiSense

   set naList ""
   set inUnit stdin

   while {[gets $inUnit line] >= 0} \
      {
       foreach item $line \
          {
           foreach c [split [string toupper $item] {}] \
              {
               set loc [string first $c "GATCU"]
  
               if {$loc > -1} \
                  {
                   if {$antiSense} \
                      {
                       set c [string index "CTAGA" $loc]
                       set naList $c$naList
                      } \
                   else \
                      {
                       set c [string index "GATCT" $loc]
                       set naList $naList$c
                      }  
                  } 
              }
          }
      }

   return $naList
} 

proc getLeader { naList } \
{
   global codonID

   set leaderList ""

   set n [string length $naList]
   set i [expr $n - 1] 

   while {$i >= 0} \
      {
       set triplet ""

       append triplet [string index $naList [expr $i - 2]]
       append triplet [string index $naList [expr $i - 1]]
       append triplet [string index $naList [expr $i - 0]]

       incr i -3

       set leaderList [linsert $leaderList 0 $triplet]
      }

   set i 0

   foreach triplet $leaderList \
      {
       if {!$i} {puts -nonewline [format "%4d. " $codonID]}

       puts -nonewline [format " %3s" $triplet]

       incr codonID [string length $triplet]
       incr i

       if {$i == 15} {puts ""}
      }

   puts ""
   puts -nonewline "    "
  
   while {$i > 0} \
      {
       puts -nonewline "    "
       incr i -1
      } 

   puts "START"
   puts ""


   return $leaderList
}

proc getTripletList { naList } \
{
   set   tripletList ""
   set n [string length $naList]
   set i 0
 
   while {$i < $n} \
      {
       set triplet ""

       append triplet [string index $naList $i]
       incr i

       append triplet [string index $naList $i]
       incr i

       append triplet [string index $naList $i]
       incr i

       lappend tripletList $triplet
      }

   return $tripletList
}

proc getMW { tripletList } \
{
   global dna2aa aaMW aaC13MW aaN15MW use13C use15N atomMW resID codonID

   set mwSum   0.0
   set aaList  ""
   set aaCount 0

   set codonStr ""
   set seqStr   ""
 
   foreach triplet $tripletList \
      {
       if {![info exists dna2aa($triplet)]} \
          {
           puts "Warning: Unknown Codon found: $triplet"
           set aaName "*"
          } \
       else \
          {
           set aaName $dna2aa($triplet)
          }

       set mwSum  [expr $mwSum + $aaMW($aaName)]

       if {$use13C} {set $mwSum [expr $mwSum + $aaC13MW($aaName)]}
       if {$use15N} {set $mwSum [expr $mwSum + $aaN15MW($aaName)]}

       append  codonStr " $triplet"
       append  seqStr   "  $aaName "
       lappend aaList $aaName
       incr    aaCount

       if {!($aaCount % 15) || ![string compare $aaName STOP]} \
          {
           puts [format "%4d. %s" $codonID $codonStr]
           puts [format "%4d. %s" $resID   $seqStr]
           puts ""
 
           set codonStr ""
           set seqStr   ""

           incr codonID 45 
           incr resID   15
          }

       if {![string compare $aaName STOP]} break

       if {$aaCount == 1} \
          {
           if {[string compare $aaName P]} \
              {
#              set mwSum [expr $mwSum + $atomMW(H)]
              }
          }
      }

   if {$aaCount} {set mwSum [expr $mwSum + $atomMW(H) + $atomMW(O)]}

   if {[string compare $aaName STOP]} \
      {
       puts "Warning: No STOP Codon found."
      }

   return $mwSum
}

na2mwInit
na2mwArgs

set naList  [getNAList $argv]
set naCount [string length $naList]

puts "Nucleic Acid Count: $naCount"
 
set loc [string first "ATG" $naList]

if {$loc < 0} \
   {
    puts "START Codon ATG was not found."
    exit
   }

puts "START Codon at Base [expr $loc+1]"
puts "Begining Transcription at Base [expr $loc+3]"

set leaderList  [getLeader [string range $naList 0 [expr $loc+2]]
set tripletList [getTripletList [string range $naList [expr $loc+3] end]]
set mw          [getMW $tripletList] 

puts "Molecular Weight: $mw"

exit
