#!/bin/csh

#
# Changes needed 5/30/2019:

if (`flagLoc $argv -help`) then
   echo "Extract Options for NESTA Parameters:"
   echo " -L1                          L1 Regularization (NESTA --method 1, Default)."
   echo " -IRL1                        IRL1 Regularization (NESTA --rwiter > 1)."
   echo " -GAUSS                       Gaussian-SL0 Regularization (NESTA --method 0)."
   echo " -nThread     n       [Auto]  Number of Parallel Threads (NESTA --threads)."
   echo " -nestaIter   iter    [Auto]  Number of Iterations (NESTA --iter)."
   echo " -nestaRWIter rwiter  [Auto]  Number of Reweighting Iterations (NESTA --rwiter)."
   echo " -nestaCutoff cuttoff [0.10]  Cutoff for Gaussian-SL0 (NESTA --cutoff)."
   echo " -nestaScale  scale   [0.98]  Scaling for Gaussian-SL0 (NESTA --scaling)."
   exit 0
endif

set nThreads  = (`getArgD $argv -t            Auto`)
set nThreads  = (`getArgD $argv -threads      $nThreads`)
set nThreads  = (`getArgD $argv -nThread      $nThreads`)

set iter      = (`getArgD $argv -nestaIter    Auto`)
set rwiter    = (`getArgD $argv -nestaRWIter  Auto`)
set scale     = (`getArgD $argv -nestaScale   0.98`)
set cutoff    = (`getArgD $argv -nestaCutoff  0.1`)
set mode      = 1
set nestaArgs = ""

if (`flagLoc $argv -L1`) then
   set mode = 1

   if (`string csame $iter Auto`) then
      set iter = 50
   endif

   if (`string csame $rwiter Auto`) then
      set rwiter = 1
   endif
else if (`flagLoc $argv -IRL1`) then
   set mode = 1

   if (`string csame $iter Auto`) then
      set iter = 50
   endif

   if (`string csame $rwiter Auto`) then
      set rwiter = 5 
   endif
else if (`flagLoc $argv -GAUSS`) then
   set mode   = 0
   set rwiter = 0

   if (`string csame $iter Auto`) then
      set iter = 8000
   endif

   if (`string csame $rwiter Auto`) then
      set rwiter = 1
   endif
else
   set mode = 1

   if (`string csame $iter Auto`) then
      set iter = 50
   endif

   if (`string csame $rwiter Auto`) then
      set rwiter = 1 
   endif
endif

set thisArgv = (nesta_wrapper.com $nestaArgs)

if (`string csame $nThreads Auto`) then
   set nThreads = (`getThreadCount.tcl -f 0.5`)
endif


if (`string notcsame $nThreads Auto`) then
   set nestaArgs = (--threads  $nThreads $nestaArgs)
endif

if (`flagLoc $thisArgv -m` == 0 && `flagLoc $thisArgv --method` == 0) then
   set nestaArgs = (--method $mode $nestaArgs)
endif

if (`flagLoc $thisArgv -s` == 0 && `flagLoc $thisArgv --scaling` == 0) then
   set nestaArgs = (--scaling $scale $nestaArgs)
endif

if (`flagLoc $thisArgv -c` == 0 && `flagLoc $thisArgv --cutoff` == 0) then
   set nestaArgs = (--cutoff $cutoff $nestaArgs)
endif

if (`flagLoc $thisArgv -i` == 0 && `flagLoc $thisArgv --iter` == 0) then
   set nestaArgs = (--iter $iter $nestaArgs)
endif

if (`flagLoc $thisArgv -r` == 0 && `flagLoc $thisArgv --rwiter` == 0) then
   set nestaArgs = (--rwiter $rwiter $nestaArgs)
endif

echo $nestaArgs

exit 0
