#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

#
# Initialize:

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set inName      old.tab
set outName     new.tab

if {[flagLoc $ARGV -help]} \
   {
    puts "Convert an old NMRDraw location table to a usable peak table."
    puts "  -in  inName  \[old.tab\]   Old location table input."
    puts "  -out outName \[new.tab\]   Updated peak table output."
    exit
   }

getArgD $ARGV -in  inName
getArgD $ARGV -out outName

#
# Initialize database system.
# Read the old location table.
# Add variables X1 X3 Y1 Y3

gdbInit
gdbCreate dBase -name ASS

set pkTab [gdbRead table -in $inName -name old -verb]
set entry [gdbFirst $pkTab]

foreach i {X1 X3 Y1 Y3 CLUSTID MEMCNT TYPE} \
   {
    gdbCreate variable -parent $pkTab -name $i -varType int -varFmt %4d
   }

gdbCreate variable -parent $pkTab -name ASS -varType text -varFmt %s

#
# For each entry in old table:
#   Set values for X1 X3 Y1 Y3.

set k 0

while {$entry} \
   {
    incr k
    set x0 [gdbGet $entry X_AXIS]
    set y0 [gdbGet $entry Y_AXIS]

    set x1 [expr int($x0 - 2.0)]
    set x3 [expr int($x0 + 2.0)]
    set y1 [expr int($y0 - 2.0)]
    set y3 [expr int($y0 + 2.0)]

    gdbSet $entry X1 $x1
    gdbSet $entry X3 $x3
    gdbSet $entry Y1 $y1
    gdbSet $entry Y3 $y3

    gdbSet $entry CLUSTID $k
    gdbSet $entry MEMCNT  1 
    gdbSet $entry TYPE    1 
    gdbSet $entry ASS     None

    set entry [gdbNext $entry]
   }

gdbWrite $pkTab $outName

set vCount [gdbGetInfo $pkTab -parms varCount]
set eCount [gdbGetInfo $pkTab -parms entryCount]

puts "GDB/TCL wrote table file $outName, $vCount variables, $eCount entries."
exit
