#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "NMRPrime: Sequential Assignment by Combinatorial Optimization:"
    puts " -trials  trialCount \[10\]     Number of Assignment Trials."
    puts " -allW    allW       \[1.0\]    Overall Weighting Factor for Score." 
    puts " -rpW     rpW        \[1.0\]    Residue Type Weighting Factor."  
    puts " -offset  sOffset    \[Auto\]   Score Offset." 
    puts " -graph                       Strip Graphics ON. (Default)"
    puts " -nograph                     Strip Graphics Off."
    puts "Files:"
    puts " -dir     baseDir    \[../\]          NMRPrime Data Base Directory."
    puts " -join    joinName   \[join.tab\]     Sequential Probabilities."
    puts " -rp      rpName     \[resProb.tab\]  Residue Type Probabilities."
    puts " -out     outName    \[prime.tab\]    Assignment Results by Trial."
    puts " -tmp     tmpName    \[tmp%05d.tab\]  Temporary Chemical Shift Data."
    exit 0
   }

set trialCount  10 
set rpW         1.00 
set allW        1.00
set scoreOffset Auto 
set graphFlag   1
set demoFlag    0
set tmpID       0

set outName     prime.tab
set joinName    join.tab
set resProbName resProb.tab
set tmpName     tmp%05d.tab
set labVar      INDEX
set baseDir     ../

set seqName     seqInfo.tab
set infoName    specInfo.tab

getArgD $ARGV -trials trialCount
getArgD $ARGV -offset scoreOffset
getArgD $ARGV -allW   allW
getArgD $ARGV -rpW    rpW

getArgD $ARGV -dir  baseDir
getArgD $ARGV -lab  labVar
getArgD $ARGV -out  outName
getArgD $ARGV -join joinName
getArgD $ARGV -rp   resProbName
getArgD $ARGV -tmp  tmpName

if {[flagLoc $ARGV -graph]}   {set graphFlag 1}
if {[flagLoc $ARGV -nograph]} {set graphFlag 0}

#
#---------------------------------------------------------------------------

proc primeLogo {} \
{
   global graphFlag env

   set w .

   wm title $w NMRPrime

   catch {image delete primeImage}
   catch {image delete pipeImage}

   image create photo primeImage -file $env(NMRTXT)/primeLogo.gif
   image create photo pipeImage  -file $env(NMRTXT)/frank.gif

   button .primeLogo  -relief sunken -image primeImage -background black
   button .fdLogo     -relief sunken -image pipeImage  -background black
   label  .primeLabel -text NMRPrime
   button .primeQuit  -text Quit -command primeExit

   checkbutton .primeGraph -onvalue 1 -offvalue 0 -variable graphFlag \
               -selectcolor green -text "Draw Strips"

   pack .primeGraph .primeLogo .primeLabel .fdLogo .primeQuit -in . \
        -side top -expand 1 -fill x

   wm geom . -10+20

   update
}

proc initGraphMatrix {} \
{
   global idList locList idCount locCount resList seqList r1 rN
   global locVec idVec locVecN idVecN

   vector create locVec($r1:$rN)
   vector create locVecN($r1:$rN)

   set nn [expr $idCount - 1]

   vector create idVec(0:$nn)
   vector create idVecN(0:$nn)

   for {set resID $r1} {$resID <= $rN} {incr resID} \
      {
       set locVec($resID)  $resID
       set locVecN($resID) 0
      }

   for {set i 0} {$i < $idCount} {incr i} \
      {
       set idVec($i)  $i
       set idVecN($i) 0
      }
}

proc addBaseDir { baseDir fileName } \
{
   set c [string index $fileName 0]

   if {![strcmp $c "/"]} {return $fileName}

   set d [string trimright $baseDir "/"]

   return $d/$fileName
}

proc initPrimeMatrix {} \
{
   global nodeCount locCount idCount resCount scoreOffset
   global baseDir joinName resProbName hnName seqName infoName demoFlag
   global stripName stripHi

   global assInfo resList nodeList idList locList 
   global joinArray placeArray
   global hnInfo seqList r1 rN 
   global allW rpW

   if {![file exists $infoName]} \
      {
       puts stderr "Error Finding Input File $infoName"
       exit 1
      }

   set infoTab [gdbRead table -in $infoName]
   set eList   [gdbGetInfo $infoTab -parms entries]

   if {[llength $eList] < 2} \
      {
       puts stderr "Error: Not Enough Spectra Listed in $infoName"
       exit 0
      }

   set entry     [lindex $eList 0]
   set hnDir     [addBaseDir $baseDir [gdbGet $entry SPEC_DIR]]
   set hnName    $hnDir/[gdbGet $entry TAB_NAME]

   set entry     [lindex $eList 1]
   set stripHi   [gdbGet $entry HEIGHT]
   set stripDir  [addBaseDir $baseDir [gdbGet $entry SPEC_DIR]]
   set stripName $stripDir/[gdbGet $entry SPEC_NAME]

   foreach thisName "$joinName $seqName $resProbName $hnName $infoName" \
      {
       if {![file exists $thisName]} \
          {
           puts stderr "Error Finding Input File $thisName"
           exit 1
          }
      }

   puts stderr "Reading $joinName ..."
   set joinTab [gdbRead table -in $joinName]

   puts stderr "Reading $resProbName ..."
   set rpTab   [gdbRead table -in $resProbName]

   set seqTab  [gdbRead table -in $seqName]
   set hnTab   [gdbRead table -in $hnName]

   set idList   ""
   set locList  ""
   set nodeList ""

#
# Establish sequence and location list:

   set seqList [getSeqList $seqTab r1 rN]
   set resID   $r1

   foreach resName $seqList \
      {
       if {[strcasecmp $resName P]} {lappend locList $resID}
       incr resID
      }

#
# Establish ID list:

   set eList [gdbGetInfo $joinTab -parms entries]

   foreach entry $eList \
      {
       set idI [gdbGet $entry INDEX_I]
       set idJ [gdbGet $entry INDEX_J]

       if {[lsearch -exact $idList $idI] < 0} {lappend idList $idI}
       if {[lsearch -exact $idList $idJ] < 0} {lappend idList $idJ}
      }

#
# Extend location list or ID list as needed:

   set idCount  [llength $idList]
   set locCount [llength $locList]

   set resList  $locList
   set resCount [llength $resList]
 
   if {$idCount > $locCount} \
      {
       set n [expr $idCount - $locCount]

       for {set i 1} {$i <= $n} {incr i} \
          {
           lappend locList @$i
          }
      } \
   elseif {$idCount < $locCount} \
      {
       set n [expr $locCount - $idCount]

       for {set i 1} {$i <= $n} {incr i} \
          {
           lappend idList _$i
          }
      }

   set locCount  [llength $locList]
   set idCount   [llength $idList]
   set nodeCount $locCount

   if {![strcasecmp $scoreOffset Auto]} \
      {
       set scoreOffset [expr double($resCount)]
      }

#
# Set default values:

   for {set i 0} {$i < $nodeCount} {incr i} {lappend nodeList $i}

   foreach idI $idList \
      {
       foreach idJ $idList \
          {
           set joinArray($idI,$idJ) [expr $allW/$locCount] 
          }
      }

   foreach idI $idList \
      {
       set hnInfo($idI,hnPPM) 9999.0
       set hnInfo($idI,nPPM)  9999.0
       set assInfo($idI)      None

       foreach locJ $locList \
          {
           if {[string first "@" $locJ] == 0} \
              {
               set placeArray($idI,$locJ) 0.0
              } \
           else \
              {
               set placeArray($idI,$locJ) [expr $allW*$rpW/22.0] 
              }
          }
      }

#
# Establish HN/N locations for strips:

   set eList   [gdbGetInfo $hnTab -parms entries]
   set varList [gdbGetInfo $hnTab -parms varNames]

   set hnVar   [getHNTabVar  $hnTab]
   set n15Var  [get15NTabVar $hnTab]

#  if {[lsearch -exact $varList ASS_INDEX] >= 0} {set demoFlag 1}
   
   foreach entry $eList \
      {
       set idI [gdbGet $entry INDEX]

       set hnInfo($idI,hnPPM) [gdbGet $entry $hnVar]
       set hnInfo($idI,nPPM)  [gdbGet $entry $n15Var]

       set assInfo($idI) [gdbGet $entry ASS]
      }

#
# Establish sequential probabilities:

   set eList [gdbGetInfo $joinTab -parms entries]

   foreach entry $eList \
      {
       set idI [gdbGet $entry INDEX_I]
       set idJ [gdbGet $entry INDEX_J]
       set s   [gdbGet $entry SCORE]

       set joinArray($idI,$idJ) [expr $allW*$s]
#      puts "join $idI $idJ $s $joinArray($idI,$idJ)"
      }

#
# Establish position probabilities:

   set eList [gdbGetInfo $rpTab -parms entries]

   foreach entry $eList \
      {
       set idI     [gdbGet $entry INDEX]
       set resName [gdbGet $entry RESNAME]
       set p       [gdbGet $entry P]
   
       set locJ $r1
   
       foreach thisResName $seqList \
          {
           if {![strcmp $thisResName $resName]} \
              {
               set placeArray($idI,$locJ) [expr $allW*$rpW*$p]
#              puts "place $idI $locJ $p $placeArray($idI,$locJ)"
              }
   
           incr locJ
          }
      }
}

proc showGECO { score } \
{
   global demoFlag trial assInfo nodeList idList locList locCount r1 rN

   set w .prime

   set thisList ""
   set j 0
   set n 0

   foreach i $nodeList \
      {
       set id  [lindex $idList  $i]
       set loc [lindex $locList $j]

       if {$demoFlag && ![strcmp $loc $assInfo($id)]} {incr n}

       lappend thisList $id
       incr j
      }

   if {$demoFlag} \
      {
       set fmt "Demo Mode. Trial: %d Score: %8.4f Correct: %4d"
       set str [format $fmt $trial $score $n]
      } \
   else \
      {
       set str [format "Trial: %d Score: %8.4f" $trial $score]
      }

   wm title $w $str

   showPrimeStrips $thisList

   update
}

proc initPrimeWin {} \
{
   global r1 rN idList locVec locVecN idVec idVecN trialCount axisfont

   setFonts

   set w     .prime
   set ww    $w.seqInfo
   set bcLoc $w.bcLoc
   set bcID  $w.bcID

   set font  $axisfont
 
   toplevel $w

   set sWide [winfo screenwidth  .]
   set sHigh [winfo screenheight .]
   set sHigh [expr $sHigh/2]

   if {$sHigh < 900} \
      {
       set xAdj  5
       set yAdj 10
       set dx    6
       set dy   26
      } \
   elseif {$sWide > 1200} \
      {
       set xAdj  5
       set yAdj 10
       set dx    7
       set dy   40
      } \
   else \
      {
       set xAdj  5
       set yAdj 10
       set dx    7
       set dy   26
      }

   set wWide [expr $sWide - $dx]
   set wHigh [expr $sHigh - $dy]
   set bHigh [expr $wHigh/2 - $dy]
 
   wm geom  $w ${wWide}x${wHigh}+0+0

   set nt  $trialCount
   set row 0
   set col 0

   button   $w.quit -text Quit -command primeExit

   barchart $bcLoc -title "Assignment Consensus" -plotbackground black \
                   -height $bHigh

   barchart $bcID  -title "Position Consensus"   -plotbackground black \
                   -height $bHigh

   $bcLoc element create place -xdata locVec -ydata locVecN -foreground orchid
   $bcID  element create place -xdata idVec  -ydata idVecN  -foreground green 

   $bcLoc axis configure x -tickfont $font -ticklength 0.05i -stepsize 10
   $bcLoc axis configure y -hide 0 -min 0 -max $nt -stepsize 1 -subdivisions 1
   $bcLoc legend configure -hide 1
   $bcLoc grid   configure -hide 1

   $bcID axis configure x -tickfont $font -ticklength 0.05i -stepsize 10
   $bcID axis configure y -hide 0 -min 0 -max $nt -stepsize 1 -subdivisions 1
   $bcID legend configure -hide 1
   $bcID grid   configure -hide 1

   frame $ww
   pack  $ww -in $w -side bottom -anchor c -padx 4 -pady 4
   pack  $bcLoc $bcID -in $w -side top -expand 1 -fill both

   raise $w 

   update
}

proc saveCurrentAss { thisIDList } \
{
   global tmpName tmpID hnInfo assInfo graphFlag

   if {!$graphFlag} {return None}

   incr tmpID

   set thisName [format $tmpName $tmpID]
   set outUnit  [open $thisName w]

   puts $outUnit "VARS   INDEX ASS HN    N      COLOR"
   puts $outUnit "FORMAT %4d   %8s %8.3f %8.3f  %s"
   puts $outUnit ""

   foreach id $thisIDList \
      {
       set color lightgray
       set ass   $assInfo($id)
 
       puts $outUnit "$id $ass $hnInfo($id,hnPPM) $hnInfo($id,nPPM) $color"
      }

   close $outUnit

   return $thisName
}
 
proc showPrimeStrips { thisIDList } \
{
   global graphFlag

   if {!$graphFlag} return

   set thisName [saveCurrentAss $thisIDList]

   catch "send scrollPrime.tcl updateScrollCells $thisName"
 
   update
}

proc initStripWin {} \
{
   global hnName stripName stripHi labVar idList

   set thisName [saveCurrentAss $idList]
   set hi       [expr 0.8*$stripHi]

   set cmnd "" 
   append cmnd "exec xterm -iconic -e scrollPrime.tcl -in $stripName"
   append cmnd " -tab $thisName -hi $hi -lab $labVar -nogrid &"

   puts $cmnd

   catch "eval $cmnd"
   catch "exec sleep 3"

   update
}

proc getMostCommonCount { aList } \
{
   if {![llength $aList]} {return 0}

   set uList [listUniq $aList]
   set nList ""

   foreach u $uList \
      {
       set n 0

       foreach a $aList \
          {
           if {![strcmp $u $a]} {incr n}
          }

       lappend nList $n
      }
  
   set n [getMax $nList]

   return $n 
}

proc showPrimeStats { trialCount } \
{
    global trialInfo resList resCount idList idCount locList locCount
    global locVec idVec locVecN idVecN

    for {set j 0} {$j < $resCount} {incr j} \
       {
        set loc   [lindex $resList $j]
        set iList ""

        for {set i 1} {$i <= $trialCount} {incr i} \
           {
            set id [lindex $trialInfo($i) $j]
            lappend iList $id
           }

        set locVecN($loc) [getMostCommonCount $iList]
       }

    foreach id $idList {set iInfo($id) ""}

    for {set j 0} {$j < $locCount} {incr j} \
       {
        set loc [lindex $locList $j]

        for {set i 1} {$i <= $trialCount} {incr i} \
           {
            set id [lindex $trialInfo($i) $j]
            lappend iInfo($id) $loc
           }
      }

    for {set j 0} {$j < $idCount} {incr j} \
       {
        set id [lindex $idList $j]
        set idVecN($j) [getMostCommonCount $iInfo($id)]
       }

   update
}

proc savePrime { trialCount } \
{
   global trialInfo outName demoFlag assInfo resList resCount

   set error [catch {set outUnit [open $outName w]}]

   if {$error} \
      {
       puts stderr "Error Opening Assignment Output $outName"
       exit 1
      }

    puts -nonewline $outUnit "DATA SCORES"

    for {set i 1} {$i <= $trialCount} {incr i} \
       {
        set s [lindex $trialInfo($i) end]
        puts -nonewline $outUnit " $s"
       } 

    puts $outUnit ""
    puts $outUnit ""
    puts -nonewline $outUnit "VARS   INDEX"

    for {set i 1} {$i <= $trialCount} {incr i} \
       {
        puts -nonewline $outUnit " ASS$i"
       }

    puts $outUnit ""
    puts -nonewline $outUnit "FORMAT   %5d"

    for {set i 1} {$i <= $trialCount} {incr i} \
       { 
        puts -nonewline $outUnit " %6s"
       }

    puts $outUnit ""
    puts $outUnit ""

    for {set j 0} {$j < $resCount} {incr j} \
       {
        set loc [lindex $resList $j]

        puts -nonewline $outUnit [format " %5d" $loc]

        for {set i 1} {$i <= $trialCount} {incr i} \
           {
            set loc [lindex $trialInfo($i) $j]

            if {$demoFlag} \
               {
                puts -nonewline $outUnit [format " %6s" $assInfo($loc)]
               } \
            else \
               {
                puts -nonewline $outUnit [format " %6s" $loc]
               }
           }

        puts $outUnit ""
       }

   close $outUnit
}

proc primeExit {} \
{
   catch "send scrollPrime.tcl exit"
   exit 0
}

#
#-----------------------------------------------------------------------

primeLogo
initPrimeMatrix
initGraphMatrix
initStripWin
initPrimeWin

set defJoin  [expr $allW/$locCount]
set defPlace [expr $allW*$rpW/22.0] 

set noI   [expr $resCount*$resCount]
set pop   [expr 50*$resCount]
set iseed 54321

for {set trial 1} {$trial <= $trialCount} {incr trial} \
   {
    set trialInfo($trial) \
     [geco -validCount $resCount \
           -scoreOffset $scoreOffset \
           -iseed $iseed -showProc showGECO \
           -idList $idList -locList $locList \
           -defJoin $defJoin -defPlace $defPlace \
           -joinArray joinArray -placeArray placeArray \
           -noI $noI -pop $pop -mfSA 32 -noverb]

    incr iseed 117

    showPrimeStats $trial
    savePrime $trial
   }

wm title .prime "$trialCount Trials, Done"

update
