#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
{
 puts "Progress Bar Display for NMRPipe (use with nmrSend):"
 puts " -xSize     bxSize \[512\]         Progress Bar Width, Pixels."
 puts " -ySize     bySize \[16\]          Progress Bar Height, Pixels."
 puts " -n         nBars  \[1\]           Number of Initial Progress Bars."
 puts " -nMax      nMax   \[14\]          Maximum Number Progress Bars."
 puts " -xOff      xOff   \[100\]         Window Horizontal Offset, Pixels."
 puts " -yOff      yOff   \[200\]         Window Vertical Offset, Pixels."
 puts " -barColor  bColor \[darkBlue\]    Progress Bar Color."
 puts " -doneColor dColor \[darkreen\]    Progress Bar Color when Done."
 puts " -errColor  eColor \[firebrick\]   Progress Bar Color on Error."
 puts " -barFont   bFont  \[bigfont\]     Progress Bar Font."
 puts " -winTitle  title  \[pwd\]         Progress Bar Window Title."
 puts " -silent                         Do Not Report Process Name."
 exit 0
}

set winTitle  [pwd]
set barList   ""

set bxSize    512 
set bySize    16 

set cellX     16
set cellY     28 

set xOff      100 
set yOff      200 

set tMargin   32 
set bMargin   6 
set lMargin   16 
set rMargin   16 

set nBars     1
set nMax      8

set silentFlag 0

if {[flagLoc $ARGV -silent]}   {set silentFlag 1}
if {[flagLoc $ARGV -nosilent]} {set silentFlag 0}

set barColor  darkblue
set doneColor darkgreen
set errColor  firebrick

set barFont   bigfont

getListArgD $ARGV -winTitle winTitle

getArgD $ARGV -nBars     nBars
getArgD $ARGV -nMax      nMax

getArgD $ARGV -xSize     bxSize
getArgD $ARGV -ySize     bySize

getArgD $ARGV -tMargin   tMargin
getArgD $ARGV -bMargin   bMargin
getArgD $ARGV -rMargin   rMargin
getArgD $ARGV -lMargin   lMargin

getArgD $ARGV -xOff      xOff
getArgD $ARGV -yOff      yOff

getArgD $ARGV -cellX     cellX
getArgD $ARGV -cellY     cellY 

getArgD $ARGV -barColor  barColor
getArgD $ARGV -doneColor doneColor
getArgD $ARGV -errColor  errColor

getArgD $ARGV -barFont   barFont

set prevWinTitle "|$winTitle|"

#
#---------------------------------------------------------------------------

proc nmrProgress args \
{
   global barInfo barColor errColor doneColor barList nBars nMax winTitle

# nmrProgress -loc 0    -n 65536 -fromID 1 -pid 1655 -axis N15 -title FT
# nmrProgress -loc 1311 -n 65536 -fromID 1 -pid 1655 -pct  2.0

   set ARGV [concat nmrProgress $args]

   if {[flagLoc $ARGV -exit]} {exit 0}

   getListArgD $ARGV -winTitle winTitle

   nmrProgressTitle

   wm deiconify .

   if {[flagLoc $ARGV -reset]} \
      {
       wm deiconify . 
       update

       set nMaxNew  $nMax
       set nBarsNew $nBars

       getArgD $ARGV -nMax  nMaxNew
       getArgD $ARGV -nBars nBarsNew

       if {$nMaxNew  < 1} {set nMaxNew  1}
       if {$nBarsNew < 1} {set nBarsNew 1}
       if {$nBarsNew > $nMaxNew} {set nBarsNew $nMaxNew}

       if {$nBarsNew > $nBars} \
          {
           for {set i $nBars} {$i < $nBarsNew} {incr i} \
              {
               addProgressBar $i
               incr nBars
              } 
          } \
       elseif {$nBarsNew < $nBars} \
          {
           for {set i [expr $nBars - 1]} {$i > $nBarsNew - 1} {incr i -1} \
              {
               set w .c$i
               destroy $w
               update
              }

           set nBars $nBarsNew
          }

       for {set i 0} {$i < $nBars} {incr i} \
          {
           set w .c$i

           set rx1 $barInfo($i,rx1)
           set rxn $barInfo($i,rxn)
           set ry1 $barInfo($i,ry1)
           set ryn $barInfo($i,ryn)

           set barInfo($i,axis)      None
           set barInfo($i,expName)   None
           set barInfo($i,title)     "Progress Stage [expr $i + 1]"
           set barInfo($i,fullTitle) $barInfo($i,title)
           set barInfo($i,timeOut)   -1

           $w coords $barInfo($i,recID) $rx1 $ry1 $rx1 $ryn

           $w itemconfigure $barInfo($i,recID)  -fill $barColor
           $w itemconfigure $barInfo($i,textID) -text $barInfo($i,fullTitle)
           $w itemconfigure $barInfo($i,pctID)  -text "0.0%" 
           update
          }

       set barList ""
      }

   if {[flagLoc $ARGV -reset]} return 

   set fromID  0 
   set fromPID 0

   getArgD $ARGV -fromID fromID 
   getArgD $ARGV -pid    fromPID

   set barID $fromID,$fromPID

   set i [lsearch -exact $barList $barID]

   if {$i < 0} \
      {
       set i [llength $barList]
       lappend barList $barID
      }

   if {$i > $nBars - 1} \
      {
       if {$i > $nMax - 1} continue
       addProgressBar $i
       incr nBars
      } 

   set w .c$i

   set loc       -1 
   set n         0
   set pct       None
   set statusStr None
 
   getArgD $ARGV -loc    loc
   getArgD $ARGV -n      n
   getArgD $ARGV -pct    pct

   getListArgD $ARGV -status statusStr

   if {![strcasecmp $statusStr None]} {set statusStr ""}

   if {$pct < 0.0}   {set pct 0}
   if {$pct > 100.0} {set pct 100}

   set titleChanged 0

   getArgD $ARGV -barTimeOut $barInfo($i,timeOut)

   if {[flagLoc $ARGV -title]} \
      {
       getListArgD $ARGV -title barInfo($i,title) 
       incr titleChanged
      }

   if {[flagLoc $ARGV -axis]} \
      {
       getArgD $ARGV -axis barInfo($i,axis)
       incr titleChanged
      }

   if {[flagLoc $ARGV -expName]} \
      {
       getArgD $ARGV -expName barInfo($i,expName)
       incr titleChanged
      }

   if {$titleChanged} \
      {     
       set thisTitle $barInfo($i,title)
       if {![strcasecmp $thisTitle None]} {set thisTitle ""}
 
       if {![strcasecmp $barInfo($i,expName) None]} \
          {
           set barInfo($i,fullTitle) $thisTitle
          } \
       else \
          {
           set barInfo($i,fullTitle) $barInfo($i,expName)
           if {[string length $thisTitle]} {append barInfo($i,fullTitle) " $thisTitle"}
          }
       
       if {[strcasecmp $barInfo($i,axis) None]} {append barInfo($i,fullTitle) " $barInfo($i,axis)"}

       $w itemconfigure $barInfo($i,textID) -text $barInfo($i,fullTitle)
      }

   set rx1 $barInfo($i,rx1)
   set rxn $barInfo($i,rxn)
   set ry1 $barInfo($i,ry1)
   set ryn $barInfo($i,ryn)

   set dx  [expr $rxn - $rx1]
   set rxn [expr $rx1 + int($pct*$dx/100.0)] 

   if {$rxn < $barInfo($i,rx1)} {set rxn $barInfo($i,rx1)}
   if {$rxn > $barInfo($i,rxn)} {set rxn $barInfo($i,rxn)}

   $w coords $barInfo($i,recID) $rx1 $ry1 $rxn $ryn

   if {![string is integer $loc] || ![strcasecmp $pct None]} \
      {
      } \
   elseif {$loc > 0 && $loc == $n} \
      {
       $w itemconfigure $barInfo($i,recID) -fill $doneColor
       $w itemconfigure $barInfo($i,pctID) -text "$statusStr Done"

       if {$barInfo($i,timeOut) >= 0} \
          {
#          after [expr 1000*$barInfo($i,timeOut)] "delProgressBar $i"
          }
      } \
   elseif {$loc < 0} \
      {
       $w itemconfigure -fill $errColor
       $w itemconfigure $barInfo($i,pctID) -text "$statusStr Error"
      } \
   else \
      {
       $w itemconfigure $barInfo($i,pctID) -text "$statusStr $pct%" 
      }

   update
}

proc nmrProgressInit {} \
{
   global xOff yOff barFont stdfont bigfont nBars nMax

   if {$nBars > $nMax} {set nBars $nMax}
   if {$nBars < 1} {exit 0}

   nmrProgressTitle

   wm geom      . [format "%+d%+d" $xOff $yOff]
   wm deiconify .

   setFonts

   if {![strcasecmp $barFont bigfont]} {set barFont $bigfont}
   if {![strcasecmp $barFont stdfont]} {set barFont $stdfont}
 
   for {set i 0} {$i < $nBars} {incr i} {addProgressBar $i}
}

proc delProgressBar { i } \
{
   global barInfo barList nBars

   if {$i < 0 || $i > $nBars - 1} return

   catch "destroy $barInfo($i,window)"
}

proc addProgressBar { i } \
{
   global cellX cellY xOff yOff bxSize bySize tMargin bMargin rMargin lMargin
   global barColor barFont barInfo barList nBars

   set cxSize [expr $bxSize + $lMargin + $rMargin]
   set cySize [expr $bySize + $tMargin + $bMargin]

   set w .c$i

   set rx1 $lMargin
   set rxn [expr $rx1 + $bxSize]
   set ry1 $tMargin
   set ryn [expr $ry1 + $bySize]

   set bx1 [expr $rx1 - 1]
   set bxn [expr $rxn + 1]
   set by1 [expr $ry1 - 1]
   set byn [expr $ryn + 1]

   set tx1 $rx1
   set txn $rxn
   set ty1 [expr $ry1 - (1 + $cellY/2)] 
   set tyn [expr $ry1 - (1 + $cellY/2)] 

   set barInfo($i,window)    $w
   set barInfo($i,axis)      None
   set barInfo($i,expName)   None
   set barInfo($i,title)     "Progress Stage [expr $i + 1]"
   set barInfo($i,fullTitle) $barInfo($i,title) 
   set barInfo($i,timeOut)   -1

   canvas $w -width $cxSize -height $cySize -border 2 -relief groove

   pack $w -in . -side top -expand 1 -fill x

   set barInfo($i,boxID) \
     [$w create rectangle $bx1 $by1 $bxn $byn -outline white -fill gray]

   set barInfo($i,recID) \
     [$w create rectangle $rx1 $ry1 $rx1 $ryn -outline gray -fill $barColor]

   set barInfo($i,textID) \
     [$w create text $tx1 $ty1 -fill black -font $barFont -anchor w -text $barInfo($i,fullTitle)]

   set barInfo($i,pctID) \
     [$w create text $txn $tyn -fill black -font $barFont -anchor e -text "0.0%"]

   set barInfo($i,rx1) $rx1
   set barInfo($i,rxn) $rxn
   set barInfo($i,ry1) $ry1
   set barInfo($i,ryn) $ryn

   wm deiconify . 

   update
} 

proc nmrProgressTitle {} \
{
   global winTitle prevWinTitle

   if {[strcasecmp $prevWinTitle $winTitle]} {wm title . $winTitle}

   set prevWinTitle $winTitle

   update
}

nmrProgressTitle
nmrProgressInit

if {!$silentFlag} {puts [winfo name .]}

