# A notebook widget for Tcl/Tk
# $Revision: 1.3 $
#
# Copyright (C) 1996,1997,1998 D. Richard Hipp
# 
# Author contact information:
#   drh@acm.org
#   http://www.hwaci.com/drh/

#
# Create a new notebook widget

proc notebook_create {w args} {
  global notebook
  set notebook($w,width) 400
  set notebook($w,height) 300
  set notebook($w,pages) {}
  set notebook($w,top) 0
  set notebook($w,pad) 5
  set notebook($w,current) none
  set notebook($w,fg,on) black
  set notebook($w,fg,off) grey50
  canvas $w -bd 0 -highlightthickness 0 -takefocus 0
  set notebook($w,bg) [$w cget -bg]
  bind $w <1> "notebook_click $w %x %y"
  bind $w <Configure> "notebook_scheduleExpand $w"
  eval notebook_config $w $args
}

#
# Change configuration options for the notebook widget

proc notebook_config {w args} {
  global notebook
  foreach {tag value} $args {
    switch -- $tag {
      -width {
        set notebook($w,width) $value
      }
      -height {
        set notebook($w,height) $value
      }
      -pages {
        set notebook($w,pages) $value
      }
      -pad {
        set notebook($w,pad) $value
      }
      -bg {
        set notebook($w,bg) $value
      }
      -fg {
        set notebook($w,fg,on) $value
      }
      -disabledforeground {
        set notebook($w,fg,off) $value
      }
    }
  }

#
# After getting new configuration values, reconstruct the widget

  $w delete all
  set notebook($w,x1) $notebook($w,pad)
  set notebook($w,x2) [expr $notebook($w,x1)+2]
  set notebook($w,x3) [expr $notebook($w,x2)+$notebook($w,width)]
  set notebook($w,x4) [expr $notebook($w,x3)+2]
  set notebook($w,y1) [expr $notebook($w,pad)+2]
  set notebook($w,y2) [expr $notebook($w,y1)+2]
  set notebook($w,y5) [expr $notebook($w,y1)+30]
  set notebook($w,y6) [expr $notebook($w,y5)+2]
  set notebook($w,y3) [expr $notebook($w,y6)+$notebook($w,height)]
  set notebook($w,y4) [expr $notebook($w,y3)+2]
  set x $notebook($w,x1)
  set cnt 0
  set y7 [expr $notebook($w,y1)+10]

  foreach p $notebook($w,pages) {
    set notebook($w,p$cnt,x5) $x
    set id [$w create text 0 0 -text $p -anchor nw -tags "p$cnt t$cnt"]
    set bbox [$w bbox $id]
    set width [lindex $bbox 2]
    $w move $id [expr $x+10] $y7
    $w create line \
       $x $notebook($w,y5)\
       $x $notebook($w,y2) \
       [expr $x+2] $notebook($w,y1) \
       [expr $x+$width+16] $notebook($w,y1) \
       -width 2 -fill white -tags p$cnt
    $w create line \
       [expr $x+$width+16] $notebook($w,y1) \
       [expr $x+$width+18] $notebook($w,y2) \
       [expr $x+$width+18] $notebook($w,y5) \
       -width 2 -fill black -tags p$cnt
    set x [expr $x+$width+20]
    set notebook($w,p$cnt,x6) [expr $x-2]
    if {![winfo exists $w.f$cnt]} {
      frame $w.f$cnt -bd 0
    }
    $w.f$cnt config -bg $notebook($w,bg)
    place $w.f$cnt -x $notebook($w,x2) -y $notebook($w,y6) \
      -width $notebook($w,width) -height $notebook($w,height)
    incr cnt
  }
  $w create line \
     $notebook($w,x1) [expr $notebook($w,y5)-2] \
     $notebook($w,x1) $notebook($w,y3) \
     -width 2 -fill white
  $w create line \
     $notebook($w,x1) $notebook($w,y3) \
     $notebook($w,x2) $notebook($w,y4) \
     $notebook($w,x3) $notebook($w,y4) \
     $notebook($w,x4) $notebook($w,y3) \
     $notebook($w,x4) $notebook($w,y6) \
     $notebook($w,x3) $notebook($w,y5) \
     -width 2 -fill black
  $w config -width [expr $notebook($w,x4)+$notebook($w,pad)] \
            -height [expr $notebook($w,y4)+$notebook($w,pad)] \
            -bg $notebook($w,bg)
  set top $notebook($w,top)
  set notebook($w,top) -1
  notebook_raise.page $w $top
}

#
# This routine is called whenever the mouse-button is pressed over
# the notebook.  It determines if any page should be raised and raises
# that page.

proc notebook_click {w x y} {
  global notebook
  if {$y<$notebook($w,y1) || $y>$notebook($w,y6)} return
  set N [llength $notebook($w,pages)]
  for {set i 0} {$i<$N} {incr i} {
    if {$x>=$notebook($w,p$i,x5) && $x<=$notebook($w,p$i,x6)} {
      notebook_raise.page $w $i
      break
    }
  }
}

#
# For internal use only.  This procedure raised the n-th page of
# the notebook

proc notebook_raise.page {w n} {
  global notebook

  set notebook($w,current) none
  if {$n<0 || $n>=[llength $notebook($w,pages)]} return
  set notebook($w,current) [lindex $notebook($w,pages) $n]
  set top $notebook($w,top)
  if {$top>=0 && $top<[llength $notebook($w,pages)]} {
    $w move p$top 0 2
  }
  $w move p$n 0 -2
  $w delete topline
  if {$n>0} {
    $w create line \
       $notebook($w,x1) $notebook($w,y6) \
       $notebook($w,x2) $notebook($w,y5) \
       $notebook($w,p$n,x5) $notebook($w,y5) \
       $notebook($w,p$n,x5) [expr $notebook($w,y5)-2] \
       -width 2 -fill white -tags topline
  }
  $w create line \
    $notebook($w,p$n,x6) [expr $notebook($w,y5)-2] \
    $notebook($w,p$n,x6) $notebook($w,y5) \
    -width 2 -fill white -tags topline
  $w create line \
    $notebook($w,p$n,x6) $notebook($w,y5) \
    $notebook($w,x3) $notebook($w,y5) \
    -width 2 -fill white -tags topline
  set notebook($w,top) $n
  raise $w.f$n
}

#
# Change the page-specific configuration options for the notebook

proc notebook_pageconfig {w cnt args} {
  global notebook
  foreach {tag value} $args {
    switch -- $tag {
      -state {
        if {"$value"=="disabled"} {
          $w itemconfig t$cnt -fill $notebook($w,fg,off)
        } else {
          $w itemconfig t$cnt -fill $notebook($w,fg,on)
        }
      }
      -onexit {
        set notebook($w,p$i,onexit) $value
      }
    }
  }
}

#
# This procedure raises a notebook page given its name.  But first
# we check the "onexit" procedure for the current page (if any) and
# if it returns false, we don't allow the raise to proceed.

proc notebook_raise {w name} {
  global notebook
  set i [lsearch $notebook($w,pages) $name]
  if {$i<0} return
  if {[info exists notebook($w,p$i,onexit)]} {
    set onexit $notebook($w,p$i,onexit)
    if {"$onexit"!="" && [eval uplevel #0 $onexit]!=0} {
      notebook_raise.page $w $i
    }
  } else {
    notebook_raise.page $w $i
  }
}

#
# Return the frame associated with a given page of the notebook.

proc notebook_frame {w name} {
  global notebook
  set i [lsearch $notebook($w,pages) $name]
  if {$i>=0} {
    return $w.f$i
  } else {
    return {}
  }
}

#
# Return the current page name of the notebook.

proc notebook_current { w } {
  global notebook

  if {[info exists notebook($w,current)]} \
     {
      return $notebook($w,current) 
     } \
  else \
     {
      return none
     }
}

#
# Try to resize the notebook to the next time we become idle.

proc notebook_scheduleExpand w {
  global notebook
  if {[info exists notebook($w,expand)]} return
  set notebook($w,expand) 1
  after idle "notebook_expand $w"
}

#
# Resize the notebook to fit inside its containing widget.

proc notebook_expand w {
  global notebook
  set wi [expr [winfo width $w]-($notebook($w,pad)*2+4)]
  set hi [expr [winfo height $w]-($notebook($w,pad)*2+36)]
  notebook_config $w -width $wi -height $hi
  catch {unset notebook($w,expand)}
}

#################################
# The following code implements an example of using the
# notebook widget.

proc notebook_demo {} \
{
   notebook_create .n -pages {One Two Three Four Five} -pad 20 
   pack .n -fill both -expand 1
   set w [notebook_frame .n One]
   label $w.l -text "Hello.\nThis is page one"
   pack $w.l -side top -padx 10 -pady 50
   set w [notebook_frame .n Two]
   text $w.t -font fixed -yscrollcommand "$w.sb set" -width 40
   $w.t insert end "This is a text widget.  Type in it, if you want\n"
   pack $w.t -side left -fill both -expand 1
   scrollbar $w.sb -orient vertical -command "$w.t yview"
   pack $w.sb -side left -fill y
   set w [notebook_frame .n Three]
   set p3 red
   frame $w.f
   pack $w.f -padx 30 -pady 30
   foreach c {red orange yellow green blue violet} {
     radiobutton $w.f.$c -fg $c -text $c -variable p3 -value $c -anchor w
     pack $w.f.$c -side top -fill x
   }
   set w [notebook_frame .n Four]
   frame $w.f
   pack $w.f -padx 30 -pady 30
   button $w.f.b -text {Goto} -command [format {
     set i [%s cursel]
     if {[string length $i]>0} {
       notebook_raise .n [%s get $i]
     }
   } $w.f.lb $w.f.lb]
   pack $w.f.b -side bottom -expand 1 -pady 5
   listbox $w.f.lb -yscrollcommand "$w.f.sb set"
   scrollbar $w.f.sb -orient vertical -command "$w.f.lb yview"
   pack $w.f.lb -side left -expand 1 -fill both
   pack $w.f.sb -side left -fill y
   $w.f.lb insert end One Two Three Four Five
   set w [notebook_frame .n Five]
   button $w.b -text Exit -command exit
   pack $w.b -side top -expand 1
}
