#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Multiply FID by an NMRPipe-Format NUS Schedule:"
    puts " -in     inName   \[fid/test%03d.fid\]  Full-Schedule Input Data."
    puts " -nus    nusName  \[projYZ.dat\]        NUS Schedule File."
    puts " -out    outName  \[nus/test%03d.fid\]  Output Product of FID and NUS Data."
    puts "Notes:"
    puts " Currently Supports Schedule for 3D FID Only."
    exit 0
   }

set inName   fid/test%03d.fid
set nusName  profYZ.dat
set outName  nus/test%03d.fid

#
#--------------------------------------------------------------------------------------

proc rdNUSData {} \
{
    global nusSize nusBuff nusUnit nusName FDATASIZE

    if {![strcmp $nusBuff 0]} \
       {
        set error [catch {set nusBuff [sysMalloc [expr $FDATASIZE + $nusSize] float]}]

        if {$error} \
           {
            puts "NUSMult Error Allocating NUS Buffer."
            exit 1
           }
       }

    set error [catch {set nusUnit [sysOpen $nusName r]}]

    if {$error} \
       {
        puts "NUSMult Error Opening NUS Data $nusName"
        exit 1
       }

    set error [catch {sysRead $nusUnit $nusBuff $FDATASIZE}]

    if {$error} \
       {
        puts "NUSMult Error Reading NUS Header in $nusName"
        exit 1
       }

    set error [catch {sysRead $nusUnit $nusBuff $nusSize}]

    if {$error} \
       {
        puts "NUSMult Error Reading NUS Data in $nusName"
        exit 1
       }

    puts "NUSMult: Read $nusName $nusSize Points."

    catch "sysClose $nusUnit"
    set nusUnit 0
}

proc rdFIDData { thisName } \
{
    global fidSize fidBuff fidUnit FDATASIZE

    if {![strcmp $fidBuff 0]} \
       {
        set error [catch {set fidBuff [sysMalloc [expr $FDATASIZE + $fidSize] float]}]

        if {$error} \
           {
            puts "NUSMult Error Allocating FID Buffer."
            exit 1
           }
       }

    set error [catch {set fidUnit [sysOpen $thisName r]}] 

    if {$error} \
       {
        puts "NUSMult Error Opening FID Data $thisName"
        exit 1
       }
   
    set error [catch {sysRead $fidUnit $fidBuff $FDATASIZE}]
    
    if {$error} \
       {
        puts "NUSMult Error Reading FID Header in $thisName"
        exit 1
       }

    set error [catch {sysRead $fidUnit $fidBuff $fidSize}]

    if {$error} \
       {
        puts "NUSMult Error Reading FID Data in $thisName"
        exit 1
       }

    puts "NUSMult: Read $thisName $fidSize Points."

    catch "sysClose $fidUnit"
    set fidUnit 0
}

proc rdFIDData { thisName } \
{
    global fidFDATA fidSize fidBuff fidUnit FDATASIZE

    if {![strcmp $fidBuff 0]} \
       {
        set error [catch {set fidBuff [sysMalloc [expr $FDATASIZE + $fidSize] float]}]

        if {$error} \
           {
            puts "NUSMult Error Allocating FID Buffer."
            exit 1
           }
       }

    set error [catch {set fidUnit [sysOpen $thisName r]}]

    if {$error} \
       {
        puts "NUSMult Error Opening Input Data $thisName"
        exit 1
       }

    set error [catch {sysRead $fidUnit $fidFDATA $FDATASIZE}]

    if {$error} \
       {
        puts "NUSMult Error Reading FID Header in $thisName"
        exit 1
       }

    set error [catch {sysRead $fidUnit $fidBuff $fidSize}]

    if {$error} \
       {
        puts "NUSMult Error Reading FID Data in $thisName"
        exit 1
       }

    catch "sysClose $fidUnit"
    set fidUnit 0
}

proc wrFIDData { thisName } \
{
    global fidFDATA fidSize fidBuff fidUnit FDATASIZE

    set dirName [file dirname $thisName]

    if {![file isdirectory $dirName]} \
       {
        catch "file mkdir $dirName"
       }

    set error [catch {set fidUnit [sysOpen $thisName w]}]

    if {$error} \
       {
        puts "NUSMult Error Opening Output Data $thisName"
        exit 1
       }

    set error [catch {sysWrite $fidUnit $fidFDATA $FDATASIZE}]

    if {$error} \
       {
        puts "NUSMult Error Writing FID Header to $thisName"
        exit 1
       }

    set error [catch {sysWrite $fidUnit $fidBuff $fidSize}]

    if {$error} \
       {
        puts "NUSMult Error Writing FID Data to $thisName"
        exit 1
       }

    catch "sysClose $fidUnit"
    set fidUnit 0
}

set fidFDATA 0
set nusFDATA 0

set fidBuff  0
set nusBuff  0

set nusUnit  0
set fidUnit  0

set thisName [format $inName 1 ]

if {![file exists $thisName]} \
   {
    puts stderr "NUSMult Error Finding FID Input $thisName"
    exit 1
   }

if {![file exists $nusName]} \
   {
    puts stderr "NUSMult Error Finding NUS Input $nusName"
    exit 1
   }

set error [rdFDATA $thisName fidFDATA]

if {$error} \
   {
    puts stderr "NUSMult Error Reading Header from FID Input $thisName"
    exit 1
   }

set dimCount [getParmI $fidFDATA FDDIMCOUNT NULL_DIM]

set xSize [getParmI $fidFDATA NDSIZE CUR_XDIM]
set ySize [getParmI $fidFDATA NDSIZE CUR_YDIM]
set zSize [getParmI $fidFDATA NDSIZE CUR_ZDIM]
set aSize [getParmI $fidFDATA NDSIZE CUR_ADIM]

set qxSize [getQuadSize [getParmI $fidFDATA NDQUADFLAG CUR_XDIM]]
set qySize [getQuadSize [getParmI $fidFDATA NDQUADFLAG CUR_YDIM]]
set qzSize [getQuadSize [getParmI $fidFDATA NDQUADFLAG CUR_ZDIM]]
set qaSize [getQuadSize [getParmI $fidFDATA NDQUADFLAG CUR_ADIM]]

set xSizeR $xSize
set ySizeR [expr $ySize/$qySize]
set zSizeR [expr $zSize/$qzSize]
set aSizeR [expr $aSize/$qaSize]

puts "NUSMult: FID $inName"
puts "NUSMult: FID Sizes X: ${xSizeR}x$qxSize Y: ${ySizeR}x$qySize Z: ${zSizeR}x$qzSize"

set error [rdFDATA $nusName nusFDATA]

if {$error} \
   {
    puts stderr "NUSMult Error Reading Header from FID Input $thisName"
    exit 1
   }

set nusDimCount [getParmI $nusFDATA FDDIMCOUNT NULL_DIM]

set nusXSize [getParmI $nusFDATA NDSIZE CUR_XDIM]
set nusYSize [getParmI $nusFDATA NDSIZE CUR_YDIM]
set nusZSize [getParmI $nusFDATA NDSIZE CUR_ZDIM]

puts "NUSMult: NUS $nusName"
puts "NUSMult: NUS Sizes X: $nusXSize Y: $nusYSize"

if {$nusXSize != $ySizeR || $nusYSize != $zSizeR} \
   {
    puts "NUSMult Error: NUS X/Y Sizes Must Match FID Y/Z Sizes."
    exit 1
   }

set fidSize [expr $xSizeR*$qxSize*$ySizeR*$qySize]
set nusSize [expr $nusXSize*$nusYSize]

rdNUSData

set sum 0.0
set iz  0

for {set kz 0} {$kz < $zSizeR} {incr kz} \
   {
    for {set qz 0} {$qz < $qzSize} {incr qz} \
       {
        set thisName [format $inName [expr $iz + 1]]
        rdFIDData $thisName

        set rPtr $fidBuff
        set iy   0

        for {set ky 0} {$ky < $ySizeR} {incr ky} \
           {
            set loc [expr $ky + $kz*$ySizeR]
            set val [getPtrVal $nusBuff $loc]

            for {set qy 0} {$qy < $qySize} {incr qy} \
               {
                for {set qx 0} {$qx < $qxSize} {incr qx} \
                   {
                    vsMult $rPtr $val $xSizeR
                    set rPtr [incrPtr $rPtr $xSizeR]
                   }
               }

            incr iy
           }

        set thisName [format $outName [expr $iz + 1]]
        wrFIDData $thisName

        incr iz
       }
   }

set sum [vSum $nusBuff $nusSize]
set n   [expr $ySizeR*$zSizeR]

if {!$n} \
   {
    set p 0.0
   } \
else \
   {
    set p [expr 100.0*$sum/$n]
   }

puts "NUSMult: [format "%.1f" $p]% Sampling Density."

exit 0
