#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Prepare Time-Domain Data for Non-Uniform Sampling Extrapolation:"
   echo " -in       inName   [test.fid]   Time-Domain Input."
   echo " -sample   nusName  [nuslist_zf] NUS Sampling Schedule Output, or Keyword None."
   echo " -out      outName  [Auto]       Zero Fill Output, or Keyword Auto."
   echo " -mask     maskName [Auto]       Mask Output, or Keyword: Auto None."
   echo " -pseudo3D                       Process Data as Pseudo-3D 2D Series."
   echo " -verb                           Verbose Mode ON (Default)."
   echo " -noverb                         Verbose Mode OFF."
   echo "Amount of Extrapolating Zero Fill (also -xNUSZF -yNUSZF ...):"
   echo " -xZFARG   xZF      [None]       Zero Fill Arguments for X-Axis."
   echo " -yZFARG   yZF      [zf=1,auto]  Zero Fill Arguments for Y-Axis."
   echo " -zZFARG   zZF      [zf=1,auto]  Zero Fill Arguments for Z-Axis (3D-4D Only)."
   echo " -aZFARG   aZF      [zf=1,auto]  Zero Fill Arguments for A-Axis (3D-4D Only)."
   echo "Notes:"
   echo " 1. Possible output names for -out Auto depend on the input name format:"
   echo "      nuszf.fid nuszf/test%03d.fid nuszf/test%03d%03d.fid"
   echo " 2. Possible mask names for -mask Auto depend on the input name format:"
   echo "      mask.fid  mask/test%03d.fid  mask/test%03d%03d.fid"
   exit 0
endif

set inName   = (`getArgD $argv -in       test.fid`)
set outName  = (`getArgD $argv -out      Auto`)
set maskName = (`getArgD $argv -mask     Auto`)
set nusName  = (`getArgD $argv -schedule nuslist_zf`)
set nusName  = (`getArgD $argv -sample   $nusName`)
set tmpDir   = (`getArgD $argv -tmpDir   tmp_zf`)
set maskFlag = 1

set pseudo3DFlag = (`getLogArgD $argv -nopseudo3D -pseudo3D  0 1`)
set vFlag        = (`getLogArgD $argv -verb       -noverb    1 0`)

if ($vFlag) then
   set vArg = "-verb"
else
   set vArg = "-noverb"
endif

if (`string csame $maskName None`) then
   set maskFlag = 0
endif

set xZFARG = (`getArgD $argv -xZFARG None`)
set yZFARG = (`getArgD $argv -yZFARG zf=1,auto`)
set zZFARG = (`getArgD $argv -zZFARG zf=1,auto`)
set aZFARG = (`getArgD $argv -aZFARG zf=1,auto`)

set xZFARG = (`getArgD $argv -xNUSZF $xZFARG`)
set yZFARG = (`getArgD $argv -yNUSZF $yZFARG`)
set zZFARG = (`getArgD $argv -zNUSZF $zZFARG`)
set aZFARG = (`getArgD $argv -aNUSZF $aZFARG`)

if (`string csame $xZFARG None`) then
   set xZF = NULL
else
   set xZF = "ZF `expandArg $xZFARG` -td"
endif

if (`string csame $yZFARG None`) then
   set yZF = NULL
else
   set yZF = "ZF `expandArg $yZFARG` -td"
endif

if (`string csame $zZFARG None`) then
   set zZF = NULL
else
   set zZF = "ZF `expandArg $zZFARG` -td"
endif

if (`string csame $aZFARG None`) then
   set aZF = NULL
else
   set aZF = "ZF `expandArg $aZFARG` -td"
endif

if ($pseudo3DFlag) then
   set zZF = NULL
   set aZF = NULL
endif

#
#-------------------------------------------------------------------------------------

set thisName = (`nmrPrintf $inName 1 1`)

if (!(-e $thisName)) then
   echo "nusZF. Error Finding Input $thisName"
   exit 1
endif

set pipeFlag     = (`getParm -in $thisName -fmt %.0f -parm FDPIPEFLAG -dim NULL_DIM`)
set origDimCount = (`getParm -in $thisName -fmt %.0f -parm FDDIMCOUNT -dim NULL_DIM`)
set dimCount     = $origDimCount

set xSize = (`getParm -in $thisName -fmt %.0f -parm NDAPOD -dim CUR_XDIM`)
set ySize = (`getParm -in $thisName -fmt %.0f -parm NDAPOD -dim CUR_YDIM`)
set zSize = (`getParm -in $thisName -fmt %.0f -parm NDAPOD -dim CUR_ZDIM`)
set aSize = (`getParm -in $thisName -fmt %.0f -parm NDAPOD -dim CUR_ADIM`)

@ fmtCount = (`echo $inName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

if (`string csame $outName Auto`) then
   if ($fmtCount == 1) then
     set outName = nuszf/test%03d.fid
   else if ($fmtCount == 2) then
     set outName = nuszf/test%03d%03d.fid
   else
     set outName = nuszf.fid
   endif
endif

if (`string csame $maskName Auto`) then
   if ($fmtCount == 1) then
     set maskName = mask/test%03d.fid
   else if ($fmtCount == 2) then
     set maskName = mask/test%03d%03d.fid
   else
     set maskName = mask.fid
   endif
endif

if ($pipeFlag == 0) then
   if ($fmtCount == 0) then
      if ($dimCount > 2) then
         set dimCount = 2
      endif
   endif
endif

if ($dimCount == 1) then
   set ySize = 1
endif

set firstCol = 3
set lastCol  = 3

if ($dimCount > 1) then
   @ lastCol++
endif

if ($dimCount > 2) then
   @ lastCol++
endif

if ($dimCount > 3) then
   @ lastCol++
endif

if ($vFlag) then
   set zfInfo = (`showhdr $thisName | fgrep "APOD SIZE"`)

   echo ""
   echo "nusZF. Time-Domain Input: $inName"
   echo "nusZF. Initial Size:      $zfInfo[$firstCol-$lastCol]"
endif

if ($dimCount == 1) then
   if ($vFlag) then
      echo "nusZF. Zero Fill:         1D"
   endif

   nmrPipe -in $inName -out $outName -ov -fn $xZF

   if ($maskFlag) then
      nmrPipe -in $inName -fn SET -c 1.0 \
      | nmrPipe -fn $xZF -out $maskName -ov
   endif
else if ($dimCount == 2) then
   if ($vFlag) then
      echo "nusZF. Zero Fill:         2D"
   endif

   nmrPipe -in $inName \
   | nmrPipe -fn $xZF \
   | nmrPipe -fn TP \
   | nmrPipe -fn $yZF \
   | nmrPipe -fn TP \
     -out $outName -ov

   if ($maskFlag) then
      nmrPipe -in $inName \
      | nmrPipe -fn SET -c 1.0 \
      | nmrPipe -fn $xZF \
      | nmrPipe -fn TP \
      | nmrPipe -fn $yZF \
      | nmrPipe -fn TP \
        -out $maskName -ov
   endif
else if ($dimCount == 3 && $fmtCount == 1) then
   if ($vFlag) then
      if ($pseudo3DFlag) then
         echo "nusZF. Zero Fill:         Pseudo-3D"
      else
         echo "nusZF. Zero Fill:         3D"
      endif
   endif

   xyz2pipe -in $inName -x \
   | nmrPipe -fn $xZF \
   | nmrPipe -fn TP \
   | nmrPipe -fn $yZF \
   | nmrPipe -fn TP \
   | nmrPipe -fn ZTP \
   | nmrPipe -fn $zZF \
   | nmrPipe -fn ZTP \
   | pipe2xyz -out $outName -x

   if ($maskFlag) then
      xyz2pipe -in $inName -x \
      | nmrPipe -fn SET -c 1.0 \
      | nmrPipe -fn $xZF \
      | nmrPipe -fn TP \
      | nmrPipe -fn $yZF \
      | nmrPipe -fn TP \
      | nmrPipe -fn ZTP \
      | nmrPipe -fn $zZF \
      | nmrPipe -fn ZTP \
      | pipe2xyz -out $maskName -x
   endif
else if ($dimCount == 3 && $fmtCount == 0) then
   if ($vFlag) then
      if ($pseudo3DFlag) then
         echo "nusZF. Zero Fill:         Pseudo-3D"
      else
         echo "nusZF. Zero Fill:         3D"
      endif
   endif

   nmrPipe -in $inName \
   | nmrPipe -fn $xZF \
   | nmrPipe -fn TP \
   | nmrPipe -fn $yZF \
   | nmrPipe -fn TP \
   | nmrPipe -fn ZTP \
   | nmrPipe -fn $zZF \
   | nmrPipe -fn ZTP \
    -out $outName -ov

   if ($maskFlag) then
      nmrPipe -in $inName \
      | nmrPipe -fn SET -c 1.0 \
      | nmrPipe -fn $xZF \
      | nmrPipe -fn TP \
      | nmrPipe -fn $yZF \
      | nmrPipe -fn TP \
      | nmrPipe -fn ZTP \
      | nmrPipe -fn $zZF \
      | nmrPipe -fn ZTP \
       -out $maskName -ov
   endif
else if ($dimCount == 4 && $fmtCount == 2) then
   if ($vFlag) then
      echo "nusZF. Zero Fill:         4D"
      echo ""
   endif

   xyz2pipe -in $inName -x $vArg \
   | nmrPipe -fn $xZF \
   | nmrPipe -fn TP \
   | nmrPipe -fn $yZF \
   | nmrPipe -fn TP \
   | nmrPipe -fn ZTP \
   | nmrPipe -fn $zZF \
   | nmrPipe -fn ZTP \
   | pipe2xyz -out $tmpDir/tmp%05d%05d.dat -x

   xyz2pipe -in $tmpDir/tmp%05d%05d.dat -a $vArg \
   | nmrPipe -fn $aZF \
   | pipe2xyz -out $outName -a -mem 64

   /bin/rm -rf $tmpDir 

   if ($maskFlag) then
      xyz2pipe -in $inName -x $vArg \
      | nmrPipe -fn SET -c 1.0 \
      | nmrPipe -fn $xZF \
      | nmrPipe -fn TP \
      | nmrPipe -fn $yZF \
      | nmrPipe -fn TP \
      | nmrPipe -fn ZTP \
      | nmrPipe -fn $zZF \
      | nmrPipe -fn ZTP \
      | pipe2xyz -out $tmpDir/tmp%05d%05d.dat -x

      xyz2pipe -in $tmpDir/tmp%05d%05d.dat -a $vArg \
      | nmrPipe -fn $aZF \
      | pipe2xyz -out $maskName -a -mem 64

      /bin/rm -rf $tmpDir 
   endif

   if ($vFlag) then
      echo ""
   endif
else if ($dimCount == 4 && $fmtCount == 1) then
   if ($vFlag) then
      echo "nusZF. Zero Fill:         4D"
      echo ""
   endif

   xyz2pipe -in $inName -x $vArg \
   | nmrPipe -fn $xZF \
   | nmrPipe -fn TP \
   | nmrPipe -fn $yZF \
   | nmrPipe -fn TP \
   | nmrPipe -fn ZTP \
   | nmrPipe -fn $zZF \
   | nmrPipe -fn ZTP \
   | pipe2xyz -out $tmpDir/tmp%05d.dat -x

   xyz2pipe -in $tmpDir/tmp%05d.dat -a $vArg \
   | nmrPipe -fn $aZF \
   | pipe2xyz -out $outName -a -mem 64

   /bin/rm -rf $tmpDir 

   if ($maskFlag) then
      xyz2pipe -in $inName -x $vArg \
      | nmrPipe -fn SET -c 1.0 \
      | nmrPipe -fn $xZF \
      | nmrPipe -fn TP \
      | nmrPipe -fn $yZF \
      | nmrPipe -fn TP \
      | nmrPipe -fn ZTP \
      | nmrPipe -fn $zZF \
      | nmrPipe -fn ZTP \
      | pipe2xyz -out $tmpDir/tmp%05d.dat -x

      xyz2pipe -in $tmpDir/tmp%05d.dat -a $vArg \
      | nmrPipe -fn $aZF \
      | pipe2xyz -out $maskName -a -mem 64

      /bin/rm -rf $tmpDir 
   endif

   if ($vFlag) then
      echo ""
   endif
else if ($dimCount == 4 && $fmtCount == 0) then
   if ($vFlag) then
      echo "nusZF. Zero Fill:         4D"
      echo ""
   endif

   nmrPipe -in $inName $vArg \
   | nmrPipe -fn $xZF \
   | nmrPipe -fn TP \
   | nmrPipe -fn $yZF \
   | nmrPipe -fn TP \
   | nmrPipe -fn ZTP \
   | nmrPipe -fn $zZF \
   | nmrPipe -fn ZTP \
   | nmrPipe -fn ATP \
   | nmrPipe -fn $aZF \
   | nmrPipe -fn ATP \
    -out $outName -ov

   if ($maskFlag) then
      nmrPipe -in $inName $vArg \
      | nmrPipe -fn SET -c 1.0 \
      | nmrPipe -fn $xZF \
      | nmrPipe -fn TP \
      | nmrPipe -fn $yZF \
      | nmrPipe -fn TP \
      | nmrPipe -fn ZTP \
      | nmrPipe -fn $zZF \
      | nmrPipe -fn ZTP \
      | nmrPipe -fn ATP \
      | nmrPipe -fn $aZF \
      | nmrPipe -fn ATP \
       -out $maskName -ov
   endif

   if ($vFlag) then
      echo ""
   endif
else
   echo "nusZF. Error: Unsupported Input Format and Dimension Count."
   exit 1
endif

if ($vFlag) then
   set thisName = (`nmrPrintf $outName 1 1`)

   if (-e $thisName) then
      set zfInfo = (`showhdr $thisName | fgrep "APOD SIZE"`)

      echo "nusZF. Zero Fill Output:  $outName"
      echo "nusZF. Final Size:        $zfInfo[$firstCol-$lastCol]"
   endif

   if ($maskFlag) then
      set thisName = (`nmrPrintf $maskName 1 1`)

      if (-e $thisName) then
         echo "nusZF. Mask Output:       $maskName"
      endif
   endif
endif

if (!(`string csame $nusName None`)) then
   if (-e $nusName) then
      /bin/rm $nusName
   endif

   set n = 0

   if ($dimCount == 1 || $dimCount == 2) then
      set iy = 0

      while ($iy < $ySize)
         nmrPrintf " %5d\n" $iy >> $nusName
         @ iy++
         @ n++
      end
   else if ($dimCount == 3) then
      set iz = 0

      while ($iz < $zSize)
         set iy = 0

         while ($iy < $ySize)
            nmrPrintf " %5d %5d\n" $iy $iz >> $nusName
            @ iy++
            @ n++
         end

         @ iz++
      end
   else
     set ia = 0
   
     while ($ia < $aSize)
        set iz = 0

         while ($iz < $zSize)
            set iy = 0

            while ($iy < $ySize)
               nmrPrintf " %5d %5d %5d\n" $iy $iz $ia >> $nusName
               @ iy++
               @ n++
            end
            @ iz++
         end
         @ ia++
      end
   endif

   if ($vFlag) then
      echo "nusZF. Schedule Output:   $nusName"
      echo "nusZF. Sample Count:      $n"
      echo ""
   endif
endif

