#!/bin/csh

if (`flagLoc $argv -help`) then
   echo ""
   echo "General-Purpose Submission Script for NUScon 2018 https://nuscon.org"
   echo "Version 7.0 2018.082.12.25 frank.delaglio@nist.gov"
   echo "" 
   echo "NMRPipe-format Time-Domain Input:"
   echo " -in         inName    [fid/test%03d.fid]    Expanded 3D Time-Domain Data." 
   echo " -nus        nusName   [nus.fid]             Un-Expanded 2D Version of Data." 
   echo " -mask       maskName  [mask/test%03d.fid]   3D NUS Mask, or Keyword 'None'."
   echo "NUS Sampling Schedule Text File Input:"
   echo " -schedName  schedName [nuslist]             Schedule File, or Keyword 'None'."
   echo "Required NMRPipe-format Spectrum Output:"
   echo " -out        outName   [recon/test%03d.ft3]  3D Spectrum, Real Only."
   echo "Optional 3D Interferogram Output:"
   echo " -ft1        ft1Name   [recon_ft1/test%03d.ft1] 3D Interferogram."
   echo "Required Region of Interest (Output Must Include this Range of Chemical Shifts):"
   echo " -x1PPM      x1PPM     [10.8]     X-Axis Start of Region of Interest, ppm."
   echo " -xnPPM      xnPPM     [5.8]      X-Axis End of Region of Interest, ppm."
   echo "Sampling Schedule Details (Use of This Information is Optional):"
   echo " -ySchedType ysType    [uniform]  Y-Axis Profile: uniform exp pgsin pgcos other."
   echo " -zSchedType zsType    [uniform]  Z-Axis Profile: uniform exp pgsin pgcos other."
   echo "Scoring Intent (Use of This Information is Optional):"
   echo " -intent     intent    [combined] Goal Keyword: fidelity accuracy combined."
   echo ""
   echo "Notes:"
   echo ""
   echo " 1. This script is an example for invoking your processing method on a given"
   echo "    conventional or Non-Uniformly Sampled (NUS) 3D input for NUScon 2018,"
   echo "    the contest to evaluate spectral reconstruction methods. https://nuscon.org"
   echo ""
   echo " 2. Make your submission by editing this script starting at the 'Edit Here'"
   echo "    section to execute your reconstruction method."
   echo ""
   echo " 3. If the script is given input including a NUS schedule, it should apply a"
   echo "    NUS reconstruction. If a NUS schedule is not given, it should apply a"
   echo "    conventional Fourier (FT) scheme."
   echo ""
   echo " 4. The script is intended to take NMRPipe-format time-domain input, and"
   echo "    produce an NMRPipe-format spectrum as output. The notes here use NMRPipe"
   echo "    nomenclature, with the directly-detected dimension being the X-Axis, and"
   echo "    the indirect dimensions being the Y-Axis and Z-Axis." 
   echo ""
   echo " 5. The full submission for NUScon 2018 includes one version of this script"
   echo "    for each of the challenge spectra, renamed to identify it. If your script"
   echo "    uses other scripts or files, name these accordingly and include them with"
   echo "    the submission. A more complete description of how to name the scripts can"
   echo "    be found here: https://nuscon.org/rules"
   echo ""
   echo "    For example, if your script needs its own auxiliary script 'msa2d.com',"
   echo "    adjust it to use a renamed version of your auxiliary script:"
   echo ""
   echo "      mark_protA_HNCA.com        Your version of this submission script." 
   echo "      mark_protA_HNCA_msa2d.com  Auxiliary script used by your submission."
   echo ""
   echo " 6. A given script will be invoked repeatedly with versions of the challenge"
   echo "    data resampled with different NUS schedules."
   echo ""
   echo " 7. The script will also be invoked with conventional data and no NUS schedule."
   echo "    If the script is invoked without a NUS schedule as input (-schedName None),"
   echo "    the script should apply conventional Fourier processing only, and generate"
   echo "    data with the same sizes and chemical shift ranges as a corresponding NUS"
   echo "    reconstruction would have. The conventional Fourier result will be used"
   echo "    to help score those NUS methods which produce an optional interferogram."
   echo ""
   echo " 8. In the case of Bruker input data, the NMRPipe-format input will have been"
   echo "    converted via 'bruk2pipe -AMX ...' so that it is not corrected for digital"
   echo "    oversampling."
   echo ""
   echo " 9. For NUS input, there are two versions of the time-domain data available."
   echo "    The input specified via the -in option is the NMRPipe-format 3D time-domain"
   echo "    data, already sorted and expanded with zeros. The input specified via the"
   echo "    -nus option is the unexpanded, unsorted NUS time-domain data saved in 2D"
   echo "    format, but with chemical shift calibration parameters set for all three"
   echo "    dimensions, Specifically, if the number of total points in the Y-Axis of"
   echo "    the 3D is yN and the number of total points in the Z-Axis of the 3D is zN,"
   echo "    then the differences in 3D and 2D headers will be:"
   echo ""
   echo "       Parameter            3D Value   2D Value"
   echo ""
   echo "       FDSPECNUM              yN        yN*zN"
   echo "       FDDIMCOUNT              3            2"
   echo "       FDNUSDIM                0            3"
   echo "       FDFILECOUNT            zN            1"
   echo "       FDF3SIZE               zN            1"
   echo "       FDF3QUADFLAG            0            1"
   echo ""
   echo "10. The input supplied by -mask is the NUS mask, a copy of the 3D time-domain"
   echo "    input supplied by the -in option, but with intensities set to either one"
   echo "    or zero depending on whether that data was measured or skipped in the NUS"
   echo "    schedule. Its use is optional."
   echo ""
   echo "11. The script should not change the input data; if your script needs to"
   echo "    adjust the input data header parameters, make a copy of the input data"
   echo "    and change the copy instead."
   echo ""
   echo "12. The directory where the script will be executed will contain the"
   echo "    vendor parameter files (Bruker acq* pulseprogram*, Varian procpar)"
   echo "    for the original fully-sampled data."
   echo ""
   echo "13. The script will be executed so that no other scripts are being used in"
   echo "    the same directory at the same time." 
   echo ""
   echo "14. Directories for -out and -ft1 will not exist when the script is executed."
   echo ""
   echo "15. If the optional interferogram output -ft1 is created by your scheme, there"
   echo "    will be an opportunity to score your method for agreement with the fully-"
   echo "    sampled interferogram. The interferogram should have only the X-Axis"
   echo "    processed, with the X-Axis imaginary components deleted." 
   echo ""
   echo "16. The output data -out and -ft1 created by your method should be in the same"
   echo "    axis order as the input 3D time-domain data."
   echo ""
   echo "17. Headers of the NMRPipe-format output must be rigorously correct. This means"
   echo "    that chemical shift calibration parameters should be maintained correctly."
   echo "    It also means that if your method extrapolates data, it should adjust the"
   echo "    header parameters of the result to record the new number of valid points in"
   echo "    the time-domain."
   exit 0
endif

set thisArgv = ($argv)

set inName       = (`getArgD $thisArgv -in         fid/test%03d.fid`)
set nusName      = (`getArgD $thisArgv -nus        nus.fid`)
set maskName     = (`getArgD $thisArgv -mask       mask/test%03d.fid`)
set outName      = (`getArgD $thisArgv -out        recon/test%03d.ft3`)
set ft1Name      = (`getArgD $thisArgv -ft1        recon_ft1/test%03d.ft1`)

set schedName    = (`getArgD $thisArgv -schedName  nuslist`)
set ysType       = (`getArgD $thisArgv -ySchedType random`)
set zsType       = (`getArgD $thisArgv -zSchedType random`)
set intent       = (`getArgD $thisArgv -intent     combined`)

set x1PPM        = (`getArgD $thisArgv -x1PPM      10.8`)
set xnPPM        = (`getArgD $thisArgv -xnPPM      5.8`)

if (`string csame $schedName None`) then
   set ftOnlyFlag = 1
else
   set ftOnlyFlag = 0
endif

#
#---------------------------------------------------------------------------------------------

#
# Edit Here ...
#
# Insert your reconstruction here:

#
# This example scheme performs the reconstruction using SMILE.
# It uses the expanded NUS input (inName) and ignores the unexpanded input (nusName).
# It ignores information about the sampling schedule type (-ySchedType -zSchedType).
# It uses the NUS schedule text file (schedName) and ignores the NUS mask (maskName).
# By requirement, the result includes the requested chemical shift range (x1PPM xnPPM).
# If ftOnlyFlag is 1 (set when '-schedName None' is used), only conventional Fourier processing is applied.
# The resulting spectrum (outName) must have the same axis order as the input data (inName). 
# This example also creates the optional interferogram output (ft1Name).

#
# Process the directly detected dimension, and reorganize the data for SMILE, which
# requires that the indirect dimensions are the first two dimensions of the result:

xyz2pipe -in $inName -x \
| nmrPipe -fn POLY -time \
| nmrPipe -fn SP -off 0.5 -end 0.95 -pow 2 -elb 0.0 -glb 0.0 -c 0.5 \
| nmrPipe -fn ZF -zf 1 -auto \
| nmrPipe -fn FT \
| nmrPipe -fn PS -p0 -158 -p1 0.0 -di \
| nmrPipe -fn EXT -x1 ${x1PPM}ppm -xn ${xnPPM}ppm -sw \
| pipe2xyz -out tmpA/test%05d.ft1 -z -ov

#
# If ftOnlyFlag is 1, only conventional Fourier processing (FT) should be applied.
#
# In the conventional FT case, window functions and additional zero filling are 
# applied, so that the conventional FT result will match the size of a SMILE 
# result, which includes extrapolation, and already has window functions applied.

if ($ftOnlyFlag) then
   xyz2pipe -in tmpA/test%05d.ft1 -x -verb \
   | nmrPipe -fn SP -off 0.5 -end 0.98 -pow 1 -c 0.5 \
   | nmrPipe -fn ZF \
   | nmrPipe -fn TP \
   | nmrPipe -fn SP -off 0.5 -end 0.98 -pow 1 -c 0.5 \
   | nmrPipe -fn ZF \
   | nmrPipe -fn TP \
   | pipe2xyz -out tmpB/test%03d.ft1 -x -ov
else
   xyz2pipe -in tmpA/test%05d.ft1 -x -verb \
   | nusPipe -fn SMILE -sample $schedName -nDim 3 -maxIter 2048 -report 1 -xCT 32 -yCT 0 \
             -xT 180 -xP0 0.0 -xP1 0.0 -xApod SP -xQ1 0.50 -xQ2 0.98 -xQ3 1 -xELB 0.0 -xGLB 0.0 -xNeg \
             -yT  88 -yP0 0.0 -yP1 0.0 -yApod SP -yQ1 0.50 -yQ2 0.98 -yQ3 1 -yELB 0.0 -yGLB 0.0 -yNeg -yAlt \
   | pipe2xyz -out tmpB/test%03d.ft1 -x -ov
endif

#
# The interferogram output is optional.
# Re-arrange the interferogram so that it is in the original dimension order:

xyz2pipe -in tmpB/test%03d.ft1 -z | pipe2xyz -out $ft1Name -ov

#
# When SMILE is complete, the result is an interpolated, extrapolated interferogram.
# Since SMILE has already applied window functions, we process the indirect dimensions
# without window functions:

xyz2pipe -in $ft1Name -x -verb \
| nmrPipe -fn POLY -auto \
| nmrPipe -fn TP \
#| nmrPipe -fn SP -off 0.5 -end 0.98 -pow 1 -c 0.5 \
| nmrPipe -fn ZF -zf 1 -auto \
| nmrPipe -fn FT -neg \
| nmrPipe -fn PS -p0 0.0 -p1 0.0 -di \
| nmrPipe -fn TP \
| nmrPipe -fn ZTP \
#| nmrPipe -fn SP -off 0.5 -end 0.98 -pow 1 -c 0.5 \
| nmrPipe -fn ZF -zf 1 -auto \
| nmrPipe -fn FT -alt -neg \
| nmrPipe -fn PS -p0 0.0 -p1 0.0 -di \
| nmrPipe -fn ZTP \
| pipe2xyz -out $outName -x -ov

#
# Remove temporary data:

/bin/rm -rf tmpA tmpB

