#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Draw Overlay of 2D Planes in a 3D Spectrum:"
    puts " -in inName \[ft/test%03d.ft3\]  Input 3D Spectrum."
    puts " -hi hi     \[Auto\]             Contour Height."
    puts "Plane Orientation Options (Use Only One):"
    puts " -xy -xz -yx -yz -zx -zy" 
    exit 1
   }

set inName ft/test%03d.ft3
set aList  "X Y Z"
set hi     Auto

getArgD $ARGV -in inName
getArgD $ARGV -hi hi

if {[flagLoc $ARGV -xy]} {set aList "X Y Z"} 
if {[flagLoc $ARGV -xz]} {set aList "X Z Y"} 
if {[flagLoc $ARGV -yx]} {set aList "Y X Z"} 
if {[flagLoc $ARGV -yz]} {set aList "Y Z X"} 
if {[flagLoc $ARGV -zx]} {set aList "Z X Y"} 
if {[flagLoc $ARGV -zy]} {set aList "Z Y X"} 

set xName [lindex $aList 0]_AXIS
set yName [lindex $aList 1]_AXIS
set zName [lindex $aList 2]_AXIS

#
# Create window with ordinary TCL/TK:

wm geom  . +20+20
wm title . "Working ..." 

set xSize 600
set ySize 580

set cx1 20
set cxn 500
set cy1 20
set cyn 500

canvas .c -width $xSize -height $ySize -border 0
button .b -text Quit -command exit
pack   .c .b -side top

#
# Create spectral drawing area with special NMRWish "pixmap" command.
# Define a drawing region with the NMRWish "defineCell" command.

.c create pixmap 0 0 -width $xSize -height $ySize -win 1

defineCell -x $cx1 $cxn -y $cy1 $cyn -cell 1 -verb -win 1
drawRec    -x 0% 100% -y 0% 100% -cell 1 -fg black

#
# Read the spectrum header.
# Find the number of planes to draw.
# Find the axis names for labeling the window.

set error [rdFDATA $inName fdata]

if {$error} {exit 1}

set zSize [getParmI $fdata NDSIZE  $zName]

set xLab  [getParmStr $fdata NDLABEL $xName]
set yLab  [getParmStr $fdata NDLABEL $yName]
set zLab  [getParmStr $fdata NDLABEL $zName]

#
# For each 2D plane:
#   Read the 2D plane as a ROI (region of interest)
#   Draw the 2D ROI plane.

for {set iz 1} {$iz <= $zSize} {incr iz} \
   {
    readROI -ndim 2 -roi 1 -in $inName -x $xName -y $yName -dz $zName $iz 0
  
    drawROI -roi 1 -cell 1 -hi $hi -bg None \
            -color1 multi -color2 multi -axis Bottom Right

    update
   }

wm title . "Done: $inName $xLab/$yLab $zSize $zLab Planes"

update
