#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts stderr "Display NMRPipe Format Series as Text Data;"
    puts stderr " $argv0 \[fileName1 fileName2 ...\]"
    puts stderr "Options:"
    puts stderr " -index indexType \[Pts\]    Point Index Used for Output."
    puts stderr " -fmt   hiFmt     \[%12.5f\] Point Index Used for Output."
    puts stderr " -real                     Display X-Axis Real Data Only."
    puts stderr " -imag                     Display X-Axis Imaginary Data Only."
    puts stderr " -nodata                   Display Index Values Only."
    puts stderr " -xy                       Display 1D Data with Two Coords."
    puts stderr " -verb                     Verbose Mode ON."
    puts stderr "Notes:"
    puts stderr " 1. All data files in the series are assumed to have"
    puts stderr "    the same sizes and spectral windows."
    puts stderr " 2. Point Index Keywords: Pts Hz PPM sec % None"
    puts stderr " 3. Use - or keyword 'stdin' for pipeline input."
    exit 0
   }

set realFlag 1
set imagFlag 1
set nodFlag  0
set xyFlag   0
set mode     pts
set verbose  0

set hiFmt    "% 14.5f"

getArgD $ARGV -index mode
getArgD $ARGV -fmt   hiFmt

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -real]}   {set imagFlag 0}
if {[flagLoc $ARGV -imag]}   {set realFlag 0}
if {[flagLoc $ARGV -nodata]} {set nodFlag  1}
if {[flagLoc $ARGV -xy]}     {set xyFlag   1}

#
#--------------------------------------------------------------------------

set aSize [llength $argv]
set zSize 0
set iz    0

set pCount 0
set iCount 0

for {set ia 0} {$ia < $aSize} {incr ia } \
   {
    set thisArg [lindex $argv $ia]

    if {![strcasecmp $thisArg "-real"]}   continue
    if {![strcasecmp $thisArg "-imag"]}   continue
    if {![strcasecmp $thisArg "-nodata"]} continue
    if {![strcasecmp $thisArg "-xy"]}     continue
    if {![strcasecmp $thisArg "-verb"]}   continue
    if {![strcasecmp $thisArg "-noverb"]} continue
    if {![strcasecmp $thisArg "-notk"]}   continue
    if {![strcasecmp $thisArg "--"]}      continue

    if {![strcasecmp $thisArg "-index"] || ![strcasecmp $thisArg "-fmt"]} \
       {
        incr ia
        continue
       }
 
    set inName $thisArg

    if {![strcasecmp $inName "-"] || ![strcasecmp $inName stdin]} \
       {
        incr pCount 
        incr iCount
       } \
    else \
       {
        if {![file exists $inName]} \
           {
            puts stderr "Error Finding Input File $inName"
            exit 1
           }

        incr iCount
       }

    incr zSize
   }

if {$pCount > 1} \
   {
    puts stderr "Error: Multiple Inputs from stdin."
    exit 1
   }

if {!$iCount} \
   {
    lappend argv stdin

    set  aSize [llength $argv]
    incr zSize
   }

for {set ia 0} {$ia < $aSize} {incr ia } \
   {
    set thisArg [lindex $argv $ia]

    if {![strcasecmp $thisArg "-real"]}   continue
    if {![strcasecmp $thisArg "-imag"]}   continue
    if {![strcasecmp $thisArg "-nodata"]} continue
    if {![strcasecmp $thisArg "-xy"]}     continue
    if {![strcasecmp $thisArg "-notk"]}   continue
    if {![strcasecmp $thisArg "--"]}      continue
 
    if {![strcasecmp $thisArg "-index"] || ![strcasecmp $thisArg "-fmt"]} \
       {
        incr ia
        continue
       }

    set  inName $thisArg

    if {![strcasecmp $inName "-"] || ![strcasecmp $inName stdin]} \
       {
        set inUnit 0
       } \
    else \
       {
        set inUnit [sysOpen $inName r]
       }

    set error  [rdFDATAU $inUnit fdata]

    if {$error} {exit 1}

    set qState [getParmI $fdata FDQUADFLAG NULL_DIM]

    if {$qState == 0.0} \
       {
        set qSize 2
       } \
    else \
       {
        set qSize 1
       }

    set dimCount [getParmI $fdata FDDIMCOUNT NULL_DIM]
    set pipeFlag [getParmI $fdata FDPIPEFLAG NULL_DIM]
    set cubeFlag [getParmI $fdata FDCUBEFLAG NULL_DIM]

    set xSize    [getParmI $fdata NDSIZE CUR_XDIM]
    set ySize    [getParmI $fdata NDSIZE CUR_YDIM]
    set pSize    1

    if {$dimCount < 2} {set ySize 1}

    if {$pipeFlag} \
       {
        if {$dimCount == 3} \
           {
            set pSize [getParmI $fdata NDSIZE CUR_ZDIM]
           }
       }

    set n     [expr $pSize*$xSize*$ySize*$qSize]
    set rdata [sysMalloc $n float]

    if {$inUnit} \
       {
        sysRead  $inUnit $rdata $n
        sysClose $inUnit
       } \
    else \
       {
        sysReadB  $inUnit $rdata $n
       }

    if {$verbose} {puts stderr $inName}

    set rPtr $rdata

    for {set ip 1} {$ip <= $pSize} {incr ip} \
        {
         incr iz

         for {set iy 1} {$iy <= $ySize} {incr iy} \
             {
              set i 0

              for {set ix 1} {$ix <= $xSize} {incr ix} \
                 {
                  if {$qSize == 2} \
                     {
                      set rVal   [getPtrValFmt $rPtr $i %e]
                      set iVal   [getPtrValFmt $rPtr [expr $i + $xSize] %e]

                      if {!$realFlag} \
                         {
                          set valStr [format " $hiFmt" $iVal]
                         } \
                      elseif {!$imagFlag} \
                         {
                          set valStr [format " $hiFmt" $rVal]
                         } \
                      else \
                         {
                          set valStr [format " $hiFmt $hiFmt" $rVal $iVal]
                         }
                     } \
                  else \
                     {
                      set rVal   [getPtrValFmt $rPtr $i %e]
                      set valStr [format " $hiFmt" $rVal]
                     }

                  if {![strcasecmp $mode None]} \
                     {
                      set locStr ""
                     } \
                  elseif {![strcasecmp $mode pts]} \
                     {
                      if {$pSize > 1 || ($ySize > 1 && $zSize > 1)} \
                         {
                          set locStr [format " %5d %5d %4d" $ix $iy $iz]
                         } \
                      elseif {$ySize == 1 && $zSize > 1} \
                         {
                          set locStr [format " %5d %4d" $ix $iz]
                         } \
                      elseif {$ySize > 1 && $zSize == 1} \
                         {
                          set locStr [format " %5d %5d" $ix $iy]
                         } \
                      else \
                         {
                          if {$xyFlag} \
                             {
                              set locStr [format " %5d   1" $ix]
                             } \
                          else \
                             {
                              set locStr [format " %5d" $ix]
                             }
                         }
                     } \
                  else \
                     {
                      set px 0.0
                      set py 0.0
                      set pz 0.0
     
                      if {$dimCount >= 1} {set px [pnt2spec $fdata X_AXIS $ix $mode]}
                      if {$dimCount >= 2} {set py [pnt2spec $fdata Y_AXIS $iy $mode]}
                      if {$dimCount >= 3} {set pz [pnt2spec $fdata Z_AXIS $iz $mode]}
     
                      switch -exact [string tolower $mode] \
                         {
                          %       {set fmt "% 8.3f"}
                          ppm     {set fmt "% 8.3f"}
                          hz      {set fmt "% 10.3f"}
                          default {set fmt "% e"}
                         }
     
                      if {$pSize > 1 || ($ySize > 1 && $zSize > 1)} \
                         {
                          set locStr [format " $fmt $fmt $fmt" $px $py $pz]
                         } \
                      elseif {$ySize == 1 && $zSize > 1} \
                         {
                          set locStr [format " $fmt %4d" $px $iz]
                         } \
                      elseif {$ySize > 1 && $zSize == 1} \
                         {
                          set locStr [format " $fmt $fmt" $px $py]
                         } \
                      else \
                         {
                          if {$xyFlag} \
                             {
                              set locStr [format " $fmt   0.0" $px]
                             } \
                          else \
                             {
                              set locStr [format " $fmt" $px]
                             }
                         }
                     }
    
                  if {$nodFlag} \
                     {
                      puts " $locStr"
                     } \
                  else \
                     {
                      puts " $locStr  $valStr"
                     }

                  incr i
                 }

              set rPtr [incrPtr $rPtr [expr $xSize*$qSize]]
             }
        }

    sysFree  $fdata
    sysFree  $rdata
   }

exit 0
