#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

#
# pix.tcl: simple 1D/2D drawing demo.
#--------------------------------------------------

#
# Procedure to move the 1D cursor.

proc moveCursor { id ix iy } \
{
   global cx1 cxn cy1 cyn

   if {$ix < $cx1 || $ix > $cxn} return
   if {$iy < $cy1 || $iy > $cyn} return

   .c coords $id $cx1 $iy $cxn $iy

   draw1D $iy
}

#
# Procedure to draw the selected 1D vector. 

proc draw1D { iy } \
{
  global cx1 cxn cy1 cyn minVal maxVal

  set yLoc [expr 100.0*($cyn - $iy)/($cyn - $cy1)]%

  readROI -roi 2 -ndim 1 -inROI 1 -x HN -dy CO $yLoc
  drawROI -roi 2 -cell 2 -zero -bg black -fhi $minVal -lhi $maxVal

  update
}

proc scale1D { iy } \
{
   global minVal maxVal iyPrev

   if {$iy > $iyPrev} \
      {
       set minVal [expr $minVal/0.9]
       set maxVal [expr $maxVal/0.9]
      } \
   else \
      {
       set minVal [expr $minVal*0.9]
       set maxVal [expr $maxVal*0.9]
      }

   drawROI -roi 2 -cell 2 -zero -bg black -fhi $minVal -lhi $maxVal

   set iyPrev $iy
}

proc move1D { iy } \
{
   global minVal maxVal iyPrev

   set range [expr $maxVal - $minVal]

   if {$iy > $iyPrev} \
      {
       set minVal [expr $minVal + $range*0.05]
       set maxVal [expr $maxVal + $range*0.05]
      } \
   else \
      {
       set minVal [expr $minVal - $range*0.05]
       set maxVal [expr $maxVal - $range*0.05]
      }

   drawROI -roi 2 -cell 2 -zero -bg black -fhi $minVal -lhi $maxVal

   set iyPrev $iy
}

#
# Definition of window sizes:

set xSize1D  600
set ySize1D  240

set xSize2D  650
set ySize2D  400

set cx1   10 
set cxn   [expr $xSize2D - 100]
set cy1   10 
set cyn   [expr $ySize2D - 70]

set qx1   10 
set qxn   [expr $xSize1D - 100]
set qy1   10 
set qyn   [expr $ySize1D - 10]

set xOff  [expr 2*$cx1]
set yOff  [expr 70 + $ySize2D]

set minVal -2.0e+4
set maxVal  2.0e+5

#
# Creation of Tk windows:

toplevel .t
frame    .cmndFrame

canvas   .c   -width $xSize2D -height $ySize2D -border 0 -closeenough 25
canvas   .t.c -width $xSize1D -height $ySize1D -border 0

button   .bQuit -text  Quit -command exit

pack   .c .bQuit -side top -fill x -expand 1
pack   .t.c

wm geometry  . +0+0
wm geometry .t +$xOff+$yOff
raise .

#
# Create pixmaps for spectral rendering in the two windows.
# Define drawing areas in each of the windows.
# Read and draw the 2D spectrum.

.c   create pixmap 0 0 -width $xSize2D -height $ySize2D -win 1
.t.c create pixmap 0 0 -width $xSize1D -height $ySize1D -win 2

defineCell -x $cx1 $cxn -y $cy1 $cyn -cell 1 -verb -win 1
defineCell -x $cx1 $cxn -y $qy1 $qyn -cell 2 -verb -win 2

newScreen

readROI -ndim 2 -roi 1 -in test.ft2 -x HN -y CO

drawROI -roi 1 -cell 1 \
    -bg black -color1 multi -color2 multi -axis Bottom Right

#
# Create the 1D cursor.
# Draw the initial 1D vector.

set iy     [expr 0.5*($cy1 + $cyn)]
set iyPrev $iy

set id [.c create line $cx1 $iy $cxn $iy -fill yellow]
.c bind $id <B1-Motion> "moveCursor $id %x %y"

bind .t.c <B1-Motion> "scale1D %y"
bind .t.c <B2-Motion> "move1D  %y"

draw1D $iy
