#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Insert or Update Distortion Values in a Peak Table:"
    puts " -in      inName      \[test.tab\] Input Peak Table."
    puts " -fid     fidName     \[test.fid\] Input Time-Domain Data (For Units Conversion)." 
    puts " -ft      ftName      \[test.t2\]  Input Freq-Domain Data (For Units Conversion)." 
    puts " -out     outName     \[adj.tab\]  Output Peak Table."
    puts " -gFactor gFactor     \[0.35\]     Factor for Creating Random Value from a Range."
    puts " -iseed   iseed       \[7294791\]  Random Number Seed (0 for Auto)."
    puts "Phase Distortions, Degrees (Use -x -y ...):"
    puts " -xP0Min  p0Min       \[0.0\]      Lower Bound for Phase Distortion."
    puts " -xP0Max  p0Max       \[0.0\]      Upper Bound for Phase Distortion." 
    puts "J Couplings, Hz (Use -x -y ...):"
    puts " -xJ1Min  j1Min       \[0.0\]      Lower Bound for J-Coupling 1." 
    puts " -xJ1Max  j1Min       \[0.0\]      Upper Bound for J-Coupling 1."
    puts "..."
    puts " -xJ4Min  j3Min       \[0.0\]      Lower Bound for J-Coupling 4."
    puts " -xJ4Max  j3Max       \[0.0\]      Upper Bound for J-Coupling 4."
    puts "Randomization of Chemical Shifts, ppm (Use -dx -dy ...):"
    puts " -dxMin   dxMin       \[0.0\]      Lower Bound for Adjustment." 
    puts " -dxMax   dxMax       \[0.0\]      Upper Bound for Adjustment." 
    puts "Randomization of Linewidths, Hz (Use -dxw -dyw ...):"
    puts " -dxwMin  dxwMin      \[0.0\]      Lower Bound for Adjustment."
    puts " -dxwMax  dxwMax      \[0.0\]      Upper Bound for Adjustment."
    puts " -xwMin   xwMin       \[0.0\]      Lower Bound for Final Value."
    puts " -xwMax   xwMax       \[None\]     Upper Bound for Final Value, or Keyword None."
    puts "Randomization of Heights, as Percent of Max Height:"
    puts " -dhiMin    dhiMin    \[0.0\]      Lower Bound for Adjustment."
    puts " -dhiMax    dhiMax    \[0.0\]      Upper Bound for Adjustment."
    puts "Randomization of Heights, as Percent of Each Height:"
    puts " -dhiMinRel dhiMinRel \[0.0\]      Lower Bound for Adjustment."
    puts " -dhiMaxRel dhiMaxRel \[0.0\]      Upper Bound for Adjustment."
    puts "Randomization of Height Evolution Values (Z_A0 Z_A1 ...)"
    puts " -dEvMin    dEvMin    \[0.0\]      Lower Bound for Adjustment."
    puts " -dEvMax    dEvMax    \[0.0\]      Upper Bound for Adjustment."
    puts "Conversion of Observed Linewidth to Time-Domain Decay (Use -x -y ...):"
    puts " -xwOff     xwOff     \[0.0\]      Linewidth Offset, Hz." 
    puts " -xwMult    xwMult    \[1.0\]      Linewidth Scale Factor." 
    puts " -xwtMin    xwtMin    \[0.0\]      Lower Bound for Decay, Hz."
    puts " -xwtMax    xwtMax    \[None\]     Upper Bound for Decay, Hz, or Keyword None."
    puts "Notes:"
    puts " 1. Time-domain decay = xwMult*(freq-domain linewdth - xwOff), clipped to the"
    puts "    range xwtMin to xwtMax."
    puts " 2. The following formula is used for creating random values or perturbations"
    puts "    from an upper and lower bound (here, maxVal and minVal), with the result"
    puts "    clipped to the range minVal to maxVal. Here, gRandom is a random number"
    puts "    from a Gaussian distribution with standard deviation 1.0:"
    puts ""
    puts "       value = 0.5*(maxVal + minVal) + gFactor*(maxVal - minVal)*gRandom"
    exit 0
   }

set inName   test.tab
set outName  distort.tab
set fidName  test.fid
set ftName   test.ft2
set iseed    7294791
set verbose  1

set dEvMin    0.0
set dEvMax    0.0
set dhiMin    0.0
set dhiMax    0.0
set dhiMinRel 0.0
set dhiMaxRel 0.0

set gFactor   0.35

getArgD $ARGV -in    inName
getArgD $ARGV -out   outName
getArgD $ARGV -fid   fidName
getArgD $ARGV -ft    ftName
getArgD $ARGV -iseed iseed

getArgD $ARGV -dEvMin    dEvMin
getArgD $ARGV -dEvMax    dEvMax
getArgD $ARGV -dhiMin    dhiMin
getArgD $ARGV -dhiMax    dhiMax
getArgD $ARGV -dhiMinRel dhiMinRel
getArgD $ARGV -dhiMaxRel dhiMaxRel

getArgD $ARGV -gFactor   gFactor

if {[flagLoc $ARGV -verb]} {set verbose 0}
if {[flagLoc $ARGV -verb]} {set verbose 1}

foreach dim {x y z a} \
   {
    set p0Min($dim) 0.0
    set p0Max($dim) 0.0

    set ppmAdjMin($dim) 0.0
    set ppmAdjMax($dim) 0.0

    set lwOff($dim)  0.0
    set lwMult($dim) 1.0
    set dlwMin($dim) 0.0
    set dlwMax($dim) 0.0 
    set lwMin($dim)  0.0
    set lwMax($dim)  None 
    set lwtMin($dim) 0.0
    set lwtMax($dim) None 

    getArgD $ARGV -${dim}P0Min p0Min($dim)
    getArgD $ARGV -${dim}P0Max p0Max($dim)

    getArgD $ARGV -d${dim}Min  ppmAdjMin($dim)
    getArgD $ARGV -d${dim}Min  ppmAdjMax($dim)

    getArgD $ARGV -${dim}wOff  lwOff($dim)
    getArgD $ARGV -${dim}wMult lwMult($dim)
    getArgD $ARGV -${dim}wMin  lwMin($dim)
    getArgD $ARGV -${dim}wMax  lwMax($dim)
    getArgD $ARGV -d${dim}wMin dlwMin($dim)
    getArgD $ARGV -d${dim}wMax dlwMax($dim)
    getArgD $ARGV -${dim}wtMin lwtMin($dim)
    getArgD $ARGV -${dim}wtMax lwtMax($dim)

    foreach i {1 2 3 4} \
       {
        set jMin($i,$dim) 0.0
        set jMax($i,$dim) 0.0

        getArgD $ARGV -${dim}J${i}Min jMin($i,$dim)
        getArgD $ARGV -${dim}J${i}Max jMax($i,$dim)
       }
   }

#
#-------------------------------------------------------------------------------------------------------

proc getValInRange { minVal maxVal } \
{
   global gFactor

   if {$minVal == $maxVal} {return $minVal}

   if {$maxVal < $minVal} \
      {
       set tmpVal $maxVal
       set maxVal $minVal
       set minVal $tmpVal
      }

   set mid [expr 0.5*($maxVal + $minVal)]
   set val [expr $mid + $gFactor*($maxVal - $minVal)*[rand -gauss]]

   if {$val < $minVal} {set val $minVal}
   if {$val > $maxVal} {set val $maxVal}

   return $val
}

proc setVarToZero { t v } \
{
   foreach e [gdbGetInfo $t -parms entries] {gdbSet $e $v 0.0}
}

#
#-------------------------------------------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts stderr "pkDistort Error Finding Input Table $inName"
    exit 1
   }

set thisName [format $fidName 1 1]

if {![file exists $thisName]} \
   {
    puts stderr "pkDistort Error Finding Input Time-Domain Data $fidName ($thisName)"
    exit 1
   }

set error [rdFDATA $thisName fdataT]

set thisName [format $ftName 1 1]

if {![file exists $thisName]} \
   {
    puts stderr "pkDistort Error Finding Input Freq-Domain Data $ftName ($thisName)"
    exit 1
   }

set error [rdFDATA $thisName fdataF]

set t     [gdbRead table -in $inName]
set vList [gdbGetInfo $t -parms varNames]

srand $iseed

set dimCount 0

foreach dim {x y z a} \
   {
    if {[lsearch -exact $vList [string toupper $dim]_AXIS] < 0} continue

    incr dimCount
   
    set dimName [string toupper $dim]
    set thisVar ${dimName}TW

    if {[lsearch -exact $vList $thisVar] < 0} \
       {
        if {$verbose} {puts "pkDistort. Creating Variable $thisVar."}

        gdbCreate variable -parent $t -varType float -name $thisVar -varFmt %8.3f
        setVarToZero $t $thisVar
       }

    if {$p0Min($dim) != 0.0 || $p0Max($dim) != 0.0} \
       {
        set thisVar ${dimName}_P0

        if {[lsearch -exact $vList $thisVar] < 0} \
           {
            if {$verbose} {puts "pkDistort. Creating Variable $thisVar."}

            gdbCreate variable -parent $t -varType float -name $thisVar -varFmt %6.1f
            setVarToZero $t $thisVar
           }
       }

    foreach i {1 2 3 4} \
       {
        if {$jMin($i,$dim) != 0.0 || $jMax($i,$dim) != 0.0} \
           {
            set thisVar ${dimName}_COSJ$i

            if {[lsearch -exact $vList $thisVar] < 0} \
               {
                if {$verbose} {puts "pkDistort. Creating Variable $thisVar"}
    
                gdbCreate variable -parent $t -varType float -name $thisVar -varFmt %8.3f
                setVarToZero $t $thisVar
               }
           }
       }
   }

if {$verbose} {puts ""}

set eList [gdbGetInfo $t -parms entries]
set vList [gdbGetInfo $t -parms varNames]
         
set maxHi [getTabStat getMaxAbs $t HEIGHT]

if {$dhiMin != 0.0 || $dhiMax != 0.0} \
   {
    if {$verbose} {puts [format "pkDistort. Height Distortion Min: % .3f%%  Max: %.3f%% (Relative to Max Height)" $dhiMin($dim) $dhiMax($dim)]}

    foreach e $eList \
       {
        set hi [gdbGet $e HEIGHT]
        gdbSet $e HEIGHT [expr $hi + $maxHi*0.01*[getValInRange $dhiMin $dhiMax]]
       }
   }

if {$dhiMinRel != 0.0 || $dhiMaxRel != 0.0} \
   {
    if {$verbose} {puts [format "pkDistort. Height Distortion Min: % .3f%%  Max: %.3f%% (Relative to Each Peak)" $dhiMinRel $dhiMaxRel]}

    foreach e $eList \
       {
        set hi [gdbGet $e HEIGHT]
        gdbSet $e HEIGHT [expr $hi + $hi*0.01*[getValInRange $dhiMinRel $dhiMaxRel]]
       }
   }

if {$dEvMin != 0.0 || $dEvMax != 0.0} \
   {
    set evVar None

    foreach v {Y_A Z_A A_A} \
       {
        if {[lsearch -exact $vList ${v}0] < 0} continue
        set evVar $v
        break
       }

    if {![strcasecmp $evVar None]} \
       {
        puts "pkDistort. Error Finding Height Evolution Values in Input Table $inName"
        exit 1
       }

    for {set evN 1} {$evN < 4096} {incr evN} \
       {
        if {[lsearch -exact $vList ${evVar}$evN] < 0} break
       }

    if {$verbose} {puts [format "pkDistort. Evolution Distortion Min: % .4f  Max: %.4f% (%s0 ... %s%d)" $dEvMin $dEvMax] $evVar $evVar [expr $evN - 1]}

    foreach e $eList \
       {
        for {set i 0} {$i < $evN} {incr i} \
           {
            set hi [gdbGet $e $evVar$i]
            gdbSet $e $evVar$i [expr $hi + [getValInRange $dEvMin $dEvMax]]
           }
       }
   }
 
foreach dim {x y z a} \
   {
    if {[lsearch -exact $vList [string toupper $dim]_AXIS] < 0} continue

    set dimName [string toupper $dim]

    if {$dlwMin($dim) != 0.0 || $dlwMax($dim) != 0.0} \
       {
        if {$verbose} {puts [format "pkDistort. %s-Axis LW Distortion Min: %7.3f Hz  Max: %7.3f Hz" $dimName $dlwMin($dim) $dlwMax($dim)]}
        if {$verbose} {puts [format "pkDistort. %s-Axis LW Limits     Min: %7.3f Hz  Max: %7.3f Hz" $dimName $lwMin($dim)  $lwMax($dim)]}

        foreach e $eList \
           {
            set lw     [gdbGet $e ${dimName}W_HZ]
            set valHz  [expr $lw + [getValInRange $dlwMin($dim) $dlwMax($dim)]]
            set valPts [spec2pntW $fdataF CUR_${dimName}DIM ${valHz}Hz]
 
            gdbSet $e ${dimName}W_HZ $valHz
            gdbSet $e ${dimName}W    $valPts
           }
       }

    if {![strcasecmp $lwMax($dim) None]} \
       {
        if {$verbose} {puts [format "pkDistort. %s-Axis LW Limits     Min: %7.3f Hz  Max:     None"    $dimName $lwMin($dim)]}

        foreach e $eList \
           {
            set valHz [gdbGet $e ${dimName}W_HZ]

            if {$valHz < $lwMin($dim)} \
               {
                set valHz  $lwMin($dim)
                set valPts [spec2pntW $fdataF CUR_${dimName}DIM ${valHz}Hz]

                gdbSet $e ${dimName}W_HZ $valHz
                gdbSet $e ${dimName}W    $valPts
               }
           }
       } \
    else \
       {
        if {$verbose} {puts [format "pkDistort. %s-Axis LW Limits     Min: %7.3f Hz  Max: %7.3f Hz" $dimName $lwMin($dim)  $lwMax($dim)]}

        foreach e $eList \
           {
            set valHz [gdbGet $e ${dimName}W_HZ]

            if {$valHz < $lwMin($dim) || $valHz > $lwMax($dim)} 
               {
                if {$valHz < $lwMin($dim)} {set valHz $lwMin($dim)}
                if {$valHz > $lwMax($dim)} {set valHz $lwMax($dim)}

                set valPts [spec2pntW $fdataF CUR_${dimName}DIM ${valHz}Hz]

                gdbSet $e ${dimName}W_HZ $valHz
                gdbSet $e ${dimName}W    $valPts
               }
           }
       }

    if {$lwMult($dim) != 1.0 || $lwOff($dim) != 0.0} \
       {
        if {$verbose} {puts [format "pkDistort. %s-Axis Width to Decay Offset: %7.3f Hz Scale: %8.3e"    $dimName $lwOff($dim) $lwMult($dim)]}

        foreach e $eList \
           {
            set lw     [gdbGet $e ${dimName}W_HZ]
            set valHz  [expr $lwMult($dim)*($lw - $lwOff($dim))]

            set valPts [spec2pntW $fdataT CUR_${dimName}DIM ${valHz}Hz]

            gdbSet $e ${dimName}TW $valPts
           }
       }

    if {![strcasecmp $lwtMax($dim) None]} \
       {
        if {$verbose} {puts [format "pkDistort. %s-Axis Decay Limits  Min: %7.3f Hz  Max:     None"     $dimName $lwtMin($dim)]}

        set valMinPts [spec2pntW $fdataT CUR_${dimName}DIM $lwtMin($dim)Hz]

        foreach e $eList \
           {
            set valPts [gdbGet $e ${dimName}TW]  

            if {$valPts < $valMinPts} {set valPts $valMinPts}

            gdbSet $e ${dimName}TW $valPts
           }
       } \
    else \
       {
        if {$verbose} {puts [format "pkDistort. %s-Axis Decay Limits  Min: %7.3f Hz  Max: %8.3f Hz" $dimName $lwtMin($dim) $lwtMax($dim)]}

        set valMinPts [spec2pntW $fdataT CUR_${dimName}DIM $lwtMin($dim)Hz]
        set valMaxPts [spec2pntW $fdataT CUR_${dimName}DIM $lwtMax($dim)Hz]
 
        foreach e $eList \
           {
            set valPts [gdbGet $e ${dimName}TW]

            if {$valPts < $valMinPts} {set valPts $valMinPts}
            if {$valPts < $valMaxPts} {set valPts $valMaxPts}

            gdbSet $e ${dimName}TW $valPts
           }
       }

    if {$p0Min($dim) != 0.0 || $p0Max($dim) != 0.0} \
       {
        if {$verbose} {puts [format "pkDistort. %s-Axis P0 Distortion Min: %7.3f Deg Max: %8.3f Deg"  $dimName $p0Min($dim) $p0Max($dim)]}
        foreach e $eList {gdbSet $e ${dimName}_P0 [getValInRange $p0Min($dim) $p0Max($dim)]}
       }

    set nn 0

    foreach i {1 2 3 4} \
       {
        if {$jMin($i,$dim) != 0.0 || $jMax($i,$dim) != 0.0} \
           {
            set thisVar ${dimName}_COSJ$i

            if {$verbose} \
               {
                puts [format "pkDistort. %s-Axis J-Coupling %d  Min: %7.3f Hz  Max: %8.3f Hz" $dimName $i $jMin($i,$dim) $jMax($i,$dim)]
               }

            set jMinPts [spec2pntW $fdataT $dim $jMin($i,$dim)Hz]
            set jMaxPts [spec2pntW $fdataT $dim $jMax($i,$dim)Hz]

            foreach e $eList {gdbSet $e $thisVar [getValInRange $jMinPts $jMaxPts]}
           }

        incr nn
       }
   }

gdbWrite $t $outName
 
exit 0
