#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "PostScript Plot of Phi/Psi Angles for Fragments and PDB Files."
    puts " -mfr    mfrNameList  \[mfr.tab\]     Fragment Tables to Display."
    puts " -phiPsi aNameList    \[phiPsi.tab\]  Phi/Psi Tables to Display."
    puts " -ref    refPDBName   \[None\]        Reference PDB (for Sequence)."
    puts " -pdb    pdbNameList  \[ref.pdb\]     PDB Files to Display (Also: -in)."
    puts " -chain  chainID      \[None\]        PDB ChainID to Use."
    puts " -class  className    \[None\]        Structure Classification Input (See ss.tcl)."
    puts " -plot   plotPrefix   \[rama\]        Prefix for PostScript Output."
    puts " -title  plotTitle    \[None\]        Title for Output Pages."
    puts " -r1     r1           \[First\]       First Residue to Display."
    puts " -rN     rN           \[Last\]        Last Residue to Display."
    puts " -skip   nSkip        \[1\]           MFR Flanking Residues to Skip."
    puts " -verb                              Verbose Mode ON."
    puts "Notes:"
    exit 1
   }

set pdbNameList     ref.pdb 
set refName         None
set mfrNameList     None 
set mfrColorList    yellow 
set className       None
set chainName       None
set plotTitle       None
set plotPrefix      rama 
set jobID           None

set phiPsiNameList  phiPsi.tab
set phiPsiColorList violet 
 
set pdbColorList    darkorchid 

set minRank      1
set maxRank      5 
set minRes       First 
set maxRes       Last 
set skip         1

set selectColor  gray30 
set avgColor     orange 
set boxColor     None
set pageColor    gray90

set serverFlag    0
set useMFRSurf    1
set useCSSurf     1
set sortFlag      0
set chiFlag       1

set boxDX      0.12 
set boxDY      0.65

set foldFlag      1
set foldPhi     -30
set foldPsi    -130

set phiLimit     30 
set psiLimit     30 

set dotFlag       0
set lineFlag      1
set boxFlag       1

set verbose       0

set roiCount      0
set currentCellID 0
set currentResID  0
set refSrc        0
set classTab      0
set titleList     ""
set modelList     ""

set pdbDir     $env(PDBH_BASE)/pdbH
set surfDir    $env(SURF_DIR)
set fragDir    None

getListArgD $ARGV -mfr    mfrNameList
getListArgD $ARGV -in     pdbNameList
getListArgD $ARGV -pdb    pdbNameList
getListArgD $ARGV -phiPsi phiPsiNameList

getArgD $ARGV -ref   refName
getArgD $ARGV -chain chainName
getArgD $ARGV -class className
getArgD $ARGV -plot  plotPrefix
getArgD $ARGV -job   jobID

if {![strcasecmp $refName None] || ![strcasecmp $refName Auto]} {set refName [lindex $pdbNameList 0]}

getListArgD $ARGV -mfrColor    mfrColorList
getListArgD $ARGV -pdbColor    pdbColorList
getListArgD $ARGV -phiPsiColor phiPsiColorList

getListArgD $ARGV -title       plotTitle
getListArgD $ARGV -titleList   titleList
getListArgD $ARGV -modelList   modelList

set plotTitle [string trim $plotTitle "{}"]

getArgD $ARGV -r1   minRes
getArgD $ARGV -rN   maxRes
getArgD $ARGV -skip skip

getArgD $ARGV -minRank  minRank
getArgD $ARGV -maxRank  maxRank

getArgD $ARGV -pdbDir   pdbDir
getArgD $ARGV -fragDir  fragDir
getArgD $ARGV -surfDir  surfDir

getArgD $ARGV -boxColor  boxColor
getArgD $ARGV -pageColor pageColor

if {[flagLoc $ARGV -server]}   {set serverFlag 1}
if {[flagLoc $ARGV -noserver]} {set serverFlag 0}

if {[flagLoc $ARGV -sort]}    {set sortFlag 1}
if {[flagLoc $ARGV -nosort]}  {set sortFlag 0}

if {[flagLoc $ARGV -chi]}     {set chiFlag 1}
if {[flagLoc $ARGV -nochi]}   {set chiFlag 0}

if {[flagLoc $ARGV -dot]}     {set dotFlag 1}
if {[flagLoc $ARGV -nodot]}   {set dotFlag 0}

if {[flagLoc $ARGV -line]}    {set lineFlag 1}
if {[flagLoc $ARGV -noline]}  {set lineFlag 0}

if {[flagLoc $ARGV -box]}     {set boxFlag 1}
if {[flagLoc $ARGV -nobox]}   {set boxFlag 0}

if {[flagLoc $ARGV -fold]}    {set foldFlag 1}
if {[flagLoc $ARGV -nofold]}  {set foldFlag 0}

if {[flagLoc $ARGV -verb]}    {set verbose 1}
if {[flagLoc $ARGV -noverb]}  {set verbose 0}

#
#-------------------------------------------------------------------------------------------------

proc drawLineLW { x1 xn y1 yn fg thickFlag } \
{
   if {$thickFlag} \
      {
       drawLine -x $x1 $xn -y $y1 $yn -fg $fg -thick
      } \
   else \
      {
       drawLine -x $x1 $xn -y $y1 $yn -fg $fg
      }
}

proc initWin {} \
{
   global boxDX boxDY pageColor plotPrefix verbose

   if {$verbose} \
      {
       set listArg "-nolist"
      } \
   else \
      {
       set listArg "-list"
      }

   initMiniPlot -landscape -c1Width 0.060in -c1Height 0.079in -c2Width 0.080in -c2Height 0.105in \
                -px1 0.70 -py1 0.85 -pxn 10.0 -bwx 0.8 -bwy 0.8 -bdx $boxDX -bdy $boxDY \
                -bg $pageColor -prefix $plotPrefix $listArg -openProc showRamaTitle
}

proc initRef {} \
{
   global refName refSrc chainName className classTab minRes maxRes resNameInfo resCount classInfo usefulChi

   dynInit -fast

   if {![file exists $refName]} \
      {
       puts stderr "Error Finding PDB Reference File $refName"
       exit 1
      }

   set refSrc [dynRead -pdb -in $refName -nohetatm]

   if {![strcasecmp $chainName None]} \
      {
       set refInfo [dynGetInfo -src $refSrc]
      } \
   else \
      {
       set refInfo [dynGetInfo -src $refSrc -chain $chainName]
      }

   set r1 [lindex $refInfo 0]
   set rN [lindex $refInfo 1]

   if {![strcasecmp $minRes First]} {set minRes $r1}
   if {![strcasecmp $maxRes Last]}  {set maxRes $rN}

   rClip minRes $r1 $rN
   rClip maxRes $r1 $rN

   set resCount [expr 1 + $maxRes - $minRes]

   for {set r0 $minRes} {$r0 <= $maxRes} {incr r0} \
      {
       if {![strcasecmp $chainName None]} \
          {
           set resNameInfo($r0)  [lindex [dynGetInfo -src $refSrc -res $r0 -resName] 0]
          } \
       else \
          {
           set resNameInfo($r0)  [lindex [dynGetInfo -src $refSrc -res $r0 -chain $chainName -resName] 0]
          }

       set cisInfo($r0) ""
      }

   if {[strcasecmp $className None]} \
      {
       if {![file exists $className]} \
          {
           puts stderr "Error Finding SS Class File $className"
          } \
       else \
          {
           set classTab [gdbRead table -in $className]
           set eList    [gdbGetInfo $classTab -parms entries]

           foreach e $eList \
              {
               set resID     [gdbGet $e RESID]
               set thisChain [gdbGet $e CHAIN]
               set class     [gdbGet $e CLASS]
               set hb        [gdbGet $e HBOND_CLASS]
               set hbStr     [gdbGet $e HBOND]
               set cInfo     [split $class ","]
              
               if {[strcasecmp $chainName None] && [strcasecmp $chainName $thisChain]} continue
 
               if {![strcasecmp $class None] || ![strcasecmp $class Other]} \
                  {
                   set class ""
                  } \
               elseif {![strcasecmp $class BetaSheet]} \
                  {
                   if {![strcasecmp $hb Beta-A]} {set class BetaSheet-A}
                   if {![strcasecmp $hb Beta-P]} {set class BetaSheet-P}
                  } \
               elseif {[llength $cInfo] > 1} \
                  {
                   set class "[lindex $cInfo 0] ..."
                  }

               set classInfo($resID) $class 
              }
          }
      }

   set usefulChi(ARG) 4
   set usefulChi(LYS) 4
   set usefulChi(MET) 3
   set usefulChi(GLU) 3
   set usefulChi(GLN) 3
   set usefulChi(ASP) 2
   set usefulChi(ASN) 2
   set usefulChi(ILE) 2
   set usefulChi(LEU) 2
   set usefulChi(HIS) 2
   set usefulChi(his) 2
   set usefulChi(TRP) 2
   set usefulChi(TYR) 2
   set usefulChi(PHE) 2
   set usefulChi(THR) 1
   set usefulChi(VAL) 1
   set usefulChi(SER) 1
   set usefulChi(CYS) 1
   set usefulChi(cys) 1
   set usefulChi(ALA) 0
   set usefulChi(GLY) 0
   set usefulChi(PRO) 3
}

proc showRamaTitle {} \
{
   global pdbNameList chainName minRes maxRes plotTitle jobID serverFlag

   set thisName [lindex $pdbNameList 0]


   if {![strcasecmp $plotTitle None]} \
      {
       set nStr $thisName
      } \
   else \
      {
       set nStr $plotTitle
      }

   if {![strcasecmp $chainName None]} \
      {
       set tStr "$nStr residues $minRes to $maxRes"
      } \
   else \
      {
       set tStr "$nStr residues $minRes to $maxRes chain $chainName"
      }
      
   set hStr "PDB Rama Utility http://spin.niddk.nih.gov/NMRPipe on [sysTime %Y.%j.%H.%M]"

   if {$serverFlag} \
      {
       if {![strcasecmp $jobID None]} \
          {
           set hStr "PDB SS Utility http://spin.niddk.nih.gov/bax/nmrserver/pdbutil on [sysTime %Y.%j.%H.%M]"
          } \
       else \
          {
           set hStr "PDB SS Utility $jobID http://spin.niddk.nih.gov/bax/nmrserver/pdbutil"
          }
      } 

   drawText -x 50% -y 0.15in -center -fg gray  -text $hStr
   drawText -x 50% -y 0.33in -center -fg black -big -text $tStr 
}

proc plotRama {} \
{
   global minRes maxRes roiCount surfDir refSrc resNameInfo
   global useCSSurf useMFRSurf boxColor avgColor
   global cellID cellCount lastPlotFlag verbose

   set thisResList ""
   set roiCount    0
   set nPage       1

   set firstRes $minRes

   while {($firstRes % 10) != 1} {incr firstRes -1}

   for {set resID $firstRes} {$resID <= $maxRes} {incr resID} \
      {
       nextMiniPlot

       if {$resID < $minRes} continue
       if {![strcasecmp $resNameInfo($resID) None]} continue

       lappend thisResList $resID

       set resName  $resNameInfo($resID)
       set thisName $surfDir/$resName.dat
       set cellID($resID) $cellCount

       if {![file exists $thisName]} {set thisName $surfDir/zero.dat}

       if {[info exists roiList($resName)]} \
          {
           set roiID $roiList($resName)
          } \
       else \
          {
           incr roiCount
           set  roiID $roiCount
           set  roiList($resName) $roiID
    
           readROI -roi $roiID -ndim 2 -in $thisName
          }

       drawROI -roi $roiID -cell $cellID($resID) -hi [getSurfPct $roiID 0.4] -pix -bg white \
               -pSat1 0 -pSat2 0 -pVal1 0.50 -pVal2 1.0 -fg $boxColor \
               -plev 32 -nlev 32 

       set thisName [format scanCS/cs%03d.dat $resID]

       if {$useCSSurf && [file exists $thisName]} \
          {
           readROI -roi 666666 -ndim 2 -in $thisName

           set maxVal [getROIStat 666666 vMax]

           drawROI -roi 666666 -cell $cellID($resID) -bg None -fg None \
                   -pHue1 0.4 -pHue2 0.4 -hi [expr $maxVal - 1.0] -plev 1
          }

       set thisName [format mfrSurf/mfr%03d.dat $resID]

       if {$useMFRSurf && [file exists $thisName]} \
          {
           readROI -roi 666666 -ndim 2 -in $thisName

           set roiInfo [getROIInfo -roi 666666]
           set rdata   [lindex $roiInfo 0]
           set fdata   [lindex $roiInfo 1]
           set thresh  [getParm $fdata FDTAU NULL_DIM]
           set thresh  [expr (1-0.68)*[vMax $rdata [expr 361*361]]]

           if {$thresh == 0.0} {set thresh 3.0}
 
           drawROI -roi 666666 -cell $cellID($resID) -bg None -fg None \
                   -pHue1 0.2 -pHue2 0.2 -hi $thresh -plev 1
          }

       if {$verbose} \
          {
           set fmt "%s %3d Range: %d %d ROI: %3d Cell: %3d Page: %2d"
           puts stderr [format $fmt $resName $resID $minRes $maxRes $roiID  $cellID($resID) $nPage]
          }

       if {$lastPlotFlag} \
          {
           drawAllTrajectories $thisResList
           set thisResList ""
           incr nPage
          }
      }

   drawAllTrajectories $thisResList
}

#
# Return a percentage of surface max height.

proc getSurfPct { thisID pct } \
{
   set maxVal [getROIStat $thisID vMax]
   set thisHi [expr $pct*$maxVal/100.0]

   return $thisHi
}

#
# Extract and draw phi/psi trajectories for each segment:

proc drawMFRTrajectories { mfrTab thisColorList tabLoc tabCount resIDList } \
{
   global dotFlag lineFlag boxFlag avgColor minRank maxRank skip
   global chainName minRes maxRes pdbDir fragDir foldFlag foldPhi foldPsi refSrc
   global cisInfo resNameInfo classInfo
   global fragInfo miniPlotInfo chiFlag
   global cellID masterInfo verbose

   set entryCount [gdbGetInfo $mfrTab -parms entryCount]
   set eList      [gdbGetInfo $mfrTab -parms entries]
   set colorCount [llength $thisColorList]

   set x0List    ""
   set y0List    ""
   set colors    ""

   set segIndex  0
   set nn        0
   set k         0
 
   set thisMinRes [getMin $resIDList]
   set thisMaxRes [getMax $resIDList]
 
   if {![info exists masterInfo($minRes)]} \
      {
       for {set r0 $minRes} {$r0 <= $maxRes} {incr r0} \
          {
           set masterInfo($r0) ""
          }
      }
 
   foreach r0 $resIDList \
      {
       set fragInfo($r0,phi)   ""
       set fragInfo($r0,psi)   ""
      }

   set str [dynGetPDBSeqStr $refSrc $chainName]

   if {$verbose} \
      {
       puts "$str"

       puts -nonewline "VARS   PDB_NAME INDEX RANK"
       puts -nonewline " RESID RESNAME PDB_RESID PDB_RESNAME PHI PSI OMEGA"

       if {$chiFlag} {puts -nonewline " CHI1 CHI1 CHI3 CHI4"}

       puts -nonewline "FORMAT %12s %4d %2d %3d %s %3d %s %8.2f %8.2f %8.2f"

       if {$chiFlag} {puts -nonewline " %8.2f %8.2f %8.2f %8.2f"}

       puts ""
      }

   drawSettings -reset

   foreach entry $eList \
      {
       set segName  [gdbGet $entry PDB_NAME]
       set rank     [gdbGet $entry RANK]

       if {$rank < $minRank} continue
       if {$rank > $maxRank} continue

       set r1 [gdbGet $entry D_RES1]
       set rN [gdbGet $entry D_RESN]

       if {$rN < $thisMinRes} continue
       if {$r1 > $thisMaxRes} continue

       if {![strcasecmp $fragDir None]} \
          {
           set fragName None

           set a1 [gdbGet $entry RES1]
           set aN [gdbGet $entry RESN]
          } \
       else \
          {
           set fragName [addExt $fragDir/[gdbGet $entry FRAG_NAME] pdb]

           set a1 [gdbGet $entry D_RES1]
           set aN [gdbGet $entry D_RESN]

           if {![file exists $fragName]} \
              {
               if {$verbose} {puts stderr "Skipping $fragName"}
               continue
              }
          }

       set aa1 [expr $a1 - $skip]
       set aaN [expr $aN + $skip]

       if {$rank > $colorCount} \
          {
           set thisColor [lindex $thisColorList end]
          } \
       else \
          {
           set thisColor [lindex $thisColorList [expr $rank - 1]]
          }

       incr k
 
       if {![strcasecmp $segName None]} continue

       set c [string index $segName 0]

       if {![strcmp $c "/"]} \
          {
           set fullName $segName
          } \
       else \
          {
           set fullName $pdbDir/$segName
          }
 
       if {![file exists $fullName]} \
          {
           if {$verbose} {puts stderr "Skipping $segName"}
           continue
          }
 
       if {$verbose} \
          {
           puts -nonewline stderr "$k of $entryCount, Rank $rank, $thisColor "
           puts -nonewline stderr " $segName $a1 to $aN: $r1 to $rN $fragName"
           puts stderr ""
          }

       if {$chiFlag} \
          {
           set atomList "*"
          } \
       else \
          {
           set atomList "C CA N"
          }

       set rootName [file root $segName]
       set segSrc   [dynRead -pdb -in $fullName -r1 $aa1 -rN $aaN -atomName $atomList]

       set fragSrc 0

       if {[file exists $fragName]} \
          {
           set fragSrc [dynRead -pdb -in $fragName -r1 $r1 -rN $rN -atomName $atomList]
          }

       set r0     $r1

       set a1     [expr $a1 + $skip]
       set aN     [expr $aN - $skip]
       set r0     [expr $r0 + $skip]
   
       set xList  ""
       set yList  ""
       set cList  ""
       set n      0

       incr segIndex

       for {set a0 $a1} {$a0 <= $aN} {incr a0} \
          {
           set refResName [getResName1 $resNameInfo($r0)]
           set segResName [getResName1 [dynGetInfo -src $segSrc -res $a0 -resName]]

           if {$fragSrc} \
              {
               set aInfo   [dynGetPhiPsi -res $r0 -src $fragSrc]
               set omega   [dynGetOmega  -res $r0 -src $fragSrc]
               set chiList [dynGetChi    -res $r0 -src $fragSrc -all]
              } \
           else \
              {
               set aInfo   [dynGetPhiPsi -res $a0 -src $segSrc]
               set omega   [dynGetOmega  -res $a0 -src $segSrc]
               set chiList [dynGetChi    -res $a0 -src $segSrc -all]
              }

           set phi     [lindex $aInfo 0]
           set psi     [lindex $aInfo 1]
           set c       ""

           append chiList " 9999.00 9999.00 9999.00 9999.00"

           if {$chiFlag} \
              {
               for {set ic 0} {$ic < 4} {incr ic} \
                  { 
                   set chi  [lindex $chiList $ic]
                   append c [format " % 8.2f" $chi]
                  }
              }

           set fmt   "%12s %3d %2d  %3d %s  %3d %s   % 8.2f % 8.2f % 8.2f %s"
   
           if {$verbose} \
              {
               puts [format $fmt $rootName $segIndex $rank  \
                            $r0 $refResName $a0 $segResName \
                            $phi $psi $omega $c]
              }

           if {$phi < 999 && $psi < 999} \
              {
               if {$foldFlag && $phi < $foldPhi && $psi < $foldPsi} \
                  {
                   set psi [expr $psi + 360.0]
                  }

               lappend masterInfo($r0) $phi,$psi

               lappend fragInfo($r0,phi)   $phi
               lappend fragInfo($r0,psi)   $psi

               set x0     [expr 100.0*(180.0 + $phi)/360.0]
               set y0     [expr 100.0*(1.0 - (180.0 + $psi)/360.0)]

               if {[lsearch -exact $resIDList $r0] >= 0} \
                  {
                   set thisX  [spec2pix -cell $cellID($r0) -x $x0%]
                   set thisY  [spec2pix -cell $cellID($r0) -y $y0%]

                   lappend x0List   $thisX
                   lappend y0List   $thisY
                   lappend colors   $thisColor
               
                   lappend xList $thisX
                   lappend yList $thisY
                   lappend cList $cellID($r0)

                   incr    nn
                   incr    n
                  }
              }

           incr r0
          }

       if {$lineFlag} \
          {
           for {set i 0} {$i < [expr $n - 1]} {incr i} \
              {
               set c1  [lindex $cList $i]
               set c2  [lindex $cList [expr $i + 1]]
               set q1  [lindex $miniPlotInfo($c1) end]
               set q2  [lindex $miniPlotInfo($c2) end]

               if {$q1 != $q2} continue

               set lx1 [lindex $xList $i]
               set ly1 [lindex $yList $i]
       
               set lx2 [lindex $xList [expr $i + 1]]
               set ly2 [lindex $yList [expr $i + 1]]
       
               drawLineLW $lx1 $lx2 $ly1 $ly2 $thisColor 1 
              }
          }

       if {$dotFlag} \
          {
           for {set i 0} {$i < $n} {incr i} \
              {
               set x0 [lindex $xList $i]
               set y0 [lindex $yList $i]

               drawLineLW $x0 $x0 $y0 $y0 $thisColor 1 
              }
          }

       dynFree $segSrc $fragSrc
      }

   if {$boxFlag} \
      {
       for {set i 0} {$i < $nn} {incr i} \
          {
           set x0       [lindex $x0List $i]
           set y0       [lindex $y0List $i]
           set thisolor [lindex $colors $i]

           set x1 [expr $x0 - 10]
           set xn [expr $x0 + 10]
           set y1 [expr $y0 - 10]
           set yn [expr $y0 + 10]

           drawRec -x $x1 $xn -y $y1 $yn -fg black
          }
      }

   foreach r0 $resIDList \
      {
       if {![llength $fragInfo($r0,phi)]} continue

       set phiAvg  [getAvgAngle    $fragInfo($r0,phi)]
       set psiAvg  [getAvgAngle    $fragInfo($r0,psi)]
       set phiDev  [getStdDevAngle $fragInfo($r0,phi)]
       set psiDev  [getStdDevAngle $fragInfo($r0,psi)]

       set phi    [expr $phiAvg - $phiDev]
       set psi    [expr $psiAvg - $psiDev]

       set x1     [expr 100.0*(180.0 + $phi)/360.0]
       set y1     [expr 100.0*(1.0 - (180.0 + $psi)/360.0)]

       set phi    [expr $phiAvg + $phiDev]
       set psi    [expr $psiAvg + $psiDev]

       set xn     [expr 100.0*(180.0 + $phi)/360.0]
       set yn     [expr 100.0*(1.0 - (180.0 + $psi)/360.0)]

       set thisX1 [spec2pix -cell $cellID($r0) -x $x1%]
       set thisY1 [spec2pix -cell $cellID($r0) -y $y1%]
       set thisXN [spec2pix -cell $cellID($r0) -x $xn%]
       set thisYN [spec2pix -cell $cellID($r0) -y $yn%]
      }
}

proc storePDBPhiPsi { pdbSrc resIDList pdbLoc } \
{
   global phiInfo psiInfo chainName

   foreach r0 $resIDList \
      {
       if {![strcasecmp $chainName None]} \
          {
           set aInfo   [dynGetPhiPsi -res $r0 -src $pdbSrc]
          } \
       else \
          {
           set aInfo   [dynGetPhiPsi -res $r0 -src $pdbSrc -chain $chainName]
          }

       set phi [lindex $aInfo 0]
       set psi [lindex $aInfo 1]

       if {$phi < 999} {lappend phiInfo($r0) $phi}
       if {$psi < 999} {lappend psiInfo($r0) $psi}
      }
}

proc drawPDBTrajectory { pdbSrc thisColor resIDList pdbLoc } \
{
   global minRes maxRes foldFlag foldPhi foldPsi usefulChi resNameInfo cisInfo
   global miniPlotInfo cellID cirInfo chainName phiInfo psiInfo

   drawSettings -reset

   set xList ""
   set yList ""
   set cList ""
   set n     0

   foreach r0 $resIDList \
      {
       if {![strcasecmp $chainName None]} \
          {
           set aInfo   [dynGetPhiPsi -res $r0 -src $pdbSrc]
           set omega   [dynGetOmega  -res $r0 -src $pdbSrc]
           set chiList [dynGetChi    -res $r0 -src $pdbSrc -all]
          } \
       else \
          {
           set aInfo   [dynGetPhiPsi -res $r0 -src $pdbSrc -chain $chainName]
           set omega   [dynGetOmega  -res $r0 -src $pdbSrc -chain $chainName]
           set chiList [dynGetChi    -res $r0 -src $pdbSrc -chain $chainName -all]
          }

       set phi     [lindex $aInfo 0]
       set psi     [lindex $aInfo 1]
       set nChi    [llength $chiList]
       set resName $resNameInfo($r0)     

       if {$phi < 999 && $psi < 999} \
          {
           if {$pdbLoc == 0} \
              {
               set iphi [expr round($phi)]
               set ipsi [expr round($psi)]

               set dphi ""
               set dpsi ""

               if {[info exists phiInfo($r0)]} \
                  {
                   if {[llength $phiInfo($r0)] > 1} \
                      {
                       set dphi [format "+/-%.0f" [expr round([getStdDevAngle $phiInfo($r0)])]]
                      }
                  }

               if {[info exists psiInfo($r0)]} \
                  {
                   if {[llength $psiInfo($r0)] > 1} \
                      {
                       set dpsi [format "+/-%.0f" [expr round([getStdDevAngle $psiInfo($r0)])]]
                      }
                  }

               drawText -cell $cellID($r0) -x  0% -y 109% -fg black -opt symbol -text  "f,y"
               drawText -cell $cellID($r0) -x 12% -y 110% -fg black -text [format " %.0f%s,%.0f%s" $iphi $dphi $ipsi $dpsi] 
              }

            
           if {$foldFlag && $phi < $foldPhi && $psi < $foldPsi} \
              {
               set psi [expr $psi + 360.0]
              }

           set x0     [expr 100.0*(180.0 + $phi)/360.0]
           set y0     [expr 100.0*(1.0 - (180.0 + $psi )/360.0)]

           set thisX  [spec2pix -cell $cellID($r0) -x $x0%]
           set thisY  [spec2pix -cell $cellID($r0) -y $y0%]

           lappend xList [expr int($thisX)]
           lappend yList [expr int($thisY)]
           lappend cList $cellID($r0)

           incr    n
          }

       set cirr    8
       set cirx1   3
       set cirxn   [expr $cirx1 + 2.0*$cirr] 
       set ciry1   -3
       set ciryn   [expr $ciry1 - 2.0*$cirr] 

       if {$omega < 999} \
          {
           if {![info exists cirInfo($r0,omega)]} \
              {
               set cirInfo($r0,omega) "$cirx1 $cirxn $ciry1 $ciryn"

               set cirxm [expr 0.5*($cirx1 + $cirxn)]
               set cirym [expr $ciryn - 3.5]

               drawCircle -cell $cellID($r0) -x ${cirx1}% ${cirxn}% -y ${ciry1}% ${ciryn}% -fg gray 
               drawText   -cell $cellID($r0) -x ${cirxm}% -y ${cirym}% -fg #ccbb11 -center -opt symbol -text W
              }

           set cirx0 [expr 0.5*($cirx1 + $cirxn)]
           set ciry0 [expr 0.5*($ciry1 + $ciryn)]

           set thisX [expr 0.5*($cirx1 + $cirxn) + $cirr*cos(3.14159265*$omega/180.0)]
           set thisY [expr 0.5*($ciry1 + $ciryn) - $cirr*sin(3.14159265*$omega/180.0)]

           if {abs($omega) <= 70.0} \
              {
               lappend cisInfo($r0) $pdbLoc
               set chiColor red
              } \
           else \
              {
               set chiColor forestGreen 
              }

           drawLine -cell $cellID($r0) -x ${cirx0}% ${thisX}% -y ${ciry0}% ${thisY}% -fg $chiColor 
          }

       set cirx1 [expr $cirxn + 5.0]
       set cirxn [expr $cirx1 + 2.0*$cirr] 

       if {[info exists usefulChi($resName)]} \
          {
           set nU $usefulChi($resName)
          } \
       else \
          {
           set nU $nChi
          }

       if {$nU > $nChi} {set nU $nChi}

       for {set i 1} {$i <= $nU} {incr i} \
           {
            set thisChi [lindex $chiList [expr $i - 1]]

            if {$thisChi < 999} \
               {
                if {![info exists cirInfo($r0,chi$i)]} \
                   {
                    set cirInfo($r0,chi$i) "$cirx1 $cirxn $ciry1 $ciryn"

                    set cirxm [expr 0.5*($cirx1 + $cirxn)]
                    set cirym [expr $ciryn - 5]

                    drawCircle -cell $cellID($r0) -x ${cirx1}% ${cirxn}% -y ${ciry1}% ${ciryn}% -fg gray

                    set cirym [expr $ciryn - 6]
                    drawText   -cell $cellID($r0) -x ${cirxm}% -y ${cirym}% -fg #ccbb11 -right -opt symbol -text c 

                    set cirym [expr $ciryn - 3]
                    drawText   -cell $cellID($r0) -x ${cirxm}% -y ${cirym}% -fg #ccbb11 -left  -opt symbol -text $i
                   }

                set cirx0 [expr 0.5*($cirx1 + $cirxn)]
                set ciry0 [expr 0.5*($ciry1 + $ciryn)]

                set thisX [expr 0.5*($cirx1 + $cirxn) + $cirr*cos(3.14159265*$thisChi/180.0)]
                set thisY [expr 0.5*($ciry1 + $ciryn) - $cirr*sin(3.14159265*$thisChi/180.0)]

                drawLine -cell $cellID($r0) -x ${cirx0}% ${thisX}% -y ${ciry0}% ${thisY}% -fg blue 
               }

            set cirx1 [expr $cirxn + 2.0]
            set cirxn [expr $cirx1 + 2.0*$cirr] 
           }
      }

   for {set i 0} {$i < [expr $n - 1]} {incr i} \
      {
       set c1  [lindex $cList $i]
       set c2  [lindex $cList [expr $i + 1]]
       set q1  [lindex $miniPlotInfo($c1) end]
       set q2  [lindex $miniPlotInfo($c2) end]

       if {$q1 != $q2} continue

       set lx1 [lindex $xList $i]
       set ly1 [lindex $yList $i]

       set lx2 [lindex $xList [expr $i + 1]]
       set ly2 [lindex $yList [expr $i + 1]]

       drawLineLW $lx1 $lx2 $ly1 $ly2 $thisColor 1 
      }

   for {set i 0} {$i < $n} {incr i} \
      {
       set x0 [lindex $xList $i]
       set y0 [lindex $yList $i]
   
       set x1 [expr $x0 - 10]
       set xn [expr $x0 + 10]
       set y1 [expr $y0 - 10]
       set yn [expr $y0 + 10]
   
       drawRec -x $x1 $xn -y $y1 $yn -fg black
      }
}

proc addRank { mfrTab } \
{
    global sortFlag

    set vList    [gdbGetInfo $mfrTab -parms varNames]
    set needID   0
    set needRank 0

    if {[lsearch -exact $vList RANK] < 0} \
       {
        set rankVar  [gdbCreate variable -parent $mfrTab -name RANK  -varType int -varFmt %2d]
        set needRank 1
       }

    if {[lsearch -exact $vList INDEX] < 0} \
       {
        set idVar  [gdbCreate variable -parent $mfrTab -name INDEX  -varType int -varFmt %4d]
        set needID 1
       }

    set entry     [gdbFirst $mfrTab]
    set prevResID 666
    set rank      0
    set id        0

    if {$needID || $needRank} \
       {
        while {$entry} \
           {
            set resID [gdbGet $entry D_RES1]
    
            if {$resID != $prevResID} \
               {
                set prevResID $resID
                set rank      0
               }

            incr rank
            incr id

            if {$needID}   {gdbSet $entry INDEX $id}
            if {$needRank} {gdbSet $entry RANK  $rank}
    
            set entry [gdbNext $entry]
           } 
       } 

    if {$sortFlag} {gdbSort <$rankVar}
}

proc drawAllTrajectories { resIDList } \
{
   global mfrNameList mfrColorList phiPsiNameList phiPsiColorList pdbNameList pdbColorList chainName
   global phiInfo psiInfo resNameInfo classInfo cisInfo cellID

   if {![llength $resIDList]} return

   set tabCount  [llength $mfrNameList]
   set nMax      [expr [llength $mfrColorList] - 1]

   plotWrite %% MFR Trajectories
   plotWrite 0.1 setlinewidth

   for {set i 0} {$i < $tabCount} {incr i} \
      {
       set mfrName [lindex $mfrNameList $i]
       set j $i

       if {[string first "color=" $mfrName] >= 0} \
          {
           set mfrColorList [lindex [split $mfrName "="] 1]
           set nMax 1

           continue
          }

       if {[string first "pdbDir=" $mfrName] >= 0} \
          {
           set pdbDir [lindex [split $mfrName "="] 1]
           continue
          }

       if {[string first "fragDir=" $mfrName] >= 0} \
          {
           set fragDir [lindex [split $mfrName "="] 1]
           continue
          }

       if {$j > $nMax} {set j $nMax}

       if {![file exists $mfrName]} continue

       set mfrTab        [gdbRead table -in $mfrName]
       set thisColorList [split [lindex $mfrColorList $j] ,]
       set thisColorList $mfrColorList

       addRank $mfrTab 

       drawMFRTrajectories $mfrTab $thisColorList $i $tabCount $resIDList

       gdbDestroy $mfrTab
      }

   set aCount [llength $phiPsiNameList]
   set nMax   [expr [llength $phiPsiColorList] - 1]

   plotWrite %% Phi/Psi Trajectories
   plotWrite 0.1 setlinewidth

   for {set i 0} {$i < $aCount} {incr i} \
      {
       set aName [lindex $phiPsiNameList $i]
       set j $i

       if {[string first "color=" $aName] >= 0} \
          {
           set phiPsiColorList [lindex [split $aName "="] 1]
           set nMax 1

           continue
          }

       if {$j > $nMax} {set j $nMax}

       if {![file exists $aName]} continue

       set aTab      [gdbRead table -in $aName]
       set thisColor [lindex $phiPsiColorList $j]

       drawPhiPsiTrajectory $aTab $thisColor $resIDList

       gdbFree $aTab
      }

   set pdbCount  [llength $pdbNameList]
   set nMax      [expr [llength $pdbColorList] - 1]

   plotWrite %% PDB Trajectories
   plotWrite 10 setlinewidth

   foreach thisResID $resIDList \
      {
       set phiInfo($thisResID) ""
       set psiInfo($thisResID) ""
      }

   if {$pdbCount > 1} \
      {
       for {set i 0} {$i < $pdbCount} {incr i} \
          {
           set pdbName [lindex $pdbNameList $i]
           if {![file exists $pdbName]} continue

           set pdbSrc [dynRead -pdb -in $pdbName]

           storePDBPhiPsi $pdbSrc $resIDList $i

           dynFree $pdbSrc
          }
      }

   for {set i 0} {$i < $pdbCount} {incr i} \
      {
       set pdbName [lindex $pdbNameList $i]
       set j $i

       if {[string first "color=" $pdbName] >= 0} \
          {
           set pdbColorList [lindex [split $pdbName "="] 1]
           set nMax 1

           continue
          }

       if {$j > $nMax} {set j $nMax}

       if {![file exists $pdbName]} continue

       set pdbSrc    [dynRead -pdb -in $pdbName]
       set thisColor [lindex $pdbColorList $j]

       drawPDBTrajectory $pdbSrc $thisColor $resIDList $i

       dynFree $pdbSrc

       set str "$pdbName"
      }

   foreach resID $resIDList \
      {
       if {![strcasecmp $resNameInfo($resID) None]} continue

       set nS [string length $resID]
       set resColor black

       if {$nS == 1} \
          {
           set xPct 85%
          } \
       elseif {$nS == 2} \
          {
           set xPct 77%
          } \
       elseif {$nS == 3} \
          {
           set xPct 69%
          } \
       else \
          {
           set xPct 50%
          }

       if {![info exists cisInfo($resID)]} {set cisInfo($resID) ""}
       if {[llength $cisInfo($resID)]} {set resColor red}

       if {![strcasecmp $chainName None]} \
          {
           set resStr [getResName1 $resNameInfo($resID)]$resID
          } \
       else \
          {
           set resStr ${chain}:[getResName1 $resNameInfo($resID)]$resID
          }

       drawText -big -cell $cellID($resID) -fg $resColor -x 97% -right -y 97% -text $resStr 

       if {[info exists classInfo($resID)]} \
          {
           drawText -cell $cellID($resID) -fg black -x 0% -y -36% -text $classInfo($resID)
          }
      }
}

#
#-----------------------------------------------------------------------------------------

initResNames

initWin
initRef
plotRama
plotClose

exit
