#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "X/Y Plot of Columns in an NMRPipe-Format Table:"
    puts " -in     inName        NMRPipe-Format Input Table." 
    puts " -xSize  xSize         Graph X-Axis Dimension Size, Pixels."
    puts " -ySize  ySize         Graph Y-Axis Dimension Size, Pixels."
    puts " -x      xVarList"
    puts " -y      yVarList"
    puts " -z      labelVar"
    puts " -zc     labelColorVar"
    puts " -xUndef xUndefVal"
    puts " -yUndef yUndefVal"
    puts " -select cond"
    puts " -verb                 Verbose Mode ON (Default)."
    puts " -noverb               Verbose Mode OFF."
    puts "Axis Parameters:"
    puts " -xMin      xMinList"
    puts " -xMax      xMaxList"
    puts " -yMin      yMinList"
    puts " -yMax      yMaxList"
    puts " -xMinExtra xMinPct"
    puts " -xMaxExtra xMaxPct"
    puts " -yMinExtra yMinPct"
    puts " -yMaxExtra yMaxPct"
    puts " -hN        histBins"
    puts " -xStep     xAxisStep"
    puts " -yStep     yAxisStep"
    puts " -hStep     histAxisStep"
    puts " -xrev"
    puts " -yrev"
    puts "Drawing Parameters:"
    puts " -lw        lineWidthList"
    puts " -pw        pointWidthList"
    puts " -lColor    lineColorList"
    puts " -pColor    pointColorList"
    puts " -zColor    labelColor"
    puts " -zFmt      zFormat"
    puts " -bg        bgColor"
    puts "PDB Options:"
    puts " -line   refLineCoordList  (x1 y1 yn yn color linewidth)"
    puts " -pdb                      Input is PDB Data (Default for .pdb)."
    puts " -nopdb                    Input is NMRPipe-Format Table."
    puts " -hetatm                   Interpret HETATMs as ATOMs."
    exit 1 
   }

setFonts

set inName     test.tab
set xVarList   i
set xMinList   None
set xMaxList   None
set pdbMode    nopdb
set hetMode    nohetatm

set yVarList   i 
set yMinList   None
set yMaxList   None

set xMinExtra  0.0
set xMaxExtra  0.0
set yMinExtra  0.0
set yMaxExtra  0.0

set xUndefVal  None
set yUndefVal  None
set refLine    None

set xRevFlag   0
set yRevFlag   0

set xStep      None
set yStep      None
set hStep      None

set xSize      Default 
set ySize      Default 

set hxSize     Default 
set hySize     Default 
set histN      16

set zVar       None
set zColorVar  None
set cond       "1"
set verbose    1

set lineColorList   "cyan green red darkOrchid yellow"
set pointColorList  white
set bgColor         black
set zColor          yellow 
set zFmt            %s

set lineWidthList   0
set pointWidthList  2
set scrollFlag      0

getArgD $ARGV -in     inName 
getArgD $ARGV -xSize  xSize

getArgD $ARGV -hxSize hxSize
getArgD $ARGV -hySize hySize
getArgD $ARGV -ySize  ySize
getArgD $ARGV -xUndef xUndefVal
getArgD $ARGV -yUndef yUndefVal
getArgD $ARGV -hN     histN

getArgD $ARGV -hStep  hStep
getArgD $ARGV -xStep  xStep
getArgD $ARGV -yStep  yStep

getArgD $ARGV -bg     bgColor
getArgD $ARGV -zColor zColor

if {![strcasecmp [file extension $inName] .pdb]} {set pdbMode pdb}

if {[flagLoc $ARGV -verb]}      {set verbose 1}
if {[flagLoc $ARGV -noverb]}    {set verbose 0}

if {[flagLoc $ARGV -xrev]}      {set xRevFlag 1}
if {[flagLoc $ARGV -yrev]}      {set yRevFlag 1}

if {[flagLoc $ARGV -pdb]}       {set pdbMode pdb}
if {[flagLoc $ARGV -nopdb]}     {set pdbMode nopdb}

if {[flagLoc $ARGV -hetatm]}    {set hetMode hetatm}
if {[flagLoc $ARGV -nohetatm]}  {set hetMode nohetatm}

if {![strcasecmp $xSize  Default]} {set xSize  900}
if {![strcasecmp $ySize  Default]} {set ySize  400}

if {![strcasecmp $hxSize Default]} {set hxSize 500}
if {![strcasecmp $hySize Default]} {set hySize 400}

if {![strcasecmp $zVar None]} \
   {
    set ppx 8
    set ppy 8 
   } \
else \
   {
    set ppx 36
    set ppy 20
   }

getArgD $ARGV -ppx ppx
getArgD $ARGV -ppx ppy

getListArgD $ARGV -x    xVarList
getListArgD $ARGV -xMin xMinList
getListArgD $ARGV -xMax xMaxList

getArgD $ARGV -xMinExtra xMinExtra
getArgD $ARGV -xMaxExtra xMaxExtra

getListArgD $ARGV -y    yVarList
getListArgD $ARGV -yMin yMinList
getListArgD $ARGV -yMax yMaxList

getArgD $ARGV -yMinExtra yMinExtra
getArgD $ARGV -yMaxExtra yMaxExtra

getListArgD $ARGV -z    zVar
getListArgD $ARGV -zc   zColorVar
getListArgD $ARGV -zFmt zFmt

getListArgD $ARGV -lw   lineWidthList
getListArgD $ARGV -pw   pointWidthList

getListArgD $ARGV -lColor lineColorList 
getListArgD $ARGV -pColor pointColorList

getListArgD $ARGV -select cond

if {[flagLoc $ARGV -scroll]} {set scrollFlag 1}

#
#----------------------------------------------------------------------

proc lindexN { iList n } \
{
   set nn [llength $iList]

   if {$n < 0 || $n >= $nn} \
      {
       return [lindex $iList end]
      }

   return [lindex $iList $n]
}

proc barFmtX { wb v } \
{
   global hxScale
   return [format %.1e [expr $hxScale*$v]]
}

#
#----------------------------------------------------------------------

if {$histN > 0} {toplevel .h}

frame .top 
frame .bot

set sWide   [winfo screenwidth  .]
set sHeight [winfo screenheight .]

setFonts

if {$scrollFlag} \
   {
    set aWide [expr int($sWide/2)]
    set aHigh [expr int($sHeight/2)]

    wm geom .  [format "%dx%d+10+16" $aWide $aHigh]
    if {$histN > 0} {wm geom .h +30-100}
   } \
else \
   {
    set aWide [expr int($sWide/2)]
    set aHigh [expr int($sHeight/2)]

    wm geom .  [format "%dx%d+16+40" $aWide $aHigh]
    if {$histN > 0} {wm geom .h +30-100}
   }
 
if {$scrollFlag} \
   {
    scrollbar .sx -orient horiz -command ".c xview"
    scrollbar .sy -orient vert  -command ".c yview"
   }

canvas .c   -width $xSize -height $ySize

graph  .c.g -plotbackground $bgColor -width $xSize -height $ySize \
            -plotpadx $ppx -plotpady $ppy -title $inName 

button .q   -text Quit -command exit

pack .c.g -in .c -side top -expand 1 -fill both

.c.g axis configure x -tickfont $axisfont -ticklength 0.05i -descending $xRevFlag
.c.g axis configure y -tickfont $axisfont -ticklength 0.05i -descending $yRevFlag

if {$histN > 0} \
   {
    barchart .h.b -plotbackground $bgColor -width $hxSize -height $hySize \
                  -plotpadx $ppx -plotpady $ppy -title $inName -barmode aligned

    button .h.q -text Quit -command exit

    .h.b axis configure x -tickfont $axisfont -ticklength 0.05i -command barFmtX
    .h.b axis configure y -tickfont $axisfont -ticklength 0.05i

    if {[strcasecmp $hStep None]} {.h.b axis configure x -step $hStep} 

    pack .h.b -in .h -side top -expand 1 -fill both
    pack .h.q -in .h -side top -fill x 
   }

if {[strcasecmp $xStep None]} {.c.g axis configure x -step $xStep} 
if {[strcasecmp $yStep None]} {.c.g axis configure y -step $yStep} 

pack .top -in . -side top -expand 1 -fill both
pack .bot -in . -side top -fill x

if {$scrollFlag} \
   {
    .c configure -scrollregion "0 0 $xSize $ySize"
    .c configure -xscrollcommand ".sx set"
    .c configure -yscrollcommand ".sy set"

    pack .c  -in .top -side left
    pack .sy -in .top -side left

    pack .sx .q -in .bot -side top
   } \
else \
   {
    pack .c -in .top -side top -expand 1 -fill both
    pack .q -in .bot -side top -fill x 
   }

if {![file exists $inName]} \
   {
    puts stderr "Error finding input table $inName"
    exit 1
   }

set nx [llength $xVarList]
set ny [llength $yVarList]

set cond [string trim $cond "{}"]

set t     [gdbRead table -in $inName -$pdbMode -$hetMode]
set eList [gdbSelect -from $t -cond $cond]
set vList [gdbGetInfo $t -parms varNames]

set tStr  "$inName, [llength $eList] entries ($xVarList) vs ($yVarList) $zVar"

if {[strcmp $cond "1"]} {append tStr " $cond"}

wm title .  $tStr

.c.g axis configure x -title $xVarList
.c.g axis configure y -title $yVarList

if {$histN > 0} \
   {
    set hStr  "$inName, [llength $eList] entries $yVarList"
    wm title .h $hStr

    .h.b axis configure x -title $yVarList
    .h.b axis configure y -title Count 
   }

foreach v "$xVarList $yVarList" \
   {
    if {[lsearch -exact $vList $v] < 0} \
       {
        puts "Error: No Variable $v in Table $inName"
        exit 1
       }

    set vID   [gdbFindByName $t $v]
    set vType [gdbGetInfo $vID -parms varType]

    set mapInfo($v,mapList) ""
    set mapInfo($v,mapFlag) 0

    if {[strcasecmp $vType text]} continue 

    foreach entry $eList \
       {
        set val [gdbGet $entry $v]

        if {[lsearch -exact $mapInfo($v,mapList) $val] < 0} \
           {
            lappend mapInfo($v,mapList) $val
           }
       }

    set mapInfo($v,mapList) [lsort $mapInfo($v,mapList)]
    set mapInfo($v,mapFlag) 1
   }

foreach r $zVar {lappend zFmt "%s"}

set n 0

for {set n 0} {$n < $ny} {incr n} \
   {
    set xVar   [lindexN $xVarList $n]
    set xMin   [lindexN $xMinList $n]
    set xMax   [lindexN $xMaxList $n]

    set yVar   [lindexN $yVarList $n]
    set yMin   [lindexN $yMinList $n]
    set yMax   [lindexN $yMaxList $n]

    set lColor [lindexN $lineColorList  $n]
    set pColor [lindexN $pointColorList $n]

    set lineWidth  [lindexN $lineWidthList  $n]
    set pointWidth [lindexN $pointWidthList $n]

    set xList  ""
    set yList  ""
    set hxList ""
    set hyList ""
    set zList  ""
    set zcList ""
    set hxScale 1.0
    set hyScale 1.0


    set thisXMin [getTabMin $t $xVar]
    set thisXMax [getTabMax $t $xVar]

    set thisYMin [getTabMin $t $yVar]
    set thisYMax [getTabMax $t $yVar]

    set xRange [expr $thisXMax - $thisXMin]
    set yRange [expr $thisYMax - $thisYMin]

    if {![strcasecmp $xMin All]} {set xMin $thisXMin}
    if {![strcasecmp $xMax All]} {set xMax $thisXMax}
    if {![strcasecmp $yMin All]} {set yMin $thisYMin}
    if {![strcasecmp $yMax All]} {set yMax $thisYMax}

    if {$xMinExtra != 0.0} \
       {
        if {![strcasecmp $xMin None]} {set xMin $thisXMin}
        set xMin [expr $xMin - abs($xMinExtra*$xRange/100.0)]
       }

    if {$xMaxExtra != 0.0} \
       {
        if {![strcasecmp $xMax None]} {set xMax $thisXMax}
        set xMax [expr $xMax + abs($xMaxExtra*$xRange/100.0)]
       }

    if {$yMinExtra != 0.0} \
       {
        if {![strcasecmp $yMin None]} {set yMin $thisYMin}
        set yMin [expr $yMin - abs($yMinExtra*$yRange/100.0)]
       }

    if {$yMaxExtra != 0.0} \
       {
        if {![strcasecmp $yMax None]} {set yMax $thisYMax}
        set yMax [expr $yMax + abs($yMaxExtra*$yRange/100.0)]
       }
 
    if {[strcasecmp $xMin None]} {.c.g axis configure x -min $xMin}
    if {[strcasecmp $xMax None]} {.c.g axis configure x -max $xMax}
    if {[strcasecmp $yMin None]} {.c.g axis configure y -min $yMin}
    if {[strcasecmp $yMax None]} {.c.g axis configure y -max $yMax}

    set i 0

    foreach entry $eList \
       {
        set uFlag 0
     
        if {![strcmp $xVar "i"]} \
           {
            set xVal $i
           } \
        else \
           {
            set xVal [gdbGet $entry $xVar]
           
            if {$mapInfo($xVar,mapFlag)} \
               {
                set xVal [lsearch -exact $mapInfo($xVar,mapList) $xVal] 
               }

            if {[strcasecmp $xUndefVal None]} \
               {
                if {$xVal > $xUndefVal} {set uFlag 1}
               }
           }

        if {![strcmp $yVar "i"]} \
           {
            set yVal $i
           } \
        else \
           {
            set yVal [gdbGet $entry $yVar]

            if {$mapInfo($yVar,mapFlag)} \
               {
                set yVal [lsearch -exact $mapInfo($yVar,mapList) $yVal] 
               }

            if {[strcasecmp $yUndefVal None]} \
               {
                if {$yVal > $yUndefVal} {set uFlag 1}
               }
           }

        if {![strcasecmp $zVar None]} \
           {
            set zVal None
           } \
        else \
           {
            set zVal  ""
            set loc   0

            foreach z $zVar \
               {
                if {$loc} {append zVal " "}

                append zVal [format [lindex $zFmt $loc] [gdbGet $entry $z]]

                incr loc
               }
           }

        if {![strcasecmp $zColorVar None]} \
           {
            set zcVal $zColor
           } \
        else \
           {
            set zcVal [gdbGet $entry $zColorVar]
           }
   
        if {![llength $xVal]} {set uFlag 1}
        if {![llength $yVal]} {set uFlag 1}
        if {![llength $zVal]} {set uFlag 1}

        if {!$uFlag} \
           {
            lappend xList  $xVal
            lappend yList  $yVal
            lappend zList  $zVal
            lappend zcList $zcVal
           }

        incr i
       }

    if {![llength $xList] || ![llength $yList]} continue

    set gg [string tolower $yVar]

    if {$histN > 0} \
       {
        getHist $yList hxList hyList $histN

        set hMin [getMin $hxList]
        set hMax [getMax $hxList]

        if {$hMax == $hMin} \
           {
            set bw 1
            set hxScale 1.0
           } \
        else \
           {
            set bw [expr ($hMax - $hMin)/double($histN*$ny)]
            set bw [expr 1.0/double($histN*$ny)]
            set hxScale [expr $hMax - $hMin] 
            set hxList  [getMultS $hxList [expr 1.0/$hxScale]]
           }
       }

    catch {.c.g element create $gg -xdata $xList -ydata $yList \
                             -pixels $pointWidth -linewidth $lineWidth \
                             -color $lColor -fill $pColor} 

    if {$histN > 0} \
       {
        catch {.h.b element create $gg -xdata $hxList -ydata $hyList \
                                   -fg $lColor -barwidth $bw}
       }


    set fmt "%-10s %12s %14.6f"

    if {$verbose} \
       { 
        puts [format $fmt $xVar N:       [llength   $xList]]
        puts [format $fmt $xVar Min:     [getMin    $xList]]
        puts [format $fmt $xVar Max:     [getMax    $xList]]
        puts [format $fmt $xVar Sum:     [getSum    $xList]]
        puts [format $fmt $xVar Average: [getAvg    $xList]]
        puts [format $fmt $xVar Median:  [getMedian $xList]]
        puts [format $fmt $xVar R:       [getPearsonR $xList $yList]]
        puts [format $fmt $xVar RMS:     [getRMS    $xList]]
        puts [format $fmt $xVar StdDev:  [getStdDev $xList]]
        puts ""

        puts [format $fmt $yVar N:       [llength   $yList]]
        puts [format $fmt $yVar Min:     [getMin    $yList]]
        puts [format $fmt $yVar Max:     [getMax    $yList]]
        puts [format $fmt $yVar Sum:     [getSum    $yList]]
        puts [format $fmt $yVar Average: [getAvg    $yList]]
        puts [format $fmt $yVar Median:  [getMedian $yList]]
        puts [format $fmt $yVar RMS:     [getRMS    $yList]]
        puts [format $fmt $yVar StdDev:  [getStdDev $yList]]
        puts ""
    
        puts [format "$xVar vs $yVar R: %.4f" [getPearsonR $xList $yList]]
       }

    update

    if {![strcasecmp $zVar None]} continue

    set dd [expr 2 + $lineWidth + $pointWidth]
    set n  [llength $zList]

    for {set i 0} {$i < $n} {incr i} \
       {
        set xVal  [lindex $xList  $i]
        set yVal  [lindex $yList  $i]
        set zVal  [lindex $zList  $i]
        set color [lindex $zcList $i]

        .c.g marker create text -font $stdfont \
           -coord "$xVal $yVal" -anchor s -foreground $color \
           -background "" -padx $dd -pady $dd -text [string trim $zVal "{}"]
       }

    update
   }

#
# User-specified graphics:

set loc [flagLoc $ARGV -line]

if {$loc} \
   {
    set nextLoc [nextFlag $ARGV $loc]
    incr loc

    while {$loc < $nextLoc - 5} \
       {
        set cList [lrange $ARGV $loc [expr $loc + 3]]
        set color [lindex $ARGV [expr $loc + 4]]
        set lw    [lindex $ARGV [expr $loc + 5]]

        .c.g marker create line -coord $cList -outline $color \
             -under no -linewidth $lw
 
        incr loc 6
       }

    update
   }

