#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts stderr "Extract values from Varian procpar file:"
    puts stderr " -in   inName \[procpar\]    Name of procpar-format Input File."
    puts stderr " -parm pList  \[All\]        List of Parameters to Extract, or Keyword All."
    puts stderr " -verb                     Verbose Mode ON (Default)."
    puts stderr " -noverb                   Verbose Mode OFF." 
    exit 0
   }

set inName   procpar
set pList    "*"
set listFlag 0
set verbose  1

getArgD     $ARGV -in   inName
getListArgD $ARGV -parm pList

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {![file exists $inName]} \
   {
    puts stderr "Error Finding parameter input file $inName"
    exit 1
   }

set n [rdProcPar $inName procpar stderr]

if {![info exists procpar(parmList)]} \
   {
    if {$verbose} {puts stderr "Error Finding parameter contents."}
    exit 0
   }

if {![strcasecmp $pList All] || ![strcmp $pList "*"]} \
   {
    set pList $procpar(parmList)
   }

if {[strcasecmp $pList None]} \
   {
    set fmt "%-12s %-10s %s"

    foreach p $pList \
       {
        if {![info exists procpar($p)]} continue

        if {[info exists procpar($p.type)]} \
           {
            set type $procpar($p.type)
           } \
        else \
           {
            set type NONE 
           }

        if {$verbose} \
           {
            puts [format $fmt $p $type $procpar($p)]
           } \
        else \
           {
            puts $procpar($p)
           }
       }
   }

exit 0
