#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set inName   test.tab
set projMode proj
set f        5.0

if {[flagLoc $ARGV -help]} \
   {
    puts "$argv0: Create 2D Shift Table Projections from 3D Peak Table."
    puts " -in inName  \[test.tab\]      Name of 3D Input Table."
    puts " -sd sdc     \[5.0\]           Factor for Computing Tolerance."
    puts " -skyline                      Skyline Projection (Default)."
    puts " -sum                          Summation projection."
    exit 1
   }

getArgD $ARGV -in   inName
getArgD $ARGV -sd   f

if {[flagLoc $ARGV -skyline]} {set projMode proj}
if {[flagLoc $ARGV -sum]}     {set projMode sum}

#
#--------------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input Table $inName"
    exit 1
   }

set pkTab [gdbRead table -in $inName]
set eList [gdbGetInfo $pkTab -parms entries]
set vList [gdbGetInfo $pkTab -parms varNames]

getKeyData $pkTab pkInfo

foreach dim {X Y Z} \
   {
    if {[lsearch -exact $vList ${dim}_AXIS] < 0} \
       {
        puts stderr "Error extracting ${dim}_AXIS data from table $inName"
        exit 1
       } 

    if {![info exists pkInfo(${dim}_AXIS)]} \
       {
        puts stderr "Error extracting ${dim}_AXIS data from table $inName"
        exit 1
       }
   }

foreach dim {X Y Z} \
   {
    set dimInfo($dim) [lindex $pkInfo(${dim}_AXIS) 0]
    set lab($dim)     [lindex $dimInfo($dim) 0]
    set d($dim)       [getEListStat getMedian $eList D$dim]

    if {$d($dim) == 0.0} {set d($dim) 0.333}

    puts stderr [format "%s: %4s +/- %.3f Pts." $dim $lab($dim) $d($dim)]
   }

puts stderr ""

foreach pair {{X Y} {X Z} {Y Z}} \
   {
    catch "unset projInfo"
    catch "unset xInfo"
    catch "unset yInfo"

    set i [lindex $pair 0]
    set j [lindex $pair 1]

    foreach entry $eList \
       {
        set rI [gdbGet $entry ${i}_AXIS]
        set rJ [gdbGet $entry ${j}_AXIS]

        set locI [expr int($rI/($f*$d($i)))]
        set locJ [expr int($rJ/($f*$d($j)))]

        lappend projInfo($locI,$locJ) $entry 
       }

    set outName $lab($i).$lab($j).tab

    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "Error opening output file $outName"
        exit 1
       }

    puts $outUnit "VARS   INDEX X_AXIS Y_AXIS X_PPM Y_PPM XW    YW    HEIGHT"
    puts $outUnit "FORMAT %5d   %8.3f  %8.3f  %8.3f %8.3f %7.3f %7.3f %e"
    puts $outUnit ""

    set id [array startsearch projInfo]
    set k  0

    while {[array anymore projInfo $id]} \
       {
        set locIJ [array nextelement projInfo $id]

        incr k

        set xAxis [getEListAvg $projInfo($locIJ) ${i}_AXIS]
        set yAxis [getEListAvg $projInfo($locIJ) ${j}_AXIS]
        set xPPM  [getEListAvg $projInfo($locIJ) ${i}_PPM]
        set yPPM  [getEListAvg $projInfo($locIJ) ${j}_PPM]
        set xw    [getEListAvg $projInfo($locIJ) ${i}W]
        set yw    [getEListAvg $projInfo($locIJ) ${j}W]

        if {![strcasecmp $projMode skyline]} \
           {
            set hi [getEListStat getSMaxAbs $projInfo($locIJ) HEIGHT]
           } \
        else \
           {
            set hi [getEListAvg $projInfo($locIJ) HEIGHT]
           }

        set fmt " %5d   % 8.3f  % 8.3f  % 8.3f % 8.3f %7.3f %7.3f % .5e"

        puts $outUnit [format $fmt $k $xAxis $yAxis $xPPM $yPPM $xw $yw $hi]
       }

    array donesearch projInfo $id

    puts stderr [format "Output: %-12s Peaks: %4d" $outName $k]

    close $outUnit
   }

exit 0
