#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "$argv0: Create 2D projections from 4D data."
    puts " -in        inName    \[ft/test%04d.ft4\]  Name of 4D Input."
    puts " -outDir    outDir    \[.\]                Output Directory."
    puts " -xyOutName xyOutName \[Auto\]             Name of 2D X,Y-Axis Output."
    puts " -xzOutName xzOutName \[Auto\]             Name of 2D X,Z-Axis Output."
    puts " -xaOutName xaOutName \[Auto\]             Name of 2D X,A-Axis Output."
    puts " -yzOutName yzOutName \[Auto\]             Name of 2D Y,Z-Axis Output."
    puts " -yaOutName yaOutName \[Auto\]             Name of 2D Y,A-Axis Output."
    puts " -zaOutName zaOutName \[Auto\]             Name of 2D Z,A-Axis Output."
    puts " -skyline                                Skyline Projection (Default)."
    puts " -sum                                    Summation projection."
    puts " -abs                                    Use Absolute Value."
    puts " -axis                                   Auto Names Use Axis Names X Y Z A."
    puts " -noaxis                                 Auto Names Use Axis Labels (Default)."
    puts " -verb                                   Verbose Mode ON (Default)."
    puts " -noverb                                 Verbose Mode OFF."
    puts "Spectral Region (Use Prefix -x -y -z -a):"    
    puts " -x1  x1 \[0%\]                            X-Axis Region Start."
    puts " -xn  xn \[100%\]                          X-Axis Region End."
    puts "Notes:"
    puts " 1. Projections are relative to original orientation of FID."
    puts " 2. Auto mode output names are generated from axis labels."
    exit 1
   }

set inName      ft/test%04d.ft4
set outDir      .
set projMode    proj
set absFlag     0
set axisFlag    0
set verbose     1

set xyOutName   Auto
set xzOutName   Auto
set xaOutName   Auto
set yzOutName   Auto
set yaOutName   Auto
set zaOutName   Auto

set ix1         0%
set ixn         100%

set iy1         0%
set iyn         100%

set iz1         0%
set izn         100%

set ia1         0%
set ian         100%

getArgD $ARGV -in     inName
getArgD $ARGV -outDir outDir

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -axis]}   {set axisFlag 1}
if {[flagLoc $ARGV -noaxis]} {set axisFlag 0}

if {$axisFlag} \
   {
    set xyOutName projXY.dat
    set xzOutName projXZ.dat
    set xaOutName projXA.dat
    set yzOutName projYZ.dat
    set yaOutName projYA.dat
    set zaOutName projZA.dat
   }

getArgD $ARGV -xyOutName xyOutName
getArgD $ARGV -xzOutName xzOutName
getArgD $ARGV -xaOutName xaOutName
getArgD $ARGV -yzOutName yzOutName
getArgD $ARGV -yaOutName yaOutName
getArgD $ARGV -zaOutName zaOutName

getArgD $ARGV -x1 ix1
getArgD $ARGV -xn ixn

getArgD $ARGV -y1 iy1
getArgD $ARGV -yn iyn

getArgD $ARGV -z1 iz1
getArgD $ARGV -zn izn

getArgD $ARGV -a1 ia1
getArgD $ARGV -an ian

if {[flagLoc $ARGV -skyline]} {set projMode proj}
if {[flagLoc $ARGV -sum]}     {set projMode sum}
if {[flagLoc $ARGV -abs]}     {set absFlag  1}

if {$absFlag} \
   {
    set absArg abs
   } \
else \
   {
    set absArg noabs
   }

#
#--------------------------------------------------------------------------

set thisName [format $inName 1 1]

if {![file exists $thisName]} \
   {
    puts stderr "Proj4D Error Finding Input $inName"
    exit 1
   }

#
# Extract header:

set error [rdFDATA $inName fdata]

if {$error} \
   {
    puts stderr "Proj4D Error Reading header from $inName"
    exit 1
   }

#
# Prepare output directory if needed:

if {![strcasecmp $outDir "."] || ![strcasecmp $outDir "./"]} \
   {
    set outPrefix ""
   } \
else \
   {
    if {![file exists $outDir]} {catch "file mkdir $outDir"}

    if {![file isdirectory $outDir]} \
       {
        puts stderr "Proj3D Error Finding or Creating Output Directory $outDir"
        exit 1
       }

    set outPrefix $outDir/
   }


#
# Find axis labels:

set tpFlag [getParmI $fdata FDTRANSPOSED 0]

if {$tpFlag} \
   {
    set xName [getParmStr $fdata NDLABEL CUR_YDIM]
    set yName [getParmStr $fdata NDLABEL CUR_XDIM]
    set zName [getParmStr $fdata NDLABEL CUR_ZDIM]
    set aName [getParmStr $fdata NDLABEL CUR_ADIM]

    set it1 $ix1
    set itn $ixn

    set ix1 $iy1
    set ixn $iyn

    set iy1 $it1
    set iyn $itn

    set xAlt  Y
    set yAlt  X
    set zAlt  Z
    set aAlt  A
   } \
else \
   {
    set xName [getParmStr $fdata NDLABEL CUR_XDIM]
    set yName [getParmStr $fdata NDLABEL CUR_YDIM]
    set zName [getParmStr $fdata NDLABEL CUR_ZDIM]
    set aName [getParmStr $fdata NDLABEL CUR_ADIM]

    set xAlt  X
    set yAlt  Y
    set zAlt  Z
    set aAlt  A
   }

#
# Adjust axis labels if any are identical:

if {![strcmp $xName $yName]} {set xName $xAlt}
if {![strcmp $xName $zName]} {set xName $xAlt}
if {![strcmp $xName $aName]} {set xName $xAlt}
if {![strcmp $yName $zName]} {set yName $yAlt}
if {![strcmp $yName $aName]} {set yName $yAlt}
if {![strcmp $zName $aName]} {set zName $zAlt}

#
# Read the projections regions.
# Write the projections.

if {$verbose} {puts "Reading Projection: $xName/$yName"}

readROI -roi 1 -in $inName -ndim 2 -$projMode -$absArg -x $xName $ix1 $ixn -y $yName $iy1 $iyn -z $zName $iz1 $izn -a $aName $ia1 $ian

if {$verbose} {puts "Reading Projection: $xName/$zName"}

readROI -roi 2 -in $inName -ndim 2 -$projMode -$absArg -x $xName $ix1 $ixn -y $zName $iz1 $izn -z $yName $iy1 $iyn -a $aName $ia1 $ian

if {$verbose} {puts "Reading Projection: $xName/$aName"}

readROI -roi 3 -in $inName -ndim 2 -$projMode -$absArg -x $xName $ix1 $ixn -y $aName $ia1 $ian -z $zName $iz1 $izn -a $yName $iy1 $iyn

if {$verbose} {puts "Reading Projection: $yName/$zName"}

readROI -roi 4 -in $inName -ndim 2 -$projMode -$absArg -x $yName $iy1 $iyn -y $zName $iz1 $izn -z $xName $ix1 $ixn -a $aName $ia1 $ian

if {$verbose} {puts "Reading Projection: $yName/$aName"}

readROI -roi 5 -in $inName -ndim 2 -$projMode -$absArg -x $yName $iy1 $iyn -y $aName $ia1 $ian -z $xName $ix1 $ixn -a $zName $iz1 $izn

if {$verbose} {puts "Reading Projection: $zName/$aName"}

readROI -roi 6 -in $inName -ndim 2 -$projMode -$absArg -x $zName $iz1 $izn -y $aName $ia1 $ian -z $xName $ix1 $ixn -a $yName $iy1 $iyn

if {$absFlag} \
   {
    foreach id {1 2 3 4 5 6} \
       {
        set roiInfo [getROIInfo -roi $id]
        set rdata   [lindex $roiInfo 0]
        set sList   [lindex $roiInfo 2]
        set nx      [lindex $sList 0]
        set ny      [lindex $sList 1]
        set n       [expr $nx*$ny]

        vAbs $rdata $n
       }
   }

if {![strcasecmp $xyOutName Auto]} \
   {
    set thisName $outPrefix$xName.$yName.dat
   } \
else \
   {
    set thisName $outPrefix$xyOutName
   }

if {$verbose} {puts "Writing Projection: $thisName"}

writeROI -roi 1 -out $thisName -ov 

if {![strcasecmp $xzOutName Auto]} \
   {
    set thisName $outPrefix$xName.$zName.dat
   } \
else \
   {
    set thisName $outPrefix$xzOutName
   }

if {$verbose} {puts "Writing Projection: $thisName"}

writeROI -roi 2 -out $thisName -ov

if {![strcasecmp $xaOutName Auto]} \
   {
    set thisName $outPrefix$xName.$aName.dat
   } \
else \
   {
    set thisName $outPrefix$xaOutName
   }

if {$verbose} {puts "Writing Projection: $thisName"}

writeROI -roi 3 -out $thisName -ov

if {![strcasecmp $yzOutName Auto]} \
   {
    set thisName $outPrefix$yName.$zName.dat
   } \
else \
   {
    set thisName $outPrefix$yzOutName
   }

if {$verbose} {puts "Writing Projection: $thisName"}

writeROI -roi 4 -out $thisName -ov

if {![strcasecmp $yaOutName Auto]} \
   {
    set thisName $outPrefix$yName.$aName.dat
   } \
else \
   {
    set thisName $outPrefix$yaOutName
   }

if {$verbose} {puts "Writing Projection: $thisName"}

writeROI -roi 5 -out $thisName -ov

if {![strcasecmp $zaOutName Auto]} \
   {
    set thisName $outPrefix$zName.$aName.dat
   } \
else \
   {
    set thisName $outPrefix$zaOutName
   }

if {$verbose} {puts "Writing Projection: $thisName"}

writeROI -roi 6 -out $thisName -ov

exit 0
