#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Add Empty Imaginary Data to Z-Axis of Pseudo-3D Data."
   echo " -in  inName   [fid/test%03d.fid]  Pseudo-3D Input  (Z-Axis Real)."
   echo " -out outName  [qfid/test%03d.fid] Pseudo-3D Output (Z-Axis Complex)."
   echo " -mask                             Input is a Mask."
   echo " -verb                             Verbose Mode ON (Default)."
   echo " -noverb                           Verbose Mode OFF."
   exit 1
endif

set inName  = (`getArgD $argv -in  fid/test%03d.fid`)
set outName = (`getArgD $argv -out qfid/test%03d.fid`)
set dirName = (`string path $outName`)

@ vFlag    = 1
@ maskFlag = 0

if (`flagLoc $argv -noverb`) then
   @ vFlag = 0
endif

if (`flagLoc $argv -mask`) then
   @ maskFlag = 1
endif

if (!(-d $dirName)) then
   mkdir $dirName
endif

set origOutName = $outName
set tmpDir      = "None"

@ fmtCount = (`echo $outName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

if ($fmtCount == 0) then
   set tmpDir  = quad$$
   set outName = $tmpDir/test%03d.fid

   /bin/rm -rf $tmpDir
   mkdir $tmpDir
endif

if ($maskFlag) then
   set modeStr = "Mask"
else
   set modeStr = "FID"
endif

@ zSize = 0

@ iz = 1
@ jz = 1

if ($vFlag) then
   echo ""
endif

while ($iz <= 9999)
   set thisInName = (`nmrPrintf $inName $iz`)

   if (!(-e $thisInName)) then
      break
   endif

   set thisOutName = (`nmrPrintf $outName $jz`)
   /bin/cp $thisInName $thisOutName

   if ($vFlag) then
      echo "Pseudo-3D to Quadrature $modeStr Mode. Source: $thisInName Destination: $thisOutName (Copy)"
   endif

   @ jz++

   set thisOutName = (`nmrPrintf $outName $jz`)

   if ($maskFlag) then
      /bin/cp $thisInName $thisOutName

      if ($vFlag) then
         echo "Pseudo-3D to Quadrature $modeStr Mode. Source: $thisInName Destination: $thisOutName (Copy)"
      endif
   else
      nmrPipe -in $thisInName -out $thisOutName -fn SET -c 0.0 -ov
 
      if ($vFlag) then
         echo "Pseudo-3D to Quadrature $modeStr Mode. Source: $thisInName Destination: $thisOutName (Zeros)"
      endif
   endif

   @ zSize = $jz
   @ iz++
   @ jz++
end

if ($zSize == 0) then
   exit 0
endif

@ jz = 1

while ($jz <= $zSize)
   set thisOutName = (`nmrPrintf $outName $jz`)
   sethdr $thisOutName -yMODE Complex -zMODE Complex -zN $zSize -fileCount $zSize
   @ jz++
end

if (`string notcsame $tmpDir None`) then
   if ($vFlag) then
      echo ""
      echo "Pseudo-3D to Quadrature $modeStr Mode. Source: $outName Destination: $origOutName"
   endif

   xyz2pipe -in $outName -out $origOutName

   /bin/rm -rf $tmpDir
endif

