#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set inList    test.ft2
set outName   None
set tabName   None
set xRefPPM   0.0
set yRefPPM   0.0
set dxPPM     Auto
set dyPPM     Auto
set f         0.1
set x1        0.1ppm
set xn       -0.1ppm
set y1        2.0ppm
set yn       -2.0ppm
set ddx       Auto 
set ddy       Auto 
set idx       Auto 
set idy       Auto 
set tolx      Auto
set toly      Auto 
set verbose   1
set briefFlag 0
set avgFlag   0

set fdataQ 0
set rdataQ 0
set fdata  0

if {[flagLoc $ARGV -help]} \
  {
   puts "Re-Reference PPM Calibration by Max Point or Peak in Region:"
   puts " -in    inName  \[test.ft2\]   Input Spectrum."
   puts " -out   outName \[None\]       Optional Region Output Spectrum."
   puts " -tab   tabName \[None\]       Optional Peak Table Output."
   puts " -xRef  xRef    \[0.0\]        Desired X-Axis PPM of Reference Signal."
   puts " -yRef  yRef    \[0.0\]        Desired Y-Axis PPM of Reference Signal."
   puts " -x1    x1      \[0.1ppm\]     X-Axis Start of Analysis Region."
   puts " -xn    xn      \[-0.1ppm\]    X-Axis End of Analyis Region."
   puts " -y1    y1      \[2.0ppm\]     Y-Axis Start of Analysis Region."
   puts " -yn    yn      \[-2.0ppm\]    Y-Axis End of Analyis Region."
   puts " -verb                       Verbose Mode ON (Default)."
   puts " -noverb                     Verbose Mode OFF."
   puts " -brief                      Brief Report Mode ON."
   puts "Specifying a Specific Adjustment to Calibration:"
   puts " -xAdj  xAdj    \[Auto\]       Amount to Adjust X-Axis Calibration, PPM"
   puts " -yAdj  yAdj    \[Auto\]       Amount to Adjust Y-Axis Calibration, PPM"
   puts "Peak Detection Parameters:"
   puts " -ddx   ddx     \[Auto\]       X-Axis +/- Width of Detection Region, Pts."
   puts " -ddy   ddy     \[Auto\]       X-Axis +/- Width of Detection Region, Pts."
   puts " -idx   idx     \[Auto\]       X-Axis +/- Width of Interpolation Region, Pts."
   puts " -idy   idy     \[Auto\]       Y-Axis +/- Width of Interpolation Region, Pts."
   puts " -tolx  tolx    \[Auto\]       X-Axis +/- Peak Position Tolerance, Pts."
   puts " -toly  toly    \[Auto\]       Y-Axis +/- Peak Position Tolerance, Pts."
   puts " -avg                        Use Average Peak Position for Multiple Peaks."
   puts " -f     fThresh \[0.1\]        Height Fraction for Detect Threshold."
   puts "Notes:"
   puts " 1. If a peak table name is given, peak analysis will be used."
   puts "    Otherwise, the largest point in the reference region will be used."
   puts " 2. The -brief report format is:"
   puts "    xAdjPPM 0.0 yAdjPPM 0.0 xAdjHz 0.0 yAdjHz 0.0"
   exit 1
  }

getListArgD $ARGV -in inList

getArgD $ARGV -out  outName
getArgD $ARGV -tab  tabName

getArgD $ARGV -xRef xRefPPM 
getArgD $ARGV -yRef yRefPPM 
getArgD $ARGV -xAdj dxPPM 
getArgD $ARGV -yAdj dyPPM 
getArgD $ARGV -f    f

getArgD $ARGV -x1   x1
getArgD $ARGV -xn   xn
getArgD $ARGV -y1   y1
getArgD $ARGV -yn   yn

getArgD $ARGV -ddx  ddx
getArgD $ARGV -ddy  ddy
getArgD $ARGV -idx  idx
getArgD $ARGV -idy  idy
getArgD $ARGV -tolx tolx
getArgD $ARGV -toly toly

if {[flagLoc $ARGV -verb]}    {set verbose 1}
if {[flagLoc $ARGV -noverb]}  {set verbose 0}

if {[flagLoc $ARGV -brief]}   {set briefFlag 1}
if {[flagLoc $ARGV -nobrief]} {set briefFlag 0}

if {[flagLoc $ARGV -avg]}     {set avgFlag 1}
if {[flagLoc $ARGV -noavg]}   {set avgFlag 0}

if {$briefFlag} {set verbose 0}

#
#--------------------------------------------------------------------------

set inName  [lindex $inList 0]

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input Spectrum $inName"
    exit 1
   }

set error [rdFDATA $inName fdata]

if {$error} \
   {
    puts stderr "Error Reading Header from Spectrum $inName"
    exit 1
   }

set xtSize [getParmI $fdata NDTDSIZE CUR_XDIM]
set ytSize [getParmI $fdata NDTDSIZE CUR_YDIM]

set xfSize [getParmI $fdata NDFTSIZE CUR_XDIM]
set yfSize [getParmI $fdata NDFTSIZE CUR_YDIM]

if {$xtSize > 0 && $xfSize > 0} \
   {
    set dxDef   [expr int(sqrt( 0.5*double($xfSize)/double($xtSize) ))]
    set xTolDef [expr 0.5*double($xfSize)/double($xtSize)]

    if {$dxDef < 1}     {set dxDef   1}
    if {$xTolDef < 2.0} {set xTolDef 2.0}
   } \
else \
   {
    set dxDef   1
    set xTolDef 2.0
   }

if {$ytSize > 0 && $yfSize > 0} \
   {
    set dyDef   [expr int(sqrt( 0.5*double($yfSize)/double($ytSize) ))]
    set yTolDef [expr 0.5*double($yfSize)/double($ytSize)]

    if {$dyDef < 1}     {set dyDef   1}
    if {$yTolDef < 2.0} {set yTolDef 2.0}
   } \
else \
   {
    set dyDef   1
    set yTolDef 2.0
   }

if {![strcasecmp $ddx  Auto]} {set ddx  $dxDef}
if {![strcasecmp $idx  Auto]} {set idx  $dxDef}
if {![strcasecmp $tolx Auto]} {set tolx $xTolDef}

if {![strcasecmp $ddy  Auto]} {set ddy  $dyDef}
if {![strcasecmp $idy  Auto]} {set idy  $dyDef}
if {![strcasecmp $toly Auto]} {set toly $yTolDef}

set x1Pts [expr int( 0.5 + [spec2pnt $fdata CUR_XDIM $x1] )]
set xnPts [expr int( 0.5 + [spec2pnt $fdata CUR_XDIM $xn] )]

set y1Pts [expr int( 0.5 + [spec2pnt $fdata CUR_XDIM $y1] )]
set ynPts [expr int( 0.5 + [spec2pnt $fdata CUR_XDIM $yn] )]

readROI -ndim 2 -roi 1 -x X_AXIS $x1 $xn -y Y_AXIS $y1 $yn -in $inName

if {[strcasecmp $outName None]} \
  {
   writeROI -roi 1 -out $outName -ov
#  readROI -ndim 2 -roi 1 -x X_AXIS 0% 100% -y Y_AXIS 0% 100% -in $outName
  }

set roiInfo [getROIInfo -roi 1]
set rdataQ  [lindex $roiInfo 0]
set fdataQ  [lindex $roiInfo 1]
set sInfo   [lindex $roiInfo 2]

if {[llength $sInfo] < 2} \
   {
    puts stderr "2D Calibration Error: dimension count < 2 for $inName"
    exit 1
   }

set xSize   [lindex $sInfo 0]
set ySize   [lindex $sInfo 1]

set maxVal  [getPtrVal $rdataQ 0]
set maxXLoc 1
set maxYLoc 1 

set xPPM    0.0
set yPPM    0.0

if {![strcasecmp $dxPPM Auto] || ![strcasecmp $dyPPM Auto]} \
   {
    set k 0

    for {set iy 1} {$iy <= $ySize} {incr iy} \
       {
        for {set ix 1} {$ix <= $xSize} {incr ix} \
           {
            set hi [getPtrVal $rdataQ $k]

            if {abs($hi) > abs($maxVal)} \
               {
                set maxVal  $hi
                set maxXLoc $ix
                set maxYLoc $iy
               } 

            incr k
           }
       }

    if {$verbose} \
       {
        puts "Reference PPM. Max Point X: [format %9.3f [expr $x1Pts + $maxXLoc - 1]] pts."
        puts "Reference PPM. Max Point Y: [format %9.3f [expr $y1Pts + $maxYLoc - 1]] pts."
       }
   }

if {![strcasecmp $dxPPM Auto] || ![strcasecmp $dyPPM Auto]} \
   {
    if {[strcasecmp $tabName None]} \
       {
        set thresh [expr $f*abs($maxVal)]
        set noise  [expr 0.1*$thresh]

        if {$verbose} \
           {
            puts "Reference PPM. Peak DDX:  [format %2d $ddx] IDX: [format %2d $idx] XTOL: [format %5.2f $tolx]"
            puts "Reference PPM. Peak DDY:  [format %2d $ddy] IDY: [format %2d $idy] YTOL: [format %5.2f $toly]"
           }

        pkFindROI -roi 1 \
             -sigma $noise -pChi 1.0e-04 -plus $thresh -minus -$thresh -ppmPrec 5 \
             -dx      $ddx  $ddy  \
             -idx     $idx  $idy  \
             -tol     $tolx $toly \
             -hiAdj   1.20 1.80   \
             -lw     10.00 10.0   \
             -reject -mask -out $tabName -noverb

        set pkTab [gdbRead table -in $tabName]

        gdbSortTab $pkTab HEIGHT -down -abs

        set eList [gdbGetInfo $pkTab -parms entries]
        set e     [lindex $eList 0]
        set n     [llength $eList]

        if {$n} \
           {
            if {$avgFlag} \
               {
                set maxXLoc [getTabStat getAvg $pkTab X_AXIS]
                set maxYLoc [getTabStat getAvg $pkTab Y_AXIS]

                if {$verbose} \
                   {
                    puts "Reference PPM. Avg Peak X:  [format %11.5f [expr $x1Pts + $maxXLoc - 1]] pts ($maxXLoc)."
                    puts "Reference PPM. Avg Peak Y:  [format %11.5f [expr $y1Pts + $maxYLoc - 1]] pts ($maxYLoc)."
                    puts "Reference PPM. Peaks Found: $n"
                   }
               } \
            else \
               {
                set maxXLoc [gdbGet $e X_AXIS]
                set maxYLoc [gdbGet $e Y_AXIS]
    
                if {$verbose} \
                   {
                    puts "Reference PPM. Max Peak X:  [format %11.5f [expr $x1Pts + $maxXLoc - 1]] pts ($maxXLoc)."
                    puts "Reference PPM. Max Peak Y:  [format %11.5f [expr $y1Pts + $maxYLoc - 1]] pts ($maxYLoc)."
                    puts "Reference PPM. Peaks Found: $n"
                   }
               }
           }
       } 
   } 

set xPPM  [pnt2spec $fdataQ CUR_XDIM $maxXLoc ppm]
set yPPM  [pnt2spec $fdataQ CUR_YDIM $maxYLoc ppm]

set xCAR  [getParm $fdata NDCAR  CUR_XDIM]
set xOBS  [getParm $fdata NDOBS  CUR_XDIM]
set xORIG [getParm $fdata NDORIG CUR_XDIM]

set yCAR  [getParm $fdata NDCAR  CUR_YDIM]
set yOBS  [getParm $fdata NDOBS  CUR_YDIM]
set yORIG [getParm $fdata NDORIG CUR_YDIM]

if {$verbose} \
   {
    set fmt "% 10.5f"

    if {![strcasecmp $dxPPM Auto]} \
       {
        puts "Reference PPM. Reference Location X: [format $fmt $xPPM] ppm."
       } \
    else \
       {
        puts "Reference PPM. Reference Location X: [format %9s None]."
       }

    if {![strcasecmp $dxPPM Auto]} \
       {
        puts "Reference PPM. Reference Location Y: [format $fmt $yPPM] ppm."
       } \
    else \
       {
        puts "Reference PPM. Reference Location Y: [format %9s None]."
       }
   }

if {![strcasecmp $dxPPM Auto]} {set dxPPM [expr $xRefPPM - $xPPM]}
if {![strcasecmp $dyPPM Auto]} {set dyPPM [expr $yRefPPM - $yPPM]}

set dxHz  [expr $xOBS*$dxPPM]
set dyHz  [expr $yOBS*$dyPPM]

if {$verbose} \
   {
    set fmt "% 10.5f"
    puts "Reference PPM. Adjustment X: [format $fmt $dxPPM] ppm [format $fmt $dxHz] Hz."
    puts "Reference PPM. Adjustment Y: [format $fmt $dyPPM] ppm [format $fmt $dyHz] Hz."
   }

setParm $fdata NDCAR  [expr $xCAR  + $dxPPM] CUR_XDIM
setParm $fdata NDORIG [expr $xORIG + $dxHz]  CUR_XDIM

setParm $fdata NDCAR  [expr $yCAR  + $dyPPM] CUR_YDIM
setParm $fdata NDORIG [expr $yORIG + $dyHz]  CUR_YDIM

if {$verbose} \
   {
    puts "Reference PPM. Saving Calibration: $inName"
    puts ""
   }

set error [wrFDATA $inName $fdata]

if {$briefFlag} {puts "xAdjPPM $dxPPM yAdjPPM $dyPPM xAdjHz $dxHz yAdjHz $dyHz"}
 
exit 0
