#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Chemical Shift Referencing of Spectral Series Listed in a Metadata Table."
   echo "Version 2025.071.20.32"
   echo " -meta    metaName [meta.tab]     Metadata Table (INDEX DIR_NAME TITLE)."
   echo " -fidName fidName  [test.fid]     Input Name of FID In Its Source Directory."
   echo " -ftName  ftName   [Auto]         Name of Processed Spectrum In Its Original Directory."
   echo " -fidDir  fidDir   [None]         Directory of Time-Domain Data, or Keyword None."
   echo " -ftDir   ftDir    [ft]           Directory of Processed Spectra, or Keyword None."
   echo " -srcDir  srcDir   [.]            Source Directory."
   echo "Chemical Shift Reference Command; Should be Last on Command-Line:"
   echo " -cmnd    refCmnd  [Auto]         Referencing Command (Auto for ref1D.tcl or ref2D.tcl ...)."
   echo "Notes:"
   echo "  1. If the -fidName option is not None, the given FID will have its chemical shift"
   echo "     calibration updated according to the spectrum."
   echo "  2. If the -fidDir option is used, the data in this directory will also have their"
   echo "     chemical shift calibration updated."
   echo "  3. If the -ftDir option is used, the spectra in this directory will also have their"
   echo "     chemical shift calibration updated."
   echo ""
   exit 0
endif

set refCmnd  = "Auto"
set loc      = (`flagLoc $argv -cmnd`)
set thisArgv = ($argv)

if ($loc >= 1) then
   @ locP = $loc - 1
   @ locN = $loc + 1

   set refCmnd  = ($argv[$locN-])
   set thisArgv = ($argv[1-$locP])
endif

set srcDir     = (`getArgD $thisArgv -srcDir  .`)
set ftDir      = (`getArgD $thisArgv -ftDir   ft`)
set fidDir     = (`getArgD $thisArgv -fidDir  None`)
set fidName    = (`getArgD $thisArgv -fidName None`)
set ftName     = (`getArgD $thisArgv -ftName  Auto`)
set maskName   = (`getArgD $thisArgv -mask    mask.fid`)
set schedName  = (`getArgD $thisArgv -sample  nuslist`)
set tabName    = (`getArgD $thisArgv -meta    meta.tab`)

#
#----------------------------------------------------------------------------------

set origDir = (`pwd`)

cd $srcDir
   set srcDirFull = (`pwd`)
cd $origDir

if (`string csame $fidDir None`) then
   set fidDirFull = None
else
   cd $fidDir
      set fidDirFull = (`pwd`)
   cd $origDir
endif

if (`string csame $ftDir None`) then
   set ftDirFull = None
else
   cd $ftDir
      set ftDirFull = (`pwd`)
   cd $origDir
endif

set idList   = (`getTabCol.tcl -in $tabName -var INDEX`)
set dirList  = (`getTabCol.tcl -in $tabName -var DIR_NAME`)
set titList  = (`getTabCol.tcl -in $tabName -var TITLE`)

@ dimCount = 2
@ nusFlag  = 0

if (`string notcsame $fidName None`) then
   set thisName = $srcDir/$dirList[1]/$fidName

   if (-e $thisName) then
      @ dimCount = (`getParm -in $thisName -parm FDDIMCOUNT -dim NULL_DIM -fmt %.0f`)
   
      if ((-e $srcDir/$dirList[1]/$maskName) || (-e $srcDir/$dirList[1]/$schedName)) then
         @ nusFlag = 1
      endif
   endif
endif

set thisExt = ft$dimCount

if (`string csame $ftName Auto`) then
   set ftName = test.${thisExt}
endif

if (`string csame $refCmnd Auto`) then
   set refCmnd = "ref${dimCount}D.tcl -tab ref.tab -max -f 0.3"
endif

echo ""

@ eCount = 0
@ i = 0

foreach d ($dirList)
   @ i++

   if (!(-d $srcDirFull/$d)) then
      echo "Referencing Error Finding Input Directory $srcDirFull/$d"
      @ eCount++
      continue
   endif

   cd $srcDirFull/$d
      set thisDir   = (`pwd`)
      set thisTitle = (`nmrPrintf "%03d-%s" $idList[$i] $titList[$i]`)

      if (!(-e $ftName)) then
         echo "Referencing Error Finding Spectrum Input $srcDirFull/$d/$ftName"
         @ eCount++
         continue
      endif

      if (`string notcsame $ftDir None`) then
         set outName = $ftDirFull/${thisTitle}.${thisExt}
      endif

      echo "Referencing Spectrum $i of $#idList"':'
      echo " Input Directory:  $thisDir"
      echo " Title:            $thisTitle"

      set refList = ($ftName)

      if (`string notcsame $fidName None` && (-e $fidName)) then
         set refList = ($refList $fidName)
      endif

      if (`string notcsame $fidDir None`) then
         set thisName = $fidDirFull/${thisTitle}.fid

         if (-e $thisName) then
            set refList = ($refList $thisName)
         else
            echo "Referencing Error Finding FID Input $thisName"
            @ eCount++
         endif
      endif

      if (`string notcsame $ftDir None`) then
         set thisName = $ftDirFull/${thisTitle}.${thisExt}

         if (-e $thisName) then
            set refList = ($refList $thisName)
         else
            echo "Referencing Error Finding Spectrum Input $thisName"
            @ eCount++
         endif
      endif

      $refCmnd -in $refList

   cd $origDir
end

if ($eCount != 0) then
   echo "Referencing Notice: Errors Occurred."
   exit 1
endif

exit 0
