#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Rename PDB Atoms to DYNAMO Form:"
    puts " -in  inPDBName    Input PDB."
    puts " -out outPDBName   Output PDB" 
    puts " -ref refPDBName   Optional Reference PDB"
    puts "Optional Setting of Segment or Chain:"
    puts " -seg   segName"
    puts " -chain chainName"
    puts "Flags:"
    puts " -na   Input PDB Consists of DNA/RNA Residues." 
    puts " -del  Delete Atoms Not Found in Reference." 
    exit 1
   }

set inName    None
set refName   None 
set outName   None
set segName   (Null) 
set chainName (Null) 

set delFlag 0
set naFlag  0

getArgD $ARGV -in    inName
getArgD $ARGV -ref   refName
getArgD $ARGV -out   outName
getArgD $ARGV -seg   segName
getArgD $ARGV -chain chainName

if {[flagLoc $ARGV -del]}   {set delFlag 1}
if {[flagLoc $ARGV -nodel]} {set delFlag 0}

if {[flagLoc $ARGV -na]}    {set naFlag 1}
if {[flagLoc $ARGV -nona]}  {set naFlag 0}

#
#----------------------------------------------------------

proc noQuote { line } \
{
   set n   [string length $line]
   set str ""

   for {set i 0} {$i < $n} {incr i} \
      {
       set c [string index $line $i]

       if {[strcmp $c \"]} \
          {
           append str $c
          } \
       else \
          {
           append str ''
          }
      }

   return $str
}

#
#----------------------------------------------------------

set naResInList   {A   G   C   T   U   RADE RGUA RCYT URA}
set naResOutList  {ADE GUA CYT THY URI ADE  GUA  CYT  URI}

set naAtomInList  {H2'1 H2'2 H5'1 H5'2 HO'2 HO2  OP1 OP2}
set naAtomOutList {H2'  H2'' H5'  H5'' HO2' H2'' O1P O2P}

set pdbTab 0
set refTab 0
set eList  ""

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input PDB $inName"
    exit
   }
 
set pdbTab [gdbRead table -in $inName -pdb]
set eList  [gdbGetInfo $pdbTab -parms entries]

if {[file exists $refName]} \
   {
    set refTab [gdbRead table -in $refName -pdb]
   }

set delList ""
set blank   " "

foreach entry $eList \
   {
    set resID     [gdbGet $entry RESID]
    set thisRName [gdbGet $entry RESNAME]
    set thisAName [gdbGet $entry ATOMNAME]

    set atomName  [noQuote $thisAName]
    set resName   $thisRName

    set c  [string index  $atomName 0]
    set n  [string length $atomName]
    set cc [string range  $atomName 1 [expr $n - 1]]

    if {!$naFlag && ![strcmp $atomName H]} {set atomName HN}
    if {[lsearch -exact "1 2 3" $c] >= 0}  {set atomName $cc$c}

    if {$naFlag} \
       {
        set loc [lsearch -exact $naResInList $resName]
        if {$loc >= 0} {set resName [lindex $naResOutList $loc]}

        set loc [lsearch -exact $naAtomInList $atomName]
        if {$loc >= 0} {set atomName [lindex $naAtomOutList $loc]}
       }

    if {![strcmp $resName ILE]} \
       {
        switch $atomName \
           {
            CD1  {set atomName CD}
            HD11 {set atomName HD1}
            HD12 {set atomName HD2}
            HD13 {set atomName HD3}
           }
       }

    if {$refTab} \
       {
        set cond  "RESID == $resID && !strcmp( ATOMNAME, \"$atomName\" )"
        set match [gdbSelect -from $refTab -cond $cond]

        if {![llength $match]} \
           {
            if {$delFlag} \
               {
                set fmt "No Match. Old: %4d %-4s %-4s New: %-4s %-4s (Deleted)"

                puts \
                 [format $fmt $resID $thisRName $thisAName $resName $atomName]

                lappend delList $entry
               } \
            else \
               {
                set fmt "No Match. Old: %4d %-4s %-4s New: %-4s %-4s"

                puts \
                 [format $fmt $resID $thisRName $thisAName $resName $atomName]
               }
           }
       }

    gdbSet $entry RESNAME  $resName
    gdbSet $entry ATOMNAME $atomName
    gdbSet $entry CHAINID  $chainName
    gdbSet $entry SEGID    $segName
   } 

if {[llength $delList]} {gdbDestroy $delList}

gdbWrite $pdbTab $outName -pdb

exit
