#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "List ND Sizes and Chemical Shift Info:"
   echo " reportND.com inName1 ... [-verb]"
endif

if ($#argv == 0) then
   exit 0
endif

@ vFlag = 0

if (`flagLoc $argv -verb`) then
   @ vFlag = 1
endif

foreach inNameOrig ($*)
   if (`isFlag $inNameOrig`) then
      continue
   endif

   set inName = (`nmrPrintf $inNameOrig 1 1`)

   if ($vFlag) then
      echo `nmrWishCmnd file normalize $inName` `getParm -in $inName -parm FDTITLE -dim NULL_DIM -text -fmt %s`
   endif

   if (!(-e $inName)) then
      nmrPrintf " $inName does not exist."
      continue
   endif

   set vInfo = (`showhdr -test $inName` 0 0)

   if ($vInfo[2] == 0) then
      nmrPrintf " $inName is not valid NMRPipe data."
      continue
   endif

   set dimCount = (`getParm -in $inName -fmt %.0f -parm FDDIMCOUNT -dim NULL_DIM`)
  
   set xSW  = (`getParm -in $inName -fmt %.0f -parm NDSW    -dim CUR_XDIM`)
   set xCAR = (`getParm -in $inName -fmt %.3f -parm NDCAR   -dim CUR_XDIM`)
   set xOBS = (`getParm -in $inName -fmt %.3f -parm NDOBS   -dim CUR_XDIM`)
   set xN   = (`getParm -in $inName -fmt %.0f -parm NDSIZE  -dim CUR_XDIM`)
   set xT   = (`getParm -in $inName -fmt %.0f -parm NDAPOD  -dim CUR_XDIM`)
   set xP0  = (`getParm -in $inName -fmt %.1f -parm NDP0    -dim CUR_XDIM`)
   set xP1  = (`getParm -in $inName -fmt %.1f -parm NDP1    -dim CUR_XDIM`)
   set xLAB = (`getParm -in $inName -fmt %.0f -parm NDLABEL -dim CUR_XDIM -text -fmt %s`)

   nmrPrintf " X-Axis %4s Size: %4d %4d P0: %6.1f P1: %6.1f SW: %7.1f Hz CAR: %7.3f ppm OBS: %7.3f MHz\n" $xLAB $xT $xN $xP0 $xP1 $xSW $xCAR $xOBS

   if ($dimCount == 1) continue

   set ySW  = (`getParm -in $inName -fmt %.0f -parm NDSW    -dim CUR_YDIM`)
   set yCAR = (`getParm -in $inName -fmt %.3f -parm NDCAR   -dim CUR_YDIM`)
   set yOBS = (`getParm -in $inName -fmt %.3f -parm NDOBS   -dim CUR_YDIM`)
   set yN   = (`getParm -in $inName -fmt %.0f -parm NDSIZE  -dim CUR_YDIM`)
   set yT   = (`getParm -in $inName -fmt %.0f -parm NDAPOD  -dim CUR_YDIM`)
   set yP0  = (`getParm -in $inName -fmt %.1f -parm NDP0    -dim CUR_YDIM`)
   set yP1  = (`getParm -in $inName -fmt %.1f -parm NDP1    -dim CUR_YDIM`)
   set yLAB = (`getParm -in $inName -fmt %.0f -parm NDLABEL -dim CUR_YDIM -text -fmt %s`)

   nmrPrintf " Y-Axis %4s Size: %4d %4d P0: %6.1f P1: %6.1f SW: %7.1f Hz CAR: %7.3f ppm OBS: %7.3f MHz\n" $yLAB $yT $yN $yP0 $yP1 $ySW $yCAR $yOBS

   if ($dimCount == 2) continue

   set zSW  = (`getParm -in $inName -fmt %.0f -parm NDSW    -dim CUR_ZDIM`)
   set zCAR = (`getParm -in $inName -fmt %.3f -parm NDCAR   -dim CUR_ZDIM`)
   set zOBS = (`getParm -in $inName -fmt %.3f -parm NDOBS   -dim CUR_ZDIM`)
   set zN   = (`getParm -in $inName -fmt %.0f -parm NDSIZE  -dim CUR_ZDIM`)
   set zT   = (`getParm -in $inName -fmt %.0f -parm NDAPOD  -dim CUR_ZDIM`)
   set zP0  = (`getParm -in $inName -fmt %.1f -parm NDP0    -dim CUR_ZDIM`)
   set zP1  = (`getParm -in $inName -fmt %.1f -parm NDP1    -dim CUR_ZDIM`)
   set zLAB = (`getParm -in $inName -fmt %.0f -parm NDLABEL -dim CUR_ZDIM -text -fmt %s`)

   nmrPrintf " Z-Axis %4s Size: %4d %4d P0: %6.1f P1: %6.1f SW: %7.1f Hz CAR: %7.3f ppm OBS: %7.3f MHz\n" $zLAB $zT $zN $zP0 $zP1 $zSW $zCAR $zOBS

  if ($dimCount == 3) continue

   set aSW  = (`getParm -in $inName -fmt %.0f -parm NDSW    -dim CUR_ADIM`)
   set aCAR = (`getParm -in $inName -fmt %.3f -parm NDCAR   -dim CUR_ADIM`)
   set aOBS = (`getParm -in $inName -fmt %.3f -parm NDOBS   -dim CUR_ADIM`)
   set aN   = (`getParm -in $inName -fmt %.0f -parm NDSIZE  -dim CUR_ADIM`)
   set aT   = (`getParm -in $inName -fmt %.0f -parm NDAPOD  -dim CUR_ADIM`)
   set aP0  = (`getParm -in $inName -fmt %.1f -parm NDP0    -dim CUR_ADIM`)
   set aP1  = (`getParm -in $inName -fmt %.1f -parm NDP1    -dim CUR_ADIM`)
   set aLAB = (`getParm -in $inName -fmt %.0f -parm NDLABEL -dim CUR_ADIM -text -fmt %s`)

   nmrPrintf " A-Axis %4s Size: %4d %4d P0: %6.1f P1: %6.1f SW: %7.1f Hz CAR: %7.3f ppm OBS: %7.3f MHz\n" $aLAB $aT $aN $aP0 $aP1 $aSW $aCAR $aOBS
end
