#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Resample NMRPipe-Format Time-Domain Data:"
   echo " -in     inName   [test.fid]    Time-Domain Data Input."
   echo " -sample sName    [nuslist_us]  NUS Schedule Input for Resampling."
   echo " -out    outName  [Auto]        Resampled Time-Domain Output."
   echo " -mask   maskName [Auto]        Output Mask."
   exit 0
endif

set inName    = (`getArgD $argv -in     test.fid`)
set schedName = (`getArgD $argv -sample nuslist_us`)
set outName   = (`getArgD $argv -out    Auto`)
set maskName  = (`getArgD $argv -mask   Auto`)

set vFlag = 1

if (`flagLoc $argv -noverb`) then
   set vFlag = 0
endif

set thisName = (`nmrPrintf $inName 1 1`)

if (!(-e $thisName)) then
   nmrPrintf -stderr "resampleNMR. Error Finding Time-Domain Input %s\n" $inName
   exit
endif

if (!(-e $schedName)) then
   if ($vFlag) then
      nmrPrintf -stderr "resampleNMR. Sampling Schedule %s Does Not Exist;\n"  $schedName
      nmrPrintf -stderr "resampleNMR. Creating Uniform Sampling Schedule %s\n" $schedName

      fakeNUS.com -nus 1.0 -in $inName -verb   > $schedName

      nmrPrintf -stderr "\n"
   else
      fakeNUS.com -nus 1.0 -in $inName -noverb > $schedName
   endif
endif 

@ dimCount = (`getParm -in $thisName -parm FDDIMCOUNT -dim NULL_DIM -fmt %.0f`)
@ fmtCount = (`echo $inName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

if ($dimCount == 1 || $dimCount == 2) then
   if (`string csame $outName Auto`) then
      set outName = nus.fid
   endif

   if (`string csame $maskName Auto`) then
      set maskName = mask.fid
   endif
else
   if (`string csame $outName Auto`) then
      set outName = nus/test%03d.fid
   endif
   
   if (`string csame $maskName Auto`) then
      set maskName = mask/test%03d.fid
   endif
endif

if ($vFlag) then
   nmrPrintf -stderr "\n"
   nmrPrintf -stderr "resampleNMR. Input FID:     %s\n" $inName
   nmrPrintf -stderr "resampleNMR. NUS Schedule:  %s\n" $schedName
   nmrPrintf -stderr "resampleNMR. Resampled FID: %s\n" $outName
   nmrPrintf -stderr "resampleNMR. Mask Output:   %s\n" $maskName

   nusCompress.tcl -mode pipe -nocompress -in $inName -sample $schedName -out $outName -mask $maskName -verb
else
   nusCompress.tcl -mode pipe -nocompress -in $inName -sample $schedName -out $outName -mask $maskName -noverb
endif

if ($vFlag) then
   nmrPrintf -stderr "\n"
endif

exit 0
