#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Scale NMRPipe-format Data to a Given Maximum Value:"
   echo " -in   inName  [test.ft2]  Input Data, 1D-4D."
   echo " -out  outName [scale.ft2] Output Data."
   echo " -max  maxVal  [100.0]     Desired Maximum Value After Scaling."
   echo " -stat maxFn   [vMaxAbs]   Statistic Used to Compute Maximum."
   echo " -real                     Scale According to Real Value Maximum (Default)."
   echo " -imag                     Scale According to Imaginary Maximim."
   echo " -complex                  Scale According to Overall Maximum."
   echo " -hdr                      Record Scale Value in Header."
   echo "Region Limits for Determination of Input Maximim Value (Use -x.. -y.. etc):"
   echo " -x1   x1      [0%]        Region Start (ppm Hz % pts)."
   echo " -xn   xn      [100%]      Region Start (ppm Hz % pts)."
   echo "Notes:"
   echo " 1. Scaling is performed according to maximum absolute value."
   echo " 2. Input and output can be the same."
   echo " 3. The entire input region is read into memory, so that this"
   echo "    script might not work for large spectra."
   exit 0
endif

set inName  = (`getArgD $argv -in   test.ft2`)
set outName = (`getArgD $argv -out  scale.ft2`)
set maxVal  = (`getArgD $argv -max  100.0`)
set maxFn   = (`getArgD $argv -stat vMaxAbs`)

set quadType  = real
set hdrFlag   = 0

if (`flagLoc $argv -imag`) then
   set quadType = imag 
endif

if (`flagLoc $argv -complex`) then
   set quadType = complex 
endif

if (`flagLoc $argv -hdr`) then
   set hdrFlag = 1
endif

set x1 = (`getArgD $argv -x1 0%`)
set xn = (`getArgD $argv -xn 100%`)

set y1 = (`getArgD $argv -y1 0%`)
set yn = (`getArgD $argv -yn 100%`)

set z1 = (`getArgD $argv -z1 0%`)
set zn = (`getArgD $argv -zn 100%`)

set a1 = (`getArgD $argv -a1 0%`)
set an = (`getArgD $argv -an 100%`)

#
#-------------------------------------------------------------------------------------

set thisName = (`nmrPrintf $inName 1 1`)

if (!(-e $thisName)) then
#  echo "specStat. Error Finding Input $thisName"
   exit 0
endif

set pipeFlag     = (`getParm -in $thisName -fmt %.0f -parm FDPIPEFLAG -dim NULL_DIM`)
set origDimCount = (`getParm -in $thisName -fmt %.0f -parm FDDIMCOUNT -dim NULL_DIM`)
set dimCount     = $origDimCount

@ fmtCount = (`echo $inName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

if ($pipeFlag == 0) then
   if ($fmtCount == 0) then
      if ($dimCount > 2) then
         set dimCount = 2
      endif
   endif
endif

if ($dimCount == 1) then
   set val = (`readROI -ndim 1 -in $inName -$quadType -silent -noverb -vec $maxFn -x X_AXIS $x1 $xn`)
else if ($dimCount == 2) then
   set val = (`readROI -ndim 2 -in $inName -$quadType -silent -noverb -vec $maxFn -x X_AXIS $x1 $xn -y Y_AXIS $y1 $yn`)
else if ($dimCount == 3) then
   set val = (`readROI -ndim 3 -in $inName -$quadType -silent -noverb -vec $maxFn -x X_AXIS $x1 $xn -y Y_AXIS $y1 $yn -z Z_AXIS $z1 $zn`)
else
   set val = (`readROI -ndim 4 -in $inName -$quadType -silent -noverb -vec $maxFn -x X_AXIS $x1 $xn -y Y_AXIS $y1 $yn -z Z_AXIS $z1 $zn -a A_AXIS $a1 $an`)
endif

if (`string same $inName $outName`) then
   set ipArg = "-inPlace"
else
   set ipArg = ""
endif

if ($fmtCount == 0) then
   nmrPipe -in $inName -fn MULT -c $val -inv | nmrPipe -fn MULT -c $maxVal -out $outName $ipArg -ov
else
   xyz2pipe -in $inName | nmrPipe -fn MULT -c $val -inv | nmrPipe -fn MULT -c $maxVal | pipe2xyz -out $outName $ipArg -ov
endif

if ($hdrFlag) then
   set s = (`MATH "div( $maxVal, $val)"`)

   sethdr $outName -scale $s
endif


exit 0
