#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Adjust Linewidth (LW) Values in Peak Table:"
    puts " -in   inName  \[test.tab\]   Input Peak Table."
    puts " -out  outName \[new.tab\]    Output Peak Table."
    puts " -var  varList \[XW ...\]     List of Linewidth Variables to Adjust."
    puts " -mult mList   \[0.0 ...\]    Scale Factor (M)."
    puts " -add  aList   \[1.0 ...\]    Linewidth Offset Values (A)."
    puts "Notes:"
    puts " New LW = M*LW + A."
    exit 1
   }

set inName  test.tab
set outName new.tab
set varList "XW  YW  ZW  AW"
set mList   "0.0 0.0 0.0 0.0"
set aList   "1.0 1.0 1.0 1.0"

getArgD $ARGV -in  inName
getArgD $ARGV -out outName

getListArgD $ARGV -var  varList
getListArgD $ARGV -mult mList
getListArgD $ARGV -add  aList

set pkTab [gdbRead table -in $inName]
set eList [gdbGetInfo $pkTab -parms entries]
set vList [gdbGetInfo $pkTab -parms varNames]

foreach e $eList \
   {
    set i 0

    foreach v $varList \
       {
        if {[lsearch -exact $vList $v] >= 0} \
           {
            set m  [lindex $mList $i]
            set a  [lindex $aList $i]
            set lw [gdbGet $e $v]

            gdbSet $e $v [expr $m*$lw + $a]
           }

        incr i
       }
   }

gdbWrite $pkTab $outName

exit 0
