
#
# Functions for drawing 1D spectra as circles etc for each point.

proc drawROISymbols args \
{
   set ARGV "drawSymbols $args"

   set thisROI  0
   set thisCell 0
   set thisWin  0

   set fhi       Auto 
   set lhi       Auto 
   set fgColor   white
   set fillColor None
   set symLW     1

   set symType   circle
   set symHigh   0
   set symWide   0

   if {[flagLoc $ARGV -help]} \
      {
       puts "Draw Individual Points of a 1D Region of Interest:"
       puts " -roi       roiID      ROI ID to Draw."
       puts " -cell      cellID     Cell ID for Drawing."
       puts " -win       winID      Window ID for Drawning."
       puts " -fhi       firstHi    Min Spectral Height (Value or % of Max)"
       puts " -lhi       lastHi     Max Spectra Height (Value or % of Max)"
       puts "Each Point in the 1D Spectrum Will Be Drawn as a Symbol:"
       puts " -symType   symType    Symbol Type (Circle Box Diamond Cross X)"
       puts " -symWide   symWidth   Symbol Width, Pixels."
       puts " -symHigh   symHeight  Symbol Height, Pixels."
       puts " -symSize   symSize    Symbol Size, Pixels (Sets both -symWide and -symHigh)."
       puts " -symLW     symLW      Symbol Line Width, Pixels."
       puts " -fg        fgColor    Symbol Color."
       puts " -fillColor fillColor  Symbol Fill Color, or Keyword None."
      }
 
   getArgD $ARGV -roi  thisROI
   getArgD $ARGV -cell thisCell
   getArgD $ARGV -win  thisWin

   getArgD $ARGV -fhi  fhi
   getArgD $ARGV -lhi  lhi

   getArgD $ARGV -symType symType
   getArgD $ARGV -symSize symWide
   getArgD $ARGV -symSize symHigh

   getArgD $ARGV -symHigh symHigh
   getArgD $ARGV -symWide symWide
   getArgD $ARGV -symLW   symLW

   getArgD $ARGV -fg        fgColor
   getArgD $ARGV -fillColor fillColor

#
#-------------------------------------------------------------------------------------------------

   set cellText "-cell $thisCell"

   if {$thisWin != 0} {append cellText " -win $thisWin"}

   set cellInfo [eval getCellInfo $cellText]

   if {[llength $cellInfo] < 3} {return 0}

   set x1 [lindex [lindex $cellInfo 1] 0]
   set xn [lindex [lindex $cellInfo 1] 1]

   set y1 [lindex [lindex $cellInfo 2] 0]
   set yn [lindex [lindex $cellInfo 2] 1]

   set rInfo [getROIInfo -roi $thisROI]

   set rdata [lindex $rInfo 0]
   set n     [lindex [lindex $rInfo 2] 0]

   set minVal [vMin $rdata $n]
   set maxVal [vMax $rdata $n]

   if {![strcasecmp $fhi Auto]} {set fhi $minVal}
   if {![strcasecmp $lhi Auto]} {set lhi $maxVal}

   set fhi [getPctHiVal $fhi $minVal $maxVal]
   set lhi [getPctHiVal $lhi $minVal $maxVal]

   set xRange [expr $n - 1]
   set yRange [expr $lhi - $fhi]

   if {$xRange == 0} {set xRange 1}
   if {$yRange == 0} {set yRange 1}

   for {set i 0} {$i < $n} {incr i} \
      {
       set v [getPtrVal $rdata $i]

       if {$v < $fhi} {set v $fhi}
       if {$v > $lhi} {set v $lhi}

       set xPix   [expr int(0.5 + $x1 + ($xn - $x1)*$i/$xRange)]
       set yPix   [expr int(0.5 + $y1 + ($lhi - $v)*($yn - $y1)/$yRange)]

       set xLeft   [expr $xPix - $symWide/2]
       set xRight  [expr $xLeft + $symWide - 1]

       set yTop    [expr $yPix - $symHigh/2]
       set yBottom [expr $yTop + $symHigh - 1]

       if {![strcasecmp $symType Circle]} \
          {
           if {[strcasecmp $fillColor None]} \
              {
               drawCircle -x $xLeft $xRight -y $yTop $yBottom -fg $fillColor -fill
              }

           drawCircle -x $xLeft $xRight -y $yTop $yBottom -fg $fgColor -lw $symLW -nofill
          } \
       elseif {![strcasecmp $symType Box] || ![strcasecmp $symType Rec] || ![strcasecmp $symType Rectangle]} \
          {
           if {[strcasecmp $fillColor None]} \
              {
               drawRec -x $xLeft $xRight -y $yTop $yBottom -fg $fillColor
              }

           drawBox -x $xLeft  $xLeft  -y $yBottom $yTop    -fg $fgColor -lw $symLW
           drawBox -x $xLeft  $xRight -y $yTop    $yTop    -fg $fgColor -lw $symLW
           drawBox -x $xRight $xRight -y $yTop    $yBottom -fg $fgColor -lw $symLW
           drawBox -x $xLeft  $xRight -y $yBottom $yBottom -fg $fgColor -lw $symLW
          } \
       elseif {![strcasecmp $symType Diamond]} \
          {
           if {[strcasecmp $fillColor None]} \
              {
               drawDiamond -x $xLeft $xRight -y $yTop $yBottom -fg $fillColor -fill
              }

           drawDiamond -x $xLeft $xRight -y $yTop $yBottom -fg $fgColor -lw $symLW
          } \
       elseif {![strcasecmp $symType Cross]} \
          {
           drawCross -x $xLeft $xRight -y $yTop $yBottom -fg $fgColor -lw $symLW
          } \
       elseif {![strcasecmp $symType X]} \
          {
           drawLine -x $xLeft $xRight -y $yTop $yBottom -fg $fgColor -lw $symLW
           drawLine -x $xLeft $xRight -y $yBottom $yTop -fg $fgColor -lw $symLW
          }
      }
}

proc getPctHiVal { thisVal thisMin thisMax } \
{
   if {[string first "%" $thisVal] < 0} {return $thisVal}

   set v      [string trim $thisVal "% "]
   set maxAbs [expr abs($thisMax)]
   set minAbs [expr abs($thisMin)]

   set maxVal $maxAbs

   if {$minAbs > $maxVal} {set maxAbs $maxVal}

   set thisHi [expr $maxAbs*$v/100.0]

   return $thisHi
}

