#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Rotate PDB Structure onto One or More PCS Alignment Frames."
    puts "Input Files:"
    puts " -pdb pdbName   \[ref.pdb\]         PDB Input File"
    puts " -in  dcInList  \[pcsCalc*.tab\]    PCS Input Tables"
    puts "PDB Output Files:"
    puts " -out outName \[rot.pdb\]   Structures Rotated Onto Alignment Frames."
    puts " -rel relName \[rel.pdb\]   Relative Alignment Frames on Structure."
    puts " -tmp tmpName \[tmp.pdb\]   Temporary Output Name."
    puts "Other Options:"
    puts " -tList  tIDList \[*\]     Tensors to Use."
    puts " -scale  s       \[1.0\]   Tensor Size Scale Factor." 
    puts " -axis                   Insert Axis Information (Default)."
    puts " -noaxis                 No Axis Information."
    puts " -halo                   Insert Halo Information."
    puts " -nohalo                 No Halo Information (Default)."
    puts " -ignore                 Ignore Segment Name Info."
    puts " -verb                   Verbose Mode ON (Default)."
    puts " -noverb                 Verbose Mode OFF."
    puts "Notes:"
    puts " PCS input tables must have DATA for tensor PCS_EULER and G_XYZ."
    exit
   }

set pcsList    [glob -nocomplain pcsCalc*.tab]
set pdbName    ref.pdb
set outName    rot.pdb
set relName    rel.pdb
set tmpName    tmp.pdb
set tList      *
set tScale     1.0
set axisFlag   1
set haloFlag   0
set ignoreFlag 0
set verbose    1

getArgD $ARGV -pdb   pdbName
getArgD $ARGV -out   outName
getArgD $ARGV -rel   relName
getArgD $ARGV -tmp   tmpName
getArgD $ARGV -scale tScale

getListArgD $ARGV -in    pcsList
getListArgD $ARGV -tList tList

if {[flagLoc $ARGV -axis]}     {set axisFlag 1}
if {[flagLoc $ARGV -noaxis]}   {set axisFlag 0}

if {[flagLoc $ARGV -halo]}     {set haloFlag 1}
if {[flagLoc $ARGV -nohalo]}   {set haloFlag 0}

if {[flagLoc $ARGV -ignore]}   {set ignoreFlag 1}
if {[flagLoc $ARGV -noignore]} {set ignoreFlag 0}

if {[flagLoc $ARGV -verb]}     {set verbose 1}
if {[flagLoc $ARGV -noverb]}   {set verbose 0}

#
#------------------------------------------------------------------

proc addTensorFrame { pdbSrc thisName frame gXYZ } \
{
   global axisFlag haloFlag tScale

   set error [catch {set outUnit [open $thisName a]}]

   set resID 0

   if {$error} \
      {
       puts stderr "Error opening PDB file $thisName"
       return
      }
 
   set pInfo [dynGetInfo -src $pdbSrc -extent -all]
   set i 0

   foreach v {x1 xn y1 yn z1 zn x0 y0 z0} \
      {
       set $v [lindex $pInfo $i]
       incr i
      }

   set xs [expr abs([lindex $gXYZ 0])]
   set ys [expr abs([lindex $gXYZ 1])]
   set zs [expr abs([lindex $gXYZ 2])]

   set xr [expr 0.5*($xn - $x1)]
   set yr [expr 0.5*($yn - $y1)]
   set zr [expr 0.5*($zn - $z1)]

   if {$xs == 0.0} \
      {
       set rr 1.0
      } \
   else \
      {
       set rr [expr 1.0/$zs]
      }

   set mr [getMax "$xr $yr $zr"]

   set xe [expr $tScale*1.5*$rr*$mr*$xs]
   set ye [expr $tScale*1.5*$rr*$mr*$ys]
   set ze [expr $tScale*1.5*$rr*$mr*$zs]

   set xq [expr $x0 + 1.0 + $xe]
   set yq [expr $y0 + 1.0 + $ye]
   set zq [expr $z0 + 1.0 + $ze]

   set str [dynGetPDBStr -resID $resID \
                         -resName XYZ -atomName CXYZ -chain $frame \
                         -x $x0 -y $y0 -z $z0]

   if {$axisFlag} \
      {
       puts $outUnit $str

       set resID 1

       for {set xc [expr $x0]} {$xc < $xq} {set xc [expr $xc+1.0]} \
          {
           set str [dynGetPDBStr -resID $resID \
                                 -resName X -atomName CX -chain $frame \
                                 -x $xc -y $y0 -z $z0]

           puts $outUnit $str
           incr resID
          }

       set resID 1

       for {set yc [expr $y0]} {$yc < $yq} {set yc [expr $yc+1.0]} \
          {
           set str [dynGetPDBStr -resID $resID \
                                 -resName Y -atomName CY -chain $frame \
                                 -x $x0 -y $yc -z $z0]

           puts $outUnit $str
           incr resID
          }

       set resID 1

       for {set zc [expr $z0]} {$zc < $zq} {set zc [expr $zc+1.0]} \
          {
           set str [dynGetPDBStr -resID $resID \
                                 -resName Z -atomName CZ -chain $frame \
                                 -x $x0 -y $y0 -z $zc]

           puts $outUnit $str
           incr resID
          }
      }

   if {$haloFlag} \
      {
       set resID 1

       for {set t 0} {$t <= 360} {incr t 1} \
          {
           set x [expr $x0 + $xe*cos(3.14159*$t/180.0)]
           set y [expr $y0 + $ye*sin(3.14159*$t/180.0)]
    
           set str [dynGetPDBStr -resID $resID \
                                 -resName XY -atomName EXY -chain $frame \
                                 -x $x -y $y -z $z0]

           puts $outUnit $str
           incr resID
          }

       set resID 1
    
       for {set t 0} {$t <= 360} {incr t 1} \
          {
           set x [expr $x0 + $xe*cos(3.14159*$t/180.0)]
           set z [expr $z0 + $ze*sin(3.14159*$t/180.0)]
    
           set str [dynGetPDBStr -resID $resID \
                                 -resName XZ -atomName EXZ -chain $frame \
                                 -x $x -y $y0 -z $z]

           puts $outUnit $str
           incr resID
          }

       set resID 1
    
       for {set t 0} {$t <= 360} {incr t 1} \
          {
           set y [expr $x0 + $ye*cos(3.14159*$t/180.0)]
           set z [expr $z0 + $ze*sin(3.14159*$t/180.0)]
    
           set str [dynGetPDBStr -resID $resID \
                                 -resName YZ -atomName EYZ -chain $frame \
                                 -x $x0 -y $y -z $z]
    
           puts $outUnit $str
           incr resID
          }
      }

   close $outUnit
}

#
#------------------------------------------------------------------

set segList    {A B C D E F G H I J K}
set frameList  {L M N O P Q R S T U V}
set eulerList  {{0.0 0.0 0.0}}

dynInit -fast

if {$ignoreFlag} \
   {
    dynSetChainMode -ignore
    dynSetSegMode   -ignore
   }

catch "file delete $outName"
catch "file delete $relName"

set refSrc [dynRead -pdb -in $pdbName]

dynTranslate -src $refSrc -center -all
dynWrite     -src $refSrc -out $relName

set i 0

foreach pcsName $pcsList \
   {
    set pcsTab [gdbRead table -in $pcsName]

    getKeyData $pcsTab dInfo

    foreach v {PARAMAGNETIC PCS_EULER G_XYZ} \
       { 
        if {![info exists dInfo($v)]} \
           {
            puts stderr "Error: No $v data is given in table $pcsName"
            exit
           }
       }

    set paraList  $dInfo(PARAMAGNETIC)
    set eulerList $dInfo(PCS_EULER)
    set gList     $dInfo(G_XYZ)
    set gCount    [llength $gList]
    set idList    ""

    if {![strcmp $tList *]} \
       {
        for {set jj 1} {$jj <= $gCount} {incr jj} {lappend idList $jj}
       } \
    else \
       {
        set idList $tList
       }

    for {set j 0} {$j < $gCount} {incr j} \
       {
        if {[lsearch -exact $idList [expr $j + 1]] < 0} continue

        set pdbSrc  [dynRead -pdb  -in $pdbName]

        set seg     [lindex $segList   $i]
        set frame   [lindex $frameList $i]

        set item    [lindex $dInfo(PARAMAGNETIC) $j] 
        set resID   [lindex $item 0]
        set resName [lindex $item 1]
        set atom    [lindex $item 2]
        set s       [lindex $item 3]

        if {$ignoreFlag || ![llength $s]} \
           {
            set atomLoc \
             [dynSelect -src $pdbSrc -atomName $atom -res $resID -list]
           } \
        else \
           {
            set atomLoc \
             [dynSelect -src $pdbSrc -atomName $atom -res $resID -seg $s -list]
           }

        set eulerInfo [lindex $eulerList $j]
        set gXYZ      [lrange [lindex $gList $j] 1 3]

        set psi   [lindex $eulerInfo 1]
        set theta [lindex $eulerInfo 2]
        set phi   [lindex $eulerInfo 3]

        set atomInfo [dynGetCoord -src $refSrc -atomEntry $atomLoc]

        set tx [lindex $atomInfo 0]
        set ty [lindex $atomInfo 1]
        set tz [lindex $atomInfo 2]

        set fmt "%s %s R: % 6.1f % 6.1f % 6.1f T: % 7.2f % 7.2f % 7.2f" 

        if {$verbose} \
           {
            puts stderr \
             [format $fmt $pcsName [lrange $item 0 3] \
                     $psi $theta $phi $tx $ty $tz]
           }

        dynSetChain -src $pdbSrc -chain $seg
        dynRotate   -src $pdbSrc -rx $psi -ry $theta -rz $phi -center -all
        dynWrite    -src $pdbSrc -pdb -out $tmpName

        catch "exec cat $tmpName | fgrep -v END >> $outName"
        catch "file delete $tmpName"

        addTensorFrame $pdbSrc $tmpName $frame $gXYZ

        dynFree $pdbSrc

        set psi   [expr -1*$psi]
        set theta [expr -1*$theta]
        set phi   [expr -1*$phi]

        set pdbSrc [dynRead -pdb -in $tmpName]

        dynRotate -src $pdbSrc -rx 0.0  -ry 0.0    -rz $phi -all
        dynRotate -src $pdbSrc -rx 0.0  -ry $theta -rz 0.0  -all
        dynRotate -src $pdbSrc -rx $psi -ry 0.0    -rz 0.0  -all

        dynTranslate -src $pdbSrc -tx $tx -ty $ty -tz $tz -all

        dynWrite -pdb -src $pdbSrc -out $tmpName

        catch "exec cat $tmpName | fgrep -v END >> $relName"
        catch "file delete $tmpName"

        dynFree $pdbSrc

        incr i
       }

    gdbFree $pcsTab 

    unset dInfo
   }

set pdbSrc [dynRead -pdb -in $outName]

addTensorFrame $pdbSrc $outName O "1.0 1.0 1.0" 

dynFree $pdbSrc

set pdbSrc [dynRead -pdb -in $outName]
set relSrc [dynRead -pdb -in $relName]

dynWrite -pdb -src $pdbSrc -out $outName -renumber
dynWrite -pdb -src $relSrc -out $relName -renumber

catch "file delete $tmpName"

exit
