#!/bin/csh

set thisVersion = "2023.059.19.10"

if (`flagLoc $argv -help`) then
   echo "SAND 1D Spectral Decomposition (Script Version $thisVersion)."
   echo "Y Wu, O Sanati, M Uchimiya, K Krishnamurthy, A Edison, and F Delaglio."
   echo "Options:"
   echo " -in          ftList  [test.ft1] List of NMRPipe-format 1D Spectra."
   echo " -sandDir     sandDir [Auto]     SAND Master Output Directory (Auto = sand0001 ...)."
   echo " -cores       nCores  [40]       Number of Processing Cores for Command-Line Processing." 
   echo "Analysis Region (pts ppm hz %):"
   echo " -x1          x1      [3%]       Start Location of Analysis Region."
   echo " -xn          xn      [97%]      End Location of Analysis Region."
   echo "Options for Creation of NMRPipe Simulated Data (Default = Use Value from Input Data):"
   echo " -lb          lb      [Default]  Exponential Broaden in Hz."
   echo " -p0          p0      [Default]  Zero Order Phase."
   echo " -p1          p1      [Default]  First Order Phase."
   echo " -rescale                        Rescale SAND Results (Default)."
   echo " -norescale                      No Rescale of SAND Results."
   echo " -sim                            Generate Simulations from SAND Results (Default)."
   echo " -nosim                          Do Not Generate Simulations from SAND Results."
   echo " -simAll                         Simulate Data for Each Individual Peak (Sets -sim)."
   echo "Executation Control:"
   echo " -nopresand                      Do Not Execute PRESAND."
   echo " -nosand                         Do Not Execute SAND."
   echo " -nosubmitsand                   Do Not Execute SUBNITSAND."
   echo " -noexec                         Same As: -nopresand  -nosand -nosubmitsand"
   echo " -noupdate                       Do Not Update Results After SAND Execution."
   echo "Options for PRESAND, SAND, and SUBMITSAND Scripts (Terminate with --):"
   echo " -presandArgs pArgs   [None]     Options for PRESAND."
   echo " -sandArgs    sArgs   [None]     Options for SAND."
   echo "Options for HTCONDOR High-Throughput Processing:"
   echo " -condor                         Use HTCONDOR Submission."
   echo " -condorArgs  sArgs   [None]     Options for SUBMITSAND."
   echo " -server      sList   [All]      List of CONDOR Servers, or Keyword All."
   echo "Locations of SAND Scripts:"
   echo " -presandCmnd    psCmnd  [PRESAND]     PRESAND Command."
   echo " -sandCmnd       sCmnd   [SAND]        SAND Command."
   echo " -submitsandCmnd ssCmnd  [SUBMITSAND]  SUBMITSAND Command."
   echo "Special Functions Used by SAND: (Use Only One with No Other Options):"
   echo " -matlab         mName   [Auto]        Auto: Find and Return a Version of MATLAB." 
   echo "Notes:"
   echo "  1. Basic use:             sand1D.com -in spike.ft1"
   echo "     Multiple spectra:      sand1D.com -in ft/*.ft1 -x1 9.5ppm -xn -0.5ppm"
   echo "     Distributed Computing: sand1D.com -in ft/*.ft1 -x1 9.5ppm -xn -0.5ppm -condor" 
   echo "  2. SAND typically takes many hours per spectrum."
   echo "  3. Input spectra should be prepared as follows for best use of sand1D.com:"
   echo "      * Acquire spectra with no first order phase correction when possible."
   echo "        Besides making autoprocessing more difficult, the need for first order"
   echo "        phase correction generally means there will also be baseline distortion."
   echo "      * When converting Bruker format data to NMRPipe-format, use options for"
   echo "        Digital Oversampling Correction: During Conversion (bruker –dmx)."
   echo "      * Use only exponential line broadening, no other window. SAND will model the"
   echo "        spectrum as-is, retaining any line broadening applied. The decay values in"
   echo "        the output tables produced by sand1D.com (sand0001/tab/*.tab and csv/*.csv)"
   echo "        will be reduced by the amount of exponential line broadening applied."
   echo "      * Zero fill exactly once and round to a power of 2."
   echo "      * Do not extract a region, keep the full spectral range."
   echo "      * Phase and baseline correct as well as possible."
   echo "      * Perform any needed chemical shift calibration before using SAND."
   echo "      * Reducing or blanking the solvent signal might improve quality and speed."
   echo "      * For speed, analyze the smallest spectral range (sand1D.com -x1 ... -xn ...)."
   echo "  4. When sand1D.com is invoked, it will automatically create a directory for the"
   echo "     various output files that will be produced. The directory name will have the"
   echo "     form sand0001 sand0002 etc."
   echo "  5. The SAND master output directory (sand0001 etc) will be deleted at the start if"
   echo "     already it exists."
   echo "  6. For every input spectrum, the SAND master directory will contain a subdirectory"
   echo "     for each input spectrum.  These subdirectories will contain copies of each input"
   echo "     spectrum, and the workspaces used directly by the SAND modules:"
   echo ""
   echo "        sand0001/sand_data/1/test.ft      Copy of Input Spectrum 1."
   echo "        sand0001/sand_data/1/SAND/        SAND Workspace Directory for Spectrum 1."
   echo "        sand0001/sand_data/2/test.ft      Copy of Input Spectrum 2."
   echo "        sand0001/sand_data/2/SAND/        SAND Workspace Directory for Spectrum 2."
   echo "        ..."
   echo ""
   echo "  7. When the SAND analysis completes, the directory will contain the following"
   echo "     subdirectories, each having one output file of analysis results for each input"
   echo "     spectrum:"
   echo ""
   echo "        sand0001/csv/*.csv        CSV-format Tables of PPM, Decay, and Amplitude."
   echo "        sand0001/tab/*.tab        NMRPipe-format Peak Tables."
   echo "        sand0001/simFID/*.fid     NMRPipe-format Time-domain Data."
   echo "        sand0001/sim/*.ft1        Corresponding NMRPipe-format Spectra."
   echo "        sand0001/ft/*.ft1         Copies of the Input Spectra."
   echo "        sand0001/src/*.txt        Original File Names of the Input Spectra."
   echo ""
   echo "  8. If the sand1D.com -simAll option is used, output will also include simulations for"
   echo "     each model peak. There will be subdirectories 1, 2, etc. for each input spectrum."
   echo "     Note\!: the -simAll option takes a lot of disk space, use only when needed:"
   echo ""
   echo "        sand0001/simAllFID/1/*.fid   Time-domain Data for Each Peak in Spectrum 1."
   echo "        sand0001/simAll/1/*.ft1      Corresponding Spectra of Each Peak in Spectrum 1."
   echo "        sand0001/simAllFID/2/*.fid   Time-domain Data for Each Peak in Spectrum 2."
   echo "        sand0001/simAll/2/*.ft1      Corresponding Spectra of Each Peak in Spectrum 2."
   echo "        ..."
   echo ""
   echo " 9. The current version of SAND produces and retains many intermediate files and"
   echo "    redundant copies of results. When SAND analysis is finished, the following"
   echo "    directories are not strictly needed:"
   echo ""
   echo "       sand0001/SAND_last_run"
   echo "       sand0001/sand_data/1/SAND/"
   echo "       sand0001/sand_data/2/SAND/"
   echo "       ..."
   exit 0
endif

if (`flagLoc $argv -matlab`) then
   set matlabExe = (`getArgD $argv -matlab Auto`)

   if (`string csame $matlabExe "Auto"` || `string csame $matlabExe "-matlab"`) then
      if (-e /usr/software/MATLAB/R2022a/bin/matlab) then
         echo "/usr/software/MATLAB/R2022a/bin/matlab"
         exit 0
      else if (-e /usr/software/MATLAB/R2022b/bin/matlab) then
         echo "/usr/software/MATLAB/R2022b/bin/matlab"
         exit 0
      else
         set mList = (`find /usr/software/MATLAB -maxdepth 1 -name R2\* -print | sort -r`)

         foreach m ($mList)
            if (-e $mList[1]/bin/matlab) then
               echo $m/bin/matlab
               exit 0
            endif
         end

         echo "None"
      endif
   else
      echo "$matlabExe"
      exit 0
   endif
endif

set ftList         = (`getListArgD  $argv -in -- test.ft1`)
set serverList     = (`getListArgD  $argv -in -- All`)

set sandCmnd       = (`getArgD $argv -sandCmnd       SAND`)
set presandCmnd    = (`getArgD $argv -presandCmnd    PRESAND`)
set submitsandCmnd = (`getArgD $argv -submitsandCmnd SUBMITSAND`)

set presandArgs    = (`getListArgDN $argv -presandArgs -- None`)
set sandArgs       = (`getListArgDN $argv -sandArgs    -- None`)
set condorArgs     = (`getListArgDN $argv -condorArgs  -- None`)

set sandDir   = (`getArgD $argv -sandDir Auto`)
set x1        = (`getArgD $argv -x1      0%`)
set xn        = (`getArgD $argv -xn      100%`)

set lbVal     = (`getArgD $argv -lb      Default`)
set p0Val     = (`getArgD $argv -p0      Default`)
set p1Val     = (`getArgD $argv -p1      Default`)

set coreCount = (`getArgD $argv -cores   40`)

@ simFlag     = 1
@ updateFlag  = 1
@ condorFlag  = 0
@ rescaleFlag = 1

@ execPRESANDFlag    = 1
@ execSANDFlag       = 1
@ execSUBMITSANDFlag = 1

if (`flagLoc $argv -nopresand`) then
   @ execPRESANDFlag = 0
endif

if (`flagLoc $argv -nosand`) then
   @ execSANDFlag = 0
endif

if (`flagLoc $argv -nosubmitsand`) then
   @ execSUBMITSANDFlag = 0
endif

if (`flagLoc $argv -noexec`) then
   @ execPRESANDFlag    = 0
   @ execSANDFlag       = 0
   @ execSUBMITSANDFlag = 0
endif

if (`flagLoc $argv -sim`) then
   @ simFlag = 1
endif

if (`flagLoc $argv -nosim`) then
   @ simFlag = 0
endif

if (`flagLoc $argv -simAll`) then
   @ simFlag = 2
endif

if (`flagLoc $argv -nosimAll`) then
   @ simFlag = 0
endif

if (`flagLoc $argv -condor`) then
   @ condorFlag = 1
endif

if (`flagLoc $argv -nocondor`) then
   @ condorFlag = 0
endif

if (`flagLoc $argv -rescale`) then
   @ rescaleFlag = 1
endif

if (`flagLoc $argv -noscale`) then
   @ rescaleFlag = 0
endif

if (`flagLoc $argv -update`) then
   @ updateFlag = 1
endif

if (`flagLoc $argv -noupdate`) then
   @ updateFlag = 0
endif

if ($condorFlag) then
   @ updateFlag = 0
endif

#
#----------------------------------------------------------------------------------------

set origDir = (`pwd`)

if (`string csame $sandDir Auto`) then
   @ i = 1

   while ($i < 1000)
      set thisDir = (`nmrPrintf sand%04d $i`)

      if (!(-e $thisDir)) then
         set sandDir = $thisDir
         break
      endif

      @ i++
   end
endif

if (`string csame $sandDir Auto`) then
   echo "sand1D. Error finding available output directory for -sandDir Auto"
   exit 1
endif

/bin/rm -rf $sandDir
mkdir $sandDir $sandDir/sand_data $sandDir/src

cd $sandDir
   set sandDirFull = (`pwd`)

   cd sand_data
      set sandWorkDirFull = (`pwd`)
   cd ../
cd $origDir

set logName = $sandDirFull/sand.log

foreach ftName ($ftList)
   if (!(-e $ftName)) then
      echo "sand1D. Error finding input spectrum $ftName"
      exit 1
   endif
end

set ftName = $ftList[1]

set x1Pts  = (`spec2pnt $ftName CUR_XDIM $x1`)
set xnPts  = (`spec2pnt $ftName CUR_XDIM $xn`)

if (`IMATH "$x1Pts > $xnPts"`) then
   set xTmp  = $x1Pts
   set x1Pts = $xnPts
   set xnPts = $xTmp

   set xTmp = $x1
   set x1   = $xn
   set xn   = $xTmp
endif

set x1PPM  = (`pnt2spec $ftName CUR_XDIM $x1Pts ppm`)
set xnPPM  = (`pnt2spec $ftName CUR_XDIM $xnPts ppm`)

set x1PPM  = (`nmrPrintf %.4f $x1PPM`)
set xnPPM  = (`nmrPrintf %.4f $xnPPM`)

echo "sand1D. Script Version:        $thisVersion"
echo "sand1D. SAND Master Directory: $sandDirFull"
echo "sand1D. SAND Input Directory:  $sandWorkDirFull"
echo "sand1D. PRESAND Options:       $presandArgs"
echo "sand1D. SAND Options:          $sandArgs"
echo "sand1D. SUBMITSAND Options:    $condorArgs"
echo "sand1D. Analysis Region:       $x1 $xn" 
echo "sand1D. Analysis Region PPM:   $x1PPM $xnPPM" 
echo "sand1D. PRESAND Command:       $presandCmnd"
echo "sand1D. SAND Command:          $sandCmnd"
echo "sand1D. SUBMITSAND Command:    $submitsandCmnd"
echo "sand1D."
echo "sand1D. Spectra:"

@ i = 0

foreach ftName ($ftList)
   @ i++
   set srcName  = (`nmrPrintf "%04d.txt" $i`)
   set thisName = (`string joinpath $origDir $ftName`)

   mkdir $sandWorkDirFull/$i

   /bin/cp $ftName $sandWorkDirFull/$i/test.ft

   echo $thisName > $sandDirFull/src/$srcName
   echo "sand1D.    $thisName $sandWorkDirFull/$i"
end

if (`string csame $presandArgs None`) then
   set presandArgs = ""
endif

if (`string csame $sandArgs None`) then
   set sandArgs = ""
endif

if (`string csame $condorArgs None`) then
   set condorArgs = ""
endif

echo "sand1D."

cd $sandDir
   echo "sand1D. Starting on `date`"
   echo "sand1D. Working Directory: `pwd`"

   echo ""                                         >  $logName
   echo "sand1D. sand1D.com Version: $thisVersion" >> $logName
   echo "sand1D. Start Directory:    $origDir"     >> $logName
   echo "sand1D. SAND Directory:     `pwd`"        >> $logName
   echo "sand1D. Command:            $0 $argv"     >> $logName
   echo "sand1D  Start Date:         `date`"       >> $logName

   echo "-------"
   echo "sand1D. To terminate this script, use: killShell sand1D.com"
   echo "sand1D. $presandCmnd -I sand_data -u $x1PPM -l $xnPPM $presandArgs"

   echo "-------" >> $logName
   echo "sand1D. $presandCmnd -I sand_data -u $x1PPM -l $xnPPM $presandArgs" >> $logName
   
   if ($execPRESANDFlag) then
      $presandCmnd -I sand_data -u $x1PPM -l $xnPPM $presandArgs |& tee -a $logName
      echo ""

      @ i = 0

      echo ""
      echo "" >> $logName

      foreach ftName ($ftList)
         @ i++

         set maskList    = (`find sand_data/$i/SAND/last_preprocess/mask_ft  -name \*.ft1 -print`)
         set maskFIDList = (`find sand_data/$i/SAND/last_preprocess/mask_fid -name \*.fid -print`)
         set maskFTList  = (`find sand_data/$i/SAND/last_preprocess/mask     -name \*.ft1 -print`)
         
         if ($#maskList == 0 || $#maskList != $#maskFIDList || $#maskList != $#maskFTList) then
            echo ""
            echo "sand1D. Error: PRESAND did not create input files for spectrum $i."
            echo "sand1D. See the log files in $sandDirFull/SAND_last_run/log/ for more."

            echo "sand1D. Error: PRESAND did not create input files for spectrum $i."     >> $logName
            echo "sand1D. See the log files in $sandDirFull/SAND_last_run/log/ for more." >> $logName
            exit 1
         else
            nmrPrintf "sand1D. Spectrum: %3d Bands: %3d\n" $i $#maskList
            nmrPrintf "sand1D. Spectrum: %3d Bands: %3d\n" $i $#maskList >> $logName
         endif
      end

      echo ""
      echo "" >> $logName
   else
      echo "sand1D. (Skipping PRESAND execution (Option -nopresand/-noexec)."
      echo "sand1D. (Skipping PRESAND execution (Option -nopresand/-noexec)." >> $logName
   endif

   if ($simFlag == 1) then
      set thisCmnd = "sand1DUpdate.com -sandDir $sandDirFull -lb $lbVal -p0 $p0Val -p1 $p1Val"
   else if ($simFlag == 2) then
      set thisCmnd = "sand1DUpdate.com -sandDir $sandDirFull -lb $lbVal -p0 $p0Val -p1 $p1Val -simAll"
   else
      set thisCmnd = "sand1DUpdate.com -sandDir $sandDirFull -lb $lbVal -p0 $p0Val -p1 $p1Val -nosim"
   endif

   if ($rescaleFlag) then
      set thisCmnd = "$thisCmnd -rescale"
   endif

   if ($updateFlag) then
      set uScript = pipeUpdate.com
   else
      set uScript = user_pipeUpdate.com
   endif

   echo "#\!/bin/csh"    >  $sandDirFull/$uScript
   echo ""               >> $sandDirFull/$uScript
   echo "$thisCmnd" '$*' >> $sandDirFull/$uScript

   chmod a+rx $sandDirFull/$uScript

   if ($condorFlag) then
      echo "-------"
      echo "sand1D.   $submitsandCmnd -I sand_data -O sand_output -a 1 -b $#ftList -z $coreCount -x all $condorArgs"

      echo "-------" >> $logName
      echo "sand1D.   $submitsandCmnd -I sand_data -O sand_output -a 1 -b $#ftList -z $coreCount -x all $condorArgs" >> $logName

      if ($execSUBMITSANDFlag) then
         $submitsandCmnd -I sand_data -O sand_output -a 1 -b $#ftList -z $coreCount -x all $condorArgs |& tee -a $logName
         echo ""
      else
         echo "sand1D. (Skipping SUBMITSAND execution (Option -nosubmitsand/-noexec)."
         echo "sand1D. (Skipping SUBMITSAND execution (Option -nosubmitsand/-noexec)." >> $logName
      endif
   else
      echo "-------"
      echo "sand1D. To terminate this script, use: killShell sand1D.com"
      echo "sand1D. $sandCmnd -I sand_data -O sand_output $sandArgs"

      echo "-------" >> $logName
      echo "sand1D. $sandCmnd -I sand_data -O sand_output $sandArgs" >> $logName

      if ($execSANDFlag) then
         SAND    -I sand_data -O sand_output $sandArgs |& tee -a $logName
         echo ""
      else
         echo "sand1D. (Skipping SAND execution (Option -nosand/-noexec)."
         echo "sand1D. (Skipping SAND execution (Option -nosand/-noexec)." >> $logName
      endif
   endif

   if ($execSANDFlag == 1 && $condorFlag == 0) then
      @ i = 0

      foreach ftName ($ftList)
         @ i++

         if (!(-e sand_data/$i/SAND/sand_output/deconvoluted_peaks_refined.txt)) then
            echo ""
            echo "sand1D. Error: SAND did not create an output table for spectrum $i."
            echo "sand1D. See the log files in $sandDirFull/SAND_last_run/log/ for more."

            echo "sand1D. Error: SAND did not create an output table for spectrum $i."    >> $logName
            echo "sand1D. See the log files in $sandDirFull/SAND_last_run/log/ for more." >> $logName
            exit 1
         endif
      end
   endif

   echo "-------"
   echo "-------" >> $logName
cd $origDir

echo "sand1D. Done on `date`"

if (-e $sandDirFull/$logName) then
   echo "sand1D  End Date:   `date`" >> $logName
endif

exit 0

