#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]


if {[flagLoc $ARGV -help]} \
   {
    puts stderr "Find Scale Factor f to Minimize Spectrum1 - f*Spectrum2:"
    puts stderr " -in1    inName1  \[A.dat\]  Input Spectrum 1."
    puts stderr " -in2    inName2  \[B.dat\]  Input Spectrum 2." 
    puts stderr " -mask   maskName \[None\]   Mask Spectrum."
    puts stderr " -ndim   dimCount \[Auto\]   Dimension Count."
    puts stderr " -verb                     Verbose Mode ON (Default)."
    puts stderr " -noverb                   Verbose Mode OFF."
    puts stderr "Largest Allowable Negative Difference:"
    puts stderr " -difLim limPct   \[None\]   Allowed Negative Difference, Pct."
    puts stderr " -iter   itMax    \[100\]    Maximum Iterations."
    puts stderr " -pos                      Allow Only Positive Scale."
    puts stderr "Region to Use, -ix -iy etc:"
    puts stderr " -ix1     x1      \[0%\]     Start Coord of Region."
    puts stderr " -ixn     xn      \[100%\]   Start Coord of Region."
    puts stderr "Adaptive Scaling Options:"
    puts stderr " -thresh1 t1    \[0.0\]      Threshold for Spectrum 1."
    puts stderr " -thresh2 t2    \[0.0\]      Threshold for Spectrum 2."
    puts stderr " -window        \[100%\]     +/- Width of Region." 
    puts stderr " -step          \[100%\]     Step Between Regions."
    puts stderr " -out outName   \[None\]     Scaling Vector Output."
    exit 1
   }

set verbose   1
set inName1   A.dat
set inName2   B.dat
set maskName  None
set outName   None
set dimCount  Auto
set window    100%
set step      100%
set defVal    0.0
set thresh1   0.0
set thresh2   0.0
set difLim    None
set maxIter   100
set posFlag   0

set ix1       0%
set ixn       100%

set iy1       0%
set iyn       100%

getArgD $ARGV -in1     inName1
getArgD $ARGV -in2     inName2
getArgD $ARGV -out     outName
getArgD $ARGV -mask    maskName
getArgD $ARGV -ndim    dimCount
getArgD $ARGV -window  window
getArgD $ARGV -step    step
getArgD $ARGV -thresh  thresh1
getArgD $ARGV -thresh  thresh2
getArgD $ARGV -thresh1 thresh1
getArgD $ARGV -thresh2 thresh2
getArgD $ARGV -def     defVal
getArgD $ARGV -difLim  difLim
getArgD $ARGV -iter    maxIter

getArgD $ARGV -ix1     ix1
getArgD $ARGV -ixn     ixn
getArgD $ARGV -iy1     iy1
getArgD $ARGV -iyn     iyn

if {[flagLoc $ARGV -pos]}    {set posFlag 1}
if {[flagLoc $ARGV -nopos]}  {set posFlag 0}

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

#
#----------------------------------------------------------------------

proc applyDifLim { rOrig pctLim rA rB rW n } \
{
    global maxIter

    set r $rOrig

    if {![strcasecmp $pctLim None]} {return $r}
    if {$pctLim <= 0.0 || $maxIter < 2} {return $r}

    set maxVal [vMax $rA $n]
    if {$maxVal <= 0.0} {return $r}

    for {set i 0} {$i < $maxIter} {incr i} \
       {
        set r [expr $rOrig*($maxIter - $i)/double($maxIter)]

        vvMove $rW $rB $n
        vsMult $rW [expr -1.0*$r] $n
        vvAdd  $rW $rA $n

        set minVal [vMin $rW $n]
        if {$minVal >= 0.0} break

        set thisPct [expr 100.0*abs($minVal/$maxVal)]

        if {$thisPct <= 0.0} break 

        if {$thisPct <= $pctLim} break
       }

    return $r 
}

if {![file exists $inName1] || ![file exists $inName2]} \
   {
    puts stderr "Scale Error Finding Input Files $inName1 or $inName2"
    exit 1
   }

set error [rdFDATA $inName1 fdata]

if {$error} \
   {
    puts stderr "Scale Error Reading Header from $inName1"
    exit 1
   }

if {![strcasecmp $dimCount Auto]} \
   {
    set dimCount [getParmI $fdata FDDIMCOUNT NULL_DIM]
   }

if {$dimCount > 2} {set dimCount 2}

set rdataA 0
set rdataB 0
set rdataM 0
set rdataO 0
set rdataW 0

readROI -ndim $dimCount -roi 1 -in $inName1 -x X_AXIS $ix1 $ixn
readROI -ndim $dimCount -roi 2 -in $inName2 -x X_AXIS $ix1 $ixn

set roiInfo   [getROIInfo -roi 1]
set rdataA    [lindex $roiInfo 0]
set sizeListA [lindex $roiInfo 2]
set xSizeA    [lindex $sizeListA 0]
set nA        [getROISize 1]

set roiInfo   [getROIInfo -roi 2]
set rdataB    [lindex $roiInfo 0]
set sizeListB [lindex $roiInfo 2]
set xSizeB    [lindex $sizeListB 0]
set nB        [getROISize 2]

if {$dimCount == 1} \
   {
    set ySizeA 1
    set ySizeB 1

    set xSize  [getMin "$xSizeA $xSizeB"]
    set ySize  1
    set n      $xSize
   } \
else \
   {
    set ySizeA [lindex $sizeListA 1]
    set ySizeB [lindex $sizeListB 1]

    if {$xSizeA != $xSizeB || $ySizeA != $ySizeB} \
       {
        puts stderr "Scale Error: Mismatched 2D Sizes $inName1 ($xSizeA $ySizeA) $inName2 ($xSizeB $ySizeB)"
        exit 1
       }

    set xSize    $xSizeA
    set ySize    $ySizeA
    set n        $nA
   }

if {[strcasecmp $outName None]} \
   {
    readROI -ndim $dimCount -roi 3 -in $inName1 -x X_AXIS $ix1 $ixn

    set roiInfo  [getROIInfo -roi 3]
    set rdataO   [lindex $roiInfo 0]
   }

if {[strcasecmp $difLim None]} \
   {
    readROI -ndim $dimCount -roi 4 -in $inName1 -x X_AXIS $ix1 $ixn

    set roiInfo  [getROIInfo -roi 4]
    set rdataW   [lindex $roiInfo 0]
   }

if {![strcasecmp $window 100%]} \
   {
    set wx $xSize
   } \
else \
   {
    set wx [spec2pntW $fdata CUR_XDIM $window]
    set wx [expr int($wx)]
   }

if {![strcasecmp $step 100%]} \
   {
    set dx $xSize
   } \
elseif {![strcasecmp $step 1]} \
   {
    set dx 1
   } \
else \
   {
    set dx [spec2pntW $fdata CUR_XDIM $step]
    set dx [expr int($dx)]
   }

if {$wx > $xSize} {set wx $xSize}
if {$dx > $xSize} {set dx $xSize}

if {$wx < 1}  {set wx 1}
if {$dx < 1}  {set dx 1}

set wx2 [expr $wx/2]

if {$wx2 < 1} {set wx2 1}

if {[strcasecmp $maskName None]} \
   {
    readROI -ndim $dimCount -roi 5 -in $maskName

    set roiInfo   [getROIInfo -roi 5]
    set rdataM    [lindex $roiInfo 0]
    set sizeListM [lindex $roiInfo 2]
    set xSizeM    [lindex $sizeListM 0]
    set nM        [getROISize 5]

    if {$dimCount == 1} \
       {
        set ySizeM 1
        set nM     [getMin "$n $nM"]
       } \
    else \
       {
        set ySizeM [lindex $sizeListM 1]

        if {$xSizeA != $xSizeM || $ySizeA != $ySizeM} \
           {
            puts stderr "Scale Error: Mismatched 2D Sizes $inName1 ($xSizeA $ySizeA) $maskName ($xSizeM $ySizeM)"
            exit 1
           }
       }

    vvMult $rdataA $rdataM $nM
    vvMult $rdataB $rdataM $nM
   }

if {[strcmp $rdataO 0]} {vsSet $rdataO 0.0 $n}

switch $dimCount \
   {
    1 {
       set wn2 [expr 2*$wx2 + 1]
       set wn  [expr 2*$wx  + 1]

       if {$wn > $xSize} {set wn $xSize}

       for {set ix 0} {$ix < $xSize} {incr ix $dx} \
          {
           set kx [expr $ix - $wx]

           rClip kx 0 [expr $xSize - $wn]

           set ptrA [incrPtr $rdataA $kx]
           set ptrB [incrPtr $rdataB $kx]

           set top  [vvDot $ptrA $ptrB $wn]
           set bot  [vvDot $ptrB $ptrB $wn]

           if {$bot == 0.0} \
              {
               set r $defVal 
              } \
           else \
              {
               set r [expr $top/$bot]
              }

           if {$thresh1 != 0.0 || $thresh2 != 0.0} \
              {
               set ptrA [incrPtr $ptrA $wx2]
               set ptrB [incrPtr $ptrB $wx2]

               set rmsA [vRMS $ptrA $wn2]
               set rmsB [vRMS $ptrB $wn2]

#              if {$rmsA < $thresh1 || $rmsB < $thresh2} {set r $defVal}
               if {$rmsB < $thresh2} {set r $defVal}
              }

           set r [applyDifLim $r $difLim $ptrA $ptrB $rdataW $wn]
           if {$posFlag && $r < 0.0} {set r 0.0}

           if {[strcmp $rdataO 0]} {setPtrVal $rdataO $ix $r}

           if {$verbose} {puts $r}
          }
      }

   default {
            set top  [vvDot $rdataA $rdataB $n]
            set bot  [vvDot $rdataB $rdataB $n]

            if {$bot == 0.0} \
               {
                set r $defVal
               } \
            else \
               {
                set r [expr $top/$bot]
               }

            set r [applyDifLim $r $difLim $rdataA $rdataB $rdataW $wn]
            if {$posFlag && $r < 0.0} {set r 0.0}

            if {$verbose} {puts $r}
           }
   }

if {[strcasecmp $outName None]} {writeROI -roi 3 -out $outName -ov}

exit 0
