#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Find and Apply Global Scaling to a Series of 1D Spectra:"
   echo " -in     inList [ft/*.ft1]   List of Input Spectra"
   echo " -x1     x1     [55%]        X-Axis Start of Statistic Region."
   echo " -xn     xn     [95%]        X-Axis End of Statistic Region."
   echo " -stat   stat   [vMaxAbs]    Statistic Function (Usually vMaxAbs or vSum)."
   echo " -max    maxVal [100.0]      Desired Global Max for Statistic."
   echo " -verb                       Verbose Mode ON (Default)."
   echo " -noverb                     Verbose Mode OFF."
   exit 0
endif

if (-d ft) then
   set thisList = (`find ft -name \*.ft1 -print`)

   if ($#thisList != 0) then
      set thisList = (ft/*.ft1)
   else
      set thisList = (test.ft1)
   endif
else
   set thisList = (test.ft1)
endif

set inList    = (`getListArgD $argv -in -- $thisList`)

set x1        = (`getArgD $argv -x1   55%`)
set xn        = (`getArgD $argv -xn   95%`)
set statFn    = (`getArgD $argv -stat vMaxAbs`)
set targetVal = (`getArgD $argv -max  100.0`)

@ vFlag = 1

if (`flagLoc $argv -verb`) then
   @ vFlag = 1
endif

if (`flagLoc $argv -noverb`) then
   @ vFlag = 0
endif

set maxList = ()

@ i = 0

if ($vFlag) then
   echo ""
   echo "Global Scaling:"
endif

foreach inName ($inList)
   @ i++

   if (!(-e $inName)) then
      echo "scale1DSeries. Error Finding Input $i $inName"
      exit 1
   endif

   set thisVal = (`specStat.com -in $inName -x1 $x1 -xn $xn -stat $statFn -brief`)
   set maxList = ($maxList $thisVal)

   if ($vFlag) then
      nmrPrintf " %3d. %.4e %s\n" $i $thisVal $inName $x1 $xn
   endif
end

set maxVal = (`getStat.tcl -brief -x $maxList -stat Max`)
set c      = (`MATH "div($targetVal,$maxVal)"`)

nmrPrintf "Global Max: %.4e Scale Factor: %.4e\n" $maxVal $c

foreach inName ($inList)
   nmrPipe -in $inName -out $inName -inPlace -fn MULT -c $c
end

exit 0
