#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set auto_path [linsert $auto_path 0 $env(NMRBASE)/dynamo/tcl]

set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
#-------------------------------------------------------------------------

set csName      csObs.tab
set surfName    scanCS/cs%03d.dat

set inPDBName   ext.pdb
set outPDBName  scanCS.pdb

if {[flagLoc $ARGV -help]} \
   {
    puts "Create Chemical Shift Deviation Surfaces:"
    puts " -inCS   \[csObs.tab\]          Chemical Shift Input Table."
    puts " -outCS  \[scanCS/cs%03d.dat\]  Surface Output Template."
    puts " -inPDB  \[ext.pdb\]            PDB File Input."
    puts " -outPDB \[scanCS.pdb\]         PDB File Output (Or None)."
    exit 0
   }

getArgD $ARGV -inCS   csName
getArgD $ARGV -outCS  surfName
getArgD $ARGV -inPDB  inPDBName
getArgD $ARGV -outPDB outPDBName

#
#-------------------------------------------------------------------------

dynInit

showDynLogo

set pdbSrc [dynRead -pdb -in $inPDBName]
set csSrc  [dynRead -cs  -in $csName]

set pInfo  [dynGetInfo -src $pdbSrc]
set minRes [lindex $pInfo 0]
set maxRes [lindex $pInfo 1]

for {set resID $minRes} {$resID <= $maxRes} {incr resID} \
   {
    wm title . "Residue $resID of $minRes to $maxRes"
    update

    set csInfo \
     [dynSimulate -cs $csSrc -src $pdbSrc -s1 $resID -sN $resID -a1 $resID \
                  -scan -out $surfName]

    set sList [lrange $csInfo 4 end]
    set sInfo [lindex $sList 0]

    set phi   [lindex $sInfo 1]
    set psi   [lindex $sInfo 2]
    set score [lindex $sInfo 3]
    set avg   [lindex $sInfo 4]
    set n     [lindex $sInfo 5]

    if {$phi > 999 || $psi > 999} continue

    set fmt "Res: %3d Phi: % 8.2f Psi: % 8.2f   Score: %6.2f Avg: %6.2f N: %s"
   
    puts [format $fmt $resID $phi $psi $score $avg $n]
 
    dynSetPhiPsi -src $pdbSrc -phi $phi -psi $psi -res $resID
   }

if {[strcasecmp $outPDBName None]} \
   {
    createPath $outPDBName

    dynWrite -pdb -src $pdbSrc -out $outPDBName

    puts "Created $outPDBName ..."
   }

exit 0
