#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -geom +265+170 -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

proc scanDir { d } \
{
   global currentDirCount currentFileCount

   incr currentDirCount

   updateDirGauge $d

   if {![strcmp $d "."]} \
      {
       set f "*"
      } \
   else \
      {
       set f "$d/*"
      }

   set fList [lsort -dictionary [glob -nocomplain $f]]

   foreach f $fList \
      {
       if {[file isdirectory $f]} \
          {
           testDir $f
           scanDir $f
          } \
       else \
          {
           incr currentFileCount
          }
      }
}

proc testDir { d } \
{
   if {[file exists $d/procpar]} \
      {
       foreach i {fid fid.Z fid.gz} \
          {
           if {[file exists $d/$i]} \
              {
               testData varian $d
               return 
              }
          }
      } \
   elseif {[file exists $d/acqus]} \
      {
       foreach i {ser ser.Z ser.gz fid fid.Z fid.gz} \
          {
           if {[file exists $d/$i]} \
              {
               testData bruker $d
               return
              }
          }
      } \
   else \
      {
       set fList [lsort -dictionary [glob -nocomplain $d/*.jdf $d/*.JDF]]

       if {[llength $fList]} \
          {
           testData delta $d
          } 
      }
}

proc testData { conv d } \
{
   global oldList currentDir tabFlag fastFlag id

   set error    0
   set dimCount 0
   set expName  None
   set cmnd     "set error 1"
   set status   New

   set xT 0
   set yT 0
   set zT 0
   set aT 0

   set xLAB None
   set yLAB None
   set zLAB None
   set aLAB None

   set xSW  0.0
   set ySW  0.0
   set zSW  0.0
   set aSW  0.0

   if {$fastFlag} \
      {
       if {[file exists $currentDir/$d/fid.com]} {set status Old}

       set xT 1
       set yT 1
       set zT 1
       set aT 1

       set dimCount 1

       if {[file isfile $currentDir/$d/ser]} \
          {
           set nBytes [file size $currentDir/$d/ser]
           set xT     [expr int($nBytes/4)]
          } \
       elseif {[file isfile $currentDir/$d/fid]} \
          {
           set nBytes [file size $currentDir/$d/fid]

           if {![strcasecmp $conv varian]} \
              {
               set xT [expr int(($nBytes - 60)/8)]
              } \
           else \
              {
               set xT [expr int($nBytes/8)]
              }
          }

       set xLAB     NMR 
       set expName  ND 
      } \
   else \
      {
       catch {set cmnd [exec $conv -list -auto -noverb -notk -cd $currentDir/$d]}

       eval $cmnd

       foreach f $oldList \
          {
           if {[file exists $d/$f]} \
              {
               set status Old
               break
              }
          }
      }

   set fmt " %4d %s %32s %10s %16s %8s %14s %s"
   set s   "${dimCount} $xT $yT $zT $aT"

   if {$tabFlag} \
      {
       puts [format $fmt $id $currentDir $d $conv $s $xLAB $expName $status]
      } \
   else \
      {
       puts "\{$id \"$currentDir\" \"$d\" $conv $s $xLAB $yLAB $zLAB $aLAB $xSW $ySW $zSW $aSW \"$expName\" $status\}"
      }
}

proc countDir { d } \
{
   global dirCount fileCount wishFlag

   incr dirCount

   if {$wishFlag && !($dirCount % 10)} update

   foreach f [lsort -dictionary [glob -nocomplain $d/*]] \
      {
       if {[file isdirectory $f]} \
          {
           countDir $f
          } \
       else \
          {
           incr fileCount
          }
      } 
}

proc updateDirGauge { f } \
{
   global wishFlag currentDirCount dirCount 
   global rHi rWide rx1 rxn ry1 ryn recID

   if {!$wishFlag} return

   if {$dirCount > 0} \
      {
       set rxn [expr int(($rWide - 2)*double($currentDirCount)/$dirCount)]
       .c coords $recID $rx1 $ry1 $rxn $ryn
      }

   wm title . "Scan $currentDirCount of $dirCount: [file normalize $f]"
   update
}

proc initDirGauge {} \
{
   global dirList wishFlag recID rx1 rx1 ry1 ryn rHi rWide

   if {!$wishFlag} return

   set rHi   20 
   set rWide 690

   set rx1 1
   set rxn 1
   set ry1 1
   set ryn [expr $rWide - 2] 

   canvas  .c -width $rWide -height $rHi
   button  .q -text "Cancel" -command cancelScan
   pack    .c .q -side left

   set recID \
    [.c create rectangle $rx1 $ry1 $rxn $ryn -outline gray -fill darkBlue]

   wm title . "Counting directories ..."
   update

   foreach dd $dirList \
      {
       foreach d $dd \
          {
           set thisDir [file normalize $d]
           if {![file isdirectory $thisDir]} continue
           countDir $thisDir
          }
      }
}

proc cancelScan {} \
{
   set msg "Scanning for NMR Files ..."

   set i \
    [tk_dialog .canScanDiag Scan $msg info 0 "Continue Scan" "Cancel Scan"]

   if {!$i} return

   exit 0
}

if {[flagLoc $ARGV -help]} \
   {
    puts stderr "Scan Directories for NMR Data:"
    puts stderr " -dir dirList \[.\] List of Directories to Scan."
    puts stderr " -tab             List Data in Table Mode."
    puts stderr " -hdr             Display Table Header."
    puts stderr " -old             List of Files Indicating OLD Data."
    puts stderr " -fast            Fast Scan Mode (Not Suitable for All Data)."
    exit 1
   }

set oldList \
 {test.dat test.ft1 test.ft2 ft/test001.dat ft/test001.ft3 ft/test01001.ft4}

set dirList  .
set tabFlag  0
set fastFlag 0
set id       0

set dirCount  0
set fileCount 0

set currentDirCount  0
set currentFileCount 0

getListArgD $ARGV -dir dirList
getListArgD $ARGV -old oldList

initDirGauge

if {[flagLoc $ARGV -fast]}   {set fastFlag 1} 
if {[flagLoc $ARGV -nofast]} {set fastFlag 0} 

if {[flagLoc $ARGV -tab]} {set tabFlag 1} 

if {[flagLoc $ARGV -hdr]} \
   {
    puts \
     "VARS   INDEX PATH DIRNAME TYPE DIMCOUNT XSIZE YSIZE ZSIZE ASIZE XLAB YLAB ZLAB ALAB XSW  YSW  ZSW  ASW  EXPNAME STATUS"

    puts \
     "FORMAT %4d   %s   %s      %s   %d       %d    %d    %d    %d    %s   %s   %s   %s   %.2f %.2f %.2f %.2f %s      %s"

    puts ""
   }

set origDir [pwd]

foreach dd $dirList \
   {
    foreach d $dd \
       {
        set currentDir [file normalize $d]
        if {![file isdirectory $currentDir]} continue

        cd $d
        scanDir . 
        cd $origDir
       }
   }

exit 0
