#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]


if {[flagLoc $ARGV -help]} \
   {
    puts stderr "Sumarize Statistics for c1*Spectrum1 - c2*Spectrum2:"
    puts stderr " -in1    inName1  \[A.dat\]  Input Spectrum 1."
    puts stderr " -in2    inName2  \[B.dat\]  Input Spectrum 2." 
    puts stderr " -c1     c1       \[1.0\]    Scale for Spectrum 1."
    puts stderr " -c2     c2       \[1.0\]    Scale for Spectrum 2." 
    puts stderr " -mask   maskName \[None\]   Mask Spectrum."
    puts stderr " -ndim   dimCount \[Auto\]   Dimension Count."
    puts stderr " -lab1   aLab     \[A\]      Label for Spectrum 1 Statistics."
    puts stderr " -lab2   bLab     \[B\]      Label for Spectrum 2 Statistics."
    puts stderr " -labD   abLab    \[AB\]     Label for Difference Statistics."
    puts stderr " -no1                      No Report for Spectrum 1 Statistics."
    puts stderr " -no2                      No Report for Spectrum 2 Statistics."
    puts stderr " -noD                      No Report for Difference Statistics."
    puts stderr " -noN                      No Report for Point Count."
 #  puts stderr " -verb                     Verbose Mode ON (Default)."
 #  puts stderr " -noverb                   Verbose Mode OFF."
    puts stderr "Region to Use, -ix -iy etc:"
    puts stderr " -ix1     x1      \[0%\]     Start Coord of Region."
    puts stderr " -ixn     xn      \[100%\]   Start Coord of Region."
    exit 1
   }

set verbose   1
set inName1   A.dat
set inName2   B.dat
set c1        1.0
set c2        1.0
set lab1      A
set lab2      B
set labD      AB
set maskName  None
set dimCount  Auto

set no1Flag   0
set no2Flag   0
set noDFlag   0
set noNFlag   0

set ix1       0%
set ixn       100%
set iy1       0%
set iyn       100%

getArgD $ARGV -in1     inName1
getArgD $ARGV -in2     inName2
getArgD $ARGV -c1      c1
getArgD $ARGV -c2      c2
getArgD $ARGV -lab1    lab1
getArgD $ARGV -lab2    lab2
getArgD $ARGV -labD    labD
getArgD $ARGV -mask    maskName
getArgD $ARGV -ndim    dimCount

getArgD $ARGV -ix1     ix1
getArgD $ARGV -ixn     ixn
getArgD $ARGV -iy1     iy1
getArgD $ARGV -iyn     iyn

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -no1]} {set no1Flag 1}
if {[flagLoc $ARGV -no2]} {set no2Flag 1}
if {[flagLoc $ARGV -noD]} {set noDFlag 1}
if {[flagLoc $ARGV -noN]} {set noNFlag 1}

#
#----------------------------------------------------------------------

if {![file exists $inName1] || ![file exists $inName2]} \
   {
    puts stderr "Score Error Finding Input Files $inName1 or $inName2"
    exit 1
   }

set error [rdFDATA $inName1 fdata]

if {$error} \
   {
    puts stderr "Score Error Reading Header from $inName1"
    exit 1
   }

if {![strcasecmp $dimCount Auto]} \
   {
    set dimCount [getParmI $fdata FDDIMCOUNT NULL_DIM]
   }

if {$dimCount > 2} {set dimCount 2}

set rdataA 0
set rdataB 0
set rdataM 0
set rdataW 0

readROI -ndim $dimCount -roi 1 -in $inName1 -x X_AXIS $ix1 $ixn
readROI -ndim $dimCount -roi 2 -in $inName2 -x X_AXIS $ix1 $ixn

set roiInfo   [getROIInfo -roi 1]
set rdataA    [lindex $roiInfo 0]
set fdataA    [lindex $roiInfo 1]
set sizeListA [lindex $roiInfo 2]
set xSizeA    [lindex $sizeListA 0]
set nA        [getROISize 1]

set roiInfo   [getROIInfo -roi 2]
set rdataB    [lindex $roiInfo 0]
set fdataB    [lindex $roiInfo 1]
set sizeListB [lindex $roiInfo 2]
set xSizeB    [lindex $sizeListB 0]
set nB        [getROISize 2]

if {$dimCount == 1} \
   {
    set ySizeA 1
    set ySizeB 1

    set xSize  [getMin "$xSizeA $xSizeB"]
    set ySize  1
    set n      $xSize
   } \
else \
   {
    set ySizeA [lindex $sizeListA 1]
    set ySizeB [lindex $sizeListB 1]

    if {$xSizeA != $xSizeB || $ySizeA != $ySizeB} \
       {
        puts stderr "Score Error: Mismatched 2D Sizes $inName1 ($xSizeA $ySizeA) $inName2 ($xSizeB $ySizeB)"
        exit 1
       }

    set xSize    $xSizeA
    set ySize    $ySizeA
    set n        $nA
   }

if {$n < 2} \
   {
    puts stderr "Score Error: Data Size too Small."
    exit 1
   }

readROI -ndim $dimCount -roi 3 -in $inName1 -x X_AXIS $ix1 $ixn

set roiInfo  [getROIInfo -roi 3]
set rdataW   [lindex $roiInfo 0]

if {[strcasecmp $maskName None]} \
   {
    readROI -ndim $dimCount -roi 4 -in $maskName

    set roiInfo   [getROIInfo -roi 4]
    set rdataM    [lindex $roiInfo 0]
    set sizeListM [lindex $roiInfo 2]
    set xSizeM    [lindex $sizeListM 0]
    set nM        [getROISize 4]

    if {$dimCount == 1} \
       {
        set ySizeM 1
        set nM     [getMin "$n $nM"]
       } \
    else \
       {
        set ySizeM [lindex $sizeListM 1]

        if {$xSizeA != $xSizeM || $ySizeA != $ySizeM} \
           {
            puts stderr "Score Error: Mismatched 2D Sizes $inName1 ($xSizeA $ySizeA) $maskName ($xSizeM $ySizeM)"
            exit 1
           }
       }

    vvMult $rdataA $rdataM $nM
    vvMult $rdataB $rdataM $nM
   }

set noiseA [expr $c1*[getParm $fdataA FDNOISE NULL_DIM]]
if {$noiseA <= 0.0} {set noiseA 1.0}

set noiseB [expr $c2*[getParm $fdataB FDNOISE NULL_DIM]]
if {$noiseB <= 0.0} {set noiseB 1.0}

vsMult $rdataA $c1 $n
vsMult $rdataB $c2 $n

vvSub  $rdataW $rdataA $rdataB $n

set minValA [vMin $rdataA $n]
set maxValA [vMax $rdataA $n]
set rmsA    [vRMS $rdataA $n]

set minValB [vMin $rdataB $n]
set maxValB [vMax $rdataB $n]
set rmsB    [vRMS $rdataB $n]

if {!$noNFlag} \
   {
    puts [format "%-26s %5d"   N  $n]
   }

if {!$no1Flag} \
   {
    puts [format "%-26s % .4e" MIN_$lab1   $minValA]
    puts [format "%-26s % .4e" MAX_$lab1   $maxValA]
    puts [format "%-26s % .4e" RMS_$lab1   $rmsA]
    puts [format "%-26s % .4e" NOISE_$lab1 $noiseA]
   }

if {!$no2Flag} \
   {
    puts [format "%-26s % .4e" MIN_$lab2   $minValB]
    puts [format "%-26s % .4e" MAX_$lab2   $maxValB]
    puts [format "%-26s % .4e" RMS_$lab2   $rmsB]
    puts [format "%-26s % .4e" NOISE_$lab2 $noiseB]
   }

if {$maxValA <= 0.0} {set maxValA 1.0}
if {$maxValB <= 0.0} {set maxValB 1.0}

if {$rmsA <= 0.0} {set rmsA 1.0}
if {$rmsB <= 0.0} {set rmsB 1.0}

set noiseAB [getMax "$noiseA $noiseB"]
set noiseABOrig $noiseAB

if {$noiseAB <= 0.0} {set noiseAB 1.0}

set rmsAB [vRMS $rdataW $n]

set dotAB [expr [vvDot $rdataA $rdataB $n]/double($n)]

set abPct [expr 100.0*$rmsAB/$rmsA]
if {$abPct <   0.0} {set abPct   0.0}
if {$abPct > 100.0} {set abPct 100.0}

set abEPct [expr 100.0 - $abPct]

if {!$noDFlag} \
   { 
    puts [format "%-26s % .4e"  MIN_$labD         [vMin $rdataW $n]]
    puts [format "%-26s % .4e"  MAX_$labD         [vMax $rdataW $n]]
    puts [format "%-26s % .4e"  RMSD_$labD        $rmsAB]
    puts [format "%-26s % .4e"  RMSD_${labD}_NORM [expr $rmsAB/$noiseAB]]
    puts [format "%-26s %11.4f" RMSD_${labD}_PCT  $abPct]
    puts [format "%-26s %11.4f" RMSD_${labD}_EPCT $abEPct]
    puts [format "%-26s % .4e"  DOT_${labD}       $dotAB]
    puts [format "%-26s % .4e"  NOISE_$labD       $noiseABOrig]
   }

exit 0
