#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Apply Scoring to Processed Blanks and Screen racks."
   echo " -blankDir  bRackDirs   Blanks_Rack_1_Pools"
   echo " -screenDir sRackDirs   Screen_Rack_1_Pools"
   echo " -rackID    rackIDs     1 ..."
   echo " -comName   cName       ext_score.com"
   echo " -verb                  Verbose Mode ON."
   echo " -noverb                Verbose Mode OFF (Default)."
   echo " -exec                  Execute Scoring Scripts (Default)."
   echo " -noexec                Create Scoring Scripts, Do Not Execute."
   echo "Arguments for scorePool.com, Should be Last on the Command Line:"
   echo " -scoreArgs scoreArgs ...  (See scorePool.com -help)"
   exit 0
endif

set prefix = ext
set vFlag  = 0
set eFlag  = 1

if (`flagLoc $argv -verb`) then
   set vFlag = 1
endif

if (`flagLoc $argv -noverb`) then
   set vFlag = 0
endif

if (`flagLoc $argv -exec`) then
   set eFlag = 1
endif

if (`flagLoc $argv -noexec`) then
   set eFlag = 0
endif

set blankDirList  = (`getListArgD $argv -blankDir  -- Blanks_Rack_1_Pools`)
set screenDirList = (`getListArgD $argv -screenDir -- Screen_Rack_1_Pools`)
set rackIDList    = (`getListArgD $argv -rackID    -- None`)

if (`string csame $rackIDList[1] None`) then
   set rackIDList = ""
   set n = $#screenDirList
   set i = 0

   while ($i < $n)
      @ i++
      set rackIDList = ($rackIDList $i)
   end
endif


set prefix    = (`getArgD $argv -prefix  ext`)
set thisName  = ${prefix}_score.com

set comName   = (`getArgD $argv -comName $thisName`)
set loc       = (`flagLoc $argv -scoreArgs`)
set scoreArgs = ""

if ($loc) then
   @ loc++

   while ($loc <= $#argv)
      set scoreArgs = ($scoreArgs $argv[$loc])
      @ loc++
   end
endif

#
#------------------------------------------------------------------------------

set origDir = (`pwd`)
set nGood   = 0
set nBad    = 0
set nRack   = 0

set fullBlankDirList  = ""
set fullScreenDirList = ""

if ($#blankDirList != $#screenDirList) then
   echo "ScoreAllPools Error: blanks and screen rack mismatch."
   exit 1
endif

if ($#rackIDList != $#screenDirList) then
   echo "ScoreAllPools Error: screen and rack ID mismatch."
   exit 1
endif

foreach blankDir ($blankDirList)
   cd $origDir

   if (!(-d $blankDir)) then
      echo "ScoreAllPools Error Finding blanks dir $blankDir"
      exit 1
   endif

   cd $blankDir

   set fullBlankDirList = ($fullBlankDirList `pwd`)
end

foreach screenDir ($screenDirList)
   cd $origDir

   if (!(-d $blankDir)) then
      echo "ScoreAllPools Error Finding screen dir $screenDir"
      exit 1
   endif

   cd $screenDir

   set fullScreenDirList = ($fullScreenDirList `pwd`)
end

set i  = 1
set ii = 0

foreach blankDir ($fullBlankDirList)
   cd $origDir
   @ ii++

   set screenDir = $fullScreenDirList[$i]

   if (!(-d $blankDir) || !(-d $screenDir)) then
      continue
   endif

   cd $screenDir
  
   set parentDir = (`pwd`)
   set fCount    = (`/bin/ls | wc -w`)

   if ($fCount == 0) then
      continue
   endif

   set fList = (`/bin/ls | sort -n`)
   set nExp  = 0

   foreach fName ($fList)
      if (!(-d $fName)) then
         continue
      endif

      if (!(-e $fName/test.ft1) || !(-e $fName/title) || !(-e $fName/pool.txt) || !(-e $fName/pool_lib.txt)) then
         continue
      endif

      if ($nExp == 0) then
         echo ""
         echo "Blanks Rack: $fullBlankDirList[$i]"
         echo "Screen Rack: $fullScreenDirList[$i]"
         @ nRack++
      endif

      @ nExp ++

      if (!(-d $blankDir/$fName)) then
         echo ""
         echo "ScoreAllPools Error Finding blank data $blankDir/$fName for screen data $parentDir/$fName"
         @ nBad++
         continue
      endif

      cd $parentDir/$fName

      if ($vFlag != 0) then
         nmrPrintf " Rack: $rackIDList[$ii] Region: $prefix Experiment: $fName\r"
      endif

      if (-e $comName) then
         /bin/rm -f $comName
      endif

      set cIDList = (`cat pool.txt`)
      set cList   = (`cat pool_lib.txt`)

      set j = 1

      foreach cDir ($cList)
         set cID = $cIDList[$j]

         if (!(-e $cDir/test.ft1)) then
            echo ""
            echo "ScoreAllPools Error Finding Compound Library Data:"
            echo " Compound: C$j $cID $cDir/test.ft1 Screen: $parentDir/$fName"
            @ nBad++
            continue
         endif

         set sTitle = (`getParm -in $cDir/test.ft1 -parm FDTITLE -txt -fmt %s`)
         set sInfo  = (`string split $sTitle "-"` None None None None None)
         set sID    = $sInfo[5]

         if (`string csame $cID $sID` == 0) then
            echo ""
            echo "ScoreAllPools Error: Mismatched Compound ID in Library Data."
            echo " Compound: C$j $sID $cDir/test.ft1 Screen: $cID $parentDir/$fName"
            @ nBad++
            continue
         endif

         @ j++
      end

      echo "#\!/bin/csh"               >  $comName
      echo ""                          >> $comName 
      echo "# if (-e scorePool) then"  >> $comName
      echo "#   /bin/rm -rf scorePool" >> $comName
      echo "# endif"                   >> $comName
      echo ""                          >> $comName
      nmrPrintf "scorePool.com "'$*'" $scoreArgs -rackID $rackIDList[$ii] \\"     >> $comName
      nmrPrintf " -blank  $blankDir/$fName/test.ft1 \\"  >> $comName
      nmrPrintf " -screen $screenDir/$fName/test.ft1 \\" >> $comName
      nmrPrintf " -in \\"                                >> $comName

      foreach cDir ($cList)
         nmrPrintf " $cDir/test.ft1 \\" >> $comName
      end

      echo " -outDir scorePool" >> $comName
      echo ""                   >> $comName

      chmod a+rx $comName

      if ($eFlag != 0) then
         $comName 
      endif

      @ nGood++

      cd $parentDir
   end

   if ($vFlag != 0) then
      echo ""
   endif

   echo " Experiments: $nExp" 
   @ i++
end

echo ""
echo "ScoreAllPools: Created $nGood scoring scripts. Bad Entries: $nBad"

exit 0
