#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set dNames    "dObsA.tab  dObsB.tab"
set mfrName   mfr.tab
set refName   ref.pdb 
set pdbDir    $env(PDBH_DIR)

set refR1     First
set refRN     Last

if {[flagLoc $ARGV -help]} \
   {
    puts "Two-Tensor Screening of MFR Table:"
    puts " -in  dcList  \[dObsA.tab dObsB.tab\]  List of Two DC Tables."
    puts " -tab mfrName \[mfr.tab\]              MFR Table Input."
    puts " -pdb pdbName \[ref.pdb\]              PDB Input Name."
    puts " -r1  r1      \[First\]                First Residue in Ref Segment."
    puts " -rN  rN      \[Last\]                 Last Residue in  Ref Segment."
    exit
   }

getListArgD $ARGV -in dNames

getArgD $ARGV -tab mfrName
getArgD $ARGV -pdb refName
getArgD $ARGV -r1  refR1 
getArgD $ARGV -rN  refRN 


#
#----------------------------------------------------------------

dynInit

dynSetSegMode -ignore

set refSrc [dynRead -pdb -in $refName -verb]
set mfrTab [gdbRead table -in $mfrName -verb]
set entry  [gdbFirst $mfrTab]

set dSrcN  [dynRead -dc  -in $dNames  -verb]
set dSrcA  [lindex $dSrcN 0]
set dSrcB  [lindex $dSrcN 1]
set dSrc   [lindex $dSrcN end]

catch {set tInfo [exec simAB.tcl -r1 $refR1 -rN $refRN -brief]}

puts $tInfo

set da    [lindex $tInfo 0]
set dr    [lindex $tInfo 1]
set psi   [lindex $tInfo 2]
set theta [lindex $tInfo 3]
set phi   [lindex $tInfo 4]

dynSetTensor -dc $dSrcA -dadr -da $da -dr $dr \
             -psiRef $psi -thetaRef $theta -phiRef $phi -auto 

set da    [lindex $tInfo 5]
set dr    [lindex $tInfo 6]
set psi   [lindex $tInfo 7]
set theta [lindex $tInfo 8]
set phi   [lindex $tInfo 9]

dynSetTensor -dc $dSrcB -fixed -da $da -dr $dr \
             -psiRef $psi -thetaRef $theta -phiRef $phi -noauto -rotref

#
#----------------------------------------------------------------------------

set atoms "C CA N CB HN HA HA1 HA2 HA3"

while {$entry} \
   {
    set thisName [gdbGet $entry PDB_NAME]
    set pdbName  $pdbDir/$thisName

    set rank     [gdbGet $entry RANK]

    set r1       [gdbGet $entry D_RES1]
    set rN       [gdbGet $entry D_RESN]

    set a1       [gdbGet $entry RES1]
    set aN       [gdbGet $entry RESN]
    set id       [gdbGet $entry ID]

    set dcQOrig  [gdbGet $entry DC_Q]

    set aa1      [expr $a1 - 1]
    set aaN      [expr $aN + 1]

    set pdbSrc   [dynRead -pdb -in $pdbName -r1 $aa1 -rN $aaN -atomName $atoms]

    set aInfo \
     [dynAlign -refSrc $refSrc -aSrc $pdbSrc \
               -atom "C CA N" -r1 $r1 -rN $rN -a1 $a1]

    set bbRMS  [lindex $aInfo end]    

    set dInfoN \
     [dynSimulate -dcList $dSrc -simultaneous -keepTensor -retry \
                  -src $pdbSrc -s1 $r1 -sN $rN -a1 $a1]

    set dInfoA [getNthTensor $dInfoN 1]
    set dInfoB [getNthTensor $dInfoN 2]
    set n      [llength $dInfoN]

    set rmsA   [lindex $dInfoA 0]
    set nA     [lindex $dInfoA end]

    set rmsB   [lindex $dInfoB 0]
    set nB     [lindex $dInfoB end]

    set dcQ    [lindex $dInfoN [expr $n-3]]
    set dd     [expr $dcQ - $dcQOrig]

    set fmt " %3d %3d  %-10s %3d %3d  %7.2f %7.2f  % 8.3f %7.2f"

    puts [format $fmt $r1 $rN $thisName $a1 $aN $dcQOrig $dcQ $dd $bbRMS]

    dynFree $pdbSrc

    set entry [gdbNext $entry]
   }

exit 0
