#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set inName    /spite1/frankie/valpha/cbcanh/ft/test%03d.ft3
set assName   /spite1/frankie/valpha/hn/ass.master.tab
set hi        1.1e+5

set xName    HN
set yName    N
set zName    CACB

set xVarName X_PPM
set yVarName Y_PPM

#
#---------------------------------------------------------------

wm geometry . =950x850+10+10
update

if {[flagLoc $ARGV -help]} \
   {
    puts "Strip plots of a single 3D spectrum."
    puts "  -in     inName       Spectral data file template."
    puts "  -ass    assName      Assignment table for strip locations."
    puts "  -hi     contHi       First Contour height."
    puts "Strip Axis Names:"
    puts "  -xName  xName   \[HN\]     First axis name of strip 2D coord."
    puts "  -yName  yName   \[N\]      Second axis name of strip 2D coord."
    puts "  -zName  zName   \[CACB\]   Axis name of strip dimension."
    puts "Coordinate Table Names:"
    puts "  -xVar   xVar    \[X_PPM\]  Variable name of first 2D PPM coord."
    puts "  -yVar   yVar    \[Y_PPM\]  Variable name of second 2D PPM coord."
    exit
   }

getArgD $ARGV -in      inName
getArgD $ARGV -ass     assName
getArgD $ARGV -hi      hi

getArgD $ARGV -xName   xName
getArgD $ARGV -yName   yName
getArgD $ARGV -zName   zName

getArgD $ARGV -xVar    xVarName
getArgD $ARGV -yVar    yVarName

#
#---------------------------------------------------------------

gdbInit
gdbCreate dBase

set assTab [gdbRead table -in $assName]
set minRes [getMinRes $assTab]
set maxRes [getMaxRes $assTab]
set count  [expr 1 + $maxRes - $minRes]

set dx 40
set qx 10
set ySize 800
set xSize [expr (2+$count)*($dx + $qx) + 2*$qx]

scrollbar .s -orient horiz -command ".c xview"

canvas .c -width $xSize -height $ySize -border 0 \
   -xscrollcommand ".s set" -scrollregion "0 0 $xSize $ySize"

button .b -text Quit -command exit
pack   .c .s .b -side top -expand 1 -fill x

.c create pixmap 0 0 -width $xSize -height $ySize -win 1


#
# For residue:
#   Extract the 2D coords.
#   Extract and plot the strips.

set x1 20
set xn [expr $x1 + $dx]
set stripCount 0

for {set resID $minRes} {$resID <= $maxRes} {incr resID} \
   {
    incr stripCount

    set resName [getNthResName $assTab $resID]
    set xPPM    [getAssShift $assTab $resID HN]
    set yPPM    [getAssShift $assTab $resID N]

    defineCell -x $x1 $xn -y 5% 90% -cell $stripCount -win 1

    if {$xPPM < 999.0 && $yPPM < 999.0} \
       {
        puts " [format %4d $resID ] [format %10s $xPPM] [format %10s $yPPM] ..."

        readROI -in $inName -roi $stripCount -w $xName -1 $yName -1 -ndim 2 \
           -dx $xName ${xPPM}ppm 0.12ppm -y $zName -dz $yName ${yPPM}ppm 1  \
           -center X

        drawROI -roi $stripCount -cell $stripCount -hi $hi \
                -xGrid -tic Bottom,lab=HN -bg black

        drawText -cell $stripCount -y -1% -x 0% -fg white  \
                 -text "$resID $resName"
       }

    if {!($stripCount % 10)} update

    set x1 [expr $xn + $qx]
    set xn [expr $x1 + $dx]
   }
