#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

#
# Extract parameters for scroll strip plot viewing.
# (c) Delaglio 2015 www.nmrscience.com


set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

#
# Initialize variables.

proc initParmVars {} \
{
   global objInfo ARGV stdfont wishFlag

   if {[flagLoc $ARGV -help]} \
      {
       puts "Create a Multi-Spectrum HN Strip Scroll Display Script:"
       puts " -tab      \[Auto\]         Coordinate Table for Strips."
       puts " -dirList  \[.\]            List of Input Data or Directories."
       puts " -ftList   \[ft lp mem ml\] List of Subdirectories to Check."
       puts " -specList \[spec%03d.ft3\] List of Spectrum Templates to Check."
       puts " -mult     \[4.0\]          Noise Multiplier for Contour Height."
       puts " -scan                    Auto-Scan Input Directories for Spectra."
       puts " -sort                    Sort Spectra by Class."
       puts " -auto                    Automatically Create Script." 
       puts " -notk                    Graphics Mode OFF. Sets -auto ON."
       puts " -verb                    Verbose Mode ON."
       exit 0
      }

   if {$wishFlag} \
      {
       setFonts
       set objInfo(font,menu) $stdfont
      }

   set objInfo(prog,progName)  "scrollParm"
   set objInfo(prog,inName)    "scrollN.com"
   set objInfo(prog,outName)   "scrollN.com"

   set objInfo(prog,tabName)   Auto
   set objInfo(prog,xVar)      Auto
   set objInfo(prog,yVar)      Auto

   set objInfo(prog,dirList)   "."
   set objInfo(prog,specList)  "spec%03d.ft3"
   set objInfo(prog,ftList)    "ft lp mem ml"
   set objInfo(prog,expTab)     0
   set objInfo(prog,stripCount) 0
   set objInfo(prog,autoFlag)   0 
   set objInfo(prog,scanFlag)   0 
   set objInfo(prog,execFlag)   0 
   set objInfo(prog,sortFlag)   0 
   set objInfo(prog,verbose)    0 

   set objInfo(prog,maxSpec)    12 
   set objInfo(prog,charX)      10
   set objInfo(prog,charY)      16
   set objInfo(prog,mult)       4.0

   set objInfo(prog,colorWindow) None 

   if {!$wishFlag} {set objInfo(prog,autoFlag) 1}

   set objInfo(prog,vList) \
          {
           type class in pk ndim hi xName yName zName aName xVar yVar zVar aVar
           xTol yTol zTol aTol xOff yOff zOff xw yw zw aw z1 zn z1Loc znLoc 
           a1 an a1Loc anLoc plev nlev colorP colorN colorC colorF cursors
          }

set objInfo(prog,parmInfo) \
   {
    {None         5   0 left   None              None    None  None}
    {in           8  24 left   "Spectrum"        None    file  lightGreen}
    {None         0   0 left   None              None    None  None}
    {type        10  16 left   "Type"            Auto    type  gray}
    {class       10  16 left   "Class"           Auto    class gray}
    {None         0   0 left   None              None    None  None}
    {xName       10  10 left   "HN-Axis"         Auto    axis  gray}
    {xw           3  10 prev   "XW"              0.12ppm xw    gray}
    {yName       10  10 left   "N-Axis"          Auto    axis  gray}
    {yw           3  10 prev   "YW"              1.20ppm yw    gray}
    {zName       10  10 left   "Z-Axis"          Auto    axis  gray}
    {zw           3  10 prev   "ZW"              0       zw    gray}
    {None         0   0 left   None              None    None  None}
    {z1          10  10 left   "Z1"              0%      coord gray}
    {zn          10  10 left   "ZN"              100%    coord gray}

    {None         0   0 mid    None              None    None  None}
    {hi          12  12 mid    "Hi"              Auto    hi    lightPink}
    {None         0   0 mid    None              None    None  None}
    {plev        12   9 mid    "Pos Levels"      16      int   gray}
    {nlev        12   9 mid    "Neg Levels"      16      int   gray}
    {None         0   0 mid    None              None    None  None}
    {colorP      12  10 mid    "Pos Color"       multi   color gray}
    {colorN      12  10 mid    "Neg Color"       multi   color gray}
    {colorC      12  10 mid    "Cursor Color"    white   color gray}
    {colorF      12  10 none   "Fold Color"      blank   color gray}
    {None         0   0 mid    None              None    None  None}
    {cursors     12   6 mid    "Cursors"         0       int   gray}

    {None         0   0 right  None              None    None  None}
    {pk          11  20 right  "Peak Table"      None    file  lightBlue}
    {None         0   0 right  None              None    None  None}
    {xVar        12  10 right  "HN-Axis Var"     Auto    dim   gray}
    {yVar        12  10 right  "N-Axis Var"      Auto    dim   gray}
    {zVar        12  10 right  "Z-Axis Var"      Auto    dim   gray}
    {None         0   0 right  None              None    None  None}
    {xTol        12  10 right  "HN-Axis Tol"     Auto    tol   gray}
    {yTol        12  10 right  "N-Axis Tol"      Auto    tol   gray}
    {zTol        12  10 right  "Z-Axis Tol"      Auto    tol   gray}
    {None         0   0 right  None              None    None  None}
   }

   for {set j 1} {$j <= $objInfo(prog,maxSpec)} {incr j} \
      {
       clearThisStrip $j

       set objInfo(data$j,in) None
       set objInfo(data$j,pk) None
      }

   set objInfo(data1,cursors) 1

   getArgD $ARGV -in      objInfo(prog,inName)
   getArgD $ARGV -out     objInfo(prog,outName)
   getArgD $ARGV -maxSpec objInfo(prog,maxSpec)

   getArgD $ARGV -charX objInfo(prog,charX)
   getArgD $ARGV -charY objInfo(prog,charY)
   getArgD $ARGV -mult  objInfo(prog,mult)

   getListArgD $ARGV -dirList  objInfo(prog,dirList)
   getListArgD $ARGV -specList objInfo(prog,specList)
   getListArgD $ARGV -ftList   objInfo(prog,ftList)

   if {[flagLoc $ARGV -sort]}   {set objInfo(prog,sortFlag) 1}
   if {[flagLoc $ARGV -nosort]} {set objInfo(prog,sortFlag) 0}

   if {[flagLoc $ARGV -scan]}   {set objInfo(prog,scanFlag) 1}
   if {[flagLoc $ARGV -noscan]} {set objInfo(prog,scanFlag) 0}

   if {[flagLoc $ARGV -auto]}   {set objInfo(prog,autoFlag) 1}
   if {[flagLoc $ARGV -noauto]} {set objInfo(prog,autoFlag) 0}

   if {[flagLoc $ARGV -verb]}   {set objInfo(prog,verbose) 1}
   if {[flagLoc $ARGV -noverb]} {set objInfo(prog,verbose) 0}
 
   readStripInfo
}

#
# Read existing parameters.

proc readStripInfo {} \
{
   global objInfo
}

#
# Save the current parameters.

proc saveStripInfo {} \
{
   global objInfo

   set types \
      {
       { {com}        {.com} }
       { {text}       {.txt} }
       { {All Files}  *      }
      }

   set outName \
     [tk_getSaveFile -title "Scroll Script" \
       -filetypes $types -initialfile $objInfo(prog,outName)]

   if {[llength $outName]} {saveStripInfo2 $outName}
}

proc saveStripInfo2 { outName } \
{
   global objInfo

   if {![strcasecmp $outName None]} return
   if {![llength $outName]} return

   set error [catch {set outUnit [open $outName w 0744]}]

   if {$error} \
      {
       puts stderr "Error opening output file $outName"
       return 1
      }

   set thisName $objInfo(prog,tabName)
   if {![strcasecmp $thisName Auto]} {set thisName None}

   puts $outUnit "#!/bin/csh"
   puts $outUnit ""
   puts $outUnit "scroll.tcl -pair \\"
   puts $outUnit " -tab $thisName -xVar $objInfo(prog,xVar) -yVar $objInfo(prog,yVar) \\"
   puts $outUnit " -lab INDEX -noyGrid -colorB dimGray -colorW black -fg black -shade \\"

   set argList {in pk hi z1 zn xw yw zw xName yName zName plev nlev cursors colorF colorC colorP colorN title}
   set pList   {in pk hi z1 zn xw yw zw xName yName zName plev nlev cursors colorF colorC colorP colorN type}

   set i 0

   foreach argName $argList \
      {
       puts -nonewline $outUnit [format " %-10s" -$argName]
       set parm [lindex $pList $i]

       set ii 0

       for {set j 1} {$j <= $objInfo(prog,maxSpec)} {incr j} \
          {
           set obj data$j

           if {![info exists objInfo($obj,in)]}      continue
           if {![strcasecmp $objInfo($obj,in) None]} continue
           if {![llength $objInfo($obj,in)]}         continue

           incr ii

           if {[info exists objInfo($obj,$parm)]} \
              {
               set val $objInfo($obj,$parm)
              } \
           else \
              {
               set val None
              }

           set fmt " %14s"

           if {![strcasecmp $val None]} \
              {
               puts -nonewline $outUnit [format $fmt $val]
              } \
           else \
              {
               if {![strcasecmp $parm in] || ![strcasecmp $val pk]} \
                  {
                   if {$ii == 1} \
                      {
                       set fmt " %s \\\n"
                      } \
                   else \
                      {
                       set fmt "            %s \\\n"
                      }

                   puts -nonewline $outUnit [format $fmt $val]
                  } \
               elseif {![strcasecmp $parm type]} \
                  {
                   set fmt " %14s"
                   puts -nonewline $outUnit [format $fmt \"$val\"]
                  } \
               else \
                  {
                   puts -nonewline $outUnit [format $fmt $val]
                  }
              }
          }

       puts $outUnit " \\"
       incr i
      }

   puts $outUnit " -cmnd"
   puts $outUnit ""

   catch "close $outUnit"

}

#
# Shift the current parameter page.

proc shiftStripInfo { jump } \
{
   global objInfo

   set currentPage [notebook_current $objInfo(winParm,notebook)]

   if {![strcasecmp $currentPage "2D Table"]}    return
   if {![strcasecmp $currentPage "2D Spectrum"]} return

   set maxSpec 0

   for {set j 1} {$j <= $objInfo(prog,maxSpec)} {incr j} \
      {
       if {[llength $objInfo(data$j,in)] && [strcasecmp $objInfo(data$j,in) None]} \
          {
           set maxSpec $j
          }
      }

   set i [lindex $currentPage 1]
 
   set maxSpec $objInfo(prog,maxSpec)

   set j [expr $i + $jump]

   if {$j < 1}        {set j $maxSpec}
   if {$j > $maxSpec} {set j 1}

   swapStrips $i $j
}

proc swapStrips { i j } \
{
   global objInfo

   foreach var $objInfo(prog,vList) \
      {
       if {[info exists objInfo(data$i,$var)]} \
          {
           set tmp $objInfo(data$i,$var)
          } \
       else \
          {
           set tmp ""
           set tmp None
          }

       if {[info exists objInfo(data$j,$var)]} \
          {
           set objInfo(data$i,$var) $objInfo(data$j,$var)
          } \
       else \
          {
           set objInfo(data$i,$var) "" 
           set objInfo(data$i,$var) None 
          }

       set objInfo(data$j,$var) $tmp
      }

   update
}

#
# Reset the current parameter page.

proc clearStripInfo {} \
{
   global objInfo wishFlag

   if {![info exists objInfo(prog,parmInfo)]} return

   if {$wishFlag} \
      {
       set currentPage [notebook_current $objInfo(winParm,notebook)]
       set i [lindex $currentPage 1]
      } \
   else \
      {
       set i 1
      }

   clearThisStrip $i
}

proc clearThisStrip { i } \
{
   global objInfo

   foreach p $objInfo(prog,parmInfo) \
      {
       set var   [lindex $p 0]
       set lWide [lindex $p 1]
       set eWide [lindex $p 2]
       set f     [lindex $p 3]
       set str   [lindex $p 4]
       set dVal  [lindex $p 5]
       set type  [lindex $p 6]
       set bg    [lindex $p 7]

       if {![strcasecmp $dVal None]} \
          {
           set objInfo(data$i,$var) ""
           set objInfo(data$i,$var) $dVal
          } \
       else \
          {
           set objInfo(data$i,$var) $dVal
          }
     }

   set objInfo(data1,cursors) 1

   update
}

#
# Save and quit.

proc closeStripInfo {} \
{
   global objInfo

   saveStripInfo
   exit
}

#
# Event handler to set an input name via file dialog.

proc setParmTabName {} \
{
   global objInfo

   set types \
      {
       { {tab}        {.tab} }
       { {text}       {.txt} }
       { {All Files}  *      }
      }

   if {![strcasecmp $objInfo(prog,tabName) Auto] || ![strcasecmp $objInfo(prog,tabName) None]} \
      { 
       set inName [tk_getOpenFile -title "Coord Table" -filetypes $types]
      } \
   else \
      {
       set inName [tk_getOpenFile -title "Coord Table" -filetypes $types -initialdir [file dirname $objInfo(prog,tabName)] -initialfile $objInfo(prog,tabName)]
      }

   if {[llength $inName]} \
      {
       set objInfo(prog,tabName) $inName
      }
}

proc setParmInName { e page obj var } \
{
   global objInfo

   set types \
      {
       { {All Files}  *      }
       { {ft}         {.ft*} }
       { {dat}        {.dat .DAT} }
       { {fid}        {.fid} }
       { {seq}        {.seq} }
       { {text}       {.txt} }
       { {tab}        {.tab} }
      }

   set inName [tk_getOpenFile -title "Spectral Data Input" -filetypes $types]

   if {[llength $inName]} \
      {
       set inName [name2fmt $inName]
       set objInfo($obj,$var) $inName
      }

   if {![strcasecmp $var in] && [llength $inName]} \
      {
       setParmSpecName $page $obj $var
      }

   $e xview moveto 1.0

   update
}

#
# Set the spectrum name, read and interpret related values.

proc setParmSpecName { page obj var } \
{
   global objInfo

   set error [rdFDATA $objInfo($obj,$var) fdata]

   if {$error} \
      {
       puts stderr $objInfo(prog,progName) "Error reading file $objInfo($obj,$var)"
       exit
      }

   set objInfo($obj,ndim)   [getParmI   $fdata FDDIMCOUNT   NULL_DIM]
   set objInfo($obj,tpFlag) [getParmI   $fdata FDTRANSPOSED NULL_DIM]
   set objInfo($obj,type)   [getParmStr $fdata FDTITLE      NULL_DIM]

   if {$objInfo($obj,ndim) != 3} \
      {
       sysFree $fdata
       return 1
      }

   set dimList {X_PPM Y_PPM Z_PPM A_PPM}

   for {set dim 1} {$dim <= $objInfo($obj,ndim)} {incr dim} \
      {
       set code [getParmI   $fdata FDDIMORDER$dim NULL_DIM]
       set size [getParmI   $fdata NDSIZE         $dim]
       set obs  [getParmI   $fdata NDOBS          $dim]
       set lab  [getParmStr $fdata NDLABEL        $dim]
        
       set first [pnt2spec $fdata $dim 1     ppm]
       set last  [pnt2spec $fdata $dim $size ppm]
       set v     [lindex $dimList [expr $dim - 1]]

       if {$code == 2} \
          {
           set objInfo($obj,xName) $lab
           set objInfo($obj,xVar)  $v
           set objInfo($obj,xTol)  0.03 
          } \
       else \
          {
           if {$first > 120.0 && $last < 120.0} \
              {
               set objInfo($obj,yName) $lab
               set objInfo($obj,yVar)  $v
               set objInfo($obj,yTol)  0.15
              } \
           else \
              {
               set objInfo($obj,zName) $lab
               set objInfo($obj,zVar)  $v
               set objInfo($obj,z1)    [format %.3fppm $first]
               set objInfo($obj,zn)    [format %.3fppm $last]
               set objInfo($obj,z1Loc) [format %.3f    $first]
               set objInfo($obj,znLoc) [format %.3f    $last]

               if {$obs > 400.0} \
                  {
                   set objInfo($obj,zTol)  0.04
                  } \
               else \
                  {
                   set objInfo($obj,zTol)  0.25
                  }
              }
          }
      }

   setSpecClass $obj

   if {[lsearch -exact "HNCO HNCA HNHA HNHN HN" $objInfo($obj,class)] < 0} {return 2}

#
# Estimate contour level:

   readROI -roi 1 -ndim 2 -in $objInfo($obj,$var) -x X_AXIS -y Y_AXIS -dz Z_AXIS 35% 0

   set roiInfo [getROIInfo -roi 1]

   set rdata   [lindex $roiInfo 0]
   set xSize   [lindex [lindex $roiInfo 2] 0]
   set ySize   [lindex [lindex $roiInfo 2] 1]
   set n       [expr $xSize*$ySize]

   set noiseRMS [vEstNoise $rdata $n]

   set objInfo($obj,hi) [format %.2e [expr $objInfo(prog,mult)*$noiseRMS]]

   updateStripLimits

   sysFree $fdata

   return 0
}

proc updateStripLimits {} \
{
   global objInfo

   set classList ""

   for {set j 1} {$j <= $objInfo(prog,maxSpec)} {incr j} \
      {
       set obj data$j

       if {![info exists objInfo($obj,in)]}      continue
       if {![strcasecmp $objInfo($obj,in) None]} continue
       if {![llength $objInfo($obj,in)]}         continue

       set class $objInfo($obj,class)

       if {[lsearch -exact $classList $class]} {lappend classList $class}
      
       lappend z1($class) $objInfo($obj,z1Loc)
       lappend zn($class) $objInfo($obj,znLoc)
      }

  for {set j 1} {$j <= $objInfo(prog,maxSpec)} {incr j} \
      {
       set obj data$j

       if {![info exists objInfo($obj,in)]}      continue
       if {![strcasecmp $objInfo($obj,in) None]} continue
       if {![llength $objInfo($obj,in)]}         continue

       set class $objInfo($obj,class)
       if {![strcasecmp $class NMR]} continue

       set thisZ1 [getMax $z1($class)]
       set thisZN [getMin $zn($class)]

       set objInfo($obj,z1)    [format %.3fppm $thisZ1]
       set objInfo($obj,zn)    [format %.3fppm $thisZN]
      }
}

proc setSpecClass { obj } \
{
   global objInfo

   if {[strcmp $objInfo(prog,expTab)]} \
      {
       set eList [gdbSelect -from $objInfo(prog,expTab) -cond "!strcasecmp( EXPNAME, '$objInfo($obj,type)' )"]

       if {[llength $eList]} \
          {
           set e [lindex $eList 0]
           set objInfo($obj,class) [string toupper [gdbGet $e CLASS]]

           setClassColors $obj
           return
          }

       set eList [gdbSelect -from $objInfo(prog,expTab) -cond "strmatch( EXPNAME, '*$objInfo($obj,type)*' )"]

       if {[llength $eList]} \
          {
           set e [lindex $eList 0]
           set objInfo($obj,class) [string toupper [gdbGet $e CLASS]]

           setClassColors $obj
           return
          }
      }

   set $objInfo($obj,class) NMR
}

proc setClassColors { obj } \
{
   global objInfo

   if {![info exists objInfo($obj,class)]} return

   set class $objInfo($obj,class)

   if {[info exists objInfo(prog,$class,colorP)]} \
      {
       set objInfo($obj,colorP) $objInfo(prog,$class,colorP)
      }

   if {[info exists objInfo(prog,$class,colorN)]} \
      {
       set objInfo($obj,colorN) $objInfo(prog,$class,colorN)
      }
}

proc incrCurrentHi { obj mode } \
{
   global objInfo

   if {![strcasecmp [string index $mode 0] d]} \
      {
       set objInfo($obj,hi) [format %.2e [expr $objInfo($obj,hi)/1.3]]
      } \
   else \
      {
       set objInfo($obj,hi) [format %.2e [expr $objInfo($obj,hi)*1.3]]
      }
}

#
# Event handler for menus.

proc parmMenuProc { page obj var val } \
{
    global objInfo

    switch $var \
       {
        hi   {
              set hi ""

              if {![strcasecmp $val Increase]} \
                 {
                  catch {set hi [expr $objInfo($obj,$var)*1.3]}
                 } \
              else \
                 {
                  catch {set hi [expr $objInfo($obj,$var)/1.3]}
                 }

              catch {set objInfo($obj,$var) [format %.2e $hi]}
             }

        type    {setParmSpecType $page $obj $var $val}

        default {set objInfo($obj,$var) [lindex $val 0]}
       }

    update
}

#
# Create a parameter menu.

proc initParmMenu { w type page obj var } \
{
    global objInfo

    menu $w

    switch $type \
       {
        type {
              set vList [concat Auto $objInfo(prog,specTypes)]
             }

        class {
               set vList [concat Auto $objInfo(prog,specClasses)]
              }

        axis {
              set vList {
                         Auto 1H 15N 13C H1 N15 C13 \
                         HN N CA CACB CO H H-acq H-ind HA HAHB
                        }
             }
 
        dim  {
              set vList {Auto X_PPM Y_PPM Z_PPM A_PPM HN N HN_PPM N_PPM H_PPM C_PPM}
             }
 
        tol  {
              set vList {
                         Auto 0.01 0.02 0.03 0.04 0.05 0.06 0.07 0.08 0.09 \
                         0.10 0.15 0.20 0.25 0.30 0.35 0.40 0.45 0.50
                        }
             }

        xw   { 
              set vList {0.10ppm 0.15ppm 0.18ppm 0.20ppm 0.25ppm 0.50ppm 0.75ppm 1.00ppm 1.25ppm 1.50ppm 2.00ppm 2.50ppm 3.00ppm} 
             }

        yw   { 
              set vList {0.10ppm 0.15ppm 0.18ppm 0.20ppm 0.25ppm 0.50ppm 0.75ppm 1.00ppm 1.25ppm 1.50ppm 2.00ppm 2.50ppm 3.00ppm} 
             }

        zw   { 
              set vList {0 1}
             }

        coord {
               set vList {
                          Auto 0% 100% \
                          -10.0ppm -5ppm -1ppm 0ppm 5ppm 10ppm 15ppm 20ppm \
                          50ppm 55ppm 65ppm 100ppm 120ppm 130ppm 140ppm 150ppm 160ppm  \
                          170ppm 180ppm 
                         }
              }
 
        count {
               set vList {0 1 2 3 4 5 6 7 8 9 10}
              }
 
        bool  {
               set vList {"1 (Yes)" "0 (No)"}
              }

        hi    {
               set vList {"Increase" "Decrease"}
              }
 
        default {
                 set vList "None"
                }
       }
 
     foreach val $vList \
        {
         $w add command -label $val -font $objInfo(font,menu) \
            -command "parmMenuProc $page $obj $var \"$val\""
        }
}

#
# Handle color choice events:

proc stripColorChoiceON { i } \
{
   global objInfo

   if {[info exists objInfo(prog,colorWindow)]} \
      {
       if {[winfo exists $objInfo(prog,colorWindow)]} \
          {
           catch "raise $objInfo(prog,colorWindow)"
           return
          }
      }

   set objInfo(prog,colorWindow) \
    [getColorChoice -color $objInfo($i) -command "stripColorChoiceProc $i"]
}

proc stripColorChoiceProc { i color } \
{
    global objInfo

    set objInfo($i) $color 
}

#
# Create the main application window.

proc initParmWin {} \
{
   global objInfo env wishFlag

   if {!$wishFlag} return

   set obj  winParm

   set objInfo($obj,window)   .pwin
   set objInfo($obj,notebook) .pwin.n

   if {[winfo exists $objInfo($obj,window)]} return

   wm iconify .

   set objInfo($obj,winXSize) [expr 82*$objInfo(prog,charX)] 
   set objInfo($obj,winYSize) [expr 14*$objInfo(prog,charY)]
   set objInfo($obj,pageList) ""

   toplevel $objInfo($obj,window) 

   wm title    $objInfo($obj,window) "NMRPipe/VNMRJ Strip Parameters"
   wm geometry $objInfo($obj,window) +20+40

   frame  .pwin.cmnd

   button .pwin.left  -text "Shift Left"  -command "shiftStripInfo -1"
   button .pwin.right -text "Shift Right" -command "shiftStripInfo  1"
   button .pwin.clear -text "Reset Page"  -command clearStripInfo
   button .pwin.save  -text "Save"        -command saveStripInfo
   button .pwin.quit  -text "Quit"        -command closeStripInfo

   frame  .pwin.coord

   label  .pwin.coordLab -text "Coord Table" -width 30 -anchor e
   entry  .pwin.coordEnt -textvariable objInfo(prog,tabName) -width 52 -bg lightYellow

   button .pwin.coordBut \
          -command setParmTabName \
          -relief raised -bitmap @$env(NMRTXT)/caret.map -fg yellow

   pack .pwin.coordLab .pwin.coordEnt .pwin.coordBut -in .pwin.coord -side left


#  lappend objInfo($obj,pageList) "2D Table"
#  lappend objInfo($obj,pageList) "2D Spectrum"

   for {set j 1} {$j <= $objInfo(prog,maxSpec)} {incr j} \
      {
       lappend objInfo($obj,pageList) "Strip $j"
      }

   notebook_create $objInfo($obj,notebook) \
      -pages $objInfo($obj,pageList)       \
      -width  $objInfo($obj,winXSize)      \
      -height $objInfo($obj,winYSize)

   pack $objInfo($obj,notebook) -in $objInfo($obj,window) \
        -fill both -expand 1 -side top

   pack .pwin.cmnd .pwin.coord -in $objInfo($obj,window) -side bottom -fill x -expand 1

   pack .pwin.left .pwin.right .pwin.clear .pwin.save .pwin.quit \
        -in .pwin.cmnd -side left -fill x -expand 1

#
# Parameter settings for the 3D spectra:

   set spaceCount 0

   for {set j 1} {$j <= $objInfo(prog,maxSpec)} {incr j} \
      {
       set pageName "Strip $j"
       set w        [notebook_frame $objInfo($obj,notebook) $pageName]
       set i        [expr $j + 1]
       set i        $j
   
       frame  $w.left
       frame  $w.mid
       frame  $w.right

       pack $w.left $w.mid $w.right -side left -fill both -expand 1
       set  prevFrame None

       foreach p $objInfo(prog,parmInfo) \
          {
           set var   [lindex $p 0]
           set lWide [lindex $p 1]
           set eWide [lindex $p 2]
           set f     [lindex $p 3]
           set str   [lindex $p 4]
           set dVal  [lindex $p 5]
           set type  [lindex $p 6]
           set bg    [lindex $p 7]

           if {![strcasecmp $f None]} continue

           if {![strcasecmp $var None]} \
              {
               incr spaceCount

               frame $w.s$spaceCount -height 8
               pack  $w.s$spaceCount -in $w.$f -side top
               continue
              }

           if {![strcasecmp $f prev]} \
              {
               set thisFrame $prevFrame
              } \
           else \
              {
               set thisFrame $w.f$var
               frame $thisFrame
              }

           label $w.l$var -text $str -width $lWide -anchor e

           if {![strcasecmp $dVal None]} \
              {
               if {![info exists objInfo(data$i,$var)]} \
                  {
                   set objInfo(data$i,$var) ""
                   set objInfo(data$i,$var) None 
                  }
              } \
           else \
              {
               set objInfo(data$i,$var) $dVal
              }

           if {![strcasecmp $type int]} \
              {
               spinbox $w.e$var -textvariable objInfo(data$i,$var) \
                 -width $eWide -bg $bg -increment 1 -from 0 -to 64
              } \
           elseif {![strcasecmp $type hi]} \
              {
               spinbox $w.e$var -textvariable objInfo(data$i,$var) \
                 -width $eWide -bg $bg -increment 0 -from -1.0e+16 -to 1.0e+16 \
                 -command "incrCurrentHi data$i %d" 
              } \
           else \
              {
               entry $w.e$var -textvariable objInfo(data$i,$var) \
                 -width $eWide -bg $bg
              }

           $w.e$var xview moveto 1.0

           pack $w.l$var $w.e$var -in $thisFrame -side left

           if {![strcasecmp $type file]} \
              {
               button $w.m$var \
                 -command "setParmInName $w.e$var $j data$i $var" \
                 -relief raised -bitmap @$env(NMRTXT)/caret.map -fg $bg 

               pack $w.m$var -in $thisFrame -side left
              } \
           elseif {![strcasecmp $type int] || ![strcasecmp $type hi]} \
              {
              } \
           elseif {![strcasecmp $type color]} \
              {
               button $w.m$var -bitmap @$env(NMRTXT)/caret.map -relief raised \
                -width $objInfo(prog,charX) -height $objInfo(prog,charY)   \
                -fg blue -command "stripColorChoiceON data$i,$var"

               pack $w.m$var -in $thisFrame -side left
              } \
           else \
              {
               menubutton $w.m$var -menu $w.m$var.menu \
                 -relief raised -bitmap @$env(NMRTXT)/caret.map

               initParmMenu $w.m$var.menu $type $j data$i $var

               pack $w.m$var -in $thisFrame -side left
              }

           if {[strcasecmp $f prev]} {pack $thisFrame -in $w.$f -side top}
           set prevFrame $w.f$var
          }
      }

   set objInfo(data1,cursors) 1
}

proc scanThisDir { dirList } \
{
   global objInfo

   foreach thisName $dirList \
      {
       if {[file isdirectory $thisName]} \
          {
           if {$objInfo(prog,verbose)} {puts "Scanning $thisName ..."}

           set dName [file tail $thisName]

           if {[lsearch -exact $objInfo(prog,ftList) $dName] >= 0} \
              {
               foreach template $objInfo(prog,specList) \
                  {
                   set sName [format $template 1]

                   if {[file exists $thisName/$sName]} \
                      {
                       incr objInfo(prog,stripCount)
                       set i $objInfo(prog,stripCount)
 
                       set objInfo(data$i,in) [name2fmt [file normalize $thisName/$sName]]
                       if {$objInfo(prog,verbose)} {puts " Adding Strip $i $objInfo(data$i,in)"}

                       set error [setParmSpecName $i data$i in]

                       if {$error} \
                          {
                           incr objInfo(prog,stripCount) -1
                           clearThisStrip $i 
                          }
                      }
                  }
              } \
           else \
              {
               scanThisDir [glob -nocomplain $thisName/*]
              }
          }
      }
}

proc updateTabName {} \
{
   global objInfo

   if {[strcasecmp $objInfo(prog,tabName) Auto] && [strcasecmp $objInfo(prog,tabName) None]} return

   for {set i 1} {$i < $objInfo(prog,maxSpec)} {incr i} \
      {
       set objI data$i

       if {![info exists objInfo($objI,in)]}      continue
       if {![strcasecmp $objInfo($objI,in) None]} continue
       if {![llength $objInfo($objI,in)]}         continue

       set thisDir [file dirname [file dirname $objInfo($objI,in)]]

       if {[file exists $thisDir/proj.tab]} \
          {
           set objInfo(prog,tabName) $thisDir/proj.tab
           return
          }
      }
}

proc sortByClass {} \
{
   global objInfo

   set classList {HNCO HNCA H1 HOMO HNHA HN HC HACA NMR}
   set nC [llength $classList]

   for {set i 1} {$i < $objInfo(prog,maxSpec)} {incr i} \
      {
       set objI data$i

       if {![info exists objInfo($objI,in)]}      continue
       if {![strcasecmp $objInfo($objI,in) None]} continue
       if {![llength $objInfo($objI,in)]}         continue

       if {![info exists objInfo($objI,class)]} continue

       set scoreI [lsearch -exact $classList $objInfo($objI,class)]
       if {$scoreI < 0} {set scoreI $nC}

       for {set j [expr $i + 1]} {$j <= $objInfo(prog,maxSpec)} {incr j} \
          {
           set objJ data$j

           if {![info exists objInfo($objJ,in)]}      continue
           if {![strcasecmp $objInfo($objJ,in) None]} continue
           if {![llength $objInfo($objJ,in)]}         continue

           set scoreJ [lsearch -exact $classList $objInfo($objJ,class)]
           if {$scoreJ < 0} {set scoreJ $nC}

           if {$scoreI > $scoreJ} \
              {
               swapStrips $i $j
               set scoreI $scoreJ
              } 
          }
      }
}

proc initSpecTypes {} \
{
   global objInfo env

   set objInfo(prog,specTypes) \
    {
     HSQC HNCA HNCO HNCACB HN(CO)CACB HNHA HBHANH \
     HBHA(CO)NH CBCANH CBCA(CO)NH N15TOCSY N15NOE
    }
 
   set objInfo(prog,specClasses) { NMR HOMO H1 HN HC HNCO HNCA HNHA HACA }
 
   if {[file exists $env(NMRBASE)/nmrtxt/expNames.tab]} \
      {
       set objInfo(prog,expTab)      [gdbRead table -in $env(NMRBASE)/nmrtxt/expNames.tab]
       set objInfo(prog,specTypes)   [getTabUniqVals $objInfo(prog,expTab) EXPNAME]
       set objInfo(prog,specClasses) [getTabUniqVals $objInfo(prog,expTab) CLASS]

       set eList [gdbGetInfo $objInfo(prog,expTab) -parms entries]

       foreach e $eList \
          {
           set class  [string toupper [gdbGet $e CLASS]]
           set pColor [gdbGet $e PCOLOR]
           set nColor [gdbGet $e NCOLOR]

           set objInfo(prog,$class,colorP) $pColor
           set objInfo(prog,$class,colorN) $nColor
          }
      }
}

initParmVars
initSpecTypes
initParmWin

if {$objInfo(prog,scanFlag)} {scanThisDir $objInfo(prog,dirList)}
if {$objInfo(prog,sortFlag)} {sortByClass}

if {![strcasecmp $objInfo(prog,tabName) Auto] || ![strcasecmp $objInfo(prog,tabName) None]} \
   {
    updateTabName
   }

if {$objInfo(prog,autoFlag)} \
   {
    saveStripInfo2 $objInfo(prog,outName)
   }

if {!$wishFlag || $objInfo(prog,autoFlag)} {exit 0}

