#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

set refName         None
set className       None

set mfrNameList     mfr.tab
set mfrColorList    "blue red"

set phiPsiNameList  phiPsi.tab
set phiPsiColorList violet 
 
set pdbNameList     None
set pdbColorList    "white green yellow red cyan"

set extraSurfName   None
set extraSurfArgs   None 

set minRank      1
set maxRank      5
set minRes       First 
set maxRes       Last 
set skip         1

set selectColor  gray30 
set boxColor     white
set avgColor     orange 

set useMFRSurf   1
set useCSSurf    1
set sortFlag     0
set chiFlag      0

set xw         250
set yw         250
set dx          10
set dy          30 
set lw           1
set cellX       10 
set cellY       14

set foldFlag     1
set foldPhi    -30
set foldPsi   -130

set phiLimit    30 
set psiLimit    30 

set dotFlag      0
set lineFlag     1
set boxFlag      1
set verbose      0

set xSize     0
set ySize     340
set wxSize    1100
set wySize    [expr $yw + 170]

set pdbDir     $env(PDBH_BASE)/pdbH
set classDir   $env(PDBH_BASE)/class
set surfDir    $env(SURF_DIR)
set fragDir    None


if {[flagLoc $ARGV -help]} \
   {
    puts "Display Phi/Psi Angles for Fragments and PDB Files."
    puts " -mfr    mfrNameList  \[mfr.tab\]     Fragment Tables to Display."
    puts " -phiPsi aNameList    \[phiPsi.tab\]  Phi/Psi Tables to Display."
    puts " -pdb    pdbNameList  \[None\]        PDB Files to Display."
    puts " -ref    refPDBName   \[ref.pdb\]     Reference PDB (for Sequence)."
    puts " -r1     r1           \[First\]       First Residue to Display."
    puts " -rN     rN           \[Last\]        Last Residue to Display."
    puts " -skip   nSkip        \[1\]           MFR Flanking Residues to Skip."
    puts " -verb                              Verbose Mode ON."
    puts "Additional Surface to Display:"
    puts " -surfName surfName   \[None\]        Surface Template (Such as surf/s%03d.dat)."
    puts " -surfArgs surfArgs   \[None\]        Surface drawROI Options, -- Terminated."
    exit 1
   }

set ARGV [getListArgDT $ARGV -surfArgs extraSurfArgs]
set ARGC [llength $ARGV]

if {[flagLoc $ARGV -talos]} \
   {
    set pdbDir       $env(TALOS_DIR)/pdb
    set pdbNameList  $env(TALOS_DIR)/tab/resolution.tab
   }

getListArgD $ARGV -mfr    mfrNameList
getListArgD $ARGV -pdb    pdbNameList
getListArgD $ARGV -phiPsi phiPsiNameList

getArgD $ARGV -ref      refName
getArgD $ARGV -class    className
getArgD $ARGV -surfName extraSurfName

if {[strcasecmp $pdbNameList None]} \
   {
    set n [llength $pdbNameList]

    if {![strcasecmp $refName None]} \
       {
        for {set i 0} {$i < $n} {incr i} \
           {
            set thisName [lindex $pdbNameList $i]

            if {[string first "color=" $thisName] >= 0} continue 

            set refName $thisName
            break
           }
       }
   }

if {![strcasecmp $refName None] && ![strcasecmp $pdbNameList None]} \
   {
    set refName ref.pdb
   }

getListArgD $ARGV -mfrColor    mfrColorList
getListArgD $ARGV -pdbColor    pdbColorList
getListArgD $ARGV -phiPsiColor phiPsiColorList

getArgD $ARGV -r1   minRes
getArgD $ARGV -rN   maxRes
getArgD $ARGV -skip skip

getArgD $ARGV -xw   xw
getArgD $ARGV -yw   yw

getArgD $ARGV -minRank minRank
getArgD $ARGV -maxRank maxRank

getArgD $ARGV -pdbDir   pdbDir
getArgD $ARGV -classDir classDir
getArgD $ARGV -surfDir  surfDir
getArgD $ARGV -fragDir  fragDir

if {[flagLoc $ARGV -sort]}    {set sortFlag 1}
if {[flagLoc $ARGV -nosort]}  {set sortFlag 0}

if {[flagLoc $ARGV -chi]}     {set chiFlag 1}
if {[flagLoc $ARGV -nochi]}   {set chiFlag 0}

if {[flagLoc $ARGV -dot]}     {set dotFlag 1}
if {[flagLoc $ARGV -nodot]}   {set dotFlag 0}

if {[flagLoc $ARGV -line]}    {set lineFlag 1}
if {[flagLoc $ARGV -noline]}  {set lineFlag 0}

if {[flagLoc $ARGV -box]}     {set boxFlag 1}
if {[flagLoc $ARGV -nobox]}   {set boxFlag 0}

if {[flagLoc $ARGV -fold]}    {set foldFlag 1}
if {[flagLoc $ARGV -nofold]}  {set foldFlag 0}

if {[flagLoc $ARGV -verb]}    {set verbose 1}
if {[flagLoc $ARGV -noverb]}  {set verbose 0}

#
#---------------------------------------------------------------

initResNames

set imageCount    0
set roiCount      0

set currentCellID 0
set currentResID  0

set x1Pix         0
set xnPix         0
set y1Pix         0
set ynPix         0

set cx1Pix        0
set cxnPix        0
set cy1Pix        0
set cynPix        0

#
#---------------------------------------------------------------

proc drawLineLW { x1 xn y1 yn fg wide } \
{
   for {set i 0} {$i < $wide} {incr i} \
      {
       drawLine -x $x1 $xn -y $y1 $yn -fg $fg

       set y1 [expr $y1 + 1]
       set yn [expr $yn + 1]
      }
}

proc initWin {} \
{
   global wxSize wySize

   puts stderr "Initializing ..."

   wm title    . \
    "Backbone Angle Viewer 2011.186.14.26 (c) Frank Delaglio"

   wm geometry . =${wxSize}x${wySize}+10+10
   update
}

proc initRef {} \
{
   global refName refSrc minRes maxRes resCount 

   dynInit -fast

   if {![file exists $refName]} \
      {
       puts stderr "Error finding PDB Reference File $refName"
       exit 1
      }

   set refSrc   [dynRead -pdb -in $refName]
   set refInfo  [dynGetInfo -src $refSrc]
   set r1       [lindex $refInfo 0]
   set rN       [lindex $refInfo 1]

   if {![strcasecmp $minRes First]} {set minRes $r1}
   if {![strcasecmp $maxRes Last]}  {set maxRes $rN}

   rClip minRes $r1 $rN
   rClip maxRes $r1 $rN

   set resCount [expr 1 + $maxRes - $minRes]
}

proc initClass {} \
{
   global minRes maxRes className classInfo

   for {set r0 $minRes} {$r0 <= $maxRes} {incr r0} \
      {
       set classInfo($r0) ""
      }

   if {![strcasecmp $className None]} return
   if {![file exists $className]}     return

   set cTab  [gdbRead table -in $className]
   set eList [gdbGetInfo $cTab -parms entries]

   foreach entry $eList \
      {
       set r0 [gdbGet $entry RESID]
       set classInfo($r0) [gdbGet $entry CLASS]
      }
}

proc initGraph {} \
{
   global xSize ySize xw yw dx dy resCount

   set xSize    [expr (2+$resCount)*($xw + $dx) + 2*$dx]

   scrollbar .s -orient horiz -command ".c xview"

   canvas .c -width $xSize -height $ySize -border 0 \
          -xscrollcommand ".s set" -scrollregion "0 0 $xSize $ySize"

   frame  .cmnd
   button .save -text Save -command saveImage
   button .quit -text Quit -command exit

   pack   .save .quit -in .cmnd -side left -expand 1 -fill x
   pack   .c .s .cmnd -side top -expand 1 -fill x

   .c create pixmap 0 0 -width $xSize -height $ySize -win 1

   bind .c <ButtonRelease-1> "+ selectPhiPsiCoord %x %y"
   bind .c <ButtonPress-3>   "+ selectPhiPsiReg   %x %y"
   bind .c <ButtonRelease-2> "+ showClasses       %x %y"
   bind .c <ButtonRelease-3> "+ selectPhiPsi      %x %y"
   bind .c <B3-Motion>       "+ trackPhiPsi       %x %y"

   drawRec -x 1 $xSize -y 1 $ySize -fg dimGray
}

proc saveImage {} \
{
   global xSize ySize imageCount

   incr imageCount

   set xInfo [.c xview]
   set yInfo [.c yview]

   set x1    [lindex $xInfo 0]
   set xn    [lindex $xInfo 1]
   set y1    [lindex $yInfo 0]
   set yn    [lindex $yInfo 1]

   set xw [expr int($xSize*($xn - $x1)) - 1]
   set yw [expr int($ySize*($yn - $y1)) - 1]

   set outName [format rama%03d.gif $imageCount]

   writeImage -out $outName -gif -x 1 $xw -y 1 $yw

   puts stderr "$outName $xw x $yw"
}

proc findResIDByPix { xPix yPix } \
{
   global minRes maxRes currentResID currentCellID
   global cx1Pix cxnPix cy1Pix cynPix

   for {set resID $minRes} {$resID <= $maxRes} {incr resID} \
      {
       set cellID [expr 1 + $resID - $minRes]

       set cInfo [getCellInfo -cell $cellID]
       set xInfo [lindex $cInfo 1]
       set yInfo [lindex $cInfo 2]

       set cx1Pix   [lindex $xInfo 0]
       set cxnPix   [lindex $xInfo 1]
       set cy1Pix   [lindex $yInfo 0]
       set cynPix   [lindex $yInfo 1]

       if {$xPix < $cx1Pix || $xPix > $cxnPix} continue
       if {$yPix < $cy1Pix || $yPix > $cynPix} continue

       set currentCellID $cellID
       set currentResID  $resID

       return 1 
      }

   return 0 
}

proc selectPhiPsiCoord { sxPix syPix } \
{
   global currentCellID currentResID 
   global lw selectColor

   global cx1Pix cxnPix cy1Pix cynPix
   global x1Pix xnPix y1Pix ynPix

   set xPix [.c canvasx $sxPix]
   set yPix [.c canvasy $syPix]

   set status [findResIDByPix $xPix $yPix]

   if {!$status} return

   set x1Pix $xPix
   set y1Pix $yPix
   set xnPix $xPix
   set ynPix $yPix

   set n [expr $lw*3]

   .c create rectangle [expr $x1Pix - $n] [expr $y1Pix - $n] \
                       [expr $xnPix + $n] [expr $ynPix + $n] \
                       -outline $selectColor -fill $selectColor \
                       -tags selected 

   set xRange [expr $cxnPix - $cx1Pix]
   set yRange [expr $cynPix - $cy1Pix]

   set phi1 [expr -180.0 + 360.0*($x1Pix - $cx1Pix)/$xRange]
   set psi1 [expr  180.0 - 360.0*($y1Pix - $cy1Pix)/$yRange]

   set fmt "ResID: %3d Phi: % 7.2f  Psi: % 7.2f"
   set str [format $fmt $currentResID $phi1 $psi1]

   puts stderr $str
   puts $str

   update
}

proc selectPhiPsiReg { sxPix syPix } \
{
   global x1Pix xnPix y1Pix ynPix currentCellID currentResID selectColor

   set xPix [.c canvasx $sxPix]
   set yPix [.c canvasy $syPix]

   set status [findResIDByPix $xPix $yPix]

   if {!$status} return

   set x1Pix $xPix
   set y1Pix $yPix
   set xnPix $xPix
   set ynPix $yPix

   .c create rectangle $x1Pix $y1Pix $xnPix $ynPix \
                       -outline $selectColor -tags zoom

   update
}

proc showClasses { sxPix syPix } \
{
   global currentCellID currentResID fragInfo
   global cx1Pix cxnPix cy1Pix cynPix
   global x1Pix xnPix y1Pix ynPix

   set xPix [.c canvasx $sxPix]
   set yPix [.c canvasy $syPix]

   set status [findResIDByPix $xPix $yPix]

   if {!$status} return

   set x1Pix $xPix
   set y1Pix $yPix
   set xnPix $xPix
   set ynPix $yPix

   puts "ResID: $currentResID"

   foreach class $fragInfo($currentResID,class) {puts " $class"}

   puts ""

   update
}

proc trackPhiPsi { sxPix syPix } \
{
   global x1Pix xnPix y1Pix ynPix

   set xPix [.c canvasx $sxPix]
   set yPix [.c canvasy $syPix]

   set xnPix $xPix
   set ynPix $yPix

   .c coords zoom $x1Pix $y1Pix $xnPix $ynPix

   update
}

proc selectPhiPsi { sxPix syPix } \
{
   global currentCellID currentResID selectColor
   global cx1Pix cxnPix cy1Pix cynPix
   global x1Pix xnPix y1Pix ynPix

   set xPix [.c canvasx $sxPix]
   set yPix [.c canvasy $syPix]

   if {$x1Pix > $xnPix} \
      {
       set itemp $x1Pix
       set x1Pix $xnPix
       set xnPix $itemp
      }

   if {$y1Pix > $ynPix} \
      {
       set itemp $y1Pix
       set y1Pix $ynPix
       set ynPix $itemp
      }

   set xRange [expr $cxnPix - $cx1Pix]
   set yRange [expr $cynPix - $cy1Pix]

   set phi1 [expr -180.0 + 360.0*($x1Pix - $cx1Pix)/$xRange]
   set phiN [expr -180.0 + 360.0*($xnPix - $cx1Pix)/$xRange]

   set psi1 [expr 180.0 - 360.0*($ynPix - $cy1Pix)/$yRange]
   set psiN [expr 180.0 - 360.0*($y1Pix - $cy1Pix)/$yRange]

   set fmt "ResID: %3d Phi: % 7.2f % 7.2f   Psi: % 7.2f % 7.2f"
   set str [format $fmt $currentResID $phi1 $phiN $psi1 $psiN]

   puts stderr $str
   puts $str

   .c delete zoom
   .c create rectangle $x1Pix $y1Pix $xnPix $ynPix \
                       -outline $selectColor -tags selected 
   update
}

proc initRama {} \
{
   global dx dy xw yw xSize ySize minRes maxRes roiCount surfDir refSrc 
   global useCSSurf useMFRSurf boxColor avgColor classInfo
   global extraSurfName extraSurfArgs

   set x1 $dx 
   set xn [expr $x1 + $xw]
   set yn [expr $ySize - $dy]
   set y1 [expr $yn - $yw]

   set roiCount 0

   for {set resID $minRes} {$resID <= $maxRes} {incr resID} \
      {
       set resName  [dynGetInfo -src $refSrc -res $resID -resName]
       set nextName [dynGetInfo -src $refSrc -res [expr $resID + 1] -resName]
       set cellID   [expr 1 + $resID - $minRes]

       if {![strcasecmp $nextName P] || ![strcasecmp $nextName PRO]} \
          {
           set thisResName xPRO
           set thisResName $resName
          } \
       else \
          {
           set thisResName $resName
          }

       set thisName $surfDir/$thisResName.dat

       if {![file exists $thisName]} {set thisName $surfDir/zero.dat}

       defineCell -cell $cellID -win 1 -x $x1 $xn -y $y1 $yn

       if {[info exists roiList($thisResName)]} \
          {
           set roiID $roiList($thisResName)
           set hi    $roiList($thisResName,hi)
          } \
       else \
          {
           incr roiCount
           set  roiID $roiCount
           set  roiList($thisResName) $roiID
    
           readROI -roi $roiID -ndim 2 -in $thisName

           set rInfo [getROIInfo -roi $roiID]
           set fdata [lindex $rInfo 1]

           set hi [getParm $fdata FDTAU NULL_DIM]
           if {$hi < 0.0} {set hi 3.0}

           set roiList($thisResName,hi) $hi
          }

       drawROI -roi $roiID -cell $cellID -hi $hi -pix -bg black \
               -pSat1 0 -pSat2 0 -pVal1 .5 -pVal2 .2 -fg $boxColor \
               -plev 10 -nlev 10 

       set thisName [format scanCS/cs%03d.dat $resID]

       if {$useCSSurf && [file exists $thisName]} \
          {
           readROI -roi 666666 -ndim 2 -in $thisName

           set maxVal [getROIStat 666666 vMax]

           drawROI -roi 666666 -cell $cellID -bg None -fg $boxColor \
                   -pHue1 0.4 -pHue2 0.4 -hi [expr $maxVal - 1.0] -plev 1
          }

       set thisName [format mfrSurf/mfr%03d.dat $resID]

       if {$useMFRSurf && [file exists $thisName]} \
          {
           readROI -roi 666666 -ndim 2 -in $thisName

           set roiInfo [getROIInfo -roi 666666]
           set rdata   [lindex $roiInfo 0]
           set fdata   [lindex $roiInfo 1]
           set thresh  [getParm $fdata FDTAU NULL_DIM]
           set thresh  [expr (1-0.68)*[vMax $rdata [expr 361*361]]]

           if {$thresh == 0.0} {set thresh 3.0}
 
           drawROI -roi 666666 -cell $cellID -bg None -fg $boxColor \
                   -pHue1 0.2 -pHue2 0.2 -hi $thresh -plev 1
          }

       if {[strcasecmp $extraSurfName None]} \
          {
           set thisName [format $extraSurfName $resID]

           readROI -roi 666666 -ndim 2 -in $thisName

           if {![strcasecmp $extraSurfArgs None]} \
              {
               set aStr ""
              } \
           else \
              {
               set aStr $extraSurfArgs 
              }

           eval "drawROI -roi 666666 -cell $cellID $aStr -bg None -fg $boxColor"
          }

       puts stderr "$resName $resID of $minRes to $maxRes, ROI $roiID"

       drawText -cell $cellID -fg $boxColor -x 0% -y 110% \
                -text $resName $resID 

       drawText -cell $cellID -fg $avgColor -x 3% -y 98% \
                -text $classInfo($resID)

       set x1 [expr $xn + $dx]
       set xn [expr $x1 + $xw]

       update
      }
}

#
# Extract and draw phi/psi trajectories for each segment:

proc drawMFRTrajectories { mfrTab thisColorList lw } \
{
   global dotFlag lineFlag boxFlag boxColor avgColor minRank maxRank skip
   global minRes maxRes pdbDir classDir foldFlag foldPhi foldPsi refSrc chiFlag
   global fragDir fragInfo verbose

   set entryCount [gdbGetInfo $mfrTab -parms entryCount]
   set eList      [gdbGetInfo $mfrTab -parms entries]
   set vList      [gdbGetInfo $mfrTab -parms varNames]
   set colorCount [llength $thisColorList]

   set fragName   None
   set thisFrag   None
   set fragSrc    0
   set useFrag    1

   set x0List    ""
   set y0List    ""
   set colors    ""

   set segIndex  0
   set nn        0
   set k         0

   if {[lsearch -exact $vList FRAG_NAME] < 0} {set useFrag 0}
   if {![strcasecmp $fragDir None]} {set useFrag 0}
  
   for {set r0 $minRes} {$r0 <= $maxRes} {incr r0} \
      {
       set fragInfo($r0,phi)   ""
       set fragInfo($r0,psi)   ""
       set fragInfo($r0,class) None
      }

   if {$verbose} \
      {
       set str [dynGetPDBSeqStr $refSrc]

       puts "$str"
       puts -nonewline "VARS   PDB_NAME INDEX RANK"
       puts -nonewline " RESID RESNAME PDB_RESID PDB_RESNAME PHI PSI OMEGA"

       if {$chiFlag} {puts -nonewline " CHI1 CHI1 CHI3 CHI4" }

       puts " CLASS"

       puts -nonewline "FORMAT %12s %4d %2d %3d %s %3d %s %8.2f %8.2f %8.2f"

       if {$chiFlag} {puts -nonewline " %8.2f %8.2f %8.2f %8.2f"}

       puts " %s"
       puts ""
      }

   drawSettings -reset

   foreach entry $eList \
      {
       set segName  [addExt [gdbGet $entry PDB_NAME] pdb]
       set rank     [gdbGet $entry RANK]
       set classTab 0

       if {$rank < $minRank} continue
       if {$rank > $maxRank} continue

       set thisFrag None
       set fragName None

       if {$useFrag} \
          {
           set thisFrag [addExt [gdbGet $entry FRAG_NAME] pdb]
           set fragName $fragDir/$thisFrag
          }

       set r1 [gdbGet $entry D_RES1]
       set rN [gdbGet $entry D_RESN]

       if {$r1 < $minRes} continue
       if {$rN > $maxRes} continue

       set a1  [gdbGet $entry RES1]
       set aN  [gdbGet $entry RESN]

       set aa1 [expr $a1 - $skip - 1]
       set aaN [expr $aN + $skip + 1]

       if {$rank > $colorCount} \
          {
           set thisColor [lindex $thisColorList end]
          } \
       else \
          {
           set thisColor [lindex $thisColorList [expr $rank - 1]]
          }

       incr k
 
       if {![strcasecmp $segName None]} continue

       set c [string index $segName 0]

       if {![strcmp $c "/"]} \
          {
           set fullName $segName
          } \
       else \
          {
           set fullName $pdbDir/$segName
          }
 
       if {![file exists $fullName]} \
          {
           puts stderr "Skipping PDB $segName"
           continue
          }

       if {$useFrag && ![file exists $fragName]} \
          {
           puts stderr "Skipping Fragment $thisFrag" 
           continue
          }
 
       puts -nonewline stderr "$k of $entryCount, Rank $rank, $thisColor "
       puts -nonewline stderr " $segName $a1 to $aN: $r1 to $rN Frag: $thisFrag"
       puts stderr ""

       if {$chiFlag} \
          {
           set atomList "*"
          } \
       else \
          {
           set atomList "C CA N"
          }

       set rootName [file root $segName]
       set thisName $classDir/$rootName.tab

       if {$verbose && [file exists $thisName]} \
          {
           set classTab [gdbRead table -in $thisName]
          }

       set segSrc [dynRead -pdb -in $fullName \
                           -r1 $aa1 -rN $aaN -atomName $atomList]

       if {$useFrag} \
          {
           set fragSrc \
            [dynRead -pdb -in $fragName -r1 $r1 -rN $rN -atomName $atomList]
          }

       set r0     $r1

       set a1     [expr $a1 + $skip]
       set aN     [expr $aN - $skip]
       set r0     [expr $r0 + $skip]
   
       set xList  ""
       set yList  ""
       set n      0

       incr segIndex

       for {set a0 $a1} {$a0 <= $aN} {incr a0} \
          {
           set class None

           if {$classTab} \
              {
               set entryC [gdbSelect -from $classTab -mSQL -cond "RESID == $a0"]

               if {[llength $entryC] == 1} \
                  {
                   set class [gdbGet $entryC CLASS]
                  } 
              }

           set refResName \
            [getResName1 [dynGetInfo -src $refSrc -res $r0 -resName]]

           set segResName \
            [getResName1 [dynGetInfo -src $segSrc -res $a0 -resName]]

           if {$useFrag} \
              {
               set aInfo   [dynGetPhiPsi -res $r0 -src $fragSrc]
               set phi     [lindex $aInfo 0]
               set psi     [lindex $aInfo 1]
              } \
           else \
              {
               set aInfo   [dynGetPhiPsi -res $a0 -src $segSrc]
               set phi     [lindex $aInfo 0]
               set psi     [lindex $aInfo 1]
              }

           set c ""

           set fmt "%12s %3d %2d  %3d %s  %3d %s   % 8.2f % 8.2f % 8.2f %s %s"

           if {$verbose} \
              {
               if {$useFrag} \
                  {
                   set omega   [dynGetOmega  -res $r0 -src $fragSrc]
                   set chiList [dynGetChi    -res $r0 -src $fragSrc -all]
                  } \
               else \
                  {
                   set omega   [dynGetOmega  -res $a0 -src $segSrc]
                   set chiList [dynGetChi    -res $a0 -src $segSrc -all]
                  }

               append chiList " 9999.00 9999.00 9999.00 9999.00"

               if {$chiFlag} \
                  {
                   for {set ic 0} {$ic < 4} {incr ic} \
                      { 
                       set chi  [lindex $chiList $ic]
                       append c [format " % 8.2f" $chi]
                      }
                  }
  
               puts \
                [format $fmt $rootName $segIndex $rank \
                             $r0 $refResName $a0 $segResName \
                             $phi $psi $omega $c $class]
              }

           if {$phi < 999 && $psi < 999} \
              {
               if {$foldFlag && $phi < $foldPhi && $psi < $foldPsi} \
                  {
                   set psi [expr $psi + 360.0]
                  }

               lappend fragInfo($r0,phi)   $phi
               lappend fragInfo($r0,psi)   $psi
               lappend fragInfo($r0,class) $class

               set x0     [expr 100.0*(180.0 + $phi)/360.0]
               set y0     [expr 100.0*(1.0 - (180.0 + $psi)/360.0)]
               set cellID [expr 1 + $r0 - $minRes]

               set thisX  [spec2pix -cell $cellID -x $x0%]
               set thisY  [spec2pix -cell $cellID -y $y0%]

               lappend x0List   $thisX
               lappend y0List   $thisY
               lappend colors   $thisColor
 
               lappend xList $thisX
               lappend yList $thisY

               incr    nn
               incr    n
              }

           incr r0
          }

       if {$lineFlag} \
          {
           for {set i 0} {$i < [expr $n - 1]} {incr i} \
              {
               set lx1 [lindex $xList $i]
               set ly1 [lindex $yList $i]
       
               set lx2 [lindex $xList [expr $i + 1]]
               set ly2 [lindex $yList [expr $i + 1]]
       
               drawLineLW $lx1 $lx2 $ly1 $ly2 $thisColor $lw

               if {!($i % 50)} update
              }
          }

       if {$dotFlag} \
          {
           for {set i 0} {$i < $n} {incr i} \
              {
               set x0 [lindex $xList $i]
               set y0 [lindex $yList $i]

               drawLineLW $x0 $x0 $y0 $y0 $thisColor $lw

               if {!($i % 50)} update
              }
          }

       dynFree $fragSrc
       dynFree $segSrc
       gdbFree $classTab

       update
      }

   if {$boxFlag} \
      {
       for {set i 0} {$i < $nn} {incr i} \
          {
           set x0       [lindex $x0List $i]
           set y0       [lindex $y0List $i]
           set thisolor [lindex $colors $i]

           set x1 [expr $x0 - 2]
           set xn [expr $x0 + 2]
           set y1 [expr $y0 - 2]
           set yn [expr $y0 + 2]

           drawRec -x $x1 $xn -y $y1 $yn -fg $thisColor
           drawBox -x $x1 $xn -y $y1 $yn -fg $boxColor 

           if {!($i % 50)} update
          }
      }

   for {set r0 $minRes} {$r0 <= $maxRes} {incr r0} \
      {
       if {![llength $fragInfo($r0,phi)]} continue
       if {![llength $fragInfo($r0,psi)]} continue

       set phiAvg  [getAvgAngle    $fragInfo($r0,phi)]
       set psiAvg  [getAvgAngle    $fragInfo($r0,psi)]
       set phiDev  [getStdDevAngle $fragInfo($r0,phi)]
       set psiDev  [getStdDevAngle $fragInfo($r0,psi)]

       set phi    [expr $phiAvg - $phiDev]
       set psi    [expr $psiAvg - $psiDev]

       set x1     [expr 100.0*(180.0 + $phi)/360.0]
       set y1     [expr 100.0*(1.0 - (180.0 + $psi)/360.0)]

       set phi    [expr $phiAvg + $phiDev]
       set psi    [expr $psiAvg + $psiDev]

       set xn     [expr 100.0*(180.0 + $phi)/360.0]
       set yn     [expr 100.0*(1.0 - (180.0 + $psi)/360.0)]

       set cellID [expr 1 + $r0 - $minRes]

       set thisX1  [spec2pix -cell $cellID -x $x1%]
       set thisY1  [spec2pix -cell $cellID -y $y1%]
       set thisXN  [spec2pix -cell $cellID -x $xn%]
       set thisYN  [spec2pix -cell $cellID -y $yn%]

       if {$verbose} \
          {
           puts [format " torsion phi %4d % 7.2f %6.2f" $r0 $phiAvg $phiDev]
           puts [format " torsion psi %4d % 7.2f %6.2f" $r0 $psiAvg $psiDev] 
          }

       set fmt     "(%.0f +/- %.0f)  (%.0f +/- %.0f)"

       drawBox -x $thisX1 $thisXN -y $thisY1 $thisYN -fg $avgColor 

       drawText -cell $cellID -x 23% -y 110% -text [format $fmt $phiAvg $phiDev $psiAvg $psiDev]
      }

   update
}

proc drawPDBTrajectory { pdbSrc thisColor lw } \
{
   global minRes maxRes foldFlag foldPhi foldPsi boxColor skip

   drawSettings -reset

   set xList ""
   set yList ""
   set n     0

   set rInfo      [dynGetInfo -src $pdbSrc] 
   set thisMinRes [expr [lindex $rInfo 0] + $skip]
   set thisMaxRes [expr [lindex $rInfo 1] - $skip]
 
   for {set r0 $minRes} {$r0 <= $maxRes} {incr r0} \
      {
       if {$r0 < $thisMinRes} continue
       if {$r0 > $thisMaxRes} continue

       set aInfo [dynGetPhiPsi -src $pdbSrc -res $r0]
       set phi   [lindex $aInfo 0]
       set psi   [lindex $aInfo 1]
   
       if {$phi < 999 && $psi < 999} \
          {
           if {$foldFlag && $phi < $foldPhi && $psi < $foldPsi} \
              {
               set psi [expr $psi + 360.0]
              }

           set x0     [expr 100.0*(180.0 + $phi)/360.0]
           set y0     [expr 100.0*(1.0 - (180.0 + $psi)/360.0)]
           set cellID [expr 1 + $r0 - $minRes]

           set thisX  [spec2pix -cell $cellID -x $x0%]
           set thisY  [spec2pix -cell $cellID -y $y0%]

           lappend xList [expr int($thisX)]
           lappend yList [expr int($thisY)]

           incr    n
          }
      }

   for {set i 0} {$i < [expr $n - 1]} {incr i} \
      {
       set lx1 [lindex $xList $i]
       set ly1 [lindex $yList $i]

       set lx2 [lindex $xList [expr $i + 1]]
       set ly2 [lindex $yList [expr $i + 1]]

       drawLineLW $lx1 $lx2 $ly1 $ly2 $thisColor [expr 2*$lw]
      }

   for {set i 0} {$i < $n} {incr i} \
      {
       set x0        [lindex $xList $i]
       set y0        [lindex $yList $i]
   
       set x1 [expr $x0 - 3]
       set xn [expr $x0 + 3]
       set y1 [expr $y0 - 3]
       set yn [expr $y0 + 3]
   
       drawRec -x $x1 $xn -y $y1 $yn -fg $thisColor 
       drawBox -x $x1 $xn -y $y1 $yn -fg $boxColor
      }

   update
}

proc drawPhiPsiTrajectory { aTab thisColor lw } \
{
   global minRes maxRes foldFlag foldPhi foldPsi boxColor

   drawSettings -reset

   addIndex $aTab $thisColor

   set minID [getTabMin $aTab INDEX]
   set maxID [getTabMax $aTab INDEX]
   set nn 0

   for {set id $minID} {$id <= $maxID} {incr id} \
      {
       set eList [gdbSelect -from $aTab -cond "INDEX == $id"]

       if {![llength $eList]} continue

       set xList ""
       set yList ""
       set cList ""
       set rList ""
       set n 0

       foreach entry $eList \
          {
           set r0     [gdbGet $entry RESID]
           set phi    [gdbGet $entry PHI]
           set psi    [gdbGet $entry PSI]
           set tColor [gdbGet $entry COLOR]
           set class  [gdbGet $entry CLASS]

           if {$r0 < $minRes || $r0 > $maxRes} continue

           if {$foldFlag && $phi < $foldPhi && $psi < $foldPsi} \
              {
               set psi [expr $psi + 360.0]
              }

           set x0     [expr 100.0*(180.0 + $phi)/360.0]
           set y0     [expr 100.0*(1.0 - (180.0 + $psi)/360.0)]
           set cellID [expr 1 + $r0 - $minRes]

           set thisX  [spec2pix -cell $cellID -x $x0%]
           set thisY  [spec2pix -cell $cellID -y $y0%]

           lappend xList [expr int($thisX)]
           lappend yList [expr int($thisY)]
           lappend cList $class
           lappend rList $r0

           incr n
          }

       for {set i 0} {$i < [expr $n - 1]} {incr i} \
          {
           set lx1 [lindex $xList $i]
           set ly1 [lindex $yList $i]
           set lr1 [lindex $rList $i]
 
           set lx2 [lindex $xList [expr $i + 1]]
           set ly2 [lindex $yList [expr $i + 1]]
           set lr2 [lindex $rList [expr $i + 1]]

           if {$lr1 == $lr2} continue
 
           drawLineLW $lx1 $lx2 $ly1 $ly2 $thisColor [expr 4*$lw]
          }

       for {set i 0} {$i < $n} {incr i} \
          {
           set class [lindex $cList $i]
           set x0    [lindex $xList $i]
           set y0    [lindex $yList $i]
       
           set x1 [expr $x0 - 5]
           set xn [expr $x0 + 5]
           set y1 [expr $y0 - 5]
           set yn [expr $y0 + 5]
       
           drawLineLW $x1 $xn $y1 $yn $thisColor [expr 2*$lw]
           drawLineLW $x1 $xn $yn $y1 $thisColor [expr 2*$lw]

           if {![strcasecmp $class Unknown]} continue

           set x0 [expr $x0 + 7]
           set y0 [expr $y0 - 24]

           drawText -x $x0 -y $y0 -fg $thisColor -text $class 
          }

       if {!($nn % 10)} update

       incr nn
      }

   update
}

proc nearPhiPsi { r0 phi psi } \
{
   global fragInfo minRes maxRes
   global phiLimit psiLimit

   if {![info exists fragInfo($r0,phi)]} {return 0} 
   if {$r0 < $minRes || $r0 > $maxRes} {return 0}
   if {$phi > 999 || $psi > 999} {return 0} 

   set n  [llength $fragInfo($r0,phi)]
   set nn 0

   for {set i 0} {$i < $n} {incr i} \
      {
       set thisPhi [lindex $fragInfo($r0,phi) $i]
       set thisPsi [lindex $fragInfo($r0,psi) $i]

       if {$thisPhi > 999 || $thisPsi > 999} continue

       set dPhi [adjustThisAngle [expr $phi - $thisPhi]]
       set dPsi [adjustThisAngle [expr $psi - $thisPsi]]

       if {abs($dPhi) < $phiLimit && abs($dPsi) < $psiLimit} {incr nn}
      }

   return $nn 
}   

proc addIndex { phiPsiTab thisColor } \
{
    set idVar    [gdbFindByName $phiPsiTab INDEX]
    set classVar [gdbFindByName $phiPsiTab CLASS]
    set colorVar [gdbFindByName $phiPsiTab COLOR]

    if {"$idVar" == ""} \
       {
        set idVar [gdbCreate variable -parent $phiPsiTab -name INDEX  \
                                      -varType int -varFmt %4d]

        set eList [gdbGetInfo $phiPsiTab -parms entries]

        foreach entry $eList {gdbSet $entry INDEX [gdbGet $entry RESID]}
       }

    if {"$classVar" == ""} \
       {
        set idVar [gdbCreate variable -parent $phiPsiTab -name CLASS  \
                                      -varType text -varFmt %s]

        set eList [gdbGetInfo $phiPsiTab -parms entries]

        foreach entry $eList {gdbSet $entry CLASS Unknown}
       }

    if {"$colorVar" == ""} \
       {
        set idVar [gdbCreate variable -parent $phiPsiTab -name COLOR  \
                                      -varType text -varFmt %s]

        set eList [gdbGetInfo $phiPsiTab -parms entries]

        foreach entry $eList {gdbSet $entry COLOR $thisColor}
       }
}

proc addRank { mfrTab } \
{
    global sortFlag

    set idVar    [gdbFindByName $mfrTab INDEX]
    set rankVar  [gdbFindByName $mfrTab RANK]

    set needID   0
    set needRank 0

    if {"$rankVar" == ""} \
       {
        set rankVar [gdbCreate variable -parent $mfrTab -name RANK  \
                                        -varType int -varFmt %2d]

        set needRank 1
       }

    if {"$idVar" == ""} \
       {
        set idVar [gdbCreate variable -parent $mfrTab -name INDEX  \
                                      -varType int -varFmt %4d]

        set needID 1
       }

    set entry     [gdbFirst $mfrTab]
    set prevResID 666
    set rank      0
    set id        0

    if {$needID || $needRank} \
       {
        while {$entry} \
           {
            set resID [gdbGet $entry D_RES1]
    
            if {$resID != $prevResID} \
               {
                set prevResID $resID
                set rank      0
               }

            incr rank
            incr id

            if {$needID}   {gdbSet $entry INDEX $id}
            if {$needRank} {gdbSet $entry RANK  $rank}
    
            set entry [gdbNext $entry]
           } 
       } 

    if {$sortFlag} {gdbSort <$rankVar}
}

#
#---------------------------------------------------------------

initWin
initRef
initClass
initGraph
initRama

set tabCount  [llength $mfrNameList]
set nMax      [expr [llength $mfrColorList] - 1]
set nUsed     0
set yOff      2.4
set charCount 1
set j         0

for {set i 0} {$i < $tabCount} {incr i} \
   {
    set mfrName [lindex $mfrNameList $i]

    if {[string first "pdbDir=" $mfrName] >= 0} \
       {
        set pdbDir [lindex [split $mfrName "="] 1]
        continue
       }

    if {[string first "fragDir=" $mfrName] >= 0} \
       {
        set fragDir [lindex [split $mfrName "="] 1]
        continue
       }

    if {[string first "color=" $mfrName] >= 0} \
       {
        set mfrColorList [lindex [split $mfrName "="] 1]
        set nMax 1

        continue
       }

    if {$j > $nMax} {set j $nMax}

    if {![file exists $mfrName]} continue

    set mfrTab        [gdbRead table -in $mfrName]
    set thisColorList [split [lindex $mfrColorList $j] ,]

    incr nUsed

    addRank $mfrTab 

    drawMFRTrajectories $mfrTab $thisColorList $lw

    gdbDestroy $mfrTab

    if {[llength $thisColorList] == 1} \
       {
        set color [lindex $thisColorList 0]
        set str   "$mfrName $minRank-$maxRank"

        drawText -x [expr int($cellX*$charCount)] -y [expr int($cellY*$yOff)] \
                 -colorS black -bg black -fg $color -text $str

        incr charCount [string length $str]
       } \
    else \
       {
        set color [lindex $thisColorList 0]
        set str   "$mfrName $minRank"

        drawText -x [expr int($cellX*$charCount)] -y [expr int($cellY*$yOff)] \
                 -bg black -fg $color -text $str

        incr charCount [string length $str]
        incr charCount 1

        set color [lindex $thisColorList 1]
        set str   "$mfrName [expr $minRank+1]-$maxRank"

        drawText -x [expr int($cellX*$charCount)] -y [expr int($cellY*$yOff)] \
                 -bg black -bg black -fg $color -text $str

        incr charCount [string length $str]
        incr charCount 1
       }

    incr j
   }

if {$nUsed} {set yOff 1.3}

set aCount    [llength $phiPsiNameList]
set nMax      [expr [llength $phiPsiColorList] - 1]
set charCount 1
set j         0

for {set i 0} {$i < $aCount} {incr i} \
   {
    set aName [lindex $phiPsiNameList $i]

    if {[string first "color=" $aName] >= 0} \
       {
        set phiPsiColorList [lindex [split $aName "="] 1]
        set nMax 1

        continue
       }

    if {$j > $nMax} {set j $nMax}

    if {![file exists $aName]} continue

    set aTab      [gdbRead table -in $aName]
    set thisColor [lindex $phiPsiColorList $j]

    drawPhiPsiTrajectory $aTab $thisColor $lw

    gdbFree $aTab

    set str "$aName"

    drawText -x [expr int($cellX*$charCount)] -y [expr int($cellY*$yOff)] \
             -bg black -fg $thisColor -text $str

    incr charCount [string length $str]
    incr charCount 1
    incr j
   }

set yOff 3.5

set pdbCount  [llength $pdbNameList]
set nMax      [expr [llength $pdbColorList] - 1]
set charCount 1
set j         0

for {set i 0} {$i < $pdbCount} {incr i} \
   {
    set pdbName [lindex $pdbNameList $i]

    if {[string first "skip=" $pdbName] >= 0} \
       {
        set skip [lindex [split $pdbName "="] 1]
        continue
       }

    if {[string first "color=" $pdbName] >= 0} \
       {
        set pdbColorList [lindex [split $pdbName "="] 1]
        set nMax 0

        continue
       }

    if {$j > $nMax} {set j $nMax}

    if {![file exists $pdbName]} continue

    set pdbSrc    [dynRead -pdb -in $pdbName]
    set thisColor [lindex $pdbColorList $j]

    drawPDBTrajectory $pdbSrc $thisColor $lw

    dynFree $pdbSrc

    set str "$pdbName"

    drawText -x [expr int($cellX*$charCount)] -y [expr int($cellY*$yOff)] \
             -bg black -fg $thisColor -text $str

    incr charCount [string length $str]
    incr charCount 1
    incr j
   }

puts stderr "Done drawing segments."
