#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Select a Subset of Planes from 3D Data:"
   echo " -inDir   inDir    [fid]            Input Directory."
   echo " -inName  inName   [test%03d.fid]   Input 3D Template Name."
   echo " -outDir  outDir   [sel]            Output Directory."
   echo " -outName outName  [Auto]           Output 3D Template Name."
   echo " -keep    keepList [1]              List of Plane Numbers to Keep."
   exit 0
endif

set inDir    = (`getArgD $argv -inDir   fid`)
set inName   = (`getArgD $argv -inName  test%03d.fid`)
set outDir   = (`getArgD $argv -outDir  sel`)
set outName  = (`getArgD $argv -outName Auto`)

set keepList = (`getListArgD $argv -keep -- 1`)

if (`string csame $outName Auto`) then
   set outName = $inName
endif

#
#---------------------------------------------------------------------------------

set origDir = (`pwd`)

if (!(-e $outDir)) then
   mkdir $outDir
endif

cd $inDir
   set inDirFull = (`pwd`)
cd $origDir

cd $outDir
   set outDirFull = (`pwd`)
cd $origDir

if (`string same $inDirFull $outDirFull` && `string same $inName $outName`) then
   echo "selectPlanes Error: Input and Output Must be Different."
   exit 1
endif

if (`string notsame $inDir $outDir`) then
   /bin/rm -rf $outDir
   mkdir $outDir
endif

set thisName = (`nmrPrintf $inDir/$inName 1`)

if (!(-e $thisName)) then
   echo "selectPlanes Error Finding Input $thisName"
   exit 1
endif

set zN   = (`getParm -in $thisName -parm NDSIZE      -dim CUR_ZDIM -fmt %.0f`)
set zT   = (`getParm -in $thisName -parm NDAPOD      -dim CUR_ZDIM -fmt %.0f`)
set zQ   = (`getParm -in $thisName -parm NDQUADFLAG  -dim CUR_ZDIM -fmt %.0f`)
set zFT  = (`getParm -in $thisName -parm NDFTFLAG    -dim CUR_ZDIM -fmt %.0f`)
set zORD = (`getParm -in $thisName -parm FDDIMORDER3 -dim NULL_DIM -fmt %.0f`)

if ($zFT == 0) then
   if ($zQ == 1) then
      set nN = $#keepList
      set nT = $nN
   else
      set nN = $#keepList
      set nT = (`IMATH "$nN/2"`)
   endif
else   
   set nN = $#keepList
   set nT = (`IMATH "$nN/2"`)
endif

@ j = 0

echo ""
echo "Selecting $nN Planes of $zN Planes:"

foreach i ($keepList)
   if (`IMATH "$i < 1 || $i > $zN"`) then
      echo "selectPlanes Error: Plane $i is Outside the Range of 1 to $zN."
      exit 1
   endif

   @ j++

   set oldName = (`nmrPrintf $inDir/$inName   $i`)
   set newName = (`nmrPrintf $outDir/$outName $j`) 

   set tau     = (`getParm -in $oldName -parm FDTAU   -dim NULL_DIM`)
   set tStr    = (`getParm -in $oldName -parm FDTITLE -dim NULL_DIM -text -fmt %s`)

   nmrPrintf " Old: %s New: %s Tau: %.4e Title: %s\n" $oldName $newName $tau $tStr

   /bin/cp $oldName $newName 

   if ($zORD == 1) then
      sethdr $newName -fileCount $nN -zN $nN -yT $nT -yTD $nT
   else
      sethdr $newName -fileCount $nN -zN $nN -zT $nT -zTD $nT
   endif
end

