#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    exit 0
   }

set rasmolFlag 1
set tabName    None
set pdbName    None

set cfName   $env(SURF_DIR)/cf.tab
set capName  $env(SURF_DIR)/cap.tab
set turnName $env(SURF_DIR)/turn.tab

getArgD $ARGV -tab  tabName
getArgD $ARGV -pdb  pdbName
getArgD $ARGV -cf   cfName
getArgD $ARGV -cap  capName
getArgD $ARGV -turn turnName

if {[flagLoc $ARGV -rasmol]}     {set rasmolFlag 1}
if {[flagLoc $ARGV -norasmol]}   {set rasmolFlag 1}

if {![strcasecmp $pdbName None]} {set rasmolFlag 0}

#
#----------------------------------------------------------------

proc getCFInfo {} \
{
   global seqList sInfo r1 rN
   global cfName

   set cfTab [gdbRead table -in $cfName]
   set type  cf
   set loc   0

   for {set res $r1} {$res <= [expr $rN + 3]} {incr res} \
      {
       set resName [lindex $seqList $loc]
       set cond    "!strcasecmp( RESCODE, \"$resName\" )"
       set entry   [lindex [gdbSelect -from $cfTab -cond $cond] 0]

       set sInfo($res,resName)     $resName
       set sInfo($res,pAlpha)      0.0
       set sInfo($res,pBeta)       0.0
       set sInfo($res,pTurn,$type) 0.0
       set sInfo($res,pt,$type)    0.0

       foreach i {0 1 2 3} {set sInfo($res,fI$i,cf) 0.0}

       if {[llength $entry] == 1} \
          {
           set sInfo($res,pt,$type) 1.0

           set sInfo($res,pAlpha)      [gdbGet $entry P_ALPHA]
           set sInfo($res,pBeta)       [gdbGet $entry P_BETA]
           set sInfo($res,pTurn,$type) [gdbGet $entry P_TURN]
	
           foreach i {0 1 2 3} \
              {
               set sInfo($res,fI$i,$type) [gdbGet $entry F_I$i]
              } 
          }

       incr loc
      }

   for {set res $r1} {$res <= $rN} {incr res} \
      {
       set sInfo($res,classAlpha)      "." 
       set sInfo($res,classBeta)       "."
       set sInfo($res,classTurn,$type) "." 

       foreach i {0 1 2 3} \
          {
           set r0 [expr $res + $i]
           set f0 $sInfo($r0,fI$i,$type)

           set sInfo($res,pt,$type) [expr $sInfo($res,pt,$type)*$f0]
          }
      }

   gdbFree $cfTab
}

proc getTurnInfo {} \
{
   global seqList sInfo r1 rN
   global turnName

   set tTab [gdbRead table -in $turnName]

   getKeyData $tTab dInfo

   set sInfo(turnTypeList) [string trim $dInfo(TURN_TYPES)  "{}"]
   set sInfo(turnAvgPList) [string trim $dInfo(AVG_PROB)    "{}"]
   set sInfo(turnMultList) [string trim $dInfo(MULT_FACTOR) "{}"]

   foreach type $sInfo(turnTypeList) \
      { 
       set loc 0

       for {set res $r1} {$res <= [expr $rN + 3]} {incr res} \
          {
           set resName [lindex $seqList $loc]

           set cond  \
            "!strcasecmp(RESCODE, \"$resName\") && !strcasecmp(TYPE, \"$type\")"

           set entry [lindex [gdbSelect -from $tTab -cond $cond] 0]

           set sInfo($res,pTurn,$type) 0.0
           set sInfo($res,pt,$type)    0.0

           foreach i {0 1 2 3} {set sInfo($res,fI$i,$type) 0.0}

           if {[llength $entry] == 1} \
              {
               set sInfo($res,pTurn,$type) [gdbGet $entry P_TURN]
               set sInfo($res,pt,$type)    1.0

               foreach i {0 1 2 3} \
                  {
                   set sInfo($res,fI$i,$type) [gdbGet $entry F_I$i]
                  } 
              }

           incr loc
          }

       for {set res $r1} {$res <= $rN} {incr res} \
          {
           set sInfo($res,classTurn,$type)  "."

           foreach i {0 1 2 3} \
              {
               set r0 [expr $res + $i]
               set f0 $sInfo($r0,fI$i,$type)

               set sInfo($res,pt,$type) [expr $sInfo($res,pt,$type)*$f0]
              }
          }
      }

   gdbFree $tTab
}
   
proc getCapInfo {} \
{
   global seqList sInfo r1 rN
   global capName

   set cTab [gdbRead table -in $capName]
   set loc  0

   for {set res $r1} {$res <= [expr $rN + 3]} {incr res} \
      {
       set resName [lindex $seqList $loc]
       set cond    "!strcasecmp( RESCODE, \"$resName\" )"
       set entry   [lindex [gdbSelect -from $cTab -cond $cond] 0]

       set sInfo($res,pTurn,NCap) 2.0
       set sInfo($res,pTurn,CCap) 2.0

       set sInfo($res,pt,NCap)    0.0
       set sInfo($res,pt,CCap)    0.0

       foreach i {0 1 2 3} {set sInfo($res,fI$i,CCap) 0.0}
       foreach i {0 1 2 3} {set sInfo($res,fI$i,NCap) 0.0}

       if {[llength $entry] == 1} \
          {
           set sInfo($res,pt,NCap) 1.0
           set sInfo($res,pt,CCap) 1.0

           set sInfo($res,fI0,NCap) [gdbGet $entry N_CAP]
           set sInfo($res,fI1,NCap) [gdbGet $entry N1]
           set sInfo($res,fI2,NCap) [gdbGet $entry N2]
           set sInfo($res,fI3,NCap) [gdbGet $entry N3]

           set sInfo($res,fI0,CCap) [gdbGet $entry C3]
           set sInfo($res,fI1,CCap) [gdbGet $entry C2]
           set sInfo($res,fI2,CCap) [gdbGet $entry C1]
           set sInfo($res,fI3,CCap) [gdbGet $entry C_CAP]
          }

       incr loc
      }

   for {set res $r1} {$res <= $rN} {incr res} \
      {
       set sInfo($res,classTurn,NCap)  "." 
       set sInfo($res,classTurn,CCap)  "." 

       foreach i {0 1 2 3} \
          {
           set r0 [expr $res + $i]

           set f0 $sInfo($r0,fI$i,NCap)
           set sInfo($res,pt,NCap) [expr $sInfo($res,pt,NCap)*$f0]

           set f0 $sInfo($r0,fI$i,CCap)
           set sInfo($res,pt,CCap) [expr $sInfo($res,pt,CCap)*$f0]
          }
      }
}

proc findPAvg { a1 aN type } \
{
   global sInfo r1 rN

   set sum 0.0
   set n   0

   for {set res $a1} {$res <= $aN} {incr res} \
      {
       if {$res < $r1 || $res > $rN} continue

       set sum [expr $sum + $sInfo($res,$type)]
       incr n
      }

   if {$n} {set sum [expr $sum/double($n)]}

   return $sum
}

proc assignSegment { a1 aN type } \
{
   global sInfo r1 rN

   set n 0

   for {set res $a1} {$res <= $aN} {incr res} \
      {
       if {$res < $r1 || $res > $rN} continue

       incr n

       set sInfo($res,$type) $n 
      }
}

proc isContigCF { a1 minCount segLength type minScore } \
{
   global sInfo r1 rN

   set n 0

   for {set offset 0} {$offset < $segLength} {incr offset} \
      {
       set res [expr $a1 + $offset]

       if {$res < $r1 || $res > $rN} continue

       if {$sInfo($res,$type) > $minScore} \
          {
           incr n

           if {$n >= $minCount} {return 1}
          } \
       else \
          {
           set n 0
          }
      }

   if {$n >= $minCount} {return 1}

   return 0
}
 
proc extendCFSegment { a1Ptr aNPtr type segLength minScore } \
{
   global sInfo r1 rN

   upvar $a1Ptr a1
   upvar $aNPtr aN

   return 10
}

#
# Find 4 out of 6 contiguous residues with P(a-helix) > 1.0
#
# Extend the helix in both directions until a set of four contiguous 
# residues that have an average P(a-helix) < 1.0 is reached.
#
# If the segment defined by this procedure is longer than 5 residues
# and the average P(a-helix) > P(b-sheet) for that segment, the 
# segment can be assigned as a helix.


proc findCFAlpha {} \
{
   global sInfo r1 rN

   for {set res $r1} {$res <= $rN} {incr res} \
      {
       if {[isContigCF $res 4 6 pAlpha 1.0]} \
          {
           set a1 $res
           set aN [expr $a1 + 5]

           set segLength [extendCFSegment a1 aN pAlpha 4 1.0] 

           if {$segLength <= 5} continue

           set alphaAvg [findPAvg $a1 $aN pAlpha]
           set betaAvg  [findPAvg $a1 $aN pBeta]

           if {$alphaAvg < $betaAvg} continue

           assignSegment $a1 $aN classAlpha
          }
      }
}

# Find a 3 out of 5 contiguous residues with a value of P(b-sheet) > 1.0 
#
# Extend the sheet in both directions until a set of four contiguous 
# residues that have an average P(b-sheet) < 1.0 is reached. 
#
# The segment is assigned as a beta-sheet if:
#   the average P(b-sheet) > 1.05 and 
#   the average P(b-sheet) > P(a-helix)


proc findCFBeta {} \
{
   global sInfo r1 rN

   for {set res $r1} {$res <= $rN} {incr res} \
      {
       if {[isContigCF $res 3 5 pBeta 1.0]} \
          {
           set a1 $res
           set aN [expr $a1 + 4]

           set segLength [extendCFSegment a1 aN pBeta 4 1.0]

           if {$segLength <= 5} continue

           set alphaAvg [findPAvg $a1 $aN pAlpha]
           set betaAvg  [findPAvg $a1 $aN pBeta]

           if {$betaAvg <= 1.05}     continue
           if {$betaAvg < $alphaAvg} continue

           assignSegment $a1 $aN classBeta
          }
      }
}

# For region of overlapping alpha-helical and beta-sheet assignments:
#
#   helical if the average P(a-helix) > P(b-sheet) for that region. 
#   beta sheet if the average P(b-sheet) > P(a-helix) for that region. 

proc adjustCFOverlap {} \
{
}

# To identify a bend at residue number j, calculate the following value 
#
#       p(t) = f(j)f(j+1)f(j+2)f(j+3) 
#
#  where the f(j+1) value for the j+1 residue is used etc.  If: 
#
#   1. p(t) > thresh.
#   2. the average value for P(turn) > 1.00 in the tetrapeptide; and 
#   3. the averages for the tetrapeptide obey the inequality 
#      P(a-helix) < P(turn) > P(b-sheet)
#
#  then a beta-turn is predicted at that location. 

proc findTurn { type thresh } \
{
   global sInfo r1 rN

   for {set res $r1} {$res <= [expr $rN - 3]} {incr res} \
      {
       set a1 $res
       set aN [expr $a1 + 3]

       set pt     [findPAvg $a1 $aN pt,$type]
       set pTurn  [findPAvg $a1 $aN pTurn,$type]
       set pAlpha [findPAvg $a1 $aN pAlpha]
       set pBeta  [findPAvg $a1 $aN pBeta]

       if {$pt    <= $thresh} continue
       if {$pTurn <= 1.0}     continue

       if {$pAlpha >= $pTurn}  continue
       if {$pBeta  >= $pTurn}  continue

       assignSegment $a1 $aN classTurn,$type
      }
}

proc showAssignments { } \
{
   global sInfo r1 rN rasmolFlag

   set fmt "%4d %-3s  %-3s %-3s %-3s  %-3s %-3s %-3s  %-3s %-3s"

   set colors(classTurn,NCap)      orchid
   set colors(classTurn,CCap)      pink 
   set colors(classAlpha)          yellow
   set colors(classBeta)           red
   set colors(classTurn,Beta-I)    cyan
   set colors(classTurn,Beta-II)   blue
   set colors(classTurn,Beta-Misc) darkBlue
 
   for {set res $r1} {$res <= $rN} {incr res} \
      {
       set cA $sInfo($res,classAlpha)
       set cB $sInfo($res,classBeta)
       set cT $sInfo($res,classTurn,cf)

       set c1 $sInfo($res,classTurn,Beta-I)
       set c2 $sInfo($res,classTurn,Beta-II)
       set cM $sInfo($res,classTurn,Beta-Misc)

       set cN $sInfo($res,classTurn,NCap)
       set cC $sInfo($res,classTurn,CCap)

       puts [format $fmt \
        $res $sInfo($res,resName) $cA $cB $cT $c1 $c2 $cM $cN $cC]

       puts -nonewline stderr [format " %3d %s %8.4f %8.4f" \
        $res $sInfo($res,resName) $sInfo($res,pAlpha) $sInfo($res,pBeta)]

       foreach type {cf Beta-I Beta-II Beta-Misc NCap CCap} \
          {
           puts -nonewline stderr \
            [format "%8.4f %.4e" $sInfo($res,pTurn,$type) $sInfo($res,pt,$type)]
          }

       puts stderr ""
      }

   if {!$rasmolFlag} return

   set tList {
              classTurn,NCap classTurn,CCap 
              classAlpha classBeta 
              classTurn,Beta-I classTurn,Beta-II classTurn,Beta-Misc
             }

   for {set res $r1} {$res <= $rN} {incr res} \
      {
       foreach type $tList \
          {
           if {[strcmp $sInfo($res,$type) .]} \
              {
               send rasmol select $res
               send rasmol colour $colors($type)
              }
          }
      }
}

proc getThisSeqList {} \
{
   global r1 rN seqList
   global tabName pdbName

   set seqList ""
   set r1      0
   set rN      0


   if {[strcasecmp $tabName None]} \
      {
       set t       [gdbRead table -in $tabName]
       set seqList [getSeqList $t r1 rN]

       gdbFree $t
      }

   if {[strcasecmp $pdbName None]} \
      {
       set t       [gdbRead table -in $pdbName -pdb]
       set seqList [getPDBSeqList $t r1 rN]

       gdbFree $t
      }
}

proc initSSMolDisplay {} \
{
   global rasmolFlag pdbName env

   if {!$rasmolFlag} return

   if {[lsearch -exact [winfo interps] rasmol] < 0} \
      {
       catch "exec xterm -iconic -e $env(NMRBIN)/rasmol &"
       catch "exec sleep 3"
      }

   if {[lsearch -exact [winfo interps] rasmol] < 0} \
      {
       set rasmolFlag 0
      }

   send rasmol zoom         150
   send rasmol set fontsize 16

   send rasmol zap
   send rasmol load   $pdbName

   send rasmol select *
   send rasmol colour gray
}
 
#
#---------------------------------------------------------------------

getThisSeqList

initSSMolDisplay

getCFInfo
getTurnInfo
getCapInfo

findCFAlpha
findCFBeta
adjustCFOverlap

findTurn cf 0.000075

set i 0

foreach type $sInfo(turnTypeList) \
   {
    set p [lindex $sInfo(turnAvgPList) $i]
    set f [lindex $sInfo(turnMultList) $i]

    set thresh [expr $p*$f]
 
    incr i

    findTurn $type $thresh
   }

findTurn NCap 1.0
findTurn CCap 1.0

showAssignments

exit 0
