#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts stderr "Convert NMRPipe 1D Series into Multi-Column Output."
    puts stderr " -in   inList \[None\]   NMRPipe-Format 1D Spectra."
    puts stderr " -x1   x1     \[0%\]     Start of Region to Use."
    puts stderr " -xn   x1     \[1000%\]  End of Region to Use."
    puts stderr " -hdr                  Print Header (Default)."
    puts stderr " -nohdr                Suppress Header."
    puts stderr " -csv                  Print as Comma Separated Values."
    exit 0
   }

set inList  None
set x1      0%
set xn      100%

set hdrFlag 1
set csvFlag 0

getListArgD $ARGV -in inList

getArgD $ARGC -x1 x1
getArgD $ARGV -xn xn

if {[flagLoc $ARGV -hdr]}   {set hdrFlag 1}
if {[flagLoc $ARGV -nohdr]} {set hdrFlag 0}

if {[flagLoc $ARGV -csv]}   {set csvFlag 1}
if {[flagLoc $ARGV -nocsv]} {set csvFlag 0}

set thisList ""

foreach inName $inList \
   {
    if {![strcasecmp $inName None]} continue

    if {![file exists $inName]} \
       {
        puts stderr "Series2Txt Error: Missing input file $inName"
        exit 1
       }

    lappend thisList $inName
   }

set inList $thisList

if {![llength $inList]} {exit 0}

#
#--------------------------------------------------------------------------

set inCount  [llength $inList]
set sizeList ""

for {set id 1} {$id <= $inCount} {incr id} \
   {
    set inName [lindex $inList [expr $id - 1]]

    readROI -in $inName -ndim 1 -roi $id -x X_AXIS $x1 $xn

    set rInfo [getROIInfo -roi $id]

    set rdata($id) [lindex $rInfo 0]
    set fdata($id) [lindex $rInfo 1]
    set xSize($id) [lindex [lindex $rInfo 2] 0]

    lappend sizeList $xSize($id)
   }

set minSize [expr int([getMin $sizeList])]
set maxSize [expr int([getMax $sizeList])]

puts stderr "Files: $inCount Min Size: $minSize Max Size: $maxSize"

if {$hdrFlag} \
   {
    if {$csvFlag} \
       {
        puts -nonewline "      PPM,          "

        for {set id 1} {$id <= $inCount} {incr id} \
           {
            puts -nonewline [format " D%03d" $id]
            if {$id < $inCount} {puts -nonewline ",         "}
           }

        puts ""
       } \
    else \
       {
        puts -nonewline "VARS   PPM  "
        for {set id 1} {$id <= $inCount} {incr id} {puts -nonewline [format " D%03d" $id]}
        puts ""

        puts -nonewline "FORMAT %6.3f"
        for {set id 1} {$id <= $inCount} {incr id} {puts -nonewline " %13.4e"}
        puts ""
        puts ""
       }
   }

for {set ix 0} {$ix < $minSize} {incr ix} \
   {
    puts -nonewline [format "   %6.3f, " [pnt2spec $fdata(1) CUR_XDIM [expr $ix + 1] ppm]]

    for {set id 1} {$id <= $inCount} {incr id} \
       {
        puts -nonewline [format " %13.4e" [getPtrVal $rdata($id) $ix]]
        if {$csvFlag && $id < $inCount} {puts -nonewline ","}
       }

    puts ""
   }

exit 0
