#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Simple 2D Drawing Application:"
    puts " -in   inName \[test.ft2\] NMRPipe-Format 2D Input Name."
    puts " -hi   hi     \[Auto\]     Height for First Contour Level (Also: -fhi)."
    puts " -min  minVal \[Auto\]     Initial Min Value for 1D Drawing Range."
    puts " -max  minVal \[Auto\]     Initial Max Value for 1D Drawing Range."
    puts "Window Positions:"
    puts " -xOff xOff   \[50\]       X-Axis Offset for Positions, Pixels."
    puts " -yOff yOff   \[50\]       Y-Axis Offset for Positions, Pixels."
    puts "1D Window Sizes:"
    puts " -xSize1D xw  \[600\]      X-Axis Size of 1D Windows, Pixels."
    puts " -ySize1D yw  \[310\]      Y-Axis Size of 1D Windows, Pixels."
    puts "2D Window Size:"
    puts " -xSize2D xw  \[650\]      X-Axis Size of 2D Window, Pixels."
    puts " -ySize2D yw  \[400\]      Y-Axis Size of 2D Window, Pixels."
    exit 0
   }

set xSize1D  600
set ySize1D  310

set xSize2D  650
set ySize2D  400

#
#------------------------------------------------------------------------------

#
# Procedure to move the given 1D cursor.

proc moveCursor { idH idV ix iy } \
{
   global cx1 cxn cy1 cyn

   if {$ix < $cx1 || $ix > $cxn} return
   if {$iy < $cy1 || $iy > $cyn} return

   .c coords $idH $cx1 $iy  $cxn $iy
   .c coords $idV $ix  $cy1 $ix  $cyn

   draw1DV $ix
   draw1DH $iy
}

#
# Procedures to draw the selected 1D vectors. 

proc draw1D { id } \
{
   global minVal maxVal

   drawROI -roi $id -cell $id -zero -bg black -fg white -colorA blue -colorS None \
           -fhi $minVal -lhi $maxVal -axis Bottom
}

proc draw1DH { iy } \
{
  global cx1 cxn cy1 cyn 

  set yLoc [expr 100.0*($cyn - $iy)/($cyn - $cy1)]%

  readROI -roi 2 -ndim 1 -inROI 1 -x X_AXIS -dy Y_AXIS $yLoc
  draw1D 2
  update
}

proc draw1DV { ix } \
{
  global cx1 cxn cy1 cyn

  set xLoc [expr 100.0*($ix - $cx1)/($cxn - $cx1)]%

  readROI -roi 3 -ndim 1 -inROI 1 -x Y_AXIS -dy X_AXIS $xLoc
  draw1D 3
  update
}

#
# Procedure to adjust scale or offset of the current 1D vectors:

proc scale1D { w iy } \
{
   global minVal maxVal iyPrev

   if {$iy > $iyPrev($w)} \
      {
       set minVal [expr $minVal/0.9]
       set maxVal [expr $maxVal/0.9]
      } \
   else \
      {
       set minVal [expr $minVal*0.9]
       set maxVal [expr $maxVal*0.9]
      }

   draw1D 2
   draw1D 3
   update

   set iyPrev($w) $iy
}

proc move1D { w iy } \
{
   global minVal maxVal iyPrev

   set range [expr $maxVal - $minVal]

   if {$iy > $iyPrev($w)} \
      {
       set minVal [expr $minVal + $range*0.05]
       set maxVal [expr $maxVal + $range*0.05]
      } \
   else \
      {
       set minVal [expr $minVal - $range*0.05]
       set maxVal [expr $maxVal - $range*0.05]
      }

   draw1D 2
   draw1D 3
   update

   set iyPrev($w) $iy
}

#
# Default parameters:

set inName   test.ft2

set xOff     50
set yOff     50

set xSize1D  600
set ySize1D  310

set xSize2D  650
set ySize2D  400

set fhi      Auto
set minVal   Auto
set maxVal   Auto 
set snapName None

getArgD $ARGV -in  inName
getArgD $ARGV -fhi fhi
getArgD $ARGV -hi  fhi
getArgD $ARGV -min minVal
getArgD $ARGV -max maxVal

getArgD $ARGV -xSize1D xSize1D
getArgD $ARGV -ySize1D ySize1D

getArgD $ARGV -xSize2D xSize2D
getArgD $ARGV -ySize2D ySize2D

getArgD $ARGV -xOff    xOff
getArgD $ARGV -yOff    yOff

getArgD $ARGV -snap    snapName

if {![file exists [format $inName 1 1]]} \
   {
    puts "Show2D Error Finding Input $inName"
    exit 1
   }

set hiInfo [exec getSpecHi.tcl -in $inName -hi $fhi]
set fhi    [lindex $hiInfo 1]

#
# Creation of Tk windows:

set cx1   10
set cxn   [expr $xSize2D - 75]
set cy1   10
set cyn   [expr $ySize2D - 55]

set qx1   10
set qxn   [expr $xSize1D - 100]
set qy1   10
set qyn   [expr $ySize1D - 70]

toplevel .v
toplevel .h
frame    .cmndFrame

canvas   .c   -width $xSize2D -height $ySize2D -border 0 -closeenough 25
canvas   .h.c -width $xSize1D -height $ySize1D -border 0
canvas   .v.c -width $xSize1D -height $ySize1D -border 0

button   .bQuit -text  Quit -command exit

pack   .c .bQuit -side top -fill x -expand 1
pack   .h.c
pack   .v.c

wm title .  "Drag LEFT Mouse Button on Cursor to Select 1D H/V"
wm title .h "1D H Display \[LEFT: Scale\] \[MIDDLE: Offset\]"
wm title .v "1D V Display \[LEFT: Scale\] \[MIDDLE: Offset\]"

raise .

update

set dx  $xOff
set dy  $yOff
wm geom .  +$dx+$dy

set dx  [expr $xOff + 25] 
set dy  [expr $yOff + $ySize2D + 75]
wm geom .h +$dx+$dy

set dx  [expr $xOff + $xSize2D + 25]
set dy  [expr $yOff + 50] 
wm geom .v +$dx+$dy

update

#
# Create pixmaps for spectral rendering in the three windows.
# Define drawing areas in each of the windows.
# Read and draw the 2D spectrum.

.c   create pixmap 0 0 -width $xSize2D -height $ySize2D -win 1
.h.c create pixmap 0 0 -width $xSize1D -height $ySize1D -win 2
.v.c create pixmap 0 0 -width $xSize1D -height $ySize1D -win 3

defineCell -x $cx1 $cxn -y $cy1 $cyn -cell 1 -verb -win 1
defineCell -x $cx1 $cxn -y $qy1 $qyn -cell 2 -verb -win 2
defineCell -x $cx1 $cxn -y $qy1 $qyn -cell 3 -verb -win 3

newScreen

readROI -ndim 2 -roi 1 -in $inName -x X_AXIS -y Y_AXIS 

set thisMin [getROIStat 1 vMin]
set thisMax [getROIStat 1 vMax]
set range   [expr $thisMax - $thisMin]

if {![strcasecmp $minVal Auto]} {set minVal [expr $thisMin - 0.05*$range]}
if {![strcasecmp $maxVal Auto]} {set maxVal [expr $thisMax + 0.05*$range]}

if {![strcasecmp $fhi Auto]} {set fhi [expr $maxVal/15.0]}

if {[strcasecmp $snapName None]} {drawRec -x 0 $xSize2D -y 0 $ySize2D -win 1 -fg white}
 
drawROI -roi 1 -cell 1 -fhi $fhi \
    -bg black -fg white -color1 multi -color2 multi -colorA gray30 -colorS None \
    -axis Bottom Right

#
# Create the 1D cursors.
# Draw the initial 1D vectors.

set ix     [expr 0.5*($cx1 + $cxn)]
set iy     [expr 0.5*($cy1 + $cyn)]

set iyPrev(.h) $iy
set iyPrev(.v) $iy

set idH [.c create line $cx1 $iy  $cxn $iy  -fill yellow]
set idV [.c create line $ix  $cy1 $ix  $cyn -fill yellow]

.c bind $idH <B1-Motion> "moveCursor $idH $idV %x %y"
.c bind $idV <B1-Motion> "moveCursor $idH $idV %x %y"

bind .h.c <B1-Motion> "scale1D .h %y"
bind .h.c <B2-Motion> "move1D  .h %y"

bind .v.c <B1-Motion> "scale1D .v %y"
bind .v.c <B2-Motion> "move1D  .v %y"

draw1DH $iy
draw1DV $ix

if {[strcasecmp $snapName None]} \
   {
    .c itemconfigure $idH -state hidden
    .c itemconfigure $idV -state hidden
    update

    set im [image create photo]

    winop snap .c $im

    $im write -format gif $snapName

    exit
   }

