#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set inName     csCalc.tab
set refName    None
set offset     0
set sdMult     3.00
set kFrac      0.90
set graphFlag  1
set hmsFlag    1
set singleFlag 0
set verbose    0

set var1      None 
set var2      None

if {[flagLoc $ARGV -help]} \
   {
    puts "Display and Analyze Chemical Shift Tables:"
    puts " -in     csInName  \[csCalc.tab\]  CS Input Table."
    puts " -sd     s         \[3.0\]         Outlier Standard Deviation Count."
    puts " -graph                         Draw CS Graphs (Default)."
    puts " -nograph                       No CS Graphs, Print Statistics Only."
    puts " -nohms                         Ignore Ring Current HM_SHIFT Values."
    puts " -single                        Single Class Only."
    puts " -verb                          Verbose Mode: Print Outlier List."
    puts "Comparison of Two Shift Tables:"
    puts " -ref    csRefName \[None\]        Optional CS Reference Table."
    puts " -off    offset    \[0\]           ResID Offset for Reference."
    puts "Notes:"
    puts " CS Input must have both CS_OBS and SHIFT values."
    exit 1
   }

getArgD $ARGV -in   inName
getArgD $ARGV -ref  refName
getArgD $ARGV -off  offset
getArgD $ARGV -sd   sdMult
getArgD $ARGV -keep kFrac
getArgD $ARGV -var1 var1
getArgD $ARGV -var2 var2

if {[flagLoc $ARGV -graph]}   {set graphFlag 1}
if {[flagLoc $ARGV -nograph]} {set graphFlag 0}

if {[flagLoc $ARGV -hms]}      {set hmsFlag 1}
if {[flagLoc $ARGV -nohms]}    {set hmsFlag 0}

if {[flagLoc $ARGV -single]}   {set singleFlag 1}
if {[flagLoc $ARGV -nosingle]} {set singleFlag 0}

if {[flagLoc $ARGV -verb]}    {set verbose 1}
if {[flagLoc $ARGV -noverb]}  {set verbose 0}

if {![strcasecmp $refName None]} \
   {
    if {![strcasecmp $var1 None]} {set var1 CS_OBS}
    if {![strcasecmp $var2 None]} {set var2 SHIFT}
   } \
else \
   {
    if {![strcasecmp $var1 None]} {set var1 CS_OBS}
    if {![strcasecmp $var2 None]} {set var2 CS_OBS}
   }

initResNames
setFonts

#
#-----------------------------------------------------------------------------

proc absSort { v1 v2 } \
{
   if {abs($v1) > abs($v2)} {return  1}
   if {abs($v1) < abs($v2)} {return -1}

   return 0
}

proc showCSOutliers { atomName xList resIDList obsList predList sdMult kFrac } \
{
    global resInfo verbose

    set nOrig [llength $obsList]
    set dList ""
    set nOut  0 

    for {set i 0} {$i < $nOrig} {incr i} \
       {
        set obs  [lindex $obsList  $i]
        set pred [lindex $predList $i]

        lappend dList [expr $obs - $pred]
       }

    set sList [lsort -command absSort $dList]
    set nBest [expr int($nOrig*$kFrac)]
    
    if {$nOrig < 1 || $nBest < 1} {return 0}

    set bList  [lrange $sList 0 [expr $nBest - 1]]
    set dev    [getStdDev $bList]
    set v      [expr $dev*3.0*sqrt($nBest)/$nBest]
    set dLimit [expr $sdMult*$dev]

    for {set i 0} {$i < $nOrig} {incr i} \
       {
        set d [lindex $dList $i]

        if {abs($d) > $dLimit} \
           {
            set resID   [lindex $resIDList  $i]
            set obs     [lindex $obsList    $i]
            set pred    [lindex $predList   $i]

            set resName $resInfo($resID)

            set fmt "Outlier %4s %4d %s % 7.2f % 7.2f % 7.2f (%.2f)"

            if {$verbose} \
               {
                puts \
                 [format $fmt $atomName $resID $resName $obs $pred $d $dLimit]
               }

            incr nOut 
           }
      }

   return $nOut
}

proc showCSGraph { atomName xList hmsList \
                   obsList predList obsSSList predSSList } \
{
   global inName refName graphCount graphFlag ssFlag hmsFlag refTab axisfont

   set thisName [file root [file tail $inName]]

   set fmt "%-22s %-4s %8.2f %8.2f  % 8.2f % 8.2f   %5.3f %5.3f"

   set csRMS [getVsStat getRMS $obsSSList $predSSList]
   set csAvg [getVsStat getAvg $obsSSList $predSSList]
   set csMin [getVsStat getMin $obsSSList $predSSList]
   set csMax [getVsStat getMax $obsSSList $predSSList]

   set pRCS  [getPearsonR $obsList   $predList]
   set pRSS  [getPearsonR $obsSSList $predSSList]

   puts \
    [format $fmt $thisName $atomName $csRMS $csAvg $csMin $csMax $pRCS $pRSS]

   if {!$graphFlag} return

   incr graphCount

   set w .g$graphCount

   toplevel $w

   frame $w.mid

   set dx [expr 15*$graphCount]
   set dy [expr 15*$graphCount]

   set xSize [expr 8*[winfo screenwidth  .]/12]
   set ySize [expr 2*[winfo screenheight .]/14]
   set sSize [expr 4*$xSize/11]

   if {$refTab} \
      {
       set lab1 [file root [file tail $inName]] 
       set lab2 [file root [file tail $refName]]
      } \
   else \
      {
       set lab1 Obs
       set lab2 Pred
      }

   set fmt "%s (%s - %s): RMS %.2fppm Avg %.2fppm.   %s"
   set msg "Left Button: Show Values for Closest Point."
   set str [format $fmt $atomName $lab1 $lab2 $csRMS $csAvg $msg]

   wm title $w $str
   wm geom  $w =+$dx+$dy

   graph $w.g1 -plotbackground black -width $xSize -height $ySize -title "" \
               -topmargin 5 -bottommargin 5 -leftmargin 70 -rightmargin 100

   graph $w.g2 -plotbackground black -width $xSize -height [expr $ySize + 30] \
               -title "" -topmargin 5 -leftmargin 70 -rightmargin 100

   graph $w.g3 -plotbackground black -width $sSize -height $sSize \
               -title "CS $lab1 vs $lab2 "

   graph $w.g4 -plotbackground black -width $sSize -height $sSize \
               -title "SS $lab1 vs $lab2"

   $w.g3 legend configure -hide 1
   $w.g4 legend configure -hide 1

   $w.g1 axis configure x -hide yes
   $w.g2 axis configure x -step 10

   $w.g1 axis configure y -tickfont $axisfont -ticklength 0.05i
   $w.g2 axis configure x -tickfont $axisfont -ticklength 0.05i
   $w.g2 axis configure y -tickfont $axisfont -ticklength 0.05i

   set minVal  [getMin "[getMin $obsList] [getMin $predList]"]
   set maxVal  [getMax "[getMax $obsList] [getMax $predList]"]

   $w.g3 axis configure x -tickfont $axisfont -ticklength 0.05i \
         -min $minVal -max $maxVal

   $w.g3 axis configure y -tickfont $axisfont -ticklength 0.05i \
         -min $minVal -max $maxVal

   $w.g3 marker create line -coords {-Inf -Inf Inf Inf} \
         -outline red -name zeroLine -under yes

   set minVal  [getMin "[getMin $obsSSList] [getMin $predSSList]"]
   set maxVal  [getMax "[getMax $obsSSList] [getMax $predSSList]"]

   $w.g4 axis configure y -tickfont $axisfont -ticklength 0.05i \
         -min $minVal -max $maxVal

   $w.g4 axis configure x -tickfont $axisfont -ticklength 0.05i \
         -min $minVal -max $maxVal

   $w.g4 marker create line -coords {-Inf -Inf Inf Inf} \
         -outline red -name zeroLine -under yes

   label  $w.i -text ""
   button $w.q -text "Quit this Graph" -command "quitCSGraph $w"

   if {$refTab} \
      {
       set lab1 Obs 
       set lab2 Ref
      }

   $w.g1 element create cs$lab1  -xdata $xList -ydata $obsList \
         -color cyan   -pixels  0 -linewidth 2

   $w.g1 element create cs$lab2 -xdata $xList -ydata $predList \
         -color yellow -pixels 0 -linewidth 2

   $w.g2 element create ss$lab1  -xdata $xList -ydata $obsSSList \
         -color cyan   -pixels 0 -linewidth 2  

   $w.g2 element create ss$lab2 -xdata $xList -ydata $predSSList \
         -color yellow -pixels 0 -linewidth 2

   if {$hmsFlag} \
      {
       $w.g2 element create HMS -xdata $xList -ydata $hmsList \
             -color orchid -pixels  0 -linewidth 2
      }

   $w.g3 element create csVs   -xdata $obsList -ydata $predList \
         -color white -fill blue -pixels 2 -linewidth 0

   $w.g4 element create ssVs   -xdata $obsSSList -ydata $predSSList \
         -color white -fill blue -pixels 2 -linewidth 0

   if {$ssFlag} \
      {
       pack $w.g1 $w.g2 -in $w -side top -expand 1 -fill x   
      } \
   else \
      {
       pack $w.g1 -in $w -side top -expand 1 -fill x   
      }

   pack $w.mid -in $w -side top -anchor c

   if {$ssFlag} \
      {
       pack $w.g3 $w.g4 -in $w.mid -side left -anchor c
      } \
   else \
      {
       pack $w.g3 -in $w.mid -side left -anchor c
      }

   pack $w.i -in $w -side top -expand 1 -fill x   
   pack $w.q -in $w -side top -expand 1 -fill x   

   update

   $w.g1 element bind cs$lab1 \
         <ButtonRelease-1> "showCSNear $atomName cs$lab1 $w %W %x %y"

   $w.g1 element bind cs$lab2 \
         <ButtonRelease-1> "showCSNear $atomName cs$lab2 $w %W %x %y"

   $w.g2 element bind ss$lab1 \
         <ButtonRelease-1> "showCSNear $atomName ss$lab1 $w %W %x %y"

   $w.g2 element bind ss$lab2 \
         <ButtonRelease-1> "showCSNear $atomName ss$lab2 $w %W %x %y"

   if {$hmsFlag} \
      {
       $w.g2 element bind HMS \
             <ButtonRelease-1> "showCSNear $atomName ss$lab2 $w %W %x %y"
      }

   bind $w.g3 <ButtonRelease-1> "+ showCSNear $atomName csVs $w %W %x %y"
   bind $w.g4 <ButtonRelease-1> "+ showCSNear $atomName ssVs $w %W %x %y"

   update
}

proc showCSNear { atomI mode parent w xPix yPix } \
{
   global xList resIDList assList 
   global hmsList obsList predList obsSSList predSSList resInfo
   global hmsFlag ssFlag

# dist 4.12311 x 154.0 index 153 y 119.72052 name csObs

   set status [$w element closest $xPix $yPix nearInfo $mode]

   if {!$status} return

   set i $nearInfo(index)

   set csObs   [lindex $obsList($atomI)    $i]
   set csPred  [lindex $predList($atomI)   $i]
   set ssObs   [lindex $obsSSList($atomI)  $i]
   set ssPred  [lindex $predSSList($atomI) $i]
   set resI    [lindex $resIDList($atomI)  $i] 
   set locI    [lindex $xList($atomI)      $i] 

   set resName $resInfo($resI) 

   if {![strcmp $atomI "*"]} \
      {
       set assI [lindex $assList $i]
       set ass  [format " %3d %s %-4s" $resI $resName $assI]
      } \
   else \
      {
       set ass [format " %3d %s %-4s" $resI $resName $atomI]
      }

   if {$ssFlag} \
      {
       set fmt "%-8s CS Obs: % 7.3f Pred: % 7.3f SS Obs: % 7.3f Pred: % 7.3f"
       set str [format $fmt $ass $csObs $csPred $ssObs $ssPred]
      } \
   else \
      {
       set fmt "%-8s CS Obs: % 7.3f Pred: % 7.3f"
       set str [format $fmt $ass $csObs $csPred]
      }

   if {$hmsFlag} \
      {
       append str [format " HMS: % 7.3f" [lindex $hmsList($atomI) $i]]
      }

   puts $str

   $parent.i configure -text $str

   catch "$parent.g1 marker delete showRes"
   catch "$parent.g2 marker delete showRes"
   catch "$parent.g3 marker delete showObs"
   catch "$parent.g3 marker delete showPred"
   catch "$parent.g4 marker delete showObs"
   catch "$parent.g4 marker delete showPred"

   $parent.g1 marker create line -coords "$resI -Inf $resI Inf" \
              -outline dimgray -name showRes -under yes

   $parent.g2 marker create line -coords "$resI -Inf $resI Inf" \
              -outline dimgray -name showRes -under yes

   $parent.g3 marker create line -coords "$csObs -Inf $csObs Inf" \
              -outline dimgray -name showObs -under yes

   $parent.g3 marker create line -coords "-Inf $csPred Inf $csPred" \
              -outline dimgray -name showPred -under yes

   $parent.g4 marker create line -coords "$ssObs -Inf $ssObs Inf" \
              -outline dimgray -name showObs -under yes

   $parent.g4 marker create line -coords "-Inf $ssPred Inf $ssPred" \
              -outline dimgray -name showPred -under yes

   update
}

proc quitCSGraph { w } \
{
   global graphCount

   catch "destroy $w"

   incr graphCount -1

   if {$graphCount < 1} exit
}

#
#-----------------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts stderr "Error finding CS File $inName"
    exit 1
   }

if {[strcasecmp $refName None] && ![file exists $refName]} \
   {
    puts stderr "Error finding CS Reference File $refName"
    exit 1
   }

if {$graphFlag} \
   {
    wm title . "CS Viewer"
    wm geom  . =220x30-10-10

    button .q -text "Quit All Graphs" -command exit
    pack   .q -side left -expand 1 -fill both
   } \
else \
   {
    wm iconify .
   }

#
#----------------------------------------------------------------------

set graphCount  0
set atomList    ""
set resUsedList ""

set ssFlag  1
set csTab   0
set refTab  0

set csTab    [gdbRead table -in $inName]
set eList    [gdbGetInfo $csTab -parms entries]
set seqList  [getSeqList $csTab r1 rN]
set vList    [gdbGetInfo $csTab -parms varNames]

foreach v {CS_OBS SHIFT} \
   {
    if {[lsearch -exact $vList $v] >= 0} continue
    puts stderr "Error: CS Input table must have CS_OBS and SHIFT values."
    puts stderr "(Use the -verb flag when creating the CS file)."
    exit 1
   }

if {$hmsFlag} \
   {
    if {[lsearch -exact $vList HM_SHIFT] < 0} \
       {
        set hmsFlag 0
       } \
    else \
       {
        set hmsFlag 0

        foreach e $eList \
           {
            if {[gdbGet $e HM_SHIFT] > 999} continue
            set hmsFlag 1
            break
           }
       }
   }

if {[strcasecmp $refName None]} \
   {
    set refTab [gdbRead table -in $refName]
   }

if {$refTab} \
   {
    set rcVar [gdbFindByName $csTab RC_SHIFT]
    if {![llength $rcVar]} {set ssFlag 0}

    set rcVar [gdbFindByName $refTab RC_SHIFT]
    if {![llength $rcVar]} {set ssFlag 0}
   } \
else \
   {
    set rcVar [gdbFindByName $csTab RC_SHIFT]
    if {![llength $rcVar]} {set ssFlag 0}
   }

foreach entry $eList \
   {
    set atomName  [gdbGet $entry ATOMNAME]
    set resID     [gdbGet $entry RESID]
    set rcObs     0.0
    set rcPred    0.0

    set resInfo($resID) [getResName3 [gdbGet $entry RESNAME]]

    if {$hmsFlag} \
       {
        set csHMS [gdbGet $entry HM_SHIFT]
        if {$csHMS > 999} {set csHMS 0.0}
       }

    if {$refTab} \
       {
        set refRes [expr $resID + $offset]
        set cond "RESID == $refRes && !strcasecmp( ATOMNAME, \"$atomName\" )"
        set eRef [gdbSelect -from $refTab -cond $cond]

        if {![llength $eRef]} continue

        set eRef   [lindex $eRef 0]
        set csObs  [gdbGet $entry $var1]
        set csPred [gdbGet $eRef  $var2]

        if {$ssFlag} \
           {
            set rcObs  [gdbGet $entry RC_SHIFT]
            set rcPred [gdbGet $eRef  RC_SHIFT]
           }
       } \
    else \
       {
        set csObs  [gdbGet $entry $var1]
        set csPred [gdbGet $entry $var2]
 
        if {$ssFlag} \
           {
            set rcObs  [gdbGet $entry RC_SHIFT]
            set rcPred $rcObs
           }
       }
 
    if {$csObs > 999 || $csPred > 999 || $rcObs > 999 || $rcPred > 999} \
       {
        continue
       }

    if {[lsearch -exact "HA1 HA2 HA3 HA#" $atomName] >= 0} {set atomName HA}

    if {[lsearch -exact $atomList $atomName] < 0} {lappend atomList $atomName}
    if {[lsearch -exact $resUsedList $resID] < 0} {lappend resUsedList $resID}

    lappend csInfo($atomName,$resID,csObs)   $csObs
    lappend csInfo($atomName,$resID,csPred)  $csPred
    lappend csInfo($atomName,$resID,ssObs)   [expr $csObs  - $rcObs] 
    lappend csInfo($atomName,$resID,ssPred)  [expr $csPred - $rcPred] 

    if {$hmsFlag} {lappend csInfo($atomName,$resID,csHMS) $csHMS}

    set atomName "*"

    lappend csInfo($atomName,$resID,csObs)   $csObs
    lappend csInfo($atomName,$resID,csPred)  $csPred
    lappend csInfo($atomName,$resID,ssObs)   [expr $csObs  - $rcObs]
    lappend csInfo($atomName,$resID,ssPred)  [expr $csPred - $rcPred] 

    if {$hmsFlag} {lappend csInfo($atomName,$resID,csHMS) $csHMS}
   }

set minRes [getMin $resUsedList]
set maxRes [getMax $resUsedList]

for {set resID $r1} {$resID <= $rN} {incr resID} \
   {
    set resInfo($resID) [getResName3 [lindex $seqList [expr $resID - $r1]]]
   }

set assList ""
set i 0

foreach a [concat $atomList *] \
   {
    set xList($a)      ""
    set resIDList($a)  ""
    set resList($a)    ""
    set obsList($a)    ""
    set predList($a)   ""
    set obsSSList($a)  ""
    set predSSList($a) ""
    set hmsList($a)    ""
   }

for {set resID $minRes} {$resID <= $maxRes} {incr resID} \
   {
    foreach a $atomList \
       {
        if {![info exists csInfo($a,$resID,csObs)]} continue

        incr i

        set csObs   [getAvg $csInfo($a,$resID,csObs)]
        set csPred  [getAvg $csInfo($a,$resID,csPred)]
        set ssObs   [getAvg $csInfo($a,$resID,ssObs)]
        set ssPred  [getAvg $csInfo($a,$resID,ssPred)]


        lappend xList($a)      $resID
        lappend resIDList($a)  $resID
        lappend resList($a)    $resInfo($resID) 

        lappend obsList($a)    $csObs
        lappend predList($a)   $csPred

        lappend obsSSList($a)  $ssObs
        lappend predSSList($a) $ssPred

        lappend assList       $a

        lappend xList(*)      $i
        lappend resIDList(*)  $resID
        lappend resList(*)    $resInfo($resID)

        lappend obsList(*)    $csObs
        lappend predList(*)   $csPred

        lappend obsSSList(*)  $ssObs
        lappend predSSList(*) $ssPred

        if {$hmsFlag} \
           {
            set csHMS [getAvg $csInfo($a,$resID,csHMS)]

            lappend hmsList($a) $csHMS
            lappend hmsList(*) $csHMS
           }
       }
   }
 
if {$singleFlag} {set atomList *}

foreach a $atomList \
   { 
    showCSGraph $a $xList($a) $hmsList($a) \
        $obsList($a) $predList($a) $obsSSList($a) $predSSList($a)
   }

foreach a $atomList \
   {
    if {$refTab} \
       {
        showCSOutliers $a $xList($a) $resIDList($a) \
         $obsSSList($a) $predSSList($a) $sdMult $kFrac
       } \
    else \
       {
        showCSOutliers $a $xList($a) $resIDList($a) \
         $obsList($a) $predList($a) $sdMult $kFrac
       }
   }

if {!$graphFlag} exit
