#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set inName     dCalcA.tab
set diHN      -21585.20
set diUser     Default

set absFlag    1
set scaleFlag  1
set allFlag    1
set singleFlag 0

set nH         20
set r1         First
set rN         Last

if {[flagLoc $ARGV -help]} \
   {
    puts "Display Simulated vs Observed Values in a Dipolar Coupling Table"
    puts " -in     dcInName \[dCalcA.tab\]  DC Input Table."
    puts " -nH     nHist    \[20\]          Number of Histogram Bins."
    puts " -r1     r1       \[First\]       First Residue in Range."
    puts " -rN     rN       \[Last\]        Last Residue in Range."
    puts " -single                        Single Class Only."
    puts " -all                           Summarize All Couplings (Default)."
    puts " -noall                         No Summary Window for All Couplings."
    puts "Scaling by Dipolar Interaction (DI) Value:"
    puts " -di     diHN-N   \[-21585.20\]   Scale Factor for Relative Scale"
    puts " -abs                           Use Absolute Value (Default)." 
    puts " -noabs                         Use Signed Scaling." 
    puts " -noscale                       No Scaling at All."
    puts "Notes:"
    puts " DC Input table must have D_OBS, D, and DI values."
    exit 1
   }

getArgD $ARGV -in inName
getArgD $ARGV -nH nH
getArgD $ARGV -di diUser
getArgD $ARGV -r1 r1
getArgD $ARGV -rN rN

if {[flagLoc $ARGV -single]} {set singleFlag 1}

if {[flagLoc $ARGV -all]}      {set allFlag   1}
if {[flagLoc $ARGV -noall]}    {set allFlag   0}

if {[flagLoc $ARGV -abs]}      {set absFlag   1}
if {[flagLoc $ARGV -noabs]}    {set absFlag   0}

if {[flagLoc $ARGV -scale]}    {set scaleFlag 1}
if {[flagLoc $ARGV -noscale]}  {set scaleFlag 0}

setFonts

#
#-----------------------------------------------------------------------------

proc showDCGraph { id \
                   atomNameI atomNameJ xList \
                   obsList predList obsWList predWList } \
{
   global graphCount nH diUser axisfont

   incr graphCount

   if {![strcasecmp $diUser Default]} \
      {
       set relStr "HN-N"
      } \
   else \
      {
       set relStr "DI=${diUser}Hz"
      }

   set w .g$graphCount

   toplevel $w

   if {![strcmp $atomNameI DC] && ![strcmp $atomNameJ DC]} \
      {
       set singleMode 1
      } \
   else \
      {
       set singleMode 0
      }

   set hxList ""
   set hyList ""

   getHist $obsWList hxList hyList $nH
 
   set dx [expr 15*$graphCount]
   set dy [expr 15*$graphCount]

   set xSize [expr 8*[winfo screenwidth  .]/12]
   set ySize [expr 3*[winfo screenheight .]/14]
   set sSize [expr 5*$xSize/11]
   set hSize [expr 8*$sSize/10]

   set dcRMS  [getVsStat getRMS $obsList  $predList]
   set dcWRMS [getVsStat getRMS $obsWList $predWList]
   set n      [llength $predList]

   set fmt    "%s-%s RMS %.2fHz (Relative to $relStr: %.2fHz) %d Couplings.  %s"
   set msg    "Left Button: Show Values for Closest Point."

   wm title $w [format $fmt $atomNameI $atomNameJ $dcRMS $dcWRMS $n $msg] 
   wm geom  $w =+$dx+$dy

   frame $w.mid

   graph $w.g1 -plotbackground black -width $xSize -height $ySize -title "" \
               -topmargin 5 -leftmargin 70 -rightmargin 100 -halo 20

   graph $w.g2 -plotbackground black -width $sSize -height $sSize \
               -title "DC Obs vs Pred" -halo 20

   barchart $w.g3 -plotbackground black -width $sSize -height $hSize \
            -title "DC Histogram (Relative to $relStr)"

   if {[llength $xList] <= 20} \
      {
       set nS 5
      } \
   elseif {[llength $xList] <= 250} \
      {
       set nS 10 
      } \
   else \
      {
       set nS 20 
      }

   $w.g1 axis   configure x -step $nS 
   $w.g3 axis   configure x -step 10.0 

   $w.g2 legend configure -hide 1
   $w.g3 legend configure -hide 1

   $w.g1 axis configure x -tickfont $axisfont -ticklength 0.05i
   $w.g1 axis configure y -tickfont $axisfont -ticklength 0.05i
   $w.g2 axis configure x -tickfont $axisfont -ticklength 0.05i
   $w.g2 axis configure y -tickfont $axisfont -ticklength 0.05i
   $w.g3 axis configure x -tickfont $axisfont -ticklength 0.05i
   $w.g3 axis configure y -tickfont $axisfont -ticklength 0.05i

   if {$singleMode} \
      {
       set minVal  [getMin "[getMin $obsWList] [getMin $predWList]"]
       set maxVal  [getMax "[getMax $obsWList] [getMax $predWList]"]
      } \
   else \
      {
       set minVal  [getMin "[getMin $obsList] [getMin $predList]"]
       set maxVal  [getMax "[getMax $obsList] [getMax $predList]"]
      }

   $w.g2 axis configure x -tickfont $axisfont -ticklength 0.05i \
         -min $minVal -max $maxVal

   $w.g2 axis configure y -tickfont $axisfont -ticklength 0.05i \
         -min $minVal -max $maxVal

   $w.g1 marker create line -coords {-Inf 0.0 Inf 0.0} \
         -outline dimGray -name zeroLine -under yes

   $w.g2 marker create line -coords {-Inf -Inf Inf Inf} \
         -outline red -name zeroLine -under yes

   label  $w.i -text ""
   button $w.q -text "Quit this Graph" -command "quitDCGraph $w"

   if {$singleMode} \
      { 
       $w.g1 element create dcObs  -xdata $xList -ydata $obsWList \
             -color cyan  -pixels  0 -linewidth 2

       $w.g1 element create dcPred -xdata $xList -ydata $predWList \
             -color yellow -pixels 0 -linewidth 2

       $w.g2 element create dcVs   -xdata $obsWList -ydata $predWList \
             -color white -fill blue -pixels 2 -linewidth 0 

       $w.g3 element create dcHist -xdata $hxList -ydata $hyList \
             -foreground green
      } \
   else \
      {
       $w.g1 element create dcObs  -xdata $xList -ydata $obsList \
             -color cyan  -pixels  0 -linewidth 2 

       $w.g1 element create dcPred -xdata $xList -ydata $predList \
             -color yellow -pixels 0 -linewidth 2

       $w.g2 element create dcVs   -xdata $obsList -ydata $predList \
             -color white -fill blue -pixels 2 -linewidth 0 

       $w.g3 element create dcHist -xdata $hxList -ydata $hyList \
             -foreground green
      }

   pack $w.g2 $w.g3 -in $w.mid -side left -anchor c

   pack $w.g1  -in $w -side top -expand 1 -fill x   
   pack $w.mid -in $w -side top -anchor c
   pack $w.i   -in $w -side top -expand 1 -fill x   
   pack $w.q   -in $w -side top -expand 1 -fill x   

   update

   $w.g1 element bind dcObs  \
         <ButtonRelease-1> "showDCNear $id dcObs  $w %W %x %y"

   $w.g1 element bind dcPred \
         <ButtonRelease-1> "showDCNear $id dcPred $w %W %x %y"

   bind $w.g2 \
         <ButtonRelease-1> "+ showDCNear $id dcVs $w %W %x %y"

   update
}

proc showDCNear { id mode parent w xPix yPix } \
{
   global dcInfo

   set status [$w element closest $xPix $yPix nearInfo $mode]

   if {!$status} return

# dist 4.12311 x 154.0 index 153 y -9.72052 name dcObs

   set i $nearInfo(index)

   set dcObs   [lindex $dcInfo($id,obsList)   $i]
   set dcPred  [lindex $dcInfo($id,predList)  $i]
   set dcWObs  [lindex $dcInfo($id,obsWList)  $i]
   set dcWPred [lindex $dcInfo($id,predWList) $i]
   set assInfo [lindex $dcInfo($id,assList)   $i]

   set resI  [lindex $assInfo 0]
   set atomI [lindex $assInfo 1]
   set resJ  [lindex $assInfo 2]
   set atomJ [lindex $assInfo 3]

   set ass [format " %3d %-4s %3d %-4s" $resI $atomI $resJ $atomJ]

   set fmt "%-16s Obs: % 7.3f Pred: % 7.3f ObsW: % 7.3f PredW: % 7.3f"
   set str [format $fmt $ass $dcObs $dcPred $dcWObs $dcWPred] 

   puts $str

   $parent.i configure -text $str

   catch "$parent.g1 marker delete showRes"
   catch "$parent.g2 marker delete showObs"
   catch "$parent.g2 marker delete showPred"

   if {[string first DC:DC $id] >= 0} \
      {
       set dcObs  $dcWObs
       set dcPred $dcWPred
       set resI   [expr $i + 1]
      }

   $parent.g1 marker create line -coords "$resI -Inf $resI Inf" \
              -outline dimGray -name showRes -under yes

   $parent.g2 marker create line -coords "$dcObs -Inf $dcObs Inf" \
              -outline dimGray -name showObs -under yes

   $parent.g2 marker create line -coords "-Inf $dcPred Inf $dcPred" \
              -outline dimGray -name showPred -under yes

   update
}

proc quitDCGraph { w } \
{
   global graphCount

   catch "destroy $w"

   incr graphCount -1

   if {$graphCount < 1} exit
}

proc swapAtomPair { rIPtr aIPtr rJPtr aJPtr } \
{
   upvar $rIPtr resIDI
   upvar $aIPtr atomNameI

   upvar $rJPtr resIDJ
   upvar $aJPtr atomNameJ

   set resIDK    $resIDI
   set atomNameK $atomNameI

   set resIDI    $resIDJ
   set atomNameI $atomNameJ 

   set resIDJ    $resIDK
   set atomNameJ $atomNameK
}

proc getDCPairName { rIPtr aIPtr rJPtr aJPtr } \
{
   global singleFlag

   upvar $rIPtr resIDI
   upvar $aIPtr atomNameI

   upvar $rJPtr resIDJ
   upvar $aJPtr atomNameJ

   if {$singleFlag} {return "DC:DC"}

   if {$resIDI < $resIDJ} \
      {
       swapAtomPair resIDI atomNameI resIDJ atomNameJ 
      }

   if {$resIDI == $resIDJ && [strcasecmp $atomNameI $atomNameJ] > 0} \
      {
       swapAtomPair resIDI atomNameI resIDJ atomNameJ 
      } 

   set offset   [expr $resIDJ - $resIDI]
   set pairName [format "%s,%s:%+d" $atomNameI $atomNameJ $offset]

   return $pairName
}

#
#-----------------------------------------------------------------------------

wm title . "DC Viewer"
wm geom  . =220x30-10-10

button .q -text "Quit All Graphs" -command exit
pack   .q -side left -expand 1 -fill both

if {![file exists $inName]} \
   {
    puts stderr "Error finding DC File $inName"
    exit 1
   }

set dcTab    [gdbRead table -in $inName]
set eList    [gdbGetInfo $dcTab -parms entries]
set vList    [gdbGetInfo $dcTab -parms varNames]

foreach v {D_OBS D DI} \
   {
    if {[lsearch -exact $vList $v] >= 0} continue
    puts stderr "Error: DC Input table must have D_OBS, D, and DI values."
    puts stderr "(Use the -verb flag when creating the DC file)."
    exit 1
   }

set graphCount 0
set pairList   ""
set resList    ""
set nList      ""
set n          0

set minI [getTabMin $dcTab RESID_I]
set minJ [getTabMin $dcTab RESID_J]
set maxI [getTabMax $dcTab RESID_I]
set maxJ [getTabMax $dcTab RESID_J]

if {![strcasecmp $r1 First]} {set r1 [getMin "$minI $minJ"]}
if {![strcasecmp $rN Last]}  {set rN [getMax "$maxI $maxJ"]}
 
foreach entry $eList \
   {
    set atomNameI  [gdbGet $entry ATOMNAME_I]
    set resIDI     [gdbGet $entry RESID_I]
    set atomNameJ  [gdbGet $entry ATOMNAME_J]
    set resIDJ     [gdbGet $entry RESID_J]

    set minIJ      [getMin "$resIDI $resIDJ"]
    set maxIJ      [getMax "$resIDI $resIDJ"]

    set dObs       [gdbGet $entry D_OBS]
    set dPred      [gdbGet $entry D]
    set di         [gdbGet $entry DI]
 
    if {$rN < $minIJ || $r1 > $maxIJ}    continue
    if {$dObs > 999.0 || $dPred > 999.0} continue

    if {$di == 0.0} {set di 1.0}

    incr n

    if {[lsearch -exact "HA1 HA2 HA3 HA#" $atomNameI] >= 0} {set atomNameI HA}
    if {[lsearch -exact "HA1 HA2 HA3 HA#" $atomNameJ] >= 0} {set atomNameJ HA}

    set pairName [getDCPairName resIDI atomNameI resIDJ atomNameJ]

    if {$singleFlag} \
       {
        set id $n
       } \
    else \
       {
        set id $resIDI
       }

    if {[lsearch -exact $pairList $pairName] < 0} {lappend pairList $pairName}
    if {[lsearch -exact $resList  $id]       < 0} {lappend resList  $id}

    lappend nList $n

    if {![strcasecmp $diUser Default]} \
       {
        set thisDI $diHN
       } \
    else \
       {
        set thisDI $diUser
       }

    if {!$scaleFlag} \
       {
        set diScale 1.0
       } \
    else \
       {
        if {$absFlag} \
           {
            set diScale [expr abs($thisDI/$di)]
           } \
        else \
           {
            set diScale [expr $thisDI/$di]
           }
       }

    lappend dcInfo($pairName,$id,dObs)   $dObs
    lappend dcInfo($pairName,$id,dPred)  $dPred
    lappend dcInfo($pairName,$id,dObsW)  [expr $diScale*$dObs]
    lappend dcInfo($pairName,$id,dPredW) [expr $diScale*$dPred]

    set dcInfo($pairName,$id,ass) "$resIDI $atomNameI $resIDJ $atomNameJ"

    if {$singleFlag} continue

    set pairName "DC:DC"

    lappend dcInfo($pairName,$n,dObs)   $dObs
    lappend dcInfo($pairName,$n,dPred)  $dPred
    lappend dcInfo($pairName,$n,dObsW)  [expr $diScale*$dObs]
    lappend dcInfo($pairName,$n,dPredW) [expr $diScale*$dPred]

    set dcInfo($pairName,$n,ass) "$resIDI $atomNameI $resIDJ $atomNameJ"
   }

if {!$singleFlag} {lappend pairList DC:DC}

set i 0

foreach pairName $pairList \
   {
    set id $pairName,$i,

    if {![strcmp $pairName DC:DC]} \
       {
        if {!$allFlag} continue

        set minRes [getMin $nList]
        set maxRes [getMax $nList]
       } \
    else \
       {
        set minRes [getMin $resList]
        set maxRes [getMax $resList]
       }

    set dcInfo($id,xList)      ""
    set dcInfo($id,obsList)    ""
    set dcInfo($id,predList)   ""
    set dcInfo($id,obsWList)   ""
    set dcInfo($id,predWList)  ""
    set dcInfo($id,assList)    ""

    set pairInfo   [split $pairName ",:"]
    set atomNameI  [lindex $pairInfo 0]
    set atomNameJ  [lindex $pairInfo 1]

    for {set resIDI $minRes} {$resIDI <= $maxRes} {incr resIDI} \
       {
        if {![info exists dcInfo($pairName,$resIDI,dObs)]} continue

        set dObs   [getSum $dcInfo($pairName,$resIDI,dObs)]
        set dPred  [getSum $dcInfo($pairName,$resIDI,dPred)]
        set dObsW  [getSum $dcInfo($pairName,$resIDI,dObsW)]
        set dPredW [getSum $dcInfo($pairName,$resIDI,dPredW)]

        lappend dcInfo($id,xList)     $resIDI

        lappend dcInfo($id,obsList)   $dObs
        lappend dcInfo($id,predList)  $dPred
        lappend dcInfo($id,obsWList)  $dObsW
        lappend dcInfo($id,predWList) $dPredW
        lappend dcInfo($id,assList)   $dcInfo($pairName,$resIDI,ass)
       }
   
    showDCGraph $id \
                $atomNameI $atomNameJ $dcInfo($id,xList) \
                $dcInfo($id,obsList) $dcInfo($id,predList) \
                $dcInfo($id,obsWList) $dcInfo($id,predWList)
   }
