#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Summarize Difference Between Two Spectra:"
   echo " -in1    [a.dat]  Specttrum 1."
   echo " -in2    [b.dat]  Specttrum 2."
   echo " -mask   [None]   Optional Mask."
   echo " -dif    [None]   Optional Difference Output."
   echo " -thresh [1.0e-4] Threshold for Zero Intensity."
   exit 1
endif

set inName1  = (`getArgD $argv -in1    a.dat`)
set inName2  = (`getArgD $argv -in2    b.dat`)
set maskName = (`getArgD $argv -mask   None`)
set difName  = (`getArgD $argv -dif    None`)
set thresh   = (`getArgD $argv -thresh 1.0e-4`)

@ fmtCount1  = (`echo $inName1 | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)
@ fmtCount2  = (`echo $inName2 | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

if ($fmtCount1 != $fmtCount2) then
   nmrPrintf -stderr "showDif. Error: if one input is a template name, all input and output must be.\n"
   nmrPrintf -stderr "showDif. Input 1: %s Input 2: %s\n" $inName1 $inName2
   exit 1
endif

if ($fmtCount1 == 0) then
   set tmpName1   = tmp1_$$_masked.dat
   set tmpName2   = tmp2_$$_masked.dat
   set tmpDifName = tmp_$$_dif.dat
else
   set tmpName1   = tmp1_$$_masked/%03d.dat
   set tmpName2   = tmp2_$$_masked/%03d.dat
   set tmpDifName = tmp_$$_dif/%03d.dat
endif

if (`string csame $maskName None`) then
   set thisName1 = $inName1
   set thisName2 = $inName2
else
   @ fmtCountM  = (`echo $maskName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

   if ($fmtCount1 != $fmtCountM) then
      nmrPrintf -stderr "showDif. Error: if one input is a template name, all input and output must be.\n"
      nmrPrintf -stderr "showDif. Input 1: %s Input 2: %s\n" $inName1 $maskName
      exit 1
   endif

   set thisName1 = $tmpName1
   set thisName2 = $tmpName2
   
   addNMR -in1 $inName1 -in2 $maskName  -out $tmpName1 -mult -force
   addNMR -in1 $inName2 -in2 $maskName  -out $tmpName2 -mult -force
endif

if (`string csame $difName None`) then
   set thisDifName = $tmpDifName
else
   set thisDifName = $difName
endif

@ fmtCountD  = (`echo $thisDifName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

if ($fmtCount1 != $fmtCountD) then
   nmrPrintf -stderr "showDif. Error: if one input is a template name, all input and output must be.\n"
   nmrPrintf -stderr "showDif. Input 1: %s Difference Output: %s (%s)\n" $inName1 $difName $thisDifName
   exit 1
endif

addNMR -in1 $thisName1 -in2 $thisName2 -out $thisDifName -sub -force

set sInfo1 = (`specStat.tcl -in $thisName1   -stat vMin vMax N -brief`)
set sInfo2 = (`specStat.tcl -in $thisName2   -stat vMin vMax N -brief`)
set sInfoD = (`specStat.tcl -in $thisDifName -stat vMin vMax N -brief`)

set minVal1 = $sInfo1[1]
set maxVal1 = $sInfo1[2]
set n1      = $sInfo1[3]

set minVal2 = $sInfo2[1]
set maxVal2 = $sInfo2[2]
set n2      = $sInfo2[3]

set minValD = $sInfoD[1]
set maxValD = $sInfoD[2]
set nD      = $sInfoD[3]

if (`IMATH "abs($minVal1) > abs($maxVal1)"`) then
   set maxAbs1 = (`MATH "abs($minVal1)"`)
else
   set maxAbs1 = (`MATH "abs($maxVal1)"`)
endif

if (`IMATH "abs($minVal2) > abs($maxVal2)"`) then
   set maxAbs2 = (`MATH "abs($minVal2)"`)
else
   set maxAbs2 = (`MATH "abs($maxVal2)"`)
endif

if (`IMATH "abs($minValD) > abs($maxValD)"`) then
   set maxAbsD = (`MATH "abs($minValD)"`)
else
   set maxAbsD = (`MATH "abs($maxValD)"`)
endif

if (`IMATH "$maxAbs1 > $thresh"`) then
   set pct1 = (`MATH "100.0*div( $maxAbsD, $maxAbs1 )"`)
else
   set pct1 = 0.0
endif

if (`IMATH "$maxAbs2 > $thresh"`) then
   set pct2 = (`MATH "100.0*div( $maxAbsD, $maxAbs2 )"`)
else
   set pct1 = 0.0
endif

echo $inName1 $sInfo1[1] $sInfo1[2] $inName2 $sInfo2[1] $sInfo2[2] N $sInfo1[3] mask $maskName dif $difName $sInfoD[1] $sInfoD[2] difPct $pct1 $pct2

if ($fmtCount1 == 0) then
   /bin/rm -f  $tmpName1 $tmpName2 $tmpDifName
else
   /bin/rm -rf `string path $tmpName1` `string path $tmpName2` `string path $tmpDifName`
endif

exit 0
