#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

#
# Display Results of autoFit.tcl -sim1D

set sWide [expr [winfo screenwidth  .] - 16]
set sHi   [expr int(0.666*[winfo screenheight .])]
 
wm geom . ${sWide}x${sHi}+10+28

set specName   test.dat 
set simName    sim.ft1
set diffName   dif.ft1
set tabName    nlin.tab

set colorInfo(Spectrum)   cyan
set colorInfo(Model)      yellow
set colorInfo(Difference) red
 
set inList     [lsort -dictionary [glob -nocomplain sim1D/*]]
set colorList  "blue green white orange magenta violet"

set clustID None
set x1      0% 
set xn      100%

if {[flagLoc $ARGV -help]} \
   {
    puts "Display 1D Fitting Results from autoFit.tcl:"
    puts " -specName \[test.dat\]     Input Spectrum Name."
    puts " -simName  \[sim.ft1\]      Input Simulation Name."
    puts " -difName  \[dif.ft1\]      Input Difference Name."
    puts " -tabName  \[nlin.tab\]     Input Difference Name."
    puts " -inList   \[sim1D/*\]      Input Simulation Components."
    puts "Select Region to Display by Coords:" 
    puts " -x1       \[0%\]           Starting Coord of Region to Draw."
    puts " -xn       \[100%\]         Ending Coord of Region to Draw."
    puts "Select Region to Display by Cluster ID:" 
    puts " -clust    \[None\]         Cluster ID Number of Region."
    exit 0
   }

getListArgD $ARGV -inList    inList
getListArgD $ARGV -colorList colorList

getArgD $ARGV -specName specName
getArgD $ARGV -simName  simName
getArgD $ARGV -difName  diffName
getArgD $ARGV -tabName  tabName
getArgD $ARGV -x1       x1
getArgD $ARGV -xn       xn
getArgD $ARGV -clust    clustID 

set inCount [llength $inList]

if {[strcasecmp $clustID None]} \
   {
    set x1 None
    set xn None
   }

set rdata None
set fdata None
set size  0
set pkTab 0

#
#--------------------------------------------------------------------------

proc readPkTab {} \
{
   global tabName pkTab pkList pkCount inList inCount 
   global clustID clustList clustLoc

   set pkList    ""
   set clustList ""

   set pkTab    0
   set pkCount  0
   set clustLoc 0

   if {![strcasecmp $tabName None]} {return 0}
   if {![file exists $tabName]} {return 0}

   set pkTab   [gdbRead table -in $tabName]
   set pkList  [gdbGetInfo $pkTab -parms entries]
   set pkCount [llength $pkList]

   if {$pkCount != $inCount} \
      {
       puts stderr "Warning: Mismatched Peak Table and 1D Input List."
       set pkList  ""
       set pkCount 0
      }

   foreach pk $pkList \
      {
       set id [gdbGet $pk CLUSTID]

       if {[lsearch -exact $clustList $id] < 0} \
          {
           lappend clustList $id
          }
     }

   if {![strcasecmp $clustID None]} \
      {
       set clustLoc 0
       set clustID  [lindex $clustList $clustLoc]
      } \
   else \
      {
       updateRegion
      }

   return $pkCount
}

proc updateRegion {} \
{
   global clustLoc clustID clustList x1 xn pkTab

   set loc [lsearch -exact $clustList $clustID]

   if {$loc >= 0} {set clustLoc $loc}

   set clustID [lindex $clustList $clustLoc]
   set eList   [gdbSelect -from $pkTab -cond "CLUSTID == $clustID"]

   set x1 [getEListStat getMin $eList X1]
   set xn [getEListStat getMax $eList X3]
   set xw [getEListStat getMax $eList XW]

   set dx  [expr 0.1*($xn -  $x1)]
   set dw  [expr 4.0*$xw]

   if {$dx < $dw} {set dx $dw}

   set x1  [expr $x1 - $dx]
   set xn  [expr $xn + $dx]
}

proc updateClust {} \
{
   global clustLoc clustID clustList

   set n [expr [llength $clustList] - 1]

   if {$clustLoc < 0}  {set clustLoc $n} 
   if {$clustLoc > $n} {set clustLoc 0} 

   set clustID [lindex $clustList $clustLoc]

   updateRegion

   read1D
   draw1D
}

proc chooseClust {} \
{
   global clustID clustLoc clustList

   set loc [lsearch -exact $clustList $clustID]

   if {$loc >= 0} {set clustLoc $loc}

   updateClust 
}

proc nextClust {} \
{
   global clustLoc

   incr clustLoc 1
   updateClust
}

proc prevClust {} \
{
   global clustLoc

   incr clustLoc -1
   updateClust
}

proc getPkStr { i } \
{
   global pkList inList

   if {$i > [llength $inList] - 1} \
      {
       set str None
      } \
   elseif {![llength $pkList]} \
      {
       set str "Component [expr $i + 1]"
      } \
   else \
      {
       set fmt "%3d %3d %7.3fppm %9.3fHz %6.2fHz % .3e"

       set e [lindex $pkList $i]

       set id    [gdbGet $e INDEX]
       set clust [gdbGet $e CLUSTID]
       set xPPM  [gdbGet $e X_PPM]
       set xHz   [gdbGet $e X_HZ]
       set xw    [gdbGet $e XW_HZ]
       set hi    [gdbGet $e HEIGHT]

       set str [format $fmt $id $clust $xPPM $xHz $xw $hi] 
      }
}

proc read1D {} \
{
   global x1 xn specName simName diffName inList inCount 

   readROI -ndim 1 -roi 1 -in $specName -x X_AXIS $x1 $xn
   readROI -ndim 1 -roi 2 -in $simName  -x X_AXIS $x1 $xn
   readROI -ndim 1 -roi 3 -in $diffName -x X_AXIS $x1 $xn

   set id 4 

   foreach inName $inList \
      {
       readROI -ndim 1 -roi $id -in $inName -x X_AXIS $x1 $xn
       incr id
      }

   set1DHi
}

proc set1DHi {} \
{
   global minVal maxVal rdata fdata size

   set roiInfo  [getROIInfo -roi 1]
   set rdata    [lindex $roiInfo 0]
   set fdata    [lindex $roiInfo 1]
   set sizeList [lindex $roiInfo 2]
   set size     [lindex $sizeList 0]

   set minVal [vMin $rdata $size]
   set maxVal [vMax $rdata $size]

   set r      [expr $maxVal - $minVal]
   set minVal [expr $minVal - 0.10*$r]
   set maxVal [expr $maxVal + 0.12*$r]
}

proc draw1D {} \
{
   global drawInfo colorInfo colorList minVal maxVal inList

   drawRec -x 0% 100% -y 0% 100% -fg gray20
   drawRec -x 0% 100% -y 0% 100% -fg black -cell 1

   set id 1

   foreach i {Spectrum Model Difference} \
      {
       if {$drawInfo($i)} \
          {
           drawROI -roi $id -cell 1 -bg None -fg $colorInfo($i) \
                   -fhi $minVal -lhi $maxVal
          }

       incr id
      }

   set n [llength $colorList]
   set i 0

   foreach inName $inList \
      {
       if {$drawInfo($i)} \
          {
           set color [lindex $colorList [expr $i % $n]]

           drawROI -roi $id -cell 1 -bg None -fg $color \
                   -fhi $minVal -lhi $maxVal
          }

       incr id
       incr i
      }

   drawROI -roi  1 -cell 1 -axis Bottom -axisOnly
   drawBox -cell 1 -x 0% 100% -y 0% 100% -fg white

   drawPk

   update
}

proc drawPk {} \
{
   global minVal maxVal pkTab pkList pkCount cy1 cyn colorList fdata size

   if {!$pkTab} return

   set xPrev   1000.0
   set yPrev 100000.0

   set xFirst [pnt2spec $fdata X_AXIS 1     ppm]
   set xLast  [pnt2spec $fdata X_AXIS $size ppm]

   set n [llength $colorList]
   set i 0

   for {set i 0} {$i < $pkCount} {incr i} \
      {
       set color [lindex $colorList [expr $i % $n]]
       set entry [lindex $pkList $i]

       set id   [gdbGet $entry INDEX]
       set xPPM [gdbGet $entry X_PPM]
       set hi   [gdbGet $entry HEIGHT]

       if {$xPPM > $xFirst || $xPPM < $xLast} \
          {
           .b$i configure -state disabled -selectcolor gray
           continue
          }

       .b$i configure -state normal -selectcolor $color

       set y1 [hi2pix $hi]
       set y1 [expr $y1 - 4]

       if {abs($xPrev - $xPPM) > 0.5 || abs($yPrev - $y1) > 30} \
          {
           set yn [expr $y1 - 16]
          } \
       else \
          {
           set yn [expr $yPrev - 16]
          }

       set yn [expr $cy1 + 20]

       if {$yn >= $cy1 && $yn <= $cyn} \
          {
           drawLine -cell 1 -roiRef 1 \
              -x ${xPPM}ppm ${xPPM}ppm -y $y1 $yn -fg gray20 
          }

       set off [expr -0.5*[string length $id]]
       set yn  [expr $yn - 4]

       if {$yn >= $cy1 && $yn <= $cyn} \
          {
           drawText -cell 1 -roiRef 1 \
              -xOff $off -x ${xPPM}ppm -y $yn -bg black -fg $color -text $id
          }

       set xPrev $xPPM
       set yPrev $yn
      }
}

proc hi2pix { hi } \
{
   global maxVal minVal cy1 cyn

   set range [expr $maxVal - $minVal]
   set n     [expr $cyn - $cy1]
   set pix   [expr int($cyn - $n*($hi - $minVal)/$range)]

   return $pix
}
proc init1DSelect {} \
{
   global colorInfo colorList drawInfo inCount

   foreach i {Spectrum Model Difference} \
      {
       set drawInfo($i) 1

       checkbutton .b$i -width 50 \
       -text "$i" -anchor w -selectcolor $colorInfo($i) \
       -onvalue 1 -offvalue 0 -command draw1D -variable drawInfo($i)

       pack .b$i -in .right -side top
      }

   set n  [llength $colorList]

   for {set i 0} {$i < $inCount} {incr i} \
      {
       set drawInfo($i) 1

       set str [getPkStr $i]

       if {[strcasecmp $str None]} \
          {
           set color [lindex $colorList [expr $i % $n]]

           checkbutton .b$i -width 50 -font Courier \
            -text $str -anchor w -selectcolor $color \
            -onvalue 1 -offvalue 0 -command draw1D -variable drawInfo($i)

           pack .b$i -in .right -side top
          }
      }
}
 
proc init1DWin {} \
{
   global clustID cx1 cxn cy1 cyn sHi sWide

   set xSize [expr $sWide - 400] 
   set ySize [expr $sHi - 40] 

   set cx1 20
   set cxn [expr $cx1 + $xSize - 40] 
   set cy1 20
   set cyn [expr $cy1 + $ySize - 80] 

   frame .top
   frame .left 
   frame .right
   frame .cmnd

   canvas .c -width $xSize -height $ySize -border 0

   button .next -text Next -command nextClust
   button .prev -text Prev -command prevClust

   label  .clab -text Cluster:
   entry  .clust -width 4 -textvariable clustID
   bind   .clust <Return> "+ chooseClust"

   button .quit -text Quit -command exit

   pack   .c    -in .left -side top 

   pack .clab .clust -in .cmnd -side left
   pack .next .prev .quit -in .cmnd -side left -fill x -expand 1 
   pack .left .right -in . -side left  -expand 1 -fill both
   pack .top  .cmnd -in .left -side top -expand 1 -fill both

   .c create pixmap 0 0 -width $xSize -height $ySize -win 1

   defineCell -x $cx1 $cxn -y $cy1 $cyn -cell 1 -verb -win 1
}

readPkTab
read1D

init1DWin   
init1DSelect
draw1D
