#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Display 2D Volume Analysis Results for Signals of Interest:"
    puts " -in  dirList \[Auto\]     List of Spectral Data Directories."
    puts " -ref refName \[ref.tab\]  Definitions of Signals of Interest."
    puts " -hi  hi                   Contour Height for Region Display."
    puts " -fhi fhi                  Contour Height for Full Spectrum Display." 

    puts "Example contents of ref.tab:"
    puts ""
    puts "VARS   ASS  X_PPM  Y_PPM P_LOW P_HIGH LAB_DX LAB_DY VOL_REF MASK_THRESH
    puts "FORMAT %4s  %8.3f  %8.3f %.1f  %.1f   %8.3f  %8.3f  %d      %10s
    puts ""
    puts "   REF      6.00  105.00  80.0  90.0  0.05  -0.20   0       0.5%
    puts "   P1       5.00  101.00  20.0  32.0  0.20  -0.10   0       6.0%
    puts "   P2       6.50   95.70  10.0  25.0  0.00  -0.25   0       6.0%
    puts "   P3       5.20  104.00  15.0  25.0  0.05  -0.20   1       1.5%
    puts "   P4       5.10   99.00  15.0  25.0  0.05  -0.15   0       8.0%
    puts "   P5       5.30  101.60  70.0  85.0  0.05  -0.35   0       3.0%
    puts "   P6       5.15  104.80  32.0  37.0  0.05  -0.20   0       8.0%
    puts "   P7       3.60   63.20   2.0  10.0  0.05  -0.20   0       7.0%
    exit 0
   }

set thisList [glob -nocomplain *.fid]
set dirList  ""

foreach d [lsort -dictionary $thisList] \
   {
    if {![file isdirectory $d]} continue

    if {![file exists $d/procpar] && ![file exists $d/pulseprogram] && ![file exists $d/pulseprogram.precomp]} continue
    if {![file exists $d/test.ft2] && ![file exists $d/nlin.tab]} continue

    lappend dirList $d
   }

set hi  Auto 
set fhi Auto 
set nhi Auto 

set refName ref.tab
set outName summary.tab
set xName   X_AXIS 
set yName   Y_AXIS 
set xVar    X_PPM 
set yVar    Y_PPM
set refAss  None 
set xyScale 0.1

set plev    20
set nlev    20

set regDX   0.1ppm
set regDY   1.5ppm

set showPkFlag   0
set showFootFlag 0
set snapFlag     0
set exitFlag     0
set updateFlag   1

getListArgD $ARGV -in dirList

getArgD $ARGV -ref refName
getArgD $ARGV -out outName
getArgD $ARGV -hi  hi 
getArgD $ARGV -fhi fhi
getArgD $ARGV -nhi nhi

getArgD $ARGV -xName xName
getArgD $ARGV -yName yName

getArgD $ARGV -xVar  xVar
getArgD $ARGV -yVar  yVar

getArgD $ARGV -xyScale xyScale

if {[flagLoc $ARGV -showPeak]}   {set showPkFlag 1}
if {[flagLoc $ARGV -noshowPeak]} {set showPkFlag 0}

if {[flagLoc $ARGV -showFoot]}   {set showFootFlag 1}
if {[flagLoc $ARGV -noshowFoot]} {set showFootFlag 0}

if {[flagLoc $ARGV -snap]}       {set snapFlag 1}
if {[flagLoc $ARGV -nosnap]}     {set snapFlag 0}

if {[flagLoc $ARGV -exit]}       {set exitFlag 1}
if {[flagLoc $ARGV -noexit]}     {set exitFlag 0}

proc captureScreen { d } \
{
   global snapFlag showFootFlag

   if {!$snapFlag} return

   update
      
   set id [file root $d]
 
   if {![file exists snap]} {catch "file mkdir snap"}
       
   catch {set im [image create photo]}
   catch {winop snap . $im}

   if {$showFootFlag} \
      {       
       set outName snap/fp_$id.gif
      } \
   else \
      {
       set outName snap/$id.gif
      }

   catch {$im write -format gif $outName}

   catch {image delete $im}
}

proc showPeaks { t ass id } \
{
   global xVar yVar xName yName showPkFlag

   if {!$showPkFlag} return
 
   set eList [gdbSelect -from $t -cond "!strcasecmp( ASS, '$ass' )"]

   foreach e $eList \
      {
       set x0 [gdbGet $e X_PPM]
       set y0 [gdbGet $e Y_PPM]
    
       volDrawRec -fg black -cell $id -roi $id \
                  -x [expr $x0 + 0.005]ppm [expr $x0 - 0.005]ppm \
                  -y [expr $y0 + 0.07]ppm [expr $y0 - 0.07]ppm
      } 
}

proc getVolSum { ass } \
{
   global idInfo

   if {!$idInfo($ass,ft) || !$idInfo($ass,volmask) || !$idInfo($ass,work)} {return 0.0}

   set nFT   [getROISize $idInfo($ass,ft)]
   set nMask [getROISize $idInfo($ass,volmask)]
   set nWork [getROISize $idInfo($ass,work)]

   if {$nFT != $nMask || $nFT != $nWork} {return 0.0}

   vvMult [getROIrdata $idInfo($ass,work)] [getROIrdata $idInfo($ass,ft)] [getROIrdata $idInfo($ass,volmask)] $nFT

   set sum [vSum [getROIrdata $idInfo($ass,work)] $nFT] 

   return $sum
}

proc getVolRes { ass } \
{
   global idInfo

   if {!$idInfo($ass,res) || !$idInfo($ass,resmask) || !$idInfo($ass,work)} {return 0.0}

   set nFT   [getROISize $idInfo($ass,res)]
   set nMask [getROISize $idInfo($ass,resmask)]
   set nWork [getROISize $idInfo($ass,work)]

   if {$nFT != $nMask || $nFT != $nWork} {return 0.0}

   vvMult [getROIrdata $idInfo($ass,work)] [getROIrdata $idInfo($ass,res)] [getROIrdata $idInfo($ass,resmask)] $nFT
   vAbs   [getROIrdata $idInfo($ass,work)] $nFT

   set sum [expr 0.5*[vSum [getROIrdata $idInfo($ass,work)] $nFT]]

   return $sum
}

proc getVol { t ass } \
{
   set vol 0.0

   set eList [gdbSelect -from $t -cond "!strcasecmp( ASS, '$ass' )"]

   foreach e $eList \
      {
       set vol [expr $vol + [gdbGet $e VOL]]
      }

   return $vol
}

proc getNear { eAss } \
{
   global pkTab refN refX refY refDX refDY xVar yVar xyScale

   set x   [gdbGet $eAss $xVar]
   set y   [gdbGet $eAss $yVar]

   set ass [gdbGet $eAss ASS]

#  set eList [gdbGetInfo $pkTab -parms entries]
   set eList [gdbSelect -from $pkTab -cond "!strcasecmp( ASS, '$ass' )"]

   set refN  [llength $eList]
   set dist  1.0e+16
   set bestE 0

   set refX  0.0
   set refY  0.0
   set refDX 0.0
   set refDY 0.0

   foreach e $eList \
      {
       set thisX [gdbGet $e X_PPM]
       set thisY [gdbGet $e Y_PPM]
       set dx    [expr $x - $thisX] 
       set dy    [expr $xyScale*($y - $thisY)] 
       set d     [expr $dx*$dx + $dy*$dy]

       if {$d > $dist} continue

       set dist  $d
       set bestE $e
      } 

#  if {[strcasecmp $ass REF]} {return $bestE}

   if {[strcmp $bestE 0]} \
      {
       set refX  [gdbGet $bestE X_PPM]
       set refY  [gdbGet $bestE Y_PPM]
       set refDX [expr $x - $refX]
       set refDY [expr $y - $refY]
      }

   return $bestE
}

proc initVolWin {} \
{
   global xSize ySize ddx ddy qdx qdy pkCount charX charY wishFlag ARGC ARGV

   if {$wishFlag} \
      {
       set xwSize [winfo screenwidth  .]
       set ywSize [winfo screenheight .]
      } \
   else \
      {
       set xwSize 1920
       set xwSize 1080
      }

   getArgD $ARGV -xwSize xwSize
   getArgD $ARGV -ywSize ywSize

   set charX   10
   set charY   16

   getArgD $ARGV -charX charX
   getArgD $ARGV -charY charY

   set qdx [expr 2*$charX]
   set qdy $qdx

   getArgD $ARGV -qdx qdx
   getArgD $ARGV -qdy qdx

   set sdx   [expr int(0.3*$xwSize)]
   set sdy   $sdx

   getArgD $ARGV -sdx sdx
   getArgD $ARGV -sdy sdx
 
   set ddx   [expr int(($xwSize - $sdx)/11.0)] 
   set ddy   $ddx

   getArgD $ARGV -ddx ddx
   getArgD $ARGV -ddy ddy

   set xSize [expr $xwSize -  4*$charX] 
   set ySize [expr $sdy    + 7.5*$charY + $ddy] 

   getArgD $ARGV -xSize xSize
   getArgD $ARGV -ySize ySize
  
   set sx1 $charX 
   set sxn [expr $sx1 + $sdx] 
   set sy1 $charX
   set syn [expr $sy1 + $sdy] 
  
   if {$wishFlag} \
      { 
       canvas .c -width $xSize -height $ySize -border 0
#      button .b -text Quit -command exit 
       pack   .c -side top
#      pack   .b -side top -expand 1 -fill x

       .c create pixmap 0 0 -width $xSize -height $ySize -win 1
      }

   volDefineCell -x $sx1 $sxn -y $sy1 $syn -cell 1 -win 1 -ass full

   if {$wishFlag} \
      {
       wm geom . +20+32
       update
      }
}

proc readSpectrum {} \
{
   global idInfo xVar yVar xName yName regDX regDY dirList dirLoc refEList

   clearIDInfo

   set thisDir [lindex $dirList $dirLoc]

   if {[file exists $thisDir/test.ft2]} \
      {
       readROI -ndim 2 -roi 1 -in $thisDir/test.ft2 -x $xName -y $yName 
       set idInfo(full,ft) 1
      }

   set id 0

   foreach e $refEList \
      {
       incr id

       set xLoc  [gdbGet $e $xVar]
       set yLoc  [gdbGet $e $yVar]
       set ass   [gdbGet $e ASS]

       if {[file exists $thisDir/test.ft2]} \
          { 
           readROI -dx $xName ${xLoc}ppm $regDX -dy $yName ${yLoc}ppm $regDY -roi [expr 100 + $id] \
                   -ndim 2 -in $thisDir/test.ft2

           set idInfo($ass,ft) [expr 100 + $id]

           readROI -dx $xName ${xLoc}ppm $regDX -dy $yName ${yLoc}ppm $regDY -roi [expr 1100 + $id] \
                   -ndim 2 -in $thisDir/test.ft2

           set idInfo($ass,work) [expr 1100 + $id]
          }

       if {[file exists $thisDir/sim.ft2]} \
          { 
           readROI -dx $xName ${xLoc}ppm $regDX -dy $yName ${yLoc}ppm $regDY -roi [expr 200 + $id] \
                   -ndim 2 -in $thisDir/sim.ft2

           set idInfo($ass,sim) [expr 200 + $id]
          }

       if {[file exists $thisDir/dif.ft2]} \
          { 
           readROI -dx $xName ${xLoc}ppm $regDX -dy $yName ${yLoc}ppm $regDY -roi [expr 300 + $id] \
                   -ndim 2 -in $thisDir/dif.ft2

           set idInfo($ass,res) [expr 300 + $id]
          }

       if {[file exists $thisDir/${ass}_volmask.dat]} \
          {
           readROI -dx $xName ${xLoc}ppm $regDX -dy $yName ${yLoc}ppm $regDY -roi [expr 400 + $id] \
                   -ndim 2 -in $thisDir/${ass}_volmask.dat

           set idInfo($ass,volmask) [expr 400 + $id]
          }

       if {[file exists $thisDir/${ass}_resmask.dat]} \
          {
           readROI -dx $xName ${xLoc}ppm $regDX -dy $yName ${yLoc}ppm $regDY -roi [expr 500 + $id] \
                   -ndim 2 -in $thisDir/${ass}_resmask.dat

           set idInfo($ass,resmask) [expr 500 + $id]
          }
      }
}

proc volDrawText args \
{
   global wishFlag

   set ARGV  [concat drawText $args]

   if {!$wishFlag} return
   catch "eval $ARGV"
}

proc volDrawLine args \
{
   global wishFlag

   set ARGV  [concat drawLine $args]

   if {!$wishFlag} return
   catch "eval $ARGV"
}

proc volDrawBox args \
{
   global wishFlag

   set ARGV  [concat drawBox $args]

   if {!$wishFlag} return
   catch "eval $ARGV"
}

proc volDrawCircle args \
{
   global wishFlag

   set ARGV  [concat drawCircle $args]

   if {!$wishFlag} return
   catch "eval $ARGV"
}

proc volDrawCross args \
{
   global wishFlag

   set ARGV  [concat drawCross $args]

   if {!$wishFlag} return
   catch "eval $ARGV"
}

proc volDrawXMark args \
{
   global wishFlag

   set ARGV  [concat drawXMark $args]

   if {!$wishFlag} return
   catch "eval $ARGV"
}

proc volDrawDiamond args \
{
   global wishFlag

   set ARGV  [concat drawDiamond $args]

   if {!$wishFlag} return
   catch "eval $ARGV"
}

proc volDrawRec args \
{
   global wishFlag

   set ARGV  [concat drawRec $args]

   if {!$wishFlag} return
   catch "eval $ARGV"
}

proc volDrawROI args \
{
   global wishFlag

   set ARGV  [concat drawROI $args]

   if {!$wishFlag} return
   catch "eval $ARGV"
}

proc volDefineCell args \
{
   global cellInfo wishFlag

   set ARGV  [concat defineCell $args]
   set ass   None
   set cell  1
   set win   1

   getArgD $ARGV -ass  ass
   getArgD $ARGV -cell cell
   getArgD $ARGV -win  win

   set locX [flagLoc $ARGV -x]
   set locY [flagLoc $ARGV -y]

   set cellInfo($ass,cell) $cell
   set cellInfo($ass,win)  $win

   set cellInfo($ass,x1)   [lindex $ARGV [expr $locX + 1]]
   set cellInfo($ass,xn)   [lindex $ARGV [expr $locX + 2]]
   set cellInfo($ass,y1)   [lindex $ARGV [expr $locY + 1]]
   set cellInfo($ass,yn)   [lindex $ARGV [expr $locY + 2]]

   if {!$wishFlag} return
   catch "eval $ARGV"
}

proc drawSpectrum {} \
{
   global pkCount refAss xName yName xVar yVar pkTab refTab refEList refEListLeft refEListRight
   global xSize ySize ddx ddy hi fhi nhi refN refDX refDY idInfo cellInfo charX charY qdx qdy
   global dirList dirLoc showFootFlag volInfo updateFlag plev nlev

   volTitle
   volDrawRec -x 0 $xSize -y 0 $xSize -fg gray90

   set thisDir [lindex $dirList $dirLoc]

   if {[strcmp $pkTab 0]} {gdbFree $pkTab}

   set pkTab   [gdbRead table -in $thisDir/nlin.tab]
   set refVol  [getVol $pkTab $refAss]
   set refSum  [getVolSum $refAss]
   set refRMS  [getVolRes $refAss]
   set refMin  [expr $refSum - $refRMS]
   set refMax  [expr $refSum + $refRMS]

   set pkCount [gdbGetInfo $pkTab -parms entryCount]
   set nDir    [llength $dirList]

   if {$idInfo(full,ft)} \
      {
       volDrawROI -roi $idInfo(full,ft) -cell 1 -noxGrid -noyGrid -hi $hi -fg black -lwdraw 2 -lw2d 3 \
                  -bg white -color1 darkblue -color2 blue -colorS None -colorA black \
                  -axis Bottom Right -plev $plev -nlev $nlev
      }

   volDrawBox -roi 1 -cell 1 -fg black -lwdraw 3

   set thisList $refEListLeft

   set qx1 [expr $cellInfo(full,xn) + 12*$charX]
   set qy1 [expr $cellInfo(full,y1) + $charY]
   set qxn [expr $qx1 + $ddx]
   set qyn [expr $qy1 + $ddy]

   set id 0

   foreach side {left right} \
      { 
       foreach e $thisList \
          {
           incr id

           set qxn [expr $qx1 + $ddx]
           set qyn [expr $qy1 + $ddy]

           set xLoc  [gdbGet $e $xVar]
           set yLoc  [gdbGet $e $yVar]

           set xPPM  [gdbGet $e $xVar]ppm
           set yPPM  [gdbGet $e $yVar]ppm 
           set pLo   [gdbGet $e P_LOW]
           set pHi   [gdbGet $e P_HIGH]
           set ass   [gdbGet $e ASS]

           set labDX [gdbGet $e LAB_DX]
           set labDY [gdbGet $e LAB_DY]

           set pk    [getNear $e]

           set xPPMStr [format %.2f%+.2fppm $xLoc $refDX]
           set yPPMStr [format %.2f%+.2fppm $yLoc $refDY]

           if {$idInfo($ass,ft)} \
              {
               volDefineCell -cell $idInfo($ass,ft) -win 1 -x $qx1 $qxn -y $qy1 $qyn -ass $ass,ft

               volDrawROI -roi $idInfo($ass,ft) -cell $idInfo($ass,ft) -noxGrid -noyGrid -hi $fhi -fg black -lwdraw 2 \
                          -bg white -color1 darkblue -color2 red -plev $plev -nlev $nlev -colorS none -colorA black

               if {$showFootFlag && $idInfo($ass,volmask)} \
                  {
                   volDrawROI -roi $idInfo($ass,volmask) -cell $idInfo($ass,ft) -bg None -color1 black -lw2d 2 \
                              -plev 1 -noxGrid -noyGrid -fg None -hi 0.5
                  }

               volDrawText -cell $idInfo($ass,ft) -x 4% -y 17% -fg black -text $ass

               showPeaks $pkTab $ass $idInfo($ass,ft)
              }

           if {$idInfo($ass,sim)} \
              {
               volDefineCell -cell $idInfo($ass,sim) -win 1 -x [expr $qx1 + $ddx + $qdx] [expr $qxn + $ddx + $qdx] -y $qy1 $qyn -ass $ass,sim 

               volDrawROI -roi $idInfo($ass,sim) -cell $idInfo($ass,sim) -noxGrid -noyGrid -hi $fhi -fg black -lwdraw 2 \
                          -bg white -color1 darkorchid -color2 magenta -plev $plev -nlev $nlev -colorS none -colorA black

               showPeaks $pkTab $ass $idInfo($ass,sim)
              }

           if {$idInfo($ass,res)} \
              {
               volDefineCell -cell $idInfo($ass,res) -win 1 -x [expr $qx1 + 2*($ddx + $qdx)] [expr $qxn + 2*($ddx + $qdx)] -y $qy1 $qyn -ass $ass,res

               volDrawROI -roi $idInfo($ass,res) -cell $idInfo($ass,res) -noxGrid -noyGrid -hi $nhi -fg black -lwdraw 2 \
                          -bg white -color1 dimGray -color2 firebrick -plev $plev -nlev $nlev -colorS none -colorA black

               if {$showFootFlag && $idInfo($ass,resmask)} \
                  {
                   volDrawROI -roi $idInfo($ass,resmask) -cell $idInfo($ass,res) -bg None -color1 black -plev 1 -lw2d 2 \
                              -noxGrid -noyGrid -fg None -hi 0.5 
                  }

               showPeaks $pkTab $ass $idInfo($ass,res)
              }

           if {$idInfo($ass,ft) && $idInfo($ass,sim) && $idInfo($ass,res)} \
              {
               set thisVol [getVol $pkTab $ass] 
               set thisSum [getVolSum $ass]
               set thisRes [getVolRes $ass]
               set sumMin  [expr $thisSum - $thisRes]
               set sumMax  [expr $thisSum + $thisRes]

               set pctVol  0.0
               set pctSum  0.0
               set pctRes  0.0
               set pctErr  0.0

               if {$refVol != 0.0} {set pctVol [format %.1f [expr 100.0*$thisVol/$refVol]]}
               if {$refSum != 0.0} {set pctSum [format %.1f [expr 100.0*$thisSum/$refSum]]}
               if {$refSum != 0.0} {set pctRes [format %.1f [expr 100.0*$thisRes/$refSum]]}

               if {![strcasecmp $ass $refAss]} \
                  {
                   set pctErr $pctRes
                  } \
               else \
                  {
                   if {$refMin != 0.0 && $refMax != 0.0} \
                      {
                       set pctMin [expr 100.0*$sumMin/$refMax]
                       set pctMax [expr 100.0*$sumMax/$refMin]
                       set pctErr [format %.1f [expr 0.5*abs($pctMax - $pctMin)]]
                      }
                  }

               set volInfo($ass,$dirLoc,pctVol) $pctVol
               set volInfo($ass,$dirLoc,pctSum) $pctSum
               set volInfo($ass,$dirLoc,pctRes) $pctRes
               set volInfo($ass,$dirLoc,pctErr) $pctErr
               set volInfo($ass,$dirLoc,xLoc)   $xLoc
               set volInfo($ass,$dirLoc,yLoc)   $yLoc
               set volInfo($ass,$dirLoc,refDX)  $refDX
               set volInfo($ass,$dirLoc,refDY)  $refDY 
               set volInfo($ass,$dirLoc,refN)   $refN
               set volInfo($ass,$dirLoc,pLo)    $pLo
               set volInfo($ass,$dirLoc,pHi)    $pHi

               set volInfo($ass,$dirLoc,valid)  1

               if {[string length $yPPM] == 10} \
                  {
                   set xOff 126%
                   set yOff 110%
                  } \
               else \
                  {
                   set xOff 129%
                   set yOff 120%
                  }

               set px1   [expr $qxn + 2*($ddx + $qdx) + 20]
               set pxn   [expr $px1 + $ddx]
               set pxp   [expr $px1 + ($pctVol/100.0)*($pxn - $px1)]
               set pxpLo [expr $px1 + ($pLo/100.0)*($pxn - $px1)]
               set pxpHi [expr $px1 + ($pHi/100.0)*($pxn - $px1)]

               volDrawText -cell $idInfo($ass,res) -win 1 -x $xOff -y 17% -fg black -text $xPPMStr 
               volDrawText -cell $idInfo($ass,res) -win 1 -x $yOff  -y 34% -fg black -text $yPPMStr

               if {$pctVol < $pLo || $pctVol > $pHi} \
                  {
                   set color  red
                   set tcolor red 
                  } \
               else \
                  {
                   set color  green 
                   set tcolor black
                  }

               volDrawText -cell $idInfo($ass,res) -win 1 -x 125% -y 58% -fg $tcolor -text ${pctVol}+/-${pctErr}%
           
               volDrawRec -cell $idInfo($ass,res) -win 1 -x $pxpLo $pxpHi -y 61% 79% -fg dimGray 
               volDrawRec -cell $idInfo($ass,res) -win 1 -x $px1 $pxn -y 65% 75% -fg black 
               volDrawRec -cell $idInfo($ass,res) -win 1 -x $px1 $pxp -y 65% 75% -fg $color 

               if {$showFootFlag} \
                  {
                   volDrawText -cell $idInfo($ass,ft)  -win 1 -x 5% -y 95% -fg black -text ${pctSum}%
                   volDrawText -cell $idInfo($ass,res) -win 1 -x 5% -y 95% -fg black -text +/-${pctRes}%
                  }

               set qy1 [expr $qyn + $qdy]
              }
          }

        set qx1 [expr $qxn + 460]
        set qy1 [expr $cellInfo(full,y1) + $charY]

        set thisList $refEListRight
       }

    volDrawText -roi 1 -cell 1 -win 1 xOff -1 -yOff -1 -x 20 -y 32 -fg orchid -text "Spectrum $thisDir"
    volDrawText -roi 1 -cell 1 -win 1 -xOff 0 -yOff  0 -x 20 -y 32 -fg black  -text "Spectrum $thisDir"

    foreach e $refEList \
       {
        set xLoc  [gdbGet $e $xVar]
        set yLoc  [gdbGet $e $yVar]

        set labDX [gdbGet $e LAB_DX]
        set labDY [gdbGet $e LAB_DY]

        set ass   [gdbGet $e ASS]

        volDrawText -roi 1 -cell 1 -win 1 -xOff -1 -yOff -1 \
                 -x [expr $xLoc + $labDX]ppm \
                 -y [expr $yLoc + $labDY]ppm -fg orchid -text $ass

        volDrawText -roi 1 -cell 1 -win 1 \
                 -x [expr $xLoc + $labDX]ppm \
                 -y [expr $yLoc + $labDY]ppm -fg black -text $ass
       }

    if {[llength $refEListLeft]} \
       {
        set e   [lindex $refEListLeft end]
        set ass [gdbGet $e ASS]

        volDrawText -x [expr 1.25*$charX + $cellInfo($ass,ft,x1)]  -y [expr 1.5*$charY + $cellInfo($ass,ft,yn)]  -fg black -text "Experiment"
        volDrawText -x [expr 3*$charX    + $cellInfo($ass,sim,x1)] -y [expr 1.5*$charY + $cellInfo($ass,sim,yn)] -fg black -text "Model"
        volDrawText -x [expr 1*$charX    + $cellInfo($ass,res,x1)] -y [expr 1.5*$charY + $cellInfo($ass,res,yn)] -fg black -text "Residual 2x"
       }

    if {[llength $refEListRight]} \
       {
        set e   [lindex $refEListRight end]
        set ass [gdbGet $e ASS]

        volDrawText -x [expr 1.25*$charX + $cellInfo($ass,ft,x1)]  -y [expr 1.5*$charY + $cellInfo($ass,ft,yn)]  -fg black -text "Experiment"
        volDrawText -x [expr 3*$charX    + $cellInfo($ass,sim,x1)] -y [expr 1.5*$charY + $cellInfo($ass,sim,yn)] -fg black -text "Model"
        volDrawText -x [expr 1*$charX    + $cellInfo($ass,res,x1)] -y [expr 1.5*$charY + $cellInfo($ass,res,yn)] -fg black -text "Residual 2x"
       }

    drawSummary

    if {$updateFlag} update

    captureScreen $thisDir
}

proc drawSummary {} \
{
   global volInfo cellInfo refEList charX charY ddx ddy ddx qdy xSize ySize dirList dirLoc

   set e   [lindex $refEList 0]
   set ass [gdbGet $e ASS]

   set byn [expr $ySize - 3.5*$charY]
   set nE  [llength $refEList]
   set nD  [llength $dirList]

   set x0  [expr 0.5*($cellInfo($ass,x1) + $xSize)]
   set bx1 [expr int($x0 - 0.5*$nE*(1.5*$nD + 6.5*$charX))]
   set bx1 $cellInfo($ass,ft,x1)

   foreach e $refEList \
      {
       set xList ""
       set yList ""
       set vList ""

       set ass  [gdbGet $e ASS]
       set aStr ""

       set bxNext   [expr $bx1 + 14*$charX]
       set badCount 0

       for {set i 0} {$i < $nD} {incr i} \
          {
           set bxn [expr $bx1 + $charX]
           set by1 [expr $byn - $ddy*$volInfo($ass,$i,pctVol)/100.0]

           if {$volInfo($ass,$i,valid)} {lappend vList $volInfo($ass,$i,pctVol)}

           lappend xList $bx1
           lappend xList $bxn

           lappend yList $by1
           lappend yList $byn

           if {$volInfo($ass,$i,pctVol) < $volInfo($ass,$i,pLo) || $volInfo($ass,$i,pctVol) > $volInfo($ass,$i,pHi)} \
              {
               incr badCount

               if {$i == $dirLoc} \
                  {
                   volDrawRec -x $bx1 $bxn -y $by1 $byn -fg red 
                  } \
               else \
                  {
                   volDrawRec -x $bx1 $bxn -y $by1 $byn -fg red4 
                  }
              } \
           else \
              {
               if {$i == $dirLoc} \
                  {
                   volDrawRec -x $bx1 $bxn -y $by1 $byn -fg green 
                  } \
               else \
                  {
                   volDrawRec -x $bx1 $bxn -y $by1 $byn -fg seagreen 
                  }
              }

           set bx1 [expr $bxn + $charX/2]
          }

        if {[llength $vList] == $nD} \
           {
            set avg  [getAvg $vList]
            set da   [expr 0.5*[getRange $vList]]
            set aStr [format "%.1f+/-%.1f%%" $avg $da]
           }

        volDrawLine -x [getMin $xList] [getMax $xList] -y [getMax $yList] [getMax $yList] -fg black -lwdraw 2

        if {$badCount} \
           {
            set thisColor red 
           } \
        else \
           {
            set thisColor black 
           }

        volDrawText -x [expr [getAvg $xList] - 0.5*$charX*[string length $ass]]  -y [expr 1.25*$charY + [getMax $yList]] -fg black -text $ass
        volDrawText -x [expr [getAvg $xList] - 0.5*$charX*([string length $aStr] - 1)] -y [expr 2.50*$charY + [getMax $yList]] -fg $thisColor -text $aStr

       set bx1 [expr $bxn + 6*$charX]
       if {$bx1 < $bxNext} {set bx1 $bxNext}
      }
}

proc parseKey { kStr xPix yPix } \
{
   global dirList dirLoc showPkFlag showFootFlag

   set n [expr [llength $dirList] - 1]
  
   if {![strcasecmp $kStr Q]} \
      {
       okExit exit
      } \
   elseif {[lsearch -exact "1 2 3 4 5 6 7 8 9" $kStr] >= 0} \
      {
       set thisLoc [expr $kStr - 1]

       if {$thisLoc > $n} {set thisLoc $n}
       if {$thisLoc < 0}  {set thisLoc  0}

       if {$thisLoc != $dirLoc} \
          {
           set dirLoc $thisLoc

           readSpectrum
           drawSpectrum
          }
      } \
   elseif {![strcasecmp $kStr N]} \
      {
       set dirLoc [expr $dirLoc + 1]
       if {$dirLoc > $n} {set dirLoc 0}

       readSpectrum
       drawSpectrum
      } \
   elseif {![strcasecmp $kStr P]} \
      {
       set dirLoc [expr $dirLoc - 1]
       if {$dirLoc < 0} {set dirLoc $n}

       readSpectrum
       drawSpectrum
      } \
   elseif {![strcasecmp $kStr F]} \
      {
       if {$showFootFlag} \
          {
           set showFootFlag 0
          } \
       else \
          {
           set showFootFlag 1
          }

       drawSpectrum
      } \
   elseif {![strcasecmp $kStr S]} \
      {
       if {$showPkFlag} \
          {
           set showPkFlag 0
          } \
       else \
          {
           set showPkFlag 1
          }

       drawSpectrum
      }
}

proc mouseWheel { xPix yPix bName } \
{
   global hi fhi nhi

   if {![strcasecmp $bName B4]} \
      {
#      set hi  [expr 1.3*$hi]
       set fhi [expr 1.3*$fhi]
       set nhi [expr 1.3*$nhi]
      } \
   elseif {![strcasecmp $bName B5]} \
      {
#      set hi  [expr $hi/1.3]
       set fhi [expr $fhi/1.3]
       set nhi [expr $nhi/1.3]
      }

   drawSpectrum 
}

proc initBindings {} \
{
   update 
   bind . <KeyPress> "+ parseKey %K %x %y"

   bind .c <Button-4> "mouseWheel %x %y B4"
   bind .c <Button-5> "mouseWheel %x %y B5"
}

proc clearVolInfo {} \
{
   global dirList volInfo refEList

   set n [llength $dirList]

   foreach e $refEList \
      {
       for {set i 0} {$i < $n} {incr i} \
          {
           set ass [gdbGet $e ASS]

           set volInfo($ass,$i,pctVol) 0.0
           set volInfo($ass,$i,pctSum) 0.0
           set volInfo($ass,$i,pctRes) 0.0
           set volInfo($ass,$i,pctErr) 0.0

           set volInfo($ass,$i,xLoc)   0.0 
           set volInfo($ass,$i,yLoc)   0.0 
           set volInfo($ass,$i,refDX)  0.0 
           set volInfo($ass,$i,refDY)  0.0 
           set volInfo($ass,$i,pLo)    0.0 
           set volInfo($ass,$i,pHi)    0.0 
           set volInfo($ass,$i,refN)   0 

           set volInfo($ass,$i,valid)  0
          }
      }
}

proc clearIDInfo {} \
{
   global idInfo cellInfo refEList

   set idInfo(full,ft) 0

   foreach e $refEList \
      {
       set ass [gdbGet $e ASS]
   
       set idInfo($ass,ft)      0
       set idInfo($ass,sim)     0
       set idInfo($ass,res)     0
       set idInfo($ass,volmask) 0
       set idInfo($ass,resmask) 0
       set idInfo($ass,work)    0

       set cellInfo($ass,cell)  0
       set cellInfo($ass,win)   0
       set cellInfo($ass,x1)    0
       set cellInfo($ass,xn)    0
       set cellInfo($ass,y1)    0
       set cellInfo($ass,yn)    0
      }
}

proc initRefTab {} \
{
   global refName refTab refEList refAss assList
   global refEListLeft refEListRight nLeft nRight

   if {![file exists $refName]} \
      {
       puts stderr "Error Finding Signal of Interest Table $refName"
       exit 1
      }

   set refTab   [gdbRead table -in $refName]
   set refEList [gdbGetInfo $refTab -parms entries]
   set eList    [gdbSelect -from $refTab -cond "VOL_REF > 0.5"]

   if {![llength $eList]} \
      {
       puts stderr "Error Finding Reference Volume in $refName (VOL_REF = 1)."
       exit 1
      }

   if {[llength $eList] > 1} \
      {
       puts stderr "Error: More Than One Reference Volume in $refName (VOL_REF = 1)."
       exit 1
      }

   set entry  [lindex $eList 0]
   set refAss [gdbGet $entry ASS]

   set n  [llength $refEList]
   set n1 [expr int($n/2)]
   set n2 [expr $n - $n1]

   set nLeft  [getMax "$n1 $n1"]
   set nRight [getMin "$n1 $n2"]

   set refEListLeft  [lrange $refEList 0 [expr $nLeft - 1]]
   set refEListRight [lrange $refEList $nLeft end]
}

proc volTitle {} \
{
   global refEList dirList dirLoc wishFlag

   if {!$wishFlag} return

   set thisDir [lindex $dirList $dirLoc]
   set nMon    [expr [llength $refEList] - 1]
   set nDir    [llength $dirList]

   wm title . "Volumes from $thisDir  Select: 1-$nDir / N-Next / P-Prev / S-Show Peaks / F-Show Footprints / Q-Quit / Mousewheel Active     2011.355.07.05 www.nmrscience.com"
}

proc saveVolInfo {} \
{
   global volInfo dirList outName refEList

   set error [catch {set outUnit [open $outName w]}]

   if {$error} \
      {
       puts stderr "Error Opening Volume Summary Output $outName"
       return 1
      }

   set n [llength $dirList]

   puts $outUnit ""

   foreach e $refEList \
      {
       set ass   [gdbGet $e ASS]
       set vList ""
       set qList ""

       for {set i 0} {$i < $n} {incr i} \
          {
           if {![info exists volInfo($ass,$i,valid)]} continue
           if {!$volInfo($ass,$i,valid)} continue

           lappend vList $volInfo($ass,$i,pctVol)
           lappend qList $volInfo($ass,$i,pctErr)
          }

        set nV [llength $vList]
        if {!$nV} continue

        set avg [getAvg $vList]
        set q   [getAvg $qList]
        set da  [expr 0.5*[getRange $vList]]
        
        puts $outUnit [format "DATA VOL_INFO  %-4s Observed: %5.1f +/- %.1f  Expected: +/- %.1f Spectra: %d" $ass $avg $da $q $nV]
       }

   puts $outUnit ""
   puts $outUnit "VARS   ASS NSIG X_PPM Y_PPM DX_PPM DY_PPM PCT_SUM PCT_RES PCT_VOL PCT_ERR DIR"
   puts $outUnit "FORMAT %4s %d   %7.3f %7.3f %6.3f  %6.3f  %5.1f   %4.1f   %5.1f   %4.1f   %s"
   puts $outUnit ""
 
   set fmt " %-4s %d %7.3f %7.3f % 6.3f % 6.3f   %5.1f %4.1f   %5.1f %4.1f  %s"

   foreach e $refEList \
      {
       set ass [gdbGet $e ASS]

       for {set i 0} {$i < $n} {incr i} \
          {
           if {![info exists volInfo($ass,$i,valid)]} continue
           if {!$volInfo($ass,$i,valid)} continue

           puts $outUnit \
            [format $fmt $ass \
                    $volInfo($ass,$i,refN)   \
                    $volInfo($ass,$i,xLoc)   \
                    $volInfo($ass,$i,yLoc)   \
                    $volInfo($ass,$i,refDX)  \
                    $volInfo($ass,$i,refDY)  \
                    $volInfo($ass,$i,pctSum) \
                    $volInfo($ass,$i,pctRes) \
                    $volInfo($ass,$i,pctVol) \
                    $volInfo($ass,$i,pctErr) \
                    [lindex $dirList $i]]
          }

        puts $outUnit ""
       }

  catch "close $outUnit"

  puts stderr "Created Volume Summary $outName"

  return 0
}

proc setVolHiInfo {} \
{
   global dirList hi fhi nhi plev nlev

   set thisDir  [lindex $dirList 0]
   set thisName $thisDir/test.ft2

   if {[file exists $thisName]} \
      {
       set mult 7.0

       if {![strcasecmp $hi  Auto]} {set hi 3.0%}
       if {![strcasecmp $fhi Auto]} {set hi 1.5%}
 
       set hiInfo [getSpecContHi $thisName $hi $plev $nlev $mult]
       set hi     [lindex $hiInfo 0]
       puts stderr "$hiInfo"

       set hiInfo [getSpecContHi $thisName $fhi $plev $nlev $mult]
       set fhi    [lindex $hiInfo 0]
       puts stderr "$hiInfo"
      } \
   else \
      {
       if {![strcasecmp $hi  Auto]} {set hi  6.0e+3}
       if {![strcasecmp $fhi Auto]} {set fhi 3.0e+3}
      }

   set nhi [expr $hi*0.5]
}

set refVol  1.0
set refX    0.0
set refY    0.0
set refDX   0.0
set refDY   0.0
set refN    0
set pkCount 0
set dirLoc  0

set refTab  0
set pkTab   0

setVolHiInfo
initRefTab
clearIDInfo
clearVolInfo
readSpectrum
initVolWin
drawSpectrum 
initBindings

set nD [llength $dirList]

for {set dirLoc 0} {$dirLoc < $nD} {incr dirLoc} \
   {
    readSpectrum
    drawSpectrum
   }

saveVolInfo

set updateFlag 1
set dirLoc     0

readSpectrum
drawSpectrum

if {$exitFlag} {exit 0}

