#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "SVD Fit of One or More PCS Tensors:"
    puts " -pdb    pdbName \[ref.pdb\]      PDB Input Table."
    puts " -in     inName  \[pcsObs.tab\]   PCS Input Table."
    puts " -out    outName \[pcsCalc.tab\]  PCS Output Table."
    puts " -ignore                        Ignore SegName/Chain Name Info."
    puts " -verb                          Verbose Mode ON (Default)."
    puts " -noverb                        Verbose Mode OFF."
    exit 0
   }

set pdbName     ref.pdb
set inName      pcsObs.tab
set outName     pcsCalc.tab
set verbose     1

getArgD $ARGV -in  inName
getArgD $ARGV -out outName
getArgD $ARGV -pdb pdbName

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {![file exists $pdbName]} \
   {
    puts stderr "Error Finding PDB Input $pdbName"
    exit 1
   }

if {![file exists $inName]} \
   {
    puts stderr "Error Finding PCS Input $inName"
    exit 1
   }

dynInit

if {[flagLoc $ARGV -ignore]} \
   {
    dynSetChainMode -ignore
    dynSetSegMode   -ignore
   }

set pdbSrc [dynRead -pdb -in $pdbName]
set pcsSrc [dynRead -pcs -in $inName]
set gInfo  [dynSimulate -pcs $pcsSrc -src $pdbSrc -svd]

set rms [lindex $gInfo 0]
set n   [lindex $gInfo 2]
set nn  [lindex $gInfo 4]

set j 6

for {set i 1} {$i <= $nn} {incr i} \
   {
    set k $j

    foreach v {DA DR RX RY RZ} \
       {
        set tInfo($i,$v) [lindex $gInfo $k]
        incr k
       }

    incr j 11
   }

if {$verbose} \
   {
    puts [format "RMS: %7.3f Number of Shifts Calculated: %3d" $rms $n] 

    for {set i 1} {$i <= $nn} {incr i} \
       {
        puts -nonewline [format " Tensor %d: " $i]

        foreach v {DA DR RX RY RZ} \
           {
            if {[string length $tInfo($i,$v)] > 8} \
               {
                puts -nonewline [format " %s: %14s" $v $tInfo($i,$v)]
               } \
            else \
               {
                puts -nonewline [format " %s: %8s" $v $tInfo($i,$v)]
               }
           }

        puts ""
       } 
   } \
else \
   {
    puts [format "%7.3f %3d" $rms $n]

    for {set i 1} {$i <= $nn} {incr i} \
       {
        puts -nonewline [format " %d" $i]

        foreach v {DA DR RX RY RZ} \
           {
            if {[string length $tInfo($i,$v)] > 8} \
               {
                puts -nonewline [format " %14s" $tInfo($i,$v)]
               } \
            else \
               {
                puts -nonewline [format " %8s" $tInfo($i,$v)]
               }
           }

        puts ""
       }
   }

if {[strcasecmp $outName None]} \
   {
    dynWrite -pcs $pcsSrc -out $outName -verb
   }

exit 
