#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Sort Entries in a Non-Uniform Sampling (NUS) Schedule:"
    puts " -sample sName \[Auto\]   NUS Schedule Text File Input."
    puts " -sorted oName \[Auto\]   Sorted NUS Schedule Text File Output."
    puts " -loc    iList \[None\]   Index Numbers in Sorted Order."
    puts " -col    col   \[1\]      Column to Sort (First Column is 0)."
    exit 0
   }

set schedName Auto
set sortName  Auto
set locList   None
set sortCol   1
set verbose   1

set inUnit    stdin
set outUnit   stdout
set msgUnit   stderr

getArgD $ARGV -sample schedName
getArgD $ARGV -sorted sortName
getArgD $ARGV -col    sortCol

getListArgD $ARGV -loc locList

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 1}

if {![strcasecmp $schedName Auto]} \
   {
    set schedName [lindex [findNUSInput .] 0]
    if {![llength $schedName]} {set schedName None}
   }

if {[strcasecmp $schedName stdin]} \
   {
    if {![file exists $schedName]} \
       {
        puts stderr "sortSched. Error Finding NUS Schedule Input $schedName"
        exit 1
       }

    if {![strcasecmp $sortName Auto]} {set sortName ${schedName}.sort}

    set error [catch {set inUnit [open $schedName r]}]
   } \
else \
   {
    if {![strcasecmp $sortName Auto]} {set sortName schedule.sort}

    set inUnit stdin
   }

    
if {$error} \
   {
    puts stderr "sortSched. Error Opening NUS Schedule Input $schedName"
    exit 1
   }

if {[strcasecmp $sortName stdout]} \
   {
    set error [catch {set outUnit [open $sortName w]}]

    if {$error} \
       {
        puts stderr "sortSched. Error Opening Sorted NUS Schedule Output $sortName"
        exit 1
       }

    set msgUnit stdout
   } \
else \
   {
    set outUnit stdout
    set msgUnit stderr
   }

set minLoc [getMin $locList]
set maxLoc [getMax $locList]

set minIncr None 
set maxIncr None 

set nn 0
set n  0

while {[gets $inUnit origLine] >= 0} \
   {
    set  line [string trim $origLine]
    incr n

    set key       [string index [lindex $line 0] 0]
    set itemCount [llength $line]

    if {$sortCol < $itemCount} \
       {
        set itemVal [lindex $line $sortCol]
       } \
    else \
       {
        set itemVal None
       }

    if {[strcmp $key "#"] && [string is integer $itemVal]} \
       {
        if {![strcasecmp $minIncr None]} {set minIncr $itemVal}
        if {![strcasecmp $maxIncr None]} {set maxIncr $itemVal}

        if {$itemVal < $minIncr} {set minIncr $itemVal}
        if {$itemVal > $maxIncr} {set maxIncr $itemVal}

        for {set i 0} {$i < $itemCount} {incr i} \
           {
            set val [lindex $origLine $i]

            if {$i == $sortCol} {set val [expr [lindex $locList $val] - 1]}

            puts -nonewline $outUnit [format "%6s" $val]
           }

        puts $outUnit ""
        incr nn
       } \
    else \
       {
        puts $outUnit $origLine
       }
   }

if {$minLoc != $minIncr + 1 || $maxLoc != $maxIncr + 1} \
   {
    puts stderr "sortSched. Error: Mismatched Index List and Increment Values:"
    puts stderr "           minSortIndex $minLoc maxSortIndex $maxLoc minIncr $minIncr maxIncr $maxIncr"
    puts stderr ""
   }

if {$verbose} \
   {
    puts $msgUnit "inName $schedName outName $sortName lineCount $n sampleCount $nn sortCol $sortCol minSortIndex $minLoc maxSortIndex $maxLoc minIncr $minIncr maxIncr $maxIncr"
   }

if {[strcasecmp $inUnit  stdin]}  {catch "close $inUnit"}
if {[strcasecmp $outUnit stdout]} {catch "close $outUnit"}

exit 0
