#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -backingstore -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set thisVersion 2022.157.12.25

if {[flagLoc $ARGV -help]} \
  {
   puts "Prepare Contour Image Patches of Peaks in a 2D Spectrum ($thisVersion):"
   puts " -in      inName    \[test.ft2\]  Input Spectrum."
   puts " -tab     tabName   \[test.tab\]  Input Peak Table."
   puts " -tmp     tmpName   \[tmp.gif\]   Temporary Output File."
   puts " -outDir  outDir    \[contour\]   Output Directory for Contour Images."
   puts " -outName outName   \[%04d.tiff\] Output Name."
   puts " -useID                         Use Peak Table INDEX for Output Name."
   puts " -nouseID                       Use Region Number for Output Name (Default)."
   puts " -exit                          Exit After Generating Images (Default)."
   puts " -noexit                        Do Not Exit After Generating Images."
   puts "Spectrum and Contour Drawing Parameters:"
   puts " -hi        hi       \[Auto\]      Contour Height."
   puts " -noise     noise    \[Auto\]      Spectrum Noise Level."
   puts " -contMult  contMult \[1.3\]       Contour Spacing Factor."
   puts " -noiseMult hi       \[7\]         Noise Multiplier for Auto Contour Height."
   puts " -plev      plev     \[16\]        Positive Contour Count."
   puts " -nlev      nlev     \[16\]        Negative Contour Count."
   puts "Spectral Limits (Units Hz Pts ppm %):"
   puts " -x1    x1          \[0%\]        X-Axis Start of Analysis Region."
   puts " -xn    xn          \[100%\]      X-Axis End of Analysis Region."
   puts " -y1    y1          \[0%\]        Y-Axis Start of Analysis Region."
   puts " -yn    yn          \[100%\]      Y-Axis End of Analysis Region."
   puts " -dx    dx          \[0.15ppm\]   X-Axis +/- Size of Patch Regions."
   puts " -dy    dy          \[1.50ppm\]   Y-Axis +/- Size of Patch Regions."
   puts "Image Patch Dimensions:"
   puts " -xw    xw          \[256\]       Width of Image Patches, Pixels."
   puts " -yw    yw          \[256\]       Height of Image Patches, Pixels."
   exit 0
  }

set inName  test.ft2
set tabName test.tab
set tmpName tmp.gif

set outDir  contour
set outName %04d.tiff

set delay   0

set x1      0%
set xn      100%
set y1      0%
set yn      100%

set dx      0.15ppm
set dy      1.50ppm

set xw      256
set yw      256

set contMult   1.3
set noiseMult 12.0
set title     Auto 

#
# Hue red: 0.0 Green: 0.3 Blue: 0.666...

set pSat1    1.0
set pSatN    1.0
set pHue1    0.667
set pHueN    0.667
set pVal1    0.75
set pValN    1.00

set nSat1    1.0
set nSatN    1.0
set nHue1    0.0
set nHueN    0.0
set nVal1    0.75
set nValN    1.00

set plev      16
set nlev      16

set hi       Auto 
set noise    Auto

set idFlag   0
set hdrFlag  1
set exitFlag 1
set verbose  1

getArgD $ARGV -in      inName
getArgD $ARGV -tab     tabName
getArgD $ARGV -outDir  outDir
getArgD $ARGV -outName outName
getArgD $ARGV -title   title

getArgD $ARGV -x1      x1
getArgD $ARGV -xn      xn
getArgD $ARGV -y1      y1
getArgD $ARGV -yn      yn

getArgD $ARGV -dx      dx 
getArgD $ARGV -dy      dy 

getArgD $ARGV -xw      xw
getArgD $ARGV -yw      yw

getArgD $ARGV -plev    plev
getArgD $ARGV -nlev    nlev

getArgD $ARGV -hi        hi
getArgD $ARGV -noise     noise

getArgD $ARGV -contMult  contMult
getArgD $ARGV -noiseMult noiseMult
getArgD $ARGV -delay     delay

set userHi    $hi
set userNoise $noise

if {[flagLoc $ARGV -hdr]}     {set hdrFlag 1}
if {[flagLoc $ARGV -hdrOnly]} {set hdrFlag 2}
if {[flagLoc $ARGV -nohdr]}   {set hdrFlag 0}

if {[flagLoc $ARGV -exit]}    {set exitFlag 1}
if {[flagLoc $ARGV -noexit]}  {set exitFlag 0}

if {[flagLoc $ARGV -useID]}   {set idFlag 1}
if {[flagLoc $ARGV -nouseID]} {set idFlag 0}

if {[flagLoc $ARGV -verb]}    {set verbose 1}
if {[flagLoc $ARGV -noverb]}  {set verbose 0}

if {![file exists $inName]} \
   {
    puts stderr "spec2patch. Error finding input spectrum $inName"
    exit 1
   }

set error [rdFDATA $inName fdata]

if {$error} \
   {
    puts stderr "spec2patch. Error reading header in input spectrum $inName"
    exit 1
   }

set tStr     [getParmStr $fdata FDTITLE    NULL_DIM]
set dimCount [getParmI   $fdata FDDIMCOUNT NULL_DIM]
set xSize    [getParmI   $fdata NDSIZE     CUR_XDIM]
set ySize    [getParmI   $fdata NDSIZE     CUR_YDIM]

if {![llength $tStr]} {set tStr None}
if {![strcasecmp $title Auto]} {set title $tStr}

if {$dimCount < 2} \
   {
    puts stderr "spec2patch. Error: input $inName must be a 2D spectrum."
    exit 1
   }

if {![file isdirectory $outDir]} {catch "file mkdir $outDir"} 

if {![file isdirectory $outDir]} \
   {
    puts stderr "spec2patch. Error finding output directory $outDir"
    exit 1
   }

if {![strcasecmp $noise Auto]} {set noise [estSpecNoise $inName]}
if {![strcasecmp $hi    Auto]} {set hi    [expr $noiseMult*$noise]} 

canvas .c -width $xw -height $yw -border 0 -borderwidth 0
pack   .c -side top

update

.c create pixmap 0 0 -width $xw -height $yw -win 1

update

defineCell -x 0 [expr $xw - 1] -y 0 [expr $yw - 1] -cell 1 -win 1

set tab     [gdbRead table -in $tabName]
set varList [gdbGetInfo $tab -parms varNames]
set pkCount [gdbGetInfo $tab -parms entryCount]

set xName   [getParmStr $fdata NDLABEL CUR_XDIM]
set yName   [getParmStr $fdata NDLABEL CUR_YDIM]

set x1Pts   [format %.0f [spec2pnt $fdata CUR_XDIM $x1]]
set xnPts   [format %.0f [spec2pnt $fdata CUR_XDIM $xn]]

set y1Pts   [format %.0f [spec2pnt $fdata CUR_YDIM $y1]]
set ynPts   [format %.0f [spec2pnt $fdata CUR_YDIM $yn]]

set x1Full  [pnt2spec $fdata CUR_XDIM 1      ppm]
set xnFull  [pnt2spec $fdata CUR_XDIM $xSize ppm]
set y1Full  [pnt2spec $fdata CUR_YDIM 1      ppm]
set ynFull  [pnt2spec $fdata CUR_YDIM $ySize ppm]

set x1PPM   [pnt2spec $fdata CUR_XDIM $x1Pts ppm]
set xnPPM   [pnt2spec $fdata CUR_XDIM $xnPts ppm]
set y1PPM   [pnt2spec $fdata CUR_YDIM $y1Pts ppm]
set ynPPM   [pnt2spec $fdata CUR_YDIM $ynPts ppm]

set cond     "X_AXIS >= $x1Pts && X_AXIS <= $xnPts && Y_AXIS >= $y1Pts && Y_AXIS <= $ynPts"
set eList    [gdbSelect -from $tab -cond $cond]
set selCount [llength $eList]

if {$verbose} \
   {
    puts stderr "spec2patch. Input Spectrum: $inName Title: $title"

    if {![strcasecmp $userNoise Auto]} \
       {
        puts -nonewline stderr "spec2patch.  Auto Noise Estimate: [format %.3e $noise]"
       } \
    else \
       {
        puts -nonewline stderr "spec2patch.  Noise: [format %.3e $noise]"
       }

    if {![strcasecmp $userHi Auto]} \
       {
        puts stderr "spec2patch.  Auto Contour Height: [format %.3e $hi]"
       } \
    else \
       {
        puts stderr "spec2patch.  Contour Height: [format %.3e $hi]"
       }

    puts stderr "spec2patch.  X-Axis: $xName $xSize Pts Y-Axis: $yName $ySize Pts."
    puts stderr "spec2patch.  X-Axis Analysis Limits: $x1 $xn ($x1Pts to $xnPts Pts, [format %.3f $x1PPM] to [format %.3f $xnPPM] ppm)."
    puts stderr "spec2patch.  Y-Axis Analysis Limits: $y1 $yn ($y1Pts to $ynPts Pts, [format %.3f $y1PPM] to [format %.3f $ynPPM] ppm)."
    puts stderr "spec2patch.  Peak Table: $tabName Peaks Selected: $selCount Total Peaks: $pkCount:"
   }

if {$verbose} {puts stderr ""}
 
set regID 0

foreach e $eList \
   {
    incr regID

    set pkID [gdbGet $e INDEX]
    set pkX  [gdbGet $e X_PPM]
    set pkY  [gdbGet $e Y_PPM]

    if {$idFlag} \
       {
        set thisName [format %s/$outName $outDir $pkID]
        set thisID   $pkID
       } \
    else \
       {
        set thisName [format %s/$outName $outDir $regID]
        set thisID   $regID
       }

    if {$verbose} \
       {
        puts -nonewline stderr [format "spec2patch.   Peak %3d Region %3d %6.3fppm %7.3fppm" $pkID $regID $pkX $pkY]
        flush stderr
       }

    readROI -ndim 2 -roi 1 -in $inName -dx X_AXIS ${pkX}ppm $dx -dy Y_AXIS ${pkY}ppm $dy

    set rInfo  [getROIInfo -roi 1]

    set fdataR [lindex $rInfo 1]
    set sList  [lindex $rInfo 2]

    set regX1  [format %.3f [pnt2spec $fdataR CUR_XDIM 1                 ppm]]
    set regXN  [format %.3f [pnt2spec $fdataR CUR_XDIM [lindex $sList 0] ppm]]

    set regY1  [format %.3f [pnt2spec $fdataR CUR_YDIM 1                 ppm]]
    set regYN  [format %.3f [pnt2spec $fdataR CUR_YDIM [lindex $sList 1] ppm]]

    if {!$exitFlag} \
       {
        drawROI -roi 1 -cell 1 -bg black -x 0% 100% -y 0% 100% \
                -pHue1 $pHue1 -pHueN $pHueN -pVal1 $pVal1 -pValN $pValN -pSat1 $pSat1 -pSatN $pSatN -pcc $plev -plev $plev \
                -nHue1 $nHue1 -nHueN $nHueN -nVal1 $nVal1 -nValN $nValN -nSat1 $nSat1 -nSatN $nSatN -ncc $nlev -nlev $nlev \
                -color1 multi -color2 multi -hi $hi

        update
       }

    if {$delay > 0} {exec sleep $delay} 

    drawROI -roi 1 -cell 1 -bg black -x 0% 100% -y 0% 100% -nobox -outPixmap $tmpName \
             -pHue1 $pHue1 -pHueN $pHueN -pVal1 $pVal1 -pValN $pValN -pSat1 $pSat1 -pSatN $pSatN -pcc $plev -plev $plev \
             -nHue1 $nHue1 -nHueN $nHueN -nVal1 $nVal1 -nValN $nValN -nSat1 $nSat1 -nSatN $nSatN -ncc $nlev -nlev $nlev \
             -color1 multi -color2 multi -hi $hi

    exec mogrify -type TrueColor -format tiff -write $thisName $tmpName

    if {$verbose} {puts ""}
   }

catch "file delete $tmpName"

if {$exitFlag} {exit 0}

