#!/bin/csh -f

if (`flagLoc $argv -help`) then
   echo "Report Statistics on an NMRPipe-format Spectrum or Region:"
   echo " -in    inName [test.ft2]   Input Spectrum."
   echo " -mult  cMult  [1.0]        Scaling Value for Statistic."
   echo " -fmt   fmt    [None]       Preferred Output Format for Numbers."
   echo " -real                      Report Statistics for Real Data."
   echo " -imag                      Report Statistics for Imaginary Data."
   echo " -complex                   Report Statistics for Complex Data."
   echo " -brief                     Brief Mode (Report Values Only)."
   echo "Region Limits (Use -x.. -y.. etc):"
   echo " -x1    x1     [0%]         Region Start (ppm Hz % pts)."
   echo " -xn    xn     [100%]       Region Start (ppm Hz % pts)."
   echo "Specification of Statistics to Report:"
   echo " -stat vList   [vMin vMax vEstNoise size N ndim]"
   echo "Statistics Vector Function Names:"
   echo " vSum vSumSq vSumPos vSumNeg vSumAbs vAvg vStdDev vRMS"
   echo " vVariance vMin vMax vMaxAbs vMinAbs vMedian vEstNoise"
   echo " istMaxRes N size ndim"
   echo "Notes:"
   echo " 1. Only statistics of the real part of data are reported."
   echo " 2. Values for N size ndim will not be multiplied by cMult." 
   echo " 3. Special Functions:"
   echo ""
   echo "    istMaxRes: Returns cMult*3.0*100.0*vNoiseEst/vMaxAbs."
   echo "    N:         Returns the number of points in the region."
   echo "    size:      Returns a list of the sizes in each dimension."
   echo "    ndim:      Returns the number of dimensions."
   exit 0
endif

set defList   = "vMin vMax vEstNoise size N ndim"
set inName    = (`getArgD     $argv -in       test.ft2`)
set cMult     = (`getArgD     $argv -mult     1.0`)
set fmt       = (`getArgD     $argv -fmt      None`)
set vList     = (`getListArgD $argv -stat --  $defList`)

set briefFlag = 0
set quadType  = real

if (`flagLoc $argv -brief`) then
   set briefFlag = 1
endif

if (`flagLoc $argv -imag`) then
   set quadType = imag 
endif

if (`flagLoc $argv -complex`) then
   set quadType = complex 
endif


if (`flagLoc $argv -nobrief`) then
   set briefFlag = 0
endif

if (`flagLoc $argv -recursive`) then
   set vList  = (`getListArgD $argv -recursive -- $defList`)
   set briefFlag = 1
endif

set x1 = (`getArgD $argv -x1 0%`)
set xn = (`getArgD $argv -xn 100%`)

set y1 = (`getArgD $argv -y1 0%`)
set yn = (`getArgD $argv -yn 100%`)

set z1 = (`getArgD $argv -z1 0%`)
set zn = (`getArgD $argv -zn 100%`)

set a1 = (`getArgD $argv -a1 0%`)
set an = (`getArgD $argv -an 100%`)

#
#-------------------------------------------------------------------------------------

set thisName = (`nmrPrintf $inName 1 1`)

if (!(-e $thisName)) then
#  echo "specStat. Error Finding Input $thisName"
   exit 0
endif

set pipeFlag     = (`getParm -in $thisName -fmt %.0f -parm FDPIPEFLAG -dim NULL_DIM`)
set origDimCount = (`getParm -in $thisName -fmt %.0f -parm FDDIMCOUNT -dim NULL_DIM`)
set dimCount     = $origDimCount

@ fmtCount = (`echo $inName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

if ($pipeFlag == 0) then
   if ($fmtCount == 0) then
      if ($dimCount > 2) then
         set dimCount = 2
      endif
   endif
endif

set rList = ""

foreach vName ($vList)
   set thisMult = $cMult

   if (`string csame $vName N` || `string csame $vName ndim` || `string csame $vName size`) then
      set thisMult = 1.0
   endif

   if (`string csame $vName istMaxRes`) then
      set sList = (`specStat.com $* -recursive vEstNoise vMaxAbs` 0.0 0.0)
      set val   = (`MATH "3.0*100.0*div( $sList[1], $sList[2] )"`)
   else
      if ($dimCount == 1) then
         set val = (`readROI -ndim 1 -in $inName -$quadType -silent -noverb -vec $vName -x X_AXIS $x1 $xn`)
      else if ($dimCount == 2) then
         set val = (`readROI -ndim 2 -in $inName -$quadType -silent -noverb -vec $vName -x X_AXIS $x1 $xn -y Y_AXIS $y1 $yn`)
      else if ($dimCount == 3) then
         set val = (`readROI -ndim 3 -in $inName -$quadType -silent -noverb -vec $vName -x X_AXIS $x1 $xn -y Y_AXIS $y1 $yn -z Z_AXIS $z1 $zn`)
      else
         set val = (`readROI -ndim 4 -in $inName -$quadType -silent -noverb -vec $vName -x X_AXIS $x1 $xn -y Y_AXIS $y1 $yn -z Z_AXIS $z1 $zn -a A_AXIS $a1 $an`)
      endif
   endif

   if (`MATH "$thisMult != 1.0"`) then
      set val = (`MATH "$thisMult*$val"`)
   endif

   if (`string notcsame $fmt None`) then
      set val = (`nmrPrintf $fmt $val`)
   endif

   if ($briefFlag) then
      set rList = ($rList $val)
   else
      set rList = ($rList $vName $val)
   endif
end

echo $rList

exit 0
