#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

#
# PCA Navigator for Multi-Spectrum Viewer.
#-----------------------------------------------------------------

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

proc initSpecViewPCAVars {} \
{
   global objInfo stdfont boldfont

   gdbInit
   gdbCreate dBase

   set objInfo(prog,name)    "SpecViewPCA"
   set objInfo(prog,version) "2022.192.12.00"

   setFonts

   set objInfo(load,id)       1
   set objInfo(load,idH)      2
   set objInfo(load,idV)      3
   set objInfo(load,rdataH)   0
   set objInfo(load,rdataV)   0
   set objInfo(load,hLoc)     1
   set objInfo(load,vLoc)     2
   set objInfo(load,count)    0

   set objInfo(pca,loadName)    load.dat
   set objInfo(pca,specName)    ft/test%05d.dat
   set objInfo(pca,sendName)    specView.tcl
   set objInfo(pca,sendAllowed) 1
   set objInfo(pca,objName)     data
   set objInfo(pca,tabName)     pca.tab
   set objInfo(pca,coordName)   pcaCoords.tab
   set objInfo(pca,clustName)   clust.tab
   set objInfo(pca,plotTitle)   None

   set objInfo(pca,pcaDir)        .
   set objInfo(pca,id)            4
   set objInfo(pca,rdata)         0
   set objInfo(pca,loc)           0
   set objInfo(pca,tab)           0
   set objInfo(pca,count)         0
   set objInfo(pca,specCount)     0
   set objInfo(pca,pcaExtraCount) 0
   set objInfo(pca,drawList)      ""
   set objInfo(pca,moveList)      ""

   set objInfo(pca,charX)         10 
   set objInfo(pca,charY)         16 

   set objInfo(pca,winXSize)      1
   set objInfo(pca,winYSize)      1

   set objInfo(pca,leftMargin)    0
   set objInfo(pca,topMargin)     0
   set objInfo(pca,rightMargin)   0
   set objInfo(pca,bottomMargin)  0

   set objInfo(pca,ix1)      0
   set objInfo(pca,ixn)      0
   set objInfo(pca,iy1)      0
   set objInfo(pca,iyn)      0

   set objInfo(pca,zx1)      0
   set objInfo(pca,zxn)      0
   set objInfo(pca,zy1)      0
   set objInfo(pca,zyn)      0

   set objInfo(pca,x1PPM)     0.0
   set objInfo(pca,x1PPM)     0.0
   set objInfo(pca,totalExpl) 0.0

   set objInfo(pca,explVarList)   ""
   set objInfo(pca,totalVarList)  ""
   set objInfo(pca,compList)      ""

   set objInfo(pca,colorWindow)          None
   set objInfo(pca,drawLabMode)          All
   set objInfo(pca,drawTextFlag)         1
   set objInfo(pca,drawBoxFlag)          1
   set objInfo(pca,drawSelectedOnlyFlag) 1
   set objInfo(pca,drawAxisFlag)         1
   set objInfo(pca,drawScreeFlag)        1

   set objInfo(pca,colorMode)     Title
   set objInfo(pca,bgColor)       black
   set objInfo(pca,fgColor)       white
   set objInfo(pca,axisColor)     gray30
   set objInfo(pca,zoomColor)     white
   set objInfo(pca,defColor)      lightGray
   set objInfo(pca,winColor)      gray10 
   set objInfo(pca,boxColor)      gray30
   set objInfo(pca,stdfont)       $stdfont
   set objInfo(pca,boldfont)      $boldfont
   set objInfo(pca,imageID)       0
   set objInfo(pca,clustFlag)     1
   set objInfo(pca,clustTarget)   5

   set objInfo(expID,maxItemCount) 0

   set objInfo(useItemLabel,None)  1

   foreach item {File Number Last ClusterID} {set objInfo(pca,useLabelItem,$item) 0}

   for {set item 1} {$item <= $objInfo(expID,maxItemCount)} {incr item} {set objInfo(pca,useLabelItem,$item) 0}

   set objInfo(pca,colorList) {
                               black  red   orange  yellow    green   blue   \
                               violet cyan  magenta lightGray dimGray gray30 \
                               gray20 white
                              }

   set objInfo(pca,colorList) \
    {cyan        yellow    red         darkOrchid    green     blue \
     orange      magenta   forestGreen violet pink   firebrick brown}
}

proc initSpecViewPCAArgs {} \
{
   global objInfo ARGC ARGV

if {[flagLoc $ARGV -help]} \
{
 puts "PCA Multivariate Spectral Viewer"
 puts " -in       specName \[ft/test%05d.dat\]  Spectral Data Input."
 puts " -inExtra  specName \[ft/test%05d.dat\]  Extra Spectral Data Input."
 puts " -load     loadName \[load.dat\]         PCA Load Vector Input."
 puts " -pcaTab   tabName  \[pca.tab\]          PCA Statistics Table Input."
 puts " -pcaExtra count    \[0\]                Number of Spectra Being Added to Previous PCA Result."
 puts " -draw     drawList \[None\]             Initial Draw List IDs"
 puts " -title    title    \[None\]             Title Text for Scatterplot (Follow by --)."
 puts "Graphics Settings:"
 puts " -pcaColors cList   \[cyan ...\]        PCA Object Color List."
 puts " -bgColor   bg      \[black\]           Background Color."
 puts " -fgColor   fg      \[white\]           Foreground Color."
 puts " -winColor  wc      \[gray10\]          Window Color."
 puts " -boxColor  bc      \[gray30\]          Outline Box Color."
 puts " -defColor  def     \[lightGray\]       Default Object Color."
 puts " -stdfont   fName   \[STDFONT\]         Standard Font Name."
 puts " -boldfont  fName   \[BOLDFONT\]        Bold Font Name."
 puts " -axis                                Draw PCA Scatterplot Axis (Default)."
 puts " -noaxis                              No PCA Scatterplot Axis."
 puts " -scree                               Draw Explained Variance Graph (Default)."
 puts " -noscree                             No Explained Variance Graph."
 puts "Interprocess Settings:"
 puts " -send     pName \[specView.tcl\]       Spectrum Display Process (Follow by --)."
 puts " -obj      obj   \[data\]               Spectrum Display Object."
 puts " -dir      dir   \[.\]                  PCA Result Directory."
 exit 0
}

   getListArgD $ARGV -draw      objInfo(pca,drawList)
   getListArgD $ARGV -pcaColors objInfo(pca,colorList)

   getArgD $ARGV -in       objInfo(pca,specName)
   getArgD $ARGV -load     objInfo(pca,loadName)
   getArgD $ARGV -pcaTab   objInfo(pca,tabName)
   getArgD $ARGV -obj      objInfo(pca,objName)
   getArgD $ARGV -dir      objInfo(pca,pcaDir)
   getArgD $ARGV -pcaExtra objInfo(pca,pcaExtraCount)

   getArgD $ARGV -bgColor   objInfo(pca,bgColor)
   getArgD $ARGV -boxColor  objInfo(pca,boxColor)
   getArgD $ARGV -defColor  objInfo(pca,defColor)
   getArgD $ARGV -winColor  objInfo(pca,winColor)
   getArgD $ARGV -axisColor objInfo(pca,axisColor)

   getListArgDT $ARGV -send  objInfo(pca,sendName)
   getListArgDT $ARGV -title objInfo(pca,plotTitle)

   set objInfo(pca,sendName)  [string trim $objInfo(pca,sendName)  "{}"]
   set objInfo(pca,plotTitle) [string trim $objInfo(pca,plotTitle) "{}"]

   if {![strcasecmp $objInfo(pca,bgColor) $objInfo(pca,zoomColor)]} \
      {
       set objInfo(pca,zoomColor) black
      }

   if {![strcasecmp $objInfo(pca,bgColor) $objInfo(pca,fgColor)]} \
      {
       set objInfo(pca,fgColor) black
      }

   getArgD $ARGV -zoomColor objInfo(pca,zoomColor)
   getArgD $ARGV -fgColor   objInfo(pca,fgColor)

   getArgD $ARGV -stdfont   objInfo(pca,stdfont)
   getArgD $ARGV -boldfont  objInfo(pca,boldfont)

   set objInfo(pca,stdfont)  [replaceStr $objInfo(pca,stdfont)  _star_ *]
   set objInfo(pca,boldfont) [replaceStr $objInfo(pca,boldfont) _star_ *]

   getArgD $ARGV -colorMode objInfo(pca,colorMode)

   if {[flagLoc $ARGV -axis]}    {set objInfo(pca,drawAxisFlag) 1}
   if {[flagLoc $ARGV -noaxis]}  {set objInfo(pca,drawAxisFlag) 0}

   if {[flagLoc $ARGV -scree]}   {set objInfo(pca,drawScreeFlag) 1}
   if {[flagLoc $ARGV -noscree]} {set objInfo(pca,drawScreeFlag) 0}

   if {[flagLoc $ARGV -clust]}   {set objInfo(pca,clustFlag) 1}
   if {[flagLoc $ARGV -noclust]} {set objInfo(pca,clustFlag) 0}

   getArgD $ARGV -charX objInfo(pca,charX) 
   getArgD $ARGV -charY objInfo(pca,charY)

   getArgD $ARGV -winXSize objInfo(pca,winXSize) 
   getArgD $ARGV -winYSize objInfo(pca,winYSize)

   getArgD $ARGV -leftMargin  objInfo(pca,leftMargin)
   getArgD $ARGV -topMargin   objInfo(pca,topMargin)
   getArgD $ARGV -rightMargin objInfo(pca,rightMargin)
   getArgD $ARGV -bottomMargin objInfo(pca,bottomMargin)

   if {![file exists $objInfo(pca,loadName)]} \
      {
       showError $objInfo(prog,name) "Error Finding load file $objInfo(pca,loadName)"
       quitSpecViewPCA 1
      }

   set thisName [format $objInfo(pca,specName) 1]

   if {![file exists $thisName]} \
      {
       showError $objInfo(prog,name) "Error Finding input file $objInfo(pca,specName)"
       quitSpecViewPCA 1
      }
}

proc initSpecViewScreeWin {} \
{
   global objInfo env

   if {!$objInfo(pca,tab)} return

   set w   .scree
   set obj scree 

   toplevel $w

   wm title $w "Explained Variance"
   wm geom  $w +30+40

   set objInfo($obj,window)       $w
   set objInfo($obj,graphXSize)   400 
   set objInfo($obj,graphYSize)   250 

   graph $w.g1 -plotbackground white \
               -width $objInfo($obj,graphXSize) -height $objInfo($obj,graphYSize) \
               -title "Percent Variance for Each Component"

   graph $w.g2 -plotbackground white \
               -width $objInfo($obj,graphXSize) -height $objInfo($obj,graphYSize) \
               -title "Cumulative Percent Variance"

   $w.g1 legend configure -hide 1
   $w.g2 legend configure -hide 1

   $w.g1 element create g1 -xdata $objInfo(pca,compList) -ydata $objInfo(pca,explVarList) \
         -color black -fill green -pixels 6 -linewidth 3

   $w.g2 element create g1 -xdata $objInfo(pca,compList) -ydata $objInfo(pca,totalVarList) \
         -color black -fill blue -pixels 6 -linewidth 3

   $w.g1 axis configure x -min 1 -step 1 -ticklength 0.05i -title "Component"
   $w.g1 axis configure y -min 0.0 -max 100.0 -ticklength 0.05i

   $w.g2 axis configure x -min 1 -step 1 -ticklength 0.05i -title "Component"
   $w.g2 axis configure y -min 0.0 -max 100.0 -ticklength 0.05i

   pack $w.g1 $w.g2 -in $w -side left -expand 1 -fill x 
}

proc initSpecViewPCAWin {} \
{
   global objInfo env

   set w .
   set obj pca

   wm title $w "$objInfo(prog,name) $objInfo(prog,version)"
   wm geom  $w -30-40

   set objInfo($obj,canvas)       .pca
   set objInfo($obj,window)       $w 

   if {$objInfo($obj,winXSize) < 10} \
      {
       set objInfo($obj,winXSize)     980 
#      if {$objInfo($obj,drawAxisFlag)} {incr objInfo($obj,winXSize) 10}
      }

   if {$objInfo($obj,winYSize) < 10} \
      {
       set objInfo($obj,winYSize)     786
#      if {$objInfo($obj,drawAxisFlag)} {incr objInfo($obj,winYSize) 86}
      }

   if {!$objInfo($obj,leftMargin)} \
      {
       set objInfo($obj,leftMargin)   70
       if {$objInfo($obj,drawAxisFlag)} {incr objInfo($obj,leftMargin)   0}
      }

   if {!$objInfo($obj,rightMargin)} \
      {
       set objInfo($obj,rightMargin)  50
       if {$objInfo($obj,drawAxisFlag)} {incr objInfo($obj,rightMargin)  10}
      }

   if {!$objInfo($obj,topMargin)} \
      {
       set objInfo($obj,topMargin)    10
       if {$objInfo($obj,drawAxisFlag)} {incr objInfo($obj,topMargin)    16}
      }

   if {!$objInfo($obj,bottomMargin)} \
      {
       set objInfo($obj,bottomMargin) 10
       if {$objInfo($obj,drawAxisFlag)} {incr objInfo($obj,bottomMargin) 60}
      }

   set objInfo($obj,drawLabMode)          All 
   set objInfo($obj,drawBoxFlag)          1
   set objInfo($obj,drawTextFlag)         1
   set objInfo($obj,drawSelectedOnlyFlag) 0

   foreach item {Last}                  {set objInfo($obj,useLabelItem,$item) 1}
   foreach item {File Number ClusterID} {set objInfo($obj,useLabelItem,$item) 0}

   for {set item 1} {$item <= $objInfo(expID,maxItemCount)} {incr item} {set objInfo($obj,useLabelItem,$item) 1}

   set objInfo($obj,ix1)    $objInfo($obj,leftMargin)
   set objInfo($obj,iy1)    $objInfo($obj,topMargin) 

   set objInfo($obj,ixn) \
    [expr $objInfo($obj,winXSize) - $objInfo($obj,rightMargin)]

   set objInfo($obj,iyn) \
    [expr $objInfo($obj,winYSize) - $objInfo($obj,bottomMargin)] 

   canvas $objInfo($obj,canvas) \
     -width $objInfo($obj,winXSize) -height $objInfo($obj,winYSize) \
     -bg $objInfo($obj,winColor) -closeenough 10 -border 0

   frame .cmndT
   frame .cmndB
   frame .cmndAll

   pack .cmndT .cmndB -in .cmndAll -side top -expand 1 -fill x
   pack .cmndAll -in . -side top 

#
# Label style:

   frame .titFrame

   label       .labLab -text "Label: "

   radiobutton .labBut -text "All" -variable objInfo($obj,drawLabMode) -value All \
               -selectcolor firebrick -command "toggleSpecViewPCALabels $obj"

   label       .labSpace1 -width 3 -text " "

   radiobutton .labSel  -text "Selected" -variable objInfo($obj,drawLabMode) -value Selected \
               -selectcolor pink -command "toggleSpecViewPCALabels $obj"

   radiobutton .labNone -text "None" -variable objInfo($obj,drawLabMode) -value None \
               -selectcolor pink -command "toggleSpecViewPCALabels $obj"

   label       .labSpace2 -width 1 -text " "

   label       .labShow -text "Show: "

   checkbutton .labTit  -text "Text" -variable objInfo($obj,drawTextFlag)  \
               -selectcolor white -command "updateSpecViewPCALabels $obj"

   checkbutton .labBox  -text "Marker" -variable objInfo($obj,drawBoxFlag) \
               -selectcolor white -command "toggleSpecViewPCALabels $obj"

   checkbutton .labOnly  -text "Selected Only" -variable objInfo($obj,drawSelectedOnlyFlag) \
               -selectcolor white -command "toggleSpecViewPCALabels $obj"

   pack .labLab .labBut .labSel .labNone .labSpace1 .labShow .labTit .labBox .labOnly -in .titFrame -side left

#
# Label pattern:

   frame .modeFrame

   label .modeLabBy -text "Label Contents: "
   label .labSpace3 -width 2 -text " "

   menubutton .modeLabByButton -bitmap @$env(NMRTXT)/caret.map \
              -relief raised -menu .modeLabByButton.menu

   menu .modeLabByButton.menu

   .modeLabByButton.menu add checkbutton \
     -label "Spectrum Number" -variable objInfo($obj,useLabelItem,Number) \
     -onvalue 1 -offvalue 0 -selectcolor green -command "updateLabelContent $obj Number"

   .modeLabByButton.menu add checkbutton \
     -label "File Name" -variable objInfo($obj,useLabelItem,File) \
     -onvalue 1 -offvalue 0 -selectcolor green -command "updateLabelContent $obj File"

   for {set i 1} {$i <= $objInfo(expID,maxItemCount)} {incr i} \
      {
       .modeLabByButton.menu add checkbutton \
         -label "Title Item $i" -variable objInfo($obj,useLabelItem,$i) \
         -onvalue 1 -offvalue 0 -selectcolor green -command "updateLabelContent $obj $i"
      }

   .modeLabByButton.menu add checkbutton \
     -label "Last Title Item" -variable objInfo($obj,useLabelItem,Last) \
     -onvalue 1 -offvalue 0 -selectcolor green -command "updateLabelContent $obj Last"

   if {$objInfo(pca,clustFlag)} \
      {
       .modeLabByButton.menu add checkbutton \
          -label "Cluster" -variable objInfo($obj,useLabelItem,ClusterID) \
          -onvalue 1 -offvalue 0 -selectcolor green -command "updateLabelContent $obj ClusterID"
      }

   pack .modeLabBy .modeLabByButton .labSpace3 -in .modeFrame -side left

#
# Color mode:

   label .modeSpace -text " " -width 2
   label .modeLab   -text "Color By: "

   entry .modeEntry -width 6 -textvariable objInfo(pca,colorMode)
   bind  .modeEntry <Return> "+ setColorMode Current"

   menubutton .modeButton -bitmap @$env(NMRTXT)/caret.map \
              -relief raised -menu .modeButton.menu

   menu .modeButton.menu

   foreach i {Title Number File} \
      {
       .modeButton.menu add command -label $i -command "setColorMode $i"
      }

   for {set i 1} {$i <= $objInfo(expID,maxItemCount)} {incr i} \
      {
       .modeButton.menu add command -label "Item $i" -command "setColorMode $i"
      }

   .modeButton.menu add command -label "Last Item" -command "setColorMode Last"

   if {$objInfo(pca,clustFlag)} \
      {
       .modeButton.menu add command -label "Cluster" -command "setColorMode Cluster"
      }

   pack .modeLab .modeEntry .modeButton .modeSpace -in .modeFrame -side left

#
# Individual color choice:

   frame .colorFrame

   label .lcolor -text " Re-Color:"
   entry .ecolor -width 9 -textvariable objInfo(pca,fgColor)

   button .bcolor -bitmap @$env(NMRTXT)/caret.map -relief raised -fg blue \
          -command colorChoiceON

   pack .lcolor .ecolor .bcolor -in .colorFrame -side left 

#
# Commands:

   label  .cSpaceLeft  -width 2 -text " "
   label  .cSpaceRight -width 4 -text " "

   label  .cSpace1 -width 3 -text " "

   button .snap -bitmap @$env(NMRTXT)/cameraIcon.xbm -command capturePCAScreen

   button .drawPCA   -text "Draw PCA"         -command drawScatter
   button .fullPCA   -text "Full PCA"         -command fullScatter
   button .clearSpec -text "Clear Spectra"    -command clearSpectra
   button .showComp  -text "Show PCA Spectra" -command showComponents
   button .clustSpec -text "Cluster"          -command clusterSpectra
   button .quit      -text "Quit"             -command "quitSpecViewPCA 0"

   if {$objInfo(pca,clustFlag)} \
      {
       pack .cSpaceLeft .snap .cSpace1 .drawPCA .fullPCA .showComp .clustSpec .quit -in .cmndT -side left -anchor w
      } \
   else \
      {
       pack .cSpaceLeft .snap .cSpace1 .drawPCA .fullPCA .showComp .clearSpec .quit -in .cmndT -side left -anchor w
      }

#
# Components:

   frame .pcaFrame

   label .lpcax -text " PCA X:"
   entry .epcax  -textvariable objInfo(load,hLoc) -width 2

   menubutton .bpcax -bitmap @$env(NMRTXT)/caret.map -relief raised \
              -menu .bpcax.menu

   label .lpcay -text " PCA Y:"
   entry .epcay  -textvariable objInfo(load,vLoc) -width 2

   menubutton .bpcay -bitmap @$env(NMRTXT)/caret.map -relief raised \
              -menu .bpcay.menu

   menu .bpcax.menu
   menu .bpcay.menu

   for {set i 1} {$i <= $objInfo(pca,count)} {incr i} \
      {
       .bpcax.menu add command -label $i -command "selectLoad hLoc $i"
       .bpcay.menu add command -label $i -command "selectLoad vLoc $i"
      }

   bind .epcax <Return> choosePCA
   bind .epcay <Return> choosePCA

   pack .lpcax .epcax .bpcax .lpcay .epcay .bpcay .cSpaceRight -in .pcaFrame -side left 

#
# Clusters:

   frame .clustFrame

   label .clustLab   -text "  Clusters:" -anchor e
   label .clustSpace -text " " -width 1

   spinbox .clustCount -width 3 -from 1 -to $objInfo(pca,specCount) -textvariable objInfo(pca,clustTarget)

   pack .clustLab .clustCount -in .clustFrame -side left
 
#
# .titFrame .pcaFrame .cmndFrame .colorFrame .modeFrame

   if {$objInfo(pca,clustFlag)} \
      {
       pack .pcaFrame .clustFrame .colorFrame -in .cmndT -side right
      } \
   else \
      {
       pack .pcaFrame .colorFrame -in .cmndT -side right
      }

   pack .titFrame -in .cmndB -side left -padx 8

   pack .modeFrame -in .cmndB -side right

   pack .pca  -in . -side bottom -expand 1 -fill both 

   update

#
# Bindings:

   set obj pca

   bind .pca <Any-Enter>  "setSpecViewPCATitle pca"
   bind .pca <Any-Leave>  "setSpecViewPCATitle title"

   bind $objInfo($obj,canvas) <Shift-ButtonPress-1>   "zoomON        $obj %x %y"
   bind $objInfo($obj,canvas) <Shift-ButtonRelease-1> "zoomScatter   $obj %x %y"
   bind $objInfo($obj,canvas) <Shift-B1-Motion>       "trackZoom     $obj %x %y"

   bind $objInfo($obj,canvas) <Shift-ButtonPress-2>   "zoomON        $obj %x %y"
   bind $objInfo($obj,canvas) <Shift-ButtonRelease-2> "selectScatter $obj %x %y"
   bind $objInfo($obj,canvas) <Shift-B2-Motion>       "trackZoom     $obj %x %y"

   bind $objInfo($obj,canvas) <Control-ButtonPress-1>   "moveON      $obj %x %y"
   bind $objInfo($obj,canvas) <Control-ButtonRelease-1> "moveOFF     $obj %x %y"
   bind $objInfo($obj,canvas) <Control-B1-Motion>       "trackMove   $obj %x %y"

   bind $objInfo($obj,canvas) <Configure> "resizeSpecViewPCA  $obj %w %h %T"
}

proc capturePCAScreen {} \
{
   global objInfo

   update

   set thisDir snapPCA

   if {![file exists $thisDir]} {catch "file mkdir $thisDir"}

   set im [image create photo]
   winop snap $objInfo(pca,canvas) $im

   set n [llength [glob -nocomplain $thisDir/*.gif]]

   for {set i 1} {$i <= $n + 1} {incr i} \
      {
       set outName [format $thisDir/im%03d.gif $i]
       if {[file exists $outName]} continue

       set objInfo(pca,imageID) $i
       break
      }

   set outName [format $thisDir/im%03d.gif $objInfo(pca,imageID)]

   $im write -format gif $outName

   image delete $im

   wm title . "Saved Snapshot $outName"
   update
}

proc colorChoiceON {} \
{
   global objInfo

   if {[winfo exists $objInfo(pca,colorWindow)]} \
      {
       catch "raise $objInfo(pca,colorWindow)"
       return
      }
 
   set objInfo(pca,colorWindow) [getColorChoice -color $objInfo(pca,fgColor) -command colorChoiceProc]
}

proc colorChoiceProc { color } \
{
   global objInfo

   set objInfo(pca,fgColor) $color
}

proc updateLabelContent { obj item } \
{
   global objInfo

   updateSpecViewPCALabels $obj
}

proc updateSpecViewPCALabels { obj } \
{
   clearScatter
   drawScatter
}

proc toggleSpecViewPCALabels { obj } \
{
   clearScatter
   drawScatter
}

proc resizeSpecViewPCA args \
{
   global objInfo
   
   set obj  [lindex $args 0]

   bind $objInfo($obj,canvas) <Configure> ""

   update

   set objInfo($obj,winXSize) [winfo width  $objInfo($obj,canvas)]
   set objInfo($obj,winYSize) [winfo height $objInfo($obj,canvas)]

   set objInfo(pca,ixn) \
    [expr $objInfo(pca,winXSize) - $objInfo(pca,rightMargin)]

   set objInfo(pca,iyn) \
    [expr $objInfo(pca,winYSize) - $objInfo(pca,bottomMargin)]

   clearScatter
   drawScatter

   bind $objInfo($obj,canvas) <Configure> "resizeSpecViewPCA $obj %w %h %T"
}

proc setSpecViewPCATitle { mode } \
{
   global objInfo

   set str "$objInfo(prog,name) $objInfo(prog,version)"

   switch $mode \
      {
       pca   {
              set fmt "%.3fppm to %.3fppm"
              set f   [format $fmt $objInfo(pca,x1PPM) $objInfo(pca,xnPPM)]
              set str "Dir: $objInfo(pca,pcaDir) $f \[Select/Shift-Zoom\] \[Current/Shift-Colorize\] \[Toggle\]"
             }

       default {}
      } 

   wm title . $str

   update
}

proc moveON { obj xPix yPix } \
{
   global objInfo

   if {$xPix < $objInfo($obj,ix1) || $xPix > $objInfo($obj,ixn)} return
   if {$yPix < $objInfo($obj,iy1) || $yPix > $objInfo($obj,iyn)} return

   set objInfo($obj,moveList) [getClosestLabel $obj $xPix $yPix]
}

proc moveOFF { obj xPix yPix } \
{
   global objInfo

   set objInfo($obj,moveList) ""
}

proc trackMove { obj xPix yPix } \
{
   global objInfo

   if {![llength $objInfo($obj,moveList)]} {return}

   set tID [lindex $objInfo($obj,moveList) 0]

   $objInfo($obj,canvas) coords $tID $xPix $yPix
   update
}

proc getClosestLabel { obj xPix yPix } \
{
   global objInfo

   set gList [$objInfo($obj,canvas) find overlap [expr $xPix - 5] [expr $yPix - 5] [expr $xPix + 5] [expr $yPix + 5]]

   foreach g $gList \
      {
       set t [$objInfo($obj,canvas) gettags $g]
       if {[string first label $t] >= 0} {return $g}
      }

   return ""
}

proc zoomON { obj xPix yPix } \
{
   global objInfo

   if {$xPix < $objInfo($obj,ix1) || $xPix > $objInfo($obj,ixn)} return
   if {$yPix < $objInfo($obj,iy1) || $yPix > $objInfo($obj,iyn)} return

   set objInfo($obj,x1Pix) $xPix
   set objInfo($obj,y1Pix) $yPix
   set objInfo($obj,xnPix) $xPix
   set objInfo($obj,ynPix) $yPix

   $objInfo($obj,canvas) create rectangle \
      $objInfo($obj,x1Pix) $objInfo($obj,y1Pix) \
      $objInfo($obj,xnPix) $objInfo($obj,ynPix) \
      -outline $objInfo(pca,zoomColor) -tags zoom

   update
}

proc trackZoom { obj xPix yPix } \
{
   global objInfo

   if {$xPix < $objInfo($obj,ix1) || $xPix > $objInfo($obj,ixn)} return
   if {$yPix < $objInfo($obj,iy1) || $yPix > $objInfo($obj,iyn)} return

   set objInfo($obj,xnPix) $xPix
   set objInfo($obj,ynPix) $yPix

   $objInfo($obj,canvas) coords zoom \
      $objInfo($obj,x1Pix) $objInfo($obj,y1Pix) \
      $objInfo($obj,xnPix) $objInfo($obj,ynPix)

   update
}

proc fullScatter {} \
{
   global objInfo

   set obj pca

   set objInfo($obj,zx1) $objInfo(load,minH)
   set objInfo($obj,zxn) $objInfo(load,maxH)
   set objInfo($obj,zy1) $objInfo(load,minV)
   set objInfo($obj,zyn) $objInfo(load,maxV)

   clearScatter
   drawScatter
}

proc zoomScatter { obj xPix yPix } \
{
   global objInfo

   $objInfo($obj,canvas) delete zoom

   if {$objInfo($obj,x1Pix) > $objInfo($obj,xnPix)} \
      {
       set itemp               $objInfo($obj,x1Pix)
       set objInfo($obj,x1Pix) $objInfo($obj,xnPix)
       set objInfo($obj,xnPix) $itemp
      }

   if {$objInfo($obj,y1Pix) > $objInfo($obj,ynPix)} \
      {
       set itemp               $objInfo($obj,y1Pix)
       set objInfo($obj,y1Pix) $objInfo($obj,ynPix)
       set objInfo($obj,ynPix) $itemp
      }

   if {$objInfo($obj,y1Pix) == $objInfo($obj,ynPix)} return
   if {$objInfo($obj,x1Pix) == $objInfo($obj,xnPix)} return

   getPCAXY $obj $objInfo($obj,x1Pix) $objInfo($obj,ynPix) zx1 zy1
   getPCAXY $obj $objInfo($obj,xnPix) $objInfo($obj,y1Pix) zxn zyn

   if {$zx1 == $zxn || $zy1 == $zyn} return

   clearScatter

   set objInfo($obj,zx1) $zx1
   set objInfo($obj,zxn) $zxn
   set objInfo($obj,zy1) $zy1
   set objInfo($obj,zyn) $zyn

   drawScatter
}

proc selectScatter { obj xPix yPix } \
{
   global objInfo

   $objInfo($obj,canvas) delete zoom

   if {$objInfo($obj,x1Pix) > $objInfo($obj,xnPix)} \
      {
       set itemp                $objInfo($obj,x1Pix)
       set $objInfo($obj,x1Pix) $objInfo($obj,xnPix)
       set $objInfo($obj,xnPix) $itemp
      }

   if {$objInfo($obj,y1Pix) > $objInfo($obj,ynPix)} \
      {
       set itemp                $objInfo($obj,y1Pix)
       set $objInfo($obj,y1Pix) $objInfo($obj,ynPix)
       set $objInfo($obj,ynPix) $itemp
      }

   clearScatter

   getPCAXY $obj $objInfo($obj,x1Pix) $objInfo($obj,ynPix) zx1 zy1
   getPCAXY $obj $objInfo($obj,xnPix) $objInfo($obj,y1Pix) zxn zyn

   for {set i 1} {$i <= $objInfo(pca,specCount)} {incr i} \
      {
       set xVal [getPtrVal $objInfo(load,rdataH) [expr $i - 1]]
       set yVal [getPtrVal $objInfo(load,rdataV) [expr $i - 1]]

       if {$xVal < $zx1} continue
       if {$xVal > $zxn} continue
       if {$yVal < $zy1} continue
       if {$yVal > $zyn} continue

       set objInfo(color,$i) $objInfo(pca,fgColor) 
      }

   drawScatter
}

proc getPCAXY { obj xPix yPix ixPtr iyPtr } \
{
   global objInfo

   upvar $ixPtr ix
   upvar $iyPtr iy

   set xw [expr double(1 + $objInfo($obj,ixn) - $objInfo($obj,ix1))]
   set yw [expr double(1 + $objInfo($obj,iyn) - $objInfo($obj,iy1))]

   set dx [expr $objInfo($obj,zxn) - $objInfo($obj,zx1)]
   set dy [expr $objInfo($obj,zyn) - $objInfo($obj,zy1)]

   if {$xw == 0.0} {set xw 1.0}
   if {$yw == 0.0} {set yw 1.0}

   set ix [expr $objInfo($obj,zx1) + $dx*($xPix - $objInfo($obj,ix1))/$xw]
   set iy [expr $objInfo($obj,zy1) + $dy*($objInfo($obj,iyn) - $yPix)/$yw]

   if {$ix < $objInfo($obj,zx1)} {set ix $objInfo($obj,zx1)}
   if {$ix > $objInfo($obj,zxn)} {set ix $objInfo($obj,zxn)}

   if {$iy < $objInfo($obj,zy1)} {set iy $objInfo($obj,zy1)}
   if {$iy > $objInfo($obj,zyn)} {set iy $objInfo($obj,zyn)}
}

proc choosePCA {} \
{
   clearScatter
   readPCA
   drawScatter
}

proc sendSpecViewPCAClusters { inName } \
{
   global objInfo

   set p     $objInfo(pca,sendName)
   set obj   $objInfo(pca,objName)
   set w     [winfo name .]

   catch "send $p readSpecViewClusters $obj $w $inName"
}

proc destroySpecViewPCA {} \
{
   global objInfo

   set p     $objInfo(pca,sendName)
   set obj   $objInfo(pca,objName)
   set w     [winfo name .]
 
   set status None

   catch {set status [send $p quitSpecViewPCA $obj $w]}
}

proc registerSpecViewPCA {} \
{
   global objInfo

   set p     $objInfo(pca,sendName)
   set obj   $objInfo(pca,objName)
   set w     [winfo name .]
   
   set status None

   set status [send $p registerSpecViewPCA $obj $w]

   if {![info exists status]} {set status Error}

   set objInfo(pca,drawList) ""

   foreach ii $status \
      {
       foreach i $ii \
          {
           if {![string is integer $i]} continue
           if {$i < 1 || $i > $objInfo(pca,specCount)} continue
 
           lappend objInfo(pca,drawList) $i
          }
      }
}

proc updateSpecViewPCADrawList args \
{
   global objInfo

   clearScatter

   set objInfo(pca,drawList) ""

   foreach ii $args \
      {
       foreach i $ii \
          {
           if {![string is integer $i]} continue
           if {$i < 1 || $i > $objInfo(pca,specCount)} continue

           lappend objInfo(pca,drawList) $i
          }
      }

   set objInfo(pca,sendAllowed) 0

   drawScatter

   set objInfo(pca,sendAllowed) 1
}

proc msgSpecViewPCA { str } \
{
   global objInfo

   catch "send $objInfo(pca,sendName) showSpecViewPCAMsg $str" 
}

proc clusterSpectra {} \
{
   global objInfo

   clearScatter

   set inName  $objInfo(pca,pcaDir)/$objInfo(pca,coordName)
   set outName $objInfo(pca,pcaDir)/$objInfo(pca,clustName)

   if {![file exists $inName]} \
      {
       showError $objInfo(prog,name) "Error Finding Cluster Input File $objInfo(pca,coordName)"
       return
      }

   catch "file delete $outName"
   catch "exec clustTab.tcl -in $inName -out $outName -nc $objInfo(pca,clustTarget) -x PCANORM$objInfo(load,hLoc) PCANORM$objInfo(load,vLoc) -noverb"

   if {![file exists $outName]} \
      {
       showError $objInfo(prog,name) "Error Finding Cluster Output File $objInfo(pca,clustName)"
       return
      }

   set thisTab [gdbRead table -in $outName]
   set eList   [gdbGetInfo $thisTab -parms entries]
   set i       1

   foreach e $eList \
      {
       set objInfo(pcaClustID,$i) [gdbGet $e CLUSTID]
       incr i
      }

   set objInfo(pca,useLabelItem,ClusterID) 1
   set objInfo(pca,colorMode) ClusterID

   gdbFree $thisTab

   setScatterColors
   drawScatter

   if {$objInfo(pca,sendAllowed)} {sendSpecViewPCAClusters $outName}
}

proc showComponents {} \
{
   global objInfo

   puts  "Executing: nmrDraw -in $objInfo(pca,pcaDir)/pca/pca%03d.dat"
   catch "exec nmrDraw -in $objInfo(pca,pcaDir)/pca/pca%03d.dat &"
}

proc clearSpectra {} \
{
   global objInfo
     
   set p     $objInfo(pca,sendName)
   set obj   $objInfo(pca,objName)
   set w     [winfo name .]

   set status None

   if {$objInfo(pca,sendAllowed)} \
      {
       catch {set status [send $p clearThisView $obj $w]}
      }
}

proc quitSpecViewPCA { status } \
{
   global objInfo

   destroySpecViewPCA
   exit $status
}

proc readPCA {} \
{
   global objInfo

   readROI -in $objInfo(pca,loadName) -ndim 2 -roi $objInfo(load,id)

   set roiInfo  [getROIInfo -roi $objInfo(load,id)]
   set sizeList [lindex $roiInfo 2]

   set objInfo(pca,specCount) [lindex $sizeList 0]
   set objInfo(pca,count)     [lindex $sizeList 1]

   if {$objInfo(load,hLoc) < 1} {set objInfo(load,hLoc) 1}
   if {$objInfo(load,vLoc) < 1} {set objInfo(load,vLoc) 1}

   if {$objInfo(load,hLoc) > $objInfo(pca,count)} \
      {
       set objInfo(load,hLoc) $objInfo(pca,count)
      }

   if {$objInfo(load,vLoc) > $objInfo(pca,count)} \
      {
       set objInfo(load,vLoc) $objInfo(pca,count)
      }

   readROI -in $objInfo(pca,loadName) -ndim 1 -roi $objInfo(load,idH) \
           -x X_AXIS -dy Y_AXIS $objInfo(load,hLoc) 0

   readROI -in $objInfo(pca,loadName) -ndim 1 -roi $objInfo(load,idV) \
           -x X_AXIS -dy Y_AXIS $objInfo(load,vLoc) 0

   set roiInfo [getROIInfo -roi $objInfo(load,idH)]

   set objInfo(load,rdataH) [lindex $roiInfo 0]
   set objInfo(load,minH)   [vMin $objInfo(load,rdataH) $objInfo(pca,specCount)]
   set objInfo(load,maxH)   [vMax $objInfo(load,rdataH) $objInfo(pca,specCount)]

   set range  [expr $objInfo(load,maxH) - $objInfo(load,minH)]

   set objInfo(load,minH) [expr $objInfo(load,minH) - 0.075*$range]
   set objInfo(load,maxH) [expr $objInfo(load,maxH) + 0.075*$range]

   set roiInfo [getROIInfo -roi $objInfo(load,idV)]

   set objInfo(load,rdataV) [lindex $roiInfo 0]
   set objInfo(load,minV)   [vMin $objInfo(load,rdataV) $objInfo(pca,specCount)]
   set objInfo(load,maxV)   [vMax $objInfo(load,rdataV) $objInfo(pca,specCount)]

   set range  [expr $objInfo(load,maxV) - $objInfo(load,minV)]

   set objInfo(load,minV) [expr $objInfo(load,minV) - 0.05*$range]
   set objInfo(load,maxV) [expr $objInfo(load,maxV) + 0.05*$range]

   set objInfo(pca,zx1) $objInfo(load,minH)
   set objInfo(pca,zxn) $objInfo(load,maxH)
   set objInfo(pca,zy1) $objInfo(load,minV)
   set objInfo(pca,zyn) $objInfo(load,maxV)

   set objInfo(pca,explVarList)  ""
   set objInfo(pca,totalVarList) ""
   set objInfo(pca,compList)     ""

   set objInfo(pca,totalExpl) 0.0

   for {set i 1} {$i <= $objInfo(pca,count)} {incr i} \
      {
       set objInfo(pca,explVar,$i)  0.0
       set objInfo(pca,totalVar,$i) 0.0
      }

   if {[file exists $objInfo(pca,tabName)]} \
      {
       set objInfo(pca,tab) [gdbRead table -in $objInfo(pca,tabName)]

       set eList [gdbGetInfo $objInfo(pca,tab) -parms entries]

       foreach e $eList \
          {
           set i [gdbGet $e COMP]
           if {!$i} continue
 
           set objInfo(pca,explVar,$i) [gdbGet $e EXPL]
           set objInfo(pca,totalExpl)  [expr $objInfo(pca,totalExpl) + $objInfo(pca,explVar,$i)]

           lappend objInfo(pca,explVarList)  $objInfo(pca,explVar,$i)
           lappend objInfo(pca,totalVarList) $objInfo(pca,totalExpl)
           lappend objInfo(pca,compList)     $i
          }

       set j 1

       set objInfo(pca,totalVar,$j) $objInfo(pca,explVar,$j)

       for {set i 2} {$i <= $objInfo(pca,count)} {incr i} \
          {
           set objInfo(pca,totalVar,$i) [expr $objInfo(pca,totalVar,$j) + $objInfo(pca,explVar,$i)]
           incr j
          }
      }
}

proc setColorMode { mode } \
{
   global objInfo

   if {[strcasecmp $mode Current]} \
      {
       set objInfo(pca,colorMode) $mode
      }

   setScatterColors
   clearScatter
   drawScatter
}

proc setScatterColors {} \
{
   global objInfo

   set cList $objInfo(pca,colorList)

   set nc      [llength $cList]
   set mode    [string tolower $objInfo(pca,colorMode)]
   set modeVal $mode

   if {[string is integer $mode]} {set mode int}

   switch -exact $mode \
      {
       title -
       int   -
       last {
             set sList ""
      
             if {![strcmp $mode last]} \
                {
                 set item end
                } \
             elseif {![strcmp $mode int]} \
                {
                 set mode $modeVal
                } \
             else \
                {
                 set item $mode
                }

             for {set i 1} {$i <= $objInfo(pca,specCount)} {incr i} \
                {
                 set t $objInfo(expID,$i)

                 if {![strcmp $mode last]} \
                    {
                     set s [lindex [split $t -] end]
                    } \
                 elseif {![strcmp $mode title]} \
                    {
                     set s $t
                    } \
                 else \
                    {
                     set s [lindex [split $t -] [expr $mode - 1]]
                    }

                 set loc [lsearch -exact $sList $s]

                 if {$loc < 0} \
                    {
                     set loc [llength $sList]
                     lappend sList $s
                    }

                 set j [expr $loc % $nc]
 
                 set objInfo(color,$i) [lindex $cList $j]
                }
            }

       number  {
                for {set i 1} {$i <= $objInfo(pca,specCount)} {incr i} \
                   {
                    set j [expr ($i - 1) %  $nc]
                    set objInfo(color,$i) [lindex $cList $j]
                   }
               }

       clusterid {
                  for {set i 1} {$i <= $objInfo(pca,specCount)} {incr i} \
                     {
                      if {[info exists objInfo(pcaClustID,$i)]} \
                         {
                          set clustID $objInfo(pcaClustID,$i)
                         } \
                      else \
                         {
                          set clustID $i
                         }

                      set j [expr ($clustID - 1) %  $nc]
                      set objInfo(color,$i) [lindex $cList $j]
                     }
                 }
 
       file    {
                for {set i 1} {$i <= $objInfo(pca,specCount)} {incr i} \
                   {
                    set j [hashCode $objInfo(fileID,$i) $nc]
                    set objInfo(color,$i) [lindex $cList $j]
                   }
               }

       default {
                for {set i 1} {$i <= $objInfo(pca,specCount)} {incr i} \
                   {
                    set j [expr $objInfo(origID,$i) % $nc]
                    set objInfo(color,color$i) [lindex $cList $j]
                   }
               }
      }
}

proc initSpecViewPCAInfo {} \
{
   global objInfo

   set lastOrigSpec [expr $objInfo(pca,specCount) - $objInfo(pca,pcaExtraCount)]

   set objInfo(expID,maxItemCount) 0

   set j 1

   for {set i 1} {$i <= $objInfo(pca,specCount)} {incr i} \
      {
       set inName [format $objInfo(pca,specName) $i]

       rdFDATA $inName fdata

       set objInfo(expID,$i)  [string toupper [getParmStr $fdata FDTITLE   NULL_DIM]]

       set objInfo(fileID,$i) [getParmStr $fdata FDCOMMENT NULL_DIM]
       set objInfo(origID,$i) [getParmI   $fdata FDID      NULL_DIM]

       if {$objInfo(origID,$i) == 0} {set objInfo(origID,$i) $i}

       set objInfo(pcaClustID,$i) $i

       if {$objInfo(pca,pcaExtraCount) > 0} \
          {
           if {$i > $objInfo(pca,specCount) - $objInfo(pca,pcaExtraCount)} \
              {
               append objInfo(expID,$i) "-New"
              } \
           else \
              {
               append objInfo(expID,$i) "-Old"
              }
          }

       if {$i == 1} \
          {
           set size [getParmI $fdata NDSIZE CUR_XDIM]
           set objInfo(pca,x1PPM) [pnt2spec $fdata X_AXIS 1     ppm]
           set objInfo(pca,xnPPM) [pnt2spec $fdata X_AXIS $size ppm]
          }

       set nItems [llength [split $objInfo(expID,$i) "-"]]
       if {$nItems > $objInfo(expID,maxItemCount)} {set objInfo(expID,maxItemCount) $nItems}
 
       sysFree $fdata
      }

   for {set i 1} {$i <= $objInfo(pca,specCount)} {incr i} \
      {
       set objInfo(color,$i) $objInfo(pca,defColor)
      }

   setScatterColors
}

proc clearScatter {} \
{
   global objInfo

   set obj pca

   $objInfo($obj,canvas) delete textbox box label

   drawRec -fg $objInfo(pca,bgColor)  -x 0% 100% -y 0% 100% -cell 1 -win 1
   drawBox -fg $objInfo(pca,boxColor) -x 0% 100% -y 0% 100% -cell 1 -win 1
}

proc drawScatter {} \
{
   global objInfo 

   drawScatterAxis

   set xSize [expr 1 + $objInfo(pca,ixn) - $objInfo(pca,ix1)]
   set ySize [expr 1 + $objInfo(pca,iyn) - $objInfo(pca,iy1)]

   set rx [expr $objInfo(pca,zxn) - $objInfo(pca,zx1)]
   set ry [expr $objInfo(pca,zyn) - $objInfo(pca,zy1)]

   if {$rx == 0} {set rx 1}
   if {$ry == 0} {set ry 1}

   set cx [expr $xSize/$rx]
   set cy [expr $ySize/$ry]

   for {set i 1} {$i <= $objInfo(pca,specCount)} {incr i} \
      {
       set xVal [getPtrVal $objInfo(load,rdataH) [expr $i - 1]]
       set yVal [getPtrVal $objInfo(load,rdataV) [expr $i - 1]]

       if {$xVal < $objInfo(pca,zx1)} continue
       if {$xVal > $objInfo(pca,zxn)} continue
       if {$yVal < $objInfo(pca,zy1)} continue
       if {$yVal > $objInfo(pca,zyn)} continue

       set xLoc [expr $objInfo(pca,ix1) + $cx*($xVal - $objInfo(pca,zx1))]
       set yLoc [expr $objInfo(pca,iy1) + $cy*($objInfo(pca,zyn) - $yVal)] 

       if {[strcasecmp $objInfo(pca,drawLabMode) None]} \
          {
           set tState normal
          } \
       else \
          {
           set tState hidden 
          }

       if {$objInfo(pca,drawBoxFlag)} \
          {
           set bState normal
          } \
       else \
          {
           set bState hidden
          }

       if {[lsearch -exact $objInfo(pca,drawList) $objInfo(origID,$i)] < 0} \
          {
           if {![strcasecmp $objInfo(pca,drawLabMode) Selected]} {set tState hidden}

           set fn        $objInfo(pca,stdfont)
           set fillColor $objInfo(pca,bgColor)
           set oState    hidden

           if {$objInfo(pca,drawSelectedOnlyFlag)} \
              {
               set bState hidden
               set tState hidden
              }
          } \
       else \
          {
           set fn        $objInfo(pca,boldfont)
           set fn        $objInfo(pca,stdfont)
           set fillColor $objInfo(color,$i)
           set oState    normal
          }

       if {![strcasecmp $objInfo(pca,drawLabMode) Selected]} {set oState hidden}

       set clustStr ""
       set s        ""

       if {$objInfo(pca,drawTextFlag)} \
          {
           if {$objInfo(pca,useLabelItem,Number)} {append s " $i"}
           if {$objInfo(pca,useLabelItem,File)}   {append s " $objInfo(fileID,$i)"}

           set sList [split $objInfo(expID,$i) "-"]
           set n     [llength $sList]

           for {set j 1} {$j < $n} {incr j} \
              {
               if {$objInfo(pca,useLabelItem,$j)} {append s " [lindex $sList [expr $j - 1]]"}
              }

           if {$objInfo(pca,useLabelItem,Last) || $objInfo(pca,useLabelItem,$n)} {append s " [lindex $sList end]"}
          }

       if {$objInfo(pca,useLabelItem,ClusterID)} \
          {
           if {[info exists objInfo(pcaClustID,$i)]} \
              {
               set clustStr $objInfo(pcaClustID,$i)=
              } \
           else \
              {
               set clustStr 0=
              }
          }

       if {![llength $s]} {set s "."}

       if {[strcasecmp $objInfo(pca,drawLabMode) None] && $objInfo(pca,drawBoxFlag)} \
          {
           set yOff -10
          } \
       else \
          {
           set yOff   0
          }

       set objInfo(tID,$i) \
        [.pca create text $xLoc [expr $yLoc + $yOff] -fill [safeColor $objInfo(color,$i)]  \
          -anchor c -font $fn -text $clustStr[join [string trim $s] "-"] -state $tState \
          -tags "$i label"]

       set oList [.pca bbox $objInfo(tID,$i)]

       set ox1   [lindex $oList 0]
       set oy1   [lindex $oList 1]
       set oxn   [lindex $oList 2]
       set oyn   [lindex $oList 3]

       if {$ox1 > $oxn} \
          {
           set itemp $ox1
           set ox1   $oxn
           set oxn   $itemp
          } 

       if {$oy1 > $oyn} \
          {
           set itemp $oy1
           set oy1   $oyn
           set oyn   $itemp
          }

       set ox1 [expr $ox1 - 2]
       set oxn [expr $oxn + 2]
       set oy1 [expr $oy1 - 2]
       set oyn [expr $oyn + 2]
 
       set bx1 [expr $xLoc - 3]
       set bxn [expr $xLoc + 3]
       set by1 [expr ($yLoc - 3) - $yOff]
       set byn [expr ($yLoc + 3) - $yOff]

       set objInfo(oID,$i) \
        [.pca create rectangle $ox1 $oy1 $oxn $oyn -width 1 \
              -outline [safeColor $objInfo(pca,fgColor)] -state $oState -tags textbox]

       set objInfo(bID,$i) \
        [.pca create rectangle $bx1 $by1 $bxn $byn -width 2 \
              -outline [safeColor $objInfo(color,$i)] -fill [safeColor $fillColor] -state $bState \
              -tags "$i box"]

       .pca bind $objInfo(tID,$i) <Any-Enter>       "+ showThisID   $i"
       .pca bind $objInfo(tID,$i) <Any-Leave>       "+ unshowThisID $i"

       .pca bind $objInfo(oID,$i) <Any-Enter>       "+ showThisID   $i"
       .pca bind $objInfo(oID,$i) <Any-Leave>       "+ unshowThisID $i"

       .pca bind $objInfo(bID,$i) <Any-Enter>       "+ showThisID   $i"
       .pca bind $objInfo(bID,$i) <Any-Leave>       "+ unshowThisID $i"

       .pca bind $objInfo(bID,$i) <ButtonRelease-1> "+ drawThisID $i 1"
       .pca bind $objInfo(bID,$i) <ButtonRelease-2> "+ drawThisID $i 2"
       .pca bind $objInfo(bID,$i) <ButtonRelease-3> "+ drawThisID $i 3"

       .pca bind $objInfo(tID,$i) <ButtonRelease-1> "+ drawThisID $i 1"
       .pca bind $objInfo(tID,$i) <ButtonRelease-2> "+ drawThisID $i 2"
       .pca bind $objInfo(tID,$i) <ButtonRelease-3> "+ drawThisID $i 3"
      }

   drawRec -fg $objInfo(pca,bgColor)  -cell 1 -win 1 -x 0% 100% -y 0% 100%
   drawBox -fg $objInfo(pca,boxColor) -cell 1 -win 1 -x 0% 100% -y 0% 100%

   update
}

proc drawScatterAxis {} \
{
   global objInfo

   set obj pca

   $objInfo($obj,canvas) delete axisPix axisBox 

   if {$objInfo($obj,drawAxisFlag)} \
      {
       $objInfo($obj,canvas) create pixmap 0 0 -win 1 \
         -width $objInfo($obj,winXSize) -height $objInfo($obj,winYSize) -tags axisPix

       defineCell -cell 1 -win 1 \
          -x $objInfo($obj,ix1) $objInfo($obj,ixn) \
          -y $objInfo($obj,iy1) $objInfo($obj,iyn)

       drawRec -x 0 $objInfo($obj,winXSize) -y 0 $objInfo($obj,winYSize) -fg $objInfo(pca,winColor)

       drawRec -x $objInfo($obj,ix1) $objInfo($obj,ixn) -y $objInfo($obj,iy1) $objInfo($obj,iyn) -fg $objInfo(pca,bgColor)
      } \
   else \
      {
       $objInfo($obj,canvas) create rectangle \
          0 0 $objInfo($obj,winXSize) $objInfo($obj,winYSize) \
          -outline $objInfo($obj,winColor) -fill $objInfo(pca,winColor) -tags axisBox

       $objInfo($obj,canvas) create rectangle \
          $objInfo($obj,ix1) $objInfo($obj,iy1) \
          $objInfo($obj,ixn) $objInfo($obj,iyn) \
          -outline $objInfo($obj,axisColor) -fill $objInfo(pca,bgColor) -tags axisBox
      }

   if {!$objInfo($obj,drawAxisFlag)} return

   set ih $objInfo(load,hLoc)
   set iv $objInfo(load,vLoc)

   set sx "PCA$ih"
   set sy "PCA$iv"

   drawROI -cell 1 -axisOnly -colorS None -colorA $objInfo($obj,axisColor) -fg $objInfo($obj,axisColor) \
    -axis Bottom,orig=$objInfo(pca,zx1),end=$objInfo(pca,zxn),units=None,lab=$sx

   drawROI -cell 1 -axisOnly -colorS None -colorA $objInfo($obj,axisColor) $objInfo($obj,axisColor) \
    -axis Left,orig=$objInfo(pca,zy1),end=$objInfo(pca,zyn),units=None,lab=$sy

   set fmt "Explained Variance ${sx}+${sy}: %.2f%% $sx: %.2f%% $sy: %.2f%%"

   set p [format $fmt [expr $objInfo(pca,explVar,$ih) + $objInfo(pca,explVar,$iv)] $objInfo(pca,explVar,$ih) $objInfo(pca,explVar,$iv)]

   $objInfo($obj,canvas) delete plotTitle

   set xLoc [expr int(0.5*($objInfo($obj,ix1) + $objInfo($obj,ixn)))]
   set yLoc [expr int(0.5*$objInfo($obj,iy1))]

   if {$objInfo(pca,pcaExtraCount)} \
      {
       set str "$objInfo(pca,plotTitle) + $objInfo(pca,pcaExtraCount) New Spectra, $p"
      } \
   else \
      {
       set str "$objInfo(pca,plotTitle), $p"
      }

   if {[strcasecmp $objInfo(pca,plotTitle) None]} \
      {
       $objInfo($obj,canvas) create text $xLoc $yLoc \
        -fill $objInfo(pca,fgColor) -anchor center -font $objInfo(pca,boldfont) \
        -text $str -tags "plotTitle"
      } \
   else \
      {
       $objInfo($obj,canvas) create text $xLoc $yLoc \
        -fill $objInfo(pca,fgColor) -anchor nw -font $objInfo(pca,boldfont) \
        -text "$p" -tags "plotTitle"
      }
}

proc drawThisID { i bID } \
{
   global objInfo

   $objInfo(pca,canvas) delete zoom

   if {[llength $objInfo(pca,moveList)]} return

   set w     [winfo name .]
   set p     $objInfo(pca,sendName)
   set id    $objInfo(origID,$i)
   set color $objInfo(color,$i)
   set obj   $objInfo(pca,objName)
   set q     toggleSpecViewPCA

   set status None 

   if {$objInfo(pca,sendAllowed)} \
      {
       catch {set status [send $p $q $obj $id $i $objInfo(expID,$i) $color $bID $w]}
       if {![info exists status]} {set status None}

       set objInfo(pca,drawList) ""

       foreach ii $status \
          {
           foreach i $ii \
              {
               if {![string is integer $i]} continue
               if {$i < 1 || $i > $objInfo(pca,specCount)} continue

               lappend objInfo(pca,drawList) $i
              }
          }
      }

   showDrawList
}

proc showDrawList {} \
{
   global objInfo

   for {set i 1} {$i <= $objInfo(pca,specCount)} {incr i} \
      {
       if {[lsearch -exact $objInfo(pca,drawList) $objInfo(origID,$i)] < 0} \
          {
#          .pca itemconfigure $objInfo(tID,$i) -font $objInfo(pca,stdfont)
           .pca itemconfigure $objInfo(bID,$i) -fill $objInfo(pca,bgColor)
           set oState hidden
          } \
       else \
          {
#          .pca itemconfigure $objInfo(tID,$i) -font $objInfo(pca,boldfont)
           .pca itemconfigure $objInfo(bID,$i) -fill [safeColor $objInfo(color,$i)]
           set oState normal
          }

       if {![strcasecmp $objInfo(pca,drawLabMode) Selected]} {set oState hidden}

       .pca itemconfigure $objInfo(oID,$i) -state $oState 
      }
}
 
proc showThisID { i } \
{
  global objInfo 

  if {[strcasecmp $objInfo(pca,drawLabMode) None]} \
     {
      .pca itemconfigure $objInfo(tID,$i) -fill $objInfo(pca,fgColor) -state normal
      .pca raise $objInfo(tID,$i)
     } \
  else \
     {
      .pca itemconfigure $objInfo(bID,$i) -state hidden
      .pca itemconfigure $objInfo(tID,$i) -fill [safeColor $objInfo(color,$i)] -state normal
      .pca raise $objInfo(tID,$i)
     }

  update
}

proc unshowThisID { i } \
{
   global objInfo 

   if {![strcasecmp $objInfo(pca,drawLabMode) All]} \
      {
       .pca itemconfigure $objInfo(tID,$i) -fill [safeColor $objInfo(color,$i)] -state normal
       .pca raise $objInfo(tID,$i)
      } \
   elseif {![strcasecmp $objInfo(pca,drawLabMode) Selected]} \
      {
       if {[lsearch -exact $objInfo(pca,drawList) $objInfo(origID,$i)] < 0} \
          {
           .pca itemconfigure $objInfo(tID,$i) -fill [safeColor $objInfo(color,$i)] -state hidden 
          } \
       else \
          {
           .pca itemconfigure $objInfo(tID,$i) -fill [safeColor $objInfo(color,$i)] -state normal
           .pca raise $objInfo(tID,$i)
          }      
      } \
   else \
      {
       .pca itemconfigure $objInfo(tID,$i) -fill [safeColor $objInfo(color,$i)] -state hidden
       .pca itemconfigure $objInfo(bID,$i) -state normal 
      }

   update
}

proc selectLoad { axis loc } \
{
   global objInfo

   set objInfo(load,$axis) $loc

   choosePCA
}

proc safeColor { c } \
{
   if {![strcasecmp $c multi]} {return cyan}
   if {![strcasecmp $c None]}  {return white}

   return $c
}

initSpecViewPCAVars 
initSpecViewPCAArgs

readPCA

initSpecViewPCAInfo
initSpecViewPCAWin
initSpecViewScreeWin

clearScatter
registerSpecViewPCA
drawScatter
