#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Split 2D NMRPipe-format Data into a 1D Series, Sorted by Tau:"
   echo " -in      inName  [test.fid]      2D Input Name."
   echo " -outDir  outDir  [dat]           Output Directory."
   echo " -outName outName [test%03d.dat]  Output Name Template."
   echo " -tau     tList   [0.0 ...]       Optional List of Tau Values."
   echo " -title                           Update Title in Headers."
   echo " -noverb                          Verbose Mode OFF."
   exit 0
endif

set inName  = (`getArgD $argv -in      test.fid`)
set outDir  = (`getArgD $argv -outDir  dat`)
set outName = (`getArgD $argv -outName test%03d.dat`)

@ vFlag = 1

if (`flagLoc $argv -title`) then
   set titFlag = 1
else
   set titFlag = 0
endif

if (`flagLoc $argv -noverb`) then
   set vFlag = 0
endif

#
# Build default tauList after checking data size:

if (!(-e $inName)) then
   echo Error finding 2D Input $inName
   exit 1
endif

set ne = (`getParm -in $inName -parm NDSIZE -dim Y -fmt %.0f`)

set titStr   = "nmr"
set thisList = ""
set cList    = ""

if ($titFlag != 0) then
   set titStr = (`getParm -in $inName -parm FDTITLE -fmt %s -txt -dim NULL_DIM`)

   if (`string length "$titStr"` == 0) then
      set titStr = "nmr"
   endif
endif


set i = 0

while ($i < $ne)
   set thisList = ($thisList $i)
   set cList    = ($cList 0.0)
   @ i++
end

set tauList = (`getListArgD $argv -tau -- $thisList`)

if (!(-d $outDir)) then
   mkdir $outDir 
endif

if ($vFlag) then
   echo "Creating $ne 1D data files from $inName"
endif

set jList = (`sortRList.tcl -r $tauList`)
set i = 1

while ($i <= $ne)
   set j         = $jList[$i]
   set thisName  =  (`nmrPrintf $outDir/$outName $i`)
   set cList[$j] = 1.0

   if ($vFlag) then
      echo " $i. $thisName Location: $j Tau: $tauList[$j]"
   endif

   nmrPipe -in $inName \
   | nmrPipe -fn COADD -axis Y -cList $cList \
   -out $thisName -ov

   set cList[$j]  = 0.0

   sethdr $thisName -ndim 1 -tau $tauList[$j] -cubeFlag 0 -pipeFlag 0 -fileCount 1

   if ($titFlag != 0) then
      sethdr $thisName -title ${titStr}-$tauList[$j]
   endif

   @ i++
end

