#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

#
# Statistics on numerical lists.

proc safeDiv { a b } \
{
   if {![string is double $a] || ![string is double $b]} {return 0.0}
   if {$b == 0.0} {return 0.0}
   return [expr $a/$b]
}

proc safeDiv1 { a b } \
{
   if {![string is double $a] || ![string is double $b]} {return 0.0}
   if {$b == 0.0} {return $a}
   return [expr $a/$b]
}

proc getNint { r } \
{
   if {$r < 0.0} \
      {
       set r [expr int($r - 0.5)]
      } \
   else \
      {
       set r [expr int($r + 0.5)]
      }

   return $r
}

proc getRepeat { r n } \
{
   set rList ""

   for {set i 0} {$i < $n} {incr i} {lappend rList $r}

   return $rList
}

proc getNeg { rList } \
{
   set nList ""

   foreach r $rList {lappend nList [expr -1*$r]}

   return $nList
}

proc getFormat { rList fmt } \
{
   set str ""

   set n [llength $rList]
   set i 0

   foreach r $rList \
      {
       if {$i} {append str " "}
       append str [format $fmt $r]
       incr i
      }

   return $str
}

proc getScale { aList bList } \
{
   set n [getMin "[llength $aList] [llength $bList]"]

   set top 0.0
   set bot 0.0

   for {set i 0} {$i < $n} {incr i} \
      {
       set a [lindex $aList $i]
       set b [lindex $bList $i]

       set top [expr $top + $a*$b]
       set bot [expr $bot + $b*$b]
      }

   if {$bot == 0.0} {return 0.0}

   return [expr $top/$bot]
}

proc getDiff { aList bList } \
{
   set n [getMin "[llength $aList] [llength $bList]"]

   set dList ""
   set bot 0.0

   for {set i 0} {$i < $n} {incr i} \
      {
       set a [lindex $aList $i]
       set b [lindex $bList $i]

       lappend dList [expr $a - $b]
      }

   return $dList
}

proc getMaxStrlen { sList } \
{
   set nMax 0

   foreach s $sList \
      {
       set n [string length $s]
       if {$n > $nMax} {set nMax $n}
      }

   return $nMax
}

proc getMultS { rList s } \
{
   set sList ""

   foreach r $rList {lappend sList [expr $r*$s]}
   return $sList 
}

proc getDivS { rList s } \
{
   set sList ""

   if {$s == 0.0} \
      {
       foreach r $rList {lappend sList 0.0}
      } \
   else \
      {
       set s [expr 1.0/$s]
       foreach r $rList {lappend sList [expr $r*$s]}
      }

   return $sList
}

proc getAddS { rList s } \
{
   set sList ""

   foreach r $rList {lappend sList [expr $r + $s]}
   return $sList
}

proc getCount { rList } \
{
   set n [llength $rList]
   return $n
}

proc getSum { rList } \
{
   set sum 0.0

   foreach i $rList {set sum [expr $sum + $i]}

   return $sum
}

proc getSumSq { rList } \
{
   set sum 0.0

   foreach i $rList {set sum [expr $sum + $i*$i]}

   return $sum
}

proc getWAvg { rList wList } \
{
   set rT 0.0
   set rB 0.0 

   set i 0

   foreach r $rList \
      {
       set w  [lindex $wList $i]
       set rT [expr $rT + $r*$w]
       set rB [expr $rB + $w]

       incr i
      }

   if {$rB == 0.0} {return 0.0}

   set avg [expr double($rT)/$rB]

   return $avg
}

proc getWRMS { rList wList } \
{
   set ws 0.0
   set rs 0.0
   set i  0 

   foreach r $rList \
      {
       set w  [lindex $wList $i]
       set d  [expr   $w*$r]
       
       set ws [expr $ws + $w]
       set rs [expr $rs + $d*$d]

       incr i
      }

   if {$rs == 0.0} {return 0.0}
   if {$ws == 0.0} {return 0.0}

   set s [expr sqrt( $rs/$ws )]

   return $s
}

proc getAvg { rList } \
{
   set n   [llength $rList]
   set sum 0.0

   if {!$n} {return $sum}

   foreach i $rList {set sum [expr $sum + $i]}

   return [expr $sum/double($n)]
}

proc getMin { rList } \
{
   set n [llength $rList]

   if {!$n} {return 0}

   set val [lindex $rList 0]

   foreach i $rList \
      {
       if {$i < $val} {set val $i}
      } 

   return $val
}

proc getMax { rList } \
{
   set n [llength $rList]

   if {!$n} {return 0}

   set val [lindex $rList 0]

   foreach i $rList \
      {
       if {$i > $val} {set val $i}
      }

   return $val
}

proc getAbsMax { rList } \
{
   set n [llength $rList]

   if {!$n} {return 0}

   set val [lindex $rList 0]

   foreach i $rList \
      {
       if {abs($i) > abs($val)} {set val $i}
      }

   return $val
}

proc getAbsMin { rList } \
{
   set n [llength $rList]

   if {!$n} {return 0}

   set val [lindex $rList 0]

   foreach i $rList \
      {
       if {abs($i) < abs($val)} {set val $i}
      }

   return $val
}

proc getSMaxAbs { rList } \
{
   set n [llength $rList]

   if {!$n} {return 0}

   set val [lindex $rList 0]

   foreach i $rList \
      {
       if {abs($i) > abs($val)} {set val $i}
      }

   return $val
}

proc getSMinAbs { rList } \
{
   set n [llength $rList]

   if {!$n} {return 0}

   set val [lindex $rList 0]

   foreach i $rList \
      {
       if {abs($i) < abs($val)} {set val $i}
      }

   return $val
}

proc getMaxAbs { rList } \
{
   set n [llength $rList]

   if {!$n} {return 0}

   set val [expr abs([lindex $rList 0])]

   foreach i $rList \
      {
       if {abs($i) > $val} {set val [expr abs($i)]}
      }

   return $val
}

proc getMinAbs { rList } \
{
   set n [llength $rList]

   if {!$n} {return 0}

   set val [expr abs([lindex $rList 0])]

   foreach i $rList \
      {
       if {abs($i) < $val} {set val [expr abs($i)]}
      }

   return $val
}

proc getPChi { fList sList nFree } \
{
   set nF [llength $fList]
   set nS [llength $sList]

   if {$nF < 1} {return -1.0}
   
   if {$nS < 1} \
      {
       set sigma 1.0
      } \
   else \
      {
       set sigma [lindex $sList 0]
      }

   if {$sigma == 0.0} {set sigma 1.0}

   set sum 0.0
   set i   0

   foreach f $fList \
      {
       if {$i < $nS} {set sigma [lindex $sList $i]}

       set f   [expr $f/$sigma]
       set sum [expr $sum + $f*$f]

       incr i
      }

   set x    [expr 0.5*$sum]
   set y    [expr 0.5*$nFree]
   set pChi [gammaq $x $y]

   return $pChi
}

proc getMedian { rList } \
{
   set n [llength $rList]

   if {!$n} {return 0.0}

   set sList [lsort -real $rList]

   if {$n % 2} \
      {
       set loc1 [expr int($n/2)]
       set val  [lindex $sList $loc1]
      } \
   else \
      {
       set loc1 [expr int($n/2) - 1]
       set loc2 [expr int($n/2)]
       set val  [expr 0.5*([lindex $sList $loc1] + [lindex $sList $loc2])]
      }

   return $val
}

proc getMAD { rList } \
{
   set rMed  [getMedian $rList]
   set dList ""

   foreach r $rList {lappend dList [expr abs($r - $rMed)]}

   set x [getMedian $dList]

   return $x
}

proc getVecVal { vList } \
{
   return $vList
}

proc getVecSort { rList } \
{
   if {![llength $rList]} {return ""}
   return [lsort -real $rList]
}

   
proc statSortLocProc { p1 p2 } \
{
   set v1 [lindex [split $p1 ","] 1]
   set v2 [lindex [split $p2 ","] 1]

   if {$v1 > $v2} {return  1}
   if {$v1 < $v2} {return -1}

   return 0
}

proc statSortLocAbsProc { p1 p2 } \
{
   set v1 [lindex [split $p1 ","] 1]
   set v2 [lindex [split $p2 ","] 1]

   if {abs($v1) > abs($v2)} {return  1}
   if {abs($v1) < abs($v2)} {return -1}

   return 0
}

proc getVecSortLoc { rList } \
{
   set n [llength $rList]

   if {!$n}     {return 0}
   if {$n == 1} {return 1}

   set riList ""
   set i 1

   foreach r $rList \
      {
       lappend riList $i,$r
       incr i
      }

   set sList [lsort -command statSortLocProc $riList]
   set iList ""

   foreach s $sList {lappend iList [lindex [split $s ","] 0]}

   return $iList
}

proc getVecSortLocAbs { rList } \
{
   set n [llength $rList]

   if {!$n}     {return 0}
   if {$n == 1} {return 1}

   set riList ""
   set i 1

   foreach r $rList \
      {
       lappend riList $i,$r
       incr i
      }

   set sList [lsort -command statSortLocAbsProc $riList]
   set iList ""

   foreach s $sList {lappend iList [lindex [split $s ","] 0]}

   return $iList
}

#
# Returns iList sorted by rList

proc sortByVec { iList rList } \
{
   if {![llength $iList]} {return ""}

   set locList [getVecSortLoc $rList]
   set kList   ""

   foreach loc $locList {lappend kList [lindex $iList [expr $loc - 1]]}
  
   return $kList 
}

proc sortByVecAbs { iList rList } \
{
   if {![llength $iList]} {return ""}

   set locList [getVecSortLocAbs $rList]
   set kList   ""

   foreach loc $locList {lappend kList [lindex $iList [expr $loc - 1]]}
   
   return $kList
}

proc getVecAbs { iList } \
{
   set aList ""

   foreach i $iList \
      {
       if {$i < 0} \
          {
           lappend $aList [expr -($i)]
          } \
       else \
          {
           lappend aList $i
          }
      }

   return $aList
}

proc getVecInt { iList } \
{
   set jList ""
   foreach i $iList {lappend jList [expr int($i)]}
   return $jList
}

proc getSimpleMedian { rList } \
{
   set n [llength $rList]

   if {!$n} {return 0.0}

   set sList [lsort -real $rList]

   set loc1 [expr int($n/2)]
   set val  [lindex $sList $loc1]

   return $val
}

proc getRange { rList } \
{
   set n [llength $rList]

   if {$n < 2} {return 0}

   set minVal [getMin $rList]
   set maxVal [getMax $rList]
   set val    [expr $maxVal - $minVal]

   return $val
}

proc getVariance { rList } \
{
   set n [llength $rList]

   if {$n == 0} \
      {
       set val 0.0
      } \
   elseif {$n == 1} \
      {
       set val [lindex $rList 0]
      } \
   else \
      {
       set avg [getAvg $rList]
       set val 0.0

       foreach i $rList \
          {
           set d   [expr $i - $avg]
           set val [expr $val + $d*$d]
          }

       set val [expr $val/double($n-1)]
      }

   return $val 
}

proc getStdDev { rList } \
{
   set n [llength $rList]

   if {$n == 0} \
      {
       set val 0.0
      } \
   elseif {$n == 1} \
      {
       set val [lindex $rList 0]
      } \
   else \
      {
       set avg [getAvg $rList]
       set val 0.0

       foreach i $rList \
          {
           set d   [expr $i - $avg]
           set val [expr $val + $d*$d]
          }

       set val [expr sqrt($val/double($n-1))]
      }

   return $val 
}

proc getRMS { rList } \
{
   set n   [llength $rList]
   set val 0.0

   if {$n > 0} \
      {
       foreach d $rList \
          {
           set val [expr $val + $d*$d]
          }

       set val [expr sqrt($val/double($n))]
      }

   return $val
}

proc getSqrtSumSq { rList } \
{
   set n   [llength $rList]
   set val 0.0

   if {$n > 0} \
      {
       foreach d $rList \
          {
           set val [expr $val + $d*$d]
          }

       set val [expr sqrt($val)]
      }

   return $val
}

#
# getOutliers diffList stdDevLimit keepFrac

proc statAbsSortProc { p1 p2 } \
{
   set v1 [lindex $p1 0]
   set v2 [lindex $p2 0]

   if {abs($v1) > abs($v2)} {return  1}
   if {abs($v1) < abs($v2)} {return -1}

   return 0
}

proc getOutliers args \
{
    set sdLimit 3.0
    set kFrac   0.9

    set iList   ""
    set pList   ""
    set qList   ""
    set sList   ""

    set nA [llength $args]

    if {$nA < 1} {return $iList}

    set dList [lindex $args 0]
    set nOrig [llength $dList]
    set nBest [expr int($nOrig*$kFrac)]

    if {$nBest < 5} {return $iList}

    if {$nA > 1} {set sdLimit [lindex $args 1]}
    if {$nA > 2} {set kFrac   [lindex $args 2]}

    set i 0

    foreach d $dList \
       {
        lappend pList "[lindex $dList $i] $i"
        incr i
       }

    set qList  [lsort -command statAbsSortProc $pList]

    foreach q $qList {lappend sList [lindex $q 0]}

    set bList  [lrange $sList 0 [expr $nBest - 1]]
    set avg    [getAvg    $bList]
    set dev    [getStdDev $bList]
    set dLimit [expr $sdLimit*$dev]

    for {set i [expr $nOrig - 1]} {$i >= 0} {incr i -1} \
       {
        set q [lindex $qList $i]
        set d [lindex $q 0]
        
        if {abs($d - $avg) > $dLimit} {lappend iList [lindex $q 1]}
       }

   return $iList 
}

proc getPearsonR { xList yList } \
{
   set nX [llength $xList]
   set nY [llength $yList]

   if {$nX != $nY} {return 0.0}
   if {!$nX}       {return 0.0}

   set n $nX

   set sumX  0.0
   set sumX2 0.0
   set sumY  0.0
   set sumY2 0.0
   set sumXY 0.0

   for {set i 0} {$i < $n} {incr i} \
      {
       set x [lindex $xList $i]
       set y [lindex $yList $i]

       set sumX  [expr $sumX  + $x]
       set sumX2 [expr $sumX2 + $x*$x]
       set sumY  [expr $sumY  + $y]
       set sumY2 [expr $sumY2 + $y*$y]
       set sumXY [expr $sumXY + $x*$y]
      }

   set xd  [expr $sumX2 - $sumX*$sumX/double($n)]
   set yd  [expr $sumY2 - $sumY*$sumY/double($n)]

   set top [expr $sumXY - $sumX*$sumY/double($n)]
   set bot [expr sqrt( abs($xd*$yd) )]

   if {$bot == 0.0} {return 0.0}

   set r [expr $top/$bot]

   return $r
}

#
# Find avg of angles, compensating for folding, range of +/-180.

proc getAvgAngleOld { wList } \
{
   set aList [adjustAngles $wList]
   set val   [getAvg $aList]

   if {$val > 180.0} {set val [expr $val - 360.0]}

   return $val
}

#
# Find std dev of angles, compensating for folding.

proc getStdDevAngleOld { aList } \
{
   set n [llength $aList]

   if {$n == 0} \
      {
       set val 0.0
      } \
   elseif {$n == 1} \
      {
       set val 180.0
      } \
   else \
      {
       set avg [getAvgAngle $aList]
       set val 0.0

       if {$avg < 0.0} {set avg [expr $avg + 360.0]}

       foreach a $aList \
          {
           if {$a < 0.0} {set a [expr $a + 360.0]}

           if {$a < $avg} \
              {
               set d1 [expr $avg - $a]
               set d2 [expr 360.0 - $d1]
              } \
           else \
              {
               set d1 [expr $a - $avg]
               set d2 [expr 360.0 - $d1]
              }

           if {$d1 < $d2} \
              {
               set d $d1 
              } \
           else \
              {
               set d $d2 
              }

           set val [expr $val + $d*$d]
          }

       set val [expr sqrt($val/double($n-1))]
      }

   return $val
}

#
# Cornelescu's angle statistics.

#
# Correction for difference of dihedral angles.

proc adjustThisAngle { diff } \
{   
    if {$diff < 999.0} \
       {
        while {$diff >  180} {set diff [expr $diff - 360]}
        while {$diff < -180} {set diff [expr 360 + $diff]}
       }

    return $diff
}

proc angleDiff { a1 a2 } \
{
   if {$a1 > 999 || $a2 > 999} {return 9999}

   set diff [expr $a1 - $a2]

   while {$diff >  180} {set diff [expr $diff - 360]}
   while {$diff < -180} {set diff [expr 360 + $diff]}

   return $diff
}

proc getAngleCount { aList } \
{
   set n 0

   foreach a $aList \
      {
       if {$a < 999.0} {incr n}
      }

   return $n
}

proc getDefinedAngles { aOrig } \
{
   set aList ""

   foreach a $aOrig \
     {
      if {$a < 999.0} {lappend aList $a}
     }

   return $aList
}

#
# Computes average angle.

proc getAvgAngle180 { aOrig } \
{
   set avg [getAvgAngle $aOrig]

   if {$avg > 999} {return $avg}

   while {$avg < -180.0} {set avg [expr $avg + 360.0]}
   while {$avg >  180.0} {set avg [expr $avg - 360.0]}

   return $avg
}

proc getAvgAngle { aOrig } \
{
   set aList [getDefinedAngles $aOrig]
   set n     [llength $aList]

   if {!$n}     {return 9999.0}
   if {$n == 1} {return $aList}

   set sum  0.0
   set psum 0.0
   set nsum 0.0

   set np 0
   set nn 0

   foreach i $aList \
      {
       if {$i >= 0} \
          {
           incr np
           set  psum [expr $psum + $i]
          } \
       else \
          {
           incr nn
           set  nsum [expr $nsum + $i]
          }
       }
   
   if {$np != 0} \
      {
       set p [expr $psum/double($np)]
      } \
   else \
      {
       return [expr $nsum/double($nn)]
      }

   if {$nn != 0} \
      {
       set n [expr $nsum/double($nn)]
      } \
   else \
      {
       return [expr $psum/double($np)]
      }

   if {$np && $nn} \
      {
       set diff [expr $p - $n]

       if {$diff < 180.0} \
          {
           set avg [expr ($psum + $nsum)/double($np+$nn)]
          } \
       else \
          {
           set avg \
            [adjustThisAngle [expr ($psum + ($nsum + 360.0*$nn))/($np+$nn)]]
          }
      }

   return $avg
}

#
# Computes Std Dev of angles.

proc getStdDevAngle { aOrig } \
{
   set aList [getDefinedAngles $aOrig]
   set n     [llength $aList]

   if {!$n}     {return 9999.0}
   if {$n == 1} {return 180.0}

   set avg   [getAvgAngle $aList]
   set sumsq 0.0

   foreach i $aList \
      {
       set diff  [adjustThisAngle [expr $i - $avg]]
       set sumsq [expr $sumsq + $diff*$diff]
      }

   set std [expr sqrt($sumsq/double($n - 1))]

   return $std
}

#
# Computes RMSD of angles.

proc getMaxDevAngle { aOrig } \
{
   set aList [getDefinedAngles $aOrig]
   set n     [llength $aList]

   if {!$n}     {return 0.0}
   if {$n == 1} {return 180.0}

   set avg [getAvgAngle $aList]
   set dev 0.0

   foreach i $aList \
      {
       set diff [adjustThisAngle [expr $i - $avg]]
       if {abs($diff) > $dev} {set dev [expr abs($diff)]}
      }

   return $dev
}

#
# Adjust wrapped angles for statistical calculations.

proc adjustAngles { wList } \
{
   set aList ""
   set fList ""

   foreach w $wList \
      {
       if {$w < 999.0} \
          {
           while {$w < 0.0}    {set w [expr $w + 360.0]}
           while {$w >= 360.0} {set w [expr $w - 360.0]}
          }

       lappend aList $w
      }

   set v [getMedian [getDefinedAngle $aList]]

   if {$v < 90 || $v > 270.0} \
      {
       foreach a $aList \
          {
           if {$a > 999.0} \
              {
               lappend fList $a
               continue
              }

           if {$a > 180.0} \
              {
               set a [expr $a - 360.0]
              }

           lappend fList $a
          }

       return $fList
      }

   foreach a $aList \
      {
       if {$a > 999.0} \
          {
           lappend fList $a
           continue
          }

       if {$a > 180.0} \
          {
           set b [expr $a - 360.0]
          } \
       else \
          {
           set b $a
          }

       if {$a > $v} \
          {
           set d1 [expr $a - $v]
           set d2 [expr 360.0 - $d1]
          } \
       else \
          {
           set d1 [expr $v - $a]
           set d2 [expr 360.0 - $d1]
          } 

       if {$d1 < $d2} \
          {
           lappend fList $a
          } \
       else \
          {
           lappend fList $b
          }
      }

   return $fList 
}

#
# Computes haversine distance on a sphere, given radius,
# and two latitude/longitude locations in degrees.
 
proc hsDist { r latI longI latJ longJ } \
{
   set d2r   [expr 4.0*acos(0.0)/360.0]
   set dLat  [expr ($latJ - $latI)*$d2r]
   set dLong [expr ($longJ - $longI)*$d2r]
   set sLat  [expr sin(0.5*$dLat)]
   set sLong [expr sin(0.5*$dLong)]
   set a     [expr $sLat*$sLat + cos($latI*$d2r)*cos($latJ*$d2r)*$sLong*$sLong]
   set c     [expr 2.0*atan2( sqrt($a), sqrt(1.0 - $a) )]
   set d     [expr $r*$c]

   return $d
                  
}

proc lselect { rList sList } \
{
   set aList ""
   set n [llength $rList]

   if {!$n} {return $aList}

   set m [llength $sList]

   if {$n > $m} {set $n $m}

   for {set i 0} {$i < $n} {incr i} \
      {
       set s [lindex $sList $i]

       if {$s != 0} \
          {
           set r [lindex $rList $i]
           lappend aList $r
          }
      }

   return $aList
}

proc lreverse { iList } \
{
   set n [llength $iList]  
   if {$n < 2} {return $iList}
   
   set sList ""

   for {set i [expr $n - 1]} {$i >= 0} {incr i -1} \
      {
       lappend sList [lindex $iList $i]
      }
   
   return $sList
}

proc lswap { iList loc1 loc2 } \
{
    set n [llength $iList]
    if {$n < 2} {return $iList}

    if {$loc1 < 0}      {set loc1 [expr $n - 1]}
    if {$loc1 > $n - 1} {set loc1 0}

    if {$loc2 < 0}      {set loc2 [expr $n - 1]}
    if {$loc2 > $n - 1} {set loc2 0}

    if {$loc1 == $loc2} {return $iList}

    set v1 [lindex $iList $loc1]
    set v2 [lindex $iList $loc2]

    set jList [lreplace $iList $loc1 $loc1 $v2]
    set iList [lreplace $jList $loc2 $loc2 $v1]

   return $iList
}

proc lcompare { iList jList } \
{
   set nI  [llength $iList]
   set nJ  [llength $jList]

   if {$nI > $nJ} \
      {
       set nMax $nI
      } \
   else \
      {
       set nMax $nJ
      }
 
   for {set loc 0} {$loc < $nMax} {incr loc} \
      {
       set i ""
       set j ""

       if {$loc < $nI} {set i [lindex $iList $loc]} 
       if {$loc < $nJ} {set j [lindex $jList $loc]} 

       set result [strcmp $i [lindex $jList $loc]]
       if {$result} {return $result}
      }

   return 0
}

proc getVsStat { procName xList yList } \
{
   set n     [getMin "[llength $xList] [llength $yList]"]
   set rList ""

   for {set i 0} {$i < $n} {incr i} \
      {
       set x [lindex $xList $i]
       set y [lindex $yList $i]

       lappend rList [expr $x - $y]
      } 

   set s [$procName $rList]

   return $s
}

#
# Chi-Square difference between a and b with noise s.

proc getChi2 { aList bList sList } \
{
   set sum 0.0

   set nA    [llength $aList]
   set nB    [llength $bList]
   set nS    [llength $sList]

   set nMax  [getMax "$nA $nB $nS"]
   set sigma [lindex $sList 0]

   for {set i 0} {$i < $nMax} {incr i} \
      {
       set a 0.0
       set b 0.0
       set s $sigma

       if {$i < $nA} {set a [lindex $aList $i]}
       if {$i < $nB} {set b [lindex $bList $i]}
       if {$i < $nS} {set s [lindex $sList $i]}

       if {$s == 0.0} {set s 1.0}

       set d   [expr ($a - $b)/double($s)]
       set sum [expr $sum + $d*$d]
      }

   return $sum
}

proc getGaussHist { rList hxList hyList xListPtr yListPtr n } \
{
   upvar $xListPtr xList
   upvar $yListPtr yList

   set hN [llength $hxList]

   if {$hN < 1 || $n < 2} return

   set xList ""
   set yList ""
   set qList ""

   set count  [llength $rList]

   set minVal [getMin $rList]
   set maxVal [getMax $rList]

   set avg    [getAvg $rList]
   set dev    [getStdDev $rList]

   if {$dev == 0.0} {set dev 1.0}

   set range  [expr $maxVal - $minVal]
   set step   [expr $range/double($n - 1)]

   foreach x $hxList \
      {
       lappend qList [expr exp( -0.5*($x - $avg)*($x - $avg)/($dev*$dev))]
      }

   set c [getScale $hyList $qList]

   for {set i 0} {$i < $n} {incr i} \
      {
       set x [expr $minVal + $step*$i]
       set y [expr $c*exp( -0.5*($x - $avg)*($x - $avg)/($dev*$dev))]

       lappend xList $x
       lappend yList $y
      }
}

proc getHist { rList xListPtr yListPtr n } \
{
   upvar $xListPtr xList
   upvar $yListPtr yList

   set minVal [getMin $rList]
   set maxVal [getMax $rList]

   getHist2 $rList xList yList $minVal $maxVal $n
}

proc getHist2 { rList xListPtr yListPtr minVal maxVal n } \
{
   upvar $xListPtr xList
   upvar $yListPtr yList

   set xList ""
   set yList ""

   set nn [expr $n - 1]

   if {$n < 1} return

   if {$n == 1} \
      {
       set xList [expr 0.5*($minVal + $maxVal)]
       set yList [llength $rList] 
       return
      }

   if {$minVal == $maxVal} \
      {
       set s 0.0
      } \
   else \
      {
       set s [expr double($nn)/($maxVal - $minVal)]
      }

   for {set i 0} {$i < $n} {incr i} {set h($i) 0}

   foreach r $rList \
      {
       set i [expr int($s*($r - $minVal))]

       if {$i < 0}   {set i 0}
       if {$i > $nn} {set i $nn}

       incr h($i)
      }

   set d [expr ($maxVal - $minVal)/double($nn)]
   set x [expr $minVal + 0.5*$d]

   for {set i 0} {$i < $n} {incr i} \
      {
       lappend xList $x
       lappend yList $h($i)

       set x [expr $x + $d]
      }
}

proc comparePDF { xListI sigmaI xListJ sigmaJ } \
{
   set x1    [getMin "[getMin $xListI] [getMin $xListJ]"]
   set xN    [getMax "[getMax $xListI] [getMax $xListJ]"]
   set sigma [getMax "$sigmaI $sigmaJ"]

   set x1 [expr $x1 - $sigma]
   set xN [expr $xN + $sigma]

   set n   64 
   set sum 0.0

   set pdfI [getPDF $xListI $x1 $xN $sigmaI $n]
   set pdfJ [getPDF $xListJ $x1 $xN $sigmaJ $n]

   for {set loc 0} {$loc < $n} {incr loc} \
      {
       set xI  [lindex $pdfI $loc]
       set xJ  [lindex $pdfJ $loc]

       set d   [expr $xI - $xJ]
       set sum [expr $sum + $d*$d]
      }

    set sum [expr sqrt($sum/double($n))]

    return $sum
}

proc getPDFCoords { x1 xN n } \
{
   if {$n < 0} {set n 0}

   set coordList ""

   if {$n > 1} \
      {
       set s [expr double($xN - $x1)/double($n - 1)]
      } \
   else \
      {
       set s 0.0
      }

   for {set i 0} {$i < $n} {incr i} \
       {
        set x [expr $x1 + $s*$i]

        if {$x < $x1} {set x $x1}
        if {$x > $xN} {set x $xN}

        lappend coordList $x
       }

   return $coordList
}

proc getPDFMax { xList yList } \
{
   set xMax [lindex $xList 0]
   set yMax [lindex $yList 0]

   set i 0

   foreach y $yList \
      {
       if {$y > $yMax} \
          {
           set xMax [lindex $xList $i]
           set yMax $y
          }

       incr i
      }

   return "$xMax $yMax"
}

proc getPDF { xList x1 xN sigma n } \
{
   if {![llength $xList]} \
      {
       if {!$n} \
          {
           set s 0.0
          } \
       else \
          {
           set s [expr 1.0/$n]
          }

       set pdfList [vPDFCoords $s $s $n]

       return $pdfList
      }

   set coordList [vPDFCoords $x1 $xN $n]
   set pdfList   [vPDF $coordList $xList $sigma]

   return $pdfList

   set pdfList ""
   set pList   ""

   if {$sigma == 0.0} {set sigma 1.0}

   set coordList [getPDFCoords $x1 $xN $n]

   foreach x $coordList \
      {
       set sum 0.0

       foreach x0 $xList \
          {
           set d   [expr ($x - $x0)/double($sigma)]
           set q   [expr -0.5*$d*$d]

           if {$q < -18.5} {set q -18.5}

           set p   [expr exp( $q )] 
           set sum [expr $sum + $p]
          }

        lappend pList $sum
       }

   set sum [getSum $pList]

   if {$sum == 0.0} {set sum 1.0}

   foreach p $pList {lappend pdfList [expr $p/double($sum)]}

   return $pdfList
}

proc getZProb { z zMax } \
{
   if {$z    < 0.0} {set z    [expr -1.0*$z]}
   if {$zMax < 0.0} {set zMax [expr -1.0*$zMax]}

   if {$z > $zMax} {set z $zMax}
    
   set n  [expr 1000*(1 + int($zMax))]
   set PI [expr 2.0*acos(0.0)]
   set c  [expr 2.0/sqrt(2.0*$PI)]
   set dz [expr $zMax/double($n - 1)]

   set beforeList ""
   set afterList  ""
   
   for {set i 0} {$i < $n} {incr i} \
      {
       set x    [expr $dz*$i]
       set gVal [expr $c*exp( -0.5*$x*$x )*$dz]

       if {$x > $z} \
          {
           lappend afterList $gVal
          } \
       else \
          {
           lappend beforeList $gVal
          }
      }

   set sB [getSum $beforeList]
   set sA [getSum $afterList]

   set p [safeDiv $sB [expr $sB + $sA]]

   return $p
}

proc getReverse { rList } \
{
   set n [llength $rList]
   if {$n < 2} {return $rList}

   set sList ""

   for {set i [expr $n - 1]} {$i >= 0} {incr i -1} \
      {
       lappend sList [lindex $rList $i]
      } 

   return $sList
}

proc cIncr { iPtr iStep minVal maxVal } \
{
   upvar $iPtr i
  
   incr i $iStep

   if {[strcasecmp $minVal None]} \
      {
       if {$i < $minVal} \
          {
           if {[strcasecmp $maxVal None]} {set i $maxVal}
          }
      }

   if {[strcasecmp $maxVal None]} \
      {
       if {$i > $maxVal} \
          {
           if {[strcasecmp $minVal None]} {set i $minVal}
          }
      }

   return $i 
}

proc iClip { iPtr minVal maxVal } \
{
   upvar $iPtr v

   if {$v < $minVal} {set v $minVal}
   if {$v > $maxVal} {set v $maxVal}
}

proc iClipC { iPtr minVal maxVal } \
{
   upvar $iPtr v

   if {$v < $minVal} {set v $maxVal}
   if {$v > $maxVal} {set v $minVal}
}

proc rClip { vPtr minVal maxVal } \
{
   upvar $vPtr v

   if {$v < $minVal} {set v $minVal}
   if {$v > $maxVal} {set v $maxVal}
}

proc rSwap { vPtr1 vPtr2 } \
{
   upvar $vPtr1 v1
   upvar $vPtr2 v2

   set temp $v1
   set v1   $v2
   set v2   $temp
}

proc rOrder { vPtr1 vPtr2 } \
{
   upvar $vPtr1 v1
   upvar $vPtr2 v2

   if {$v1 > $v2} \
      {
       set t  $v1
       set v1 $v2
       set v2 $t
      }
}

proc nextPower2 { n } \
{
    set m 1

    while {$n > $m} {set m [expr 2*$m]}

    return $m
}

proc getZFSize { n nZF } \
{
   if {$nZF < 1} {return $n}

   set nFinal $n

   for {set i 0} {$i < $nZF} {incr i} {set nFinal [expr 2*$nFinal]}

   return $nFinal
}

proc polySolve { xList yList n } \
{
   set q ""

   foreach x $xList \
      {
       for {set i $n} {$i >= 0} {incr i -1} \
          {
           lappend q [expr pow( $x, $i )]
          }
      }

   set c [svdLinSol -x $q -y $yList]

   return $c
}

proc polyEval { xList cList } \
{
   set n     [llength $cList]
   set yList ""

   if {!$n} {return $yList}

   foreach x $xList \
      {
       set i [expr $n - 1]
       set y 0.0

       foreach c $cList \
          {
           set y [expr $y + $c*pow( $x, $i )]
           incr i -1
          } 

       lappend yList $y
      }

   return $yList 
}

