#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts stderr "Report STD Values for a Given Spectrum:"
    puts stderr " -dir      dirName \[./\]         Directory of Spectral Data."
    puts stderr " -off      oName   \[off.dat\]    Off-Resonance STD Spectrum Input."
    puts stderr " -dif      dName   \[dif.dat\]    On - Off STD Difference Input."
    puts stderr " -scaled   sName   \[scaled.dat\] Scaled STD Difference Input."
    puts stderr " -maskName mName   \[None\]       NMRPipe-format Mask File Input."
    puts stderr " -ax1      nx1     \[9.0ppm\]     Analysis Region X1."
    puts stderr " -axn      nxn     \[6.0ppm\]     Analysis Region XN."
    puts stderr " -plx1     plx1    \[Auto\]       Plot Region X1 (Such as 9.0ppm)."
    puts stderr " -plxn     plxn    \[Auto\]       Plot Region XN (Such as 6.0ppm)."
    puts stderr " -pkdx     pkxw    \[10\]         Peak Detection Width +/- Pts."
    puts stderr " -dx       dx      \[0.003ppm\]   +/- Width for STD Calc."
    puts stderr " -sx       sx      \[0.0\]        +/- Width for STD Scale;"
    puts stderr "                                (Used for Report Only)." 
    puts stderr " -fit                           Use Optional 1D Fitting."
    puts stderr "Output Files:"
    puts stderr " -lab    labName  \[label.dat\]   Labeled Region Spectrum."
    puts stderr " -tab    tabName  \[label.tab\]   Labeled Region Table."
    puts stderr " -pk     pkTab    \[peak.tab\]    Off Spectrum Peak Table."
    puts stderr " -std    stdName  \[std.tab\]     STD Report Table."
    puts stderr " -plot   plotName \[std.ps\]      STD PostScript Plot."
    puts stderr " -pdf                           Convert PostScript to PDF (Default);"
    puts stderr "                                Requires ps2pdf program."
    puts stderr " -nopdf                         No PostScript to PDF .Conversion."
    puts stderr "Noise and Analysis Thresholds:"
    puts stderr " -noise  sigma   \[Auto\]   Spectral Noise Estimate."
    puts stderr " -nadj   sAdj    \[1.0\]    Automated Noise Adjustment Factor."
    puts stderr " -thresh t       \[Auto\]   Height Detection Threshold."
    puts stderr " -pChi   p       \[0.2\]    Chi-Square Probability Threshold."
    puts stderr " -mult   mult    \[8\]      Factor for Threshold from Noise."
    puts stderr " -nx1    nx1     \[9.6ppm\] Noise Region X1."
    puts stderr " -nxn    nxn     \[9.2ppm\] Noise Region XN."
    puts stderr "Hard Copy Plot Details:"
    puts stderr " -plotWidth w    \[9.5\]    Plot Width in Inches." 
    puts stderr " -multiPlot id   \[0\]      ID for Multiple Plots."
    exit 0
   }

set dirName        ./

set tabName        label.tab
set pkTabName      peak.tab
set stdName        std.tab
set plotName       std.ps
set fitName        fit.dat

set inName(Off)    off.dat
set inName(Dif)    dif.dat
set inName(Scaled) scaled.dat
set inName(Label)  label.dat
set inName(Weight) w.dat
set inName(Mask)   None
set inName(Peak)   off.dat 

set roiID(Off)    1
set roiID(Dif)    2
set roiID(Scaled) 3
set roiID(Label)  4
set roiID(Weight) 5
set roiID(Mask)   6
set roiID(Peak)   7

set nx1 9.6ppm
set nxn 9.2ppm
set ax1 9.0ppm
set axn 6.0ppm

set plx1      Auto
set plxn      Auto
set plotWidth 9.5

set stdSX 0.0
set stdDX 0.003ppm 

set pkDX  10

set pChiThresh    0.2
set mult          8.0
set thresh        Auto 
set noise         Auto 
set noiseAdj      1.0
set fitFlag       0
set pdfFlag       1
set multiPlot     0

getArgD $ARGV -dir    dirName

getArgD $ARGV -off      inName(Off)
getArgD $ARGV -dif      inName(Dif)
getArgD $ARGV -scaled   inName(Scaled)
getArgD $ARGV -lab      inName(Label)
getArgD $ARGV -w        inName(Weight) 
getArgD $ARGV -maskName inName(Mask) 

set inName(Peak) $inName(Off)

getArgD $ARGV -lab  labName
getArgD $ARGV -tab  tabName
getArgD $ARGV -pk   pkTabName
getArgD $ARGV -std  stdName
getArgD $ARGV -plot plotName

getArgD $ARGV -nx1  nx1
getArgD $ARGV -nxn  nxn
getArgD $ARGV -plx1 plx1
getArgD $ARGV -plxn plxn
getArgD $ARGV -ax1  ax1
getArgD $ARGV -axn  axn
getArgD $ARGV -dx   stdDX
getArgD $ARGV -sx   stdSX
getArgD $ARGV -pkdx pkDX 

getArgD $ARGV -thresh thresh
getArgD $ARGV -noise  noise
getArgD $ARGV -nadj   noiseAdj
getArgD $ARGV -mult   mult
getArgD $ARGV -pChi   pChiThresh

getArgD $ARGV -plotWidth plotWidth
getArgD $ARGV -multiPlot multiPlot

if {[flagLoc $ARGV -fit]}   {set fitFlag 1}
if {[flagLoc $ARGV -nofit]} {set fitFlag 0}

if {[flagLoc $ARGV -pdf]}   {set pdfFlag 1}
if {[flagLoc $ARGV -nopdf]} {set pdfFlag 0}

if {[flagLoc $ARGV -showNoise]}   {set showNoiseFlag 1}
if {[flagLoc $ARGV -noshowNoise]} {set showNoiseFlag 0}

if {$pdfFlag} \
   {
    set pCmnd [auto_execok ps2pdf]
    if {[string length $pCmnd] < 1} {set pdfFlag 0}
   }

set cx1  0.5in
set cxn  [expr $plotWidth + 0.5]in
set cy1  2.0in
set cyn  7.0in

set my1  6.90in
set myn  7.00in

set qx1  0.5in
set qxn  [expr $plotWidth + 0.5]in
set qy1  0.5in
set qyn  2.0in

set sx1  0.80in
set sxn  2.00in
set sy1  7.40in
set syn  7.70in
set sxt  0.75in
set syt  7.90in

#
#------------------------------------------------------------------------------

proc getThisROI { type x1 xn } \
{
   global rdata fdata size dirName inName roiID

   if {![strcasecmp $inName($type) None]} \
      {
       set rdata($type) 0
       set fdata($type) 0
       set size($type)  0
       return
      }

   if {![file exists $dirName/$inName($type)]} \
      {
       puts stderr "Error Finding Spectrum $dirName/$inName($type)"
       exit 1
      }

   readROI -roi $roiID($type) -in $dirName/$inName($type) \
           -ndim 1 -x X_AXIS $x1 $xn

   set roiInfo [getROIInfo -roi $roiID($type)]

   set rdata($type)  [lindex $roiInfo 0]
   set fdata($type)  [lindex $roiInfo 1]
   set size($type)   [lindex [lindex $roiInfo 2] 0]
}

proc getThisStat { type rangeFrac } \
{
   global rdata size sum rms plotMin plotMax minVal maxVal

   if {![strcmp $rdata($type) 0]} \
      {
       set sum($type)          0.0
       set rms($type)          0.0
       set minVal($type)       0.0
       set maxVal($type)       0.0
       set plotMin($type)      0.0 
       set plotMax($type)      0.0
       return
      }

   set sum($type) [vSum $rdata($type) $size($type)]
   set rms($type) [vRMS $rdata($type) $size($type)]

   set minVal($type) [vMin $rdata($type) $size($type)]
   set maxVal($type) [vMax $rdata($type) $size($type)]

   set r [expr $maxVal($type) - $minVal($type)]

   set plotMin($type) [expr $minVal($type) - 0.05*$r]
   set plotMax($type) [expr $maxVal($type) + $rangeFrac*$r]

   set sum($type,Mask)     $sum($type) 
   set rms($type,Mask)     $rms($type) 
   set minVal($type,Mask)  $minVal($type) 
   set maxVal($type,Mask)  $maxVal($type) 
   set plotMin($type,Mask) $plotMin($type)
   set plotMax($type,Mask) $plotMax($type) 

   if {![strcmp $rdata(Mask) 0]} return
   if {$size($type) != $size(Mask)} return

   set rPtr [sysMalloc $size($type) float]

   vvMult  $rPtr $rdata($type) $rdata(Mask) $size($type)]

   set sum($type,Mask) [vSum $rPtr $size($type)]
   set rms($type,Mask) [vRMS $rPtr $size($type)]

   set minVal($type,Mask) [vMin $rPtr $size($type)]
   set maxVal($type,Mask) [vMax $rPtr $size($type)]

   set r [expr $maxVal($type,Mask) - $minVal($type,Mask)]

   set plotMin($type,Mask) [expr $minVal($type,Mask) - 0.05*$r]
   set plotMax($type,Mask) [expr $maxVal($type,Mask) + $rangeFrac*$r]
}

proc getThisVal { type xLoc f } \
{
   global rdata size

   if {![strcmp $rdata($type) 0]} {return 1.0} 
     
   set ix [expr int($xLoc + 0.5) - 1]
   set n  [expr $size($type) - 1]

   if {$ix < 0}  {set ix 0}
   if {$ix > $n} {set ix $n}

   set s [getPtrVal $rdata($type) $ix]

   return [expr $s*$f]
}

proc draw1DMask { roiID bx1 bxn by1 byn thisBG thisFG thresh } \
{
   set rInfo [getROIInfo -roi $roiID]
   set rPtr  [lindex $rInfo 0]
   set n     [lindex [lindex $rInfo 2] 0]
   set nn    [expr $n - 1]

   if {!$nn} return

   drawRec -x $bx1 $bxn -y $by1 $byn -fg $thisBG 

   set bbx1 [string trim [string tolower $bx1] incm]
   set bbxn [string trim [string tolower $bxn] incm]

   for {set i 0} {$i < $n} {incr i} \
      {
       set val [getPtrVal $rPtr $i]
       if {$val >= $thresh} continue

       set loc1 $i

       while {$val < $thresh && $i < $n} \
          {
           incr i
           set val [getPtrVal $rPtr $i]
          }

       if {$i > $nn} {set i $nn}

       set locN $i

       set thisX1 [expr $bbx1 + ($bbxn - $bbx1)*double($loc1)/$nn]in
       set thisXN [expr $bbx1 + ($bbxn - $bbx1)*double($locN)/$nn]in

       drawRec -x $thisX1 $thisXN -y $by1 $byn -fg $thisFG
      }
}
 
#
# Noise:

if {![strcasecmp $noise Auto] || ![strcasecmp $noise None]} {set noise 0.0}

if {$noise == 0.0} \
   {
    getThisROI Off $nx1 $nxn
    set noise [expr $noiseAdj*[vStdDev $rdata(Off) $size(Off)]]
   }

if {$noise == 0.0} {set noise 1.0}

if {![strcasecmp $thresh Auto] || ![strcasecmp $thresh None]} \
   {
    set thresh [expr $noise*$mult]
   }

getThisROI Off  0% 100% 

set stdDXN [expr int( 0.5 + [spec2pntW $fdata(Off) CUR_XDIM $stdDX])]
set stdSXN [expr int( 0.5 + [spec2pntW $fdata(Off) CUR_XDIM $stdSX])]

catch {exec nmr2Mask -in $dirName/$inName(Off) -out $dirName/$inName(Label) -max $thresh -label > $dirName/$tabName}

wrFDATA $dirName/$inName(Label) $fdata(Off)

getThisROI Mask   $ax1 $axn
getThisROI Off    $ax1 $axn
getThisROI Dif    $ax1 $axn
getThisROI Scaled $ax1 $axn
getThisROI Label  $ax1 $axn
getThisROI Weight $ax1 $axn
getThisROI Peak   $ax1 $axn

vvDiv $rdata(Label) $rdata(Label) $size(Label)

getThisStat Mask   0.05
getThisStat Label  0.05
getThisStat Off    0.30
getThisStat Scaled 0.30

#
# Peak analysis:

writeROI -out $dirName/$fitName -ov -roi $roiID(Off)

pkFindROI -roi $roiID(Peak) \
   -sigma $noise -pChi 1e-04 -plus $thresh -minus -1.0e16 \
   -dx    $pkDX \
   -idx   $pkDX \
   -tol   10.00 \
   -hiAdj  1.20 \
   -lw    15.00 \
   -reject -mask -out $dirName/$pkTabName -noverb

if {$fitFlag} \
   {
    catch \
     {exec xterm -e autoFit.tcl -dir $dirName -modX LORENTZ1D -simX 16000 \
                       -sim1D -specName $fitName -inTab peak.tab}
   }

getThisROI Off $ax1 $axn

set pkTab [gdbRead table -in $dirName/$pkTabName]
set eList [gdbGetInfo $pkTab -parms entries]

if {![llength $eList]} exit

#
# Avg STD:

set hiSum   0.0
set stdSum  0.0
set stdList ""
set ppmList ""

foreach entry $eList \
   {
    set hi     [expr [gdbGet $entry HEIGHT]/$noise]
    set id     [gdbGet $entry INDEX]
    set xLoc   [gdbGet $entry X_AXIS]
    set xPPM   [gdbGet $entry X_PPM]

    set s($id) ""
    set m($id) [getThisVal Mask $xLoc 1.0]

    if {$m($id) == 0.0} continue

    for {set i -$stdDXN} {$i <= $stdDXN} {incr i} \
       { 
        lappend s($id) [getThisVal Weight [expr $xLoc + $i] 100.0]
        lappend p($id) [getThisVal Dif    [expr $xLoc + $i] 1.0]
       }

    set p($id) [getPChi $p($id) $noise [expr 1 + 2*$stdDXN]]
    set s($id) [getAvg $s($id)]
    set r($id) $s($id)

    if {$p($id) < $pChiThresh} {set s($id) 0.0}

    set hiSum  [expr $hiSum + abs($hi)]
    set stdSum [expr $stdSum + $s($id)*abs($hi)]

    lappend ppmList $xPPM
    lappend stdList $s($id)
   }

if {$hiSum == 0.0} {set hiSum 1.0}

set stdAvg [expr $stdSum/$hiSum]
set stdMin [getMin $stdList]
set stdMax [getMax $stdList]

#
# STD Output table:

set error [catch "set outUnit [open $dirName/$stdName w]"]

if {$error} \
   {
    puts stderr "Error Opening Output Table $stdName"
    exit 1
   }

puts $outUnit "DATA STD_DIR        $dirName"
puts $outUnit "DATA STD_NOISE      [format %.4e $noise]"
puts $outUnit "DATA STD_NOISE_REG  $nx1 $nxn"
puts $outUnit "DATA STD_PK_THRESH  [format %.4e $thresh]"
puts $outUnit "DATA STD_CHI_THRESH [format %.4f $pChiThresh]"
puts $outUnit "DATA STD_REG        $ax1 $axn"
puts $outUnit "DATA STD_SCALE_WIN  $stdSX ${stdSXN}pts"
puts $outUnit "DATA STD_CALC_WIN   $stdDX ${stdDXN}pts"
puts $outUnit "DATA STD_MIN        [format %.2f $stdMin]"
puts $outUnit "DATA STD_MAX        [format %.2f $stdMax]"
puts $outUnit "DATA STD_WAVG       [format %.2f $stdAvg]"
puts $outUnit ""

puts $outUnit "VARS   INDEX X_PPM HEIGHT PCHI  STD_R STD_PCT"
puts $outUnit "FORMAT %4d   %8.3f %7.2f  %6.4f %6.2f %6.2f"
puts $outUnit ""

set specLabel ""

append specLabel "Min: [format %.2f $stdMin]"
append specLabel " Max: [format %.2f $stdMax]"
append specLabel " Weighted Avg: [format %.2f $stdAvg]"

puts "Spectrum: $dirName STD $specLabel"

foreach entry $eList \
   {
    set hi   [expr [gdbGet $entry HEIGHT]/$noise]
    set id   [gdbGet $entry INDEX]
    set xLoc [gdbGet $entry X_AXIS]
    set xPPM [gdbGet $entry X_PPM]

    if {$m($id) == 0.0} continue

    set fmt \
     " Peak: %3d %8.3fppm   S/N: %7.2f   PChi: %6.4f  R: %6.2f  STD%%: %6.2f"

    puts [format $fmt $id $xPPM $hi $p($id) $r($id) $s($id)]

    set fmt " %3d %8.3f %7.2f %6.4f %6.2f %6.2f"
    puts $outUnit [format $fmt $id $xPPM $hi $p($id) $r($id) $s($id)]
   }

close $outUnit

#
# Plot:

plotOpen -plotName $dirName/$plotName -landscape -color 

if {![strcasecmp $plx1 Auto]} \
   {
    set px1 [expr [getMin $ppmList] - 0.1]ppm
   } \
else \
   {
    set px1 $plx1
   }

if {![strcasecmp $plxn Auto]} \
   {
    set pxn [expr [getMax $ppmList] + 0.1]ppm
   } \
else \
   {
    set pxn $plxn
   }

getThisROI Mask   $px1 $pxn
getThisROI Off    $px1 $pxn
getThisROI Dif    $px1 $pxn
getThisROI Scaled $px1 $pxn

getThisStat Mask    0.05
getThisStat Off     0.30
getThisStat Dif     0.05 
getThisStat Scaled  0.05

defineCell -cell 1 -roiRef $roiID(Off)    -x $cx1 $cxn -y $cy1 $cyn
defineCell -cell 2 -roiRef $roiID(Scaled) -x $qx1 $qxn -y $qy1 $qyn
defineCell -cell 3 -roiRef $roiID(Off)    -x $sx1 $sxn -y $sy1 $syn
defineCell -cell 4 -roiRef $roiID(Off)    -x $cx1 $cxn -y $my1 $myn

if {[strcmp $rdata(Mask) 0]} \
   {
    if {$minVal(Mask) == 0 && $maxVal(Mask) > 0} \
       {
        draw1DMask $roiID(Mask) $cx1 $cxn $my1 $myn white yellow 0.1
       }
   }

if {$multiPlot < 1} \
   {
    set boxArg box
   } \
else \
   {
    set boxArg nobox
   }

drawROI -cell 1 -roi $roiID(Dif) -zero \
        -fhi $plotMin(Off,Mask) -lhi $plotMax(Off,Mask) -fg red -top -$boxArg

drawROI -cell 1 -roi $roiID(Off) \
        -fhi $plotMin(Off,Mask) -lhi $plotMax(Off,Mask) -fg black -axis Bottom -top -$boxArg

drawROI -cell 2 -roi $roiID(Dif) -zero -nobox \
        -fhi $plotMin(Scaled,Mask) -lhi $plotMax(Scaled,Mask) -fg red -top

drawROI -cell 2 -roi $roiID(Scaled) -nobox \
        -fhi $plotMin(Scaled,Mask) -lhi $plotMax(Scaled,Mask) -fg black -top

if {![strcasecmp $boxArg nobox]} \
   {
    drawLine -thick -fg black -x $cx1 $cxn -y $cyn $cyn
   }

set entry    [lindex $eList 0]

set dPPM       0.05
set groupCount 0

set firstPPM   [gdbGet $entry X_PPM]
set prevPPM    [expr $firstPPM + 2.0*$dPPM]
set range      [expr $plotMax(Off,Mask) - $plotMin(Off,Mask)]

if {$range == 0.0} {set range 1.0}

foreach entry $eList \
   {
    set id   [gdbGet $entry INDEX]
    set hi   [gdbGet $entry HEIGHT]
    set xPPM [gdbGet $entry X_PPM]
    
    if {$m($id) == 0.0} continue
 
    if {$prevPPM > $xPPM + $dPPM} \
       {
        incr groupCount
       }

    lappend gInfo($groupCount,eList) $entry
    lappend gInfo($groupCount,ppm)   $xPPM
    lappend gInfo($groupCount,hi)    $hi

    set prevPPM $xPPM
   }

if {$multiPlot <= 1} {set specLabel "Spectrum: $dirName STD $specLabel"}

drawText -big -cell 1 -roiRef $roiID(Off) -x 2% -y 5% -fg black -text $specLabel

if {$multiPlot <= 1} \
   {
    drawText -x 70% -y 95% -fg black -text "NMRPipe STD at [sysTime "%Y.%j.%H.%M"]"
   }

set dty 3.0 

for {set group 1} {$group <= $groupCount} {incr group} \
   {
    set n  [llength $gInfo($group,eList)]

    if {!$n} continue

    set hi [getMax  $gInfo($group,hi)]
    set ty [expr -$dty*$n + 100.0*($plotMax(Off,Mask) - $hi)/$range]

    if {$ty < $dty} {set ty $dty}

    for {set i 0} {$i < $n} {incr i} \
       {
        set entry [lindex $gInfo($group,eList) $i]
        set xPPM  [gdbGet $entry X_PPM]
        set id    [gdbGet $entry INDEX]

        if {$m($id) == 0.0} continue

        if {$p($id) < $pChiThresh} \
           {
            set color red
           } \
        else \
           {
            set color blue 
           }

        drawText -cell 1 -roiRef $roiID(Off) -center \
                 -x ${xPPM}ppm -y ${ty}% -bg None \
                 -fg $color -nobox -text [format "%.3f" $s($id)]

        set ty [expr $ty + $dty] 
       }
   }

getThisROI  Mask $nx1 $nxn
getThisROI  Off  $nx1 $nxn

getThisStat Mask 0.05
getThisStat Off  0.05

if {$multiPlot <= 1} \
   {
    drawROI -cell 3 -roi $roiID(Off) -hi Auto -zero -fg black -fhi $plotMin(Off) -lhi $plotMax(Off)

    drawText  -x $sxt -y $syt -text "Noise Region $nx1 $nxn"
 
    plotClose
   }

if {$pdfFlag} \
   {
    set pdfName [file root $plotName].pdf
    catch "exec stdPDF.com $dirName/$plotName $dirName/$pdfName $dirName/pdf.log"
   }
 
exit
