#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Convert Object Coordinates from a Table into Density Surfaces:"
    puts " -in     inName    \[pcaCoords.tab\]  Input Table." 
    puts " -surf   surfDir   \[surf\]           Surface Directory."
    puts " -var    varList   \[PCA1 PCA2\]      Coordinate Variables."
    puts " -report rList     \[TITLE\]          Other Variables to Report."
    exit 0
   }

set inName    pcaCoords.tab
set surfDir   surf

set varList    "PCA1 PCA2"
set reportList "TITLE"

getArgD $ARGV -in         inName
getArgD $ARGV -surf       surfDir

getListArgD $ARGV -var    varList
getListArgD $ARGV -report reportList

#
#------------------------------------------------------------------------------------------------------------

proc getSurfVal { roiID valList } \
{
   global xSize ySize fdata rdata maxVal sumVal

   set xLoc [expr int(0.5+[spec2pnt $fdata($roiID) X_AXIS [lindex $valList 0]user])] 
   set yLoc [expr int(0.5+[spec2pnt $fdata($roiID) Y_AXIS [lindex $valList 1]user])]

   if {$xLoc < 1} {set xLoc 1} 
   if {$xLoc > $xSize($roiID)} {set xLoc $xSize($roiID)}

   if {$yLoc < 1} {set yLoc 1} 
   if {$yLoc > $ySize($roiID)} {set yLoc $ySize($roiID)}


   set loc [expr $xLoc + $xSize($roiID)*($yLoc - 1) - 1]
   set val [expr 256*256*[safeDiv [getPtrVal $rdata($roiID) $loc] $sumVal($roiID)]]

   return $val
}

if {![file exists $inName]} \
   {
    puts "surf2score Error Finding Input Table $inName"
    exit 1
   }

set fList [glob -nocomplain $surfDir/*]

if {![llength $fList]} \
   {
    puts "surf2score Error Finding Data in Directory $surfDir"
    exit 1
   }

set surfList ""
set allFlag  0

foreach f $fList \
   {
    set fName [file tail $f]

    if {![strcmp $fName All.dat]} \
       {
        set allFlag 1
       } \
    else \
       {
        lappend surfList $f
       }
   }

if {$allFlag} {lappend surfList $surfDir/All.dat}

set surfCount [llength $surfList]
set varCount  [llength $varList]

set t     [gdbRead table -in $inName]
set eList [gdbGetInfo $t -parms entries]

for {set i 0} {$i < $surfCount} {incr i} \
   {
    set roiID [expr $i + 1]

    puts -nonewline [format " %-12s" [file rootname [file tail [lindex $surfList $i]]]]

    readROI -ndim 2 -in [lindex $surfList $i] -x X_AXIS 0% 100% -y Y_AXIS 0% 100% -roi $roiID

    set rInfo [getROIInfo -roi $roiID]

    set rdata($roiID) [lindex $rInfo 0]
    set fdata($roiID) [lindex $rInfo 1]

    set xSize($roiID) [lindex [lindex $rInfo 2] 0] 
    set ySize($roiID) [lindex [lindex $rInfo 2] 1] 

    set maxVal($roiID) [getROIStat $roiID vMax]
    set sumVal($roiID) [getROIStat $roiID vSum]
   }

foreach var $reportList \
   {
    puts -nonewline [format " %-12s" $var]
   }

puts ""
puts ""

foreach e $eList \
   {
    set valList [getEntryVals $e $varList]

    for {set i 0} {$i < $surfCount} {incr i} \
       {
        puts -nonewline [format " %8.4f" [getSurfVal [expr $i + 1] $valList]]
       }

    foreach var $reportList \
       {
        puts -nonewline [format " %s" [gdbGet $e $var]]
       }

    puts ""
   }

