#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "SVD on Columns of a Table:"
    puts " -in inName \[test.tab\]  Table Input."
    puts " -y  yVar   \[None\]      Table Variable for Y-Values."
    puts " -x  xList  \[None\]      Table Variables for X-Matrix."
    puts " -c                     Include a Constant Column in X-Matrix."
    puts " -verb                  Verbose Mode ON."
    puts "PDB Options:"
    puts " -pdb                   Input is PDB Data (Default for .pdb)."
    puts " -nopdb                 Input is NMRPipe-Format Table."
    puts " -hetatm                Interpret HETATMs as ATOMs."
    exit 1
   }

set inName  test.tab
set yVar    None
set xList   None
set pdbMode nopdb
set hetMode nohetatm
set verbose 0
set cFlag   0

getArgD $ARGV -in inName
getArgD $ARGV -y  yVar

getListArgD $ARGV -x  xList

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -c]}   {set cFlag 1}
if {[flagLoc $ARGV -noc]} {set cFlag 0}

if {![strcasecmp [file extension $inName] .pdb]} {set pdbMode pdb}

if {[flagLoc $ARGV -pdb]}       {set pdbMode pdb}
if {[flagLoc $ARGV -nopdb]}     {set pdbMode nopdb}

if {[flagLoc $ARGV -hetatm]}    {set hetMode hetatm}
if {[flagLoc $ARGV -nohetatm]}  {set hetMode nohetatm}

#
#-------------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts stderr "Error finding input table $inName"
    exit 1
   }

set t   [gdbRead table -in $inName -$pdbMode -$hetMode]
set yID [gdbFindByName $t $yVar]

if {![llength $yID]} \
   {
    puts "Error Finding Table Y Variable $yVar"
    exit 0
   }

foreach xVar $xList \
   {
    set xID [gdbFindByName $t $xVar]

    if {![llength $xID]} \
       {
        puts "Error Finding Table Variable $xVar"
        exit 0
       }
   }

set entry [gdbFirst $t]

set xMatrix ""
set yArray  ""

while {$entry} \
   {
    lappend yArray [gdbGet $entry $yVar]

    foreach xVar $xList \
       {
        lappend xMatrix [gdbGet $entry $xVar]
       }

    if {$cFlag} {lappend xMatrix 1.0}

    set entry [gdbNext $entry]
   }

set coef  [svdLinSol -x $xMatrix -y $yArray]

if {!$verbose} \
   {
    puts $coef
    exit
   }

set i 0

foreach c $coef \
   {
    puts stderr [format " %3d % 10.6f" $i $c]
    incr i
   }

set i 0

foreach y $yArray \
   {
    set s 0.0
    set j 0

    foreach a $coef \
       {
        set x [lindex $xMatrix $i]
        set s [expr $s + $a*$x]
        incr i
       } 

    puts [format "%f %f" $y $s]
   }

exit
