#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

# tView.tcl
#
#   3D TOCSY Viewer w/Sidechain Assigment
#   (C) Frank Delaglio 1998-2007
#-----------------------------------------------------------

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

proc initVars {} \
{
   global objInfo osType

   set    objInfo(prog,title) "TOCSYView 2007.068.09.07 (C) F. Delaglio"
   append objInfo(prog,title) "   "
   append objInfo(prog,title) "Mouse: \[Add by Peak\] \[Delete\] \[Add by Loc\]"
   append objInfo(prog,title) "   "
   append objInfo(prog,title) "Modifiers: Shift Key"

   setOSType
   setFonts

   gdbInit
   gdbCreate dBase

#  option add *Label.font  "-*-helvetica-bold-r-*-*-*-100-*-*-*-*-*-*"
#  option add *Button.font "-*-helvetica-bold-r-*-*-*-100-*-*-*-*-*-*"
#  option add *Entry.font  "-*-helvetica-medium-r-*-*-*-100-*-*-*-*-*-*"
#  option add *Text.font   "-*-courier-*-r-*-*-*-100-*-*-*-*-*-*"

#
# User-adjustable parameters:

   set objInfo(prog,verbose)   1

   if {[string match "IRIX*" $osType]} \
      {
       showVerb "Initializing in SGI Mode ..."
       set objInfo(prog,sgiFlag) 1
      } \
   else \
      {
       showVerb "Initializing in Generic Mode ..."
       set objInfo(prog,sgiFlag) 0
      }

   set objInfo(data,inName)    hcchtocsy/ft/hcchtocsy%03d.ft
   set objInfo(peak,inName)    tocsy.tab
   set objInfo(ass,inName)     assign.tab
   set objInfo(ref,inName)     ref.tab

   set objInfo(data,hi)           4.0e+6
   set objInfo(data,mult)         1.3
   set objInfo(data,cFirst)      70.0ppm
   set objInfo(data,cLast)       15.0ppm
   set objInfo(data,cFirst)      0%
   set objInfo(data,cLast)       100% 
   set objInfo(data,roiList)     ""
   set objInfo(data,readAllFlag) 0
 
   set objInfo(peak,xName)      X_PPM 
   set objInfo(peak,yName)      Y_PPM 
   set objInfo(peak,zName)      Z_PPM 

   set objInfo(ass,cTol)         0.600
   set objInfo(ass,hTol)         0.075
   set objInfo(ass,labelFlag)    0
   set objInfo(ass,trackFlag)    0
   
   set objInfo(color,bg)         black
   set objInfo(color,shadow)     gray30
   set objInfo(color,label)      white
   set objInfo(color,peak)       green 
   set objInfo(color,peakHilite) yellow
   set objInfo(color,cursor)     green

   if {$objInfo(prog,sgiFlag)} \
      {
       set objInfo(color,grid)   ltGray
      } \
   else \
      {
       set objInfo(color,grid)   dimGray
      }

#
# Other initializations:

   set objInfo(winHH,active)   1
   set objInfo(winCH,active)   1
   set objInfo(winHH,yAdj)     1.075
   set objInfo(winCH,yAdj)     1.033
   set objInfo(data,zoom)      1.0

   set objInfo(ass,tab)        0
   set objInfo(ref,tab)        0
   set objInfo(peak,tab)       0

   set objInfo(ass,resID)      0
   set objInfo(ass,resName)    ?
   set objInfo(ass,hList)      ""
   set objInfo(ass,cList)      ""
   set objInfo(ass,pkListHH)   ""
   set objInfo(ass,pkListCH)   ""
   set objInfo(ass,assList)    ""
   set objInfo(ass,hilite)     None

   set objInfo(ref,refList)    ""
   set objInfo(ref,selection)  0
   set objInfo(ref,hilite)     None
}

proc initArgs {} \
{
   global objInfo ARGC ARGV

   if {[flagLoc $ARGV -help]} \
      {
       puts "3D TOCSY Viewer w/Sidechain Assigment:"
       puts " -peak pName  NMRDraw Peak Table Name."
       puts " -ref  rName  Chemical Shift Reference Table Name."
       puts " -ass  aName  Assignment Table Name."
       puts " -in   sName  TOCSY Spectrum Input Template."
       puts " -hi   hiVal  First Contour Height."
       puts " -mult c      Contour Height Multiplier."
       puts " -all         Read Entire 3D at Start."
       puts " -noall       Don't Read Entire 3D at Start (Default)."
       puts " -lab         Assign Labels ON."
       puts " -nolab       Assign Labels OFF (Default)."
       puts " -track       Cursor Tracking ON."
       puts " -notrack     Cursor Tracking OFF (Default)."
       exit 0
      }

   if {[flagLoc $ARGV -all]}   {set objInfo(data,readAllFlag) 1}
   if {[flagLoc $ARGV -noall]} {set objInfo(data,readAllFlag) 0}

   if {[flagLoc $ARGV -lab]}   {set objInfo(ass,labelFlag) 1}
   if {[flagLoc $ARGV -nolab]} {set objInfo(ass,labelFlag) 0}

   if {[flagLoc $ARGV -track]}   {set objInfo(ass,trackFlag) 1}
   if {[flagLoc $ARGV -notrack]} {set objInfo(ass,trackFlag) 0}

   getArgD $ARGV -peak    objInfo(peak,inName)
   getArgD $ARGV -ref     objInfo(ref,inName)
   getArgD $ARGV -ass     objInfo(ass,inName)
   getArgD $ARGV -in      objInfo(data,inName)
   getArgD $ARGV -hi      objInfo(data,hi)
   getArgD $ARGV -mult    objInfo(data,mult)
}

proc initTabs {} \
{
   global objInfo

   showVerb "Reading peak table ..."

   set objInfo(peak,tab) \
      [gdbRead table -in $objInfo(peak,inName) -name "Peaks"]

   showVerb "Reading assignment table ..."

   set objInfo(ass,tab) \
      [gdbRead table -in $objInfo(ass,inName) -name "Assignments"]

   showVerb "Reading reference table ..."

   set objInfo(ref,tab) \
      [gdbRead table -in $objInfo(ref,inName) -name "Typical Shifts"]

   if {$objInfo(ass,tab)} \
      {
       set entry [gdbFirst $objInfo(ass,tab)]
       set objInfo(ass,resID)   [gdbGet $entry RESID]
       set objInfo(ass,resName) [gdbGet $entry RESNAME]
      }

   showVerb "Done reading tables."
}

proc initSpectralData {} \
{
   global objInfo

   set obj data

   set objInfo($obj,valid)    0
   set objInfo($obj,rdata)    0
   set objInfo($obj,fdata)    0
   set objInfo($obj,roi3D)    10

   set objInfo($obj,minVal)   0.0
   set objInfo($obj,maxVal)   0.0
   set objInfo($obj,xSize)    0
   set objInfo($obj,ySize)    0
   set objInfo($obj,zSize)    0 
   set objInfo($obj,xPos)     ""
   set objInfo($obj,yPos)     ""
   set objInfo($obj,zPos)     ""

#
# Read data header.
# Extract size parameters:

   set error [rdFDATA $objInfo(data,inName) fdata]
   if {$error} return

   set objInfo(data,fdata)  $fdata
   set objInfo(data,tpFlag) [getParm $fdata FDTRANSPOSED NULL_DIM]

   if {$objInfo(data,tpFlag)} \
      {
       set hDim CUR_YDIM
       set vDim CUR_XDIM

       set objInfo(peak,xName)     Y_PPM
       set objInfo(peak,yName)     X_PPM
      } \
   else \
      {
       set hDim CUR_XDIM
       set vDim CUR_YDIM

       set objInfo(peak,xName)     X_PPM
       set objInfo(peak,yName)     Y_PPM
      }

   set objInfo($obj,xAxis) [getParmStr $fdata NDLABEL $hDim]
   set objInfo($obj,yAxis) [getParmStr $fdata NDLABEL $vDim]
   set objInfo($obj,zAxis) [getParmStr $fdata NDLABEL CUR_ZDIM]

   set objInfo($obj,xSize) [expr int([getParm $fdata NDSIZE $hDim])]
   set objInfo($obj,ySize) [expr int([getParm $fdata NDSIZE $vDim])]
   set objInfo($obj,zSize) [expr int([getParm $fdata NDSIZE CUR_ZDIM])]

   set objInfo($obj,zSWHz)  [getParm $fdata NDSW    CUR_ZDIM]
   set objInfo($obj,zOBS)   [getParm $fdata NDOBS   CUR_ZDIM]
   set objInfo($obj,zSW)    [expr $objInfo($obj,zSWHz) / $objInfo($obj,zOBS)]

   set objInfo($obj,zFirst) [pnt2spec $fdata CUR_ZDIM 1 ppm]
   set objInfo($obj,zLast)  [pnt2spec $fdata CUR_ZDIM $objInfo($obj,zSize) ppm]

   set objInfo($obj,ix)    [expr int($objInfo($obj,xSize)/2)]
   set objInfo($obj,iy)    [expr int($objInfo($obj,ySize)/2)]
   set objInfo($obj,iz)    [expr int($objInfo($obj,zSize)/2)]

   set objInfo($obj,x1)    1
   set objInfo($obj,xn)    $objInfo($obj,xSize)
   set objInfo($obj,y1)    1
   set objInfo($obj,yn)    $objInfo($obj,ySize)

#
# Read entire spectrum if needed:

   if {$objInfo(data,readAllFlag)} \
      {
       wm title . "Reading 3D Data ..."
       update

       showVerb "Reading 3D spectrum ..."

       readROI -ndim 3 -in $objInfo($obj,inName) -roi $objInfo($obj,roi3D) \
               -x X_AXIS 0% 100% -y Y_AXIS 0% 100% -z Z_AXIS 0% 100%

       wm title . $objInfo(prog,title)
       update
      }
}

proc resizeView args \
{
   global objInfo

   set obj  [lindex $args 0]

   bind $objInfo($obj,canvas) <Configure> ""

   showVerb "Resize event $args"

   update

   set objInfo($obj,winXSize) [winfo width  $objInfo($obj,window)]
   set objInfo($obj,winYSize) [winfo height $objInfo($obj,window)]

   set objInfo($obj,cellXSize) \
       [expr $objInfo($obj,winXSize)     - \
             ($objInfo($obj,leftMargin)  + \
              $objInfo($obj,rightMargin) + \
              $objInfo($obj,winXAdj))]

   set objInfo($obj,cellYSize) \
       [expr $objInfo($obj,winYSize)      - \
             ($objInfo($obj,topMargin)    + \
              $objInfo($obj,bottomMargin) + \
              $objInfo($obj,winYAdj))]
    
   set objInfo($obj,cellXSize) \
       [expr int($objInfo(data,zoom)*$objInfo($obj,cellXSize))] 

   set objInfo($obj,cellYSize) \
       [expr int($objInfo(data,zoom)*$objInfo($obj,cellYSize))] 

   if {$objInfo(data,zoom) > 1.0} \
      {
       set objInfo($obj,xSize) [expr $objInfo($obj,cellXSize)  + \
                                     $objInfo($obj,leftMargin) + \
                                     $objInfo($obj,rightMargin)]
      } \
   else \
      {
       set objInfo($obj,xSize) \
           [expr $objInfo($obj,winXSize) - $objInfo($obj,winXAdj)]
      }

   if {$objInfo($obj,xSize)     < 50} {set objInfo($obj,xSize) 50}
   if {$objInfo($obj,cellXSize) < 50} {set objInfo($obj,cellXSize) 50}
   if {$objInfo($obj,cellYSize) < 50} {set objInfo($obj,cellYSize) 50}

   $objInfo($obj,canvas) delete pixmap -tags $obj

   $objInfo($obj,canvas) create pixmap 0 0 \
      -width $objInfo($obj,xSize) -height $objInfo($obj,ySize) \
      -win $objInfo($obj,pixmap) -tags $obj

   $objInfo($obj,canvas) configure \
      -scrollregion "0 0 $objInfo($obj,xSize) $objInfo($obj,ySize)"
 
   if {![flagLoc $args "-noread"]} \
      {
       readRes
      }

   if {![flagLoc $args "-nodraw"]} \
      {
       showRes
       showPeaks
      }

   bind $objInfo($obj,canvas) <Configure> "resizeView $obj %w %h %T"
}

proc initHHWindow {} \
{
    global objInfo

    set obj winHH

    if {!$objInfo($obj,active)} return

    set objInfo($obj,winXSize)     870
    set objInfo($obj,winYSize)     440 

    set objInfo($obj,leftMargin)    20 
    set objInfo($obj,rightMargin)  120 
    set objInfo($obj,topMargin)     10
    set objInfo($obj,bottomMargin) 100

    set objInfo($obj,winXAdj)        5 
    set objInfo($obj,winYAdj)       20 
    set objInfo($obj,cellX)         12
    set objInfo($obj,cellY)         18 

    set objInfo($obj,cellXSize) \
        [expr $objInfo($obj,winXSize)     - \
              ($objInfo($obj,leftMargin)  + \
               $objInfo($obj,rightMargin) + \
               $objInfo($obj,winXAdj))]

    set objInfo($obj,cellYSize) \
        [expr $objInfo($obj,winYSize)      - \
              ($objInfo($obj,topMargin)    + \
               $objInfo($obj,bottomMargin) + \
               $objInfo($obj,winYAdj))]
 
    set objInfo($obj,xSize) \
        [expr $objInfo($obj,winXSize) - $objInfo($obj,winXAdj)]

    set objInfo($obj,ySize)       9500

    set objInfo($obj,window)       .
    set objInfo($obj,canvas)       .c
    set objInfo($obj,scrollH)      .sh
    set objInfo($obj,scrollV)      .sv
    set objInfo($obj,pixmap)       1
    set objInfo($obj,firstCell)    0
    set objInfo($obj,lastCell)     0
    set objInfo($obj,firstID)      100
    set objInfo($obj,cursorList)   ""

    wm title $objInfo($obj,window) $objInfo(prog,title)

    frame .top
    frame .bottom

    button .next -text "Next"   -command "nextRes"
    button .prev -text "Prev"   -command "prevRes"
    button .draw -text "Draw"   -command "drawRes"
    button .ltog -text "Labels" -command "toggleAssLabels"
    button .save -text "Save"   -command "saveView"
    button .help -text "Help"   -command "helpView"
    button .quit -text "Quit"   -command "quitView"

    label  .space1 -text "   "
    label  .labr   -text "ResID:"
    entry  .resid  -textvariable objInfo(ass,resID) -width 6
    bind   .resid  <Return> "+ newRes"
   
    label  .space2 -text "   "
    label  .labz   -text "Zoom:"
    entry  .zoom   -textvariable objInfo(data,zoom) -width 6
    bind   .zoom   <Return> "+ newZoom"

    label  .space3 -text "   "
    label  .labf   -text "First:"
    entry  .first1 -textvariable objInfo(data,hi)   -width 12
    button .incr1  -text "+" -command incrHi
    button .decr1  -text "-" -command decrHi

    bind   .first1 <Return> "+ setHi"
 
    pack .next .prev .draw .ltog .save .quit \
         .space1 .labr .resid                \
         .space2 .labz .zoom                 \
         .space3 .labf .first1 .incr1 .decr1 \
         -in .top -side left 

    pack .help -in .top -side right

#
# Canvas and scrollbars:

    scrollbar $objInfo($obj,scrollV) \
       -orient vert -command "$objInfo($obj,canvas) yview"

    scrollbar $objInfo($obj,scrollH) \
       -orient horiz -command "$objInfo($obj,canvas) xview"

    canvas $objInfo($obj,canvas) -border 0 \
       -width $objInfo($obj,xSize) -height $objInfo($obj,ySize) \
       -xscrollcommand "$objInfo($obj,scrollH) set"             \
       -yscrollcommand "$objInfo($obj,scrollV) set"             \
       -scrollregion "0 0 $objInfo($obj,xSize) $objInfo($obj,ySize)"

    wm geom  $objInfo($obj,window) \
       =$objInfo($obj,winXSize)x$objInfo($obj,winYSize)+5+5

    update

    pack $objInfo($obj,scrollV) -in .bottom -side right  -expand 0 -fill y 
    pack $objInfo($obj,scrollH) -in .bottom -side bottom -expand 0 -fill x
    pack $objInfo($obj,canvas)  -in .bottom -side left   -expand 1 -fill both

    pack .top .bottom -side top -expand 1 -fill both

    update

    $objInfo($obj,canvas) create pixmap 0 0 \
       -width $objInfo($obj,xSize) -height $objInfo($obj,ySize) \
       -win $objInfo($obj,pixmap) -tags $obj

    bind $objInfo($obj,canvas) <Configure> "resizeView $obj %w %h %T"
}

proc initCHWindow {} \
{
    global objInfo

    set obj winCH

    if {!$objInfo($obj,active)} return

    set objInfo($obj,winXSize)    870 
    set objInfo($obj,winYSize)    400 

    set objInfo($obj,leftMargin)    20
    set objInfo($obj,rightMargin)  120
    set objInfo($obj,topMargin)     10
    set objInfo($obj,bottomMargin) 100

    set objInfo($obj,winXAdj)        5 
    set objInfo($obj,winYAdj)        5
    set objInfo($obj,cellX)         12
    set objInfo($obj,cellY)         18 

    set objInfo($obj,cellXSize) \
        [expr $objInfo($obj,winXSize)     - \
              ($objInfo($obj,leftMargin)  + \
               $objInfo($obj,rightMargin) + \
               $objInfo($obj,winXAdj))]

    set objInfo($obj,cellYSize) \
        [expr $objInfo($obj,winYSize)      - \
              ($objInfo($obj,topMargin)    + \
               $objInfo($obj,bottomMargin) + \
               $objInfo($obj,winYAdj))]

    set objInfo($obj,xSize) \
        [expr $objInfo($obj,winXSize) - $objInfo($obj,winXAdj)]

    set objInfo($obj,ySize)       9500

    set objInfo($obj,window)       .q
    set objInfo($obj,canvas)       .q.c
    set objInfo($obj,scrollH)      .q.sh
    set objInfo($obj,scrollV)      .q.sv
    set objInfo($obj,pixmap)       2
    set objInfo($obj,firstCell)    0
    set objInfo($obj,lastCell)     0
    set objInfo($obj,firstID)      200
    set objInfo($obj,cursorList)   ""

    toplevel $objInfo($obj,window)

    wm title $objInfo($obj,window) $objInfo(prog,title)

#
# Canvas and scrollbars:

    scrollbar $objInfo($obj,scrollV) \
       -orient vert -command "$objInfo($obj,canvas) yview"

    scrollbar $objInfo($obj,scrollH) \
       -orient horiz -command "$objInfo($obj,canvas) xview"

    canvas $objInfo($obj,canvas) -border 0 \
       -width $objInfo($obj,xSize) -height $objInfo($obj,ySize) \
       -xscrollcommand "$objInfo($obj,scrollH) set"             \
       -yscrollcommand "$objInfo($obj,scrollV) set"             \
       -scrollregion "0 0 $objInfo($obj,xSize) $objInfo($obj,ySize)"

    wm geom  $objInfo($obj,window) \
       =$objInfo($obj,winXSize)x$objInfo($obj,winYSize)+5-5

    update

    pack $objInfo($obj,scrollV) -side right  -expand 0 -fill y 
    pack $objInfo($obj,scrollH) -side bottom -expand 0 -fill x
    pack $objInfo($obj,canvas)  -side left   -expand 1 -fill both

    update

    $objInfo($obj,canvas) create pixmap 0 0 \
       -width $objInfo($obj,xSize) -height $objInfo($obj,ySize) \
       -win $objInfo($obj,pixmap) -tags $obj

    bind $objInfo($obj,canvas) <Configure> "resizeView $obj %w %h %T"
}

proc initRefWindow {} \
{
   global objInfo

   set obj ref

   set objInfo($obj,viewer) .refView
   set objInfo($obj,vList)  ""

   set nameList  "ATOMNAME SHIFT"
   set vNameList [gdbGetInfo $objInfo($obj,tab) -parms varNames]

   foreach vName $vNameList \
      {
       if {"$vName" != "ATOMNAME" && $vName != "SHIFT"} \
          {
           lappend nameList $vName
          }
      }

   foreach vName $nameList \
      {
       lappend objInfo($obj,vList) [gdbFindByName $objInfo($obj,tab) $vName]
      }

   showTable -tab $objInfo($obj,tab)  -var $nameList \
             -win $objInfo($obj,viewer) -event refWindowProc

   wm geom $objInfo($obj,viewer) -5+10
}

proc initAssWindow {} \
{
   global objInfo

   set obj ass

   set objInfo($obj,viewer) .assView
   set objInfo($obj,vList)  ""

   set nameList  "ATOMNAME SHIFT"
   set vNameList [gdbGetInfo $objInfo($obj,tab) -parms varNames]

   foreach vName $vNameList \
      {
       if {"$vName" != "ATOMNAME" && $vName != "SHIFT"} \
          {
           lappend nameList $vName
          }
      }

   foreach vName $nameList \
      {
       lappend objInfo($obj,vList) [gdbFindByName $objInfo($obj,tab) $vName]
      }

   showTable -tab $objInfo($obj,tab)  -var $nameList \
             -win $objInfo($obj,viewer) -event assWindowProc

   wm geom $objInfo($obj,viewer) -5-5
}

proc refWindowProc { w type ix iy } \
{
   global objInfo tableToolInfo

   if {"$type" == "Enter"} return
   if {"$type" == "Leave"} return

   set obj      ref
   set toolName $objInfo($obj,viewer)

   set eInfo   $tableToolInfo($toolName|$ix,$iy)
   set entry   [lindex $eInfo 0]
   set var     [lindex $eInfo 1]

   if {"$var" == "0" || "$entry" == "0"} return

   set varName [gdbGetInfo $var -parms name]

   if {"$varName" != "ATOMNAME"} return

   if {"$objInfo($obj,hilite)" != "None"} \
      {
       $objInfo($obj,hilite) configure \
          -background $tableToolInfo($toolName,bgColor)

       set objInfo($obj,hilite) "None"
      }

   set newAss [$w get]
   set oldAss [gdbGet $entry $varName]
   set s      [gdbGet $entry SHIFT]

   set objInfo($obj,selection) $entry
   set objInfo($obj,hilite)    $w

   $w configure -background green

   switch $type \
      {
       Return  {
                showVerb "Residue $objInfo(ass,resID)"
                showVerb "Manual Reference. Old: $oldAss New: $newAss ${s}ppm."
               }

       Release {
                showVerb "Residue $objInfo(ass,resID)"
                showVerb "Reference Selection: $newAss ${s}ppm."
               }

       default { return }
      }
}

proc assWindowProc { w type ix iy } \
{
   global objInfo tableToolInfo

   if {!$objInfo(ass,trackFlag)} \
      {
       if {"$type" == "Enter"} return
       if {"$type" == "Leave"} return
      }

   set obj      ass
   set toolName $objInfo($obj,viewer)

   set eInfo   $tableToolInfo($toolName|$ix,$iy)
   set entry   [lindex $eInfo 0]
   set var     [lindex $eInfo 1]

   if {"$var" == "0" || "$entry" == "0"} return

   set varName [gdbGetInfo $var -parms name]

   if {"$objInfo($obj,hilite)" != "None"} \
      {
       $objInfo($obj,hilite) configure \
          -background $tableToolInfo($toolName,bgColor)

       set objInfo($obj,hilite) "None"
      }

   set objInfo($obj,hilite) $w

   $w configure -background green

   switch $type \
      {
       Enter  {
               set atomName [gdbGet $entry ATOMNAME]
               set shift    [gdbGet $entry SHIFT]

               if {[string match H* $atomName] && ![string match HN $atomName]} \
                  {
                   set objInfo(data,xPos) ${shift}ppm
                   set objInfo(data,yPos) ${shift}ppm

                   updateCursor
                  }

               if {[string match C* $atomName] && ![string match C $atomName]} \
                  {
                   set objInfo(data,zPos) ${shift}ppm

                   updateCursor
                  }
              }

       Leave  {
              }

       Return {
               set newAss [$w get]

               if {"$newAss" == ""} \
                  {
                   showVerb "Destroying [gdbGetInfo $entry -parms values]"
                   gdbDestroy $entry

                   findRes
                   readRes
                   showRes
                   showPeaks
                  } \
               else \
                  {
                   set oldAss [gdbGet $entry $varName]
                   set s      [gdbGet $entry SHIFT]

                   gdbSet $entry $varName $newAss

                   showVerb "Residue $objInfo(ass,resID)"
                   showVerb "Manual Assignment. Old: $oldAss New: $newAss."
                  }
              }

       Release {
                if {"$varName" == "ATOMNAME"} \
                   {
                    if {!$objInfo(ref,selection)} return

                    set newAss   [gdbGet $objInfo(ref,selection) ATOMNAME]
                    set refShift [gdbGet $objInfo(ref,selection) SHIFT]
                    set oldAss   [gdbGet $entry $varName]
                    set s        [gdbGet $entry SHIFT]

                    set    thisName [$w cget -textvariable]
                    global $thisName
                             
                    gdbSet $entry $varName $newAss
                    set    $thisName $newAss

                    showVerb "Residue $objInfo(ass,resID)"

                    showVerb \
                     "Reference Assignment. Old: $oldAss New: $newAss ${s}ppm."

                    if {$objInfo(data,zSW) < 2.0*abs($refShift - $s)} \
                       {
                        set eInfo   $tableToolInfo($toolName|[expr $ix+1],$iy)
                        set ww      [lindex $eInfo 2]

                        $ww configure -background cyan 

                        if {$s > $refShift} \
                           {
                            set s [expr $s - $objInfo(data,zSW)]
                           } \
                        else \
                           {
                            set s [expr $s + $objInfo(data,zSW)]
                           }

                        set    thisName [$ww cget -textvariable]
                        global $thisName

                        gdbSet $entry SHIFT $s
                        set    $thisName $s

                        showVerb "Folding. Old: $oldAss New: $newAss ${s}ppm."
                       }
                   } \
                elseif {"$varName" == "SHIFT"} \
                   {
                    jumpView $entry
                   } 
               }

       default { return }
      }

   showAssLabels
}

proc showVerb args \
{
   global objInfo

   if {$objInfo(prog,verbose)} {puts [string trim $args "\{\}"]}
}

proc newRes {} \
{
   global objInfo

   wm title . "Working ..."
   update

   foreach obj {winHH winCH} \
      {
       set objInfo($obj,firstCell)   0
       set objInfo($obj,lastCell)    0
      }

   set objInfo(data,roiList)  ""
   set objInfo(ass,hList)     ""
   set objInfo(ass,cList)     ""

   readRes
   drawRes

   wm title . $objInfo(prog,title)
   update
}

proc drawRes {} \
{
   showRes
   showPeaks

   jumpView 0
}

proc newZoom {} \
{
   jumpView 0
   clearRes
   update

   resizeView winHH -noread -nodraw
   resizeView winCH -noread -nodraw

   readRes
   showRes
   showPeaks
}

proc nextRes {} \
{
   global objInfo

   incr objInfo(ass,resID)
   newRes
}

proc prevRes {} \
{
   global objInfo

   incr objInfo(ass,resID) -1
   newRes
}

proc incrHi {} \
{
   global objInfo

   set objInfo(data,hi) [expr $objInfo(data,hi)*$objInfo(data,mult)]
}

proc decrHi {} \
{
   global objInfo

   set objInfo(data,hi) [expr $objInfo(data,hi)/$objInfo(data,mult)]
}

proc setHi {} \
{
   global objInfo

   wm title . "Working ..."

   drawRes

   wm title . $objInfo(prog,title)
}

proc helpView {} \
{
}

proc quitView {} \
{
   saveView
   exit
}

proc saveView {} \
{
   global objInfo

   if {!$objInfo(ass,tab)} return

   set types \
      {
       { {All Files}  *      }
       { {tab}        {.tab} }
       { {ser}        {ser*} }
       { {fid}        {fid*} }
       { {dat}        {.dat .DAT} }
       { {seq}        {.seq} }
       { {Text}       {.txt} }
       { {TCL}        {.tcl} }
       { {C-Shell}    {.com .csh} }
       { {K-Shell}    {.ksh} }
       { {Shell}      {.sh}  }
      }

   set thisName \
     [tk_getSaveFile -title "Assignment Output" \
       -filetypes $types -initialfile $objInfo(ass,inName)]

   if {"$thisName" != "None" && "$thisName" != ""} \
      {
       gdbWrite $objInfo(ass,tab) $thisName
       showVerb "Wrote assignment table $thisName"
      }

}

proc findRes {} \
{
   global objInfo

#
# Update Table Viewers:

   set objInfo(ass,resName)   "?"
   set objInfo(ass,hList)     ""
   set objInfo(ass,cList)     ""
   set objInfo(ass,pkListHH)  ""
   set objInfo(ass,pkListCH)  ""
   set objInfo(ass,assList)   ""
   set objInfo(ass,hilite)    "None"

   set objInfo(ref,refList)   ""
   set objInfo(ref,selection) 0
   set objInfo(ref,hilite)    "None"

   set cond "RESID == $objInfo(ass,resID)"

   set objInfo(ass,assList) \
       [gdbSelect -from $objInfo(ass,tab) -cond $cond]

   if {[llength $objInfo(ass,assList)]} \
      {
       set objInfo(ass,resName) \
           [gdbGet [lindex $objInfo(ass,assList) 0] RESNAME]
      }

#  gdbUpdateTableTool -tab $objInfo(ass,tab) -vars $objInfo(ass,vList)  \
#     -name $objInfo(ass,viewer) -entries $objInfo(ass,assList)         \
#     -title "Assignments for $objInfo(ass,resName)$objInfo(ass,resID)"

   set cond "!strcmp( RESNAME, \"$objInfo(ass,resName)\" )"

   set objInfo(ref,refList) \
       [gdbSelect -from $objInfo(ref,tab) -cond $cond]

#  gdbUpdateTableTool -tab $objInfo(ref,tab) -vars $objInfo(ref,vList) \
#     -name $objInfo(ref,viewer) -entries $objInfo(ref,refList)        \
#     -title "Typical Shifts for $objInfo(ass,resName)"

   if {!$objInfo(ass,resID)}           return
   if {"$objInfo(ass,resName)" == "?"} return

#
# Update lists of 1H and 13C for this residue:
 
   set    cond "\$RESID == $objInfo(ass,resID) "
   append cond " && \[string match C* \$ATOMNAME\]"
   append cond " && !\[string match C \$ATOMNAME\]"

   set objInfo(ass,cList) \
       [gdbSelect -from $objInfo(ass,tab) -tcl -cond $cond]

   set    cond "\$RESID == $objInfo(ass,resID) "
   append cond " && \[string match H* \$ATOMNAME\]"
   append cond " && !\[string match HN \$ATOMNAME\]"

   set objInfo(ass,hList) \
      [gdbSelect -from $objInfo(ass,tab) -tcl -cond $cond]

#
# Peaks in each HH plane:

   set id $objInfo(winHH,firstID)

   foreach cEntry $objInfo(ass,cList) \
      {
       set cPos [gdbGet $cEntry SHIFT]

       while {$cPos > $objInfo(data,zFirst)} \
          {
           set cPos [expr $cPos - $objInfo(data,zSW)]
          }

       while {$cPos < $objInfo(data,zLast)} \
          {
           set cPos [expr $cPos + $objInfo(data,zSW)]
          }

       set cond   "near( $cPos, $objInfo(peak,zName), $objInfo(ass,cTol) )"
       set pkList [gdbSelect -from $objInfo(peak,tab) -cond $cond]

       foreach pk $pkList \
          {
           gdbSet $pk CLUSTID $id
           lappend objInfo(ass,pkListHH) $pk
          }

       incr id
      }

#
# Peaks in each CH plane:

   set id $objInfo(winCH,firstID)

   foreach hEntry $objInfo(ass,hList) \
      {
       set hPos [gdbGet $hEntry SHIFT]

       set cond   "near( $hPos, $objInfo(peak,xName), $objInfo(ass,hTol) )"
       set pkList [gdbSelect -from $objInfo(peak,tab) -cond $cond]
 
       foreach pk $pkList \
          {
           gdbSet $pk CLUSTID $id
           lappend objInfo(ass,pkListCH) $pk
          }

       incr id
      }
}

proc readRes {} \
{
   global objInfo

   if {!$objInfo(ass,resID)} return

   findRes

   readResHH
   readResCH
}

proc readResHH {} \
{
   global objInfo

   set obj winHH

   if {!$objInfo($obj,active)} return

   set ix1 $objInfo($obj,leftMargin)
   set ixn [expr $ix1 + $objInfo($obj,cellXSize)]
   set iy1 $objInfo($obj,topMargin)
   set iyn [expr $iy1 + $objInfo($obj,cellYSize)]

   set id $objInfo($obj,firstID) 

   set objInfo($obj,firstCell)  $id
   set objInfo($obj,cursorList) ""

   set objInfo(data,xPos)       ""
   set objInfo(data,yPos)       ""
   set objInfo(data,zPos)       ""

   $objInfo($obj,canvas) delete cursorH cursorV

   foreach cEntry $objInfo(ass,cList) \
      {
       set cName   [gdbGet $cEntry ATOMNAME]
       set cPPM    [gdbGet $cEntry SHIFT]ppm

       defineCell -cell $id \
          -x $ix1 $ixn -y $iy1 $iyn -win $objInfo($obj,pixmap)

       set qx [expr $ix1 - 4]
       set qy [expr $iy1 - 4]

       set idH [$objInfo($obj,canvas) create line \
                $ix1 $qy $ixn $qy -fill $objInfo(color,cursor) -tags cursorH]

       set idV [$objInfo($obj,canvas) create line \
                $qx $iy1 $qx $iyn -fill $objInfo(color,cursor) -tags cursorV]

       $objInfo($obj,canvas) raise $idH
       $objInfo($obj,canvas) raise $idV

       lappend objInfo($obj,cursorList) "$idH $idV $ix1 $ixn $iy1 $iyn $id"

       if {[lsearch -exact $objInfo(data,roiList) $id] < 0} \
          {
           showVerb "Defining H/H plane $id"

           if {$objInfo(data,readAllFlag)} \
              {
               showVerb "Reading HH plane from memory ..."

               readROI -roi $id -ndim 2 -inROI $objInfo(data,roi3D) \
                  -x  $objInfo(data,xAxis) 0%    100% \
                  -y  $objInfo(data,yAxis) 0%    100% \
                  -dz $objInfo(data,zAxis) $cPPM 1    \
                  -w  $objInfo(data,zAxis) -1
              }  \
            else \
              {
               showVerb "Reading HH plane from file ..."

               readROI -roi $id -ndim 2 -in $objInfo(data,inName) \
                  -x  $objInfo(data,xAxis) 0%    100% \
                  -y  $objInfo(data,yAxis) 0%    100% \
                  -dz $objInfo(data,zAxis) $cPPM 1    \
                  -w  $objInfo(data,zAxis) -1
              }

           lappend objInfo(data,roiList) $id
          }

       set iy1 [expr $iyn + $objInfo($obj,bottomMargin)]
       set iyn [expr $iy1 + $objInfo($obj,cellYSize)]

       set objInfo($obj,lastCell) $id
       incr id
      }

    bind $objInfo($obj,canvas) <B1-Motion> "moveCursorHH %x %y"
}

proc readResCH {} \
{
   global objInfo

   set obj winCH

   if {!$objInfo($obj,active)} return

   set ix1 $objInfo($obj,leftMargin)
   set ixn [expr $ix1 + $objInfo($obj,cellXSize)]
   set iy1 $objInfo($obj,topMargin)
   set iyn [expr $iy1 + $objInfo($obj,cellYSize)]

   set id $objInfo($obj,firstID) 

   set objInfo($obj,firstCell)  $id
   set objInfo($obj,cursorList) ""

   set objInfo(data,xPos)       ""
   set objInfo(data,yPos)       ""
   set objInfo(data,zPos)       ""

   $objInfo($obj,canvas) delete cursorH cursorV

   foreach hEntry $objInfo(ass,hList) \
      {
       set hName   [gdbGet $hEntry ATOMNAME]
       set hPPM    [gdbGet $hEntry SHIFT]ppm

       defineCell -cell $id \
          -x $ix1 $ixn -y $iy1 $iyn -win $objInfo($obj,pixmap)

       set qx [expr $ix1 - 4]
       set qy [expr $iy1 - 4]

       set idH [$objInfo($obj,canvas) create line \
                $ix1 $qy $ixn $qy -fill $objInfo(color,cursor) -tags cursorH]

       set idV [$objInfo($obj,canvas) create line \
                $qx $iy1 $qx $iyn -fill $objInfo(color,cursor) -tags cursorV]

       $objInfo($obj,canvas) raise $idH
       $objInfo($obj,canvas) raise $idV

       lappend objInfo($obj,cursorList) "$idH $idV $ix1 $ixn $iy1 $iyn $id"

       if {[lsearch -exact $objInfo(data,roiList) $id] < 0} \
          {
           showVerb "Defining C/H plane $id"

           if {$objInfo(data,readAllFlag)} \
              {
               showVerb "Reading CH plane from memory ..."

               readROI -roi $id -ndim 2 -inROI $objInfo(data,roi3D) \
                       -x  $objInfo(data,yAxis) 0%    100%  \
                       -y  $objInfo(data,zAxis) 0%    100%  \
                       -dz $objInfo(data,xAxis) $hPPM 1     \
                       -w  $objInfo(data,xAxis) -1
              } \
           else \
              {
               showVerb "Reading CH plane from file ..."

               readROI -roi $id -ndim 2 -in $objInfo(data,inName) \
                       -x  $objInfo(data,yAxis) 0%    100%  \
                       -y  $objInfo(data,zAxis) 0%    100%  \
                       -dz $objInfo(data,xAxis) $hPPM 1     \
                       -w  $objInfo(data,xAxis) -1
              }

           lappend objInfo(data,roiList) $id
          }

       set iy1 [expr $iyn + $objInfo($obj,bottomMargin)]
       set iyn [expr $iy1 + $objInfo($obj,cellYSize)]

       set objInfo($obj,lastCell) $id
       incr id
      }

    bind $objInfo($obj,canvas) <B1-Motion> "moveCursorCH %x %y"
}

proc clearRes {} \
{
   global objInfo

   foreach obj {winHH winCH} \
      {
       if {$objInfo($obj,active)} \
          {
           $objInfo($obj,canvas) delete peak plane ass

           newScreen -win $objInfo($obj,pixmap)
          }
      }
}

proc moveCursorHH { sxPix syPix } \
{
   global objInfo

   set obj winHH

   set xPix [$objInfo($obj,canvas) canvasx $sxPix]
   set yPix [$objInfo($obj,canvas) canvasy $syPix]
   set id   [findCellByPix $obj $sxPix $syPix]

   if {$id} \
      {
       set xPPM [pix2spec -cell $id -roi $id -x $xPix -units ppm]
       set yPPM [pix2spec -cell $id -roi $id -y $yPix -units ppm]

       set xStr "$objInfo(data,xAxis)=[format %.3f $xPPM]"
       set yStr "$objInfo(data,yAxis)=[format %.3f $yPPM]"

       wm title $objInfo($obj,window)  "$xStr $yStr"
       wm title $objInfo(winCH,window) $objInfo(prog,title) 

       set objInfo(data,xPos) ${xPPM}ppm
       set objInfo(data,yPos) ${yPPM}ppm
      } \
   else \
      {
       set objInfo(data,xPos) ""
       set objInfo(data,yPos) "" 
      }

   updateCursor
}

proc moveCursorCH { sxPix syPix } \
{
   global objInfo

   set obj winCH

   set xPix [$objInfo($obj,canvas) canvasx $sxPix]
   set yPix [$objInfo($obj,canvas) canvasy $syPix]
   set id   [findCellByPix $obj $sxPix $syPix]

   if {$id} \
      {
       set xPPM [pix2spec -cell $id -roi $id -x $xPix -units ppm]
       set yPPM [pix2spec -cell $id -roi $id -y $yPix -units ppm]

       set xStr "$objInfo(data,xAxis)=[format %.3f $xPPM]"
       set yStr "$objInfo(data,zAxis)=[format %.3f $yPPM]"

       wm title $objInfo($obj,window) "$xStr $yStr"
       wm title $objInfo(winHH,window) $objInfo(prog,title)

       set objInfo(data,xPos) ${xPPM}ppm
       set objInfo(data,zPos) ${yPPM}ppm
      } \
   else \
      {
       set objInfo(data,xPos) ""
       set objInfo(data,zPos) "" 
      }

   updateCursor
}

proc updateCursor {} \
{
   global objInfo

   set obj winHH

   foreach cInfo $objInfo($obj,cursorList) \
      {
       set idH [lindex $cInfo 0]
       set idV [lindex $cInfo 1]

       set ix1 [lindex $cInfo 2]
       set ixn [lindex $cInfo 3]
       set iy1 [lindex $cInfo 4]
       set iyn [lindex $cInfo 5]

       set id  [lindex $cInfo 6]

       if {$objInfo(data,xPos) == ""} \
          { 
           set qx [expr $ix1 - 4]
          } \
       else \
          {
           set qx [spec2pix -cell $id -roi $id -x $objInfo(data,xPos)]
          }

       if {$objInfo(data,yPos) == ""} \
          {
           set qy [expr $iy1 - 4]
          } \
       else \
          {
           set qy [spec2pix -cell $id -roi $id -y $objInfo(data,yPos)]
          }

       $objInfo($obj,canvas) coords $idV $qx $iy1 $qx $iyn
       $objInfo($obj,canvas) coords $idH $ix1 $qy $ixn $qy
      }

   set obj winCH

   foreach cInfo $objInfo($obj,cursorList) \
      {
       set idH [lindex $cInfo 0]
       set idV [lindex $cInfo 1]

       set ix1 [lindex $cInfo 2]
       set ixn [lindex $cInfo 3]
       set iy1 [lindex $cInfo 4]
       set iyn [lindex $cInfo 5]

       set id  [lindex $cInfo 6]

       if {$objInfo(data,xPos) == ""} \
          {
           set qx [expr $ix1 - 4]
          } \
       else \
          {
           set qx [spec2pix -cell $id -roi $id -x $objInfo(data,xPos)]
          }

       if {$objInfo(data,zPos) == ""} \
          {
           set qy [expr $iy1 - 4]
          } \
       else \
          {
           set qy [spec2pix -cell $id -roi $id -y $objInfo(data,zPos)]
          }

       $objInfo($obj,canvas) coords $idV $qx $iy1 $qx $iyn
       $objInfo($obj,canvas) coords $idH $ix1 $qy $ixn $qy
      }

   update
}

proc showThisPeak { obj tIDH tIDV pk } \
{
   global objInfo

   set iStr "Peak [gdbGet $pk INDEX]"
   set xStr "$objInfo(data,xAxis)=[gdbGet $pk $objInfo(peak,xName)]ppm"
   set yStr "$objInfo(data,yAxis)=[gdbGet $pk $objInfo(peak,xName)]ppm"
   set zStr "$objInfo(data,zAxis)=[gdbGet $pk $objInfo(peak,xName)]ppm"
   set hStr "Height=[gdbGet $pk HEIGHT]"

   wm title $objInfo($obj,window) "$iStr $xStr $yStr $zStr $hStr"

   $objInfo($obj,canvas) itemconfigure $tIDH -fill $objInfo(color,peakHilite)
   $objInfo($obj,canvas) raise         $tIDH

   $objInfo($obj,canvas) itemconfigure $tIDV -fill $objInfo(color,peakHilite)
   $objInfo($obj,canvas) raise         $tIDV

   update
}

proc unshowThisPeak { obj tIDH tIDV pk } \
{
   global objInfo

   wm title $objInfo($obj,window) $objInfo(prog,title)

   $objInfo($obj,canvas) itemconfigure $tIDH -fill ""
   $objInfo($obj,canvas) raise         $tIDH

   $objInfo($obj,canvas) itemconfigure $tIDV -fill ""
   $objInfo($obj,canvas) raise         $tIDV

   update
}

proc findCellByPix { obj sxPix syPix } \
{
   global objInfo

   set xPix [$objInfo($obj,canvas) canvasx $sxPix]
   set yPix [$objInfo($obj,canvas) canvasy $syPix]

   set first $objInfo($obj,firstCell)
   set last  $objInfo($obj,lastCell)

   if {!$first || !$last} {return 0}

   for {set id $first} {$id && ($id <= $last)} {incr id} \
      {
       set cInfo [getCellInfo -cell $id]
       set xInfo [lindex $cInfo 1]
       set yInfo [lindex $cInfo 2]

       set ix1   [lindex $xInfo 0]
       set ixn   [lindex $xInfo 1]
       set iy1   [lindex $yInfo 0]
       set iyn   [lindex $yInfo 1]

       if {$xPix < $ix1 || $xPix > $ixn} continue
       if {$yPix < $iy1 || $yPix > $iyn} continue

       return $id
      }

   return 0
}

proc addHShiftPix { obj sxPix syPix axis } \
{
   global objInfo

   set xPix [$objInfo($obj,canvas) canvasx $sxPix]
   set yPix [$objInfo($obj,canvas) canvasy $syPix]
   set id   [findCellByPix $obj $sxPix $syPix]

   if {!$id} return

   wm title . "Working ..."
   update

   showVerb "Residue $objInfo(ass,resID)"

   set xPPM [pix2spec -cell $id -roi $id -x $xPix -units ppm]
   set yPPM [pix2spec -cell $id -roi $id -y $yPix -units ppm]

   if {"$axis" == "X_AXIS"} \
      {
       set shift $xPPM
      } \
   else \
      {
       set shift $yPPM
      }

   showVerb "Added by loc: $axis H? ${shift}ppm ($xPPM $yPPM) in ROI $id"

   set entry [gdbCreate entry -parent $objInfo(ass,tab)]
    
   gdbSet $entry RESID    $objInfo(ass,resID)
   gdbSet $entry RESNAME  $objInfo(ass,resName)
   gdbSet $entry SHIFT    $shift
   gdbSet $entry ATOMNAME "H?"

   findRes
   readRes
   showRes
   showPeaks

   wm title . $objInfo(prog,title)
   update
}

proc delHShiftPix { obj sxPix syPix axis } \
{
   global objInfo

   set xPix [$objInfo($obj,canvas) canvasx $sxPix]
   set yPix [$objInfo($obj,canvas) canvasy $syPix]
   set id   [findCellByPix $obj $sxPix $syPix]

   if {!$id} return

   wm title . "Working ..."
   update

   showVerb "Residue $objInfo(ass,resID)"

   set xPPM [pix2spec -cell $id -roi $id -x $xPix -units ppm]
   set yPPM [pix2spec -cell $id -roi $id -y $yPix -units ppm]

   if {"$axis" == "X_AXIS"} \
      {
       set shift $xPPM
      } \
   else \
      {
       set shift $yPPM
      }

   showVerb "Deleting ${shift}ppm in ROI $id"

   set bestDist  1000000.0
   set bestEntry 0
   
   foreach entry $objInfo(ass,hList) \
      {
       set s    [gdbGet $entry SHIFT]
       set dist [expr abs($s - $shift)]

       if {$dist < $bestDist} \
          {
           set bestEntry $entry
           set bestDist  $dist 
          }
      }

   if {$bestEntry} \
      { 
       showVerb "Destroying [gdbGetInfo $bestEntry -parms values]"
       gdbDestroy $bestEntry
      }

   findRes
   readRes
   showRes
   showPeaks

   wm title . $objInfo(prog,title)
   update
}

proc addCShiftPix { obj sxPix syPix axis } \
{
   global objInfo

   set xPix [$objInfo($obj,canvas) canvasx $sxPix]
   set yPix [$objInfo($obj,canvas) canvasy $syPix]
   set id   [findCellByPix $obj $sxPix $syPix]

   if {!$id} return

   wm title . "Working ..."
   update

   showVerb "Residue $objInfo(ass,resID)"

   set xPPM  [pix2spec -cell $id -roi $id -x $xPix -units ppm]
   set yPPM  [pix2spec -cell $id -roi $id -y $yPix -units ppm]

   if {"$axis" == "X_AXIS"} \
      {
       set shift $xPPM
      } \
   else \
      {
       set shift $yPPM
      }

   showVerb "Added by loc: $axis C? ${shift}ppm in ROI $id"

   set entry [gdbCreate entry -parent $objInfo(ass,tab)]

   gdbSet $entry RESID    $objInfo(ass,resID)
   gdbSet $entry RESNAME  $objInfo(ass,resName)
   gdbSet $entry SHIFT    $shift
   gdbSet $entry ATOMNAME "C?"

   findRes
   readRes
   showRes
   showPeaks

   wm title . $objInfo(prog,title)
   update
}

proc delCShiftPix { obj sxPix syPix axis } \
{
   global objInfo

   set xPix [$objInfo($obj,canvas) canvasx $sxPix]
   set yPix [$objInfo($obj,canvas) canvasy $syPix]
   set id   [findCellByPix $obj $sxPix $syPix]

   if {!$id} return

   wm title . "Working ..."
   update

   showVerb "Residue $objInfo(ass,resID)"

   set xPPM  [pix2spec -cell $id -roi $id -x $xPix -units ppm]
   set yPPM  [pix2spec -cell $id -roi $id -y $yPix -units ppm]

   if {"$axis" == "X_AXIS"} \
      {
       set shift $xPPM
      } \
   else \
      {
       set shift $yPPM
      }

   showVerb "Deleting $axis ${shift}ppm in ROI $id"

   set bestDist  1000000.0
   set bestEntry 0
   
   foreach entry $objInfo(ass,cList) \
      {
       set s    [gdbGet $entry SHIFT]
       set dist [expr abs($s - $shift)]

       if {$dist < $bestDist} \
          {
           set bestEntry $entry
           set bestDist  $dist 
          }
      }

   if {$bestEntry} \
      { 
       showVerb "Destroying [gdbGetInfo $bestEntry -parms values]"
       gdbDestroy $bestEntry
      }

   findRes
   readRes
   showRes
   showPeaks

   wm title . $objInfo(prog,title)
   update
}

proc addHShiftPk { pk axis } \
{
   global objInfo

   if {![llength $objInfo(ass,assList)]} return

   wm title . "Working ..."
   update

   showVerb "Residue $objInfo(ass,resID)"

   switch $axis \
      {
       X_AXIS {set shift [gdbGet $pk $objInfo(peak,xName)]}
       Y_AXIS {set shift [gdbGet $pk $objInfo(peak,yName)]}
       Z_AXIS {set shift [gdbGet $pk $objInfo(peak,zName)]}
      }

   showVerb "Added by peak: $axis H? ${shift}ppm."

   set entry [gdbCreate entry -parent $objInfo(ass,tab)]

   gdbSet $entry RESID    $objInfo(ass,resID)
   gdbSet $entry RESNAME  $objInfo(ass,resName)
   gdbSet $entry SHIFT    $shift
   gdbSet $entry ATOMNAME "H?"

   findRes
   readRes
   showRes
   showPeaks

   wm title . $objInfo(prog,title)
   update
}

proc addCShiftPk { pk axis } \
{
   global objInfo

   if {![llength $objInfo(ass,assList)]} return

   wm title . "Working ..."
   update

   showVerb "Residue $objInfo(ass,resID)"

   switch $axis \
      {
       X_AXIS {set shift [gdbGet $pk $objInfo(peak,xName)]}
       Y_AXIS {set shift [gdbGet $pk $objInfo(peak,yName)]}
       Z_AXIS {set shift [gdbGet $pk $objInfo(peak,zName)]}
      }

   showVerb "Added by peak: $axis C? ${shift}ppm."

   set entry [gdbCreate entry -parent $objInfo(ass,tab)]

   gdbSet $entry RESID    $objInfo(ass,resID)
   gdbSet $entry RESNAME  $objInfo(ass,resName)
   gdbSet $entry SHIFT    $shift
   gdbSet $entry ATOMNAME "C?"

   findRes
   readRes
   showRes
   showPeaks

   wm title . $objInfo(prog,title)
   update
}

proc showPeaks {} \
{
   showPeaksHH
   showPeaksCH
}

proc showPeaksHH {} \
{
   global objInfo

   set obj winHH

   foreach pk $objInfo(ass,pkListHH) \
      {
       set xPPM [gdbGet $pk $objInfo(peak,xName)]ppm
       set yPPM [gdbGet $pk $objInfo(peak,yName)]ppm
       set id   [gdbGet $pk CLUSTID]
       set str  [gdbGet $pk INDEX]

       set tx0 [spec2pix -roi $id -cell $id -x $xPPM]
       set ty0 [spec2pix -roi $id -cell $id -y $yPPM]

       set tx1 [expr $tx0 - 5]
       set txn [expr $tx0 + 5]
       set ty1 [expr $ty0 + 6]
       set tyn [expr $ty0 - 6]

       set tIDH [$objInfo($obj,canvas) create line $tx1 $ty0 $txn $ty0 \
                  -width 2 -fill "" -tags peak]

       set tIDV [$objInfo($obj,canvas) create line $tx0 $ty1 $tx0 $tyn \
                  -width 2 -fill "" -tags peak]

       $objInfo($obj,canvas) bind $tIDH <Any-Enter> \
          "+ showThisPeak   $obj $tIDH $tIDV $pk"

       $objInfo($obj,canvas) bind $tIDH <Any-Leave> \
          "+ unshowThisPeak $obj $tIDH $tIDV $pk"

       $objInfo($obj,canvas) bind $tIDV <Any-Enter> \
          "+ showThisPeak   $obj $tIDH $tIDV $pk"

       $objInfo($obj,canvas) bind $tIDV <Any-Leave> \
          "+ unshowThisPeak $obj $tIDH $tIDV $pk"

       $objInfo($obj,canvas) bind $tIDH <ButtonRelease-1>       \
          "+ addHShiftPk $pk X_AXIS"

       $objInfo($obj,canvas) bind $tIDV <ButtonRelease-1>       \
          "+ addHShiftPk $pk X_AXIS"

       $objInfo($obj,canvas) bind $tIDH <Shift-ButtonRelease-1> \
          "+ addHShiftPk $pk Y_AXIS"

       $objInfo($obj,canvas) bind $tIDV <Shift-ButtonRelease-1> \
          "+ addHShiftPk $pk Y_AXIS"
      }

   bind $objInfo($obj,canvas) <ButtonRelease-2>       \
        "delHShiftPix $obj %x %y X_AXIS"

   bind $objInfo($obj,canvas) <ButtonRelease-3>       \
        "addHShiftPix $obj %x %y X_AXIS"

   bind $objInfo($obj,canvas) <Shift-ButtonRelease-2> \
        "delHShiftPix $obj %x %y Y_AXIS"

   bind $objInfo($obj,canvas) <Shift-ButtonRelease-3> \
        "addHShiftPix $obj %x %y Y_AXIS"
}

proc showPeaksCH {} \
{
   global objInfo

   set obj winCH

   foreach pk $objInfo(ass,pkListCH) \
      {
       set xPPM [gdbGet $pk $objInfo(peak,yName)]ppm
       set yPPM [gdbGet $pk $objInfo(peak,zName)]ppm
       set id   [gdbGet $pk CLUSTID]
       set str  [gdbGet $pk INDEX]

       set tx0 [spec2pix -roi $id -cell $id -x $xPPM]
       set ty0 [spec2pix -roi $id -cell $id -y $yPPM]

       set tx1 [expr $tx0 - 5]
       set txn [expr $tx0 + 5]
       set ty1 [expr $ty0 + 6]
       set tyn [expr $ty0 - 6]

       set tIDH [$objInfo($obj,canvas) create line $tx1 $ty0 $txn $ty0 \
                  -width 2 -fill "" -tags peak]

       set tIDV [$objInfo($obj,canvas) create line $tx0 $ty1 $tx0 $tyn \
                  -width 2 -fill "" -tags peak]

       $objInfo($obj,canvas) bind $tIDH <Any-Enter> \
          "+ showThisPeak   $obj $tIDH $tIDV $pk"

       $objInfo($obj,canvas) bind $tIDH <Any-Leave> \
          "+ unshowThisPeak $obj $tIDH $tIDV $pk"

       $objInfo($obj,canvas) bind $tIDV <Any-Enter> \
          "+ showThisPeak   $obj $tIDH $tIDV $pk"

       $objInfo($obj,canvas) bind $tIDV <Any-Leave> \
          "+ unshowThisPeak $obj $tIDH $tIDV $pk"

       $objInfo($obj,canvas) bind $tIDH <ButtonRelease-1>       \
          "+ addCShiftPk $pk Z_AXIS"

       $objInfo($obj,canvas) bind $tIDV <ButtonRelease-1>       \
          "+ addCShiftPk $pk Z_AXIS"

       $objInfo($obj,canvas) bind $tIDH <Shift-ButtonRelease-1> \
          "+ addHShiftPk $pk Y_AXIS"

       $objInfo($obj,canvas) bind $tIDV <Shift-ButtonRelease-1> \
          "+ addHShiftPk $pk Y_AXIS"
      }

   bind $objInfo($obj,canvas) <ButtonRelease-2>       \
        "delCShiftPix $obj %x %y Y_AXIS"

   bind $objInfo($obj,canvas) <ButtonRelease-3>       \
        "addCShiftPix $obj %x %y Y_AXIS"

   bind $objInfo($obj,canvas) <Shift-ButtonRelease-2> \
        "delHShiftPix $obj %x %y X_AXIS"

   bind $objInfo($obj,canvas) <Shift-ButtonRelease-3> \
        "addHShiftPix $obj %x %y X_AXIS"
}

proc showRes {} \
{
   global objInfo

   if {!$objInfo(ass,resID)} return

   clearRes

   showResHH
   showResCH
   showAssLabels
}

proc jumpView { entry } \
{
   global objInfo

   if {$entry == 0} \
      {
       foreach obj {winHH winCH} \
          {
           set sInfo [$objInfo($obj,scrollV) get]
           set y1    [lindex $sInfo 0]
           set yn    [lindex $sInfo 1]

           $objInfo($obj,canvas) yview moveto 0.0
           $objInfo($obj,scrollV) set 0.0 [expr $yn - $y1]
          }
      }

   set obj winHH
   set id  $objInfo($obj,firstCell)
   set n   0

   set sInfo [$objInfo($obj,canvas) yview]
   set y1    [lindex $sInfo 0]
   set yn    [lindex $sInfo 1]
   set dy    [expr $objInfo($obj,yAdj)*($yn - $y1)]

   foreach thisEntry $objInfo(ass,cList) \
      {
       if {"$entry" == "$thisEntry"} \
          {
           $objInfo($obj,canvas) yview moveto [expr double($n)*$dy] 
           break
          }

       incr id
       incr n
      }

   set obj winCH
   set id  $objInfo($obj,firstCell)
   set n   0

   set sInfo [$objInfo($obj,canvas) yview]
   set y1    [lindex $sInfo 0]
   set yn    [lindex $sInfo 1]
   set dy    [expr $objInfo($obj,yAdj)*($yn - $y1)]

   foreach thisEntry $objInfo(ass,hList) \
      {
       if {"$entry" == "$thisEntry"} \
          {
           $objInfo($obj,canvas) yview moveto [expr double($n)*$dy]
           break
          }

       incr id
       incr n
      }
}

proc showResHH {} \
{
   global objInfo

   set obj winHH

   if {!$objInfo($obj,active)} return

   set id $objInfo($obj,firstID)

   foreach cEntry $objInfo(ass,cList) \
      {
       set resName [gdbGet $cEntry RESNAME]
       set cName   [gdbGet $cEntry ATOMNAME]
       set cPPM    [gdbGet $cEntry SHIFT]ppm

       showVerb "Drawing plane at $cPPM"

       drawRec -x 0% 100% -y 0% 100% -cell $id -fg $objInfo(color,bg)

       foreach hEntry $objInfo(ass,hList) \
          {
           set hName [gdbGet $hEntry ATOMNAME]
           set hPPM  [gdbGet $hEntry SHIFT]ppm

           drawLine -cell $id -roi $id -x 0% 100% -y $hPPM $hPPM \
                    -fg $objInfo(color,grid)

           drawLine -cell $id -roi $id -y 0% 100% -x $hPPM $hPPM \
                    -fg $objInfo(color,grid)
          }

       drawROI -cell $id -roi $id -bg None -hi $objInfo(data,hi) \
               -axis Bottom Right

       incr id
      }
}

proc showResCH {} \
{
   global objInfo

   set obj winCH

   if {!$objInfo($obj,active)} return

   set id  $objInfo($obj,firstID)

   foreach hEntry $objInfo(ass,hList) \
      {
       set resName [gdbGet $hEntry RESNAME]
       set hName   [gdbGet $hEntry ATOMNAME]
       set hPPM    [gdbGet $hEntry SHIFT]ppm

       showVerb "Drawing plane at $hPPM"

       drawRec -x 0% 100% -y 0% 100% -cell $id -fg $objInfo(color,bg)

       foreach qEntry $objInfo(ass,hList) \
          {
           set qPPM  [gdbGet $qEntry SHIFT]ppm

           drawLine -cell $id -roi $id -x $qPPM $qPPM -y 0% 100% \
                    -fg $objInfo(color,grid)
          }

       foreach qEntry $objInfo(ass,cList) \
          {
           set qPPM  [gdbGet $qEntry SHIFT]ppm

           drawLine -cell $id -roi $id -y $qPPM $qPPM -x 0% 100% \
                    -fg $objInfo(color,grid)
          }

       drawROI -cell $id -roi $id -bg None -hi $objInfo(data,hi) \
               -axis Bottom Right

       incr id
      }
}

proc toggleAssLabels {} \
{
   global objInfo

   if {$objInfo(ass,labelFlag)} \
      {
       set objInfo(ass,labelFlag) 0
    
       $objInfo(winHH,canvas) delete ass
       $objInfo(winCH,canvas) delete ass
      } \
   else \
      {
       set objInfo(ass,labelFlag) 1

       showAssLabels
      }
}

proc showAssLabels {} \
{
   global objInfo stdfont

   set obj winHH
   set id  $objInfo($obj,firstID)

   $objInfo($obj,canvas) delete plane ass

   foreach cEntry $objInfo(ass,cList) \
      {
       set resName [gdbGet $cEntry RESNAME]
       set cName   [gdbGet $cEntry ATOMNAME]
       set cPPM    [gdbGet $cEntry SHIFT]ppm

       showVerb "Labeling plane at $cPPM"

       set cInfo [getCellInfo -cell $id]
       set xInfo [lindex $cInfo 1]
       set yInfo [lindex $cInfo 2]

       set ix1   [lindex $xInfo 0]
       set ixn   [lindex $xInfo 1]
       set iy1   [lindex $yInfo 0]
       set iyn   [lindex $yInfo 1]

       set ix0   [expr $ix1 + $objInfo($obj,cellX)]
       set iy0   [expr $iy1 + $objInfo($obj,cellY)]

       $objInfo($obj,canvas) create text $ix0 $iy0 \
          -fill $objInfo(color,label) -tags plane -anchor sw \
          -font $stdfont -text "$resName$objInfo(ass,resID) $cName $cPPM"

       if {$objInfo(ass,labelFlag)} \
          {
           foreach hEntry $objInfo(ass,hList) \
              {
               set hName [gdbGet $hEntry ATOMNAME]
               set hPPM  [gdbGet $hEntry SHIFT]ppm

               set ix0   [expr 4 + [spec2pix -cell $id -roi $id -x $hPPM]]
               set iy0   [expr $iyn - 4]

               $objInfo($obj,canvas) create text $ix0 $iy0 \
                  -fill $objInfo(color,label) -tags ass -anchor sw \
                  -font $stdfont -text "$hName"

               set ix0   [expr $ixn - 4]
               set iy0   [expr [spec2pix -cell $id -roi $id -y $hPPM] - 2]

               $objInfo($obj,canvas) create text $ix0 $iy0 \
                  -fill $objInfo(color,label) -tags ass -anchor se \
                  -font $stdfont -text "$hName"
              }
          }

       incr id
      }

   set obj winCH
   set id  $objInfo($obj,firstID)

   $objInfo($obj,canvas) delete plane ass

   foreach cEntry $objInfo(ass,hList) \
      {
       set resName [gdbGet $cEntry RESNAME]
       set hName   [gdbGet $cEntry ATOMNAME]
       set hPPM    [gdbGet $cEntry SHIFT]ppm

       showVerb "Labeling plane at $hPPM"

       set cInfo [getCellInfo -cell $id]
       set xInfo [lindex $cInfo 1]
       set yInfo [lindex $cInfo 2]

       set ix1   [lindex $xInfo 0]
       set ixn   [lindex $xInfo 1]
       set iy1   [lindex $yInfo 0]
       set iyn   [lindex $yInfo 1]

       set ix0   [expr $ix1 + $objInfo($obj,cellX)]
       set iy0   [expr $iy1 + $objInfo($obj,cellY)]

       $objInfo($obj,canvas) create text $ix0 $iy0 \
          -fill $objInfo(color,label) -tags plane -anchor sw \
          -font $stdfont -text "$resName$objInfo(ass,resID) $hName $hPPM"

       if {$objInfo(ass,labelFlag)} \
          {
           foreach hEntry $objInfo(ass,hList) \
              {
               set hName [gdbGet $hEntry ATOMNAME]
               set hPPM  [gdbGet $hEntry SHIFT]ppm

               set ix0   [expr 4 + [spec2pix -cell $id -roi $id -x $hPPM]]
               set iy0   [expr $iyn - 4]

               $objInfo($obj,canvas) create text $ix0 $iy0 \
                  -fill $objInfo(color,label) -tags ass -anchor sw \
                  -font $stdfont -text "$hName"
              }

           foreach cEntry $objInfo(ass,cList) \
              {
               set cName [gdbGet $cEntry ATOMNAME]
               set cPPM  [gdbGet $cEntry SHIFT]ppm

               set ix0   [expr $ixn - 4]
               set iy0   [expr [spec2pix -cell $id -roi $id -y $cPPM] - 2]

               $objInfo($obj,canvas) create text $ix0 $iy0 \
                  -fill $objInfo(color,label) -tags ass -anchor se \
                  -font $stdfont -text "$cName"
              }
          }

       incr id
      }
}

initVars
initArgs

initTabs
initSpectralData

initHHWindow
initCHWindow
initRefWindow
initAssWindow

newRes

#
# Bottom.
