proc rdTalos4PDF {} \
{
   global objInfo pdf

   set obj talos4

   set pdf(classList) ""

   if {![strcasecmp $objInfo($obj,pdfName) None]} return

   if {![file exists $objInfo($obj,pdfName)]} \
      {
       puts stderr "Error Finding PDF File $objInfo($obj,pdfName)"
       exit 1
      }

   loadArray $objInfo($obj,pdfName) pdf
}

proc t4pSort { s1 s2 } \
{
   set v1 [lindex $s1 1]
   set v2 [lindex $s2 1]

   if {$v1 > $v2} {return -1}
   if {$v1 < $v2} {return  1}

   return 0
}

proc initTalos4ResProb {} \
{
   global objInfo resP resNP rpInfo

   set obj talos4
   set id  target

   if {![strcasecmp $objInfo($obj,turnName) None]} return

#
# Get turn type data:

   if {![file exists $objInfo($obj,turnName)]} \
      {
       puts stderr \
        "Error Finding Turn Probability Table $objInfo($obj,turnName)"

       exit 1
      }

   set tTab  [gdbRead table -in $objInfo($obj,turnName) -verb]
   set eList [gdbGetInfo $tTab -parms entries]

   set seqList $objInfo($obj,$id,seqList)
   set r1 $objInfo($obj,$id,r1)
   set rN $objInfo($obj,$id,rN)

   for {set res $r1} {$res <= $rN} {incr res} \
      {
       foreach class $objInfo($obj,classes) \
          {
           set rpInfo($res,$class,rank) 9999
           set rpInfo($res,$class,p)    0.0
           set turnSize($class) 0
          }
      }

   set turnList ""
   set resList  ""

   foreach entry $eList \
      {
       set turn    [gdbGet $entry TURN_CLASS]
       set resName [gdbGet $entry RESNAME]
       set n       [gdbGet $entry TURN_SIZE]

       if {[lsearch -exact $turnList $turn] < 0} \
          {
           lappend turnList $turn
           set turnSize($turn) $n
          }

       if {[lsearch -exact $resList $resName] < 0} \
          {
           lappend resList $resName
          }
      }

#
# Establish default (null) probabilities:

   foreach turn $turnList \
      {
       foreach resName $resList \
          {
           for {set i 0} {$i < $objInfo($obj,maxSegLength)} {incr i} \
              {
               set resP($resName,$turn,$i)  0.0
               set resNP($resName,$turn,$i) 0.0 
              }
          }

       foreach resName $seqList \
          {
           for {set i 0} {$i < $objInfo($obj,maxSegLength)} {incr i} \
              {
               set resP($resName,$turn,$i)  0.0
               set resNP($resName,$turn,$i) 0.0 
              }
          }
      }

#
# Extract actual probability data:

   foreach entry $eList \
      {
       set turn    [gdbGet $entry TURN_CLASS]
       set resName [gdbGet $entry RESNAME]

       for {set i 0} {$i < $objInfo($obj,maxSegLength)} {incr i} \
          {
           set resP($resName,$turn,$i)  [expr 0.01*[gdbGet $entry P_I$i]]
           set resNP($resName,$turn,$i) [expr 0.01*[gdbGet $entry N_I$i]]
          }
      }

#
# Rank results:

   for {set res $r1} {$res <= $rN} {incr res} \
      {
       set sList ""

       foreach turn $objInfo($obj,classes) \
          {
           set a1 $res
           set aN [expr $a1 + $turnSize($turn) - 1]   

           if {$aN > $rN} continue

           set seq ""

           for {set a0 $a1} {$a0 <= $aN} {incr a0} \
              {
               append seq [lindex $seqList [expr $a0 - $r1]] 
              }

           lappend sList [getTurnScore $seq $turn $turnSize($turn)]
          }

       set rpInfo($res) [lsort -command t4pSort $sList]
      }

   for {set res $r1} {$res <= $rN} {incr res} \
      {
       set resName [lindex $seqList [expr $res - $r1]]
       set rank 1

       foreach score $rpInfo($res) \
          {
           set class [lindex $score 0]
           set p     [lindex $score 1]
           set n     [lindex $score 2]

           if {[lsearch -exact $objInfo($obj,classes) $class] < 0} continue

           set rpInfo($res,$class,rank) $rank
           set rpInfo($res,$class,p)    [expr 100.0*$p]

           incr rank
          }
      }

   gdbFree $tTab
}

proc getTurnScore { seq turn n } \
{
   global resP resNP

   set pVal 1.0
   set nVal 1

   for {set i 0} {$i < $n} {incr i} \
      {
       set resName [string index $seq $i]

       set pVal [expr $pVal*$resP($resName,$turn,$i)]
       set nVal [expr $nVal*$resNP($resName,$turn,$i)]
      }

   set pVal [expr pow( $pVal, 1.0/$n )]
   set nVal [expr pow( $nVal, 1.0/$n )]

   return "$turn $pVal $nVal"
}

proc rdTalos4CSTab { tabName id } \
{
   global objInfo

   set obj talos4

   if {![strcasecmp $tabName None]} return

   if {![strcmp $id target]} \
      {
       puts stderr "Talos4 Target: $tabName"

       if {![file exists $tabName]} \
          {
           puts stderr "Error Finding Target CS Table $tabName"
           exit 1
          }
      }

   if {![file exists $tabName]} \
      {
       puts stderr "Error Finding Database CS Table $tabName"
       return 
      }

   set t       [gdbRead table -in $tabName]
   set eList   [gdbGetInfo $t -parms entries]
   set seqList [getSeqList $t r1 rN]

   set objInfo($obj,$id,seqList) $seqList
   set objInfo($obj,$id,r1) $r1
   set objInfo($obj,$id,rN) $rN

   if {![strcmp $id target]} \
      {
       if {![strcasecmp $objInfo($obj,firstRes) First]} \
          {
           set objInfo($obj,firstRes) $r1
          }

       if {![strcasecmp $objInfo($obj,lastRes) Last]} \
          {
           set objInfo($obj,lastRes)  $rN
          }

       rClip objInfo($obj,firstRes) $r1 $rN
       rClip objInfo($obj,lastRes)  $r1 $rN

       rOrder objInfo($obj,firstRes) objInfo($obj,lastRes)

       set a1 [expr $objInfo($obj,firstRes) - $objInfo($obj,maxSegLength)]
       set aN [expr $objInfo($obj,lastRes)  + $objInfo($obj,maxSegLength)]
      } \
   else \
      {
       set a1 $r1
       set aN $rN
      }

   for {set res $a1} {$res <= $aN} {incr res} \
      {
       if {$res >= $r1 && $res <= $rN} \
          {
           set objInfo($obj,$id,$res,resName) [lindex $seqList [expr $res-$r1]]
          } \
       else \
          {
           set objInfo($obj,$id,$res,resName) "?" 
          }

       set objInfo($obj,$id,$res,ssInfo) None
       set objInfo($obj,$id,$res,phi)    9999.0
       set objInfo($obj,$id,$res,psi)    9999.0
       set objInfo($obj,$id,$res,rcP)    "" 

       foreach atom {HA HA_DIFF CA CB C N} \
          {
           set objInfo($obj,$id,$res,$atom) ""
          }
      }

   set uCount 0
   set gList  ""

   foreach entry $eList \
      {
       set atom    [gdbGet $entry ATOMNAME]
       set shift   [gdbGet $entry SHIFT]
       set res     [gdbGet $entry RESID]
       set resName [gdbGet $entry RESID]

       if {$res < $a1 || $res > $aN || $shift > 999.0} continue

       if {![strcasecmp $resName G] || ![strcasecmp $resName GLY]} \
          {
           lappend gList $resID 
          }

       switch -exact -- $atom \
          {
           HA      {lappend objInfo($obj,$id,$res,HA)    $shift}
           HA1     {lappend objInfo($obj,$id,$res,HA)    $shift}
           HA2     {lappend objInfo($obj,$id,$res,HA)    $shift}
           HA3     {lappend objInfo($obj,$id,$res,HA)    $shift}
           HA#     {lappend objInfo($obj,$id,$res,HA)    $shift}
           CA      {lappend objInfo($obj,$id,$res,$atom) $shift}
           CB      {lappend objInfo($obj,$id,$res,$atom) $shift}
           C       {lappend objInfo($obj,$id,$res,$atom) $shift}
           N       {lappend objInfo($obj,$id,$res,$atom) $shift}
           default {incr uCount}
          }
      }

   for {set res $a1} {$res <= $aN} {incr res} \
      {
       if {[lsearch -exact $gList $res] >= 0} \
          {
           if {[length objInfo($obj,$id,$res,HA)] == 2} \
              {
               set s1 [lindex $objInfo($obj,$id,$res,HA) 0]
               set s2 [lindex $objInfo($obj,$id,$res,HA) 1]

               set objInfo($obj,$id,$res,HA_DIFF) [expr abs($s1 - $s2)]
              }
          }

       foreach atom {HA CA CB C N} \
          {
           if {[llength $objInfo($obj,$id,$res,$atom)]} \
              {
               set shift [getAvg $objInfo($obj,$id,$res,$atom)]
               set rc    [getTalos4RC $objInfo($obj,$id,$res,resName) $atom]
               
               if {$rc < 999.0} \
                  {
                   set d [expr $shift - $rc]
                   
                   lappend \
                    objInfo($obj,$id,$res,rcP) [expr $d/$objInfo($obj,w,$atom)]

                   set objInfo($obj,$id,$res,$atom) $d
                  } \
               else \
                  {
                   set objInfo($obj,$id,$res,$atom) 99999.0 
                  }
              } \
           else \
              {
               set objInfo($obj,$id,$res,$atom) 99999.0
              } 
          }

       set n [llength $objInfo($obj,$id,$res,rcP)]

       if {$n} \
          {
           set r [getRMS $objInfo($obj,$id,$res,rcP)]

           set objInfo($obj,$id,$res,rcP) $r
          } \
       else \
          {
           set objInfo($obj,$id,$res,rcP) 0.0
          }
      }

   gdbFree $t
}

proc rdTalos4RC {} \
{
   global objInfo

   set obj talos4

   set t     [gdbRead table -in $objInfo($obj,surfDir)/$objInfo($obj,rcName)]
   set eList [gdbGetInfo $t -parms entries]

   foreach entry $eList \
      {
       set resName  [gdbGet $entry RESNAME]
       set atom     [gdbGet $entry ATOMNAME]
       set shift    [gdbGet $entry SHIFT]
       set type     [gdbGet $entry TYPE]
       set offset   [gdbGet $entry OFFSET]

       if {[strcasecmp $type RC]} continue
       if {$offset} continue

       set objInfo($obj,rc,$resName,$atom) $shift
      }

   foreach entry $eList \
      {
       set resName  [gdbGet $entry RESNAME]
       set atom     [gdbGet $entry ATOMNAME]
       set shift    [gdbGet $entry SHIFT]
       set type     [gdbGet $entry TYPE]
       set offset   [gdbGet $entry OFFSET]

       if {[strcasecmp $type ADJ]} continue
       if {$offset} continue

       set objInfo($obj,rc,$resName,$atom) \
        [expr $shift + $objInfo($obj,rc,$resName,$atom)]
      }

   gdbFree $t
}

proc getTalos4RC { resName atom } \
{
   global objInfo

   set obj talos4

   if {[info exists objInfo($obj,rc,$resName,$atom)]} \
      {
       return $objInfo($obj,rc,$resName,$atom)
      }

   return 99999.0
}

proc adjT4ClassName { class } \
{
   set n [expr [string length $class] - 2]

   if {$n < 0} {return $class}

   if {[string first "@1" $class] == $n} \
      {
       set class [string range $class 0 [expr $n - 1]]
      }

   return $class
}

proc rdTalos4SSTab { tabName id } \
{
   global objInfo

   set obj talos4

   set t     [gdbRead table -in $tabName]
   set eList [gdbGetInfo $t -parms entries]

   foreach entry $eList \
      {
       set res    [gdbGet $entry RESID]
       set dssp   [gdbGet $entry DSSP_CLASS]
       set angle  [gdbGet $entry ANGLE_CLASS]
       set class  [gdbGet $entry CLASS]
       set phi    [gdbGet $entry PHI]
       set psi    [gdbGet $entry PSI]
       set hb     [gdbGet $entry HBOND]

       set class [adjT4ClassName $class]

       if {[lsearch -exact $objInfo($obj,classList) $class] < 0} \
          {
           lappend objInfo($obj,classList) $class
          }

       set ssInfo $dssp\($phi,$psi\):$angle,$class|

       if {[string first CCap $class] < 0} \
          { 
           set objInfo($obj,$id,$res,ssInfo)   $ssInfo
           set objInfo($obj,$id,$res,class)    $class
          } \
       else \
          {
           set res3 [expr $res - 3]
           set objInfo($obj,$id,$res3,ssInfo)   $ssInfo
           set objInfo($obj,$id,$res3,class)    $class
          }

       set objInfo($obj,$id,$res,subClass) $hb
       set objInfo($obj,$id,$res,phi)      $phi
       set objInfo($obj,$id,$res,psi)      $psi
      }

   gdbFree $t
}

