
#
# Initialize table of random coil reference shifts.

proc initRandCoil {} \
{
   global rcArray adjArray prevArray nextArray
   global rcName adjName prevName nextName
   global rcTab adjTab prevTab nextTab

   foreach i {HN N HA C CA CB} \
      {
       set resName "?"

       set rcArray($resName,$i)   0.0 
       set adjArray($resName,$i)  0.0
       set prevArray($resName,$i) 0.0
       set nextArray($resName,$i) 0.0
      }

   puts "Reading Random Coil Shifts from $rcName"

   set rcTab [gdbRead table -in $rcName -name "Random Coil Shifts"]
   set entry [gdbFirst $rcTab]

   while {$entry} \
      {
       set resName [gdbGet $entry RESNAME]

       foreach i {HN N HA C CA CB} \
          {
           set d [gdbGet $entry $i]

           set rcArray($resName,$i)   $d
           set adjArray($resName,$i)  0.0
           set prevArray($resName,$i) 0.0
           set nextArray($resName,$i) 0.0
          }

       set entry [gdbNext $entry]
      }

   if {"$adjName" != "None"} \
      {
       puts "Reading RC Adjustments from $adjName"

       set adjTab [gdbRead table -in $adjName -name "RC Adjustments"]
       set entry  [gdbFirst $adjTab]

       while {$entry} \
          {
           set resName [gdbGet $entry RESNAME]
    
           foreach i {HN N HA C CA CB} \
              {
               set d [gdbGet $entry $i]
    
               set adjArray($resName,$i) $d
              }

           set entry [gdbNext $entry]
          }
      }

   if {"$prevName" != "None"} \
      {
       puts "Reading Previous Residue RC Adjustments from $prevName"

       set prevTab [gdbRead table -in $prevName -name "Prev RC Adjustments"]
       set entry   [gdbFirst $prevTab]

       while {$entry} \
          {
           set resName [gdbGet $entry RESNAME]

           foreach i {HN N HA C CA CB} \
              {
               set d [gdbGet $entry $i]

               set prevArray($resName,$i) $d
              }

           set entry [gdbNext $entry]
          }
      }

   if {"$nextName" != "None"} \
      {
       puts "Reading Next Residue RC Adjustments from $nextName"

       set nextTab [gdbRead table -in $nextName -name "Next RC Adjustments"]
       set entry   [gdbFirst $nextTab]

       while {$entry} \
          {
           set resName [gdbGet $entry RESNAME]

           foreach i {HN N HA C CA CB} \
              {
               set d [gdbGet $entry $i]

               set nextArray($resName,$i) $d
              }

           set entry [gdbNext $entry]
          }
      }
}

#
# Initialize table of secondary shift clip limits

proc initSSLimits {} \
{
   global clipFlag clipArray clipName clipTab

   set resList {A R N D C c Q E G H h I L K M F P S T W Y V ?}
   set clipTab 0

   foreach resName $resList \
      {
       foreach i {HN N HA C CA CB} \
          {
           set clipArray($resName,$i,min) -999999.0 
           set clipArray($resName,$i,max)  999999.0 
          }
      }

   if {!$clipFlag} return

   puts "Reading Secondary Shift Limits from $clipName"

   set clipTab [gdbRead table -in $clipName -name "Secondary Shift Limits"]
   set entry   [gdbFirst $clipTab]

   while {$entry} \
      {
       set resName [gdbGet $entry RESNAME]

       foreach i {HN N HA C CA CB} \
          {
           set minVal [gdbGet $entry ${i}_MIN]
           set maxVal [gdbGet $entry ${i}_MAX]

           set clipArray($resName,$i,min) $minVal
           set clipArray($resName,$i,max) $maxVal
          }

       set entry [gdbNext $entry]
      }
}

#
# Initialize table of residue homology values.

proc initHomology {} \
{
   global homoArray homoName homoTab

   puts "Reading Homology Values from $homoName"

   set homoTab [gdbRead table -in $homoName -name "Residue Homology"]
   set resList {A R N D C c Q E G H h I L K M F P S T W Y V}

   set entry   [gdbFirst $homoTab]
   set maxVal  0.0
   set c       2.0

   while {$entry} \
      {
       foreach j $resList \
          {
           set val [gdbGet $entry $j]

           if {$val > $maxVal} {set maxVal $val}
          }

       set entry [gdbNext $entry]
      }

   set entry   [gdbFirst $homoTab]

   while {$entry} \
      {
       set i [gdbGet $entry RESNAME]
   
       foreach j $resList \
          {
           set val [gdbGet $entry $j]
#           set homoArray($i,$j) [expr $c - $val*$c/$maxVal]
           set homoArray($i,$j) $val
          }

       set entry [gdbNext $entry]
      }
}

#   
# Rename HA2/HA3 in a chemical shift table.
    
proc adjGLYHA { csTab csVar } \
{   
   set    c "!strcasecmp( RESNAME, 'G' ) || !strcasecmp( RESNAME, 'GLY' )"
   append c " && (!strcasecmp( ATOMNAME, 'HA2' )"
   append c " ||  !strcasecmp( ATOMNAME, 'HA3' ))"
   append c " && $csVar < 999"
    
   set eList   [gdbSelect -from $csTab -cond $c]
   set resList ""
    
   foreach e $eList \
      { 
       set resID    [gdbGet $e RESID]
       set atomName [gdbGet $e ATOMNAME]
    
       if {[lsearch -exact $resList $resID] < 0} {lappend resList $resID}
    
       set haInfo($resID,$atomName) $e
      } 
    
   foreach resID $resList \
      {
       if {![info exists haInfo($resID,HA2)]} continue
       if {![info exists haInfo($resID,HA3)]} continue

       set shift2 [gdbGet $haInfo($resID,HA2) $csVar]
       set shift3 [gdbGet $haInfo($resID,HA3) $csVar]

       if {$shift2 >= $shift3} continue

       gdbSet $haInfo($resID,HA2) ATOMNAME HA3
       gdbSet $haInfo($resID,HA3) ATOMNAME HA2
      }
}

