#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Display Statistics of TALOS Predictions:"
    puts " -sum    sumName   \[pred.tab\]   Prediction summary input/output."
    puts " -ref    refName   \[ref.pdb\]    PDB Reference."
    puts " -verb                          Verbose Mode ON (Default)."
    puts " -noverb                        Verbose Mode OFF."
    puts " -hdr                           Display Table Header." 
    exit 1
   }

set sumName pred.tab
set refName ref.pdb
set verbose 1

getArgD $ARGV -sum sumName
getArgD $ARGV -ref refName

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -hdr]} \
   {
    puts -nonewline "VARS   DPHI_PRED DPHI_PRED_N DPSI_PRED DPSI_PRED_N"
    puts -nonewline " DPHI_GOOD DPHI_GOOD_N DPSI_GOOD DPSI_GOOD_N"
    puts -nonewline " PHI_NSIGMA1 PHI_NSIGMA2 PHI_NSIGMA3"
    puts -nonewline " PSI_NSIGMA1 PSI_NSIGMA2 PSI_NSIGMA3"
    puts -nonewline " GOOD BAD WARN TOTAL SOURCE"
    puts ""
    puts -nonewline "FORMAT"
    puts -nonewline " %6.2f %3d %6.2f %3d %6.2f %3d %6.2f %3d"
    puts -nonewline " %3d %2d %2d %3d %2d %2d"
    puts -nonewline " %3d %2d %3d %3d %s"
    puts ""
    puts ""
   }

if {![strcasecmp $sumName None]} {exit 0}

if {![file exists $sumName]} \
   {
    puts stderr "Error Finding TALOS Prediction Input Summary $sumName"
    exit 1
   }

if {![file exists $refName]} \
   {
    puts stderr "Error Finding TALOS PDB Reference Input $refName"
    exit 1
   }

dynInit -fast

set pdbSrc [dynRead -in $refName -pdb]
set sTab   [gdbRead table -in $sumName]
set eList  [gdbGetInfo $sTab -parms entries]

set ddPhiList  ""
set ddPsiList  ""
set dPhiList   ""
set dPsiList   ""
set aInfo      ""

set n1Phi 0
set n2Phi 0
set n3Phi 0

set n1Psi 0
set n2Psi 0
set n3Psi 0

set nB    0
set nG    0
set nW    0

foreach e $eList \
   {
    set class [gdbGet $e CLASS]

    if {![strcasecmp $class Good]} {incr nG}
    if {![strcasecmp $class Bad]}  {incr nB}
    if {![strcasecmp $class Warn]} {incr nW}

    if {[strcasecmp $class Good] && [strcasecmp $class Bad]} continue

    set resID  [gdbGet $e RESID]
    set aInfo  [dynGetPhiPsi -src $pdbSrc -res $resID]
    if {[llength $aInfo] < 2} continue

    set refPhi [lindex $aInfo 0]
    set phi    [gdbGet $e PHI]
    set dPhi   [gdbGet $e DPHI]

    set refPsi [lindex $aInfo 1]
    set psi    [gdbGet $e PSI]
    set dPsi   [gdbGet $e DPSI]

    if {$refPhi > 999 || $phi > 999} continue
    if {$refPsi > 999 || $psi > 999} continue

    set ddPhi [angleDiff $refPhi $phi]

    if {![strcasecmp $class Good]} {lappend ddPhiList $ddPhi}
    
    lappend dPhiList $dPhi

    if {$dPhi == 0.0} \
       {
        set sd 4.0 
       } \
    else \
       { 
        set sd [expr abs($ddPhi/$dPhi)]
       }

    if {$sd <= 1.0} \
       {
        incr n1Phi
       } \
    elseif {$sd <= 2.0} \
       {
        incr n2Phi
       } \
    elseif {$sd <= 3.0} \
       {
        incr n3Phi
       }

    set ddPsi [angleDiff $refPsi $psi]

    if {![strcasecmp $class Good]} {lappend ddPsiList $ddPsi}

    lappend dPsiList $dPsi

    if {$dPsi == 0.0} \
       {
        set sd 4.0
       } \
    else \
       {
        set sd [expr abs($ddPsi/$dPsi)]
       }

    if {$sd <= 1.0} \
       {
        incr n1Psi
       } \
    elseif {$sd <= 2.0} \
       {
        incr n2Psi
       } \
    elseif {$sd <= 3.0} \
       {
        incr n3Psi
       }
   }

if {$verbose} \
   {
    set fmt "%-19s %6.2f %3d"

    puts \
     [format $fmt "dPhi RMS Predicted:" [getRMS $dPhiList] [llength $dPhiList]]

    puts \
     [format $fmt "dPsi RMS Predicted:" [getRMS $dPsiList] [llength $dPsiList]]

    puts \
     [format $fmt "dPhi RMS Observed:" [getRMS $ddPhiList] [llength $ddPhiList]]

    puts \
     [format $fmt "dPsi RMS Observed:" [getRMS $ddPsiList] [llength $ddPsiList]]

    set fmt "Phi  1-Sigma: %3d 2-Sigma: %3d 3-Sigma: %3d"
    puts [format $fmt $n1Phi $n2Phi $n3Phi]

    set fmt "Psi  1-Sigma: %3d 2-Sigma: %3d 3-Sigma: %3d"
    puts [format $fmt $n1Psi $n2Psi $n3Psi]

    puts " [file root [file tail $refName]]"
   } \
else \
   {
    set fmt " %6.2f %3d"

    puts -nonewline [format $fmt [getRMS $dPhiList]  [llength $dPhiList]]
    puts -nonewline [format $fmt [getRMS $dPsiList]  [llength $dPsiList]]
    puts -nonewline [format $fmt [getRMS $ddPhiList] [llength $ddPhiList]]
    puts -nonewline [format $fmt [getRMS $ddPsiList] [llength $ddPsiList]]

    puts -nonewline [format " %3d %2d %2d" $n1Phi $n2Phi $n3Phi]
    puts -nonewline [format " %3d %2d %2d" $n1Psi $n2Psi $n3Psi]

    set nT [expr $nG + $nB + $nW]

    puts -nonewline [format " %3d %2d %3d %3d" $nG $nB $nW $nT]
    puts " [file root [file tail $refName]]"
   }

exit 0
