#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Convert a Titration Table for Use with Table-Based PCA."
    puts " -in    titrTab \[titrView.tab\]  Input Table."
    puts " -mode  mode    \[xy\]            Output Mode: x y xy combined."
    puts " -cx    cx      \[1.0\]           Scale Factor for X_PPM Values."
    puts " -cy    cy      \[1.0\]           Scale Factor for Y_PPM Values."
    puts " -sub                           Subtract Spectrum 1 PPM Values from All Spectra."  
    puts " -verb                          Verbose Mode ON (Default)."
    puts "Notes:"
    puts " Input table is created by: specView.tcl -export titrView ..."
    exit 0
   }

set inName  titrView.tab
set mode    xy
set cx      1.0
set cy      1.0

set subFlag 0
set verbose 1

getArgD $ARGV -in   inName
getArgD $ARGV -mode mode

getArgD $ARGV -cx cx
getArgD $ARGV -cy cy

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -sub]}    {set subFlag 1}
if {[flagLoc $ARGV -nosub]}  {set subFlag 0}

#
#---------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Titration Table $inName"
    exit 1
   }

set t [gdbRead table -in $inName]

set eList [gdbGetInfo $t -parms entries]
set vList [gdbGetInfo $t -parms varNames]

set pkCount   [llength $eList]
set varCount  [llength $vList] 
set specCount 0

for {set i 1} {$i < $varCount} {incr i} \
   {
    if {[lsearch -exact $vList X_PPM$i] < 0} break
    set specCount $i
   }

if {$verbose} {puts stderr "Spectra: $specCount Peaks: $pkCount"}

puts ""
puts "DATA SPECTRUM_COUNT $specCount"
puts "DATA PEAK_COUNT     $pkCount"
puts "DATA MODE           $mode"
puts "DATA CX             $cx"
puts "DATA CY             $cy"
puts ""

puts -nonewline "VARS   SPECTRUM"

for {set i 1} {$i <= $pkCount} {incr i} \
   {
    if {![strcasecmp $mode xy]} \
       {
        puts -nonewline " PEAK${i}_X PEAK${i}_Y"
       } \
    elseif {![strcasecmp $mode x]} \
       {
        puts -nonewline " PEAK${i}_X"
       } \
    elseif {![strcasecmp $mode y]} \
       {
        puts -nonewline " PEAK${i}_Y"
       } \
    else \
       {
        puts -nonewline " PEAK${i}_XY"
       }
   }

puts ""
puts -nonewline "FORMAT %4d     "

for {set i 1} {$i <= $pkCount} {incr i} \
   {
    if {![strcasecmp $mode xy]} \
       {
        puts -nonewline " %8.4f   %8.4f  "
       } \
    else \
       {
        puts -nonewline " %8.4f  "
       }
   }

puts ""
puts ""

for {set j 1} {$j <= $specCount} {incr j} \
   {
    puts -nonewline [format " %3d" $j]

    for {set i 1} {$i <= $pkCount} {incr i} \
       {
        set e [lindex $eList [expr $i - 1]]

        set xPPM [expr $cx*[gdbGet $e X_PPM$j]]
        set yPPM [expr $cy*[gdbGet $e Y_PPM$j]]

        if {$subFlag} \
           {
            set xPPM [expr $xPPM - $cx*[gdbGet $e X_PPM1]]
            set yPPM [expr $yPPM - $cy*[gdbGet $e Y_PPM1]]
           }

        if {![strcasecmp $mode xy]} \
           {
            puts -nonewline [format " %8.4f %8.4f" $xPPM $yPPM]
           } \
        elseif {![strcasecmp $mode x]} \
           {
            puts -nonewline [format " %8.4f" $xPPM]
           } \
        elseif {![strcasecmp $mode y]} \
           {
            puts -nonewline [format " %8.4f" $yPPM]
           } \
        else \
           {
            set xyPPM [expr sqrt($xPPM*$xPPM + $yPPM*$yPPM)]
            puts -nonewline [format " %8.4f" $xyPPM]
           }
       }

    puts ""
   }

exit 0

