#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
# 2D Titration series plots.

set specName  ft/test%05d.dat
set tabName   tab/test%05d.tab
set assName   ass.tab 

set tList     {30 31   32  33  34  35  36}
set pHList    {0  0.05 0.1 0.2 0.4 0.8 1.6}

set colorList {red orange yellow green cyan blue magenta}
set hi        2.6e+5
set sigma     1.7e+3

set xName     HN
set yName     N 

set assNameX  X_PPM 
set assNameY  Y_PPM 
set assNameA  INDEX

set dx        0.10
set dy        0.30

#
#------------------------------------------------------------------

#
# Find the peak entry nearest to the given X and Y position.

proc findPeak { xPPM yPPM tabPtr } \
{
    upvar $tabPtr pkTab

    set entryJ  [gdbFirst $pkTab]
    set minDist 1000000.0
    set s1      0.015
    set s2      0.225
    set entry   0

    while {$entryJ} \
       {
        set x    [gdbGet $entryJ X_PPM]
        set y    [gdbGet $entryJ Y_PPM]
        set dist [expr pow(($x-$xPPM)/$s1,2) + pow(($y-$yPPM)/$s2,2)]
        set dist [expr pow($dist,0.5)]

        if {$dist < $minDist} \
           {
            set minDist $dist
            set entry   $entryJ
           }

        set entryJ [gdbNext $entryJ]
       }

    return $entry
}

#
# Main:

gdbInit
gdbCreate dBase -name TITR
initMiniPlot

#
# Peak detection:

if {![file exists tab]} \
   {
    catch {file mkdir tab}
   }

set i 1

foreach tVal $tList \
   {
    if {![file exists test.$tVal.tab]} \
       {
        set thisName [format $specName $i]
        set thisTab  [format $tabName  $i]

        readROI -roi 1 -ndim 2 -in $thisName -x $xName -y $yName -verb

        pkFindROI -roi 1 \
          -sigma $sigma -pChi 0.0001 -plus $hi -minus -$hi \
          -dx        1     1 \
          -idx       1     1 \
          -tol    2.00  2.00 \
          -hiAdj  1.20  1.80 \
          -lw    15.00  0.00 \
          -mask -out $thisTab -verb
       }

    incr i
   }

#
# Read the tables:

set assTab [gdbRead table -in $assName -name ass]
set i 1

foreach tVal $tList \
   {
    set  thisTab  [format $tabName  $i]
    set  tab$tVal [gdbRead table -in $thisTab -name $tVal -verb]
    incr i
   }

#
# For each assignment:
#  For each spectrum:
#    Find the associated peak.


set entryI [gdbFirst $assTab]

while {$entryI} \
   {
    set thisHN  [gdbGet $entryI $assNameX]
    set thisN15 [gdbGet $entryI $assNameY]
    set ass     [gdbGet $entryI $assNameA]

    set hnPrev  $thisHN
    set n15Prev $thisN15
    set eList   ""
    set i       1

    nextMiniPlot

    puts -nonewline "$ass"

    foreach tVal $tList \
       {
        set thisName [format $specName $i]
        set entryJ   [findPeak $hnPrev $n15Prev tab$tVal]
        set hnPrev   [gdbGet $entryJ X_PPM]
        set n15Prev  [gdbGet $entryJ Y_PPM]
        set color    [lindex $colorList $i]

        puts -nonewline " ($hnPrev $n15Prev)"

        set  xPPM [format %sppm $thisHN]
        set  yPPM [format %sppm $thisN15]

        readROI -ndim 2 -in $thisName -roi $i -center X Y \
           -dx $xName $xPPM ${dx}ppm -dy $yName $yPPM ${dy}ppm

        if {$i == 1} \
           {
            drawROI -cell $cellCount -hi $hi -bg white \
               -color1 $color -color2 $color -roi $i
           } \
        else \
           {
            drawROI -cell $cellCount -hi $hi \
               -color1 $color -color2 $color -roi $i
           }

        drawText -cell $cellCount -x 50% -y 0% -yOff -0.2 -center \
           -text $ass $thisHN,$thisN15
 
        lappend eList $entryJ
        incr i
       }

    foreach entryJ $eList \
       {
        set hnPrev  [gdbGet $entryJ X_PPM]
        set n15Prev [gdbGet $entryJ Y_PPM]

        set x1  [expr $thisHN  - $dx]
        set xn  [expr $thisHN  + $dx]
        set y1  [expr $thisN15 - $dy]
        set yn  [expr $thisN15 + $dy]

        set qx1 [expr $hnPrev  - 0.004]
        set qxn [expr $hnPrev  + 0.004]
        set qy1 [expr $n15Prev - 0.02]
        set qyn [expr $n15Prev + 0.02]

        if {$qx1 < $x1} {set qx1 $x1}
        if {$qx1 > $xn} {set qx1 $xn}
        if {$qxn < $x1} {set qxn $x1}
        if {$qxn > $xn} {set qxn $xn}

        if {$qy1 < $y1} {set qy1 $y1}
        if {$qy1 > $yn} {set qy1 $yn}
        if {$qyn < $y1} {set qyn $y1}
        if {$qyn > $yn} {set qyn $yn}

        drawRec -x ${qxn}ppm ${qx1}ppm -y ${qyn}ppm ${qy1}ppm \
            -fg black -cell $cellCount -roiRef 1
       }

    puts ""

    set entryI [gdbNext $entryI]
   }

plotClose
exit
