#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

#
# Establish OS Type for the application.

proc setOSType {} \
{
   global osType pcFlag shExt exeExt env

   set osType other

   if {![info exists osType]} \
      {
#      set error    [catch {set osType [lindex "[exec uname -a]" 0]}]
#      if  {$error} {set osType Windows_NT}
      }

   if {![string compare Windows_NT $osType]} \
      {
       set pcFlag 1
       set exeExt .exe
       set shExt  .ksh
      } \
   else \
      {
       set pcFlag 0
       set exeExt "" 
       set shExt  .com
      }

   if {[info exists env(Path)] && ![info exists env(PATH)]} \
      {
       set env(PATH) $env(Path)
      }
}

#
# Establish colors for the application.

proc setColors {} \
{
   global goFGColor goBGColor slowFGColor slowBGColor stopFGColor stopBGColor
   global color1 color2 color3 color4 mainColor
   global osType pcFlag wishFlag

   setOSType

   if {$pcFlag} \
      {
       set color1      gray
       set color2      gray
       set color3      gray
       set color4      navy

       set mainColor   black
      } \
   else \
      {
       set color1      darkOrchid4
       set color2      darkOrchid3
       set color3      gray
       set color4      navy

       set mainColor   white 

       set color1      gray
       set color2      gray
       set color3      gray
       set color4      navy

       set mainColor   black
      }

   set goBGColor   green 
   set goFGColor   black 
   set slowBGColor yellow
   set slowFGColor black
   set stopFGColor red
   set stopBGColor black

   if {!$wishFlag} return

   . configure -background $color1

   if {$pcFlag} \
      {
       option add *Label.font  "-*-helvetica-*-b-*-*-*-100-*-*-*-*-*-*"
       option add *Button.font "-*-helvetica-*-b-*-*-*-100-*-*-*-*-*-*"
       option add *Entry.font  "-*-helvetica-*-r-*-*-*-100-*-*-*-*-*-*"
       option add *Text.font   "-*-courier-*-r-*-*-*-100-*-*-*-*-*-*"
      }

   option add *insertOffTime               0
   option add *HighlightThickness          0
   option add *Background                  $color1
   option add *Foreground                  $mainColor 
   option add *activeForeground            $mainColor 
   option add *selectBackground            $mainColor 
   option add *selectForeground            $color1
   option add *activeBackground            $color2
   option add *insertBackground            $slowBGColor
   option add *disabledForeground          $color3
   option add *disabledBackground          $color4

   option add *Scrollbar.background        $color3
   option add *Scrollbar.foreground        $color3
   option add *Scrollbar.activeForeground  $color3

   option add *Dialog.background        $mainColor 
   option add *Dialog.foreground        $color1
}

proc testFonts { modeFlag } \
{
   global wishFlag glbFontInitInfo

   if {!$wishFlag} \
      {
       set glbFontInitInfo "200 34 6 12"
       return $glbFontInitInfo
      }

   set w .initFontSize

   catch {destroy $w}

   if {![strcmp $modeFlag -done]} \
      {
       update
       return
      }

   set f "-adobe-helvetica-medium-r-*-*-*-100-*-*-*-*-*-*"
   set s "ABCDEFGHIJK abcdefghijk 0123456789"
   set n [string length $s]

   set hi   24
   set wide [expr 12*$n]
 
   toplevel $w 
   wm title $w "Initialization Window"

   canvas $w.c -width $wide -height $hi -background gray
   pack   $w.c 
   update

   set tID [$w.c create text 4 4 -fill gray -anchor nw -font $f -text $s]
   update

   set tInfo [$w.c bbox $tID]

   set x1 [lindex $tInfo 0]
   set y1 [lindex $tInfo 1]
   set xn [lindex $tInfo 2]
   set yn [lindex $tInfo 3]

   set dx [expr int(abs($xn - $x1))]
   set dy [expr int(abs($yn - $y1))]
   set dc [expr int(1.0 + double($dx)/$n)]

   set glbFontInitInfo "$dx $n $dc $dy"

   return $glbFontInitInfo 
}

proc dataSizeIsSafe { sizeList } \
{
   set bStr [getNMRBinType]

   if {[string first _64 $bStr] >= 0} {return 1}

   set f [getSafeDataSize $sizeList]

   if {$f > 0.95*2.0*1024.0*1024.0*1024.0} {return 0}

   return 1
}

proc getRootWindowAdj { } \
{
   global wishFlag env glbUbuntuVersion

   set xAdj 0
   set yAdj 0

   set macFlag 0

   getUbuntuVersion

   if {$glbUbuntuVersion >= 20} {return "68 0"}

   if {[info exists env(NMRBINTYPE)]} \
      {
       if {[string first "mac" [string tolower $env(NMRBINTYPE)]] >= 0} {set macFlag 1}
      }

   if {$macFlag} \
      {
       set dockSize   68
       set dockOrient bottom

       catch {set aInfo [exec defaults read com.apple.dock]}
       if {![info exists aInfo]} {set aInfo ""}

       set n [string length $aInfo]

       if {$n} \
          {
           catch {set dockSize [exec defaults read com.apple.dock tilesize]}

           if {![info exists dockSize]}        {set dockSize 68}
           if {![string is integer $dockSize]} {set dockSize 68}

           catch {set dockOrient [exec defaults read com.apple.dock orientation]}
           if {![info exists dockOrient]} {set dockOrient bottom}
          }

       if {![strcasecmp $key r]} \
          {
           incr dockSize 12
           set xAdj $dockSize
          } \
       elseif {![strcasecmp $key l]} \
          {
           incr dockSize 14
           set xAdj $dockSize
          } \
       elseif {![strcasecmp $key t]} \
          {
           incr dockSize 18
           set yAdj $dockSize
          } \
       else \
          {
           incr dockSize 18
           set yAdj $dockSize
          }
      } \
   else \
      {
       catch {set wInfo [exec xprop -root | fgrep _NET_WORKAREA(CARDINAL) | tr "," " "]} 

       if {![info exists wInfo]}  {return "0 0"}
       if {[llength $wInfo] < 6 } {return "0 0"}

       set ix0 [lindex $wInfo 2]
       set iy0 [lindex $wInfo 3]
       set ixw [lindex $wInfo 4]
       set iyw [lindex $wInfo 5]

       if {![string is integer $ix0] || ![string is integer $iy0]} {return "0 0"}
       if {![string is integer $ix0] || ![string is integer $iy0]} {return "0 0"}
    
       if {$wishFlag} \
          {
           set wide [winfo screenwidth  .]
           set hi   [winfo screenheight .]
          } \
       else \
          {
           catch {set hiInfo [exec xwininfo -root -shape | fgrep "Height:"]}
           if {![info exists hiInfo]} {return "0 0"}

           catch {set wideInfo [exec xwininfo -root -shape | fgrep "Width:"]}
           if {![info exists wideInfo]} {return "0 0"}

           set wide [lindex $wideInfo   1]
           if {![string is integer $wide]} {return "0 0"}

           set hi [lindex $hiInfo   1]
           if {![string is integer $hi]} {return "0 0"}
          }

       set xAdj [expr $wide - $ixw]
       set yAdj [expr $hi - $iyw]

       if {$xAdj < 0} {set xAdj 0}
       if {$yAdj < 0} {set yAdj 0}
      }

   return "$xAdj $yAdj"
}

proc getSafeDataSize { sizeList } \
{
   set f 4.0

   foreach size $sizeList \
      {
       if {$size > 0} {set f [expr $f*double($size)]}
      }
 
   if {$f > 0.95*2.0*1024.0*1024.0*1024.0} {return $f}

   set n 4

   foreach size $sizeList \
      {
       if {$size > 0} {set n [expr $n*$size]}
      }

   return $n
}

proc getNMRBinType {} \
{
   global env

    if {[info exists env(NMRBINTYPE)]} \
       {
        return $env(NMRBINTYPE)
       } \
    elseif {[info exists env(BINTYPE)]} \
       {
        return $env(BINTYPE)
       } \

   return Unknown
}

proc setBinType {} \
{
   global env glbBinType

   if {[info exists env(NMRBINTYPE)]} \
      {
       set glbBinType $env(NMRBINTYPE)
      } \
   elseif {[info exists env(BINTYPE)]} \
      {
       set glbBinType $env(BINTYPE)
      } \
   else \
      {
       set glbBinType other
      }

   return $glbBinType
}

proc setFonts {} \
{
   global smallfont bigfont stdfont boldfont fixedfont axisfont typingfont env
   global wishFlag glbFontInitInfo glbBinType glbUbuntuVersion

   getUbuntuVersion

   if {![info exists glbFontInitInfo]}  {set glbFontInitInfo "200 34 6 12"}

   set varFontInfo      "family=Lucida,size=9,weight=bold"
   set fixedFontInfo    "family=Fixed,size=9,weight=normal"

   set varFontWidgets   {Button Checkbutton Radiobutton Label Menu Menubutton Message Scale}
   set fixedFontWidgets {Entry Listbox Menuentry Text} 
   set resetFontFlag    0

   if {$glbUbuntuVersion >= 23} {set resetFontFlag 1} 

   if {[info exists env(NMR_TCL_VARFONT)]} \
      {
       if {[strcasecmp $env(NMR_TCL_VARFONT) "Default"]} \
         {
          set varFontInfo $env(NMR_TCL_VARFONT) 
         }
      }

  if {[info exists env(NMR_TCL_FIXEDFONT)]} \
      {
       if {[strcasecmp $env(NMR_TCL_FIXEDFONT) "Default"]} \
         {
          set fixedFontInfo $env(NMR_TCL_FIXEDFONT)
         }
      }

   if {[info exists env(NMR_TCL_SETFONTS)]} \
      { 
       if {[valIsYes $env(NMR_TCL_SETFONTS)]} \
          {
           set resetFontFlag 1
          } \
       elseif {[valIsNo $env(NMR_TCL_SETFONTS)]} \
          {
           set resetFontFlag 0
          }
      }

   if {$resetFontFlag} \
      {
       eval "font create ubvFont [expandArg $varFontInfo]"
       eval "font create ubfFont [expandArg $fixedFontInfo]"
       
       foreach ww $varFontWidgets   {option add *$ww.font ubvFont}
       foreach ww $fixedFontWidgets {option add *$ww.font ubfFont}
      }

   set dy [lindex $glbFontInitInfo end]

   if {$dy == 12} \
      {
       if {$glbUbuntuVersion >= 23} \
          {
           set smallfont  "-*-fixed-medium-r-*-*-*-120-*-*-*-*-*-*"
           set bigfont    "-*-fixed-medium-r-*-*-*-160-*-*-*-*-*-*"
           set stdfont    "-*-fixed-medium-r-*-*-*-120-*-*-*-*-*-*"
           set boldfont   "-*-fixed-medium-r-*-*-*-120-*-*-*-*-*-*"
           set fixedfont  "-*-fixed-medium-r-*-*-*-120-*-*-*-*-*-*"
           set typingfont "-*-fixed-medium-r-*-*-*-120-*-*-*-*-*-*"
           set axisfont   "-*-lucidatypewriter-bold-r-*-*-*-120-*-*-*-*-*-*"
          } \
       else \
          {
           set smallfont  "-adobe-helvetica-medium-r-*-*-*-100-*-*-*-*-*-*"
           set bigfont    "-adobe-helvetica-medium-r-*-*-*-160-*-*-*-*-*-*"
           set stdfont    "-adobe-helvetica-medium-r-*-*-*-100-*-*-*-*-*-*"
           set boldfont   "-adobe-helvetica-bold-r-*-*-*-120-*-*-*-*-*-*"
           set fixedfont  "-*-courier-medium-r-*-*-*-120-*-*-*-*-*-*"
           set typingfont "-*-courier-medium-r-*-*-*-100-*-*-*-*-*-*"
           set axisfont   "-*-lucidatypewriter-bold-r-*-*-*-100-*-*-*-*-*-*"
          } 
      } \
   else \
      {
       set smallfont  "-adobe-helvetica-medium-r-*-*-*-100-*-*-*-*-*-*"
       set bigfont    "-adobe-helvetica-medium-r-*-*-*-140-*-*-*-*-*-*"
       set stdfont    "-adobe-helvetica-medium-r-*-*-*-80-*-*-*-*-*-*"
       set boldfont   "-adobe-helvetica-bold-r-*-*-*-80-*-*-*-*-*-*"
       set fixedfont  "-*-courier-medium-r-*-*-*-100-*-*-*-*-*-*"
       set typingfont "-*-courier-medium-r-*-*-*-100-*-*-*-*-*-*"
       set axisfont   "-*-lucidatypewriter-bold-r-*-*-*-100-*-*-*-*-*-*"
      }

   foreach n {small big std bold fixed typing axis} \
      {
       set fName [string toupper NMRPIPE_${n}FONT]
       if {![info exists env($fName)]} continue

       set ${n}font $env($fName)
      }
}

proc showError args \
{
   global wishFlag

   if {[llength $args] < 2} \
      {
       set who error
       set msg $args
      } \
   else \
      {
       set who [lindex $args 0]
       set msg [lrange $args 1 end]
      }

   if {$wishFlag} \
      {
       tk_dialog .errDialog $who $msg error 0 Continue
      } \
   else \
      {
       puts stderr $msg
      }
}

proc getDirName { d } \
{
   if {[file isdirectory $d]} {return $d}
   return [file dirname $d]
}

proc okWrite args \
{
   global wishFlag

   if {!$wishFlag} {return 1}

   set fileName [lindex $args 0]

   if {![file exists $fileName]} \
      {
       return 1
      }


   if {[llength $args] == 1} \
      {
       set txt   "File \"$fileName\" exists."
       set title "Overwrite"

       if {![tk_dialog .ovDialog $title $txt question 1 Overwrite Cancel]} \
          {
           return 1
          }
      } \
   else \
      {
       set txt   "File \"$fileName\" exists. Overwrite?"
       set title "Overwrite"

       set status [tk_messageBox -type yesno -title $title -message $txt -icon question -parent [lindex $args 1]]
       if {![strcasecmp $status yes]} {return 1}
      }

   return 0
}

proc okSaveDiscard args \
{
   global wishFlag

   if {!$wishFlag} {return 1}

   set fileName [lindex $args 0]

   set txt   "Save Current Data in File $fileName?"
   set title "Save"

   if {[llength $args] == 1} \
      {
       if {![tk_dialog .sdDialog $title $txt question 0 Save Discard]} \
          {
           return 1
          }
      } \
   else \
      {
       set status [tk_messageBox -type yesno -title Save -message $txt -icon question -parent [lindex $args 1]]
       if {![strcasecmp $status yes]} {return 1}
      }

   return 0
}

proc okExit args \
{
   global wishFlag

   if {!$wishFlag} {return 1}

   set n [llength $args]

   set exitProc exit
   set exitMsg  "Exit now?"
   set n        [llength $args]

   if {$n > 0} \
      {
       set exitProc [lindex $args 0]
      }

   if {$n > 1} \
      {
       set exitMsg [lrange $args 1 end]
      }

   if {![tk_dialog .exDialog Exit $exitMsg question 1 Exit Cancel]} \
      {
       $exitProc
      }
}

#
# parmExists: test if array parameter exists or is empty.

proc parmExists { arrayPtr parm } \
{
   upvar $arrayPtr array

   if {![info exists array($parm)]} \
      {
       puts stderr "Error: missing parameter $parm from $arrayPtr"
       return 0
      }

   if {$array($parm) == ""} \
      {
       puts stderr "Error: blank parameter $parm in $arrayPtr"
       return 0
      }

   return 1
}

#
# getDate: returns date.

proc getDate {} \
{
   global pcFlag osType exeExt

   setOSType

   return [exec date$exeExt]
}

#
# getFileBytes: return the uncompressed size in bytes of the given file.
#               returns -1 on failure.

proc getFileBytes { fName {mode -exact} } \
{
   global osType pcFlag exeExt

   setOSType

   if {"$mode" == "-exact" } \
      {
       switch [file extension $fName] \
         {
          .Z      \
            { set cmnd "set ans \[exec cat$exeExt $fName | zcat$exeExt | wc$exeExt -c \]" }
          .gz     \
            { set cmnd "set ans \[exec cat$exeExt $fName | zcat$exeExt | wc$exeExt -c \]" }
          default \
            { set cmnd "set ans \[file size $fName\]"  }
         }

       set error [catch $cmnd]
      } \
   else \
      {
       set ans   2048
       set cmnd  "set ans \[file size $fName\]"
       set error [catch $cmnd]

       switch [file extension $fName] \
          {
           .Z      { set ans [expr 2*$ans] }
           .gz     { set ans [expr 2*$ans]  }
           default { set ans $ans }
          }
      }
 
   if {$error}              { return -1 }
   if {[llength $ans] < 1}  { return -1 }

   return [lindex $ans 0]
}

proc getWinAdj {} \
{
   global glbUbuntuVersion glbWinXAdj glbWinYAdj

   getUbuntuVersion

   if {$glbUbuntuVersion >= 20} \
      {
       set glbWinXAdj 100 
       set glbWinYAdj  20
      } \
   else \
      {
       set glbWinXAdj  5
       set glbWinYAdj 20
      }
}

proc getUbuntuVersion {} \
{
   global glbUbuntuVersion

   if {[info exists glbUbuntuVersion]} {return $glbUbuntuVersion}

   set glbUbuntuVersion 0

   if {![file exists /etc/os-release]} {return 0}

   catch {set uInfo [exec grep -F -i ubuntu /etc/os-release | grep NAME]}
   if {![info exists uInfo]} {set uInfo ""}

   if {![llength $uInfo]} {return 0} 

   catch {set vInfo [exec grep -F VERSION_ID /etc/os-release | tr -s \\075\\042\\056 \\040]}
   if {![info exists vInfo]} {set vInfo ""}

   if {[llength $vInfo] > 1} \
      {
       set glbUbuntuVersion [lindex $vInfo 1]
       return $glbUbuntuVersion
      }

   return 0
}

#
# Create path for output file if needed.

proc createPath { inName } \
{
   set dirName [file dirname $inName]

   if {![file exists $dirName]} \
      {
       catch {file mkdir $dirName}
       return $dirName
      }

   return ""
}

#
# Return file name with extension added if needed.

proc addExt { fName ext } \
{
   if {[strcmp [string index $ext 0] "."]} {set ext .$ext}

   set oldExt [file extension $fName]

   if {[strcasecmp $oldExt $ext]} {set fName [file root $fName]$ext}

   return $fName
}

#
# Bottom.
