#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts stderr "List Varian Header and Block Header Info:"
    puts stderr " -in       inName \[fid\]   Input Name, or Keyword stdin." 
    puts stderr " -vSize    vSize  \[Auto\]  Vector Size, Words." 
    puts stderr " -vCount   vCount \[Auto\]  Number of Vector Groups." 
    puts stderr " -expCount eCount \[Auto\]  Vectors Per Vector Group." 
    puts stderr " -wordSize wSize  \[Auto\]  Word Size, Bytes." 
    puts stderr "Byte Swap Options:"

    if {$SYS_BYTESWAP} \
       {
        puts stderr " -swap     Do Four-Byte Swap."
        puts stderr " -noswap   Suppress Four-Byte Swap."
        puts stderr " -aswap    Suppress Four-Byte Swap (Meaning is System-dependant)."
        puts stderr " -noaswap  Do Four-Byte Swap (Default. Meaning is System-dependant)."
       } \
    else \
       {
        puts stderr " -swap     Do Four-Byte Swap."
        puts stderr " -noswap   Suppress Four-Byte Swap."
        puts stderr " -noaswap  Suppress Four-Byte Swap (Default. Meaning is System-dependant)."
        puts stderr " -aswap    Do Four-Byte Swap (Meaning is System-dependant)."
       }
 
    puts stderr "Notes:"
    puts stderr " 1. Use -vCount 0 to read and display only information from the initial"
    puts stderr "    header and the first block header."
    exit 0
   }

if {$SYS_BYTESWAP} \
   {
    set swapFlag 1
   } \
else \
   {
    set swapFlag 0
   }

set inName   fid
set outName  None
set vSize    Auto
set vCount   Auto
set expCount Auto
set wordSize Auto

set vHdr      0
set vBlockHdr 0
set dataBuff  0
set oneBuff   0
set buffSize  0
set wordCount 0
set inUnit    0
set outUnit   1

getArgD $ARGV -in       inName
getArgD $ARGV -out      outName
getArgD $ARGV -vSize    vSize
getArgD $ARGV -vCount   vCount
getArgD $ARGV -expCount vCount
getArgD $ARGV -wordSize wordSize

if {[flagLoc $ARGV -swap]}   {set swapFlag 1}
if {[flagLoc $ARGV -noswap]} {set swapFlag 0}

if {$SYS_BYTESWAP} \
   {
    if {[flagLoc $ARGV -aswap]}   {set swapFlag 0}
    if {[flagLoc $ARGV -noaswap]} {set swapFlag 1}
   } \
else \
   {
    if {[flagLoc $ARGV -aswap]}   {set swapFlag 1}
    if {[flagLoc $ARGV -noaswap]} {set swapFlag 0}
   }

if {$swapFlag} \
   {
    set swapArg swap
   } \
else \
   {
    set swapArg noswap
   }

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Varian fid Input $inName"
    exit 1
   }

set vHdr      [sysMalloc 8 int]
set vBlockHdr [sysMalloc 7 int]

if {![strcasecmp $inName stdin]} \
   {
    set inUnit 0
   } \
else \
   {
    set error [catch {set inUnit [sysOpen $inName r]}]

    if {$error} \
       {
        puts stderr "Error Opening Varian fid Input $inName"
        exit 1
       }
   }

if {![strcasecmp $outName stdout]} \
   {
    set outUnit 1
   } \
elseif {[strcasecmp $outName None]} \
   {
    if {![strcmp $inName $outName]} \
       {
        puts stderr "Error: Input and Output Names Must Be Different."
        exit 1
       }

    set error [catch {set outUnit [sysOpen $outName w]}]

    if {$error} \
       {
        puts stderr "Error Opening Varian fid Output $outName"
        exit 1
       }
   }

set n [sysReadB $inUnit $vHdr 8]

if {$n != 4*8} \
   {
    puts stderr "Error Reading Inital Header in Varian fid Input $inName"
    exit 1
   }

#
# LengthType long WordType int QuadType complex WordSize 4 FIDSize 1024 FIDCount 8192 ExpCount 1

set vInfo [getVHdrInfo $vHdr -$swapArg]
puts stderr $vInfo

if {![strcasecmp $vSize    Auto]} {set vSize    [getNamedItem FIDSize  $vInfo]}
if {![strcasecmp $vCount   Auto]} {set vCount   [getNamedItem FIDCount $vInfo]}
if {![strcasecmp $expCount Auto]} {set expCount [getNamedItem ExpCount $vInfo]}
if {![strcasecmp $wordSize Auto]} {set wordSize [getNamedItem WordSize $vInfo]}

if {$vCount > 0} \
   {
    set error [catch {set buffSize [expr $wordSize*$expCount*$vSize]}]

    if {$error} \
       {
        puts stderr "Error Computing Vector Buffer Size."
        exit 1
       }

    set wordCount [expr $vSize*$expCount]
    set dataBuff  [sysMalloc $buffSize  char]
    set oneBuff   [sysMalloc $wordCount float]
   }

puts stderr "BuffSize $buffSize"

if {[strcasecmp $outName None]} \
   {
    setVHdrInfo $vHdr "WordType float" -$swapArg

    set n [sysWrite $outUnit $vHdr 8]

    if {$n != 4*8} \
       {
        puts stderr "Error Writing Inital Header to Varian fid Output $outName"
        exit 1
       }
   }

if {$vCount} \
   {
    set loopCount $vCount
   } \
else \
   {
    set loopCount 1
   }

for {set i 0} {$i < $loopCount} {incr i} \
    {
     set n [sysReadB $inUnit $vBlockHdr 7]

     if {$n != 4*7} \
        {
         puts stderr "Error Reading Block Header $i in Varian fid Input $inName"
         exit 1
        }

     set bInfo [getVBlockHdrInfo $vBlockHdr -$swapArg]
     puts stderr "Block $i $bInfo"

     if {[strcasecmp $outName None]} \
        {
         set n [sysWrite $outUnit $vBlockHdr 7]
        
         if {$n != 4*7} \
            {
             puts stderr "Error Writing Block Header $i in Varian fid Output $outName"
             exit 1
            }
        }

     if {$i > $vCount - 1} {break}

     set n [sysReadB $inUnit $dataBuff $buffSize]

     if {$n != $buffSize} \
        {
         puts stderr "Error Reading Vector $i in Varian fid Input $inName"
         exit 1
        }

     if {[strcasecmp $outName None]} \
        {
         vsSet $oneBuff $i $wordCount

         if {$swapFlag} {vByteSwap $oneBuff $wordCount}

         set n [sysWrite $outUnit $oneBuff $wordCount]
    
         if {$n != [expr 4*$wordCount]} \
            {
             puts stderr "Error Writing Vector $i in Varian fid Output $outName"
             exit 1
            }
        }
    }

if {[strcasecmp $inName stdin]} {catch "sysClose $inUnit"}

exit 0
