
#
# Event handler for reading Varian parameters.

proc readParams_VARIAN {} \
{
   global nusFlag nusRevFlag nusName nusOffset nusInfo nusDimCount nusSampleCount verbose
   global Tx Ty Tz Ta Nx Ny Nz Na NUCx NUCy NUCz NUCa aq2D dimCount dimCountName
   global inName specialArgs dataSource vj2pipeMode

   clearScript

   set dataSource  [getDataSource $inName]
   set specialArgs "-noaswap"
   set vjInfoARGV  ""

   if {![info exists NUCx]} {set NUCx None}
   if {![info exists NUCy]} {set NUCy None}
   if {![info exists NUCz]} {set NUCz None}
   if {![info exists NUCa]} {set NUCa None}

   if {![info exists verbose]} {set verbose 1}

#
# Extract NUS schedule info if needed:

   if {$nusFlag && [strcasecmp $nusName None] && [file exists $nusName]} \
      {
       set nusInfo None

       if {$nusRevFlag} \
          {
           catch {set nusInfo [exec getNUSInfo.tcl -sample $nusName -sampleCount $nusSampleCount -off $nusOffset -rev]}
          } \
       else \
          {
           catch {set nusInfo [exec getNUSInfo.tcl -sample $nusName -sampleCount $nusSampleCount -off $nusOffset]}
          }

       if {![info exists nusInfo]} {set nusInfo None}

       set itemCount [llength $nusInfo]

       if {$verbose} \
          {
           if {$itemCount > 1} \
              {
               puts ""
               puts "NUS Schedule Information from $nusName:"

               for {set i 0} {$i < $itemCount} {incr i 2} \
                  {
                   puts [format " %-17s %8s" [lindex $nusInfo $i]: [lindex $nusInfo [expr $i + 1]]]
                  }
              } \
           else \
              {
               puts ""
               puts "NUS Schedule Information from $nusName: $nusInfo"
              }
          }

       set nusDimCount [getNamedItemD nusDimCount $nusInfo 0]

       if {$nusDimCount >= 2} \
          {
           set dimCount $nusDimCount

           if {![strcasecmp $nusSampleCount Auto]} {set nusSampleCount [getNamedItemD sampleCount $nusInfo 0]}
           if {![strcasecmp $nusOffset Auto]}      {set nusOffset      [getNamedItemD offset      $nusInfo 0]}
          }

       if {$verbose} {puts ""}
      }

#
# Update parameters using vj2pipe scheme if requested:

   if {$vj2pipeMode} \
      {
       if {$verbose} \
          {
           if {$vj2pipeMode == 2} \
              {
               puts "Varian Conversion: Using vj2pipe to update parameters and PPM calibration;"
               puts "                   use option -novj2pipe to suppress this mode."
              } \
           else \
              {
               puts "Varian Conversion: Using vj2pipe to update parameters;"
               puts "                   use option -novj2pipe to suppress this mode."
              }
          }

       set vjInfoArgs ""
       set vjDimCount 0
       set nn         0

       catch {set vjInfoArgs [exec vj2pipe.com -sinfo -nonusadj -msgName stdout -noverb]}

       if {![info exists vjInfoArgs]} {set vjInfoArgs ""}

       if {[llength $vjInfoArgs] < 2} \
          {
           if {$verbose} {puts "Varian Conversion Warning: Error Executing vj2pipe.com"}
           return
          }

       if {$verbose} {puts "Varian Conversion: vj2pipe reported [llength $vjInfoArgs] options and values."}

       set vjInfoARGV "vjInfo $vjInfoArgs"

       if {$vj2pipeMode == 1} \
          {
           getArgD $vjInfoARGV -ndim vjDimCount

           for {set i 0} {$i < $vjDimCount} {incr i} \
              {
               set dimName [lindex "x y z a" $i]
               set p       NUC

               if {![flagLoc $vjInfoARGV -$dimName$p]} continue

               eval "global $p$dimName"
               eval "set oldVal \$$p$dimName"

               if {![info exists oldVal]} {set oldVal ""}

               set newVal ""

               getArgD $vjInfoARGV -$dimName$p newVal

               if {![llength $newVal]} continue

               eval "set $p$dimName $newVal"
    
               if {$verbose && [strcmp $oldVal $newVal]} {puts [format " %5s Old: %10s New: %10s" $dimName$p $oldVal $newVal]}
              }
          } 
      }

#
# Extract parameters using NMRPipe scheme:

   getVarianND $inName

#
# Update parameters according to NUS schedule:

   if {$nusDimCount >= 2} {set dimCount $nusDimCount}

   if {$nusDimCount >= 2} \
      {
       set yTNUS [getNamedItemD yTNUS $nusInfo 0]

       if {$yTNUS > 0} \
          {
           set Ty $yTNUS
           set Ny [expr 2*$Ty]
          }
      }

   if {$nusDimCount >= 3} \
      {
       set zTNUS [getNamedItemD zTNUS $nusInfo 0]

       if {$zTNUS > 0} \
          {
           set Tz $zTNUS
           set Nz [expr 2*$Tz]
          }
      }

   if {$nusDimCount >= 4} \
      {
       set aTNUS [getNamedItemD aTNUS $nusInfo 0]

       if {$aTNUS > 0} \
          {
           set Ta $aTNUS
           set Na [expr 2*$Ta]
          }
      }

#
# Update the number of dimensions:

    if {![string compare $dimCountName N]} \
       {
        if {$Ty < 2 && $dimCount > 1} \
           {
            set dimCount 1
           } \
        elseif {$Tz < 2 && $dimCount > 2} \
           {
            set dimCount 2
           } \
        elseif {$Ta < 2 && $dimCount > 3} \
           {
            set dimCount 3
           }
       }

    setDimCount N

#
# Update parameters using vj2pipe scheme if requested:

   if {$vj2pipeMode && [llength $vjInfoARGV] > 2} \
      {
       if {$vj2pipeMode == 1} \
          {
           set updateList "MODE LAB"
          } \
       else \
          {
           set updateList "MODE LAB CAR OBS"
          }

       if {[flagLoc $vjInfoARGV -aq2D]} \
          {
           set oldVal $aq2D

           getArgD $vjInfoARGV -aq2D aq2D

           set newVal $aq2D

           if {$verbose && [strcmp $oldVal $newVal]} {puts [format " %5s Old: %10s New: %10s" $dimName$p $oldVal $newVal]}
          }

       for {set i 0} {$i < $dimCount} {incr i} \
          {
           set dimName [lindex "x y z a" $i]

           foreach p $updateList \
              {
               if {![flagLoc $vjInfoARGV -$dimName$p]} continue

               eval "global $p$dimName"
               eval "set oldVal \$$p$dimName"  

               if {![info exists oldVal]} {set oldVal ""}

               set newVal ""

               getArgD $vjInfoARGV -$dimName$p newVal

               if {![llength $newVal]} continue

               if {![strcasecmp $p OBS]} {set newVal [safeFormat $newVal %.3f]}

               eval "set $p$dimName $newVal"

               if {$verbose && [strcmp $oldVal $newVal]} {puts [format " %5s Old: %10s New: %10s" $dimName$p $oldVal $newVal]}
              }
          }
      }
}

#
# getVarianND: get parameters from a Varian parameter file.

proc getVarianND { fidName } \
{
   global Nx Tx MODEx SWx OBSx CARx FTx LABx NUCx \
          Ny Ty MODEy SWy OBSy CARy FTy LABy NUCy \
          Nz Tz MODEz SWz OBSz CARz FTz LABz NUCz \
          Na Ta MODEa SWa OBSa CARa FTa LABa NUCa EXPNAME

   global aqORD aq2D dimCount dimCountName vendorScans temperature carrierInfo infoRequest infoResult dateText
   global tcl_precision old_tcl_precision shutdown

#
# Initialize:

   set fidBytes -1
   set pFile    procpar
   set nmrPath  [tkFileFn dirname $fidName] 
   set aqORD    0

   if {![info exists infoRequest]} {set infoRequest None}

#
# Set the precision for the PPM expressions, enable restore of precision later:

   if {[info exists tcl_precision]} {set old_tcl_precision $tcl_precision}

   set shutdown \
     {
      if {[info exists old_tcl_precision]} \
         {
          set tcl_precision $old_tcl_precision
         }
     }

   set tcl_precision 17
#
# Test for the existence of the parameter and data files.
# Read their contents into arrays.

   if {[tkFileFn exists $fidName]} \
      {
#      set fidBytes [getFileBytes $fidName]
      }
     
   if {[tkFileFn exists $nmrPath/$pFile]} \
      {
       set status [rdProcPar $nmrPath/$pFile pList stderr]
      } \
   else \
      {
       puts stderr "Error: Unable to find file $nmrPath/$pFile"
       eval $shutdown
       return 1
      }

#
# Set the dimension count if needed:

   if {![strcasecmp $dimCountName N]} \
      {
       set i 0

       foreach v {np ni ni2 ni3} \
          {
           if {![info exists pList($v)]} \
              {
               set pList($v) 1
               break
              }

           incr i           
          }

       set dimCount $i

       setDimCount $dimCountName
      }

   if {![info exists pList(date)]} {set dateText $pList(date)}

   if {![info exists pList(sw)]}   {set pList(sw)   10000.0}
   if {![info exists pList(sfrq)]} {set pList(sfrq)   500.0}
   if {![info exists pList(tn)]}   {set pList(tn)     H1}
   if {![info exists pList(dn)]}   {set pList(dn)     $pList(tn)}
   if {![info exists pList(dfrq)]} {set pList(dfrq)   $pList(sfrq)}
   if {![info exists pList(nt)]}   {set pList(nt)     1}

   if {![strcasecmp $pList(dn)   "(Null)"]} {set pList(dn)    $pList(tn)}
   if {![strcasecmp $pList(dfrq) "(Null)"]} {set pList(dfrq)  $pList(sfrq)}

   set vendorScans $pList(nt)

   updateDataScale

   foreach i {1 2 3 4} \
      {
       if {![info exists pList(sw$i)]}   {set pList(sw$i)    $pList(sw)}
       if {![info exists pList(dfrq$i)]} {set pList(dfrq$i)  $pList(dfrq)}
       if {![info exists pList(dn$i)]}   {set pList(dn$i)    $pList(dn)}

       if {![strcasecmp $pList(dn$i)   "(Null)"]} {set pList(dn$i)    $pList(tn)}
       if {![strcasecmp $pList(dfrq$i) "(Null)"]} {set pList(dfrq$i)  $pList(dfrq)}
      }

#
# Set the axis labels and acquisition order:

   if {[info exists pList(seqfil)]} {set EXPNAME $pList(seqfil)}

   if {![info exists pList(array)]} {set pList(array) None}

   switch $dimCount \
      {
       1 {
          set LABx  X 
          set LABy  Y
          set LABz  Z
          set LABa  A
          set aqORD 0
         }

       2 {
          set LABx  F2 
          set LABy  F1 
          set LABz  Z
          set LABa  A
          set aqORD 0
         }

       3 {
          set LABx  F3
          set LABy  F1
          set LABz  F2 
          set LABa  A
          set aqORD 0

          if {![string compare $pList(array) "phase,phase2"]} \
             { 
              set aqORD 1
             }
         }

       4 {
          set LABx F4
          set LABy F1
          set LABz F2
          set LABa F3 
          set aqORD 0

          if {![string compare $pList(array) "phase,phase2,phase3"]} \
             {
              set aqORD 1
             }
         }

       default {
               }
      }

   set MODEx Complex
   set MODEy Complex
   set MODEz Complex
   set MODEa Complex

   catch {set Nx    $pList(np)}
   catch {set Tx    [expr $Nx / 2]}
   catch {set SWx   $pList(sw)}
   catch {set OBSx  $pList(sfrq)}
   catch {set LABx  $pList(tn)}

   catch {set Ty    $pList(ni)}
   catch {set Ny    [expr $Ty * 2]}
   catch {set SWy   $pList(sw1)}
   catch {set OBSy  $pList(dfrq)}
   catch {set LABy  $pList(dn)}
  
   catch {set Tz    $pList(ni2)}
   catch {set Nz    [expr $Tz * 2]}
   catch {set SWz   $pList(sw2)}
   catch {set OBSz  $pList(dfrq2)}
   catch {set LABz  $pList(dn2)}
 
   catch {set Ta    $pList(ni3)}
   catch {set Na    [expr $Ta * 2]}
   catch {set SWa   $pList(sw3)}
   catch {set OBSa  $pList(dfrq3)}
   catch {set LABa  $pList(dn3)}

#
# Test for absolute value:

   if {[info exists pList(dmg1)] && $Ty > 1} \
      {

       set key [string range [string trim $pList(dmg1)] 0 1]

       if {![strcasecmp $key av] || ![strcasecmp $key pw]} \
          {
           set aq2D   Magnitude
           set MODEy  Real
           set Ny     $Ty
          }
      }

#
# Update the menus associated with the ambiguous values:

   set frqList   {sfrq dfrq dfrq1 dfrq2 dfrq3 dfrq4 dfrq5 dfrq6}
   set nucList   {tn   dn   dn1   dn2   dn3   dn4   dn5   dn6}
   set offList   {tof  dof  dof1  dof2  dof3  dof4  dof5  dof6}

   set swList  {}
   set obsList {}
   set carList {}

   if {![info exists pList(temp)]} \
      {
       set pList(temp) 0.0
      } \
   else \
      {
       if {[llength $pList(temp)] > 1} \
          {
           set pList(temp,array) $pList(temp)
           set pList(temp) [lindex $pList(temp) 0]
          } 
      }

   foreach i {sw sw1 sw2 sw3 sw4 sw5} \
      {
       if {[info exists pList($i)]} \
          {
           if {$pList($i) != 0.0} \
              {
               set     line [list "[format %9.3f $pList($i)] $i" $pList($i)]
               lappend swList $line
              }
          }
      }

   foreach i $nucList \
      {
       if {![info exists pList($i)]} {set pList($i) NONE}
      }

   foreach i $offList \
      {
       if {![info exists pList($i)]} {set pList($i) 0.0}
      }

   set k 0

   foreach i $frqList \
      {
       if {[info exists pList($i)]} \
          {
           set j [lindex $nucList $k]

           if {$pList($i) != 0.0} \
              {
               set     val  $pList($i)
               set     line [list "[format %9.3f $val] ($i) $pList($j)" $val]
               lappend obsList $line
              }
          }

       incr k
      }

   foreach i $frqList \
      {
       if {![info exists pList($i)]} {set pList($i) 500.00}
      }

#
# Update the carrier positions:

   set updateCar 0

   if {"$temperature" == "From File"} \
      {
       set temperature [expr $pList(temp) + 273.0]
      }

   if {[nucIsProton $pList(tn)]} {set updateCar 1}

   if {"$CARx" == "H2O"} \
      {
       if {[nucIsProton $pList(tn)]} \
          {
           set h2oPPM  [getH2Oppm $temperature]
           set rName   H2O
          } \
       else \
          {
           set CARx    0.0
           set h2oPPM  0.0
           set rName   Null
          }
      } \
   else \
      {
       set h2oPPM  $CARx
       set rName   User
      }

   set h2oOBS  $pList(sfrq)

   set line    [list "    H2O (sfrq) $pList(tn)" H2O]
   lappend     carList $line

   set car     [format %7.3f $h2oPPM]
   set line    [list "$car (sfrq) $pList(tn)/$rName" $car]
   lappend     carList $line

   set k       0

   foreach i $frqList \
      {
       if {![nucIsProton $pList(tn)]} break
 
       set j [lindex $nucList $k]
       set g [getGamma $pList($j)]

       set val $pList($i)

       if {$g != 0.0 && $val != 0.0} \
          {
           set car [expr (1.0e+6*$val-((($h2oOBS*1.0e+6)-($h2oOBS*$h2oPPM))*$g))/$val]
           set car [format %7.3f $car]

           if {$car > 0.0} \
              {
               set line [list "$car ($i) $pList($j)" $car]
               lappend  carList $line

               set carrierInfo($i) [string trim $car]

               set thisNuc [normalizeNucName $pList($j)]

               set nucInfo($thisNuc,LAB) $pList($j)
               set nucInfo($thisNuc,OBS) $pList($i)
               set nucInfo($thisNuc,CAR) $car

               if {$k == 0} {set CARx $car}
               if {$k == 1} {set CARy $car}
               if {$k == 3} {set CARz $car}
               if {$k == 4} {set CARa $car}
              }
          }

       incr k
      }

   if {[strcasecmp $NUCx None]} \
      {
       set thisNuc [normalizeNucName $NUCx]

       if {[info exists nucInfo($thisNuc,LAB)]} \
          {
           set LABx $nucInfo($thisNuc,LAB)
           set OBSx $nucInfo($thisNuc,OBS)
           set CARx $nucInfo($thisNuc,CAR)
          }
      }

   if {[strcasecmp $NUCy None]} \
      {
       set thisNuc [normalizeNucName $NUCy]

       if {[info exists nucInfo($thisNuc,LAB)]} \
          {
           set LABy $nucInfo($thisNuc,LAB)
           set OBSy $nucInfo($thisNuc,OBS)
           set CARy $nucInfo($thisNuc,CAR)
          }
      }

   if {[strcasecmp $NUCz None]} \
      {
       set thisNuc [normalizeNucName $NUCz]

       if {[info exists nucInfo($thisNuc,LAB)]} \
          {
           set LABz $nucInfo($thisNuc,LAB)
           set OBSz $nucInfo($thisNuc,OBS)
           set CARz $nucInfo($thisNuc,CAR)
          }
      }

   if {[strcasecmp $NUCa None]} \
      {
       set thisNuc [normalizeNucName $NUCa]

       if {[info exists nucInfo($thisNuc,LAB)]} \
          {
           set LABa $nucInfo($thisNuc,LAB)
           set OBSa $nucInfo($thisNuc,OBS)
           set CARa $nucInfo($thisNuc,CAR)
          }
      }

   if {!$updateCar} {set carList {}}

   foreach i {x y z a} \
      {
       if {[llength $swList]  > 0} {updateGenMenu SW  $i $swList}
       if {[llength $obsList] > 0} {updateGenMenu OBS $i $obsList}
       if {[llength $carList] > 0} {updateGenMenu CAR $i $carList}
      }

#
# Rough test for gradient-enhanced experiment:

   set poorList {CARx CARy CARz CARa OBSy OBSz OBSa}

   if {[info exists pList(pslabel)]} \
      {
       if {[string first G [string toupper $pList(pslabel)]] >= 0} \
          {
           if {$dimCount == 2} \
              {
               if {[strcasecmp $MODEy Real]} {set MODEy Rance-Kay}
              } \
           else \
              {
               if {![strcasecmp $pList(dn2) 15n] || ![strcasecmp $pList(dn2) n15]} \
                  {
                   set MODEz Rance-Kay
                  } \
               else \
                  {
                   if {[strcasecmp $MODEy Real]} {set MODEy Rance-Kay}
                  }
              }

           if {$dimCount > 2} \
              {
               lappend poorList MODEy
               lappend poorList MODEz
               lappend poorList MODEa
              }
          }
      }

   showPoorEntries $poorList

#
# Adjust precision of calculated parameters:

   set SWx  [format "%.3f" $SWx]
   set SWy  [format "%.3f" $SWy]
   set SWz  [format "%.3f" $SWz]

   set CARx [format "%.3f" $CARx]
   set CARy [format "%.3f" $CARy]
   set CARz [format "%.3f" $CARz]

   set OBSx [format "%.3f" $OBSx]
   set OBSy [format "%.3f" $OBSy]
   set OBSz [format "%.3f" $OBSz]

   set temperature [format "%.3f" $temperature]

# Extract any requested values:

   if {[strcasecmp $infoRequest None]} \
      {
       set infoResult ""

       foreach var $infoRequest \
          {
           lappend infoResult $var

           if {[info exists pList($var)]} \
              {
               lappend infoResult $pList($var)
              } \
           else \
              {
               lappend infoResult None
              }
          }
      }

   eval $shutdown

   return 0
}

#
# Bottom.
