#!/bin/csh

set inName1 = (`getArgD $argv -in  ft/test%03d.ft3`)
set inName1 = (`getArgD $argv -in1 $inName1`)
set xyzName = (`getArgD $argv -xyz $inName1`)

set inName2 = (`getArgD $argv -in2 xzy/test%03d.ft3`)
set xzyName = (`getArgD $argv -xzy $inName2`)

set inName3 = (`getArgD $argv -in3 yzx/test%03d.ft3`)
set yzxName = (`getArgD $argv -yzx $inName3`)
set hi      = (`getArgD $argv -hi  Auto`)


if (`flagLoc $argv -help`) then
   echo "Create and View XYZ XZY YZX Planes of 3D."
   echo "Required Input:"
   echo " -xyz  xyzName  3D Input (Also: -in or -in1). [$xyzName]"
   echo "Reordered Input or Keyword None:"
   echo " -xzy  xzyName  3D in XZY Order (Also: -in2)  [$xzyName]"
   echo " -yzx  yzxName  3D in YZX Order (Also: -in3)  [$yzxName]"
   echo "Notes:"
   echo " 1. If it not specified as None, xzyName and yzxName will"
   echo "    be created if they do not exist:"
   echo " 3. All Remaining Options Will be Passed to nmrDraw. Example:" 
   echo ""
   echo "       viewAllPlanes.com -in hnca/test%03d.ft3 -hi 5.0"
   exit 0
endif

set drawArgs = ""

@ n = $#argv
@ i = 1

while ($i <= $n)
   if (`string same $argv[$i] -xyz`) then
      @ i++
   else if (`string same $argv[$i] -xzy`) then
      @ i++
   else if (`string same $argv[$i] -yzx`) then
      @ i++
   else
      set drawArgs = ($drawArgs $argv[$i])
   endif

   @ i++
end

if (`string notcsame $xyzName None`) then
   set thisName = (`nmrPrintf $xyzName 1 1`)

   if (!(-e $thisName)) then
      echo "Error finding 3D Input $xyzName ($thisName)".
      exit 1
   endif
endif

if (`string notcsame $xzyName None`) then
   set thisName = (`nmrPrintf $xzyName 1 1`)

   if (!(-e $thisName)) then
      echo "Creating XZY Planes $xzyName ..."

      xyz2pipe -in $xyzName -x \
      | nmrPipe -fn TP  \
      | nmrPipe -fn ZTP \
      | nmrPipe -fn TP  \
      | pipe2xyz -out $xzyName -x

      showhdr $xzyName
   endif
endif

if (`string notcsame $yzxName None`) then
   set thisName = (`nmrPrintf $yzxName 1 1`)

   if (!(-e $thisName)) then
      echo "Creating YZX Planes $yzxName ..."

      xyz2pipe -in $xyzName -x \
      | nmrPipe -fn ZTP \
      | nmrPipe -fn TP -nohdr \
      | pipe2xyz -out $yzxName -x
   
      showhdr $yzxName
   endif
endif

set hi       = (`getSpecHi.tcl -brief -in $xyzName -hi $hi`)
set drawArgs = ($drawArgs -hi $hi)

nmrDraw $drawArgs -in $xyzName -broadcast `getGeom -left`  -nophase -group 1 &
sleep 3

nmrDraw $drawArgs -in $xzyName -broadcast `getGeom -mid`   -nophase -group 1 &
sleep 3

nmrDraw $drawArgs -in $yzxName -broadcast `getGeom -right` -nophase -group 1 &

