#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

global ARGV ARGC
global database

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV       [concat $argv0 $argv]
set ARGC       [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Create XPLOR PSF File from Sequence in Existing PDB File."
    puts " -pdb  pdbName   \[ref.pdb\]         PDB Input Name."
    puts " -psf  psfName   \[ref.psf\]         PSF Output Name."
    puts " -inp  inpName   \[generate.inp\]    XPLOR PSF Generation Output."
    puts " -seg  segName   \[Default\]         Segment Name, or 'None'."
    exit 1
   }

dynInit -fast

set pdbName ref.pdb
set psfName ref.psf
set inpName generate.inp
set segName Default

getArgD $ARGV -pdb pdbName
getArgD $ARGV -psf psfName
getArgD $ARGV -inp inpName
getArgD $ARGV -seg segName

#
#----------------------------------------------------------------------------

if {![file exists $pdbName]} \
   {
    puts stderr "Error Finding PDB Input $pdbName"
    exit 1
   }

set pdbSrc [dynRead -pdb -in $pdbName -verb]

if {![strcasecmp $segName Default]} \
   {
    set segName [dynGetInfo -src $pdbSrc -segName]
   }

if {[string length $segName] != 4} \
   {
    puts stderr "WARNING segID with less than 4 characters!"
   }

set error [catch {set inpUnit [open $inpName w]}]

if {$error} \
   {
    puts stderr "Error Opening XPLOR Output $inpName"
    exit 1
   }

puts $inpUnit "remarks  file  generate.inp"
puts $inpUnit "remarks  generate structure file and hydrogens for a protein"

puts $inpUnit " "

puts $inpUnit "topology"
puts $inpUnit "  @TOPPAR:topallhdg.pro"
puts $inpUnit "  @TOPPAR:top_axis.pro"
puts $inpUnit "end"
puts $inpUnit " "
puts $inpUnit "parameter"
puts $inpUnit "  @TOPPAR:parallhdg.pro"
puts $inpUnit "  @TOPPAR:par_axis_3.pro"
puts $inpUnit "end"
puts $inpUnit " "
puts $inpUnit "parameter"
puts $inpUnit "   nbonds"
puts $inpUnit "     repel=0.8 nbxmod=3"
puts $inpUnit "   end"
puts $inpUnit " end"

puts $inpUnit " "
puts $inpUnit "segment"

if {[strcasecmp $segName None]} \
   {
    puts $inpUnit "  name=$segName"
   }

#
# Suppress parameters that add C- and N-terminus:
 
puts $inpUnit "  chain"
puts $inpUnit "!   @TOPPAR:toph19.pep"
puts $inpUnit "    LINK PEPP HEAD - * TAIL + PRO END"
puts $inpUnit "    LINK PEPT HEAD - * TAIL + *   END"
puts $inpUnit "    coordinates @$pdbName"
puts $inpUnit "  end"
puts $inpUnit "end"

puts $inpUnit " "
puts $inpUnit "vector do (name=\"CD\")  ( name CD1  and resname ile )"
puts $inpUnit "vector do (name=\"HD1\") ( name HD11 and resname ile )"
puts $inpUnit "vector do (name=\"HD2\") ( name HD12 and resname ile )"
puts $inpUnit "vector do (name=\"HD3\") ( name HD13 and resname ile )"
puts $inpUnit " "

puts $inpUnit " "
puts $inpUnit "write structure output=$psfName end"
puts $inpUnit " "
puts $inpUnit "stop"

close $inpUnit

puts "XPLOR Script $inpName Created, Segment Name: $segName."

exit
