#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@" -notk

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Create an XPLOR Script for Dipolar Coupling Refinement:"
    puts " -in     inName     \[ref.pdb\]    PDB Input Name."
    puts " -psf    psfList    \[ref.psf\]    PSF Input."
    puts " -out    outName    \[sa.pdb\]     PDB Output Name."
    puts " -script scriptName \[sa.inp\]     XPLOR Script Output Name."
    puts "Optional Restraints:"
    puts " -noes   xNOEName   \[None\]       NOE Restraints."
    puts " -tor    xTorName   \[None\]       Torsion Restraints."
    puts " -vean   xVEANName  \[None\]       Vector Angle Restraints."
    puts " -rama                           Ramachandran Potential ON (Default)."
    puts " -norama                         Ramachandran Potential Off."
    puts "XPLOR Force Field Parameter Files:"
    puts " -par  parList  \[TOPPAR:parallhdg.pro TOPPAR:par_axis_3.pro\]"
    puts "Notes:"
    puts " 1. The refinement script requires DC restraint input,"
    puts "    as produced by xDCSetup.tcl etc."
    puts " 2. Expects the following files in the current directory,"
    puts "    or they will be taken from the NMRTXT directory:"
    puts "    pdbHeader.inp pdbStat.inp ramps.inp"
    exit 1 
   }

set parList    "TOPPAR:parallhdg.pro TOPPAR:par_axis_3.pro"
set pdbInList  ref.pdb
set pdbOutName sa.pdb
set ramaFlag   1

set scriptName sa.inp

set nModel      1
set svANIflg    0
set atomList    CA

set psfList     ref.psf

set pdbHead     pdbHeader.inp
set pdbStat     pdbStat.inp
set rampName    ramps.inp

set xdipName    xdipSetup.inp
set xForce      xdipForce.inp
set xStat       xdipStat.inp

set noeList     noes.xdcn
set torList     torsions.xdih 
set veanList    vean.xang

set noeList     None 
set torList     None
set veanList    None 

getArgD $ARGV -out       pdbOutName
getArgD $ARGV -script    scriptName

getListArgD $ARGV -in    pdbInList
getListArgD $ARGV -psf   psfList
getListArgD $ARGV -par   parList

getListArgD $ARGV -noes  noeList
getListArgD $ARGV -tor   torList
getListArgD $ARGV -vean  veanList 

if {[flagLoc $ARGV -rama]}   {set ramaFlag 1}
if {[flagLoc $ARGV -norama]} {set ramaFlag 0}

#
#----------------------------------------------------------------------

if {![file exists $rampName]} {set rampName $env(NMRTXT)/ramps.inp}
if {![file exists $pdbHead]}  {set pdbHead  $env(NMRTXT)/pdbHead.inp}
if {![file exists $pdbStat]}  {set pdbStat  $env(NMRTXT)/pdbStat.inp}

if {![strcasecmp $noeList  None]} {set noeList  ""}
if {![strcasecmp $torList  None]} {set torList  ""}
if {![strcasecmp $veanList None]} {set veanList ""}

set flagList "bonds angl impr vdw"

set maxNOE  1000
set maxDIH  1000
set maxVEAN 1000

set outUnit [open $scriptName w]

puts $outUnit "!"
puts $outUnit "! X-PLOR script for refinement against dipolar couplings"
puts $outUnit "!"
puts $outUnit ""

puts $outUnit "! read in the PAR file(s)"
puts $outUnit ""
puts $outUnit "paramter"

foreach parName $parList \
   {
    puts $outUnit "   @$parName"
   }

puts $outUnit "end"
puts $outUnit ""

puts $outUnit "! read in the PSF file(s)"
puts $outUnit ""
puts $outUnit "structure"

foreach psfName $psfList \
   {
    puts $outUnit "   @$psfName"
   }

puts $outUnit "end"
puts $outUnit ""

puts $outUnit "! read in the PDB file(s)"
puts $outUnit ""

foreach pdbName $pdbInList \
   {
    puts $outUnit "coordinates @$pdbName"
   }

puts $outUnit ""
puts $outUnit "! delete atoms w/o coordinates"
puts $outUnit "dele sele (not known) end"

puts $outUnit ""
puts $outUnit "! save coordinates for further reference"
puts $outUnit "vector do (refx=x) (all)"
puts $outUnit "vector do (refy=y) (all)"
puts $outUnit "vector do (refz=z) (all)"

puts $outUnit ""
puts $outUnit "! fix the pseudomolecules representing the axes of tensor"
puts $outUnit "constraints fix ((resid 500:510) and name OO) end"

puts $outUnit ""
puts $outUnit "! setup force constants and restraint tables"
puts $outUnit ""

puts $outUnit "! set the initial weights for the experimental energy terms"
puts $outUnit ""

puts $outUnit "! covalent"
puts $outUnit "evaluate (\$k_bond = 1.0)"
puts $outUnit "evaluate (\$k_ang  = 1.0)"
puts $outUnit "evaluate (\$k_imp  = 1.0)"
puts $outUnit ""
puts $outUnit "! nonbonded"
puts $outUnit "evaluate (\$k_vdw   = 1.0)"
puts $outUnit "evaluate (\$radius  = 0.8)"
puts $outUnit ""
puts $outUnit "! ramachandran"
puts $outUnit "evaluate (\$krama = 0.002)   !rama"
puts $outUnit "evaluate (\$kramalr = 0.001) !long range rama"
puts $outUnit ""
puts $outUnit "evaluate (\$kvirt1doverall=0.0000)"
puts $outUnit "evaluate (\$kvirt1d=0.000)"
puts $outUnit "evaluate (\$kvirt2d=0.000)"
puts $outUnit "evaluate (\$kvirt3d=0.000)"
puts $outUnit ""
puts $outUnit "! noe, dihedral restraints"
puts $outUnit "evaluate (\$knoe  = 30.0)   ! noes"
puts $outUnit "evaluate (\$asym  = 1.0)    ! slope of NOE potential"
puts $outUnit "evaluate (\$kcdih  = 300.0) ! torsion angles"
puts $outUnit ""
puts $outUnit "! dipolar restraints"
puts $outUnit "evaluate (\$kdipo = 1.00)"
puts $outUnit ""
puts $outUnit "! vean (vector angles) restraints"
puts $outUnit "evaluste (\$kvean_bor = 40.0)   !vector angle border exclusion"
puts $outUnit "evaluste (\$kvean_cen = 10.0)   !vector angle center exclusion"
puts $outUnit ""
puts $outUnit "! harmonic coordinate restraint"
puts $outUnit "evaluate (\$kharm = 10.0)"
puts $outUnit ""
puts $outUnit "! radius of gyration"
puts $outUnit "evaluate (\$krg = 50.0)"

puts $outUnit "! apply the weights"
puts $outUnit ""

puts $outUnit "! covalent terms"
puts $outUnit "constraints interaction (all) (all) weights"
puts $outUnit "    * 1 bonds \$k_bond angles \$k_ang improper \$k_imp"
puts $outUnit "end end"
puts $outUnit ""
puts $outUnit "!non bonded energy terms"
puts $outUnit "parameters"
puts $outUnit "  nbonds"
puts $outUnit "    atom"
puts $outUnit "    nbxmod    3"
puts $outUnit "    wmin   =  0.01    ! warning off"
puts $outUnit "    cutnb  =  4.5     ! nonbonded cutoff"
puts $outUnit "    ctonnb =  2.99    ! begin"
puts $outUnit "    ctofnb =  3.      ! end of nb shifting"
puts $outUnit "    tolerance 0.5     ! for NB update"
puts $outUnit "    repel  =  \$radius ! scale factor for vdW radii = 1 ( L-J radii)"
puts $outUnit "    rexp   =  2       ! exponents in (r^irex - R0^irex)^rexp"
puts $outUnit "    irex   =  2"
puts $outUnit "    rcon   =  \$k_vdw  ! actually set the vdW weight"
puts $outUnit "  end"
puts $outUnit "end"
puts $outUnit ""

if {$ramaFlag} \
   {
    puts $outUnit "set message off echo off end"
    puts $outUnit "! ramachandran term setup"
    puts $outUnit "rama"
    puts $outUnit "    nres=10000"
    puts $outUnit ""
    puts $outUnit "    @GAUSSIANS:shortrange_gaussians.tbl"
    puts $outUnit "    @GAUSSIANS:new_shortrange_force.tbl"
    puts $outUnit ""
    puts $outUnit "    @GAUSSIANS:longrange_gaussians.tbl"
    puts $outUnit "    @GAUSSIANS:longrange_4D_hstgp_force.tbl"
    puts $outUnit ""
    puts $outUnit "    @GAUSSIANS:virtualTA_hstgp_gauss.tbl"
    puts $outUnit "    @GAUSSIANS:virtualTA_force_hstgp.tbl"
    puts $outUnit "end"
    puts $outUnit ""
    puts $outUnit "@GAUSSIANS:newshortrange_setup.tbl"
    puts $outUnit "@GAUSSIANS:setup_4D_hstgp.tbl"
    puts $outUnit "@GAUSSIANS:setup_virtualTA_hstgp.tbl"

    lappend flagList rama
   }

if {[llength $noeList]} \
   {
    puts $outUnit ""
    puts $outUnit "! read NOE tables"
    puts $outUnit ""
    puts $outUnit "set echo off message off end"
    puts $outUnit ""

    puts $outUnit "noe"
    puts $outUnit "  reset"
    puts $outUnit "  nrestraints = $maxNOE"

    foreach noeName $noeList \
       {
        if {[file exists $noeName]} \
           {
            puts $outUnit ""
            puts $outUnit "  @$noeName"
           }
       }

    puts $outUnit "end"
    puts $outUnit ""

    puts $outUnit "set echo on message on end"
    puts $outUnit ""
    puts $outUnit ""
    puts $outUnit "! NOE energy parameters"
    puts $outUnit ""
    puts $outUnit "noe"
    puts $outUnit "  ceiling = 1000"
    puts $outUnit "  scale * \$knoe"
    puts $outUnit "  averaging  * cent"
    puts $outUnit "  potential  * soft"
    puts $outUnit "  sqoffset   * 0.0"
    puts $outUnit "  sqconstant * 1.0"
    puts $outUnit "  sqexponent * 2"
    puts $outUnit "  soexponent * 1"
    puts $outUnit "  asymptote  * \$asym"
    puts $outUnit "  rswitch    * 0.5"
    puts $outUnit "end"
    puts $outUnit ""

    lappend flagList noe
   }

if {[llength $torList]} \
   {
    puts $outUnit ""
    puts $outUnit "! read dihedral restraint tables"
    puts $outUnit ""
    puts $outUnit "set echo off message off end"
    puts $outUnit ""

    puts $outUnit "restraints dihedral"
    puts $outUnit "  reset"
    puts $outUnit "  nass  = $maxDIH"

    foreach torName $torList \
       {
        if {[file exists $torName]} \
           {
            puts $outUnit ""
            puts $outUnit "  @$torName"
           }
       }
    puts $outUnit "end"
    puts $outUnit ""
    puts $outUnit "set echo on message on end"
    puts $outUnit ""
    puts $outUnit "CDIH force constant"
    puts $outUnit "restraints dihedral"
    puts $outUnit "  scale = \$kdih"
    puts $outUnit "end"

    lappend flagList cdih
   }

if {[llength $atomList]} \
   {
    set nAtom [llength $atomList]

    puts $outUnit ""
    puts $outUnit "! harmonic coordinate restraint"
    puts $outUnit ""
    puts $outUnit "vector do (harmonic = 0.0) (all)"

    puts -nonewline $outUnit "vector do (harmonic = \$kharm)"
    puts -nonewline $outUnit " ("

    for {set i 0} {$i < $nAtom} {incr i} \
       {
        set atomName [lindex $atomList $i]
        if {$i} {puts -nonewline $outUnit " or "}
        puts -nonewline $outUnit "name $atomName"
       }

    puts $outUnit ")"
    puts $outUnit "restraints harmonic exponent=2 end"
    puts $outUnit ""

    lappend flagList harm
   }

if {[llength $veanList] } \
   {
    puts $outUnit ""
    puts $outUnit "! read VEAN (vector angle) restraint lists"
    puts $outUnit ""
    puts $outUnit "set echo off message off end"
    puts $outUnit ""

    puts $outUnit "vean"
    puts $outUnit "  reset"
    puts $outUnit "  nrestraints=$maxVEAN"
    foreach veanName $veanList \
       {
        if {[file exists $veanName]} \
           {
            puts $outUnit ""
            puts $outUnit "  @$veanName"
           }
       }
    puts $outUnit "end"
    puts $outUnit ""

    puts $outUnit "set echo on message on end"
    puts $outUnit ""
    puts $outUnit "! VEAN (vector angles) energy parameters"
    puts $outUnit ""
    puts $outUnit "vean"
    puts $outUnit "  force = \$kvean_bor \$kvean_cen"
    puts $outUnit "end"

    set flagList [lappend flagList "vean"]
   }

puts $outUnit ""
puts $outUnit "! setup the dipolar term and read restraints"
puts $outUnit "@$xdipName"

set flagList [lappend flagList "xdip"]

puts $outUnit ""
puts $outUnit "flags"
puts $outUnit "  exclude *"
puts $outUnit "  include $flagList"
puts $outUnit "end"

puts $outUnit ""
puts $outUnit "! initial minimisation"
puts $outUnit "mini powell nstep=1000 nprint=100 end"

puts $outUnit ""
puts $outUnit "! execute xplor commands from ramp definition script $rampName"
puts $outUnit "! setup ramps for simulated annealing"
puts $outUnit ""
puts $outUnit "@$rampName"
puts $outUnit ""

puts $outUnit "! loop for \$end_count models"
puts $outUnit ""

puts $outUnit "coor copy end"
puts $outUnit "evaluate (\$count = 0)"
puts $outUnit "evaluate (\$end_count = $nModel)"

puts $outUnit ""
puts $outUnit "while (\$count < \$end_count) loop main"

puts $outUnit ""
puts $outUnit "  evaluate (\$count = \$count + 1)"
puts $outUnit ""
puts $outUnit "! temporarily save initial coordinates for next trial"
puts $outUnit "  coor swap end"
puts $outUnit "  coor copy end"

puts $outUnit ""
puts $outUnit "! initialize SA"

puts $outUnit ""
puts $outUnit "  evaluate (\$bath  = \$init_t)"
puts $outUnit ""
puts $outUnit "  vector do (vx = maxwell(\$bath)) (all)"
puts $outUnit "  vector do (vy = maxwell(\$bath)) (all)"
puts $outUnit "  vector do (vz = maxwell(\$bath)) (all)"

puts $outUnit ""
puts $outUnit "! high temperature dynamics (optional)"
puts $outUnit ""
puts $outUnit "  dynamics  verlet"
puts $outUnit "      nstep = \$high_steps  timestep = \$tstep"
puts $outUnit "      iasvel = current  firsttemp = \$bath"
puts $outUnit "      tcoupling = true  tbath = \$bath   nprint = \$nstep"
puts $outUnit "      iprfrq = 0  ntrfr = 99999999"
puts $outUnit "  end"

puts $outUnit ""
puts $outUnit "! start annealing"
puts $outUnit ""
puts $outUnit "  evaluate (\$i_cool = 0)"
puts $outUnit "  while (\$i_cool < \$ncycle) loop cool"
puts $outUnit "    evaluate (\$i_cool = \$i_cool + 1)"

puts $outUnit ""
puts $outUnit "    evaluate (\$bath = \$bath - \$tempstep)"
puts $outUnit ""
puts $outUnit "!   ramp up the force constants and apply the forces"

puts $outUnit "    evaluate (\$k_bond = \$k_bond*\$bond_fac)"
puts $outUnit "    evaluate (\$k_ang = \$k_ang*\$ang_fac)"
puts $outUnit "    evaluate (\$k_imp = \$k_imp*\$imp_fac)"
puts $outUnit "    evaluate (\$k_vdw = min(\$fin_con,\$k_vdw*\$k_vdwfact))"
puts $outUnit "    evaluate (\$radius = max(\$fin_rad,\$radius*\$radfact))"
puts $outUnit "    evaluate (\$knoe = \$knoe*\$noe_fac)"
puts $outUnit "    evaluate (\$kcdih=min(\$fin_cdih,\$kcdih*\$cdih_fac))"
puts $outUnit "    evaluate (\$krama = \$krama*\$rama_fac)"
puts $outUnit "    evaluate (\$kramalr = \$kramalr*\$lr_fac)"
puts $outUnit "    evaluate (\$krg = \$krg*\$rg_fac)"
puts $outUnit "    evaluate (\$kvean_bor = \$kvean_bor*\$bor_fac)"
puts $outUnit "    evaluate (\$kvean_cen = \$kvean_cen*\$cen_fac)"
puts $outUnit "    evaluate (\$kharm = \$kharm*\$harm_fac)"
puts $outUnit "    evaluate (\$kdipo = \$kdipo*\$dipo_fac)"
puts $outUnit ""
puts $outUnit "!   covalent"
puts $outUnit "    constraints interaction (all) (all) weights"
puts $outUnit "        * 1 bonds \$k_bond angles \$k_ang improper \$k_imp"
puts $outUnit "    end end"

puts $outUnit ""
puts $outUnit "!   nonbonded"
puts $outUnit "    parameter"
puts $outUnit "        nbonds"
puts $outUnit "        cutnb=4.5 nbxmod=3 rcon=\$k_vdw repel=\$radius"
puts $outUnit "        end"
puts $outUnit "    end"

if {$ramaFlag} \
   {
    puts $outUnit ""
    puts $outUnit "!   scale the rama terms"

    puts $outUnit "    set echo off message off end"
    puts $outUnit "    rama"
    puts $outUnit "       @GAUSSIANS:new_shortrange_force.tbl"
    puts $outUnit "       @GAUSSIANS:longrange_4D_hstgp_force.tbl"
    puts $outUnit "       @GAUSSIANS:virtualTA_force_hstgp.tbl"
    puts $outUnit "    end"
    puts $outUnit "    set echo on message on end"
   }

if {[llength $noeList]} \
   {
    puts $outUnit ""
    puts $outUnit "    noe scale all \$knoe end"
   }

if {[llength $torList]} \
   {
    puts $outUnit ""
    puts $outUnit "    restraints dihedral scale \$kcdih end"
   }

if {[llength $atomList]} \
   {
    puts $outUnit ""
    puts $outUnit "    vector do (harmonic = 0.0) (all)"
    puts -nonewline $outUnit "    vector do (harmonic = \$kharm)"
    puts -nonewline $outUnit " ("

    for {set i 0} {$i < $nAtom} {incr i} \
       {
        set atomName [lindex $atomList $i]
        if {$i} {puts -nonewline $outUnit " or "}
        puts -nonewline $outUnit "name $atomName"
       }

    puts $outUnit ")"
    puts $outUnit "    restraints harmonic exponent=2 end"
   }

if {[llength $veanList]} \
   {
    puts $outUnit ""
    puts $outUnit "    vean force = \$kvean_bor \$kvean_cen end"
   }

puts $outUnit ""
puts $outUnit "    @$xForce"

puts $outUnit ""
puts $outUnit "!   do dynamics"
puts $outUnit "    dynamics  verlet"
puts $outUnit "        nstep = \$nstep  timestep = \$tstep"
puts $outUnit "        iasvel = current  firsttemp = \$bath"
puts $outUnit "        tcoupling = true  tbath = \$bath   nprint = \$nstep"
puts $outUnit "        iprfrq = 0  ntrfr = 99999999"
puts $outUnit "    end"
  
puts $outUnit ""
puts $outUnit "  end loop cool"
puts $outUnit ""
puts $outUnit "! final minimization"
puts $outUnit "  mini powell nstep = 1000 nprint = 100 end"
puts $outUnit ""

# analysis

puts $outUnit ""
puts $outUnit "! analysis of PDB file"
puts $outUnit "  @$pdbStat"
puts $outUnit "  @$xStat"


# bottom of xplor script

puts $outUnit ""
puts $outUnit "! create PDB header"
puts $outUnit "  @$pdbHead"

puts $outUnit ""
puts $outUnit "! store refined PDB file"
puts $outUnit ""
puts $outUnit "  evaluate (\$filename = \"$pdbOutName\" )"
puts $outUnit "    write coordinates"

if {$svANIflg == 0} \
   {
    puts $outUnit "    selection = (not (resid 500:510))"
   }

puts $outUnit "    output = \$filename"
puts $outUnit "  end"

puts $outUnit ""
puts $outUnit "end loop main"
puts $outUnit ""
puts $outUnit "stop"

close $outUnit

exit

