#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set thisVersion "2015.320.15.26"

if {[flagLoc $ARGV -help]} \
   {
    puts "Zip2Pipe Version $thisVersion"
    puts "Convert Spectral Data in a Zippped Folder."
    puts "  -inZipName inName  \[None\]          Input Zipfile."
    puts "  -convDir   convDir \[Auto\]          Output Directory for Converted Result."
    puts "  -outZipDir outDir  \[None\]          Output Directory for Zipfile Result."
    puts "  -csv       csvName \[master.csv\]    CSV Text Input."
    puts "  -tab       tabName \[master.tab\]    Intermediate Table Output."
    puts "  -anon      vList   \[DATAFILE_NAME\] CSV Variables to Anonymize."
    puts "  -findDir                           Print Next Available Output Directory and Exit."
    exit 1
   }

set inZipName  None 
set outZipDir  None 
set csvName    master.csv 
set tabName    master.tab
set scriptName zip2fid.com
set convDir    Auto 
set cdDir      .

getArgD $ARGV -inZipName inZipName
getArgD $ARGV -outZipDir outZipDir
getArgD $ARGV -convDir   convDir
getArgD $ARGV -csv       csvName
getArgD $ARGV -tab       tabName
getArgD $ARGV -cd        cdDir

set fullInZipDir  None
set fullOutZipDir None
set zipDir        None
set baseDir       [pwd] 

set anonList   "DATAFILE_NAME"

set expectedVars \
 {
  DATA_ID DATAFILE_NAME SAMPLE_INFO \
  SPECTRUM_TYPE GRADIENT_ENHANCED \
  X_SIZE_COMPLEX X_NUCLEUS X_SW_HZ X_OBS_MHZ X_CAR_PPM \
  Y_SIZE_COMPLEX Y_NUCLEUS Y_SW_HZ Y_OBS_MHZ Y_CAR_PPM \
  TEMPERATURE_K TEMPERATURE_CALIBRATED SCANS NUS_NAME NUS_SAMPLE_COUNT
 }

getListArgD $ARGV -anon     anonList
getListArgD $ARGV -expected expectedVars

#
#------------------------------------------------------------------------------------------

proc findConvDir {} \
{
   global convDir fullOutZipDir

   if {![strcasecmp $convDir Auto]} \
      {
       for {set i 1} {$i < 100000} {incr i} \
         {
          set thisName [format "conv%05d" $i]

          if {![file isdirectory $thisName]} \
             {
              if {[strcasecmp $fullOutZipDir None] && [file exists $fullOutZipDir/${thisName}.zip]} continue
              set convDir $thisName
              break
             }
         }
      }
}

proc getThisParm { e v } \
{
   global varList

   if {[lsearch -exact $varList $v] < 0} {return None}

   set p [gdbGet $e $v]
   if {![llength $p]} {set p None}

   return $p
}

proc duplicateNUS { inName outName } \
{
   set error [catch {set inUnit  [open $inName  r]}]
   if {$error} {return $error} 

   set error [catch {set outUnit [open $outName w]}]
   if {$error} {return $error} 

   while {[gets $inUnit line] >= 0} \
      {
       foreach item $line \
          {
           puts -nonewline $outUnit [format " %6s" $item]
          }

       puts $outUnit ""
      }

    catch "close $inUnit"
    catch "close $outUnit"

    return 0
}

proc saveNonAnon { outName } \
{
   global anonList expectedVars varInfo

   set error [catch {set outUnit [open $outName w]}]
   if {$error} {return $error} 

   foreach v $expectedVars \
      {
       if {[lsearch -exact $anonList $v] >= 0} continue

       set val None

       if {[info exists varInfo($v)]} {set val $varInfo($v)}

       puts $outUnit [format "%-24s %s" $v $val]
      }

   catch "close $outUnit"

   return 0
}

if {[strcasecmp $outZipDir None]} \
   {
    set fullOutZipDir [file normalize $outZipDir]
   }

if {[flagLoc $ARGV -findDir]} \
   {
    if {[strcmp $cdDir .]} {catch "cd $cdDir"}
    findConvDir
    puts $convDir
    exit 0
   }

foreach var $expectedVars {set varInfo($var) None}

set varList ""
set eList   ""

puts "Zip2Pipe. Version:               $thisVersion"
puts "Zip2Pipe. Starting Time:         [sysTime %c]"
puts "Zip2Pipe. Zip Input:             $inZipName"
puts "Zip2Pipe. Base Directory:        $cdDir"

if {[strcmp $cdDir .]} {catch "cd $cdDir"}

set baseDir [pwd]

puts "Zip2Pipe. Actual Directory:      $baseDir"

if {[strcasecmp $inZipName None]} \
   {
    if {![file exists $inZipName]} \
       {
        puts "Zip2Pipe. Error Finding Input Zip File $inZipName"
        exit 1
       }

    set thisName [file tail $inZipName]   
    set zipDir   [file root $thisName]

    catch "exec unzip $inZipName"

    if {![file isdirectory $zipDir]} \
       {
        puts "Zip2Pipe. Error Finding Zip Folder $zipDir"
        exit 1
       }

    set fullInZipDir [file normalize $zipDir]

    catch "cd $zipDir"

    set thisDir [pwd]

    puts "Zip2Pipe. Zip Directory:         $zipDir"
    puts "Zip2Pipe. Final Directory:       $thisDir"
   }

if {![file exists $csvName]} \
   {
    puts "Zip2Pipe. Error Finding Input CSV File $csvName"
    exit 1
   }

puts "Zip2Pipe. CSV Input:             $csvName"

findConvDir

if {![strcasecmp $convDir Auto]} \
   {
    puts "Zip2Pipe. Error Creating Name for Output Directory."
    exit 1
   }

if {![file isdirectory $convDir]} {catch "file mkdir $convDir"}

if {![file isdirectory $convDir]} \
   {
    puts "Zip2Pipe. Error Finding or Creating Output Conversion Directory $convDir"
    exit 1
   }

puts "Zip2Pipe. Output Directory:      $convDir"

catch {exec csv2tab.tcl -in $csvName -out $tabName -csvVar -allTxt -silent}

if {![file exists $tabName]} \
   {
    puts "Zip2Pipe. Error Creating Table $tabName"
    exit 1
   }

set t       [gdbRead table -in $tabName]
set eList   [gdbGetInfo $t -parms entries]
set varList [gdbGetInfo $t -parms varNames]

puts "Zip2Pipe. Data Table:            $tabName Variables: [llength $varList] Entries: [llength $eList]"

foreach v {DATA_ID DATAFILE_NAME SAMPLE_INFO} \
   {
    if {[lsearch -exact $varList $v] < 0} \
       {
        puts "Zip2Pipe. Error Finding Required Variable $v in Table $tabName"
        exit 1
       }
   }

set sCount 0

set origDir [pwd]

foreach e $eList \
   {
    foreach var $expectedVars {set varInfo($var) None}

    cd $origDir

    set sID     [getThisParm $e DATA_ID]
    set rawName [getThisParm $e DATAFILE_NAME]
    set sInfo   [split [getThisParm $e SAMPLE_INFO] _]

    if {![strcasecmp $rawName None]} continue

    incr sCount

    puts ""

    foreach v $expectedVars {set varInfo($v) [getThisParm $e $v]}

    if {![file exists $rawName]} \
       {
        puts "Zip2Pipe. Error Finding Input Data Entry $sCount: $rawName"
        continue
       } 

    set rawDir   [file dirname   $rawName]
    set thisExt  [file extension $rawName]
    set destDir  [file normalize $convDir]
    set outName  [file normalize $convDir/${sID}.fid]
    set convArgs "-auto -nosleep -notk"
    set nusArgs  ""
    set userArgs ""

    puts "Zip2Pipe. [format "Entry %2d." $sCount] ID:          $sID"
    puts "Zip2Pipe. [format "Entry %2d." $sCount] Sample:      $sInfo"
    puts "Zip2Pipe. [format "Entry %2d." $sCount] Directory:   $rawDir (Not Anonymous)"
    puts "Zip2Pipe. [format "Entry %2d." $sCount] Input Data:  [file tail $rawName] (Not Anonymous)"
    puts "Zip2Pipe. [format "Entry %2d." $sCount] Input Size:  [file size $rawName] bytes (Not Anonymous)"
    puts "Zip2Pipe. [format "Entry %2d." $sCount] Destination: $destDir"

    cd $rawDir

    if {[file exists acqus] && ([file exists pulseprogram] || [file exists pulseprogram.precomp])} \
       {
        set convProc bruker
        append convArgs " -nodf"
       } \
    elseif {[file exists procpar]} \
       {
        set convProc varian 
        append convArgs " -nodf"
       } \
    elseif {![strcasecmp $thisExt .jdf]} \
       {
        set convProc delta 
        append convArgs " -nodf"
       } \
    else \
       {
        set convProc unknown 
       }

    if {[strcasecmp $varInfo(TEMPERATURE_K) None]} {append convArgs " -t $varInfo(TEMPERATURE_K) -useTemp"}
    if {[strcasecmp $varInfo(SCANS) None]}         {append convArgs " -scans $varInfo(SCANS)"}

    if {[strcasecmp $varInfo(NUS_NAME) None]} \
       {
        if {![file exists $varInfo(NUS_NAME)]} \
           {
            puts "Zip2Pipe. [format "Entry %2d." $sCount] Error Finding NUS File $varInfo(NUS_NAME)."
            continue
           } 

        puts "Zip2Pipe. [format "Entry %2d." $sCount] NUS Input :  $varInfo(NUS_NAME) (Not Anonymous)"

        set error [duplicateNUS $varInfo(NUS_NAME) ${sID}.nus]

        if {$error} \
           {
            puts "Zip2Pipe. [format "Entry %2d." $sCount] Error Copying NUS Data to ${sID}.nus"
            continue
           }

        set error [duplicateNUS $varInfo(NUS_NAME) $destDir/${sID}.nus]

        if {$error} \
           {
            puts "Zip2Pipe. [format "Entry %2d." $sCount] Error Copying NUS Data to $convDir/${sID}.nus"
            continue
           }

        puts "Zip2Pipe. [format "Entry %2d." $sCount] NUS Output:  ${sID}.nus"
        puts "Zip2Pipe. [format "Entry %2d." $sCount] NUS Mask:    ${sID}_mask.fid"

        append nusArgs " -nus ${sID}.nus -mask [file normalize $destDir/${sID}_mask.fid]"

        if {[strcasecmp $varInfo(NUS_SAMPLE_COUNT) None]} \
           {
            puts "Zip2Pipe. [format "Entry %2d." $sCount] NUS Samples: $varInfo(NUS_SAMPLE_COUNT)"
            append nusArgs " -sampleCount $varInfo(NUS_SAMPLE_COUNT)"
           }
       }

    set error [saveNonAnon $destDir/${sID}.info]

    if {$error} \
       {
        puts "Zip2Pipe. [format "Entry %2d." $sCount] Error Copying Parameter Data to $destDir/${sID}.info"
        continue
       }

    if {[strcasecmp $varInfo(X_OBS_MHZ) None]}     {append userArgs " -xOBS $varInfo(X_OBS_MHZ)"}
    if {[strcasecmp $varInfo(X_CAR_PPM) None]}     {append userArgs " -xCAR $varInfo(X_CAR_PPM)"}

    if {[valIsYes   $varInfo(GRADIENT_ENHANCED)]}  {append userArgs " -yMODE Echo-AntiEcho"}
    if {[valIsNo    $varInfo(GRADIENT_ENHANCED)]}  {append userArgs " -yMODE Complex"}

    if {[strcasecmp $varInfo(Y_SW_HZ)   None]}     {append userArgs " -ySW  $varInfo(Y_SW_HZ)"}
    if {[strcasecmp $varInfo(Y_OBS_MHZ) None]}     {append userArgs " -yOBS $varInfo(Y_OBS_MHZ)"}
    if {[strcasecmp $varInfo(Y_CAR_PPM) None]}     {append userArgs " -yCAR $varInfo(Y_CAR_PPM)"}

#   set xLAB [string toupper $varInfo(X_NUCLEUS)]
#   set yLAB [string toupper $varInfo(Y_NUCLEUS)]
#
#   if {![strcasecmp $yLAB 15N] || ![strcasecmp $yLAB N15]} {set yLAB N15}
#   if {![strcasecmp $yLAB 13C] || ![strcasecmp $yLAB C13]} {set yLAB C13}
#   if {![strcasecmp $xLAB 1H]  || ![strcasecmp $yLAB H1]}  {set xLAB H1}
#   if {![strcasecmp $xLAB H1]  && ![strcasecmp $yLAB N15]} {set xLAB HN}
#
#   if {[strcasecmp $xLAB None]} {append userArgs " -xLAB $xLAB"}
#   if {[strcasecmp $yLAB None]} {append userArgs " -yLAB $yLAB"}

    set xNUC [string toupper $varInfo(X_NUCLEUS)]
    set yNUC [string toupper $varInfo(Y_NUCLEUS)]

    if {[strcasecmp $xNUC None]} {append convArgs " -xNUC $xNUC"}
    if {[strcasecmp $yNUC None]} {append convArgs " -yNUC $yNUC"}

    if {![llength $nusArgs]}  {set nusArgs  None}
    if {![llength $userArgs]} {set userArgs None}

    puts "Zip2Pipe. [format "Entry %2d." $sCount] Conversion:  $convProc (Not Anonymous)"
    puts "Zip2Pipe. [format "Entry %2d." $sCount] Info:        ${sID}.info"
    puts "Zip2Pipe. [format "Entry %2d." $sCount] Log:         ${sID}_conv.log"
    puts "Zip2Pipe. [format "Entry %2d." $sCount] Script:      $scriptName"

    if {![strcasecmp $convProc unknown]} \
       {
        puts "Zip2Pipe. [format "Entry %2d." $sCount] Error Finding Data Type."
        continue
       } 

    if {[file exists $outName]} \
       {
        puts "Zip2Pipe. [format "Entry %2d." $sCount] Error: Output $outName Already Exists."
        continue
       }

    puts "Zip2Pipe. [format "Entry %2d." $sCount] Arguments 1: [string trim $convArgs]"
    puts "Zip2Pipe. [format "Entry %2d." $sCount] Arguments 2: [string trim $nusArgs]"
    puts "Zip2Pipe. [format "Entry %2d." $sCount] Arguments 3: [string trim $userArgs]"

    if {[strcasecmp $nusArgs  None]} {append convArgs " $nusArgs"}
    if {[strcasecmp $userArgs None]} {append convArgs " -user $userArgs"}

    catch "eval exec $convProc -out $outName -script $scriptName $convArgs >& ${sID}_conv.log"

    if {![file exists $scriptName]} \
       {
        puts "Zip2Pipe. [format "Entry %2d." $sCount] Error Creating Script $scriptName"
        continue
       } 

    catch "exec $scriptName >>& ${sID}_conv.log"

    if {![file exists $outName]} \
       {
        puts "Zip2Pipe. [format "Entry %2d." $sCount] Error Creating Output $outName"
        continue
       }

    puts "Zip2Pipe. [format "Entry %2d." $sCount] Output Data: $outName"
    puts "Zip2Pipe. [format "Entry %2d." $sCount] Output Size: [file size $outName] bytes"

    set scaleInfo "Unknown Unknown"

    catch {set scaleInfo [exec scale2D $outName -all -brief -showMin -showMax]}

    if {![info exists scaleInfo]} {set scaleInfo "Unknown Unknown"}

    lappend scaleInfo Unknown
    lappend scaleInfo Unknown

    puts "Zip2Pipe. [format "Entry %2d." $sCount] Output Min:  [lindex $scaleInfo 0]"
    puts "Zip2Pipe. [format "Entry %2d." $sCount] Output Max:  [lindex $scaleInfo 1]"

    catch {exec sethdr $outName -title \"$sInfo\"}
   }

catch "cd $origDir"

if {[strcasecmp $outZipDir None]} \
   {
    set tarName $outZipDir/[file tail $convDir].zip

    catch "exec zip -r $tarName $convDir"

    puts ""
    puts "Zip2Pipe. Created Archive $tarName [file size $tarName] bytes"
   }

if {[strcasecmp $fullInZipDir None] && [file isdirectory $fullInZipDir]} \
   {
    if {[file exists $fullInZipDir/debug.txt]} \
       {
        puts "Zip2Pipe. Suppressing Deletion of Zip Directory $fullInZipDir"
       } \
    else \
       {
        puts "Zip2Pipe. Deleting Zip Directory $fullInZipDir"

        catch "file delete -force -- $fullInZipDir"

        if {[file isdirectory $fullInZipDir]} \
           {
            puts "Zip2Pipe. Error Deleting Zip Directory $fullInZipDir"
           }
       }
   }

exit 0 
