/***/
/* NMRPipe Copyright 1992 - 2024 Frank Delaglio
/* Utility used for reading NMRPipe-format tables.
/***/

/***/
/* isDataLine: Return 0 if line is blank, or if line begins with FORMAT # etc.
/*             Return 1 otherwise.
/***/

#include <stdio.h>
#include <string.h>

#include "stralloc.h"
#include "token.h"
#include "prec.h"

int isDataLine( line )

    char *line;
{
    char  token[NAMELEN+1];
    int   status;

    status = 1;

    (void) strcpy( token, getToken( 1, line ));
    (void) raiseStr( token );

    if (!cntToken( token ))
       status = 0;
    else if (!strcmp( token, "REMARK" ))
       status = 0;
    else if (!strcmp( token, "FORMAT" ))
       status = 0;
    else if (!strcmp( token, "VARS" ))
       status = 0;
    else if (!strcmp( token, "TOLERANCE" ))
       status = 0;
    else if (!strcmp( token, "NULLSTRING" ))
       status = 0;
    else if (!strcmp( token, "NULLVALUE" ))
       status = 0;
    else if (!strcmp( token, "DATA" ))
       status = 0;
    else if (!strcmp( token, "END" ))
       status = 0;
    else if (*token == '#' )
       status = 0;

    return( status );
}

/***/
/* cntDataLine: count number of valid data lines in inName.
/***/

int cntDataLine( inName, count )

   char *inName;
   int  *count;
{
    char *line, cBuff[NAMELEN+1], inBuff[NAMELEN+1];
    int  cBuffLen, inLen, maxLen;
    FILE *inUnit;

    line          = cBuff;
    inLen         = NAMELEN+1;
    cBuffLen      = NAMELEN+1;
    maxLen        = NAMELEN+1;

    cBuff[0]      = '\0';
    inBuff[0]     = '\0';

    if (!(inUnit = fopen( inName, "r")))
       {
        (void) fprintf( stderr, "CntDataLine File open error: %s\n", inName );
        return( 1 );
       }

    *count = 0;
 
    while( fgetsAlloc( &line, &maxLen, inUnit, inBuff, inLen, cBuff, cBuffLen ))
       {
        if (isDataLine( line )) (*count)++;
       }

    (void) strAllocFree( &line, cBuff, cBuffLen, &maxLen );
    (void) fclose( inUnit );

    return( 0 );
}
