 
/***/
/* setfdata: set numerical FDATA values in the specified NMRi format file. 
/***/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "cmndargs.h"
#include "inquire.h"
#include "fdatap.h"
#include "prec.h"
#include "atof.h"

#define FPR (void)fprintf

int main( argc, argv )

    int  argc;
    char **argv;

{
    int   iz, ia, zN, aN, fmtCount, error;
    char  inName[NAMELEN+1], thisName[NAMELEN+1];
    float fdata[FDATASIZE];
/**/
/*  Initialize error status, file unit: 
/**/

    error = 0;

    (void) initArgs( argc, argv, CHECK_NONE );
    (void) initDataIO();

     if (flagLoc( argc, argv, "-help" ) || argc < 4)
        {
         FPR( stderr, "Set Values in NMRPipe-Format Header (FDATA):\n" );
         FPR( stderr, " %s inName [fdataLoc1 val1 ...] [options] (Values on Command Line).\n", argv[0] );
         FPR( stderr, " %s inName [fdataLoc1 ASK ...]  [options] (Enter Values Interactively).\n", argv[0] );
         (void) parseHdr( argc, argv, fdata );
         return( 0 );
        }

     (void) strcpy( inName, getNthArg( argc, argv, 1 ));

     fmtCount = getFmtCount( inName );

     if (fmtCount == 2)
        {
         for( ia = 1; ia < 999999; ia++ )
            {
             for( iz = 1; iz < 999999; iz++ )
                {
                 (void) sprintf( thisName, inName, ia, iz );
    
                 if (!fileExists( thisName ))
                    {
                     if (iz == 1 && ia == 1)
                        {
                         FPR( stderr, "SetHdr Error finding file %s\n", thisName );
                         return( -1 );
                        }
                     else
                        {
                         break;
                        }
                    }

                 error = setThisHdr( argc, argv, thisName );
                }
            }
        }
     else if (fmtCount == 1)
        {
         for( iz = 1; iz < 999999; iz++ )
            {
             (void) sprintf( thisName, inName, iz );

             if (!fileExists( thisName ))
                {
                 if (iz == 1)
                    {
                     FPR( stderr, "SetHdr Error finding file %s\n", thisName );
                     return( -1 );
                    }
                 else
                    {
                     break;
                    }
                }

             error = setThisHdr( argc, argv, thisName );
            }
        }
     else
        {
         (void) strcpy( thisName, inName );

         if (!fileExists( thisName ))
            {
             FPR( stderr, "SetHdr Error finding file %s\n", thisName );
             return( -1 );
            }

         error = setThisHdr( argc, argv, inName );
        }

    return( 0 );
}

int setThisHdr( argc, argv, inName )

   int   argc;
   char  **argv;
   char  *inName;
{
    int   loc, locArg, valArg, swapDone, error;
    float fdata[FDATASIZE], val;

    if (!fileExists( inName ))
       {
        FPR( stderr, "SetHdr Error finding file %s\n", inName );
        goto shutdown;
       }

    if (error = rdFDATAS( inName, fdata, &swapDone ))
       {
        FPR( stderr, "SetHdr Error reading file %s\n", inName );
        error = 1;
        goto shutdown; 
       }

   if (error = parseHdr( argc, argv, fdata ))  goto shutdown;
   if (error = parseHdr2( argc, argv, fdata )) goto shutdown;
 
   locArg = 2;
   valArg = 3;

   while( (locArg < argc) && (valArg < argc) && !isFlag( argv[locArg] )) /* No getNthArg needed. */
      {
       loc = ((INT4) ATOF( getNthArg( argc, argv, locArg ))) - 1;

       if (toupper( *argv[valArg] ) == 'A')  /* No getNthArg needed. */
          {
           (void) getNthArg( argc, argv, valArg );
           (void) fprintf( stderr, "Enter fdata(%d): ", loc+1 );
           (void) scanf( "%f", &val );
          }
       else
          {
           val = (REAL4) ATOF( getNthArg( argc, argv, valArg ));
          }

       if (loc >= 0 && loc < FDATASIZE)
          {
           FPR( stderr, "%s fdata[%d] Old: %g New: %g\n", inName, loc+1, fdata[loc], val );
           fdata[loc] = val;
          }
       else
          {
           FPR( stderr, "SetHdr Error: bad FDATA location %s.\n", getNthArg( argc, argv, locArg ));
           error = 1;
           goto shutdown;
          }

       locArg += 2;
       valArg += 2;
      }  

   if (swapDone) (void) swapHdr( fdata );

   if (error = wrFDATA( inName, fdata ))
      {
       FPR( stderr, "Error writing header in %s\n", inName );
       goto shutdown; 
      }

shutdown:

  if (error) FPR( stderr, "No changes made to %s\n", inName );

  return( error );
}

