#!/bin/csh

#
# Bruker AMX 4D 13C-NOE Conversion and Processing:
#   Conversion is piped directly into processing scheme.
#   "EXT -time" clips padding added by Bruker to each 1D FID.
#
#   The scheme processes (acq1H,indir1H), then 13Cz, then 13Ca.
#   The scheme changes the final axis ordering for later analysis.
#
#   Original order: X=acq1H   Y=indir1H Z=13Cz  A=13Ca.
#   Final order:    X=indir1H Y=13Cz    Z=acq1H A=13Ca.

bruk2pipe -in /spain1/rclubb/ser \
 -xN         768  -yN         128 -zN          36 -aN           36  \
 -xT         300  -yT          64 -zT          18 -aT           18  \
 -xMODE  Complex  -yMODE  Complex -zMODE  Complex -aMODE   Complex  \
 -xSW   4166.667  -ySW   4166.667 -zSW   2604.167 -aSW    2604.167  \
 -xOBS   500.189  -yOBS   500.189 -zOBS   125.777 -aOBS    125.777  \
 -xCAR      3.73  -yCAR      3.73 -zCAR     63.71 -aCAR      63.71  \
 -xLAB     acq1H  -yLAB   indir1H -zLAB      13Cz -aLAB       13Ca  \
 -ndim         4  -aq2D    States -verb                             \
| nmrPipe -fn EXT -time                             \
| nmrPipe -fn SP  -off 0.45 -end 0.95 -pow 2 -c 0.5 \
| nmrPipe -fn ZF  -size 512                         \
| nmrPipe -fn FT                                    \
| nmrPipe -fn PS  -p0 0.0 -p1 0.0 -di               \
| nmrPipe -fn TP                                    \
| nmrPipe -fn SP  -off 0.45 -end 1.0 -pow 1  -c 0.5 \
| nmrPipe -fn ZF  -size 128                         \
| nmrPipe -fn FT                                    \
| nmrPipe -fn PS  -p0 0.0 -p1 0.0 -di               \
| pipe2xyz -out ft/c13c13noe%02d%03d.DAT -z -ov

xyz2pipe -in ft/c13c13noe%02d%03d.DAT -y -verb      \
| nmrPipe -fn SP -off 0.5 -end 0.95 -pow 2  -c 1.0  \
| nmrPipe -fn ZF -size 64                           \
| nmrPipe -fn FT                                    \
| nmrPipe -fn PS  -p0 -90  -p1 180  -di             \
| pipe2xyz -out ft/c13c13noe%02d%03d.DAT -y -ov -inPlace

xyz2pipe -in ft/c13c13noe%02d%03d.DAT -a -verb     \
| nmrPipe -fn SP -off 0.5 -end 0.95 -pow 2  -c 1.0 \
| nmrPipe -fn ZF -size 64                          \
| nmrPipe -fn FT                                   \
| nmrPipe -fn PS  -p0 -90  -p1 180  -di            \
| pipe2xyz -out ft/c13c13noe%02d%03d.DAT -a -ov -inPlace
