/***/
/* maxPlane.M: report real 2D plane loc with max abs intensity;
/*             restricted to inner 25% of data.
/*
/* Requires command-line definition of strings "name".
/*   M $NMRTXT/maxLoc.M -str name test.ft2 -quit
/***/

/***/
/* Parameters taken from "fdatap.h"; these will be pre-defined
/* in future implementations of "M":
/***/

    FDATASIZE = 512;

    float fdata[FDATASIZE];

    FDQUADFLAG   = 106;
    NULL_DIM     =   0;

/***/
/* Open the first plane.
/* Read the header from the first plane.
/* Extract the size parameters.
/***/

    inFD  = dOpen( name, "r" );

    (void) dReadHdr( inFD, fdata, 0 );

    xSize = getParm( fdata, NDSIZE, CUR_XDIM );
    ySize = getParm( fdata, NDSIZE, CUR_YDIM );
    itemp = getParm( fdata, FDQUADFLAG, NULL_DIM );

    if (itemp == 0 || itemp == 2)
       {
        quadState = 2;
        jump      = xSize;
       }
    else
       {
        quadState = 1;
        jump      = 0;
       };

    count = xSize*ySize*quadState;
    midX  = 1 + integer(xSize/2);
    midY  = 1 + integer(ySize/2);

/***/
/* Read the plane.
/* Find the max abs value.
/***/

    float buff[count];

    maxVal  = 0.0;
    maxLocX = 1;
    maxLocY = 1;

    (void) dRead( inFD,  buff, wordLen*count );
    (void) dClose( inFD );

    j = 0;

    for( iy = 0; iy < ySize; iy++ )
       {
        for( ix = 0; ix < xSize; ix++ )
           {
            if (ix > 0.25*xSize && ix < 0.75*xSize && 
		iy > 0.25*ySize && iy < 0.75*ySize && 
		 abs(buff[j]) > abs(maxVal))
               {
                maxVal  = buff[j];
                maxLocX = ix;
                maxLocY = iy;
               };

            j++;
           }

        j += jump; 
       }

    maxLocX = maxLocX + 1;
    maxLocY = maxLocY + 1;

    offX = maxLocX - midX;
    offY = maxLocY - midY;

    (void) printf( "%e %.0f %.0f", maxVal, maxLocX, maxLocY );
    (void) printf( " %.0f %.0f %.0f %.0f\n", midX, midY, offX, offY );
