#!/bin/csh -f

#
# proc1d.com: Used by NMRDraw's "Auto-Process 1D" Command.
#  Usage: proc1d.com axisName quadMode ftMode [userArgs]
#  Data is read and written via stdin/stdout.
#
#  If input is time-domain:
#    If data is HN: Applies solvent filter, extracts HN-amide range.
#    Otherwise: Window, Zero Fill, Fourier Transform.
#
#  If input is freq-domain:
#    Restore imaginaries as needed, Inverse Phase, FT, ZF and Window.

#echo 1D Macro=$0 Axis=$1 Type=$2 Domain=$3

if ($3 == Time) then
   if ($1 == HN || $1 == HNx) then
      nmrPipe    -fn SOL -tty \
      | nmrPipe  -fn SP -off 0.5 -end 0.98 -pow 2 -c 0.5 \
      | nmrPipe  -fn ZF -auto \
      | nmrPipe  -fn FT -auto \
      | nmrPipe  -fn PS -hdr  \
      | nmrPipe  -fn EXT -x1 11ppm -xn 5.5ppm -sw \
        -tty -verb -inc 1
   else
      nmrPipe    -fn SP -off 0.5 -end 0.98 -pow 2 -c 1.0 -tty \
      | nmrPipe  -fn ZF -auto \
      | nmrPipe  -fn FT -auto \
      | nmrPipe  -fn PS -hdr  -tty -verb -inc 1
   endif
else
   nmrPipe    -fn HT -auto -tty    \
   | nmrPipe  -fn PS   -inv -hdr   \
   | nmrPipe  -fn FT   -inv        \
   | nmrPipe  -fn ZF   -inv        \
   | nmrPipe  -fn APOD -inv -hdr   \
     -tty -verb -inc 1
endif
