/***/
/* xz4d.M: extract the first XZ plane of a 3D or 4D hypercomplex FID.
/*
/* Requires command-line definition of strings "inName" and "outName",
/* and variables "xzYLoc" and "xzALoc" e.g.:
/*
/*   M xz4d.M -str inName fid/test%03.0f.fid outName xz.fid -var xzYLoc 1 xzALoc 1 -quit
/***/

/***/
/* Parameters taken from "fdatap.h"; these will be pre-defined
/* in future implementations of "M":
/***/

    FDATASIZE = 512;

    float fdata[FDATASIZE];
    char  thisName[256];

    FDDIMCOUNT   =   9;
    FDDIMORDER1  =  24;
    FDDIMORDER2  =  25;
    FDDIMORDER3  =  26;
    FDDIMORDER4  =  27;
    FDFILECOUNT  = 442;
    NULL_DIM     = 0;

/***/
/* Open the first 4D plane.
/* Read the header from the first 4D plane.
/* Extract the size parameters.
/***/

    (void) sprintf( thisName, inName, 1, 1 );

    inFD  = dOpen( thisName, "r" );
    outFD = dOpen( outName,  "w" );

    (void) dReadHdr( inFD, fdata, 0 );
    (void) dClose( inFD );

    dimCount = getParm( fdata, FDDIMCOUNT, NULL_DIM );

    xSize = getParm( fdata, NDSIZE, CUR_XDIM );
    ySize = getParm( fdata, NDSIZE, CUR_YDIM );
    zSize = getParm( fdata, NDSIZE, CUR_ZDIM );
    aSize = getParm( fdata, NDSIZE, CUR_ADIM );

    if (dimCount < 3)
       {
        (void) printf( "Error: the dimension count is too small.\n" );
        exit( 1 );
       };

    if (dimCount < 4)
       {
        aSize = 1;
       };

/***/
/* Adjust header of input to reflect an output with Y and Z axis exchanged:
/***/

    (void) setParm( fdata, NDSIZE, zSize, CUR_YDIM );
    (void) setParm( fdata, NDSIZE, ySize, CUR_ZDIM );

    yID = getParm( fdata, FDDIMORDER2, NULL_DIM );
    zID = getParm( fdata, FDDIMORDER3, NULL_DIM );

    (void) setParm( fdata, FDDIMORDER2, zID, NULL_DIM );
    (void) setParm( fdata, FDDIMORDER3, yID, NULL_DIM );

    (void) setParm( fdata, FDFILECOUNT,  ySize*aSize, NULL_DIM );

    (void) dWrite( outFD, fdata, wordLen*FDATASIZE );

/***/
/* Read the first complex 1D vector from every XY plane in cube A=1:
/***/

    float buff[xSize*2];

    ia = xzALoc;

    for( iz = 1; iz <= zSize; iz++ )
       {
        if (dimCount == 3)
           {
            (void) sprintf( thisName, inName, iz );
           }
        else
           {
            (void) sprintf( thisName, inName, ia, iz );
           }

        inFD   = dOpen( thisName, "r" );
        offset = wordLen*2*xSize*(xzYLoc - 1);

        (void) dSeek( inFD, wordLen*FDATASIZE + offset );
        (void) dRead(  inFD,  buff, wordLen*xSize*2 );
        (void) dWrite( outFD, buff, wordLen*xSize*2 );
        (void) dClose( inFD ); 
       }

    (void) dClose( outFD );
