#!/bin/csh

set inName   = (`getArgDD $argv -in      _inName_  data/spec%03d.fid`)
set outName  = (`getArgDD $argv -out     _outName_ spec.ft2`)
set auxName  = (`getArgDD $argv -aux     _auxName_ spec_xa.fid`)
set specType = (`getArgDD $argv -type    _type_    NMR`)
set yPlane   = (`getArgDD $argv -yPlane  _yPlane_  1`)
set zPlane   = (`getArgDD $argv -zPlane  _zPlane_  1`)
set planeHi  = (`getArgDD $argv -planeHi _planeHi_ 2.5@`)
set dimCount = (`getArgDD $argv -ndim    _ndim_    3` )

set graphFlag = (`getLogArgDD $argv -graph     -nograph     _graphFlag_ 1 0`)
set drawPlane = (`getLogArgDD $argv -drawPlane -nodrawPlane _drawPlane_ 1 0`)

if (`string csame $specType HN`) then
   set xSOL = (`getArgDD $argv -xSOL _xSOL_ SOL`)
else
   set xSOL = (`getArgDD $argv -xSOL _xSOL_ None`)
endif

if (`string csame $xSOL None`) then
   set xSOL = NULL
endif
 
set xAPOD = (`getArgDD $argv -xAPOD _xAPOD_ SP`)
set xQ1   = (`getArgDD $argv -xQ1   _xQ1_   0.5`)
set xQ2   = (`getArgDD $argv -xQ2   _xQ2_   0.95`)
set xQ3   = (`getArgDD $argv -xQ3   _xQ3_   2`)
set xELB  = (`getArgDD $argv -xELB  _xELB_  0.0`)
set xGLB  = (`getArgDD $argv -xGLB  _xGLB_  0.0`)
set xGOFF = (`getArgDD $argv -xGOFF _xGOFF_ 0.0`)
set xC1   = (`getArgDD $argv -xC1   _xC1_   1.0`)
set xP0   = (`getArgDD $argv -xP0   _xP0_   0.0`)
set xP1   = (`getArgDD $argv -xP1   _xP1_   0.0`)

set xZFARG   = (`getArgDD $argv -xZFARG   _xZFARG_   zf=1,auto`)
set xEXTX1   = (`getArgDD $argv -xEXTX1   _xEXTX1_   0%`)
set xEXTXN   = (`getArgDD $argv -xEXTXN   _xEXTXN_   100%`)
set xBASEARG = (`getArgDD $argv -xBASEARG _xBASEARG_ POLY,auto`)

if (`string csame $xBASEARG Auto`) then
   set xBASEARG = POLY,auto
endif

if (`string csame $xC1 Auto`) then
   set v = (`printf "%f" $xP1`)
   set i = (`IMATH "abs($v) < 10.0"`)

   if ($i == 0) then
      set xC1 = 1.0
   else
      set xC1 = 0.5
   endif
endif

set aAPOD = (`getArgDD $argv -aAPOD _aAPOD_ SP`)
set aQ1   = (`getArgDD $argv -aQ1   _aQ1_   0.5`)
set aQ2   = (`getArgDD $argv -aQ2   _aQ2_   0.95`)
set aQ3   = (`getArgDD $argv -aQ3   _aQ3_   1`)
set aELB  = (`getArgDD $argv -aELB  _aELB_  0.0`)
set aGLB  = (`getArgDD $argv -aGLB  _aGLB_  0.0`)
set aGOFF = (`getArgDD $argv -aGOFF _aGOFF_ 0.0`)
set aC1   = (`getArgDD $argv -aC1   _aC1_   1.0`)
set aP0   = (`getArgDD $argv -aP0   _aP0_   0.0`)
set aP1   = (`getArgDD $argv -aP1   _aP1_   0.0`)

set aZFARG   = (`getArgDD $argv -aZFARG _aZFARG_   zf=1,auto`)
set aFTARG   = (`getArgDD $argv -aFTARG _aFTARG_   None`)
set aBASEARG = (`getArgDD $argv -aFTARG _aBASEARG_ NULL`)

if (`string csame $aC1 Auto`) then
   set v = (`printf "%f" $aP1`)
   set i = (`IMATH "abs($v) < 10.0"`)

   if ($i == 0) then
      set aC1 = 1.0
   else
      set aC1 = 0.5
   endif
endif

if (`string csame $aBASEARG Auto`) then
   set v = (`printf "%f" $aP1`)
   set i = (`IMATH "abs($v) > 10.0 && abs($v-180.0) > 10.0"`)

   if ($i == 0) then
      echo "A-Axis Baseline Auto Mode: No Baseline Correction Used."
      set aBASEARG = NULL
   else
      echo "A-Axis Baseline Auto Mode: Using POLY -auto Correction."
      set aBASEARG = POLY,auto,ord=0
   endif
endif

set yP0   = (`getArgDD $argv -yP0   _yP0_   0.0`)
set yP1   = (`getArgDD $argv -yP1   _yP1_   0.0`)

if (`string csame $yPlane Auto`) then
   set v0 = (`printf "%f" $yP0`)
   set v1 = (`printf "%f" $yP1`)
   set i  = (`IMATH "abs(abs($v0) - 90.0) < 5.0 && abs($v1) < 5.0"`)

   if ($i == 0) then
      set yPlane = 1
   else
      set yPlane = 2
   endif
endif

set zP0   = (`getArgDD $argv -zP0   _zP0_   0.0`)
set zP1   = (`getArgDD $argv -zP1   _zP1_   0.0`)

if (`string csame $zPlane Auto`) then
   set v0 = (`printf "%f" $zP0`)
   set v1 = (`printf "%f" $zP1`)
   set i  = (`IMATH "abs(abs($v0) - 90.0) < 5.0 && abs($v1) < 5.0"`)

   if ($i == 0) then
      set zPlane = 1
   else
      set zPlane = 2
   endif
endif

set thisName = (`nmrPrintf $inName 1 1`)

if (!(-e $thisName)) then
   echo "$0 Notice: Missing Input $thisName"
   exit 1
endif

ext_xa.com $inName $auxName $yPlane $zPlane

if (!(-e $auxName)) then
   echo "$0 Notice: Missing Input $auxName"
   exit 0
endif

nmrPipe -in $auxName \
| nmrPipe -fn `expandArg0 $xSOL` \
| nmrPipe -fn APOD -qName $xAPOD -q1 $xQ1 -q2 $xQ2 -q3 $xQ3 -elb $xELB -glb $xGLB -goff $xGOFF -c $xC1 \
| nmrPipe -fn ZF `expandArg $xZFARG` \
| nmrPipe -fn FT -verb \
| nmrPipe -fn PS -p0 $xP0 -p1 $xP1 -di \
| nmrPipe -fn EXT -x1 $xEXTX1 -xn $xEXTXN -sw \
| nmrPipe -fn TP \
| nmrPipe -fn APOD -qName $aAPOD -q1 $aQ1 -q2 $aQ2 -q3 $aQ3 -elb $aELB -glb $aGLB -goff $aGOFF -c $aC1 \
| nmrPipe -fn ZF `expandArg $aZFARG` \
| nmrPipe -fn FT `expandArg $aFTARG` -verb \
| nmrPipe -fn PS -p0 $aP0 -p1 $aP1 -di \
| nmrPipe -fn `expandArg0 $aBASEARG` \
| nmrPipe -fn TP \
| nmrPipe -fn `expandArg0 $xBASEARG` -verb \
  -out $outName -ov

if ($graphFlag != 0 && $drawPlane != 0) then
   set hiInfo = (`getSpecHi.tcl -in $outName $* -hi $planeHi`)
   nmrDraw -in $outName $hiInfo `getGeom -center` &
   sleep 3
endif

